/* Generated by Cython 3.2.8 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [],
        "name": "tables.hdf5extension",
        "sources": [
            "tables/hdf5extension.pyx"
        ]
    },
    "module_name": "tables.hdf5extension"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
/* InitLimitedAPI */
#if defined(Py_LIMITED_API)
  #if !defined(CYTHON_LIMITED_API)
  #define CYTHON_LIMITED_API 1
  #endif
#elif defined(CYTHON_LIMITED_API)
  #ifdef _MSC_VER
  #pragma message ("Limited API usage is enabled with 'CYTHON_LIMITED_API' but 'Py_LIMITED_API' does not define a Python target version. Consider setting 'Py_LIMITED_API' instead.")
  #else
  #warning Limited API usage is enabled with 'CYTHON_LIMITED_API' but 'Py_LIMITED_API' does not define a Python target version. Consider setting 'Py_LIMITED_API' instead.
  #endif
#endif

#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x03080000
    #error Cython requires Python 3.8+.
#else
#define __PYX_ABI_VERSION "3_2_8"
#define CYTHON_HEX_VERSION 0x030208F0
#define CYTHON_FUTURE_DIVISION 0
/* CModulePreamble */
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(_WIN32) && !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#define __PYX_LIMITED_VERSION_HEX PY_VERSION_HEX
#if defined(GRAALVM_PYTHON)
  /* For very preliminary testing purposes. Most variables are set the same as PyPy.
     The existence of this section does not imply that anything works or is even tested */
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 1
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 1
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#elif defined(PYPY_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 1
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PYPY_VERSION_NUM >= 0x07030C00)
  #endif
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC (PYPY_VERSION_NUM >= 0x07031100)
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#elif defined(CYTHON_LIMITED_API)
  #ifdef Py_LIMITED_API
    #undef __PYX_LIMITED_VERSION_HEX
    #define __PYX_LIMITED_VERSION_HEX Py_LIMITED_API
  #endif
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 1
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 1
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #endif
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND (__PYX_LIMITED_VERSION_HEX >= 0x030A0000)
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #ifndef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 1
  #endif
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #ifdef Py_GIL_DISABLED
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 1
  #else
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #endif
  #if PY_VERSION_HEX < 0x030A0000
    #undef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #elif !defined(CYTHON_USE_TYPE_SLOTS)
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #ifndef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLIST_INTERNALS)
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #elif !defined(CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS)
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_FAST_GIL
    #define CYTHON_FAST_GIL 0
  #elif !defined(CYTHON_FAST_GIL)
    #define CYTHON_FAST_GIL (PY_VERSION_HEX < 0x030C00A6)
  #endif
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #ifndef CYTHON_USE_SYS_MONITORING
    #define CYTHON_USE_SYS_MONITORING (PY_VERSION_HEX >= 0x030d00B1)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS 0
  #elif !defined(CYTHON_USE_DICT_VERSIONS)
    #define CYTHON_USE_DICT_VERSIONS  (PY_VERSION_HEX < 0x030C00A5 && !CYTHON_USE_MODULE_STATE)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 1
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
  #ifndef CYTHON_USE_FREELISTS
    #define CYTHON_USE_FREELISTS (!CYTHON_COMPILING_IN_CPYTHON_FREETHREADING)
  #endif
  #if defined(CYTHON_IMMORTAL_CONSTANTS) && PY_VERSION_HEX < 0x030C0000
    #undef CYTHON_IMMORTAL_CONSTANTS
    #define CYTHON_IMMORTAL_CONSTANTS 0  // definitely won't work
  #elif !defined(CYTHON_IMMORTAL_CONSTANTS)
    #define CYTHON_IMMORTAL_CONSTANTS (PY_VERSION_HEX >= 0x030C0000 && !CYTHON_USE_MODULE_STATE && CYTHON_COMPILING_IN_CPYTHON_FREETHREADING)
  #endif
#endif
#ifndef CYTHON_COMPRESS_STRINGS
  #define CYTHON_COMPRESS_STRINGS 1
#endif
#ifndef CYTHON_FAST_PYCCALL
#define CYTHON_FAST_PYCCALL  CYTHON_FAST_PYCALL
#endif
#ifndef CYTHON_VECTORCALL
#if CYTHON_COMPILING_IN_LIMITED_API
#define CYTHON_VECTORCALL  (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
#else
#define CYTHON_VECTORCALL  (CYTHON_FAST_PYCCALL)
#endif
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(maybe_unused) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(maybe_unused)
        #define CYTHON_UNUSED [[maybe_unused]]
      #endif
    #endif
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
  #define CYTHON_MAYBE_UNUSED_VAR(x) CYTHON_UNUSED_VAR(x)
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON && !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_USE_CPP_STD_MOVE
  #if defined(__cplusplus) && (\
    __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600))
    #define CYTHON_USE_CPP_STD_MOVE 1
  #else
    #define CYTHON_USE_CPP_STD_MOVE 0
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#include <stdint.h>
typedef uintptr_t  __pyx_uintptr_t;
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(fallthrough) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(fallthrough)
        #define CYTHON_FALLTHROUGH [[fallthrough]]
      #endif
    #endif
    #ifndef CYTHON_FALLTHROUGH
      #if __has_cpp_attribute(clang::fallthrough)
        #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
      #elif __has_cpp_attribute(gnu::fallthrough)
        #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
      #endif
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif
#ifndef Py_UNREACHABLE
  #define Py_UNREACHABLE()  assert(0); abort()
#endif
#ifdef __cplusplus
  template <typename T>
  struct __PYX_IS_UNSIGNED_IMPL {static const bool value = T(0) < T(-1);};
  #define __PYX_IS_UNSIGNED(type) (__PYX_IS_UNSIGNED_IMPL<type>::value)
#else
  #define __PYX_IS_UNSIGNED(type) (((type)-1) > 0)
#endif
#if CYTHON_COMPILING_IN_PYPY == 1
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX < 0x030A0000)
#else
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX < 0x03090000)
#endif
#define __PYX_REINTERPRET_FUNCION(func_pointer, other_pointer) ((func_pointer)(void(*)(void))(other_pointer))

/* CInitCode */
#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

/* PythonCompatibility */
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#define __Pyx_BUILTIN_MODULE_NAME "builtins"
#define __Pyx_DefaultClassType PyType_Type
#if CYTHON_COMPILING_IN_LIMITED_API
    #ifndef CO_OPTIMIZED
    static int CO_OPTIMIZED;
    #endif
    #ifndef CO_NEWLOCALS
    static int CO_NEWLOCALS;
    #endif
    #ifndef CO_VARARGS
    static int CO_VARARGS;
    #endif
    #ifndef CO_VARKEYWORDS
    static int CO_VARKEYWORDS;
    #endif
    #ifndef CO_ASYNC_GENERATOR
    static int CO_ASYNC_GENERATOR;
    #endif
    #ifndef CO_GENERATOR
    static int CO_GENERATOR;
    #endif
    #ifndef CO_COROUTINE
    static int CO_COROUTINE;
    #endif
#else
    #ifndef CO_COROUTINE
      #define CO_COROUTINE 0x80
    #endif
    #ifndef CO_ASYNC_GENERATOR
      #define CO_ASYNC_GENERATOR 0x200
    #endif
#endif
static int __Pyx_init_co_variables(void);
#if PY_VERSION_HEX >= 0x030900A4 || defined(Py_IS_TYPE)
  #define __Pyx_IS_TYPE(ob, type) Py_IS_TYPE(ob, type)
#else
  #define __Pyx_IS_TYPE(ob, type) (((const PyObject*)ob)->ob_type == (type))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_Is)
  #define __Pyx_Py_Is(x, y)  Py_Is(x, y)
#else
  #define __Pyx_Py_Is(x, y) ((x) == (y))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsNone)
  #define __Pyx_Py_IsNone(ob) Py_IsNone(ob)
#else
  #define __Pyx_Py_IsNone(ob) __Pyx_Py_Is((ob), Py_None)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsTrue)
  #define __Pyx_Py_IsTrue(ob) Py_IsTrue(ob)
#else
  #define __Pyx_Py_IsTrue(ob) __Pyx_Py_Is((ob), Py_True)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsFalse)
  #define __Pyx_Py_IsFalse(ob) Py_IsFalse(ob)
#else
  #define __Pyx_Py_IsFalse(ob) __Pyx_Py_Is((ob), Py_False)
#endif
#define __Pyx_NoneAsNull(obj)  (__Pyx_Py_IsNone(obj) ? NULL : (obj))
#if PY_VERSION_HEX >= 0x030900F0 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyObject_GC_IsFinalized(o) PyObject_GC_IsFinalized(o)
#else
  #define __Pyx_PyObject_GC_IsFinalized(o) _PyGC_FINALIZED(o)
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef Py_TPFLAGS_SEQUENCE
  #define Py_TPFLAGS_SEQUENCE 0
#endif
#ifndef Py_TPFLAGS_MAPPING
  #define Py_TPFLAGS_MAPPING 0
#endif
#ifndef Py_TPFLAGS_IMMUTABLETYPE
  #define Py_TPFLAGS_IMMUTABLETYPE (1UL << 8)
#endif
#ifndef Py_TPFLAGS_DISALLOW_INSTANTIATION
  #define Py_TPFLAGS_DISALLOW_INSTANTIATION (1UL << 7)
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#ifndef METH_FASTCALL
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #if PY_VERSION_HEX >= 0x030d00A4
  #  define __Pyx_PyCFunctionFast PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords PyCFunctionFastWithKeywords
  #else
  #  define __Pyx_PyCFunctionFast _PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
  #endif
#endif
#if CYTHON_METH_FASTCALL
  #define __Pyx_METH_FASTCALL METH_FASTCALL
  #define __Pyx_PyCFunction_FastCall __Pyx_PyCFunctionFast
  #define __Pyx_PyCFunction_FastCallWithKeywords __Pyx_PyCFunctionFastWithKeywords
#else
  #define __Pyx_METH_FASTCALL METH_VARARGS
  #define __Pyx_PyCFunction_FastCall PyCFunction
  #define __Pyx_PyCFunction_FastCallWithKeywords PyCFunctionWithKeywords
#endif
#if CYTHON_VECTORCALL
  #define __pyx_vectorcallfunc vectorcallfunc
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  PY_VECTORCALL_ARGUMENTS_OFFSET
  #define __Pyx_PyVectorcall_NARGS(n)  PyVectorcall_NARGS((size_t)(n))
#else
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  0
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(n))
#endif
#if PY_VERSION_HEX >= 0x030900B1
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_CheckExact(func)
#else
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_Check(func)
#endif
#define __Pyx_CyOrPyCFunction_Check(func)  PyCFunction_Check(func)
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  (((PyCFunctionObject*)(func))->m_ml->ml_meth)
#elif !CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  PyCFunction_GET_FUNCTION(func)
#endif
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FLAGS(func)  (((PyCFunctionObject*)(func))->m_ml->ml_flags)
static CYTHON_INLINE PyObject* __Pyx_CyOrPyCFunction_GET_SELF(PyObject *func) {
    return (__Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_STATIC) ? NULL : ((PyCFunctionObject*)func)->m_self;
}
#endif
static CYTHON_INLINE int __Pyx__IsSameCFunction(PyObject *func, void (*cfunc)(void)) {
#if CYTHON_COMPILING_IN_LIMITED_API
    return PyCFunction_Check(func) && PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
#else
    return PyCFunction_Check(func) && PyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
#endif
}
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCFunction(func, cfunc)
#if PY_VERSION_HEX < 0x03090000 || (CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000)
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  ((void)m, PyType_FromSpecWithBases(s, b))
  typedef PyObject *(*__Pyx_PyCMethod)(PyObject *, PyTypeObject *, PyObject *const *, size_t, PyObject *);
#else
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  PyType_FromModuleAndSpec(m, s, b)
  #define __Pyx_PyCMethod  PyCMethod
#endif
#ifndef METH_METHOD
  #define METH_METHOD 0x200
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)
#elif CYTHON_COMPILING_IN_GRAAL && defined(GRAALPY_VERSION_NUM) && GRAALPY_VERSION_NUM > 0x19000000
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) GraalPyFrame_SetLineNumber((frame), (lineno))
#elif CYTHON_COMPILING_IN_GRAAL
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) _PyFrame_SetLineNumber((frame), (lineno))
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#elif !CYTHON_FAST_THREAD_STATE
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyThreadState_Current PyThreadState_GetUnchecked()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#endif
#if CYTHON_USE_MODULE_STATE
static CYTHON_INLINE void *__Pyx__PyModule_GetState(PyObject *op)
{
    void *result;
    result = PyModule_GetState(op);
    if (!result)
        Py_FatalError("Couldn't find the module state");
    return result;
}
#define __Pyx_PyModule_GetState(o) (__pyx_mstatetype *)__Pyx__PyModule_GetState(o)
#else
#define __Pyx_PyModule_GetState(op) ((void)op,__pyx_mstate_global)
#endif
#define __Pyx_PyObject_GetSlot(obj, name, func_ctype)  __Pyx_PyType_GetSlot(Py_TYPE((PyObject *) obj), name, func_ctype)
#define __Pyx_PyObject_TryGetSlot(obj, name, func_ctype) __Pyx_PyType_TryGetSlot(Py_TYPE(obj), name, func_ctype)
#define __Pyx_PyObject_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#define __Pyx_PyObject_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((type)->name)
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype) __Pyx_PyType_GetSlot(type, name, func_ctype)
  #define __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype) (((type)->sub) ? ((type)->sub->name) : NULL)
  #define __Pyx_PyType_TryGetSubSlot(type, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype)
#else
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((func_ctype) PyType_GetSlot((type), Py_##name))
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype)\
    ((__PYX_LIMITED_VERSION_HEX >= 0x030A0000 ||\
     (PyType_GetFlags(type) & Py_TPFLAGS_HEAPTYPE) || __Pyx_get_runtime_version() >= 0x030A0000) ?\
     __Pyx_PyType_GetSlot(type, name, func_ctype) : NULL)
  #define __Pyx_PyType_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSlot(obj, name, func_ctype)
  #define __Pyx_PyType_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSlot(obj, name, func_ctype)
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
#define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStrWithError(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStr(PyObject *dict, PyObject *name) {
    PyObject *res = __Pyx_PyDict_GetItemStrWithError(dict, name);
    if (res == NULL) PyErr_Clear();
    return res;
}
#elif !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000
#define __Pyx_PyDict_GetItemStrWithError  PyDict_GetItemWithError
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#else
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStrWithError(PyObject *dict, PyObject *name) {
#if CYTHON_COMPILING_IN_PYPY
    return PyDict_GetItem(dict, name);
#else
    PyDictEntry *ep;
    PyDictObject *mp = (PyDictObject*) dict;
    long hash = ((PyStringObject *) name)->ob_shash;
    assert(hash != -1);
    ep = (mp->ma_lookup)(mp, name, hash);
    if (ep == NULL) {
        return NULL;
    }
    return ep->me_value;
#endif
}
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#endif
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetFlags(tp)   (((PyTypeObject *)tp)->tp_flags)
  #define __Pyx_PyType_HasFeature(type, feature)  ((__Pyx_PyType_GetFlags(type) & (feature)) != 0)
#else
  #define __Pyx_PyType_GetFlags(tp)   (PyType_GetFlags((PyTypeObject *)tp))
  #define __Pyx_PyType_HasFeature(type, feature)  PyType_HasFeature(type, feature)
#endif
#define __Pyx_PyObject_GetIterNextFunc(iterator)  __Pyx_PyObject_GetSlot(iterator, tp_iternext, iternextfunc)
#if CYTHON_USE_TYPE_SPECS
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  {\
    PyTypeObject *type = Py_TYPE((PyObject*)obj);\
    assert(__Pyx_PyType_HasFeature(type, Py_TPFLAGS_HEAPTYPE));\
    PyObject_GC_Del(obj);\
    Py_DECREF(type);\
}
#else
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  PyObject_GC_Del(obj)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_ReadChar(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((void)u, 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((void)u, (0))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)u)
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)k, PyUnicode_ReadChar((PyObject*)(d), i))
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GetLength(u))
#else
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         ((int)PyUnicode_KIND(u))
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, (Py_UCS4) ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #if !defined(PyUnicode_DecodeUnicodeEscape)
    #define PyUnicode_DecodeUnicodeEscape(s, size, errors)  PyUnicode_Decode(s, size, "unicode_escape", errors)
  #endif
  #if !defined(PyUnicode_Contains)
    #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
  #endif
  #if !defined(PyByteArray_Check)
    #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
  #endif
  #if !defined(PyObject_Format)
    #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
  #endif
#endif
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && PyUnstable_Object_IsUniquelyReferenced(obj)) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#elif CYTHON_COMPILING_IN_CPYTHON
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && Py_REFCNT(obj) == 1) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#else
  #define __Pyx_PySequence_ListKeepNew(obj)  PySequence_List(obj)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        __Pyx_IS_TYPE(obj, &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
enum __Pyx_ReferenceSharing {
  __Pyx_ReferenceSharing_DefinitelyUnique, // We created it so we know it's unshared - no need to check
  __Pyx_ReferenceSharing_OwnStrongReference,
  __Pyx_ReferenceSharing_FunctionArgument,
  __Pyx_ReferenceSharing_SharedReference, // Never trust it to be unshared because it's a global or similar
};
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && PY_VERSION_HEX >= 0x030E0000
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing)\
    (sharing == __Pyx_ReferenceSharing_DefinitelyUnique ? 1 :\
      (sharing == __Pyx_ReferenceSharing_FunctionArgument ? PyUnstable_Object_IsUniqueReferencedTemporary(o) :\
      (sharing == __Pyx_ReferenceSharing_OwnStrongReference ? PyUnstable_Object_IsUniquelyReferenced(o) : 0)))
#elif (CYTHON_COMPILING_IN_CPYTHON && !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING) || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing) (((void)sharing), Py_REFCNT(o) == 1)
#else
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing) (((void)o), ((void)sharing), 0)
#endif
#if CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    #define __Pyx_PyList_GetItemRef(o, i) PyList_GetItemRef(o, i)
  #elif CYTHON_COMPILING_IN_LIMITED_API || !CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyList_GetItemRef(o, i) (likely((i) >= 0) ? PySequence_GetItem(o, i) : (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
  #else
    #define __Pyx_PyList_GetItemRef(o, i) PySequence_ITEM(o, i)
  #endif
#elif CYTHON_COMPILING_IN_LIMITED_API || !CYTHON_ASSUME_SAFE_MACROS
  #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    #define __Pyx_PyList_GetItemRef(o, i) PyList_GetItemRef(o, i)
  #else
    #define __Pyx_PyList_GetItemRef(o, i) __Pyx_XNewRef(PyList_GetItem(o, i))
  #endif
#else
  #define __Pyx_PyList_GetItemRef(o, i) __Pyx_NewRef(PyList_GET_ITEM(o, i))
#endif
#if CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS && !CYTHON_COMPILING_IN_LIMITED_API && CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PyList_GetItemRefFast(o, i, unsafe_shared) (__Pyx_IS_UNIQUELY_REFERENCED(o, unsafe_shared) ?\
    __Pyx_NewRef(PyList_GET_ITEM(o, i)) : __Pyx_PyList_GetItemRef(o, i))
#else
  #define __Pyx_PyList_GetItemRefFast(o, i, unsafe_shared) __Pyx_PyList_GetItemRef(o, i)
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyDict_GetItemRef(dict, key, result) PyDict_GetItemRef(dict, key, result)
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyObject_GetItem(dict, key);
  if (*result == NULL) {
    if (PyErr_ExceptionMatches(PyExc_KeyError)) {
      PyErr_Clear();
      return 0;
    }
    return -1;
  }
  return 1;
}
#else
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyDict_GetItemWithError(dict, key);
  if (*result == NULL) {
    return PyErr_Occurred() ? -1 : 0;
  }
  Py_INCREF(*result);
  return 1;
}
#endif
#if defined(CYTHON_DEBUG_VISIT_CONST) && CYTHON_DEBUG_VISIT_CONST
  #define __Pyx_VISIT_CONST(obj)  Py_VISIT(obj)
#else
  #define __Pyx_VISIT_CONST(obj)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_ITEM(o, i) PySequence_ITEM(o, i)
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) (PyTuple_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GET_ITEM(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) (PyList_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GET_ITEM(o, i)
#else
  #define __Pyx_PySequence_ITEM(o, i) PySequence_GetItem(o, i)
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) PyTuple_SetItem(o, i, v)
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GetItem(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) PyList_SetItem(o, i, v)
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GetItem(o, i)
#endif
#if CYTHON_ASSUME_SAFE_SIZE
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_GET_SIZE(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_GET_SIZE(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_GET_SIZE(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_GET_SIZE(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_GET_SIZE(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GET_LENGTH(o)
#else
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_Size(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_Size(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_Size(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_Size(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_Size(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GetLength(o)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_InternFromString)
  #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
#endif
#define __Pyx_PyLong_FromHash_t PyLong_FromSsize_t
#define __Pyx_PyLong_AsHash_t   __Pyx_PyIndex_AsSsize_t
#if __PYX_LIMITED_VERSION_HEX >= 0x030A0000
    #define __Pyx_PySendResult PySendResult
#else
    typedef enum {
        PYGEN_RETURN = 0,
        PYGEN_ERROR = -1,
        PYGEN_NEXT = 1,
    } __Pyx_PySendResult;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX < 0x030A00A3
  typedef __Pyx_PySendResult (*__Pyx_pyiter_sendfunc)(PyObject *iter, PyObject *value, PyObject **result);
#else
  #define __Pyx_pyiter_sendfunc sendfunc
#endif
#if !CYTHON_USE_AM_SEND
#define __PYX_HAS_PY_AM_SEND 0
#elif __PYX_LIMITED_VERSION_HEX >= 0x030A0000
#define __PYX_HAS_PY_AM_SEND 1
#else
#define __PYX_HAS_PY_AM_SEND 2  // our own backported implementation
#endif
#if __PYX_HAS_PY_AM_SEND < 2
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#else
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
        __Pyx_pyiter_sendfunc am_send;
    } __Pyx_PyAsyncMethodsStruct;
    #define __Pyx_SlotTpAsAsync(s) ((PyAsyncMethods*)(s))
#endif
#if CYTHON_USE_AM_SEND && PY_VERSION_HEX < 0x030A00F0
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (1UL << 21)
#else
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (0)
#endif
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyInterpreterState_Get() PyInterpreterState_Get()
#else
#define __Pyx_PyInterpreterState_Get() PyThreadState_Get()->interp
#endif
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030A0000
#ifdef __cplusplus
extern "C"
#endif
PyAPI_FUNC(void *) PyMem_Calloc(size_t nelem, size_t elsize);
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_init_co_variable(PyObject *inspect, const char* name, int *write_to) {
    int value;
    PyObject *py_value = PyObject_GetAttrString(inspect, name);
    if (!py_value) return 0;
    value = (int) PyLong_AsLong(py_value);
    Py_DECREF(py_value);
    *write_to = value;
    return value != -1 || !PyErr_Occurred();
}
static int __Pyx_init_co_variables(void) {
    PyObject *inspect;
    int result;
    inspect = PyImport_ImportModule("inspect");
    result =
#if !defined(CO_OPTIMIZED)
        __Pyx_init_co_variable(inspect, "CO_OPTIMIZED", &CO_OPTIMIZED) &&
#endif
#if !defined(CO_NEWLOCALS)
        __Pyx_init_co_variable(inspect, "CO_NEWLOCALS", &CO_NEWLOCALS) &&
#endif
#if !defined(CO_VARARGS)
        __Pyx_init_co_variable(inspect, "CO_VARARGS", &CO_VARARGS) &&
#endif
#if !defined(CO_VARKEYWORDS)
        __Pyx_init_co_variable(inspect, "CO_VARKEYWORDS", &CO_VARKEYWORDS) &&
#endif
#if !defined(CO_ASYNC_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_ASYNC_GENERATOR", &CO_ASYNC_GENERATOR) &&
#endif
#if !defined(CO_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_GENERATOR", &CO_GENERATOR) &&
#endif
#if !defined(CO_COROUTINE)
        __Pyx_init_co_variable(inspect, "CO_COROUTINE", &CO_COROUTINE) &&
#endif
        1;
    Py_DECREF(inspect);
    return result ? 0 : -1;
}
#else
static int __Pyx_init_co_variables(void) {
    return 0;  // It's a limited API-only feature
}
#endif

/* MathInitCode */
#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #ifndef _USE_MATH_DEFINES
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#ifndef CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#define CYTHON_CLINE_IN_TRACEBACK_RUNTIME 0
#endif
#ifndef CYTHON_CLINE_IN_TRACEBACK
#define CYTHON_CLINE_IN_TRACEBACK CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#endif
#if CYTHON_CLINE_IN_TRACEBACK
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; __pyx_clineno = __LINE__; (void) __pyx_clineno; }
#else
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; (void) __pyx_clineno; }
#endif
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifdef CYTHON_EXTERN_C
    #undef __PYX_EXTERN_C
    #define __PYX_EXTERN_C CYTHON_EXTERN_C
#elif defined(__PYX_EXTERN_C)
    #ifdef _MSC_VER
    #pragma message ("Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.")
    #else
    #warning Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.
    #endif
#else
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__tables__hdf5extension
#define __PYX_HAVE_API__tables__hdf5extension
/* Early includes */
#include <string.h>
#include <stdio.h>

    /* Using NumPy API declarations from "numpy/__init__.cython-30.pxd" */
    
#include "numpy/arrayobject.h"
#include "numpy/ndarrayobject.h"
#include "numpy/ndarraytypes.h"
#include "numpy/arrayscalars.h"
#include "numpy/ufuncobject.h"
#include "time.h"
#include "hdf5.h"
#include "H5ATTR.h"
#include "H5ARRAY.h"
#include "utils.h"
#include "typeconv.h"
#include "blosc2_filter.h"
#include "blosc_filter.h"
#include <stdlib.h>
#include <stddef.h>
#include "H5ARRAY-opt.h"
#include "H5VLARRAY.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifdef CYTHON_FREETHREADING_COMPATIBLE
#if CYTHON_FREETHREADING_COMPATIBLE
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_NOT_USED
#else
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_USED
#endif
#else
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_NOT_USED
#endif
#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s);
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char*);
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AS_STRING(s)
#else
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AsString(s)
#endif
#define __Pyx_PyObject_AsWritableString(s)    ((char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#define __Pyx_PyUnicode_FromOrdinal(o)       PyUnicode_FromOrdinal((int)o)
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
static CYTHON_INLINE PyObject *__Pyx_NewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_NewRef)
    return Py_NewRef(obj);
#else
    Py_INCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_XNewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_XNewRef)
    return Py_XNewRef(obj);
#else
    Py_XINCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b);
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __Pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AS_DOUBLE(x)
#else
#define __Pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AsDouble(x)
#endif
#define __Pyx_PyFloat_AsFloat(x) ((float) __Pyx_PyFloat_AsDouble(x))
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_VERSION_HEX >= 0x030C00A7
  #ifndef _PyLong_SIGN_MASK
    #define _PyLong_SIGN_MASK 3
  #endif
  #ifndef _PyLong_NON_SIZE_BITS
    #define _PyLong_NON_SIZE_BITS 3
  #endif
  #define __Pyx_PyLong_Sign(x)  (((PyLongObject*)x)->long_value.lv_tag & _PyLong_SIGN_MASK)
  #define __Pyx_PyLong_IsNeg(x)  ((__Pyx_PyLong_Sign(x) & 2) != 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (!__Pyx_PyLong_IsNeg(x))
  #define __Pyx_PyLong_IsZero(x)  (__Pyx_PyLong_Sign(x) & 1)
  #define __Pyx_PyLong_IsPos(x)  (__Pyx_PyLong_Sign(x) == 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  (__Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  ((Py_ssize_t) (((PyLongObject*)x)->long_value.lv_tag >> _PyLong_NON_SIZE_BITS))
  #define __Pyx_PyLong_SignedDigitCount(x)\
        ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * __Pyx_PyLong_DigitCount(x))
  #if defined(PyUnstable_Long_IsCompact) && defined(PyUnstable_Long_CompactValue)
    #define __Pyx_PyLong_IsCompact(x)     PyUnstable_Long_IsCompact((PyLongObject*) x)
    #define __Pyx_PyLong_CompactValue(x)  PyUnstable_Long_CompactValue((PyLongObject*) x)
  #else
    #define __Pyx_PyLong_IsCompact(x)     (((PyLongObject*)x)->long_value.lv_tag < (2 << _PyLong_NON_SIZE_BITS))
    #define __Pyx_PyLong_CompactValue(x)  ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * (Py_ssize_t) __Pyx_PyLong_Digits(x)[0])
  #endif
  typedef Py_ssize_t  __Pyx_compact_pylong;
  typedef size_t  __Pyx_compact_upylong;
  #else
  #define __Pyx_PyLong_IsNeg(x)  (Py_SIZE(x) < 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (Py_SIZE(x) >= 0)
  #define __Pyx_PyLong_IsZero(x)  (Py_SIZE(x) == 0)
  #define __Pyx_PyLong_IsPos(x)  (Py_SIZE(x) > 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  ((Py_SIZE(x) == 0) ? 0 : __Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  __Pyx_sst_abs(Py_SIZE(x))
  #define __Pyx_PyLong_SignedDigitCount(x)  Py_SIZE(x)
  #define __Pyx_PyLong_IsCompact(x)  (Py_SIZE(x) == 0 || Py_SIZE(x) == 1 || Py_SIZE(x) == -1)
  #define __Pyx_PyLong_CompactValue(x)\
        ((Py_SIZE(x) == 0) ? (sdigit) 0 : ((Py_SIZE(x) < 0) ? -(sdigit)__Pyx_PyLong_Digits(x)[0] : (sdigit)__Pyx_PyLong_Digits(x)[0]))
  typedef sdigit  __Pyx_compact_pylong;
  typedef digit  __Pyx_compact_upylong;
  #endif
  #if PY_VERSION_HEX >= 0x030C00A5
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->long_value.ob_digit)
  #else
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->ob_digit)
  #endif
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#elif __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeASCII(c_str, size, NULL)
#else
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
/* PretendToInitialize */
#ifdef __cplusplus
#if __cplusplus > 201103L
#include <type_traits>
#endif
template <typename T>
static void __Pyx_pretend_to_initialize(T* ptr) {
#if __cplusplus > 201103L
    if ((std::is_trivially_default_constructible<T>::value))
#endif
        *ptr = T();
    (void)ptr;
}
#else
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }
#endif


#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_m = NULL;
#endif
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * const __pyx_cfilenm = __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif (defined(_Complex_I) && !defined(_MSC_VER)) || ((defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) && !defined(__STDC_NO_COMPLEX__) && !defined(_MSC_VER))
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif

/* #### Code section: filename_table ### */

static const char* const __pyx_f[] = {
  "tables/hdf5extension.pyx",
  "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd",
  "(tree fragment)",
  "cpython/type.pxd",
};
/* #### Code section: utility_code_proto_before_types ### */
/* Atomics.proto (used by UnpackUnboundCMethod) */
#include <pythread.h>
#ifndef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 1
#endif
#define __PYX_CYTHON_ATOMICS_ENABLED() CYTHON_ATOMICS
#define __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING() CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __pyx_atomic_int_type int
#define __pyx_nonatomic_int_type int
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__))
    #include <stdatomic.h>
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)))
    #include <atomic>
#endif
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__) &&\
                       ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type atomic_int
    #define __pyx_atomic_ptr_type atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) atomic_fetch_add_explicit(value, 1, memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) atomic_fetch_add_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) atomic_fetch_sub_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) atomic_load(value)
    #define __pyx_atomic_store(value, new_value) atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) atomic_load_explicit(value, memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) atomic_load_explicit(value, memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #define __pyx_atomic_pointer_cmp_exchange(value, expected, desired) atomic_compare_exchange_strong(value, expected, desired)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C atomics"
    #endif
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)) &&\
                    ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type std::atomic_int
    #define __pyx_atomic_ptr_type std::atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) std::atomic_fetch_sub_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) std::atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) std::atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) std::atomic_load(value)
    #define __pyx_atomic_store(value, new_value) std::atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) std::atomic_load_explicit(value, std::memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) std::atomic_load_explicit(value, std::memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) std::atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #define __pyx_atomic_pointer_cmp_exchange(value, expected, desired) std::atomic_compare_exchange_strong(value, expected, desired)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C++ atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C++ atomics"
    #endif
#elif CYTHON_ATOMICS && (__GNUC__ >= 5 || (__GNUC__ == 4 &&\
                    (__GNUC_MINOR__ > 1 ||\
                    (__GNUC_MINOR__ == 1 && __GNUC_PATCHLEVEL__ >= 2))))
    #define __pyx_atomic_ptr_type void*
    #define __pyx_nonatomic_ptr_type void*
    #define __pyx_atomic_incr_relaxed(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) __sync_fetch_and_sub(value, 1)
    #define __pyx_atomic_sub(value, arg) __sync_fetch_and_sub(value, arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = __sync_val_compare_and_swap(value, *expected, desired);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_store(value, new_value) __sync_lock_test_and_set(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_load_acquire(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) __sync_lock_test_and_set(value, (__pyx_atomic_ptr_type)new_value)
    static CYTHON_INLINE int __pyx_atomic_pointer_cmp_exchange(__pyx_atomic_ptr_type* value, __pyx_nonatomic_ptr_type* expected, __pyx_nonatomic_ptr_type desired) {
        __pyx_nonatomic_ptr_type old = __sync_val_compare_and_swap(value, *expected, desired);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using GNU atomics"
    #endif
#elif CYTHON_ATOMICS && defined(_MSC_VER)
    #include <intrin.h>
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type long
    #define __pyx_atomic_ptr_type void*
    #undef __pyx_nonatomic_int_type
    #define __pyx_nonatomic_int_type long
    #define __pyx_nonatomic_ptr_type void*
    #pragma intrinsic (_InterlockedExchangeAdd, _InterlockedExchange, _InterlockedCompareExchange, _InterlockedCompareExchangePointer, _InterlockedExchangePointer)
    #define __pyx_atomic_incr_relaxed(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) _InterlockedExchangeAdd(value, -1)
    #define __pyx_atomic_sub(value, arg) _InterlockedExchangeAdd(value, -arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = _InterlockedCompareExchange(value, desired, *expected);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) _InterlockedExchangeAdd(value, 0)
    #define __pyx_atomic_store(value, new_value) _InterlockedExchange(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) *(void * volatile *)value
    #define __pyx_atomic_pointer_load_acquire(value) _InterlockedCompareExchangePointer(value, 0, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) _InterlockedExchangePointer(value, (__pyx_atomic_ptr_type)new_value)
    static CYTHON_INLINE int __pyx_atomic_pointer_cmp_exchange(__pyx_atomic_ptr_type* value, __pyx_nonatomic_ptr_type* expected, __pyx_nonatomic_ptr_type desired) {
        __pyx_atomic_ptr_type old = _InterlockedCompareExchangePointer(value, desired, *expected);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #ifdef __PYX_DEBUG_ATOMICS
        #pragma message ("Using MSVC atomics")
    #endif
#else
    #undef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 0
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Not using atomics"
    #endif
#endif

/* CriticalSectionsDefinition.proto (used by CriticalSections) */
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyCriticalSection void*
#define __Pyx_PyCriticalSection2 void*
#define __Pyx_PyCriticalSection_End(cs)
#define __Pyx_PyCriticalSection2_End(cs)
#else
#define __Pyx_PyCriticalSection PyCriticalSection
#define __Pyx_PyCriticalSection2 PyCriticalSection2
#define __Pyx_PyCriticalSection_End PyCriticalSection_End
#define __Pyx_PyCriticalSection2_End PyCriticalSection2_End
#endif

/* CriticalSections.proto (used by ParseKeywordsImpl) */
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyCriticalSection_Begin(cs, arg) (void)(cs)
#define __Pyx_PyCriticalSection2_Begin(cs, arg1, arg2) (void)(cs)
#else
#define __Pyx_PyCriticalSection_Begin PyCriticalSection_Begin
#define __Pyx_PyCriticalSection2_Begin PyCriticalSection2_Begin
#endif
#if PY_VERSION_HEX < 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_BEGIN_CRITICAL_SECTION(o) {
#define __Pyx_END_CRITICAL_SECTION() }
#else
#define __Pyx_BEGIN_CRITICAL_SECTION Py_BEGIN_CRITICAL_SECTION
#define __Pyx_END_CRITICAL_SECTION Py_END_CRITICAL_SECTION
#endif

/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()

/* IncludeStructmemberH.proto (used by FixUpExtensionType) */
#include <structmember.h>

/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* #### Code section: numeric_typedefs ### */

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":761
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
*/
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":762
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
*/
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":763
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * 
*/
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":764
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_uint8      uint8_t
*/
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":766
 * ctypedef npy_int64      int64_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
*/
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":767
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
*/
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":768
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * 
*/
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":769
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_float32    float32_t
*/
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":771
 * ctypedef npy_uint64     uint64_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
*/
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":772
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
*/
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":779
 * ctypedef double complex complex128_t
 * 
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
*/
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":780
 * 
 * ctypedef npy_longlong   longlong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
*/
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":782
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
*/
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":783
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
*/
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":785
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
*/
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":786
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
*/
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":787
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef float complex       cfloat_t
*/
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
/* #### Code section: complex_type_declarations ### */
/* Declarations.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

/* Declarations.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);

/* Declarations.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    typedef ::std::complex< long double > __pyx_t_long_double_complex;
  #else
    typedef long double _Complex __pyx_t_long_double_complex;
  #endif
#else
    typedef struct { long double real, imag; } __pyx_t_long_double_complex;
#endif
static CYTHON_INLINE __pyx_t_long_double_complex __pyx_t_long_double_complex_from_parts(long double, long double);

/* #### Code section: type_declarations ### */

/*--- Type declarations ---*/
struct __pyx_obj_6tables_13hdf5extension_Node;
struct __pyx_obj_6tables_13hdf5extension_Leaf;
struct __pyx_obj_6tables_13hdf5extension_Array;
struct __pyx_obj_6tables_13hdf5extension_File;
struct __pyx_obj_6tables_13hdf5extension_AttributeSet;
struct __pyx_obj_6tables_13hdf5extension_Group;
struct __pyx_obj_6tables_13hdf5extension_VLArray;
struct __pyx_obj_6tables_13hdf5extension_UnImplemented;

/* "tables/hdf5extension.pxd":17
 * 
 * # Declaration of instance variables for shared classes
 * cdef class Node:             # <<<<<<<<<<<<<<
 *   cdef object name
 *   cdef hid_t  parent_id
*/
struct __pyx_obj_6tables_13hdf5extension_Node {
  PyObject_HEAD
  PyObject *name;
  hid_t parent_id;
};


/* "tables/hdf5extension.pxd":21
 *   cdef hid_t  parent_id
 * 
 * cdef class Leaf(Node):             # <<<<<<<<<<<<<<
 *   cdef hid_t   dataset_id
 *   cdef hid_t   type_id
*/
struct __pyx_obj_6tables_13hdf5extension_Leaf {
  struct __pyx_obj_6tables_13hdf5extension_Node __pyx_base;
  struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf *__pyx_vtab;
  hid_t dataset_id;
  hid_t type_id;
  hid_t base_type_id;
  hid_t disk_type_id;
  hsize_t *dims;
};


/* "tables/hdf5extension.pxd":30
 *   cdef _convert_time64(self, ndarray nparr, int sense)
 * 
 * cdef class Array(Leaf):             # <<<<<<<<<<<<<<
 *   cdef int      rank
 *   cdef hsize_t *maxdims
*/
struct __pyx_obj_6tables_13hdf5extension_Array {
  struct __pyx_obj_6tables_13hdf5extension_Leaf __pyx_base;
  int rank;
  hsize_t *maxdims;
  hsize_t *dims_chunk;
  hbool_t blosc2_support_read;
  hbool_t blosc2_support_write;
};


/* "tables/hdf5extension.pyx":440
 * # Python classes)
 * 
 * cdef class File:             # <<<<<<<<<<<<<<
 *   cdef hid_t   file_id
 *   cdef hid_t   access_plist
*/
struct __pyx_obj_6tables_13hdf5extension_File {
  PyObject_HEAD
  hid_t file_id;
  hid_t access_plist;
  PyObject *name;
};


/* "tables/hdf5extension.pyx":757
 * 
 * 
 * cdef class AttributeSet:             # <<<<<<<<<<<<<<
 *   cdef object name
 * 
*/
struct __pyx_obj_6tables_13hdf5extension_AttributeSet {
  PyObject_HEAD
  PyObject *name;
};


/* "tables/hdf5extension.pyx":1068
 * 
 * 
 * cdef class Group(Node):             # <<<<<<<<<<<<<<
 *   cdef hid_t   group_id
 * 
*/
struct __pyx_obj_6tables_13hdf5extension_Group {
  struct __pyx_obj_6tables_13hdf5extension_Node __pyx_base;
  hid_t group_id;
};


/* "tables/hdf5extension.pyx":2158
 * 
 * 
 * cdef class VLArray(Leaf):             # <<<<<<<<<<<<<<
 *   # Instance variables
 *   cdef hsize_t nrecords
*/
struct __pyx_obj_6tables_13hdf5extension_VLArray {
  struct __pyx_obj_6tables_13hdf5extension_Leaf __pyx_base;
  hsize_t nrecords;
};


/* "tables/hdf5extension.pyx":2456
 * 
 * 
 * cdef class UnImplemented(Leaf):             # <<<<<<<<<<<<<<
 * 
 *   def _open_unimplemented(self):
*/
struct __pyx_obj_6tables_13hdf5extension_UnImplemented {
  struct __pyx_obj_6tables_13hdf5extension_Leaf __pyx_base;
};



/* "tables/hdf5extension.pyx":1232
 * 
 * 
 * cdef class Leaf(Node):             # <<<<<<<<<<<<<<
 *   # Instance variables declared in .pxd
 * 
*/

struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf {
  PyObject *(*_get_type_ids)(struct __pyx_obj_6tables_13hdf5extension_Leaf *);
  PyObject *(*_convert_time64)(struct __pyx_obj_6tables_13hdf5extension_Leaf *, PyArrayObject *, int);
};
static struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf *__pyx_vtabptr_6tables_13hdf5extension_Leaf;


/* "tables/hdf5extension.pyx":1466
 *     return (opt_read, opt_write)
 * 
 * cdef class Array(Leaf):             # <<<<<<<<<<<<<<
 *   # Instance variables declared in .pxd
 * 
*/

struct __pyx_vtabstruct_6tables_13hdf5extension_Array {
  struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf __pyx_base;
};
static struct __pyx_vtabstruct_6tables_13hdf5extension_Array *__pyx_vtabptr_6tables_13hdf5extension_Array;


/* "tables/hdf5extension.pyx":2158
 * 
 * 
 * cdef class VLArray(Leaf):             # <<<<<<<<<<<<<<
 *   # Instance variables
 *   cdef hsize_t nrecords
*/

struct __pyx_vtabstruct_6tables_13hdf5extension_VLArray {
  struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf __pyx_base;
};
static struct __pyx_vtabstruct_6tables_13hdf5extension_VLArray *__pyx_vtabptr_6tables_13hdf5extension_VLArray;


/* "tables/hdf5extension.pyx":2456
 * 
 * 
 * cdef class UnImplemented(Leaf):             # <<<<<<<<<<<<<<
 * 
 *   def _open_unimplemented(self):
*/

struct __pyx_vtabstruct_6tables_13hdf5extension_UnImplemented {
  struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf __pyx_base;
};
static struct __pyx_vtabstruct_6tables_13hdf5extension_UnImplemented *__pyx_vtabptr_6tables_13hdf5extension_UnImplemented;
/* #### Code section: utility_code_proto ### */

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, Py_ssize_t);
    void (*DECREF)(void*, PyObject*, Py_ssize_t);
    void (*GOTREF)(void*, PyObject*, Py_ssize_t);
    void (*GIVEREF)(void*, PyObject*, Py_ssize_t);
    void* (*SetupContext)(const char*, Py_ssize_t, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_XINCREF(r)  do { if((r) == NULL); else {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) == NULL); else {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) == NULL); else {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) == NULL); else {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_Py_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; Py_XDECREF(tmp);\
    } while (0)
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyErrExceptionMatches.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto (used by PyErrFetchRestore) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#if PY_VERSION_HEX >= 0x030C00A6
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->current_exception != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->current_exception ? (PyObject*) Py_TYPE(__pyx_tstate->current_exception) : (PyObject*) NULL)
#else
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->curexc_type != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->curexc_type)
#endif
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  (PyErr_Occurred() != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A6
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* PyObjectGetAttrStr.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetAttrStrNoError.proto (used by GetBuiltinName) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* GetTopmostException.proto (used by SaveResetException) */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyImportError_Check.proto */
#define __Pyx_PyExc_ImportError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_ImportError)

/* PyObjectCall.proto (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectFastCall.proto */
#define __Pyx_PyObject_FastCall(func, args, nargs)  __Pyx_PyObject_FastCallDict(func, args, (size_t)(nargs), NULL)
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs);

/* RaiseException.export */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* ListAppend.proto (used by append) */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectCall2Args.proto (used by PyObjectCallMethod1) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectCallOneArg.proto (used by PyObjectCallMethod1) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectGetMethod.proto (used by PyObjectCallMethod1) */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);
#endif

/* PyObjectCallMethod1.proto (used by append) */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* append.proto */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x);

/* PyDictVersioning.proto (used by GetModuleGlobalName) */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __Pyx_XNewRef(__pyx_dict_cached_value);\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_mstate_global->__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyObjectFastCallMethod.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyObject_FastCallMethod(name, args, nargsf) PyObject_VectorcallMethod(name, args, nargsf, NULL)
#else
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf);
#endif

/* PyTypeError_Check.proto */
#define __Pyx_PyExc_TypeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_TypeError)

/* RaiseUnboundLocalError.proto */
static void __Pyx_RaiseUnboundLocalError(const char *varname);

/* TupleAndListFromArray.proto (used by fastcall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n);
#endif
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject* __Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n);
#endif

/* IncludeStringH.proto (used by BytesEquals) */
#include <string.h>

/* BytesEquals.proto (used by UnicodeEquals) */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto (used by fastcall) */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* fastcall.proto */
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_PySequence_ITEM(args, i)
#elif CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_NewRef(__Pyx_PyTuple_GET_ITEM(args, i))
#else
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_XNewRef(PyTuple_GetItem(args, i))
#endif
#define __Pyx_NumKwargs_VARARGS(kwds) PyDict_Size(kwds)
#define __Pyx_KwValues_VARARGS(args, nargs) NULL
#define __Pyx_GetKwValue_VARARGS(kw, kwvalues, s) __Pyx_PyDict_GetItemStrWithError(kw, s)
#define __Pyx_KwargsAsDict_VARARGS(kw, kwvalues) PyDict_Copy(kw)
#if CYTHON_METH_FASTCALL
    #define __Pyx_ArgRef_FASTCALL(args, i) __Pyx_NewRef(args[i])
    #define __Pyx_NumKwargs_FASTCALL(kwds) __Pyx_PyTuple_GET_SIZE(kwds)
    #define __Pyx_KwValues_FASTCALL(args, nargs) ((args) + (nargs))
    static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues);
  #else
    #define __Pyx_KwargsAsDict_FASTCALL(kw, kwvalues) _PyStack_AsDict(kwvalues, kw)
  #endif
#else
    #define __Pyx_ArgRef_FASTCALL __Pyx_ArgRef_VARARGS
    #define __Pyx_NumKwargs_FASTCALL __Pyx_NumKwargs_VARARGS
    #define __Pyx_KwValues_FASTCALL __Pyx_KwValues_VARARGS
    #define __Pyx_GetKwValue_FASTCALL __Pyx_GetKwValue_VARARGS
    #define __Pyx_KwargsAsDict_FASTCALL __Pyx_KwargsAsDict_VARARGS
#endif
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) PyTuple_GetSlice(args, start, stop)
#if CYTHON_METH_FASTCALL || (CYTHON_COMPILING_IN_CPYTHON && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) __Pyx_PyTuple_FromArray(args + start, stop - start)
#else
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) PyTuple_GetSlice(args, start, stop)
#endif

/* py_dict_items.proto (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d);

/* CallCFunction.proto (used by CallUnboundCMethod0) */
#define __Pyx_CallCFunction(cfunc, self, args)\
    ((PyCFunction)(void(*)(void))(cfunc)->func)(self, args)
#define __Pyx_CallCFunctionWithKeywords(cfunc, self, args, kwargs)\
    ((PyCFunctionWithKeywords)(void(*)(void))(cfunc)->func)(self, args, kwargs)
#define __Pyx_CallCFunctionFast(cfunc, self, args, nargs)\
    ((__Pyx_PyCFunctionFast)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs)
#define __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, nargs, kwnames)\
    ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs, kwnames)

/* UnpackUnboundCMethod.proto (used by CallUnboundCMethod0) */
typedef struct {
    PyObject *type;
    PyObject **method_name;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && CYTHON_ATOMICS
    __pyx_atomic_int_type initialized;
#endif
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
static CYTHON_INLINE int __Pyx_CachedCFunction_GetAndSetInitializing(__Pyx_CachedCFunction *cfunc) {
#if !CYTHON_ATOMICS
    return 1;
#else
    __pyx_nonatomic_int_type expected = 0;
    if (__pyx_atomic_int_cmp_exchange(&cfunc->initialized, &expected, 1)) {
        return 0;
    }
    return expected;
#endif
}
static CYTHON_INLINE void __Pyx_CachedCFunction_SetFinishedInitializing(__Pyx_CachedCFunction *cfunc) {
#if CYTHON_ATOMICS
    __pyx_atomic_store(&cfunc->initialized, 2);
#endif
}
#else
#define __Pyx_CachedCFunction_GetAndSetInitializing(cfunc) 2
#define __Pyx_CachedCFunction_SetFinishedInitializing(cfunc)
#endif

/* CallUnboundCMethod0.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* py_dict_values.proto (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d);

/* OwnedDictNext.proto (used by ParseKeywordsImpl) */
#if CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, PyObject **ppos, PyObject **pkey, PyObject **pvalue);
#else
CYTHON_INLINE
static int __Pyx_PyDict_NextRef(PyObject *p, Py_ssize_t *ppos, PyObject **pkey, PyObject **pvalue);
#endif

/* RaiseDoubleKeywords.proto (used by ParseKeywordsImpl) */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywordsImpl.export */
static int __Pyx_ParseKeywordsTuple(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);
static int __Pyx_ParseKeywordDictToDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name
);
static int __Pyx_ParseKeywordDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);

/* CallUnboundCMethod2.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* ParseKeywords.proto */
static CYTHON_INLINE int __Pyx_ParseKeywords(
    PyObject *kwds, PyObject *const *kwvalues, PyObject ** const argnames[],
    PyObject *kwds2, PyObject *values[],
    Py_ssize_t num_pos_args, Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* DictGetItem.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* PyValueError_Check.proto */
#define __Pyx_PyExc_ValueError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_ValueError)

/* dict_getitem_default.proto */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value);

/* CallUnboundCMethod1.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* RaiseUnexpectedTypeError.proto */
static int __Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj);

/* PyObjectDelAttr.proto (used by PyObjectSetAttrStr) */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
#define __Pyx_PyObject_DelAttr(o, n) PyObject_SetAttr(o, n, NULL)
#else
#define __Pyx_PyObject_DelAttr(o, n) PyObject_DelAttr(o, n)
#endif

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   __Pyx_PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* AssertionsEnabled.proto */
#if CYTHON_COMPILING_IN_LIMITED_API  ||  PY_VERSION_HEX >= 0x030C0000
  static int __pyx_assertions_enabled_flag;
  #define __pyx_assertions_enabled() (__pyx_assertions_enabled_flag)
  #if __clang__ || __GNUC__
  __attribute__((no_sanitize("thread")))
  #endif
  static int __Pyx_init_assertions_enabled(void) {
    PyObject *builtins, *debug, *debug_str;
    int flag;
    builtins = PyEval_GetBuiltins();
    if (!builtins) goto bad;
    debug_str = PyUnicode_FromStringAndSize("__debug__", 9);
    if (!debug_str) goto bad;
    debug = PyObject_GetItem(builtins, debug_str);
    Py_DECREF(debug_str);
    if (!debug) goto bad;
    flag = PyObject_IsTrue(debug);
    Py_DECREF(debug);
    if (flag == -1) goto bad;
    __pyx_assertions_enabled_flag = flag;
    return 0;
  bad:
    __pyx_assertions_enabled_flag = 1;
    return -1;
  }
#else
  #define __Pyx_init_assertions_enabled()  (0)
  #define __pyx_assertions_enabled()  (!Py_OptimizeFlag)
#endif

/* PyAssertionError_Check.proto */
#define __Pyx_PyExc_AssertionError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_AssertionError)

/* PyRuntimeError_Check.proto */
#define __Pyx_PyExc_RuntimeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_RuntimeError)

/* RejectKeywords.export */
static void __Pyx_RejectKeywords(const char* function_name, PyObject *kwds);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* PyObjectVectorCallKwBuilder.proto */
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#if CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_Vectorcall_CallFromBuilder PyObject_Vectorcall
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder _PyObject_Vectorcall
#endif
#define __Pyx_MakeVectorcallBuilderKwds(n) PyTuple_New(n)
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder __Pyx_PyObject_FastCallDict
#define __Pyx_MakeVectorcallBuilderKwds(n) __Pyx_PyDict_NewPresized(n)
#define __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n) PyDict_SetItem(builder, key, value)
#define __Pyx_VectorcallBuilder_AddArgStr(key, value, builder, args, n) PyDict_SetItemString(builder, key, value)
#endif

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck, unsafe_shared) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck, unsafe_shared) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck, unsafe_shared) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck, int unsafe_shared);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* PyObjectFormatAndDecref.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f);
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f);

/* JoinPyUnicode.export */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* BuildPyUnicode.proto (used by COrdinalToPyUnicode) */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, const char* chars, int clength,
                                                int prepend_sign, char padding_char);

/* COrdinalToPyUnicode.proto (used by CIntToPyUnicode) */
static CYTHON_INLINE int __Pyx_CheckUnicodeValue(int value);
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromOrdinal_Padded(int value, Py_ssize_t width, char padding_char);

/* GCCDiagnostics.proto (used by CIntToPyUnicode) */
#if !defined(__INTEL_COMPILER) && defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* IncludeStdlibH.proto (used by CIntToPyUnicode) */
#include <stdlib.h>

/* CIntToPyUnicode.proto */
#define __Pyx_PyUnicode_From_enum__H5T_class_t(value, width, padding_char, format_char) (\
    ((format_char) == ('c')) ?\
        __Pyx_uchar___Pyx_PyUnicode_From_enum__H5T_class_t(value, width, padding_char) :\
        __Pyx____Pyx_PyUnicode_From_enum__H5T_class_t(value, width, padding_char, format_char)\
    )
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_enum__H5T_class_t(enum H5T_class_t value, Py_ssize_t width, char padding_char);
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_enum__H5T_class_t(enum H5T_class_t value, Py_ssize_t width, char padding_char, char format_char);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* ArgTypeTestFunc.export */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely(__Pyx_IS_TYPE(obj, type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))

/* CIntToPyUnicode.proto */
#define __Pyx_PyUnicode_From_Py_ssize_t(value, width, padding_char, format_char) (\
    ((format_char) == ('c')) ?\
        __Pyx_uchar___Pyx_PyUnicode_From_Py_ssize_t(value, width, padding_char) :\
        __Pyx____Pyx_PyUnicode_From_Py_ssize_t(value, width, padding_char, format_char)\
    )
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_Py_ssize_t(Py_ssize_t value, Py_ssize_t width, char padding_char);
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_Py_ssize_t(Py_ssize_t value, Py_ssize_t width, char padding_char, char format_char);

/* PyObjectFormatSimple.proto */
#if CYTHON_COMPILING_IN_PYPY
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#elif CYTHON_USE_TYPE_SLOTS
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        likely(PyLong_CheckExact(s)) ? PyLong_Type.tp_repr(s) :\
        likely(PyFloat_CheckExact(s)) ? PyFloat_Type.tp_repr(s) :\
        PyObject_Format(s, f))
#else
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#endif

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* HasAttr.proto */
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_HasAttr(o, n)  PyObject_HasAttrWithError(o, n)
#else
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);
#endif

/* PyLongBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static CYTHON_INLINE PyObject* __Pyx_PyLong_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyLong_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* PyDictContains.proto */
static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* PyObjectVectorCallMethodKwBuilder.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_VectorcallMethod_CallFromBuilder PyObject_VectorcallMethod
#else
static PyObject *__Pyx_Object_VectorcallMethod_CallFromBuilder(PyObject *name, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#endif

/* PyLongCompare.proto */
static CYTHON_INLINE int __Pyx_PyLong_BoolEqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* AllocateExtensionType.proto */
static PyObject *__Pyx_AllocateExtensionType(PyTypeObject *t, int is_final);

/* DeallocKeepAlive.proto */
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_DeallocKeepAliveBegin(o) do {\
        _Py_atomic_store_uintptr_relaxed(&(o)->ob_tid, _Py_ThreadId());\
        _Py_atomic_store_uint32_relaxed(&(o)->ob_ref_local, 1);\
        _Py_atomic_store_ssize_relaxed(&(o)->ob_ref_shared, 0);\
    } while (0)
#define __Pyx_DeallocKeepAliveEnd(o)\
        _Py_atomic_store_uint32_relaxed(&(o)->ob_ref_local, 0)
#else
#define __Pyx_DeallocKeepAliveBegin(o) __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1)
#define __Pyx_DeallocKeepAliveEnd(o)   __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1)
#endif

/* CallTypeTraverse.proto */
#if !CYTHON_USE_TYPE_SPECS || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x03090000)
#define __Pyx_call_type_traverse(o, always_call, visit, arg) 0
#else
static int __Pyx_call_type_traverse(PyObject *o, int always_call, visitproc visit, void *arg);
#endif

/* LimitedApiGetTypeDict.proto (used by SetItemOnTypeDict) */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_GetTypeDict(PyTypeObject *tp);
#endif

/* SetItemOnTypeDict.proto (used by FixUpExtensionType) */
static int __Pyx__SetItemOnTypeDict(PyTypeObject *tp, PyObject *k, PyObject *v);
#define __Pyx_SetItemOnTypeDict(tp, k, v) __Pyx__SetItemOnTypeDict((PyTypeObject*)tp, k, v)

/* FixUpExtensionType.proto */
static CYTHON_INLINE int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type);

/* PyObjectCallNoArg.proto (used by PyObjectCallMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);

/* PyObjectCallMethod0.proto (used by PyType_Ready) */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* ValidateBasesTuple.proto (used by PyType_Ready) */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases);
#endif

/* PyType_Ready.proto */
CYTHON_UNUSED static int __Pyx_PyType_Ready(PyTypeObject *t);

/* DelItemOnTypeDict.proto (used by SetupReduce) */
static int __Pyx__DelItemOnTypeDict(PyTypeObject *tp, PyObject *k);
#define __Pyx_DelItemOnTypeDict(tp, k) __Pyx__DelItemOnTypeDict((PyTypeObject*)tp, k)

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyTypeObject* typeptr , void* vtable);

/* GetVTable.proto (used by MergeVTables) */
static void* __Pyx_GetVtable(PyTypeObject *type);

/* MergeVTables.proto */
static int __Pyx_MergeVtables(PyTypeObject *type);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto_3_2_8
#define __PYX_HAVE_RT_ImportType_proto_3_2_8
#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
#include <stdalign.h>
#endif
#if (defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) || __cplusplus >= 201103L
#define __PYX_GET_STRUCT_ALIGNMENT_3_2_8(s) alignof(s)
#else
#define __PYX_GET_STRUCT_ALIGNMENT_3_2_8(s) sizeof(void*)
#endif
enum __Pyx_ImportType_CheckSize_3_2_8 {
   __Pyx_ImportType_CheckSize_Error_3_2_8 = 0,
   __Pyx_ImportType_CheckSize_Warn_3_2_8 = 1,
   __Pyx_ImportType_CheckSize_Ignore_3_2_8 = 2
};
static PyTypeObject *__Pyx_ImportType_3_2_8(PyObject* module, const char *module_name, const char *class_name, size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_2_8 check_size);
#endif

/* FunctionImport.proto */
static int __Pyx_ImportFunction_3_2_8(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* ImportImpl.export */
static PyObject *__Pyx__Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, PyObject *moddict, int level);

/* Import.proto */
static CYTHON_INLINE PyObject *__Pyx_Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* ListPack.proto */
static PyObject *__Pyx_PyList_Pack(Py_ssize_t n, ...);

/* dict_setdefault.proto (used by FetchCommonType) */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value);

/* AddModuleRef.proto (used by FetchSharedCythonModule) */
#if ((CYTHON_COMPILING_IN_CPYTHON_FREETHREADING ) ||\
     __PYX_LIMITED_VERSION_HEX < 0x030d0000)
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name);
#else
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#endif

/* FetchSharedCythonModule.proto (used by FetchCommonType) */
static PyObject *__Pyx_FetchSharedCythonABIModule(void);

/* FetchCommonType.proto (used by CommonTypesMetaclass) */
static PyTypeObject* __Pyx_FetchCommonTypeFromSpec(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases);

/* CommonTypesMetaclass.proto (used by CythonFunctionShared) */
static int __pyx_CommonTypesMetaclass_init(PyObject *module);
#define __Pyx_CommonTypesMetaclass_USED

/* PyMethodNew.proto (used by CythonFunctionShared) */
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ);

/* PyVectorcallFastCallDict.proto (used by CythonFunctionShared) */
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw);
#endif

/* CythonFunctionShared.proto (used by CythonFunction) */
#define __Pyx_CyFunction_USED
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CYFUNCTION_COROUTINE     0x08
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_CyFunction_GetClassObj(f)\
      (((__pyx_CyFunctionObject *) (f))->func_classobj)
#else
  #define __Pyx_CyFunction_GetClassObj(f)\
      ((PyObject*) ((PyCMethodObject *) (f))->mm_class)
#endif
#define __Pyx_CyFunction_SetClassObj(f, classobj)\
    __Pyx__CyFunction_SetClassObj((__pyx_CyFunctionObject *) (f), (classobj))
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject_HEAD
    PyObject *func;
#elif PY_VERSION_HEX < 0x030900B1
    PyCFunctionObject func;
#else
    PyCMethodObject func;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API && CYTHON_METH_FASTCALL
    __pyx_vectorcallfunc func_vectorcall;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_weakreflist;
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_dict;
#endif
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_classobj;
#endif
    PyObject *defaults;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
    PyObject *func_is_coroutine;
} __pyx_CyFunctionObject;
#undef __Pyx_CyOrPyCFunction_Check
#define __Pyx_CyFunction_Check(obj)  __Pyx_TypeCheck(obj, __pyx_mstate_global->__pyx_CyFunctionType)
#define __Pyx_CyOrPyCFunction_Check(obj)  __Pyx_TypeCheck2(obj, __pyx_mstate_global->__pyx_CyFunctionType, &PyCFunction_Type)
#define __Pyx_CyFunction_CheckExact(obj)  __Pyx_IS_TYPE(obj, __pyx_mstate_global->__pyx_CyFunctionType)
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void));
#undef __Pyx_IsSameCFunction
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCyOrCFunction(func, cfunc)
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj);
static CYTHON_INLINE PyObject *__Pyx_CyFunction_InitDefaults(PyObject *func,
                                                         PyTypeObject *defaults_type);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(PyObject *module);
#if CYTHON_METH_FASTCALL
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#if CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_func_vectorcall(f) (((__pyx_CyFunctionObject*)f)->func_vectorcall)
#else
#define __Pyx_CyFunction_func_vectorcall(f) (((PyCFunctionObject*)f)->vectorcall)
#endif
#endif

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* CLineInTraceback.proto (used by AddTraceback) */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#else
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#endif

/* CodeObjectCache.proto (used by AddTraceback) */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject __Pyx_CachedCodeObjectType;
#else
typedef PyCodeObject __Pyx_CachedCodeObjectType;
#endif
typedef struct {
    __Pyx_CachedCodeObjectType* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_int_type accessor_count;
  #endif
};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CheckUnpickleChecksum.proto */
static CYTHON_INLINE int __Pyx_CheckUnpickleChecksum(long checksum, long checksum1, long checksum2, long checksum3, const char *members);

/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
    #define __Pyx_c_eq_long__double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_long__double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_long__double(a, b) ((a)-(b))
    #define __Pyx_c_prod_long__double(a, b) ((a)*(b))
    #define __Pyx_c_quot_long__double(a, b) ((a)/(b))
    #define __Pyx_c_neg_long__double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_long__double(z) ((z)==(long double)0)
    #define __Pyx_c_conj_long__double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_long__double(z)     (::std::abs(z))
        #define __Pyx_c_pow_long__double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_long__double(z) ((z)==0)
    #define __Pyx_c_conj_long__double(z)    (conjl(z))
    #if 1
        #define __Pyx_c_abs_long__double(z)     (cabsl(z))
        #define __Pyx_c_pow_long__double(a, b)  (cpowl(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_long__double(__pyx_t_long_double_complex, __pyx_t_long_double_complex);
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_sum_long__double(__pyx_t_long_double_complex, __pyx_t_long_double_complex);
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_diff_long__double(__pyx_t_long_double_complex, __pyx_t_long_double_complex);
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_prod_long__double(__pyx_t_long_double_complex, __pyx_t_long_double_complex);
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_quot_long__double(__pyx_t_long_double_complex, __pyx_t_long_double_complex);
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_neg_long__double(__pyx_t_long_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_long__double(__pyx_t_long_double_complex);
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_conj_long__double(__pyx_t_long_double_complex);
    #if 1
        static CYTHON_INLINE long double __Pyx_c_abs_long__double(__pyx_t_long_double_complex);
        static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_pow_long__double(__pyx_t_long_double_complex, __pyx_t_long_double_complex);
    #endif
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_hid_t(hid_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE hid_t __Pyx_PyLong_As_hid_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int(int value);

/* CIntFromPy.proto */
static CYTHON_INLINE uint32_t __Pyx_PyLong_As_uint32_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE hsize_t __Pyx_PyLong_As_hsize_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_hsize_t(hsize_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyLong_As_size_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE hbool_t __Pyx_PyLong_As_hbool_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_uintptr_t(uintptr_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE enum H5F_scope_t __Pyx_PyLong_As_enum__H5F_scope_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_htri_t(htri_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_haddr_t(haddr_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_unsigned_int(unsigned int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_time_t(time_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_hbool_t(hbool_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_npy_intp(npy_intp value);

/* CIntFromPy.proto */
static CYTHON_INLINE npy_intp __Pyx_PyLong_As_npy_intp(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_enum__H5T_order_t(enum H5T_order_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_herr_t(herr_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_enum__H5S_seloper_t(enum H5S_seloper_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_PY_LONG_LONG(PY_LONG_LONG value);

/* UpdateUnpickledDict.proto */
static int __Pyx_UpdateUnpickledDict(PyObject *obj, PyObject *state, Py_ssize_t index);

/* FormatTypeName.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%U"
#define __Pyx_DECREF_TypeName(obj) Py_XDECREF(obj)
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyType_GetFullyQualifiedName PyType_GetFullyQualifiedName
#else
static __Pyx_TypeName __Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp);
#endif
#else  // !LIMITED_API
typedef const char *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%.200s"
#define __Pyx_PyType_GetFullyQualifiedName(tp) ((tp)->tp_name)
#define __Pyx_DECREF_TypeName(obj)
#endif

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) __Pyx_IsAnySubtype2(Py_TYPE(obj), (PyTypeObject *)type1, (PyTypeObject *)type2)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) (PyObject_TypeCheck(obj, (PyTypeObject *)type1) || PyObject_TypeCheck(obj, (PyTypeObject *)type2))
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2) {
    return PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2);
}
#endif
#define __Pyx_PyErr_ExceptionMatches2(err1, err2)  __Pyx_PyErr_GivenExceptionMatches2(__Pyx_PyErr_CurrentExceptionType(), err1, err2)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#ifdef PyExceptionInstance_Check
  #define __Pyx_PyBaseException_Check(obj) PyExceptionInstance_Check(obj)
#else
  #define __Pyx_PyBaseException_Check(obj) __Pyx_TypeCheck(obj, PyExc_BaseException)
#endif

/* GetRuntimeVersion.proto */
#if __PYX_LIMITED_VERSION_HEX < 0x030b0000
static unsigned long __Pyx_cached_runtime_version = 0;
static void __Pyx_init_runtime_version(void);
#else
#define __Pyx_init_runtime_version()
#endif
static unsigned long __Pyx_get_runtime_version(void);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer);

/* DecompressString.proto */
static PyObject *__Pyx_DecompressString(const char *s, Py_ssize_t length, int algo);

/* MultiPhaseInitModuleState.proto */
#if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
static PyObject *__Pyx_State_FindModule(void*);
static int __Pyx_State_AddModule(PyObject* module, void*);
static int __Pyx_State_RemoveModule(void*);
#elif CYTHON_USE_MODULE_STATE
#define __Pyx_State_FindModule PyState_FindModule
#define __Pyx_State_AddModule PyState_AddModule
#define __Pyx_State_RemoveModule PyState_RemoveModule
#endif

/* #### Code section: module_declarations ### */
/* CythonABIVersion.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
    #if CYTHON_METH_FASTCALL
        #define __PYX_FASTCALL_ABI_SUFFIX  "_fastcall"
    #else
        #define __PYX_FASTCALL_ABI_SUFFIX
    #endif
    #define __PYX_LIMITED_ABI_SUFFIX "limited" __PYX_FASTCALL_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#else
    #define __PYX_LIMITED_ABI_SUFFIX
#endif
#if __PYX_HAS_PY_AM_SEND == 1
    #define __PYX_AM_SEND_ABI_SUFFIX
#elif __PYX_HAS_PY_AM_SEND == 2
    #define __PYX_AM_SEND_ABI_SUFFIX "amsendbackport"
#else
    #define __PYX_AM_SEND_ABI_SUFFIX "noamsend"
#endif
#ifndef __PYX_MONITORING_ABI_SUFFIX
    #define __PYX_MONITORING_ABI_SUFFIX
#endif
#if CYTHON_USE_TP_FINALIZE
    #define __PYX_TP_FINALIZE_ABI_SUFFIX
#else
    #define __PYX_TP_FINALIZE_ABI_SUFFIX "nofinalize"
#endif
#if CYTHON_USE_FREELISTS || !defined(__Pyx_AsyncGen_USED)
    #define __PYX_FREELISTS_ABI_SUFFIX
#else
    #define __PYX_FREELISTS_ABI_SUFFIX "nofreelists"
#endif
#define CYTHON_ABI  __PYX_ABI_VERSION __PYX_LIMITED_ABI_SUFFIX __PYX_MONITORING_ABI_SUFFIX __PYX_TP_FINALIZE_ABI_SUFFIX __PYX_FREELISTS_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#define __PYX_ABI_MODULE_NAME "_cython_" CYTHON_ABI
#define __PYX_TYPE_MODULE_PREFIX __PYX_ABI_MODULE_NAME "."

static CYTHON_INLINE PyTypeObject *__pyx_f_5numpy_5dtype_7typeobj_typeobj(PyArray_Descr *__pyx_v_self); /* proto*/
static CYTHON_INLINE char __pyx_f_5numpy_5dtype_4kind_kind(PyArray_Descr *__pyx_v_self); /* proto*/
static CYTHON_INLINE char __pyx_f_5numpy_5dtype_4type_type(PyArray_Descr *__pyx_v_self); /* proto*/
static CYTHON_INLINE char __pyx_f_5numpy_5dtype_9byteorder_byteorder(PyArray_Descr *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_5numpy_5dtype_8type_num_type_num(PyArray_Descr *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp __pyx_f_5numpy_5dtype_8itemsize_itemsize(PyArray_Descr *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp __pyx_f_5numpy_5dtype_9alignment_alignment(PyArray_Descr *__pyx_v_self); /* proto*/
static CYTHON_INLINE PyObject *__pyx_f_5numpy_5dtype_6fields_fields(PyArray_Descr *__pyx_v_self); /* proto*/
static CYTHON_INLINE PyObject *__pyx_f_5numpy_5dtype_5names_names(PyArray_Descr *__pyx_v_self); /* proto*/
static CYTHON_INLINE PyArray_ArrayDescr *__pyx_f_5numpy_5dtype_8subarray_subarray(PyArray_Descr *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_uint64 __pyx_f_5numpy_5dtype_5flags_flags(PyArray_Descr *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_5numpy_9broadcast_7numiter_numiter(PyArrayMultiIterObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp __pyx_f_5numpy_9broadcast_4size_size(PyArrayMultiIterObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp __pyx_f_5numpy_9broadcast_5index_index(PyArrayMultiIterObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_5numpy_9broadcast_2nd_nd(PyArrayMultiIterObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp *__pyx_f_5numpy_9broadcast_10dimensions_dimensions(PyArrayMultiIterObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE void **__pyx_f_5numpy_9broadcast_5iters_iters(PyArrayMultiIterObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE PyObject *__pyx_f_5numpy_7ndarray_4base_base(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE PyArray_Descr *__pyx_f_5numpy_7ndarray_5descr_descr(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_5numpy_7ndarray_4ndim_ndim(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp *__pyx_f_5numpy_7ndarray_5shape_shape(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp *__pyx_f_5numpy_7ndarray_7strides_strides(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp __pyx_f_5numpy_7ndarray_4size_size(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE char *__pyx_f_5numpy_7ndarray_4data_data(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_5numpy_5ufunc_3nin_nin(PyUFuncObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_5numpy_5ufunc_4nout_nout(PyUFuncObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_5numpy_5ufunc_5nargs_nargs(PyUFuncObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE PyUFuncGenericFunction *__pyx_f_5numpy_5ufunc_9functions_functions(PyUFuncObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE void **__pyx_f_5numpy_5ufunc_4data_data(PyUFuncObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_5numpy_5ufunc_6ntypes_ntypes(PyUFuncObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE char const *__pyx_f_5numpy_5ufunc_4name_name(PyUFuncObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE char const *__pyx_f_5numpy_5ufunc_3doc_doc(PyUFuncObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE void *__pyx_f_5numpy_5ufunc_3ptr_ptr(PyUFuncObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE PyObject *__pyx_f_5numpy_5ufunc_3obj_obj(PyUFuncObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE PyObject *__pyx_f_5numpy_5ufunc_9userloops_userloops(PyUFuncObject *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6tables_13hdf5extension_4Leaf__get_type_ids(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6tables_13hdf5extension_4Leaf__convert_time64(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self, PyArrayObject *__pyx_v_nparr, int __pyx_v_sense); /* proto*/

/* Module declarations from "libc.string" */

/* Module declarations from "libc.stdio" */

/* Module declarations from "__builtin__" */

/* Module declarations from "cpython.type" */

/* Module declarations from "cpython" */

/* Module declarations from "cpython.object" */

/* Module declarations from "cpython.ref" */

/* Module declarations from "numpy" */

/* Module declarations from "numpy" */
static CYTHON_INLINE int __pyx_f_5numpy_import_array(void); /*proto*/

/* Module declarations from "tables.definitions" */

/* Module declarations from "libc.stdlib" */

/* Module declarations from "cpython.bytes" */

/* Module declarations from "libc.stddef" */

/* Module declarations from "cpython.unicode" */

/* Module declarations from "tables.utilsextension" */
static hsize_t *(*__pyx_f_6tables_14utilsextension_malloc_dims)(PyObject *); /*proto*/
static hid_t (*__pyx_f_6tables_14utilsextension_get_native_type)(hid_t); /*proto*/
static PyObject *(*__pyx_f_6tables_14utilsextension_cstr_to_pystr)(char const *); /*proto*/
static int (*__pyx_f_6tables_14utilsextension_load_reference)(hid_t, hobj_ref_t *, size_t, PyArrayObject *); /*proto*/

/* Module declarations from "tables.hdf5extension" */
static int __pyx_v_6tables_13hdf5extension_H5T_CSET_DEFAULT;
static hsize_t *__pyx_f_6tables_13hdf5extension_npy_malloc_dims(int, npy_intp *); /*proto*/
static PyObject *__pyx_f_6tables_13hdf5extension_getshape(int, hsize_t *); /*proto*/
static PyObject *__pyx_f_6tables_13hdf5extension_get_attribute_string_or_none(hid_t, char *); /*proto*/
static PyObject *__pyx_f_6tables_13hdf5extension_get_dtype_scalar(hid_t, enum H5T_class_t, size_t); /*proto*/
static void *__pyx_f_6tables_13hdf5extension__array_data(PyArrayObject *); /*proto*/
static PyObject *__pyx_f_6tables_13hdf5extension___pyx_unpickle_File__set_state(struct __pyx_obj_6tables_13hdf5extension_File *, PyObject *); /*proto*/
static PyObject *__pyx_f_6tables_13hdf5extension___pyx_unpickle_AttributeSet__set_state(struct __pyx_obj_6tables_13hdf5extension_AttributeSet *, PyObject *); /*proto*/
static PyObject *__pyx_f_6tables_13hdf5extension___pyx_unpickle_Node__set_state(struct __pyx_obj_6tables_13hdf5extension_Node *, PyObject *); /*proto*/
static PyObject *__pyx_f_6tables_13hdf5extension___pyx_unpickle_Group__set_state(struct __pyx_obj_6tables_13hdf5extension_Group *, PyObject *); /*proto*/
/* #### Code section: typeinfo ### */
/* #### Code section: before_global_var ### */
#define __Pyx_MODULE_NAME "tables.hdf5extension"
extern int __pyx_module_is_main_tables__hdf5extension;
int __pyx_module_is_main_tables__hdf5extension = 0;

/* Implementation of "tables.hdf5extension" */
/* #### Code section: global_var ### */
static PyObject *__pyx_builtin_super;
static PyObject *__pyx_builtin_object;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_map;
/* #### Code section: string_decls ### */
static const char __pyx_k_name[] = "name";
static const char __pyx_k_name_parent_id[] = "name, parent_id";
static const char __pyx_k_group_id_name_parent_id[] = "group_id, name, parent_id";
static const char __pyx_k_access_plist_file_id_name[] = "access_plist, file_id, name";
static const char __pyx_k_Cython_interface_between_several[] = "Cython interface between several PyTables classes and HDF5 library.\n\nClasses (type extensions):\n\n    File\n    AttributeSet\n    Node\n    Leaf\n    Group\n    Array\n    VLArray\n    UnImplemented\n\nFunctions:\n\nMisc variables:\n\n";
/* #### Code section: decls ### */
static PyObject *__pyx_pf_6tables_13hdf5extension_4File__g_new(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_pymode, PyObject *__pyx_v_params); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4File_2get_file_image(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4File_4get_filesize(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4File_6get_userblock_size(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4File_8_get_file_id(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4File_10fileno(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4File_12_flush_file(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self, PyObject *__pyx_v_scope); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4File_14_close_file(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self); /* proto */
static void __pyx_pf_6tables_13hdf5extension_4File_16__dealloc__(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4File_18__reduce_cython__(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4File_20__setstate_cython__(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_12AttributeSet__g_new(struct __pyx_obj_6tables_13hdf5extension_AttributeSet *__pyx_v_self, PyObject *__pyx_v_node); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_12AttributeSet_2_g_list_attr(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_AttributeSet *__pyx_v_self, PyObject *__pyx_v_node); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_12AttributeSet_4_g_setattr(struct __pyx_obj_6tables_13hdf5extension_AttributeSet *__pyx_v_self, PyObject *__pyx_v_node, PyObject *__pyx_v_name, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_12AttributeSet_6_g_getattr(struct __pyx_obj_6tables_13hdf5extension_AttributeSet *__pyx_v_self, PyObject *__pyx_v_node, PyObject *__pyx_v_attrname); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_12AttributeSet_8_g_remove(struct __pyx_obj_6tables_13hdf5extension_AttributeSet *__pyx_v_self, PyObject *__pyx_v_node, PyObject *__pyx_v_attrname); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_12AttributeSet_10__reduce_cython__(struct __pyx_obj_6tables_13hdf5extension_AttributeSet *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_12AttributeSet_12__setstate_cython__(struct __pyx_obj_6tables_13hdf5extension_AttributeSet *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4Node__g_new(struct __pyx_obj_6tables_13hdf5extension_Node *__pyx_v_self, PyObject *__pyx_v_where, PyObject *__pyx_v_name, CYTHON_UNUSED PyObject *__pyx_v_init); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4Node_2_g_delete(struct __pyx_obj_6tables_13hdf5extension_Node *__pyx_v_self, PyObject *__pyx_v_parent); /* proto */
static void __pyx_pf_6tables_13hdf5extension_4Node_4__dealloc__(struct __pyx_obj_6tables_13hdf5extension_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4Node_6_get_obj_info(struct __pyx_obj_6tables_13hdf5extension_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4Node_8_get_obj_timestamps(struct __pyx_obj_6tables_13hdf5extension_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4Node_10__reduce_cython__(struct __pyx_obj_6tables_13hdf5extension_Node *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4Node_12__setstate_cython__(struct __pyx_obj_6tables_13hdf5extension_Node *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Group__g_create(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_2_g_open(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_4_g_get_objinfo(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self, PyObject *__pyx_v_h5name); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_6_g_list_group(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self, PyObject *__pyx_v_parent); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_8_g_get_gchild_attr(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self, PyObject *__pyx_v_group_name, PyObject *__pyx_v_attr_name); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_10_g_get_lchild_attr(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self, PyObject *__pyx_v_leaf_name, PyObject *__pyx_v_attr_name); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_12_g_flush_group(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_14_g_close_group(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_16_g_move_node(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self, hid_t __pyx_v_oldparent, PyObject *__pyx_v_oldname, hid_t __pyx_v_newparent, PyObject *__pyx_v_newname, PyObject *__pyx_v_oldpathname, PyObject *__pyx_v_newpathname); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_18__reduce_cython__(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_20__setstate_cython__(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf__get_storage_size(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_2_get_obj_track_times(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_4_g_new(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self, PyObject *__pyx_v_where, PyObject *__pyx_v_name, PyObject *__pyx_v_init); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_6_g_chunk_info(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self, PyArrayObject *__pyx_v_coords); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_8_g_read_chunk(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self, PyArrayObject *__pyx_v_coords, PyArrayObject *__pyx_v_out); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_10_g_write_chunk(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self, PyArrayObject *__pyx_v_coords, PyArrayObject *__pyx_v_data, uint32_t __pyx_v_filters); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_12_g_truncate(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self, hsize_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_14_g_flush(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_16_g_close(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_18__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_20__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension__supports_opt_blosc2_read_write(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_byteorder, PyObject *__pyx_v_filter_list, PyObject *__pyx_v_file_mode); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Array__create_array(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyArrayObject *__pyx_v_nparr, PyObject *__pyx_v_title, PyObject *__pyx_v_atom); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_2_create_carray(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyObject *__pyx_v_title); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_4_open_array(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_6_append(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyArrayObject *__pyx_v_nparr); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_8_read_array(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, hsize_t __pyx_v_start, hsize_t __pyx_v_stop, hsize_t __pyx_v_step, PyArrayObject *__pyx_v_nparr); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_10_g_read_slice(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyArrayObject *__pyx_v_startl, PyArrayObject *__pyx_v_stopl, PyArrayObject *__pyx_v_stepl, PyArrayObject *__pyx_v_nparr); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_12_g_read_coords(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyArrayObject *__pyx_v_coords, PyArrayObject *__pyx_v_nparr); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_14perform_selection(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyObject *__pyx_v_space_id, PyObject *__pyx_v_start, PyObject *__pyx_v_count, PyObject *__pyx_v_step, PyObject *__pyx_v_idx, PyObject *__pyx_v_mode); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_16_g_read_selection(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyObject *__pyx_v_selection, PyArrayObject *__pyx_v_nparr); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_18_g_write_slice(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyArrayObject *__pyx_v_startl, PyArrayObject *__pyx_v_stepl, PyArrayObject *__pyx_v_countl, PyArrayObject *__pyx_v_nparr); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_20_g_write_coords(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyArrayObject *__pyx_v_coords, PyArrayObject *__pyx_v_nparr); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_22_g_write_selection(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyObject *__pyx_v_selection, PyArrayObject *__pyx_v_nparr); /* proto */
static void __pyx_pf_6tables_13hdf5extension_5Array_24__dealloc__(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_26__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_28__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_7VLArray__create_array(struct __pyx_obj_6tables_13hdf5extension_VLArray *__pyx_v_self, PyObject *__pyx_v_title); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_7VLArray_2_open_array(struct __pyx_obj_6tables_13hdf5extension_VLArray *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_7VLArray_4_append(struct __pyx_obj_6tables_13hdf5extension_VLArray *__pyx_v_self, PyArrayObject *__pyx_v_nparr, int __pyx_v_nobjects); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_7VLArray_6_modify(struct __pyx_obj_6tables_13hdf5extension_VLArray *__pyx_v_self, hsize_t __pyx_v_nrow, PyArrayObject *__pyx_v_nparr, int __pyx_v_nobjects); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_7VLArray_8_get_memory_size(struct __pyx_obj_6tables_13hdf5extension_VLArray *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_7VLArray_10_read_array(struct __pyx_obj_6tables_13hdf5extension_VLArray *__pyx_v_self, hsize_t __pyx_v_start, hsize_t __pyx_v_stop, hsize_t __pyx_v_step); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_7VLArray_12get_row_size(struct __pyx_obj_6tables_13hdf5extension_VLArray *__pyx_v_self, PyObject *__pyx_v_row); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_7VLArray_14__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_VLArray *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_7VLArray_16__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_VLArray *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_13UnImplemented__open_unimplemented(struct __pyx_obj_6tables_13hdf5extension_UnImplemented *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_13UnImplemented_2_g_close(struct __pyx_obj_6tables_13hdf5extension_UnImplemented *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_13UnImplemented_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_UnImplemented *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_13UnImplemented_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_UnImplemented *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_2__pyx_unpickle_File(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_4__pyx_unpickle_AttributeSet(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_6__pyx_unpickle_Node(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6tables_13hdf5extension_8__pyx_unpickle_Group(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_6tables_13hdf5extension_Node(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6tables_13hdf5extension_Leaf(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6tables_13hdf5extension_Array(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6tables_13hdf5extension_File(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6tables_13hdf5extension_AttributeSet(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6tables_13hdf5extension_VLArray(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6tables_13hdf5extension_UnImplemented(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
/* #### Code section: late_includes ### */
/* #### Code section: module_state ### */
/* SmallCodeConfig */
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

typedef struct {
  PyObject *__pyx_d;
  PyObject *__pyx_b;
  PyObject *__pyx_cython_runtime;
  PyObject *__pyx_empty_tuple;
  PyObject *__pyx_empty_bytes;
  PyObject *__pyx_empty_unicode;
  PyTypeObject *__pyx_ptype_7cpython_4type_type;
  PyTypeObject *__pyx_ptype_5numpy_dtype;
  PyTypeObject *__pyx_ptype_5numpy_flatiter;
  PyTypeObject *__pyx_ptype_5numpy_broadcast;
  PyTypeObject *__pyx_ptype_5numpy_ndarray;
  PyTypeObject *__pyx_ptype_5numpy_generic;
  PyTypeObject *__pyx_ptype_5numpy_number;
  PyTypeObject *__pyx_ptype_5numpy_integer;
  PyTypeObject *__pyx_ptype_5numpy_signedinteger;
  PyTypeObject *__pyx_ptype_5numpy_unsignedinteger;
  PyTypeObject *__pyx_ptype_5numpy_inexact;
  PyTypeObject *__pyx_ptype_5numpy_floating;
  PyTypeObject *__pyx_ptype_5numpy_complexfloating;
  PyTypeObject *__pyx_ptype_5numpy_flexible;
  PyTypeObject *__pyx_ptype_5numpy_character;
  PyTypeObject *__pyx_ptype_5numpy_ufunc;
  PyObject *__pyx_type_6tables_13hdf5extension_Node;
  PyObject *__pyx_type_6tables_13hdf5extension_Leaf;
  PyObject *__pyx_type_6tables_13hdf5extension_Array;
  PyObject *__pyx_type_6tables_13hdf5extension_File;
  PyObject *__pyx_type_6tables_13hdf5extension_AttributeSet;
  PyObject *__pyx_type_6tables_13hdf5extension_Group;
  PyObject *__pyx_type_6tables_13hdf5extension_VLArray;
  PyObject *__pyx_type_6tables_13hdf5extension_UnImplemented;
  PyTypeObject *__pyx_ptype_6tables_13hdf5extension_Node;
  PyTypeObject *__pyx_ptype_6tables_13hdf5extension_Leaf;
  PyTypeObject *__pyx_ptype_6tables_13hdf5extension_Array;
  PyTypeObject *__pyx_ptype_6tables_13hdf5extension_File;
  PyTypeObject *__pyx_ptype_6tables_13hdf5extension_AttributeSet;
  PyTypeObject *__pyx_ptype_6tables_13hdf5extension_Group;
  PyTypeObject *__pyx_ptype_6tables_13hdf5extension_VLArray;
  PyTypeObject *__pyx_ptype_6tables_13hdf5extension_UnImplemented;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_get;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_items;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_pop;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_values;
  PyObject *__pyx_tuple[3];
  PyObject *__pyx_codeobj_tab[77];
  PyObject *__pyx_string_tab[679];
  PyObject *__pyx_number_tab[7];
/* #### Code section: module_state_contents ### */
/* CommonTypesMetaclass.module_state_decls */
PyTypeObject *__pyx_CommonTypesMetaclassType;

/* CachedMethodType.module_state_decls */
#if CYTHON_COMPILING_IN_LIMITED_API
PyObject *__Pyx_CachedMethodType;
#endif

/* CythonFunctionShared.module_state_decls */
PyTypeObject *__pyx_CyFunctionType;

/* CodeObjectCache.module_state_decls */
struct __Pyx_CodeObjectCache __pyx_code_cache;

/* #### Code section: module_state_end ### */
} __pyx_mstatetype;

#if CYTHON_USE_MODULE_STATE
#ifdef __cplusplus
namespace {
extern struct PyModuleDef __pyx_moduledef;
} /* anonymous namespace */
#else
static struct PyModuleDef __pyx_moduledef;
#endif

#define __pyx_mstate_global (__Pyx_PyModule_GetState(__Pyx_State_FindModule(&__pyx_moduledef)))

#define __pyx_m (__Pyx_State_FindModule(&__pyx_moduledef))
#else
static __pyx_mstatetype __pyx_mstate_global_static =
#ifdef __cplusplus
    {};
#else
    {0};
#endif
static __pyx_mstatetype * const __pyx_mstate_global = &__pyx_mstate_global_static;
#endif
/* #### Code section: constant_name_defines ### */
#define __pyx_kp_u_ __pyx_string_tab[0]
#define __pyx_kp_u_0 __pyx_string_tab[1]
#define __pyx_kp_u_Asking_for_a_range_of_rows_excee __pyx_string_tab[2]
#define __pyx_kp_u_Attribute __pyx_string_tab[3]
#define __pyx_kp_u_Attribute_2 __pyx_string_tab[4]
#define __pyx_kp_u_Can_t_create_the_group_s __pyx_string_tab[5]
#define __pyx_kp_u_Can_t_get_dims_info_on_attribute __pyx_string_tab[6]
#define __pyx_kp_u_Can_t_get_type_info_on_attribute __pyx_string_tab[7]
#define __pyx_kp_u_Can_t_open_the_group_s __pyx_string_tab[8]
#define __pyx_kp_u_Can_t_read_attribute __pyx_string_tab[9]
#define __pyx_kp_u_Can_t_set_attribute __pyx_string_tab[10]
#define __pyx_kp_u_Cannot_append_to_the_reference_t __pyx_string_tab[11]
#define __pyx_kp_u_Cannot_write_reference_types_yet __pyx_string_tab[12]
#define __pyx_kp_u_Could_not_get_dataset_creation_p __pyx_string_tab[13]
#define __pyx_kp_u_Could_not_get_dataset_track_time __pyx_string_tab[14]
#define __pyx_kp_u_Internal_error_modifying_the_ele __pyx_string_tab[15]
#define __pyx_kp_u_Internal_error_reading_the_eleme __pyx_string_tab[16]
#define __pyx_kp_u_Invalid_dataset_id_s __pyx_string_tab[17]
#define __pyx_kp_u_Invalid_or_not_supported_driver __pyx_string_tab[18]
#define __pyx_kp_u_Non_existing_node __pyx_string_tab[19]
#define __pyx_kp_u_Note_that_Cython_is_deliberately __pyx_string_tab[20]
#define __pyx_kp_u_Offending_HDF5_class __pyx_string_tab[21]
#define __pyx_kp_u_Output_buffer_is_too_short __pyx_string_tab[22]
#define __pyx_kp_u_Problems_appending_the_elements __pyx_string_tab[23]
#define __pyx_kp_u_Problems_appending_the_records __pyx_string_tab[24]
#define __pyx_kp_u_Problems_closing_the_Group_s __pyx_string_tab[25]
#define __pyx_kp_u_Problems_closing_the_file_s __pyx_string_tab[26]
#define __pyx_kp_u_Problems_creating_the __pyx_string_tab[27]
#define __pyx_kp_u_Problems_creating_the_VLArray __pyx_string_tab[28]
#define __pyx_kp_u_Problems_creating_the_s __pyx_string_tab[29]
#define __pyx_kp_u_Problems_getting_chunk_info_for __pyx_string_tab[30]
#define __pyx_kp_u_Problems_getting_file_descriptor __pyx_string_tab[31]
#define __pyx_kp_u_Problems_getting_ndims __pyx_string_tab[32]
#define __pyx_kp_u_Problems_modifying_the_record __pyx_string_tab[33]
#define __pyx_kp_u_Problems_moving_the_node __pyx_string_tab[34]
#define __pyx_kp_u_Problems_reading_chunk_from_s __pyx_string_tab[35]
#define __pyx_kp_u_Problems_reading_the_array_data __pyx_string_tab[36]
#define __pyx_kp_u_Problems_truncating_the_leaf_s __pyx_string_tab[37]
#define __pyx_kp_u_Problems_writing_chunk_to_s __pyx_string_tab[38]
#define __pyx_kp_u_Problems_writing_the_array_data __pyx_string_tab[39]
#define __pyx_kp_u_S_s __pyx_string_tab[40]
#define __pyx_kp_u_The_DRIVER_CORE_IMAGE_must_be_a __pyx_string_tab[41]
#define __pyx_kp_u_The_DRIVER_CORE_IMAGE_parameter __pyx_string_tab[42]
#define __pyx_kp_u_The_H5FD_DIRECT_driver_is_not_av __pyx_string_tab[43]
#define __pyx_kp_u_The_H5FD_WINDOWS_driver_is_not_a __pyx_string_tab[44]
#define __pyx_kp_u_The_HDF5_file_already_esists_the __pyx_string_tab[45]
#define __pyx_kp_u_The_USER_BLOCK_SIZE_must_be_a_po __pyx_string_tab[46]
#define __pyx_kp_u_Unable_to_allecote_meomory_fir_t __pyx_string_tab[47]
#define __pyx_kp_u_Unable_to_get_array_info __pyx_string_tab[48]
#define __pyx_kp_u_Unable_to_get_object_info_for_s __pyx_string_tab[49]
#define __pyx_kp_u_Unable_to_get_the_creation_prope __pyx_string_tab[50]
#define __pyx_kp_u_Unable_to_open_create_file_s __pyx_string_tab[51]
#define __pyx_kp_u_Unable_to_retrieve_the_HDF5_file __pyx_string_tab[52]
#define __pyx_kp_u_Unable_to_retrieve_the_file_imag __pyx_string_tab[53]
#define __pyx_kp_u_Unable_to_retrieve_the_size_of_t __pyx_string_tab[54]
#define __pyx_kp_u_Unable_to_set_the_file_access_pr __pyx_string_tab[55]
#define __pyx_kp_u_Unable_to_set_the_file_image __pyx_string_tab[56]
#define __pyx_kp_u_Unable_to_set_the_user_block_siz __pyx_string_tab[57]
#define __pyx_kp_u_Unexpected_classname_s __pyx_string_tab[58]
#define __pyx_kp_u_Unsupported_type_for_attribute __pyx_string_tab[59]
#define __pyx_kp_u_VLArray__read_array_Problems_rea __pyx_string_tab[60]
#define __pyx_kp_u_VLArray__read_array_error_freein __pyx_string_tab[61]
#define __pyx_kp_u__3 __pyx_string_tab[62]
#define __pyx_kp_u__4 __pyx_string_tab[63]
#define __pyx_kp_u__5 __pyx_string_tab[64]
#define __pyx_kp_u__6 __pyx_string_tab[65]
#define __pyx_kp_u_add_note __pyx_string_tab[66]
#define __pyx_kp_u_an_invalid_mode_string_s_passed __pyx_string_tab[67]
#define __pyx_kp_u_but_can_t_get_it __pyx_string_tab[68]
#define __pyx_kp_u_but_cannot_be_deleted __pyx_string_tab[69]
#define __pyx_kp_u_disable __pyx_string_tab[70]
#define __pyx_kp_u_enable __pyx_string_tab[71]
#define __pyx_kp_u_exists_in_node __pyx_string_tab[72]
#define __pyx_kp_u_exists_in_node_2 __pyx_string_tab[73]
#define __pyx_kp_u_f_s __pyx_string_tab[74]
#define __pyx_kp_u_gc __pyx_string_tab[75]
#define __pyx_kp_u_i_s __pyx_string_tab[76]
#define __pyx_kp_u_in_node __pyx_string_tab[77]
#define __pyx_kp_u_in_node_2 __pyx_string_tab[78]
#define __pyx_kp_u_in_node_3 __pyx_string_tab[79]
#define __pyx_kp_u_invalid_base_type_ID_for_atom __pyx_string_tab[80]
#define __pyx_kp_u_invalid_disk_type_ID_for_atom __pyx_string_tab[81]
#define __pyx_kp_u_isenabled __pyx_string_tab[82]
#define __pyx_kp_u_m_h5 __pyx_string_tab[83]
#define __pyx_kp_u_numpy__core_multiarray_failed_to __pyx_string_tab[84]
#define __pyx_kp_u_numpy__core_umath_failed_to_impo __pyx_string_tab[85]
#define __pyx_kp_u_problems_deleting_the_node_s __pyx_string_tab[86]
#define __pyx_kp_u_r_2 __pyx_string_tab[87]
#define __pyx_kp_u_r_h5 __pyx_string_tab[88]
#define __pyx_kp_u_s __pyx_string_tab[89]
#define __pyx_kp_u_self_dims_cannot_be_converted_to __pyx_string_tab[90]
#define __pyx_kp_u_self_dims_self_dims_chunk_self_m __pyx_string_tab[91]
#define __pyx_kp_u_tables_atom __pyx_string_tab[92]
#define __pyx_kp_u_tables_collections __pyx_string_tab[93]
#define __pyx_kp_u_tables_description __pyx_string_tab[94]
#define __pyx_kp_u_tables_exceptions __pyx_string_tab[95]
#define __pyx_kp_u_tables_hdf5extension_pyx __pyx_string_tab[96]
#define __pyx_kp_u_tables_numpy __pyx_string_tab[97]
#define __pyx_kp_u_tables_os __pyx_string_tab[98]
#define __pyx_kp_u_tables_pickle __pyx_string_tab[99]
#define __pyx_kp_u_tables_platform __pyx_string_tab[100]
#define __pyx_kp_u_tables_sys __pyx_string_tab[101]
#define __pyx_kp_u_tables_utils __pyx_string_tab[102]
#define __pyx_kp_u_tables_utilsextension __pyx_string_tab[103]
#define __pyx_kp_u_tables_warnings __pyx_string_tab[104]
#define __pyx_kp_u_to __pyx_string_tab[105]
#define __pyx_kp_u_tree_fragment __pyx_string_tab[106]
#define __pyx_kp_u_u_s __pyx_string_tab[107]
#define __pyx_kp_u_unable_to_retrieve_the_user_bloc __pyx_string_tab[108]
#define __pyx_kp_u_under __pyx_string_tab[109]
#define __pyx_kp_u_utf_8 __pyx_string_tab[110]
#define __pyx_kp_u_variable_length_strings_are_not __pyx_string_tab[111]
#define __pyx_n_u_AND __pyx_string_tab[112]
#define __pyx_n_u_Array __pyx_string_tab[113]
#define __pyx_n_u_Array___reduce_cython __pyx_string_tab[114]
#define __pyx_n_u_Array___setstate_cython __pyx_string_tab[115]
#define __pyx_n_u_Array__append __pyx_string_tab[116]
#define __pyx_n_u_Array__create_array __pyx_string_tab[117]
#define __pyx_n_u_Array__create_carray __pyx_string_tab[118]
#define __pyx_n_u_Array__g_read_coords __pyx_string_tab[119]
#define __pyx_n_u_Array__g_read_selection __pyx_string_tab[120]
#define __pyx_n_u_Array__g_read_slice __pyx_string_tab[121]
#define __pyx_n_u_Array__g_write_coords __pyx_string_tab[122]
#define __pyx_n_u_Array__g_write_selection __pyx_string_tab[123]
#define __pyx_n_u_Array__g_write_slice __pyx_string_tab[124]
#define __pyx_n_u_Array__open_array __pyx_string_tab[125]
#define __pyx_n_u_Array__read_array __pyx_string_tab[126]
#define __pyx_n_u_Array_perform_selection __pyx_string_tab[127]
#define __pyx_n_u_Atom __pyx_string_tab[128]
#define __pyx_n_u_AttributeSet __pyx_string_tab[129]
#define __pyx_n_u_AttributeSet___reduce_cython __pyx_string_tab[130]
#define __pyx_n_u_AttributeSet___setstate_cython __pyx_string_tab[131]
#define __pyx_n_u_AttributeSet__g_getattr __pyx_string_tab[132]
#define __pyx_n_u_AttributeSet__g_list_attr __pyx_string_tab[133]
#define __pyx_n_u_AttributeSet__g_new __pyx_string_tab[134]
#define __pyx_n_u_AttributeSet__g_remove __pyx_string_tab[135]
#define __pyx_n_u_AttributeSet__g_setattr __pyx_string_tab[136]
#define __pyx_n_u_CArray __pyx_string_tab[137]
#define __pyx_n_u_CHUNK_CACHE_NELMTS __pyx_string_tab[138]
#define __pyx_n_u_CHUNK_CACHE_PREEMPT __pyx_string_tab[139]
#define __pyx_n_u_CHUNK_CACHE_SIZE __pyx_string_tab[140]
#define __pyx_n_u_DRIVER __pyx_string_tab[141]
#define __pyx_n_u_DRIVER_CORE_BACKING_STORE __pyx_string_tab[142]
#define __pyx_n_u_DRIVER_CORE_IMAGE __pyx_string_tab[143]
#define __pyx_n_u_DRIVER_CORE_INCREMENT __pyx_string_tab[144]
#define __pyx_n_u_DRIVER_DIRECT_ALIGNMENT __pyx_string_tab[145]
#define __pyx_n_u_DRIVER_DIRECT_BLOCK_SIZE __pyx_string_tab[146]
#define __pyx_n_u_DRIVER_DIRECT_CBUF_SIZE __pyx_string_tab[147]
#define __pyx_n_u_DRIVER_SPLIT_META_EXT __pyx_string_tab[148]
#define __pyx_n_u_DRIVER_SPLIT_RAW_EXT __pyx_string_tab[149]
#define __pyx_n_u_DataTypeWarning __pyx_string_tab[150]
#define __pyx_n_u_EArray __pyx_string_tab[151]
#define __pyx_n_u_ExternalLink __pyx_string_tab[152]
#define __pyx_n_u_File __pyx_string_tab[153]
#define __pyx_n_u_File___reduce_cython __pyx_string_tab[154]
#define __pyx_n_u_File___setstate_cython __pyx_string_tab[155]
#define __pyx_n_u_File__close_file __pyx_string_tab[156]
#define __pyx_n_u_File__flush_file __pyx_string_tab[157]
#define __pyx_n_u_File__g_new __pyx_string_tab[158]
#define __pyx_n_u_File__get_file_id __pyx_string_tab[159]
#define __pyx_n_u_File_fileno __pyx_string_tab[160]
#define __pyx_n_u_File_get_file_image __pyx_string_tab[161]
#define __pyx_n_u_File_get_filesize __pyx_string_tab[162]
#define __pyx_n_u_File_get_userblock_size __pyx_string_tab[163]
#define __pyx_n_u_Group __pyx_string_tab[164]
#define __pyx_n_u_Group___reduce_cython __pyx_string_tab[165]
#define __pyx_n_u_Group___setstate_cython __pyx_string_tab[166]
#define __pyx_n_u_Group__g_close_group __pyx_string_tab[167]
#define __pyx_n_u_Group__g_create __pyx_string_tab[168]
#define __pyx_n_u_Group__g_flush_group __pyx_string_tab[169]
#define __pyx_n_u_Group__g_get_gchild_attr __pyx_string_tab[170]
#define __pyx_n_u_Group__g_get_lchild_attr __pyx_string_tab[171]
#define __pyx_n_u_Group__g_get_objinfo __pyx_string_tab[172]
#define __pyx_n_u_Group__g_list_group __pyx_string_tab[173]
#define __pyx_n_u_Group__g_move_node __pyx_string_tab[174]
#define __pyx_n_u_Group__g_open __pyx_string_tab[175]
#define __pyx_n_u_H5FD_CORE __pyx_string_tab[176]
#define __pyx_n_u_H5FD_DIRECT __pyx_string_tab[177]
#define __pyx_n_u_H5FD_SEC2 __pyx_string_tab[178]
#define __pyx_n_u_H5FD_SPLIT __pyx_string_tab[179]
#define __pyx_n_u_H5FD_STDIO __pyx_string_tab[180]
#define __pyx_n_u_H5FD_WINDOWS __pyx_string_tab[181]
#define __pyx_n_u_HAVE_DIRECT_DRIVER __pyx_string_tab[182]
#define __pyx_n_u_HAVE_WINDOWS_DRIVER __pyx_string_tab[183]
#define __pyx_n_u_HDF5ExtError __pyx_string_tab[184]
#define __pyx_n_u_Leaf __pyx_string_tab[185]
#define __pyx_n_u_Leaf___reduce_cython __pyx_string_tab[186]
#define __pyx_n_u_Leaf___setstate_cython __pyx_string_tab[187]
#define __pyx_n_u_Leaf__g_chunk_info __pyx_string_tab[188]
#define __pyx_n_u_Leaf__g_close __pyx_string_tab[189]
#define __pyx_n_u_Leaf__g_flush __pyx_string_tab[190]
#define __pyx_n_u_Leaf__g_new __pyx_string_tab[191]
#define __pyx_n_u_Leaf__g_read_chunk __pyx_string_tab[192]
#define __pyx_n_u_Leaf__g_truncate __pyx_string_tab[193]
#define __pyx_n_u_Leaf__g_write_chunk __pyx_string_tab[194]
#define __pyx_n_u_Leaf__get_obj_track_times __pyx_string_tab[195]
#define __pyx_n_u_Leaf__get_storage_size __pyx_string_tab[196]
#define __pyx_n_u_MAX_BLOSC_THREADS __pyx_string_tab[197]
#define __pyx_n_u_METADATA_CACHE_SIZE __pyx_string_tab[198]
#define __pyx_n_u_NOTB __pyx_string_tab[199]
#define __pyx_n_u_NamedType __pyx_string_tab[200]
#define __pyx_n_u_NoSuchNode __pyx_string_tab[201]
#define __pyx_n_u_Node __pyx_string_tab[202]
#define __pyx_n_u_Node___reduce_cython __pyx_string_tab[203]
#define __pyx_n_u_Node___setstate_cython __pyx_string_tab[204]
#define __pyx_n_u_Node__g_delete __pyx_string_tab[205]
#define __pyx_n_u_Node__g_new __pyx_string_tab[206]
#define __pyx_n_u_Node__get_obj_info __pyx_string_tab[207]
#define __pyx_n_u_Node__get_obj_timestamps __pyx_string_tab[208]
#define __pyx_n_u_O8 __pyx_string_tab[209]
#define __pyx_n_u_ObjInfo __pyx_string_tab[210]
#define __pyx_n_u_ObjTimestamps __pyx_string_tab[211]
#define __pyx_n_u_PT_DEFAULT_B2_BLOCKSIZE __pyx_string_tab[212]
#define __pyx_n_u_PYTABLES_SYS_ATTRS __pyx_string_tab[213]
#define __pyx_n_u_Pyx_PyDict_NextRef __pyx_string_tab[214]
#define __pyx_n_u_S __pyx_string_tab[215]
#define __pyx_n_u_SizeType __pyx_string_tab[216]
#define __pyx_n_u_SoftLink __pyx_string_tab[217]
#define __pyx_n_u_Table __pyx_string_tab[218]
#define __pyx_n_u_U __pyx_string_tab[219]
#define __pyx_n_u_USER_BLOCK_SIZE __pyx_string_tab[220]
#define __pyx_n_u_UnImplemented __pyx_string_tab[221]
#define __pyx_n_u_UnImplemented___reduce_cython __pyx_string_tab[222]
#define __pyx_n_u_UnImplemented___setstate_cython __pyx_string_tab[223]
#define __pyx_n_u_UnImplemented__g_close __pyx_string_tab[224]
#define __pyx_n_u_UnImplemented__open_unimplemente __pyx_string_tab[225]
#define __pyx_n_u_Unknown __pyx_string_tab[226]
#define __pyx_n_u_V __pyx_string_tab[227]
#define __pyx_n_u_VLArray __pyx_string_tab[228]
#define __pyx_n_u_VLArray___reduce_cython __pyx_string_tab[229]
#define __pyx_n_u_VLArray___setstate_cython __pyx_string_tab[230]
#define __pyx_n_u_VLArray__append __pyx_string_tab[231]
#define __pyx_n_u_VLArray__create_array __pyx_string_tab[232]
#define __pyx_n_u_VLArray__get_memory_size __pyx_string_tab[233]
#define __pyx_n_u_VLArray__modify __pyx_string_tab[234]
#define __pyx_n_u_VLArray__open_array __pyx_string_tab[235]
#define __pyx_n_u_VLArray__read_array __pyx_string_tab[236]
#define __pyx_n_u_VLArray_get_row_size __pyx_string_tab[237]
#define __pyx_n_u__7 __pyx_string_tab[238]
#define __pyx_n_u_a __pyx_string_tab[239]
#define __pyx_n_u_access_plist __pyx_string_tab[240]
#define __pyx_n_u_addr __pyx_string_tab[241]
#define __pyx_n_u_annotate __pyx_string_tab[242]
#define __pyx_n_u_append __pyx_string_tab[243]
#define __pyx_n_u_append_2 __pyx_string_tab[244]
#define __pyx_n_u_args __pyx_string_tab[245]
#define __pyx_n_u_array __pyx_string_tab[246]
#define __pyx_n_u_astype __pyx_string_tab[247]
#define __pyx_n_u_asyncio_coroutines __pyx_string_tab[248]
#define __pyx_n_u_atime __pyx_string_tab[249]
#define __pyx_n_u_atom __pyx_string_tab[250]
#define __pyx_n_u_atom_2 __pyx_string_tab[251]
#define __pyx_n_u_atom_from_hdf5_type __pyx_string_tab[252]
#define __pyx_n_u_atom_to_hdf5_type __pyx_string_tab[253]
#define __pyx_n_u_atomicdtype __pyx_string_tab[254]
#define __pyx_n_u_atomicshape __pyx_string_tab[255]
#define __pyx_n_u_atomicsize __pyx_string_tab[256]
#define __pyx_n_u_atomictype __pyx_string_tab[257]
#define __pyx_n_u_attr_name __pyx_string_tab[258]
#define __pyx_n_u_attrname __pyx_string_tab[259]
#define __pyx_n_u_b __pyx_string_tab[260]
#define __pyx_n_u_b1 __pyx_string_tab[261]
#define __pyx_n_u_backing_store __pyx_string_tab[262]
#define __pyx_n_u_base __pyx_string_tab[263]
#define __pyx_n_u_baseatom __pyx_string_tab[264]
#define __pyx_n_u_blocksize __pyx_string_tab[265]
#define __pyx_n_u_blosc2 __pyx_string_tab[266]
#define __pyx_n_u_blosc2_support_wirte __pyx_string_tab[267]
#define __pyx_n_u_btime __pyx_string_tab[268]
#define __pyx_n_u_buf __pyx_string_tab[269]
#define __pyx_n_u_buf_len __pyx_string_tab[270]
#define __pyx_n_u_buffer __pyx_string_tab[271]
#define __pyx_n_u_byteorder __pyx_string_tab[272]
#define __pyx_n_u_byteorders __pyx_string_tab[273]
#define __pyx_n_u_bytes __pyx_string_tab[274]
#define __pyx_n_u_byteswap __pyx_string_tab[275]
#define __pyx_n_u_c __pyx_string_tab[276]
#define __pyx_n_u_c_classid __pyx_string_tab[277]
#define __pyx_n_u_cattrname __pyx_string_tab[278]
#define __pyx_n_u_cbyteorder __pyx_string_tab[279]
#define __pyx_n_u_check_file_access __pyx_string_tab[280]
#define __pyx_n_u_chunkshape __pyx_string_tab[281]
#define __pyx_n_u_chunkshapes __pyx_string_tab[282]
#define __pyx_n_u_chunksize __pyx_string_tab[283]
#define __pyx_n_u_cimage __pyx_string_tab[284]
#define __pyx_n_u_class __pyx_string_tab[285]
#define __pyx_n_u_class_2 __pyx_string_tab[286]
#define __pyx_n_u_class_getitem __pyx_string_tab[287]
#define __pyx_n_u_class_id __pyx_string_tab[288]
#define __pyx_n_u_classname __pyx_string_tab[289]
#define __pyx_n_u_cline_in_traceback __pyx_string_tab[290]
#define __pyx_n_u_close_file __pyx_string_tab[291]
#define __pyx_n_u_cname __pyx_string_tab[292]
#define __pyx_n_u_collections __pyx_string_tab[293]
#define __pyx_n_u_complevel __pyx_string_tab[294]
#define __pyx_n_u_complib __pyx_string_tab[295]
#define __pyx_n_u_coords __pyx_string_tab[296]
#define __pyx_n_u_copy __pyx_string_tab[297]
#define __pyx_n_u_correct_byteorder __pyx_string_tab[298]
#define __pyx_n_u_count __pyx_string_tab[299]
#define __pyx_n_u_count_2 __pyx_string_tab[300]
#define __pyx_n_u_countl __pyx_string_tab[301]
#define __pyx_n_u_countp __pyx_string_tab[302]
#define __pyx_n_u_create_array __pyx_string_tab[303]
#define __pyx_n_u_create_carray __pyx_string_tab[304]
#define __pyx_n_u_create_nested_type __pyx_string_tab[305]
#define __pyx_n_u_create_plist __pyx_string_tab[306]
#define __pyx_n_u_cset __pyx_string_tab[307]
#define __pyx_n_u_ctime __pyx_string_tab[308]
#define __pyx_n_u_d __pyx_string_tab[309]
#define __pyx_n_u_data __pyx_string_tab[310]
#define __pyx_n_u_data_size __pyx_string_tab[311]
#define __pyx_n_u_datalist __pyx_string_tab[312]
#define __pyx_n_u_descr_from_dtype __pyx_string_tab[313]
#define __pyx_n_u_description __pyx_string_tab[314]
#define __pyx_n_u_descriptor __pyx_string_tab[315]
#define __pyx_n_u_dflt __pyx_string_tab[316]
#define __pyx_n_u_dflts __pyx_string_tab[317]
#define __pyx_n_u_dict __pyx_string_tab[318]
#define __pyx_n_u_dict_2 __pyx_string_tab[319]
#define __pyx_n_u_dims __pyx_string_tab[320]
#define __pyx_n_u_dims_arr __pyx_string_tab[321]
#define __pyx_n_u_driver __pyx_string_tab[322]
#define __pyx_n_u_dset_id __pyx_string_tab[323]
#define __pyx_n_u_dtype __pyx_string_tab[324]
#define __pyx_n_u_dtype_2 __pyx_string_tab[325]
#define __pyx_n_u_dumps __pyx_string_tab[326]
#define __pyx_n_u_e __pyx_string_tab[327]
#define __pyx_n_u_empty __pyx_string_tab[328]
#define __pyx_n_u_enc_meta_ext __pyx_string_tab[329]
#define __pyx_n_u_enc_raw_ext __pyx_string_tab[330]
#define __pyx_n_u_encname __pyx_string_tab[331]
#define __pyx_n_u_encode __pyx_string_tab[332]
#define __pyx_n_u_encode_filename __pyx_string_tab[333]
#define __pyx_n_u_encoded_attr_name __pyx_string_tab[334]
#define __pyx_n_u_encoded_attrname __pyx_string_tab[335]
#define __pyx_n_u_encoded_group_name __pyx_string_tab[336]
#define __pyx_n_u_encoded_leaf_name __pyx_string_tab[337]
#define __pyx_n_u_encoded_name __pyx_string_tab[338]
#define __pyx_n_u_encoded_newname __pyx_string_tab[339]
#define __pyx_n_u_encoded_oldname __pyx_string_tab[340]
#define __pyx_n_u_encoded_title __pyx_string_tab[341]
#define __pyx_n_u_enumerate __pyx_string_tab[342]
#define __pyx_n_u_enumtype __pyx_string_tab[343]
#define __pyx_n_u_environ __pyx_string_tab[344]
#define __pyx_n_u_err __pyx_string_tab[345]
#define __pyx_n_u_exceptions __pyx_string_tab[346]
#define __pyx_n_u_exists __pyx_string_tab[347]
#define __pyx_n_u_extdim __pyx_string_tab[348]
#define __pyx_n_u_f __pyx_string_tab[349]
#define __pyx_n_u_file_handle __pyx_string_tab[350]
#define __pyx_n_u_file_mode __pyx_string_tab[351]
#define __pyx_n_u_filename __pyx_string_tab[352]
#define __pyx_n_u_fileno __pyx_string_tab[353]
#define __pyx_n_u_fill_data __pyx_string_tab[354]
#define __pyx_n_u_fill_status __pyx_string_tab[355]
#define __pyx_n_u_filter_list __pyx_string_tab[356]
#define __pyx_n_u_filter_mask __pyx_string_tab[357]
#define __pyx_n_u_filters __pyx_string_tab[358]
#define __pyx_n_u_flags __pyx_string_tab[359]
#define __pyx_n_u_fletcher32 __pyx_string_tab[360]
#define __pyx_n_u_flush __pyx_string_tab[361]
#define __pyx_n_u_flush_file __pyx_string_tab[362]
#define __pyx_n_u_fname __pyx_string_tab[363]
#define __pyx_n_u_from_dtype __pyx_string_tab[364]
#define __pyx_n_u_func __pyx_string_tab[365]
#define __pyx_n_u_g_chunk_info __pyx_string_tab[366]
#define __pyx_n_u_g_close __pyx_string_tab[367]
#define __pyx_n_u_g_close_group __pyx_string_tab[368]
#define __pyx_n_u_g_create __pyx_string_tab[369]
#define __pyx_n_u_g_delete __pyx_string_tab[370]
#define __pyx_n_u_g_flush __pyx_string_tab[371]
#define __pyx_n_u_g_flush_group __pyx_string_tab[372]
#define __pyx_n_u_g_get_gchild_attr __pyx_string_tab[373]
#define __pyx_n_u_g_get_lchild_attr __pyx_string_tab[374]
#define __pyx_n_u_g_get_objinfo __pyx_string_tab[375]
#define __pyx_n_u_g_getattr __pyx_string_tab[376]
#define __pyx_n_u_g_list_attr __pyx_string_tab[377]
#define __pyx_n_u_g_list_group __pyx_string_tab[378]
#define __pyx_n_u_g_move_node __pyx_string_tab[379]
#define __pyx_n_u_g_new __pyx_string_tab[380]
#define __pyx_n_u_g_open __pyx_string_tab[381]
#define __pyx_n_u_g_read_chunk __pyx_string_tab[382]
#define __pyx_n_u_g_read_coords __pyx_string_tab[383]
#define __pyx_n_u_g_read_selection __pyx_string_tab[384]
#define __pyx_n_u_g_read_slice __pyx_string_tab[385]
#define __pyx_n_u_g_remove __pyx_string_tab[386]
#define __pyx_n_u_g_setattr __pyx_string_tab[387]
#define __pyx_n_u_g_truncate __pyx_string_tab[388]
#define __pyx_n_u_g_write_chunk __pyx_string_tab[389]
#define __pyx_n_u_g_write_coords __pyx_string_tab[390]
#define __pyx_n_u_g_write_selection __pyx_string_tab[391]
#define __pyx_n_u_g_write_slice __pyx_string_tab[392]
#define __pyx_n_u_gchild_id __pyx_string_tab[393]
#define __pyx_n_u_generic __pyx_string_tab[394]
#define __pyx_n_u_get __pyx_string_tab[395]
#define __pyx_n_u_get_file_id __pyx_string_tab[396]
#define __pyx_n_u_get_file_image __pyx_string_tab[397]
#define __pyx_n_u_get_filesize __pyx_string_tab[398]
#define __pyx_n_u_get_filters __pyx_string_tab[399]
#define __pyx_n_u_get_memory_size __pyx_string_tab[400]
#define __pyx_n_u_get_obj_info __pyx_string_tab[401]
#define __pyx_n_u_get_obj_timestamps __pyx_string_tab[402]
#define __pyx_n_u_get_obj_track_times __pyx_string_tab[403]
#define __pyx_n_u_get_row_size __pyx_string_tab[404]
#define __pyx_n_u_get_storage_size __pyx_string_tab[405]
#define __pyx_n_u_get_userblock_size __pyx_string_tab[406]
#define __pyx_n_u_getstate __pyx_string_tab[407]
#define __pyx_n_u_group_name __pyx_string_tab[408]
#define __pyx_n_u_h5bt __pyx_string_tab[409]
#define __pyx_n_u_h5name __pyx_string_tab[410]
#define __pyx_n_u_hdf5_class_to_string __pyx_string_tab[411]
#define __pyx_n_u_hdf5_to_np_ext_type __pyx_string_tab[412]
#define __pyx_n_u_i __pyx_string_tab[413]
#define __pyx_n_u_i8 __pyx_string_tab[414]
#define __pyx_n_u_idx __pyx_string_tab[415]
#define __pyx_n_u_image __pyx_string_tab[416]
#define __pyx_n_u_img_buf_len __pyx_string_tab[417]
#define __pyx_n_u_img_buf_p __pyx_string_tab[418]
#define __pyx_n_u_init __pyx_string_tab[419]
#define __pyx_n_u_insert __pyx_string_tab[420]
#define __pyx_n_u_int32 __pyx_string_tab[421]
#define __pyx_n_u_intp __pyx_string_tab[422]
#define __pyx_n_u_isPTFile __pyx_string_tab[423]
#define __pyx_n_u_is_coroutine __pyx_string_tab[424]
#define __pyx_n_u_is_pow_of_2 __pyx_string_tab[425]
#define __pyx_n_u_item_size __pyx_string_tab[426]
#define __pyx_n_u_items __pyx_string_tab[427]
#define __pyx_n_u_itemsize __pyx_string_tab[428]
#define __pyx_n_u_kind __pyx_string_tab[429]
#define __pyx_n_u_leaf_id __pyx_string_tab[430]
#define __pyx_n_u_leaf_name __pyx_string_tab[431]
#define __pyx_n_u_lower __pyx_string_tab[432]
#define __pyx_n_u_main __pyx_string_tab[433]
#define __pyx_n_u_maindim __pyx_string_tab[434]
#define __pyx_n_u_map __pyx_string_tab[435]
#define __pyx_n_u_mem_space_id __pyx_string_tab[436]
#define __pyx_n_u_mem_type __pyx_string_tab[437]
#define __pyx_n_u_meta_ext __pyx_string_tab[438]
#define __pyx_n_u_meta_name __pyx_string_tab[439]
#define __pyx_n_u_meta_plist_id __pyx_string_tab[440]
#define __pyx_n_u_mode __pyx_string_tab[441]
#define __pyx_n_u_modify __pyx_string_tab[442]
#define __pyx_n_u_module __pyx_string_tab[443]
#define __pyx_n_u_mtime __pyx_string_tab[444]
#define __pyx_n_u_n __pyx_string_tab[445]
#define __pyx_n_u_name __pyx_string_tab[446]
#define __pyx_n_u_name_2 __pyx_string_tab[447]
#define __pyx_n_u_namedtuple __pyx_string_tab[448]
#define __pyx_n_u_native_type __pyx_string_tab[449]
#define __pyx_n_u_ndarray __pyx_string_tab[450]
#define __pyx_n_u_ndv __pyx_string_tab[451]
#define __pyx_n_u_ndvalue __pyx_string_tab[452]
#define __pyx_n_u_nelements __pyx_string_tab[453]
#define __pyx_n_u_new __pyx_string_tab[454]
#define __pyx_n_u_newname __pyx_string_tab[455]
#define __pyx_n_u_newparent __pyx_string_tab[456]
#define __pyx_n_u_newpathname __pyx_string_tab[457]
#define __pyx_n_u_nobjects __pyx_string_tab[458]
#define __pyx_n_u_node __pyx_string_tab[459]
#define __pyx_n_u_node_type __pyx_string_tab[460]
#define __pyx_n_u_np __pyx_string_tab[461]
#define __pyx_n_u_nparr __pyx_string_tab[462]
#define __pyx_n_u_npext_prefixes_to_ptkinds __pyx_string_tab[463]
#define __pyx_n_u_nrecords __pyx_string_tab[464]
#define __pyx_n_u_nrow __pyx_string_tab[465]
#define __pyx_n_u_nrows __pyx_string_tab[466]
#define __pyx_n_u_numpy __pyx_string_tab[467]
#define __pyx_n_u_object __pyx_string_tab[468]
#define __pyx_n_u_offset __pyx_string_tab[469]
#define __pyx_n_u_oinfo __pyx_string_tab[470]
#define __pyx_n_u_oldname __pyx_string_tab[471]
#define __pyx_n_u_oldparent __pyx_string_tab[472]
#define __pyx_n_u_oldpathname __pyx_string_tab[473]
#define __pyx_n_u_open_array __pyx_string_tab[474]
#define __pyx_n_u_open_unimplemented __pyx_string_tab[475]
#define __pyx_n_u_opt_read __pyx_string_tab[476]
#define __pyx_n_u_opt_write __pyx_string_tab[477]
#define __pyx_n_u_os __pyx_string_tab[478]
#define __pyx_n_u_out __pyx_string_tab[479]
#define __pyx_n_u_params __pyx_string_tab[480]
#define __pyx_n_u_parent __pyx_string_tab[481]
#define __pyx_n_u_path __pyx_string_tab[482]
#define __pyx_n_u_perform_selection __pyx_string_tab[483]
#define __pyx_n_u_pickle __pyx_string_tab[484]
#define __pyx_n_u_platform __pyx_string_tab[485]
#define __pyx_n_u_platform_byteorder __pyx_string_tab[486]
#define __pyx_n_u_plist_id __pyx_string_tab[487]
#define __pyx_n_u_pop __pyx_string_tab[488]
#define __pyx_n_u_pt_special_kinds __pyx_string_tab[489]
#define __pyx_n_u_ptparams __pyx_string_tab[490]
#define __pyx_n_u_pttype_to_hdf5 __pyx_string_tab[491]
#define __pyx_n_u_pure_numpy_types __pyx_string_tab[492]
#define __pyx_n_u_pymode __pyx_string_tab[493]
#define __pyx_n_u_pyx_checksum __pyx_string_tab[494]
#define __pyx_n_u_pyx_result __pyx_string_tab[495]
#define __pyx_n_u_pyx_state __pyx_string_tab[496]
#define __pyx_n_u_pyx_type __pyx_string_tab[497]
#define __pyx_n_u_pyx_unpickle_AttributeSet __pyx_string_tab[498]
#define __pyx_n_u_pyx_unpickle_File __pyx_string_tab[499]
#define __pyx_n_u_pyx_unpickle_Group __pyx_string_tab[500]
#define __pyx_n_u_pyx_unpickle_Node __pyx_string_tab[501]
#define __pyx_n_u_pyx_vtable __pyx_string_tab[502]
#define __pyx_n_u_qualname __pyx_string_tab[503]
#define __pyx_n_u_r __pyx_string_tab[504]
#define __pyx_n_u_rabyteorder __pyx_string_tab[505]
#define __pyx_n_u_rank __pyx_string_tab[506]
#define __pyx_n_u_rarr __pyx_string_tab[507]
#define __pyx_n_u_raw_ext __pyx_string_tab[508]
#define __pyx_n_u_raw_name __pyx_string_tab[509]
#define __pyx_n_u_raw_plist_id __pyx_string_tab[510]
#define __pyx_n_u_rbuf __pyx_string_tab[511]
#define __pyx_n_u_rc __pyx_string_tab[512]
#define __pyx_n_u_rdata __pyx_string_tab[513]
#define __pyx_n_u_read_array __pyx_string_tab[514]
#define __pyx_n_u_reduce __pyx_string_tab[515]
#define __pyx_n_u_reduce_cython __pyx_string_tab[516]
#define __pyx_n_u_reduce_ex __pyx_string_tab[517]
#define __pyx_n_u_refbuf __pyx_string_tab[518]
#define __pyx_n_u_reference __pyx_string_tab[519]
#define __pyx_n_u_ret __pyx_string_tab[520]
#define __pyx_n_u_retvalue __pyx_string_tab[521]
#define __pyx_n_u_row __pyx_string_tab[522]
#define __pyx_n_u_rstrip __pyx_string_tab[523]
#define __pyx_n_u_scatom __pyx_string_tab[524]
#define __pyx_n_u_scope __pyx_string_tab[525]
#define __pyx_n_u_select_mode __pyx_string_tab[526]
#define __pyx_n_u_select_modes __pyx_string_tab[527]
#define __pyx_n_u_selection __pyx_string_tab[528]
#define __pyx_n_u_self __pyx_string_tab[529]
#define __pyx_n_u_set_blosc2_max_threads __pyx_string_tab[530]
#define __pyx_n_u_set_blosc_max_threads __pyx_string_tab[531]
#define __pyx_n_u_set_name __pyx_string_tab[532]
#define __pyx_n_u_setdefault __pyx_string_tab[533]
#define __pyx_n_u_setstate __pyx_string_tab[534]
#define __pyx_n_u_setstate_cython __pyx_string_tab[535]
#define __pyx_n_u_shape __pyx_string_tab[536]
#define __pyx_n_u_shape_2 __pyx_string_tab[537]
#define __pyx_n_u_shape_atom __pyx_string_tab[538]
#define __pyx_n_u_shuffle_bitshuffle __pyx_string_tab[539]
#define __pyx_n_u_size __pyx_string_tab[540]
#define __pyx_n_u_space_id __pyx_string_tab[541]
#define __pyx_n_u_start __pyx_string_tab[542]
#define __pyx_n_u_start_2 __pyx_string_tab[543]
#define __pyx_n_u_startl __pyx_string_tab[544]
#define __pyx_n_u_startp __pyx_string_tab[545]
#define __pyx_n_u_startswith __pyx_string_tab[546]
#define __pyx_n_u_state __pyx_string_tab[547]
#define __pyx_n_u_step __pyx_string_tab[548]
#define __pyx_n_u_step_2 __pyx_string_tab[549]
#define __pyx_n_u_stepl __pyx_string_tab[550]
#define __pyx_n_u_stepp __pyx_string_tab[551]
#define __pyx_n_u_stop __pyx_string_tab[552]
#define __pyx_n_u_stopl __pyx_string_tab[553]
#define __pyx_n_u_str __pyx_string_tab[554]
#define __pyx_n_u_str_value __pyx_string_tab[555]
#define __pyx_n_u_str_values __pyx_string_tab[556]
#define __pyx_n_u_stype __pyx_string_tab[557]
#define __pyx_n_u_stype_atom __pyx_string_tab[558]
#define __pyx_n_u_super __pyx_string_tab[559]
#define __pyx_n_u_supported_drivers __pyx_string_tab[560]
#define __pyx_n_u_supports_opt_blosc2_read_write __pyx_string_tab[561]
#define __pyx_n_u_sys __pyx_string_tab[562]
#define __pyx_n_u_system __pyx_string_tab[563]
#define __pyx_n_u_tables_hdf5extension __pyx_string_tab[564]
#define __pyx_n_u_temp __pyx_string_tab[565]
#define __pyx_n_u_test __pyx_string_tab[566]
#define __pyx_n_u_time __pyx_string_tab[567]
#define __pyx_n_u_time64 __pyx_string_tab[568]
#define __pyx_n_u_title __pyx_string_tab[569]
#define __pyx_n_u_track_times __pyx_string_tab[570]
#define __pyx_n_u_type __pyx_string_tab[571]
#define __pyx_n_u_type_2 __pyx_string_tab[572]
#define __pyx_n_u_type_id __pyx_string_tab[573]
#define __pyx_n_u_type_precision __pyx_string_tab[574]
#define __pyx_n_u_type_size __pyx_string_tab[575]
#define __pyx_n_u_u __pyx_string_tab[576]
#define __pyx_n_u_u1 __pyx_string_tab[577]
#define __pyx_n_u_update __pyx_string_tab[578]
#define __pyx_n_u_use_setstate __pyx_string_tab[579]
#define __pyx_n_u_user_block_size __pyx_string_tab[580]
#define __pyx_n_u_utf8 __pyx_string_tab[581]
#define __pyx_n_u_utils __pyx_string_tab[582]
#define __pyx_n_u_utilsextension __pyx_string_tab[583]
#define __pyx_n_u_v_dtype __pyx_string_tab[584]
#define __pyx_n_u_v_file __pyx_string_tab[585]
#define __pyx_n_u_v_name __pyx_string_tab[586]
#define __pyx_n_u_v_new __pyx_string_tab[587]
#define __pyx_n_u_v_node __pyx_string_tab[588]
#define __pyx_n_u_v_objectid __pyx_string_tab[589]
#define __pyx_n_u_v_parent __pyx_string_tab[590]
#define __pyx_n_u_v_pathname __pyx_string_tab[591]
#define __pyx_n_u_v_unimplemented __pyx_string_tab[592]
#define __pyx_n_u_v_version __pyx_string_tab[593]
#define __pyx_n_u_value __pyx_string_tab[594]
#define __pyx_n_u_values __pyx_string_tab[595]
#define __pyx_n_u_version __pyx_string_tab[596]
#define __pyx_n_u_vllen __pyx_string_tab[597]
#define __pyx_n_u_w __pyx_string_tab[598]
#define __pyx_n_u_want_track_times __pyx_string_tab[599]
#define __pyx_n_u_warn __pyx_string_tab[600]
#define __pyx_n_u_warnings __pyx_string_tab[601]
#define __pyx_n_u_wbuf __pyx_string_tab[602]
#define __pyx_n_u_where __pyx_string_tab[603]
#define __pyx_n_u_windows __pyx_string_tab[604]
#define __pyx_n_u_writeable __pyx_string_tab[605]
#define __pyx_n_u_x __pyx_string_tab[606]
#define __pyx_n_u_zeros __pyx_string_tab[607]
#define __pyx_kp_b_ __pyx_string_tab[608]
#define __pyx_kp_b_PyObject_char_const_hid_t_hid_t __pyx_string_tab[609]
#define __pyx_kp_b__2 __pyx_string_tab[610]
#define __pyx_kp_b_iso88591_1D_q_t2Q_L_2T_a_Qa __pyx_string_tab[611]
#define __pyx_kp_b_iso88591_1F __pyx_string_tab[612]
#define __pyx_kp_b_iso88591_1_E_4uG1A_4q_t7_6_T_T_avQ_a_t_1 __pyx_string_tab[613]
#define __pyx_kp_b_iso88591_1_E_4uG1A_XV1_uCuCt7_Q_uF_d_Qa __pyx_string_tab[614]
#define __pyx_kp_b_iso88591_1_xwaq_d_1IQ_t2Q_L_1_Q __pyx_string_tab[615]
#define __pyx_kp_b_iso88591_4q __pyx_string_tab[616]
#define __pyx_kp_b_iso88591_4q_t2Q_L_9_4q_A __pyx_string_tab[617]
#define __pyx_kp_b_iso88591_4uG1A_1D_N_q_t2Q_L_6b_A_A_4q __pyx_string_tab[618]
#define __pyx_kp_b_iso88591_4uG1A_1F_a_t2Q_L_Rt1_1 __pyx_string_tab[619]
#define __pyx_kp_b_iso88591_4uG1A_81F_N __pyx_string_tab[620]
#define __pyx_kp_b_iso88591_4uG1A_L_q_E_Qj_gQd_nA_Qa_G_d __pyx_string_tab[621]
#define __pyx_kp_b_iso88591_4uG1A_gQd_nA_t_r_L_gT_1_t_d_at1 __pyx_string_tab[622]
#define __pyx_kp_b_iso88591_4uG1A_gQd_nA_t_r_L_gT_1_t_d_at1_2 __pyx_string_tab[623]
#define __pyx_kp_b_iso88591_4uG1A_l_t2Q_L_7r_Q_A_4q __pyx_string_tab[624]
#define __pyx_kp_b_iso88591_6 __pyx_string_tab[625]
#define __pyx_kp_b_iso88591_6_2 __pyx_string_tab[626]
#define __pyx_kp_b_iso88591_6_A_at_a_t4r_D_1_S_S_S_k_k_4t1 __pyx_string_tab[627]
#define __pyx_kp_b_iso88591_7_1_q_q_K_1_z_1_L_d_1Kq_AQ_1 __pyx_string_tab[628]
#define __pyx_kp_b_iso88591_A __pyx_string_tab[629]
#define __pyx_kp_b_iso88591_AT __pyx_string_tab[630]
#define __pyx_kp_b_iso88591_AT_1 __pyx_string_tab[631]
#define __pyx_kp_b_iso88591_AT_A __pyx_string_tab[632]
#define __pyx_kp_b_iso88591_AWF_vRvRt1_L_Q_q_Qa_q_A_q_1G1_1 __pyx_string_tab[633]
#define __pyx_kp_b_iso88591_A_aq_N_4_M_a_t2Q_L_4q __pyx_string_tab[634]
#define __pyx_kp_b_iso88591_A_t_r_Ja_r_Q_4q_y_L_0_A_q_l_102 __pyx_string_tab[635]
#define __pyx_kp_b_iso88591_A_vWD_aq_uCq_Q_t7_t3auBa_Ja_5Qc __pyx_string_tab[636]
#define __pyx_kp_b_iso88591_Bd_1 __pyx_string_tab[637]
#define __pyx_kp_b_iso88591_E_4uG1A_4q_A_1D_1D_t1_1D_t1_q_L __pyx_string_tab[638]
#define __pyx_kp_b_iso88591_HHD_A_5_a_3a_gQa_gQa_WAQ_gQa_gQ __pyx_string_tab[639]
#define __pyx_kp_b_iso88591_Kq_A_1D_5_1Cq_a_A_t5_c_vQe6_1_Q __pyx_string_tab[640]
#define __pyx_kp_b_iso88591_Kq_A_1D_5_1Cq_a_az_v_AQ_t5_c_vQ __pyx_string_tab[641]
#define __pyx_kp_b_iso88591_Kq_A_AWF_t5_c_vQfBa_Q_wavQ_t1_k __pyx_string_tab[642]
#define __pyx_kp_b_iso88591_Kq_A_K_1A_l_1_l_1_t5_c_vQe6_1_Q __pyx_string_tab[643]
#define __pyx_kp_b_iso88591_Q __pyx_string_tab[644]
#define __pyx_kp_b_iso88591_Q_1 __pyx_string_tab[645]
#define __pyx_kp_b_iso88591_Q_1_A_xwaq_d_1_ay_1IQa_1A_t2Q_L __pyx_string_tab[646]
#define __pyx_kp_b_iso88591_Q_2 __pyx_string_tab[647]
#define __pyx_kp_b_iso88591_T_A_G1F_a_vWE_Q_q_t6_q_T_G1_T_A __pyx_string_tab[648]
#define __pyx_kp_b_iso88591_T_D_t1_G1F_a_vWE_Q_q_t6_q_d_7_W __pyx_string_tab[649]
#define __pyx_kp_b_iso88591_T_G1F_a_vWE_Q_q_t6_q_T_G1_T_A __pyx_string_tab[650]
#define __pyx_kp_b_iso88591_T_Jd_G1F_a_vWE_Q_q_t6_q_T_G1_T __pyx_string_tab[651]
#define __pyx_kp_b_iso88591_U_2 __pyx_string_tab[652]
#define __pyx_kp_b_iso88591_a_AT_6_uBa_L_QfA_t1_L_Q_a_AT_7 __pyx_string_tab[653]
#define __pyx_kp_b_iso88591_a_A_1_31_1_V1A_wgU_gWA_Ja_r_wc __pyx_string_tab[654]
#define __pyx_kp_b_iso88591_a_Qd_t2Q_L_a_Q_7_5_uA __pyx_string_tab[655]
#define __pyx_kp_b_iso88591_a_Qd_t2Q_L_a_Q_he85_a __pyx_string_tab[656]
#define __pyx_kp_b_iso88591_a_at1_Ba_L_t2Q_aq_L_AQ_1 __pyx_string_tab[657]
#define __pyx_kp_b_iso88591_a_j_t2Q_L_1 __pyx_string_tab[658]
#define __pyx_kp_b_iso88591_aq_at_QfAQ_t2Q_L_4q_A_c_a __pyx_string_tab[659]
#define __pyx_kp_b_iso88591_gQd_8_xr_L_T_1IQ_AQ_1 __pyx_string_tab[660]
#define __pyx_kp_b_iso88591_gWAQ_gWAQ_k_q_t2Q_L_q_1 __pyx_string_tab[661]
#define __pyx_kp_b_iso88591_q_0_kQR_4xq_7_awnA_1 __pyx_string_tab[662]
#define __pyx_kp_b_iso88591_q_0_kQR_5_7_q_a_1 __pyx_string_tab[663]
#define __pyx_kp_b_iso88591_q_0_kQR_xq_7_a_nA_1 __pyx_string_tab[664]
#define __pyx_kp_b_iso88591_q_A_4waq_A_d_z_1_b_aq_7_Ja_V6_U __pyx_string_tab[665]
#define __pyx_kp_b_iso88591_q_M_Zq_t2Q_L_Ba_Ja_z_Zq_q_L_d_4 __pyx_string_tab[666]
#define __pyx_kp_b_iso88591_q_U_1_AWA_Q_Qd_t1_0_Kq_t2Q_L_D __pyx_string_tab[667]
#define __pyx_kp_b_iso88591_q_a_1_1_7_7 __pyx_string_tab[668]
#define __pyx_kp_b_iso88591_q_q_U_1_AWA_Qd_t1_Ja_t2Q_L_1 __pyx_string_tab[669]
#define __pyx_kp_b_iso88591_s_1_Jc_A_AS_3k_hgRvS_a_1_Ja __pyx_string_tab[670]
#define __pyx_kp_b_iso88591_t3d_L_Q_1D_5_q_Qj_8_gQ_t2Q_a_at __pyx_string_tab[671]
#define __pyx_kp_b_iso88591_t5_c_Jaq_1D_5_1Cq_a_A_q_t5_c_7 __pyx_string_tab[672]
#define __pyx_kp_b_iso88591_t5_c_Jaq_1D_5_1Cq_a_az_v_AQ_q_t __pyx_string_tab[673]
#define __pyx_kp_b_iso88591_t5_c_Jaq_at7_aq_q_t5_c_7_T_1D_T __pyx_string_tab[674]
#define __pyx_kp_b_iso88591_t5_c_Jaq_q_K_1A_l_1_K_1A_t5_c_7 __pyx_string_tab[675]
#define __pyx_kp_b_iso88591_t7_Q_Q_Qd_j_Rq_aq_1 __pyx_string_tab[676]
#define __pyx_kp_b_iso88591_t9Cq_at1_t_A_at1_t_A_at1_t_s_at __pyx_string_tab[677]
#define __pyx_kp_b_iso88591_t_s __pyx_string_tab[678]
#define __pyx_int_0 __pyx_number_tab[0]
#define __pyx_int_neg_1 __pyx_number_tab[1]
#define __pyx_int_1 __pyx_number_tab[2]
#define __pyx_int_55151181 __pyx_number_tab[3]
#define __pyx_int_123500641 __pyx_number_tab[4]
#define __pyx_int_136983863 __pyx_number_tab[5]
#define __pyx_int_223533300 __pyx_number_tab[6]
/* #### Code section: module_state_clear ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_clear(PyObject *m) {
  __pyx_mstatetype *clear_module_state = __Pyx_PyModule_GetState(m);
  if (!clear_module_state) return 0;
  Py_CLEAR(clear_module_state->__pyx_d);
  Py_CLEAR(clear_module_state->__pyx_b);
  Py_CLEAR(clear_module_state->__pyx_cython_runtime);
  Py_CLEAR(clear_module_state->__pyx_empty_tuple);
  Py_CLEAR(clear_module_state->__pyx_empty_bytes);
  Py_CLEAR(clear_module_state->__pyx_empty_unicode);
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __Pyx_State_RemoveModule(NULL);
  #endif
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_4type_type);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_dtype);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_flatiter);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_broadcast);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_ndarray);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_generic);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_number);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_integer);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_signedinteger);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_unsignedinteger);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_inexact);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_floating);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_complexfloating);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_flexible);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_character);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_ufunc);
  Py_CLEAR(clear_module_state->__pyx_ptype_6tables_13hdf5extension_Node);
  Py_CLEAR(clear_module_state->__pyx_type_6tables_13hdf5extension_Node);
  Py_CLEAR(clear_module_state->__pyx_ptype_6tables_13hdf5extension_Leaf);
  Py_CLEAR(clear_module_state->__pyx_type_6tables_13hdf5extension_Leaf);
  Py_CLEAR(clear_module_state->__pyx_ptype_6tables_13hdf5extension_Array);
  Py_CLEAR(clear_module_state->__pyx_type_6tables_13hdf5extension_Array);
  Py_CLEAR(clear_module_state->__pyx_ptype_6tables_13hdf5extension_File);
  Py_CLEAR(clear_module_state->__pyx_type_6tables_13hdf5extension_File);
  Py_CLEAR(clear_module_state->__pyx_ptype_6tables_13hdf5extension_AttributeSet);
  Py_CLEAR(clear_module_state->__pyx_type_6tables_13hdf5extension_AttributeSet);
  Py_CLEAR(clear_module_state->__pyx_ptype_6tables_13hdf5extension_Group);
  Py_CLEAR(clear_module_state->__pyx_type_6tables_13hdf5extension_Group);
  Py_CLEAR(clear_module_state->__pyx_ptype_6tables_13hdf5extension_VLArray);
  Py_CLEAR(clear_module_state->__pyx_type_6tables_13hdf5extension_VLArray);
  Py_CLEAR(clear_module_state->__pyx_ptype_6tables_13hdf5extension_UnImplemented);
  Py_CLEAR(clear_module_state->__pyx_type_6tables_13hdf5extension_UnImplemented);
  for (int i=0; i<3; ++i) { Py_CLEAR(clear_module_state->__pyx_tuple[i]); }
  for (int i=0; i<77; ++i) { Py_CLEAR(clear_module_state->__pyx_codeobj_tab[i]); }
  for (int i=0; i<679; ++i) { Py_CLEAR(clear_module_state->__pyx_string_tab[i]); }
  for (int i=0; i<7; ++i) { Py_CLEAR(clear_module_state->__pyx_number_tab[i]); }
/* #### Code section: module_state_clear_contents ### */
/* CommonTypesMetaclass.module_state_clear */
Py_CLEAR(clear_module_state->__pyx_CommonTypesMetaclassType);

/* CythonFunctionShared.module_state_clear */
Py_CLEAR(clear_module_state->__pyx_CyFunctionType);

/* #### Code section: module_state_clear_end ### */
return 0;
}
#endif
/* #### Code section: module_state_traverse ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_traverse(PyObject *m, visitproc visit, void *arg) {
  __pyx_mstatetype *traverse_module_state = __Pyx_PyModule_GetState(m);
  if (!traverse_module_state) return 0;
  Py_VISIT(traverse_module_state->__pyx_d);
  Py_VISIT(traverse_module_state->__pyx_b);
  Py_VISIT(traverse_module_state->__pyx_cython_runtime);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_tuple);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_bytes);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_unicode);
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_4type_type);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_dtype);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_flatiter);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_broadcast);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_ndarray);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_generic);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_number);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_integer);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_signedinteger);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_unsignedinteger);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_inexact);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_floating);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_complexfloating);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_flexible);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_character);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_ufunc);
  Py_VISIT(traverse_module_state->__pyx_ptype_6tables_13hdf5extension_Node);
  Py_VISIT(traverse_module_state->__pyx_type_6tables_13hdf5extension_Node);
  Py_VISIT(traverse_module_state->__pyx_ptype_6tables_13hdf5extension_Leaf);
  Py_VISIT(traverse_module_state->__pyx_type_6tables_13hdf5extension_Leaf);
  Py_VISIT(traverse_module_state->__pyx_ptype_6tables_13hdf5extension_Array);
  Py_VISIT(traverse_module_state->__pyx_type_6tables_13hdf5extension_Array);
  Py_VISIT(traverse_module_state->__pyx_ptype_6tables_13hdf5extension_File);
  Py_VISIT(traverse_module_state->__pyx_type_6tables_13hdf5extension_File);
  Py_VISIT(traverse_module_state->__pyx_ptype_6tables_13hdf5extension_AttributeSet);
  Py_VISIT(traverse_module_state->__pyx_type_6tables_13hdf5extension_AttributeSet);
  Py_VISIT(traverse_module_state->__pyx_ptype_6tables_13hdf5extension_Group);
  Py_VISIT(traverse_module_state->__pyx_type_6tables_13hdf5extension_Group);
  Py_VISIT(traverse_module_state->__pyx_ptype_6tables_13hdf5extension_VLArray);
  Py_VISIT(traverse_module_state->__pyx_type_6tables_13hdf5extension_VLArray);
  Py_VISIT(traverse_module_state->__pyx_ptype_6tables_13hdf5extension_UnImplemented);
  Py_VISIT(traverse_module_state->__pyx_type_6tables_13hdf5extension_UnImplemented);
  for (int i=0; i<3; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_tuple[i]); }
  for (int i=0; i<77; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_codeobj_tab[i]); }
  for (int i=0; i<679; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_string_tab[i]); }
  for (int i=0; i<7; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_number_tab[i]); }
/* #### Code section: module_state_traverse_contents ### */
/* CommonTypesMetaclass.module_state_traverse */
Py_VISIT(traverse_module_state->__pyx_CommonTypesMetaclassType);

/* CythonFunctionShared.module_state_traverse */
Py_VISIT(traverse_module_state->__pyx_CyFunctionType);

/* #### Code section: module_state_traverse_end ### */
return 0;
}
#endif
/* #### Code section: module_code ### */

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":230
 * 
 *     ctypedef class numpy.dtype [object PyArray_Descr, check_size ignore]:
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline PyTypeObject* typeobj(self) noexcept nogil:
 *             return PyDataType_TYPEOBJ(self)
*/

static CYTHON_INLINE PyTypeObject *__pyx_f_5numpy_5dtype_7typeobj_typeobj(PyArray_Descr *__pyx_v_self) {
  PyTypeObject *__pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":232
 *         @property
 *         cdef inline PyTypeObject* typeobj(self) noexcept nogil:
 *             return PyDataType_TYPEOBJ(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = PyDataType_TYPEOBJ(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":230
 * 
 *     ctypedef class numpy.dtype [object PyArray_Descr, check_size ignore]:
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline PyTypeObject* typeobj(self) noexcept nogil:
 *             return PyDataType_TYPEOBJ(self)
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":234
 *             return PyDataType_TYPEOBJ(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline char kind(self) noexcept nogil:
 *             return PyDataType_KIND(self)
*/

static CYTHON_INLINE char __pyx_f_5numpy_5dtype_4kind_kind(PyArray_Descr *__pyx_v_self) {
  char __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":236
 *         @property
 *         cdef inline char kind(self) noexcept nogil:
 *             return PyDataType_KIND(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = PyDataType_KIND(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":234
 *             return PyDataType_TYPEOBJ(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline char kind(self) noexcept nogil:
 *             return PyDataType_KIND(self)
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":238
 *             return PyDataType_KIND(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline char type(self) noexcept nogil:
 *             return PyDataType_TYPE(self)
*/

static CYTHON_INLINE char __pyx_f_5numpy_5dtype_4type_type(PyArray_Descr *__pyx_v_self) {
  char __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":240
 *         @property
 *         cdef inline char type(self) noexcept nogil:
 *             return PyDataType_TYPE(self)             # <<<<<<<<<<<<<<
 * 
 *         # Numpy sometimes mutates this without warning (e.g. it'll
*/
  __pyx_r = PyDataType_TYPE(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":238
 *             return PyDataType_KIND(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline char type(self) noexcept nogil:
 *             return PyDataType_TYPE(self)
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":247
 *         # PyArray_IsNativeByteOrder(dtype.byteorder) instead of
 *         # directly accessing this field.
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline char byteorder(self) noexcept nogil:
 *             return PyDataType_BYTEORDER(self)
*/

static CYTHON_INLINE char __pyx_f_5numpy_5dtype_9byteorder_byteorder(PyArray_Descr *__pyx_v_self) {
  char __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":249
 *         @property
 *         cdef inline char byteorder(self) noexcept nogil:
 *             return PyDataType_BYTEORDER(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = PyDataType_BYTEORDER(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":247
 *         # PyArray_IsNativeByteOrder(dtype.byteorder) instead of
 *         # directly accessing this field.
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline char byteorder(self) noexcept nogil:
 *             return PyDataType_BYTEORDER(self)
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":251
 *             return PyDataType_BYTEORDER(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline int type_num(self) noexcept nogil:
 *             return PyDataType_TYPENUM(self)
*/

static CYTHON_INLINE int __pyx_f_5numpy_5dtype_8type_num_type_num(PyArray_Descr *__pyx_v_self) {
  int __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":253
 *         @property
 *         cdef inline int type_num(self) noexcept nogil:
 *             return PyDataType_TYPENUM(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = PyDataType_TYPENUM(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":251
 *             return PyDataType_BYTEORDER(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline int type_num(self) noexcept nogil:
 *             return PyDataType_TYPENUM(self)
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":255
 *             return PyDataType_TYPENUM(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline npy_intp itemsize(self) noexcept nogil:
 *             return PyDataType_ELSIZE(self)
*/

static CYTHON_INLINE npy_intp __pyx_f_5numpy_5dtype_8itemsize_itemsize(PyArray_Descr *__pyx_v_self) {
  npy_intp __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":257
 *         @property
 *         cdef inline npy_intp itemsize(self) noexcept nogil:
 *             return PyDataType_ELSIZE(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = PyDataType_ELSIZE(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":255
 *             return PyDataType_TYPENUM(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline npy_intp itemsize(self) noexcept nogil:
 *             return PyDataType_ELSIZE(self)
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":259
 *             return PyDataType_ELSIZE(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline npy_intp alignment(self) noexcept nogil:
 *             return PyDataType_ALIGNMENT(self)
*/

static CYTHON_INLINE npy_intp __pyx_f_5numpy_5dtype_9alignment_alignment(PyArray_Descr *__pyx_v_self) {
  npy_intp __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":261
 *         @property
 *         cdef inline npy_intp alignment(self) noexcept nogil:
 *             return PyDataType_ALIGNMENT(self)             # <<<<<<<<<<<<<<
 * 
 *         # Use fields/names with care as they may be NULL.  You must check
*/
  __pyx_r = PyDataType_ALIGNMENT(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":259
 *             return PyDataType_ELSIZE(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline npy_intp alignment(self) noexcept nogil:
 *             return PyDataType_ALIGNMENT(self)
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":265
 *         # Use fields/names with care as they may be NULL.  You must check
 *         # for this using PyDataType_HASFIELDS.
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline object fields(self):
 *             return <object>PyDataType_FIELDS(self)
*/

static CYTHON_INLINE PyObject *__pyx_f_5numpy_5dtype_6fields_fields(PyArray_Descr *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1;
  __Pyx_RefNannySetupContext("fields", 0);

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":267
 *         @property
 *         cdef inline object fields(self):
 *             return <object>PyDataType_FIELDS(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyDataType_FIELDS(__pyx_v_self);
  __Pyx_INCREF(((PyObject *)__pyx_t_1));
  __pyx_r = ((PyObject *)__pyx_t_1);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":265
 *         # Use fields/names with care as they may be NULL.  You must check
 *         # for this using PyDataType_HASFIELDS.
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline object fields(self):
 *             return <object>PyDataType_FIELDS(self)
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":269
 *             return <object>PyDataType_FIELDS(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline tuple names(self):
 *             return <tuple>PyDataType_NAMES(self)
*/

static CYTHON_INLINE PyObject *__pyx_f_5numpy_5dtype_5names_names(PyArray_Descr *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1;
  __Pyx_RefNannySetupContext("names", 0);

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":271
 *         @property
 *         cdef inline tuple names(self):
 *             return <tuple>PyDataType_NAMES(self)             # <<<<<<<<<<<<<<
 * 
 *         # Use PyDataType_HASSUBARRAY to test whether this field is
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyDataType_NAMES(__pyx_v_self);
  __Pyx_INCREF(((PyObject*)__pyx_t_1));
  __pyx_r = ((PyObject*)__pyx_t_1);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":269
 *             return <object>PyDataType_FIELDS(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline tuple names(self):
 *             return <tuple>PyDataType_NAMES(self)
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":276
 *         # valid (the pointer can be NULL). Most users should access
 *         # this field via the inline helper method PyDataType_SHAPE.
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline PyArray_ArrayDescr* subarray(self) noexcept nogil:
 *             return PyDataType_SUBARRAY(self)
*/

static CYTHON_INLINE PyArray_ArrayDescr *__pyx_f_5numpy_5dtype_8subarray_subarray(PyArray_Descr *__pyx_v_self) {
  PyArray_ArrayDescr *__pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":278
 *         @property
 *         cdef inline PyArray_ArrayDescr* subarray(self) noexcept nogil:
 *             return PyDataType_SUBARRAY(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = PyDataType_SUBARRAY(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":276
 *         # valid (the pointer can be NULL). Most users should access
 *         # this field via the inline helper method PyDataType_SHAPE.
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline PyArray_ArrayDescr* subarray(self) noexcept nogil:
 *             return PyDataType_SUBARRAY(self)
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":280
 *             return PyDataType_SUBARRAY(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline npy_uint64 flags(self) noexcept nogil:
 *             """The data types flags."""
*/

static CYTHON_INLINE npy_uint64 __pyx_f_5numpy_5dtype_5flags_flags(PyArray_Descr *__pyx_v_self) {
  npy_uint64 __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":283
 *         cdef inline npy_uint64 flags(self) noexcept nogil:
 *             """The data types flags."""
 *             return PyDataType_FLAGS(self)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = PyDataType_FLAGS(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":280
 *             return PyDataType_SUBARRAY(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline npy_uint64 flags(self) noexcept nogil:
 *             """The data types flags."""
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":292
 *     ctypedef class numpy.broadcast [object PyArrayMultiIterObject, check_size ignore]:
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline int numiter(self) noexcept nogil:
 *             """The number of arrays that need to be broadcast to the same shape."""
*/

static CYTHON_INLINE int __pyx_f_5numpy_9broadcast_7numiter_numiter(PyArrayMultiIterObject *__pyx_v_self) {
  int __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":295
 *         cdef inline int numiter(self) noexcept nogil:
 *             """The number of arrays that need to be broadcast to the same shape."""
 *             return PyArray_MultiIter_NUMITER(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = PyArray_MultiIter_NUMITER(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":292
 *     ctypedef class numpy.broadcast [object PyArrayMultiIterObject, check_size ignore]:
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline int numiter(self) noexcept nogil:
 *             """The number of arrays that need to be broadcast to the same shape."""
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":297
 *             return PyArray_MultiIter_NUMITER(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline npy_intp size(self) noexcept nogil:
 *             """The total broadcasted size."""
*/

static CYTHON_INLINE npy_intp __pyx_f_5numpy_9broadcast_4size_size(PyArrayMultiIterObject *__pyx_v_self) {
  npy_intp __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":300
 *         cdef inline npy_intp size(self) noexcept nogil:
 *             """The total broadcasted size."""
 *             return PyArray_MultiIter_SIZE(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = PyArray_MultiIter_SIZE(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":297
 *             return PyArray_MultiIter_NUMITER(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline npy_intp size(self) noexcept nogil:
 *             """The total broadcasted size."""
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":302
 *             return PyArray_MultiIter_SIZE(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline npy_intp index(self) noexcept nogil:
 *             """The current (1-d) index into the broadcasted result."""
*/

static CYTHON_INLINE npy_intp __pyx_f_5numpy_9broadcast_5index_index(PyArrayMultiIterObject *__pyx_v_self) {
  npy_intp __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":305
 *         cdef inline npy_intp index(self) noexcept nogil:
 *             """The current (1-d) index into the broadcasted result."""
 *             return PyArray_MultiIter_INDEX(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = PyArray_MultiIter_INDEX(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":302
 *             return PyArray_MultiIter_SIZE(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline npy_intp index(self) noexcept nogil:
 *             """The current (1-d) index into the broadcasted result."""
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":307
 *             return PyArray_MultiIter_INDEX(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline int nd(self) noexcept nogil:
 *             """The number of dimensions in the broadcasted result."""
*/

static CYTHON_INLINE int __pyx_f_5numpy_9broadcast_2nd_nd(PyArrayMultiIterObject *__pyx_v_self) {
  int __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":310
 *         cdef inline int nd(self) noexcept nogil:
 *             """The number of dimensions in the broadcasted result."""
 *             return PyArray_MultiIter_NDIM(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = PyArray_MultiIter_NDIM(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":307
 *             return PyArray_MultiIter_INDEX(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline int nd(self) noexcept nogil:
 *             """The number of dimensions in the broadcasted result."""
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":312
 *             return PyArray_MultiIter_NDIM(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline npy_intp* dimensions(self) noexcept nogil:
 *             """The shape of the broadcasted result."""
*/

static CYTHON_INLINE npy_intp *__pyx_f_5numpy_9broadcast_10dimensions_dimensions(PyArrayMultiIterObject *__pyx_v_self) {
  npy_intp *__pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":315
 *         cdef inline npy_intp* dimensions(self) noexcept nogil:
 *             """The shape of the broadcasted result."""
 *             return PyArray_MultiIter_DIMS(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = PyArray_MultiIter_DIMS(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":312
 *             return PyArray_MultiIter_NDIM(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline npy_intp* dimensions(self) noexcept nogil:
 *             """The shape of the broadcasted result."""
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":317
 *             return PyArray_MultiIter_DIMS(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline void** iters(self) noexcept nogil:
 *             """An array of iterator objects that holds the iterators for the arrays to be broadcast together.
*/

static CYTHON_INLINE void **__pyx_f_5numpy_9broadcast_5iters_iters(PyArrayMultiIterObject *__pyx_v_self) {
  void **__pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":321
 *             """An array of iterator objects that holds the iterators for the arrays to be broadcast together.
 *             On return, the iterators are adjusted for broadcasting."""
 *             return PyArray_MultiIter_ITERS(self)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = PyArray_MultiIter_ITERS(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":317
 *             return PyArray_MultiIter_DIMS(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline void** iters(self) noexcept nogil:
 *             """An array of iterator objects that holds the iterators for the arrays to be broadcast together.
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":335
 *         # Instead, we use properties that map to the corresponding C-API functions.
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline PyObject* base(self) noexcept nogil:
 *             """Returns a borrowed reference to the object owning the data/memory.
*/

static CYTHON_INLINE PyObject *__pyx_f_5numpy_7ndarray_4base_base(PyArrayObject *__pyx_v_self) {
  PyObject *__pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":339
 *             """Returns a borrowed reference to the object owning the data/memory.
 *             """
 *             return PyArray_BASE(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = PyArray_BASE(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":335
 *         # Instead, we use properties that map to the corresponding C-API functions.
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline PyObject* base(self) noexcept nogil:
 *             """Returns a borrowed reference to the object owning the data/memory.
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":341
 *             return PyArray_BASE(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline dtype descr(self):
 *             """Returns an owned reference to the dtype of the array.
*/

static CYTHON_INLINE PyArray_Descr *__pyx_f_5numpy_7ndarray_5descr_descr(PyArrayObject *__pyx_v_self) {
  PyArray_Descr *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyArray_Descr *__pyx_t_1;
  __Pyx_RefNannySetupContext("descr", 0);

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":345
 *             """Returns an owned reference to the dtype of the array.
 *             """
 *             return <dtype>PyArray_DESCR(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_1 = PyArray_DESCR(__pyx_v_self);
  __Pyx_INCREF((PyObject *)((PyArray_Descr *)__pyx_t_1));
  __pyx_r = ((PyArray_Descr *)__pyx_t_1);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":341
 *             return PyArray_BASE(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline dtype descr(self):
 *             """Returns an owned reference to the dtype of the array.
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":347
 *             return <dtype>PyArray_DESCR(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline int ndim(self) noexcept nogil:
 *             """Returns the number of dimensions in the array.
*/

static CYTHON_INLINE int __pyx_f_5numpy_7ndarray_4ndim_ndim(PyArrayObject *__pyx_v_self) {
  int __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":351
 *             """Returns the number of dimensions in the array.
 *             """
 *             return PyArray_NDIM(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = PyArray_NDIM(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":347
 *             return <dtype>PyArray_DESCR(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline int ndim(self) noexcept nogil:
 *             """Returns the number of dimensions in the array.
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":353
 *             return PyArray_NDIM(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline npy_intp *shape(self) noexcept nogil:
 *             """Returns a pointer to the dimensions/shape of the array.
*/

static CYTHON_INLINE npy_intp *__pyx_f_5numpy_7ndarray_5shape_shape(PyArrayObject *__pyx_v_self) {
  npy_intp *__pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":359
 *             Can return NULL for 0-dimensional arrays.
 *             """
 *             return PyArray_DIMS(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = PyArray_DIMS(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":353
 *             return PyArray_NDIM(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline npy_intp *shape(self) noexcept nogil:
 *             """Returns a pointer to the dimensions/shape of the array.
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":361
 *             return PyArray_DIMS(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline npy_intp *strides(self) noexcept nogil:
 *             """Returns a pointer to the strides of the array.
*/

static CYTHON_INLINE npy_intp *__pyx_f_5numpy_7ndarray_7strides_strides(PyArrayObject *__pyx_v_self) {
  npy_intp *__pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":366
 *             The number of elements matches the number of dimensions of the array (ndim).
 *             """
 *             return PyArray_STRIDES(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = PyArray_STRIDES(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":361
 *             return PyArray_DIMS(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline npy_intp *strides(self) noexcept nogil:
 *             """Returns a pointer to the strides of the array.
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":368
 *             return PyArray_STRIDES(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline npy_intp size(self) noexcept nogil:
 *             """Returns the total size (in number of elements) of the array.
*/

static CYTHON_INLINE npy_intp __pyx_f_5numpy_7ndarray_4size_size(PyArrayObject *__pyx_v_self) {
  npy_intp __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":372
 *             """Returns the total size (in number of elements) of the array.
 *             """
 *             return PyArray_SIZE(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = PyArray_SIZE(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":368
 *             return PyArray_STRIDES(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline npy_intp size(self) noexcept nogil:
 *             """Returns the total size (in number of elements) of the array.
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":374
 *             return PyArray_SIZE(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline char* data(self) noexcept nogil:
 *             """The pointer to the data buffer as a char*.
*/

static CYTHON_INLINE char *__pyx_f_5numpy_7ndarray_4data_data(PyArrayObject *__pyx_v_self) {
  char *__pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":381
 *             of `PyArray_DATA()` instead, which returns a 'void*'.
 *             """
 *             return PyArray_BYTES(self)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = PyArray_BYTES(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":374
 *             return PyArray_SIZE(self)
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline char* data(self) noexcept nogil:
 *             """The pointer to the data buffer as a char*.
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":794
 * ctypedef long double complex clongdouble_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
*/

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":795
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 795, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":794
 * ctypedef long double complex clongdouble_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":797
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
*/

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":798
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 798, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":797
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":800
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
*/

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":801
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 801, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":800
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":803
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
*/

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":804
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 804, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":803
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":806
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
*/

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":807
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":806
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":809
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
*/

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2;
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 0);

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":810
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
*/
  __pyx_t_1 = PyDataType_HASSUBARRAY(__pyx_v_d);
  if (__pyx_t_1) {

    /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":811
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_5numpy_5dtype_8subarray_subarray(__pyx_v_d)->shape;
    __Pyx_INCREF(((PyObject*)__pyx_t_2));
    __pyx_r = ((PyObject*)__pyx_t_2);
    goto __pyx_L0;

    /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":810
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
*/
  }

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":813
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * 
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_empty_tuple);
    __pyx_r = __pyx_mstate_global->__pyx_empty_tuple;
    goto __pyx_L0;
  }

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":809
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":947
 * 
 *     ctypedef class numpy.ufunc [object PyUFuncObject, check_size ignore]:
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline int nin(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).nin
*/

static CYTHON_INLINE int __pyx_f_5numpy_5ufunc_3nin_nin(PyUFuncObject *__pyx_v_self) {
  int __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":949
 *         @property
 *         cdef inline int nin(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).nin             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = _PyUFuncObject_GET_ITEM_DATA(__pyx_v_self)->nin;
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":947
 * 
 *     ctypedef class numpy.ufunc [object PyUFuncObject, check_size ignore]:
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline int nin(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).nin
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":951
 *             return _PyUFuncObject_GET_ITEM_DATA(self).nin
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline int nout(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).nout
*/

static CYTHON_INLINE int __pyx_f_5numpy_5ufunc_4nout_nout(PyUFuncObject *__pyx_v_self) {
  int __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":953
 *         @property
 *         cdef inline int nout(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).nout             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = _PyUFuncObject_GET_ITEM_DATA(__pyx_v_self)->nout;
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":951
 *             return _PyUFuncObject_GET_ITEM_DATA(self).nin
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline int nout(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).nout
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":955
 *             return _PyUFuncObject_GET_ITEM_DATA(self).nout
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline int nargs(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).nargs
*/

static CYTHON_INLINE int __pyx_f_5numpy_5ufunc_5nargs_nargs(PyUFuncObject *__pyx_v_self) {
  int __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":957
 *         @property
 *         cdef inline int nargs(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).nargs             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = _PyUFuncObject_GET_ITEM_DATA(__pyx_v_self)->nargs;
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":955
 *             return _PyUFuncObject_GET_ITEM_DATA(self).nout
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline int nargs(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).nargs
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":959
 *             return _PyUFuncObject_GET_ITEM_DATA(self).nargs
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline PyUFuncGenericFunction* functions(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).functions
*/

static CYTHON_INLINE PyUFuncGenericFunction *__pyx_f_5numpy_5ufunc_9functions_functions(PyUFuncObject *__pyx_v_self) {
  PyUFuncGenericFunction *__pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":961
 *         @property
 *         cdef inline PyUFuncGenericFunction* functions(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).functions             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = _PyUFuncObject_GET_ITEM_DATA(__pyx_v_self)->functions;
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":959
 *             return _PyUFuncObject_GET_ITEM_DATA(self).nargs
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline PyUFuncGenericFunction* functions(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).functions
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":963
 *             return _PyUFuncObject_GET_ITEM_DATA(self).functions
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline void ** data(self) noexcept nogil:
 *             return <void **>_PyUFuncObject_GET_ITEM_DATA(self).data
*/

static CYTHON_INLINE void **__pyx_f_5numpy_5ufunc_4data_data(PyUFuncObject *__pyx_v_self) {
  void **__pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":965
 *         @property
 *         cdef inline void ** data(self) noexcept nogil:
 *             return <void **>_PyUFuncObject_GET_ITEM_DATA(self).data             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = ((void **)_PyUFuncObject_GET_ITEM_DATA(__pyx_v_self)->data);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":963
 *             return _PyUFuncObject_GET_ITEM_DATA(self).functions
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline void ** data(self) noexcept nogil:
 *             return <void **>_PyUFuncObject_GET_ITEM_DATA(self).data
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":967
 *             return <void **>_PyUFuncObject_GET_ITEM_DATA(self).data
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline int ntypes(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).ntypes
*/

static CYTHON_INLINE int __pyx_f_5numpy_5ufunc_6ntypes_ntypes(PyUFuncObject *__pyx_v_self) {
  int __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":969
 *         @property
 *         cdef inline int ntypes(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).ntypes             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = _PyUFuncObject_GET_ITEM_DATA(__pyx_v_self)->ntypes;
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":967
 *             return <void **>_PyUFuncObject_GET_ITEM_DATA(self).data
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline int ntypes(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).ntypes
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":971
 *             return _PyUFuncObject_GET_ITEM_DATA(self).ntypes
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline const char* name(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).name
*/

static CYTHON_INLINE char const *__pyx_f_5numpy_5ufunc_4name_name(PyUFuncObject *__pyx_v_self) {
  char const *__pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":973
 *         @property
 *         cdef inline const char* name(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).name             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = _PyUFuncObject_GET_ITEM_DATA(__pyx_v_self)->name;
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":971
 *             return _PyUFuncObject_GET_ITEM_DATA(self).ntypes
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline const char* name(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).name
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":975
 *             return _PyUFuncObject_GET_ITEM_DATA(self).name
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline const char* doc(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).doc
*/

static CYTHON_INLINE char const *__pyx_f_5numpy_5ufunc_3doc_doc(PyUFuncObject *__pyx_v_self) {
  char const *__pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":977
 *         @property
 *         cdef inline const char* doc(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).doc             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = _PyUFuncObject_GET_ITEM_DATA(__pyx_v_self)->doc;
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":975
 *             return _PyUFuncObject_GET_ITEM_DATA(self).name
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline const char* doc(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).doc
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":979
 *             return _PyUFuncObject_GET_ITEM_DATA(self).doc
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline void* ptr(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).ptr
*/

static CYTHON_INLINE void *__pyx_f_5numpy_5ufunc_3ptr_ptr(PyUFuncObject *__pyx_v_self) {
  void *__pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":981
 *         @property
 *         cdef inline void* ptr(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).ptr             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = _PyUFuncObject_GET_ITEM_DATA(__pyx_v_self)->ptr;
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":979
 *             return _PyUFuncObject_GET_ITEM_DATA(self).doc
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline void* ptr(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).ptr
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":983
 *             return _PyUFuncObject_GET_ITEM_DATA(self).ptr
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline PyObject* obj(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).obj
*/

static CYTHON_INLINE PyObject *__pyx_f_5numpy_5ufunc_3obj_obj(PyUFuncObject *__pyx_v_self) {
  PyObject *__pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":985
 *         @property
 *         cdef inline PyObject* obj(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).obj             # <<<<<<<<<<<<<<
 * 
 *         @property
*/
  __pyx_r = _PyUFuncObject_GET_ITEM_DATA(__pyx_v_self)->obj;
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":983
 *             return _PyUFuncObject_GET_ITEM_DATA(self).ptr
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline PyObject* obj(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).obj
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":987
 *             return _PyUFuncObject_GET_ITEM_DATA(self).obj
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline PyObject* userloops(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).userloops
*/

static CYTHON_INLINE PyObject *__pyx_f_5numpy_5ufunc_9userloops_userloops(PyUFuncObject *__pyx_v_self) {
  PyObject *__pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":989
 *         @property
 *         cdef inline PyObject* userloops(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).userloops             # <<<<<<<<<<<<<<
 * 
 *     PyUFuncObject_fields *_PyUFuncObject_GET_ITEM_DATA(ufunc) nogil
*/
  __pyx_r = _PyUFuncObject_GET_ITEM_DATA(__pyx_v_self)->userloops;
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":987
 *             return _PyUFuncObject_GET_ITEM_DATA(self).obj
 * 
 *         @property             # <<<<<<<<<<<<<<
 *         cdef inline PyObject* userloops(self) noexcept nogil:
 *             return _PyUFuncObject_GET_ITEM_DATA(self).userloops
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1065
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base) except *:             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
*/

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1066
 * 
 * cdef inline void set_array_base(ndarray arr, object base) except *:
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
*/
  Py_INCREF(__pyx_v_base);

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1067
 * cdef inline void set_array_base(ndarray arr, object base) except *:
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
*/
  __pyx_t_1 = PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(1, 1067, __pyx_L1_error)

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1065
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base) except *:             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("numpy.set_array_base", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1069
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
*/

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1070
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
*/
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1071
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
*/
  __pyx_t_1 = (__pyx_v_base == NULL);
  if (__pyx_t_1) {

    /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1072
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1071
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
*/
  }

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1073
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_base));
  __pyx_r = ((PyObject *)__pyx_v_base);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1069
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1077
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
*/

static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_array", 0);

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1078
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1079
 * cdef inline int import_array() except -1:
 *     try:
 *         __pyx_import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy._core.multiarray failed to import")
*/
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 1079, __pyx_L3_error)

      /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1078
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
*/
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1080
 *     try:
 *         __pyx_import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy._core.multiarray failed to import")
 * 
*/
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_Exception))));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(1, 1080, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);

      /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1081
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy._core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
*/
      __pyx_t_9 = NULL;
      __pyx_t_10 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_mstate_global->__pyx_kp_u_numpy__core_multiarray_failed_to};
        __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ImportError)), __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 1081, __pyx_L5_except_error)
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(1, 1081, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;

    /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1078
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
*/
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1077
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1083
 *         raise ImportError("numpy._core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
*/

static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_umath", 0);

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1084
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1085
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy._core.umath failed to import")
*/
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 1085, __pyx_L3_error)

      /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1084
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
*/
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1086
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy._core.umath failed to import")
 * 
*/
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_Exception))));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(1, 1086, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);

      /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1087
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy._core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
*/
      __pyx_t_9 = NULL;
      __pyx_t_10 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_mstate_global->__pyx_kp_u_numpy__core_umath_failed_to_impo};
        __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ImportError)), __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 1087, __pyx_L5_except_error)
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(1, 1087, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;

    /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1084
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
*/
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1083
 *         raise ImportError("numpy._core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1089
 *         raise ImportError("numpy._core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
*/

static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_ufunc", 0);

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1090
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1091
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy._core.umath failed to import")
*/
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 1091, __pyx_L3_error)

      /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1090
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
*/
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1092
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy._core.umath failed to import")
 * 
*/
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_Exception))));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(1, 1092, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);

      /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1093
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy._core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * 
*/
      __pyx_t_9 = NULL;
      __pyx_t_10 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_mstate_global->__pyx_kp_u_numpy__core_umath_failed_to_impo};
        __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ImportError)), __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 1093, __pyx_L5_except_error)
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(1, 1093, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;

    /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1090
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
*/
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1089
 *         raise ImportError("numpy._core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1096
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj) noexcept:             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
*/

static CYTHON_INLINE int __pyx_f_5numpy_is_timedelta64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1108
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyTimedeltaArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyTimedeltaArrType_Type));
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1096
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj) noexcept:             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1111
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj) noexcept:             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
*/

static CYTHON_INLINE int __pyx_f_5numpy_is_datetime64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1123
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyDatetimeArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyDatetimeArrType_Type));
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1111
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj) noexcept:             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1126
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) noexcept nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
*/

static CYTHON_INLINE npy_datetime __pyx_f_5numpy_get_datetime64_value(PyObject *__pyx_v_obj) {
  npy_datetime __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1133
 *     also needed.  That can be found using `get_datetime64_unit`.
 *     """
 *     return _PyDatetimeScalarObject_GetValue(obj)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = _PyDatetimeScalarObject_GetValue(__pyx_v_obj);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1126
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) noexcept nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1136
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) noexcept nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
*/

static CYTHON_INLINE npy_timedelta __pyx_f_5numpy_get_timedelta64_value(PyObject *__pyx_v_obj) {
  npy_timedelta __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1140
 *     returns the int64 value underlying scalar numpy timedelta64 object
 *     """
 *     return _PyTimedeltaScalarObject_GetValue(obj)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = _PyTimedeltaScalarObject_GetValue(__pyx_v_obj);
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1136
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) noexcept nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1143
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) noexcept nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
*/

static CYTHON_INLINE NPY_DATETIMEUNIT __pyx_f_5numpy_get_datetime64_unit(PyObject *__pyx_v_obj) {
  NPY_DATETIMEUNIT __pyx_r;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1147
 *     returns the unit part of the dtype for a numpy datetime64 object.
 *     """
 *     return _PyDatetimeScalarObject_GetMetadata(obj).base             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = _PyDatetimeScalarObject_GetMetadata(__pyx_v_obj).base;
  goto __pyx_L0;

  /* "../../../../usr/lib/python3.14/site-packages/numpy/__init__.cython-30.pxd":1143
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) noexcept nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":317
 * # Helper functions
 * 
 * cdef hsize_t *npy_malloc_dims(int rank, npy_intp *pdims):             # <<<<<<<<<<<<<<
 *   """Returns a malloced hsize_t dims from a npy_intp *pdims."""
 * 
*/

static hsize_t *__pyx_f_6tables_13hdf5extension_npy_malloc_dims(int __pyx_v_rank, npy_intp *__pyx_v_pdims) {
  int __pyx_v_i;
  hsize_t *__pyx_v_dims;
  hsize_t *__pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "tables/hdf5extension.pyx":323
 *   cdef hsize_t *dims
 * 
 *   dims = NULL             # <<<<<<<<<<<<<<
 *   if rank > 0:
 *     dims = <hsize_t *>malloc(rank * sizeof(hsize_t))
*/
  __pyx_v_dims = NULL;

  /* "tables/hdf5extension.pyx":324
 * 
 *   dims = NULL
 *   if rank > 0:             # <<<<<<<<<<<<<<
 *     dims = <hsize_t *>malloc(rank * sizeof(hsize_t))
 *     for i from 0 <= i < rank:
*/
  __pyx_t_1 = (__pyx_v_rank > 0);
  if (__pyx_t_1) {

    /* "tables/hdf5extension.pyx":325
 *   dims = NULL
 *   if rank > 0:
 *     dims = <hsize_t *>malloc(rank * sizeof(hsize_t))             # <<<<<<<<<<<<<<
 *     for i from 0 <= i < rank:
 *       dims[i] = pdims[i]
*/
    __pyx_v_dims = ((hsize_t *)malloc((__pyx_v_rank * (sizeof(hsize_t)))));

    /* "tables/hdf5extension.pyx":326
 *   if rank > 0:
 *     dims = <hsize_t *>malloc(rank * sizeof(hsize_t))
 *     for i from 0 <= i < rank:             # <<<<<<<<<<<<<<
 *       dims[i] = pdims[i]
 *   return dims
*/
    __pyx_t_2 = __pyx_v_rank;
    for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_2; __pyx_v_i++) {

      /* "tables/hdf5extension.pyx":327
 *     dims = <hsize_t *>malloc(rank * sizeof(hsize_t))
 *     for i from 0 <= i < rank:
 *       dims[i] = pdims[i]             # <<<<<<<<<<<<<<
 *   return dims
 * 
*/
      (__pyx_v_dims[__pyx_v_i]) = (__pyx_v_pdims[__pyx_v_i]);
    }

    /* "tables/hdf5extension.pyx":324
 * 
 *   dims = NULL
 *   if rank > 0:             # <<<<<<<<<<<<<<
 *     dims = <hsize_t *>malloc(rank * sizeof(hsize_t))
 *     for i from 0 <= i < rank:
*/
  }

  /* "tables/hdf5extension.pyx":328
 *     for i from 0 <= i < rank:
 *       dims[i] = pdims[i]
 *   return dims             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = __pyx_v_dims;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":317
 * # Helper functions
 * 
 * cdef hsize_t *npy_malloc_dims(int rank, npy_intp *pdims):             # <<<<<<<<<<<<<<
 *   """Returns a malloced hsize_t dims from a npy_intp *pdims."""
 * 
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":331
 * 
 * 
 * cdef object getshape(int rank, hsize_t *dims):             # <<<<<<<<<<<<<<
 *   """Return a shape (tuple) from a dims C array of rank dimensions."""
 * 
*/

static PyObject *__pyx_f_6tables_13hdf5extension_getshape(int __pyx_v_rank, hsize_t *__pyx_v_dims) {
  int __pyx_v_i;
  PyObject *__pyx_v_shape = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getshape", 0);

  /* "tables/hdf5extension.pyx":337
 *   cdef object shape
 * 
 *   shape = []             # <<<<<<<<<<<<<<
 *   for i from 0 <= i < rank:
 *     shape.append(SizeType(dims[i]))
*/
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_shape = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":338
 * 
 *   shape = []
 *   for i from 0 <= i < rank:             # <<<<<<<<<<<<<<
 *     shape.append(SizeType(dims[i]))
 * 
*/
  __pyx_t_2 = __pyx_v_rank;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_2; __pyx_v_i++) {

    /* "tables/hdf5extension.pyx":339
 *   shape = []
 *   for i from 0 <= i < rank:
 *     shape.append(SizeType(dims[i]))             # <<<<<<<<<<<<<<
 * 
 *   return tuple(shape)
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_SizeType); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 339, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyLong_From_hsize_t((__pyx_v_dims[__pyx_v_i])); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 339, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_5};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 339, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_t_7 = __Pyx_PyObject_Append(__pyx_v_shape, __pyx_t_1); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 339, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "tables/hdf5extension.pyx":341
 *     shape.append(SizeType(dims[i]))
 * 
 *   return tuple(shape)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PySequence_Tuple(__pyx_v_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 341, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":331
 * 
 * 
 * cdef object getshape(int rank, hsize_t *dims):             # <<<<<<<<<<<<<<
 *   """Return a shape (tuple) from a dims C array of rank dimensions."""
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("tables.hdf5extension.getshape", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_shape);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":345
 * 
 * # Helper function for quickly fetch an attribute string
 * cdef object get_attribute_string_or_none(hid_t node_id, char* attr_name):             # <<<<<<<<<<<<<<
 *   """Returns a string/unicode attribute if it exists in node_id.
 * 
*/

static PyObject *__pyx_f_6tables_13hdf5extension_get_attribute_string_or_none(hid_t __pyx_v_node_id, char *__pyx_v_attr_name) {
  char *__pyx_v_attr_value;
  int __pyx_v_cset;
  PyObject *__pyx_v_retvalue = 0;
  hsize_t __pyx_v_size;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_attribute_string_or_none", 0);

  /* "tables/hdf5extension.pyx":354
 * 
 *   cdef char *attr_value
 *   cdef int cset = H5T_CSET_DEFAULT             # <<<<<<<<<<<<<<
 *   cdef object retvalue
 *   cdef hsize_t size
*/
  __pyx_v_cset = __pyx_v_6tables_13hdf5extension_H5T_CSET_DEFAULT;

  /* "tables/hdf5extension.pyx":358
 *   cdef hsize_t size
 * 
 *   attr_value = NULL             # <<<<<<<<<<<<<<
 *   retvalue = None   # Default value
 *   if H5ATTRfind_attribute(node_id, attr_name):
*/
  __pyx_v_attr_value = NULL;

  /* "tables/hdf5extension.pyx":359
 * 
 *   attr_value = NULL
 *   retvalue = None   # Default value             # <<<<<<<<<<<<<<
 *   if H5ATTRfind_attribute(node_id, attr_name):
 *     size = H5ATTRget_attribute_string(node_id, attr_name, &attr_value, &cset)
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_retvalue = Py_None;

  /* "tables/hdf5extension.pyx":360
 *   attr_value = NULL
 *   retvalue = None   # Default value
 *   if H5ATTRfind_attribute(node_id, attr_name):             # <<<<<<<<<<<<<<
 *     size = H5ATTRget_attribute_string(node_id, attr_name, &attr_value, &cset)
 *     if size == 0:
*/
  __pyx_t_1 = (H5ATTRfind_attribute(__pyx_v_node_id, __pyx_v_attr_name) != 0);
  if (__pyx_t_1) {

    /* "tables/hdf5extension.pyx":361
 *   retvalue = None   # Default value
 *   if H5ATTRfind_attribute(node_id, attr_name):
 *     size = H5ATTRget_attribute_string(node_id, attr_name, &attr_value, &cset)             # <<<<<<<<<<<<<<
 *     if size == 0:
 *       if cset == H5T_CSET_UTF8:
*/
    __pyx_v_size = H5ATTRget_attribute_string(__pyx_v_node_id, __pyx_v_attr_name, (&__pyx_v_attr_value), (&__pyx_v_cset));

    /* "tables/hdf5extension.pyx":362
 *   if H5ATTRfind_attribute(node_id, attr_name):
 *     size = H5ATTRget_attribute_string(node_id, attr_name, &attr_value, &cset)
 *     if size == 0:             # <<<<<<<<<<<<<<
 *       if cset == H5T_CSET_UTF8:
 *         retvalue = np.str_('')
*/
    __pyx_t_1 = (__pyx_v_size == 0);
    if (__pyx_t_1) {

      /* "tables/hdf5extension.pyx":363
 *     size = H5ATTRget_attribute_string(node_id, attr_name, &attr_value, &cset)
 *     if size == 0:
 *       if cset == H5T_CSET_UTF8:             # <<<<<<<<<<<<<<
 *         retvalue = np.str_('')
 *       else:
*/
      __pyx_t_1 = (__pyx_v_cset == H5T_CSET_UTF8);
      if (__pyx_t_1) {

        /* "tables/hdf5extension.pyx":364
 *     if size == 0:
 *       if cset == H5T_CSET_UTF8:
 *         retvalue = np.str_('')             # <<<<<<<<<<<<<<
 *       else:
 *         retvalue = np.bytes_(b'')
*/
        __pyx_t_3 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 364, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_str); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 364, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 364, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_DECREF_SET(__pyx_v_retvalue, __pyx_t_2);
        __pyx_t_2 = 0;

        /* "tables/hdf5extension.pyx":363
 *     size = H5ATTRget_attribute_string(node_id, attr_name, &attr_value, &cset)
 *     if size == 0:
 *       if cset == H5T_CSET_UTF8:             # <<<<<<<<<<<<<<
 *         retvalue = np.str_('')
 *       else:
*/
        goto __pyx_L5;
      }

      /* "tables/hdf5extension.pyx":366
 *         retvalue = np.str_('')
 *       else:
 *         retvalue = np.bytes_(b'')             # <<<<<<<<<<<<<<
 *     elif cset == H5T_CSET_UTF8:
 *       retvalue = PyUnicode_DecodeUTF8(attr_value, size, NULL)
*/
      /*else*/ {
        __pyx_t_5 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 366, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_bytes); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 366, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_5);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_b_};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 366, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_DECREF_SET(__pyx_v_retvalue, __pyx_t_2);
        __pyx_t_2 = 0;
      }
      __pyx_L5:;

      /* "tables/hdf5extension.pyx":362
 *   if H5ATTRfind_attribute(node_id, attr_name):
 *     size = H5ATTRget_attribute_string(node_id, attr_name, &attr_value, &cset)
 *     if size == 0:             # <<<<<<<<<<<<<<
 *       if cset == H5T_CSET_UTF8:
 *         retvalue = np.str_('')
*/
      goto __pyx_L4;
    }

    /* "tables/hdf5extension.pyx":367
 *       else:
 *         retvalue = np.bytes_(b'')
 *     elif cset == H5T_CSET_UTF8:             # <<<<<<<<<<<<<<
 *       retvalue = PyUnicode_DecodeUTF8(attr_value, size, NULL)
 *       retvalue = np.str_(retvalue)
*/
    __pyx_t_1 = (__pyx_v_cset == H5T_CSET_UTF8);
    if (__pyx_t_1) {

      /* "tables/hdf5extension.pyx":368
 *         retvalue = np.bytes_(b'')
 *     elif cset == H5T_CSET_UTF8:
 *       retvalue = PyUnicode_DecodeUTF8(attr_value, size, NULL)             # <<<<<<<<<<<<<<
 *       retvalue = np.str_(retvalue)
 *     else:
*/
      __pyx_t_2 = PyUnicode_DecodeUTF8(__pyx_v_attr_value, __pyx_v_size, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 368, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF_SET(__pyx_v_retvalue, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "tables/hdf5extension.pyx":369
 *     elif cset == H5T_CSET_UTF8:
 *       retvalue = PyUnicode_DecodeUTF8(attr_value, size, NULL)
 *       retvalue = np.str_(retvalue)             # <<<<<<<<<<<<<<
 *     else:
 *       retvalue = PyBytes_FromStringAndSize(attr_value, size)
*/
      __pyx_t_4 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 369, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_str); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 369, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        assert(__pyx_t_4);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_retvalue};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 369, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_DECREF_SET(__pyx_v_retvalue, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "tables/hdf5extension.pyx":367
 *       else:
 *         retvalue = np.bytes_(b'')
 *     elif cset == H5T_CSET_UTF8:             # <<<<<<<<<<<<<<
 *       retvalue = PyUnicode_DecodeUTF8(attr_value, size, NULL)
 *       retvalue = np.str_(retvalue)
*/
      goto __pyx_L4;
    }

    /* "tables/hdf5extension.pyx":371
 *       retvalue = np.str_(retvalue)
 *     else:
 *       retvalue = PyBytes_FromStringAndSize(attr_value, size)             # <<<<<<<<<<<<<<
 *       # AV: oct 2012
 *       # since now we use the string size got form HDF5 we have to strip
*/
    /*else*/ {
      __pyx_t_2 = PyBytes_FromStringAndSize(__pyx_v_attr_value, __pyx_v_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 371, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF_SET(__pyx_v_retvalue, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "tables/hdf5extension.pyx":379
 *       # strlen(attr_value) is the actual length of the attribute
 *       # and np.bytes_(attr_value) can give a truncated pickle string
 *       retvalue = retvalue.rstrip(b'\x00')             # <<<<<<<<<<<<<<
 *       retvalue = np.bytes_(retvalue)
 * 
*/
      __pyx_t_3 = __pyx_v_retvalue;
      __Pyx_INCREF(__pyx_t_3);
      __pyx_t_6 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_b__2};
        __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_rstrip, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 379, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_DECREF_SET(__pyx_v_retvalue, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "tables/hdf5extension.pyx":380
 *       # and np.bytes_(attr_value) can give a truncated pickle string
 *       retvalue = retvalue.rstrip(b'\x00')
 *       retvalue = np.bytes_(retvalue)             # <<<<<<<<<<<<<<
 * 
 *     # Important to release attr_value, because it has been malloc'ed!
*/
      __pyx_t_3 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 380, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_bytes); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 380, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
        assert(__pyx_t_3);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_retvalue};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 380, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_DECREF_SET(__pyx_v_retvalue, __pyx_t_2);
      __pyx_t_2 = 0;
    }
    __pyx_L4:;

    /* "tables/hdf5extension.pyx":383
 * 
 *     # Important to release attr_value, because it has been malloc'ed!
 *     if attr_value:             # <<<<<<<<<<<<<<
 *       free(<void *>attr_value)
 * 
*/
    __pyx_t_1 = (__pyx_v_attr_value != 0);
    if (__pyx_t_1) {

      /* "tables/hdf5extension.pyx":384
 *     # Important to release attr_value, because it has been malloc'ed!
 *     if attr_value:
 *       free(<void *>attr_value)             # <<<<<<<<<<<<<<
 * 
 *   return retvalue
*/
      free(((void *)__pyx_v_attr_value));

      /* "tables/hdf5extension.pyx":383
 * 
 *     # Important to release attr_value, because it has been malloc'ed!
 *     if attr_value:             # <<<<<<<<<<<<<<
 *       free(<void *>attr_value)
 * 
*/
    }

    /* "tables/hdf5extension.pyx":360
 *   attr_value = NULL
 *   retvalue = None   # Default value
 *   if H5ATTRfind_attribute(node_id, attr_name):             # <<<<<<<<<<<<<<
 *     size = H5ATTRget_attribute_string(node_id, attr_name, &attr_value, &cset)
 *     if size == 0:
*/
  }

  /* "tables/hdf5extension.pyx":386
 *       free(<void *>attr_value)
 * 
 *   return retvalue             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_retvalue);
  __pyx_r = __pyx_v_retvalue;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":345
 * 
 * # Helper function for quickly fetch an attribute string
 * cdef object get_attribute_string_or_none(hid_t node_id, char* attr_name):             # <<<<<<<<<<<<<<
 *   """Returns a string/unicode attribute if it exists in node_id.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("tables.hdf5extension.get_attribute_string_or_none", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_retvalue);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":390
 * 
 * # Get the numpy dtype scalar attribute from an HDF5 type as fast as possible
 * cdef object get_dtype_scalar(hid_t type_id, H5T_class_t class_id,             # <<<<<<<<<<<<<<
 *                              size_t itemsize):
 *   cdef H5T_sign_t sign
*/

static PyObject *__pyx_f_6tables_13hdf5extension_get_dtype_scalar(hid_t __pyx_v_type_id, enum H5T_class_t __pyx_v_class_id, size_t __pyx_v_itemsize) {
  enum H5T_sign_t __pyx_v_sign;
  PyObject *__pyx_v_stype = 0;
  PyObject *__pyx_v_ntype = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_dtype_scalar", 0);

  /* "tables/hdf5extension.pyx":395
 *   cdef object stype
 * 
 *   if class_id == H5T_BITFIELD:             # <<<<<<<<<<<<<<
 *     stype = "b1"
 *   elif class_id == H5T_INTEGER:
*/
  switch (__pyx_v_class_id) {
    case H5T_BITFIELD:

    /* "tables/hdf5extension.pyx":396
 * 
 *   if class_id == H5T_BITFIELD:
 *     stype = "b1"             # <<<<<<<<<<<<<<
 *   elif class_id == H5T_INTEGER:
 *     # Get the sign
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_b1);
    __pyx_v_stype = __pyx_mstate_global->__pyx_n_u_b1;

    /* "tables/hdf5extension.pyx":395
 *   cdef object stype
 * 
 *   if class_id == H5T_BITFIELD:             # <<<<<<<<<<<<<<
 *     stype = "b1"
 *   elif class_id == H5T_INTEGER:
*/
    break;
    case H5T_INTEGER:

    /* "tables/hdf5extension.pyx":399
 *   elif class_id == H5T_INTEGER:
 *     # Get the sign
 *     sign = H5Tget_sign(type_id)             # <<<<<<<<<<<<<<
 *     if (sign > 0):
 *       stype = "i%s" % (itemsize)
*/
    __pyx_v_sign = H5Tget_sign(__pyx_v_type_id);

    /* "tables/hdf5extension.pyx":400
 *     # Get the sign
 *     sign = H5Tget_sign(type_id)
 *     if (sign > 0):             # <<<<<<<<<<<<<<
 *       stype = "i%s" % (itemsize)
 *     else:
*/
    __pyx_t_1 = (__pyx_v_sign > 0);
    if (__pyx_t_1) {

      /* "tables/hdf5extension.pyx":401
 *     sign = H5Tget_sign(type_id)
 *     if (sign > 0):
 *       stype = "i%s" % (itemsize)             # <<<<<<<<<<<<<<
 *     else:
 *       stype = "u%s" % (itemsize)
*/
      __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_v_itemsize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 401, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_i_s, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 401, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_v_stype = __pyx_t_3;
      __pyx_t_3 = 0;

      /* "tables/hdf5extension.pyx":400
 *     # Get the sign
 *     sign = H5Tget_sign(type_id)
 *     if (sign > 0):             # <<<<<<<<<<<<<<
 *       stype = "i%s" % (itemsize)
 *     else:
*/
      goto __pyx_L3;
    }

    /* "tables/hdf5extension.pyx":403
 *       stype = "i%s" % (itemsize)
 *     else:
 *       stype = "u%s" % (itemsize)             # <<<<<<<<<<<<<<
 *   elif class_id ==  H5T_FLOAT:
 *     stype = "f%s" % (itemsize)
*/
    /*else*/ {
      __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_itemsize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 403, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_2 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_u_s, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 403, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_v_stype = __pyx_t_2;
      __pyx_t_2 = 0;
    }
    __pyx_L3:;

    /* "tables/hdf5extension.pyx":397
 *   if class_id == H5T_BITFIELD:
 *     stype = "b1"
 *   elif class_id == H5T_INTEGER:             # <<<<<<<<<<<<<<
 *     # Get the sign
 *     sign = H5Tget_sign(type_id)
*/
    break;
    case H5T_FLOAT:

    /* "tables/hdf5extension.pyx":405
 *       stype = "u%s" % (itemsize)
 *   elif class_id ==  H5T_FLOAT:
 *     stype = "f%s" % (itemsize)             # <<<<<<<<<<<<<<
 *   elif class_id ==  H5T_STRING:
 *     if H5Tis_variable_str(type_id):
*/
    __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_v_itemsize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 405, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_f_s, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 405, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_stype = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "tables/hdf5extension.pyx":404
 *     else:
 *       stype = "u%s" % (itemsize)
 *   elif class_id ==  H5T_FLOAT:             # <<<<<<<<<<<<<<
 *     stype = "f%s" % (itemsize)
 *   elif class_id ==  H5T_STRING:
*/
    break;
    case H5T_STRING:

    /* "tables/hdf5extension.pyx":407
 *     stype = "f%s" % (itemsize)
 *   elif class_id ==  H5T_STRING:
 *     if H5Tis_variable_str(type_id):             # <<<<<<<<<<<<<<
 *       raise TypeError("variable length strings are not supported yet")
 *     stype = "S%s" % (itemsize)
*/
    __pyx_t_1 = (H5Tis_variable_str(__pyx_v_type_id) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "tables/hdf5extension.pyx":408
 *   elif class_id ==  H5T_STRING:
 *     if H5Tis_variable_str(type_id):
 *       raise TypeError("variable length strings are not supported yet")             # <<<<<<<<<<<<<<
 *     stype = "S%s" % (itemsize)
 * 
*/
      __pyx_t_2 = NULL;
      __pyx_t_4 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_variable_length_strings_are_not};
        __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 408, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 408, __pyx_L1_error)

      /* "tables/hdf5extension.pyx":407
 *     stype = "f%s" % (itemsize)
 *   elif class_id ==  H5T_STRING:
 *     if H5Tis_variable_str(type_id):             # <<<<<<<<<<<<<<
 *       raise TypeError("variable length strings are not supported yet")
 *     stype = "S%s" % (itemsize)
*/
    }

    /* "tables/hdf5extension.pyx":409
 *     if H5Tis_variable_str(type_id):
 *       raise TypeError("variable length strings are not supported yet")
 *     stype = "S%s" % (itemsize)             # <<<<<<<<<<<<<<
 * 
 *   # Try to get a NumPy type.  If this can't be done, return None.
*/
    __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_itemsize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 409, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_S_s, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 409, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_stype = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "tables/hdf5extension.pyx":406
 *   elif class_id ==  H5T_FLOAT:
 *     stype = "f%s" % (itemsize)
 *   elif class_id ==  H5T_STRING:             # <<<<<<<<<<<<<<
 *     if H5Tis_variable_str(type_id):
 *       raise TypeError("variable length strings are not supported yet")
*/
    break;
    default: break;
  }

  /* "tables/hdf5extension.pyx":412
 * 
 *   # Try to get a NumPy type.  If this can't be done, return None.
 *   try:             # <<<<<<<<<<<<<<
 *     ntype = np.dtype(stype)
 *   except TypeError:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_7);
    /*try:*/ {

      /* "tables/hdf5extension.pyx":413
 *   # Try to get a NumPy type.  If this can't be done, return None.
 *   try:
 *     ntype = np.dtype(stype)             # <<<<<<<<<<<<<<
 *   except TypeError:
 *     ntype = None
*/
      __pyx_t_3 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 413, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 413, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_v_stype)) { __Pyx_RaiseUnboundLocalError("stype"); __PYX_ERR(0, 413, __pyx_L5_error) }
      __pyx_t_4 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_9))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_9);
        assert(__pyx_t_3);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
        __pyx_t_4 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_stype};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 413, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __pyx_v_ntype = __pyx_t_2;
      __pyx_t_2 = 0;

      /* "tables/hdf5extension.pyx":412
 * 
 *   # Try to get a NumPy type.  If this can't be done, return None.
 *   try:             # <<<<<<<<<<<<<<
 *     ntype = np.dtype(stype)
 *   except TypeError:
*/
    }
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    goto __pyx_L10_try_end;
    __pyx_L5_error:;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "tables/hdf5extension.pyx":414
 *   try:
 *     ntype = np.dtype(stype)
 *   except TypeError:             # <<<<<<<<<<<<<<
 *     ntype = None
 *   return ntype
*/
    __pyx_t_10 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_TypeError))));
    if (__pyx_t_10) {
      __Pyx_ErrRestore(0,0,0);

      /* "tables/hdf5extension.pyx":415
 *     ntype = np.dtype(stype)
 *   except TypeError:
 *     ntype = None             # <<<<<<<<<<<<<<
 *   return ntype
 * 
*/
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ntype, Py_None);
      goto __pyx_L6_exception_handled;
    }
    goto __pyx_L7_except_error;

    /* "tables/hdf5extension.pyx":412
 * 
 *   # Try to get a NumPy type.  If this can't be done, return None.
 *   try:             # <<<<<<<<<<<<<<
 *     ntype = np.dtype(stype)
 *   except TypeError:
*/
    __pyx_L7_except_error:;
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
    goto __pyx_L1_error;
    __pyx_L6_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
    __pyx_L10_try_end:;
  }

  /* "tables/hdf5extension.pyx":416
 *   except TypeError:
 *     ntype = None
 *   return ntype             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_ntype);
  __pyx_r = __pyx_v_ntype;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":390
 * 
 * # Get the numpy dtype scalar attribute from an HDF5 type as fast as possible
 * cdef object get_dtype_scalar(hid_t type_id, H5T_class_t class_id,             # <<<<<<<<<<<<<<
 *                              size_t itemsize):
 *   cdef H5T_sign_t sign
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("tables.hdf5extension.get_dtype_scalar", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_stype);
  __Pyx_XDECREF(__pyx_v_ntype);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":445
 *   cdef object  name
 * 
 *   def _g_new(self, name, pymode, **params):             # <<<<<<<<<<<<<<
 *     cdef herr_t err = 0
 *     cdef hid_t access_plist, create_plist = H5P_DEFAULT
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_1_g_new(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4File_1_g_new = {"_g_new", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_1_g_new, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_1_g_new(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_pymode = 0;
  PyObject *__pyx_v_params = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_new (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  __pyx_v_params = PyDict_New(); if (unlikely(!__pyx_v_params)) return NULL;
  __Pyx_GOTREF(__pyx_v_params);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_name,&__pyx_mstate_global->__pyx_n_u_pymode,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 445, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 445, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 445, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, __pyx_v_params, values, kwd_pos_args, __pyx_kwds_len, "_g_new", 1) < (0)) __PYX_ERR(0, 445, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_new", 1, 2, 2, i); __PYX_ERR(0, 445, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 445, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 445, __pyx_L3_error)
    }
    __pyx_v_name = values[0];
    __pyx_v_pymode = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_new", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 445, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_DECREF(__pyx_v_params); __pyx_v_params = 0;
  __Pyx_AddTraceback("tables.hdf5extension.File._g_new", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4File__g_new(((struct __pyx_obj_6tables_13hdf5extension_File *)__pyx_v_self), __pyx_v_name, __pyx_v_pymode, __pyx_v_params);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_DECREF(__pyx_v_params);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4File__g_new(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_pymode, PyObject *__pyx_v_params) {
  herr_t __pyx_v_err;
  hid_t __pyx_v_access_plist;
  hid_t __pyx_v_create_plist;
  hid_t __pyx_v_meta_plist_id;
  hid_t __pyx_v_raw_plist_id;
  size_t __pyx_v_img_buf_len;
  size_t __pyx_v_user_block_size;
  void *__pyx_v_img_buf_p;
  PyObject *__pyx_v_encname = 0;
  PyObject *__pyx_v_driver = NULL;
  PyObject *__pyx_v_meta_ext = NULL;
  PyObject *__pyx_v_raw_ext = NULL;
  PyObject *__pyx_v_meta_name = NULL;
  PyObject *__pyx_v_raw_name = NULL;
  PyObject *__pyx_v_enc_meta_ext = NULL;
  PyObject *__pyx_v_enc_raw_ext = NULL;
  PyObject *__pyx_v_image = NULL;
  PyObject *__pyx_v_n = NULL;
  PyObject *__pyx_v_backing_store = NULL;
  PyObject *__pyx_v_exists = NULL;
  PyObject *__pyx_v_is_pow_of_2 = NULL;
  PyObject *__pyx_v_e = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  double __pyx_t_12;
  size_t __pyx_t_13;
  size_t __pyx_t_14;
  hbool_t __pyx_t_15;
  char *__pyx_t_16;
  char *__pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_new", 0);

  /* "tables/hdf5extension.pyx":446
 * 
 *   def _g_new(self, name, pymode, **params):
 *     cdef herr_t err = 0             # <<<<<<<<<<<<<<
 *     cdef hid_t access_plist, create_plist = H5P_DEFAULT
 *     cdef hid_t meta_plist_id = H5P_DEFAULT, raw_plist_id = H5P_DEFAULT
*/
  __pyx_v_err = 0;

  /* "tables/hdf5extension.pyx":447
 *   def _g_new(self, name, pymode, **params):
 *     cdef herr_t err = 0
 *     cdef hid_t access_plist, create_plist = H5P_DEFAULT             # <<<<<<<<<<<<<<
 *     cdef hid_t meta_plist_id = H5P_DEFAULT, raw_plist_id = H5P_DEFAULT
 *     cdef size_t img_buf_len = 0, user_block_size = 0
*/
  __pyx_v_create_plist = H5P_DEFAULT;

  /* "tables/hdf5extension.pyx":448
 *     cdef herr_t err = 0
 *     cdef hid_t access_plist, create_plist = H5P_DEFAULT
 *     cdef hid_t meta_plist_id = H5P_DEFAULT, raw_plist_id = H5P_DEFAULT             # <<<<<<<<<<<<<<
 *     cdef size_t img_buf_len = 0, user_block_size = 0
 *     cdef void *img_buf_p = NULL
*/
  __pyx_v_meta_plist_id = H5P_DEFAULT;
  __pyx_v_raw_plist_id = H5P_DEFAULT;

  /* "tables/hdf5extension.pyx":449
 *     cdef hid_t access_plist, create_plist = H5P_DEFAULT
 *     cdef hid_t meta_plist_id = H5P_DEFAULT, raw_plist_id = H5P_DEFAULT
 *     cdef size_t img_buf_len = 0, user_block_size = 0             # <<<<<<<<<<<<<<
 *     cdef void *img_buf_p = NULL
 *     cdef bytes encname
*/
  __pyx_v_img_buf_len = 0;
  __pyx_v_user_block_size = 0;

  /* "tables/hdf5extension.pyx":450
 *     cdef hid_t meta_plist_id = H5P_DEFAULT, raw_plist_id = H5P_DEFAULT
 *     cdef size_t img_buf_len = 0, user_block_size = 0
 *     cdef void *img_buf_p = NULL             # <<<<<<<<<<<<<<
 *     cdef bytes encname
 *     #cdef bytes logfile_name
*/
  __pyx_v_img_buf_p = NULL;

  /* "tables/hdf5extension.pyx":455
 * 
 *     # Check if we can handle the driver
 *     driver = params["DRIVER"]             # <<<<<<<<<<<<<<
 *     if driver is not None and driver not in _supported_drivers:
 *       raise ValueError("Invalid or not supported driver: '%s'" % driver)
*/
  __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_params, __pyx_mstate_global->__pyx_n_u_DRIVER); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_driver = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":456
 *     # Check if we can handle the driver
 *     driver = params["DRIVER"]
 *     if driver is not None and driver not in _supported_drivers:             # <<<<<<<<<<<<<<
 *       raise ValueError("Invalid or not supported driver: '%s'" % driver)
 *     if driver == "H5FD_SPLIT":
*/
  __pyx_t_3 = (__pyx_v_driver != Py_None);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_supported_drivers); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 456, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = (__Pyx_PySequence_ContainsTF(__pyx_v_driver, __pyx_t_1, Py_NE)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 456, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_2)) {

    /* "tables/hdf5extension.pyx":457
 *     driver = params["DRIVER"]
 *     if driver is not None and driver not in _supported_drivers:
 *       raise ValueError("Invalid or not supported driver: '%s'" % driver)             # <<<<<<<<<<<<<<
 *     if driver == "H5FD_SPLIT":
 *       meta_ext = params.get("DRIVER_SPLIT_META_EXT", "-m.h5")
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Invalid_or_not_supported_driver, __pyx_v_driver); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 457, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_5};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 457, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 457, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":456
 *     # Check if we can handle the driver
 *     driver = params["DRIVER"]
 *     if driver is not None and driver not in _supported_drivers:             # <<<<<<<<<<<<<<
 *       raise ValueError("Invalid or not supported driver: '%s'" % driver)
 *     if driver == "H5FD_SPLIT":
*/
  }

  /* "tables/hdf5extension.pyx":458
 *     if driver is not None and driver not in _supported_drivers:
 *       raise ValueError("Invalid or not supported driver: '%s'" % driver)
 *     if driver == "H5FD_SPLIT":             # <<<<<<<<<<<<<<
 *       meta_ext = params.get("DRIVER_SPLIT_META_EXT", "-m.h5")
 *       raw_ext = params.get("DRIVER_SPLIT_RAW_EXT", "-r.h5")
*/
  __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_n_u_H5FD_SPLIT, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 458, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "tables/hdf5extension.pyx":459
 *       raise ValueError("Invalid or not supported driver: '%s'" % driver)
 *     if driver == "H5FD_SPLIT":
 *       meta_ext = params.get("DRIVER_SPLIT_META_EXT", "-m.h5")             # <<<<<<<<<<<<<<
 *       raw_ext = params.get("DRIVER_SPLIT_RAW_EXT", "-r.h5")
 *       meta_name = meta_ext % name if "%s" in meta_ext else name + meta_ext
*/
    __pyx_t_1 = __Pyx_PyDict_GetItemDefault(__pyx_v_params, __pyx_mstate_global->__pyx_n_u_DRIVER_SPLIT_META_EXT, __pyx_mstate_global->__pyx_kp_u_m_h5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 459, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_meta_ext = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":460
 *     if driver == "H5FD_SPLIT":
 *       meta_ext = params.get("DRIVER_SPLIT_META_EXT", "-m.h5")
 *       raw_ext = params.get("DRIVER_SPLIT_RAW_EXT", "-r.h5")             # <<<<<<<<<<<<<<
 *       meta_name = meta_ext % name if "%s" in meta_ext else name + meta_ext
 *       raw_name = raw_ext % name if "%s" in raw_ext else name + raw_ext
*/
    __pyx_t_1 = __Pyx_PyDict_GetItemDefault(__pyx_v_params, __pyx_mstate_global->__pyx_n_u_DRIVER_SPLIT_RAW_EXT, __pyx_mstate_global->__pyx_kp_u_r_h5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 460, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_raw_ext = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":461
 *       meta_ext = params.get("DRIVER_SPLIT_META_EXT", "-m.h5")
 *       raw_ext = params.get("DRIVER_SPLIT_RAW_EXT", "-r.h5")
 *       meta_name = meta_ext % name if "%s" in meta_ext else name + meta_ext             # <<<<<<<<<<<<<<
 *       raw_name = raw_ext % name if "%s" in raw_ext else name + raw_ext
 *       enc_meta_ext = encode_filename(meta_ext)
*/
    __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_mstate_global->__pyx_kp_u_s, __pyx_v_meta_ext, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 461, __pyx_L1_error)
    if (__pyx_t_2) {
      __pyx_t_5 = PyNumber_Remainder(__pyx_v_meta_ext, __pyx_v_name); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 461, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_1 = __pyx_t_5;
      __pyx_t_5 = 0;
    } else {
      __pyx_t_5 = PyNumber_Add(__pyx_v_name, __pyx_v_meta_ext); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 461, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_1 = __pyx_t_5;
      __pyx_t_5 = 0;
    }
    __pyx_v_meta_name = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":462
 *       raw_ext = params.get("DRIVER_SPLIT_RAW_EXT", "-r.h5")
 *       meta_name = meta_ext % name if "%s" in meta_ext else name + meta_ext
 *       raw_name = raw_ext % name if "%s" in raw_ext else name + raw_ext             # <<<<<<<<<<<<<<
 *       enc_meta_ext = encode_filename(meta_ext)
 *       enc_raw_ext = encode_filename(raw_ext)
*/
    __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_mstate_global->__pyx_kp_u_s, __pyx_v_raw_ext, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 462, __pyx_L1_error)
    if (__pyx_t_2) {
      __pyx_t_5 = PyNumber_Remainder(__pyx_v_raw_ext, __pyx_v_name); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 462, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_1 = __pyx_t_5;
      __pyx_t_5 = 0;
    } else {
      __pyx_t_5 = PyNumber_Add(__pyx_v_name, __pyx_v_raw_ext); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 462, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_1 = __pyx_t_5;
      __pyx_t_5 = 0;
    }
    __pyx_v_raw_name = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":463
 *       meta_name = meta_ext % name if "%s" in meta_ext else name + meta_ext
 *       raw_name = raw_ext % name if "%s" in raw_ext else name + raw_ext
 *       enc_meta_ext = encode_filename(meta_ext)             # <<<<<<<<<<<<<<
 *       enc_raw_ext = encode_filename(raw_ext)
 * 
*/
    __pyx_t_5 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_encode_filename); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 463, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_5);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_v_meta_ext};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 463, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_v_enc_meta_ext = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":464
 *       raw_name = raw_ext % name if "%s" in raw_ext else name + raw_ext
 *       enc_meta_ext = encode_filename(meta_ext)
 *       enc_raw_ext = encode_filename(raw_ext)             # <<<<<<<<<<<<<<
 * 
 *     # Create a new file using default properties
*/
    __pyx_t_4 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_encode_filename); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 464, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_raw_ext};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 464, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_v_enc_raw_ext = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":458
 *     if driver is not None and driver not in _supported_drivers:
 *       raise ValueError("Invalid or not supported driver: '%s'" % driver)
 *     if driver == "H5FD_SPLIT":             # <<<<<<<<<<<<<<
 *       meta_ext = params.get("DRIVER_SPLIT_META_EXT", "-m.h5")
 *       raw_ext = params.get("DRIVER_SPLIT_RAW_EXT", "-r.h5")
*/
  }

  /* "tables/hdf5extension.pyx":467
 * 
 *     # Create a new file using default properties
 *     self.name = name             # <<<<<<<<<<<<<<
 * 
 *     # Encode the filename in case it is unicode
*/
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = __pyx_v_name;

  /* "tables/hdf5extension.pyx":470
 * 
 *     # Encode the filename in case it is unicode
 *     encname = encode_filename(name)             # <<<<<<<<<<<<<<
 * 
 *     # These fields can be seen from Python.
*/
  __pyx_t_5 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_encode_filename); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_5);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_6 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_v_name};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 470, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 470, __pyx_L1_error)
  __pyx_v_encname = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":473
 * 
 *     # These fields can be seen from Python.
 *     self._v_new = None  # this will be computed later             # <<<<<<<<<<<<<<
 *     # """Is this file going to be created from scratch?"""
 * 
*/
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_new, Py_None) < (0)) __PYX_ERR(0, 473, __pyx_L1_error)

  /* "tables/hdf5extension.pyx":476
 *     # """Is this file going to be created from scratch?"""
 * 
 *     self._isPTFile = True  # assume a PyTables file by default             # <<<<<<<<<<<<<<
 *     # """Does this HDF5 file have a PyTables format?"""
 * 
*/
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_isPTFile, Py_True) < (0)) __PYX_ERR(0, 476, __pyx_L1_error)

  /* "tables/hdf5extension.pyx":479
 *     # """Does this HDF5 file have a PyTables format?"""
 * 
 *     assert pymode in ('r', 'r+', 'a', 'w'), ("an invalid mode string ``%s`` "             # <<<<<<<<<<<<<<
 *            "passed the ``check_file_access()`` test; "
 *            "please report this to the authors" % pymode)
*/
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(__pyx_assertions_enabled())) {
    __Pyx_INCREF(__pyx_v_pymode);
    __pyx_t_1 = __pyx_v_pymode;
    __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_r, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 479, __pyx_L1_error)
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_r_2, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 479, __pyx_L1_error)
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_a, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 479, __pyx_L1_error)
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_w, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 479, __pyx_L1_error)
    __pyx_t_2 = __pyx_t_3;
    __pyx_L7_bool_binop_done:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_3 = __pyx_t_2;
    if (unlikely(!__pyx_t_3)) {

      /* "tables/hdf5extension.pyx":481
 *     assert pymode in ('r', 'r+', 'a', 'w'), ("an invalid mode string ``%s`` "
 *            "passed the ``check_file_access()`` test; "
 *            "please report this to the authors" % pymode)             # <<<<<<<<<<<<<<
 * 
 *     image = params.get('DRIVER_CORE_IMAGE')
*/
      __pyx_t_1 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_an_invalid_mode_string_s_passed, __pyx_v_pymode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 481, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), __pyx_t_1, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 479, __pyx_L1_error)
    }
  }
  #else
  if ((1)); else __PYX_ERR(0, 479, __pyx_L1_error)
  #endif

  /* "tables/hdf5extension.pyx":483
 *            "please report this to the authors" % pymode)
 * 
 *     image = params.get('DRIVER_CORE_IMAGE')             # <<<<<<<<<<<<<<
 *     if image:
 *       if driver != "H5FD_CORE":
*/
  __pyx_t_1 = __Pyx_PyDict_GetItemDefault(__pyx_v_params, __pyx_mstate_global->__pyx_n_u_DRIVER_CORE_IMAGE, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_image = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":484
 * 
 *     image = params.get('DRIVER_CORE_IMAGE')
 *     if image:             # <<<<<<<<<<<<<<
 *       if driver != "H5FD_CORE":
 *         warnings.warn("The DRIVER_CORE_IMAGE parameter will be ignored by "
*/
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_image); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 484, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":485
 *     image = params.get('DRIVER_CORE_IMAGE')
 *     if image:
 *       if driver != "H5FD_CORE":             # <<<<<<<<<<<<<<
 *         warnings.warn("The DRIVER_CORE_IMAGE parameter will be ignored by "
 *                       "the '%s' driver" % driver)
*/
    __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_n_u_H5FD_CORE, Py_NE)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 485, __pyx_L1_error)
    if (__pyx_t_3) {

      /* "tables/hdf5extension.pyx":486
 *     if image:
 *       if driver != "H5FD_CORE":
 *         warnings.warn("The DRIVER_CORE_IMAGE parameter will be ignored by "             # <<<<<<<<<<<<<<
 *                       "the '%s' driver" % driver)
 *       elif not PyBytes_Check(image):
*/
      __pyx_t_4 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_warnings); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 486, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_warn); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 486, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "tables/hdf5extension.pyx":487
 *       if driver != "H5FD_CORE":
 *         warnings.warn("The DRIVER_CORE_IMAGE parameter will be ignored by "
 *                       "the '%s' driver" % driver)             # <<<<<<<<<<<<<<
 *       elif not PyBytes_Check(image):
 *         raise TypeError("The DRIVER_CORE_IMAGE must be a string of bytes")
*/
      __pyx_t_5 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_The_DRIVER_CORE_IMAGE_parameter, __pyx_v_driver); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 487, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_7);
        assert(__pyx_t_4);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_5};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 486, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "tables/hdf5extension.pyx":485
 *     image = params.get('DRIVER_CORE_IMAGE')
 *     if image:
 *       if driver != "H5FD_CORE":             # <<<<<<<<<<<<<<
 *         warnings.warn("The DRIVER_CORE_IMAGE parameter will be ignored by "
 *                       "the '%s' driver" % driver)
*/
      goto __pyx_L12;
    }

    /* "tables/hdf5extension.pyx":488
 *         warnings.warn("The DRIVER_CORE_IMAGE parameter will be ignored by "
 *                       "the '%s' driver" % driver)
 *       elif not PyBytes_Check(image):             # <<<<<<<<<<<<<<
 *         raise TypeError("The DRIVER_CORE_IMAGE must be a string of bytes")
 * 
*/
    __pyx_t_3 = (!PyBytes_Check(__pyx_v_image));
    if (unlikely(__pyx_t_3)) {

      /* "tables/hdf5extension.pyx":489
 *                       "the '%s' driver" % driver)
 *       elif not PyBytes_Check(image):
 *         raise TypeError("The DRIVER_CORE_IMAGE must be a string of bytes")             # <<<<<<<<<<<<<<
 * 
 *     # After the following check we can be quite sure
*/
      __pyx_t_7 = NULL;
      __pyx_t_6 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_The_DRIVER_CORE_IMAGE_must_be_a};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 489, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 489, __pyx_L1_error)

      /* "tables/hdf5extension.pyx":488
 *         warnings.warn("The DRIVER_CORE_IMAGE parameter will be ignored by "
 *                       "the '%s' driver" % driver)
 *       elif not PyBytes_Check(image):             # <<<<<<<<<<<<<<
 *         raise TypeError("The DRIVER_CORE_IMAGE must be a string of bytes")
 * 
*/
    }
    __pyx_L12:;

    /* "tables/hdf5extension.pyx":484
 * 
 *     image = params.get('DRIVER_CORE_IMAGE')
 *     if image:             # <<<<<<<<<<<<<<
 *       if driver != "H5FD_CORE":
 *         warnings.warn("The DRIVER_CORE_IMAGE parameter will be ignored by "
*/
  }

  /* "tables/hdf5extension.pyx":493
 *     # After the following check we can be quite sure
 *     # that the file or directory exists and permissions are right.
 *     if driver == "H5FD_SPLIT":             # <<<<<<<<<<<<<<
 *       for n in meta_name, raw_name:
 *         check_file_access(n, pymode)
*/
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_n_u_H5FD_SPLIT, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 493, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":494
 *     # that the file or directory exists and permissions are right.
 *     if driver == "H5FD_SPLIT":
 *       for n in meta_name, raw_name:             # <<<<<<<<<<<<<<
 *         check_file_access(n, pymode)
 *     else:
*/
    if (unlikely(!__pyx_v_meta_name)) { __Pyx_RaiseUnboundLocalError("meta_name"); __PYX_ERR(0, 494, __pyx_L1_error) }
    if (unlikely(!__pyx_v_raw_name)) { __Pyx_RaiseUnboundLocalError("raw_name"); __PYX_ERR(0, 494, __pyx_L1_error) }
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 494, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_meta_name);
    __Pyx_GIVEREF(__pyx_v_meta_name);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_meta_name) != (0)) __PYX_ERR(0, 494, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_raw_name);
    __Pyx_GIVEREF(__pyx_v_raw_name);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_raw_name) != (0)) __PYX_ERR(0, 494, __pyx_L1_error);
    __pyx_t_7 = __pyx_t_1; __Pyx_INCREF(__pyx_t_7);
    __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (__pyx_t_8 >= 2) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_1 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_8));
      #else
      __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_7, __pyx_t_8);
      #endif
      ++__pyx_t_8;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 494, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_n, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "tables/hdf5extension.pyx":495
 *     if driver == "H5FD_SPLIT":
 *       for n in meta_name, raw_name:
 *         check_file_access(n, pymode)             # <<<<<<<<<<<<<<
 *     else:
 *       backing_store = params.get("DRIVER_CORE_BACKING_STORE", 1)
*/
      __pyx_t_5 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_check_file_access); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 495, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
        assert(__pyx_t_5);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_v_n, __pyx_v_pymode};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 495, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "tables/hdf5extension.pyx":494
 *     # that the file or directory exists and permissions are right.
 *     if driver == "H5FD_SPLIT":
 *       for n in meta_name, raw_name:             # <<<<<<<<<<<<<<
 *         check_file_access(n, pymode)
 *     else:
*/
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "tables/hdf5extension.pyx":493
 *     # After the following check we can be quite sure
 *     # that the file or directory exists and permissions are right.
 *     if driver == "H5FD_SPLIT":             # <<<<<<<<<<<<<<
 *       for n in meta_name, raw_name:
 *         check_file_access(n, pymode)
*/
    goto __pyx_L13;
  }

  /* "tables/hdf5extension.pyx":497
 *         check_file_access(n, pymode)
 *     else:
 *       backing_store = params.get("DRIVER_CORE_BACKING_STORE", 1)             # <<<<<<<<<<<<<<
 *       if driver != "H5FD_CORE" or backing_store:
 *         check_file_access(name, pymode)
*/
  /*else*/ {
    __pyx_t_7 = __Pyx_PyDict_GetItemDefault(__pyx_v_params, __pyx_mstate_global->__pyx_n_u_DRIVER_CORE_BACKING_STORE, __pyx_mstate_global->__pyx_int_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 497, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_v_backing_store = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "tables/hdf5extension.pyx":498
 *     else:
 *       backing_store = params.get("DRIVER_CORE_BACKING_STORE", 1)
 *       if driver != "H5FD_CORE" or backing_store:             # <<<<<<<<<<<<<<
 *         check_file_access(name, pymode)
 * 
*/
    __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_n_u_H5FD_CORE, Py_NE)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 498, __pyx_L1_error)
    if (!__pyx_t_2) {
    } else {
      __pyx_t_3 = __pyx_t_2;
      goto __pyx_L18_bool_binop_done;
    }
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_backing_store); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 498, __pyx_L1_error)
    __pyx_t_3 = __pyx_t_2;
    __pyx_L18_bool_binop_done:;
    if (__pyx_t_3) {

      /* "tables/hdf5extension.pyx":499
 *       backing_store = params.get("DRIVER_CORE_BACKING_STORE", 1)
 *       if driver != "H5FD_CORE" or backing_store:
 *         check_file_access(name, pymode)             # <<<<<<<<<<<<<<
 * 
 *     # Should a new file be created?
*/
      __pyx_t_1 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_check_file_access); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 499, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
        assert(__pyx_t_1);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_1, __pyx_v_name, __pyx_v_pymode};
        __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 499, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "tables/hdf5extension.pyx":498
 *     else:
 *       backing_store = params.get("DRIVER_CORE_BACKING_STORE", 1)
 *       if driver != "H5FD_CORE" or backing_store:             # <<<<<<<<<<<<<<
 *         check_file_access(name, pymode)
 * 
*/
    }
  }
  __pyx_L13:;

  /* "tables/hdf5extension.pyx":502
 * 
 *     # Should a new file be created?
 *     if image:             # <<<<<<<<<<<<<<
 *       exists = True
 *     elif driver == "H5FD_SPLIT":
*/
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_image); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 502, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":503
 *     # Should a new file be created?
 *     if image:
 *       exists = True             # <<<<<<<<<<<<<<
 *     elif driver == "H5FD_SPLIT":
 *       exists = os.path.exists(meta_name) and os.path.exists(raw_name)
*/
    __Pyx_INCREF(Py_True);
    __pyx_v_exists = Py_True;

    /* "tables/hdf5extension.pyx":502
 * 
 *     # Should a new file be created?
 *     if image:             # <<<<<<<<<<<<<<
 *       exists = True
 *     elif driver == "H5FD_SPLIT":
*/
    goto __pyx_L20;
  }

  /* "tables/hdf5extension.pyx":504
 *     if image:
 *       exists = True
 *     elif driver == "H5FD_SPLIT":             # <<<<<<<<<<<<<<
 *       exists = os.path.exists(meta_name) and os.path.exists(raw_name)
 *     else:
*/
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_n_u_H5FD_SPLIT, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 504, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":505
 *       exists = True
 *     elif driver == "H5FD_SPLIT":
 *       exists = os.path.exists(meta_name) and os.path.exists(raw_name)             # <<<<<<<<<<<<<<
 *     else:
 *       exists = os.path.exists(name)
*/
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_os); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 505, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_path); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 505, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_1 = __pyx_t_9;
    __Pyx_INCREF(__pyx_t_1);
    if (unlikely(!__pyx_v_meta_name)) { __Pyx_RaiseUnboundLocalError("meta_name"); __PYX_ERR(0, 505, __pyx_L1_error) }
    __pyx_t_6 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_v_meta_name};
      __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_exists, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 505, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 505, __pyx_L1_error)
    if (__pyx_t_3) {
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_7 = __pyx_t_4;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      goto __pyx_L21_bool_binop_done;
    }
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_os); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 505, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_path); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 505, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_9 = __pyx_t_5;
    __Pyx_INCREF(__pyx_t_9);
    if (unlikely(!__pyx_v_raw_name)) { __Pyx_RaiseUnboundLocalError("raw_name"); __PYX_ERR(0, 505, __pyx_L1_error) }
    __pyx_t_6 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_v_raw_name};
      __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_exists, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 505, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_7 = __pyx_t_4;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_L21_bool_binop_done:;
    __pyx_v_exists = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "tables/hdf5extension.pyx":504
 *     if image:
 *       exists = True
 *     elif driver == "H5FD_SPLIT":             # <<<<<<<<<<<<<<
 *       exists = os.path.exists(meta_name) and os.path.exists(raw_name)
 *     else:
*/
    goto __pyx_L20;
  }

  /* "tables/hdf5extension.pyx":507
 *       exists = os.path.exists(meta_name) and os.path.exists(raw_name)
 *     else:
 *       exists = os.path.exists(name)             # <<<<<<<<<<<<<<
 *     self._v_new = not (pymode in ('r', 'r+') or (pymode == 'a' and exists))
 * 
*/
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_os); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 507, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_path); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 507, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_4 = __pyx_t_9;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_6 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_name};
      __pyx_t_7 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_exists, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 507, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }
    __pyx_v_exists = __pyx_t_7;
    __pyx_t_7 = 0;
  }
  __pyx_L20:;

  /* "tables/hdf5extension.pyx":508
 *     else:
 *       exists = os.path.exists(name)
 *     self._v_new = not (pymode in ('r', 'r+') or (pymode == 'a' and exists))             # <<<<<<<<<<<<<<
 * 
 *     user_block_size = params.get("USER_BLOCK_SIZE", 0)
*/
  __Pyx_INCREF(__pyx_v_pymode);
  __pyx_t_7 = __pyx_v_pymode;
  __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_r, Py_EQ)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 508, __pyx_L1_error)
  if (!__pyx_t_10) {
  } else {
    __pyx_t_2 = __pyx_t_10;
    goto __pyx_L25_bool_binop_done;
  }
  __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_r_2, Py_EQ)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 508, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_10;
  __pyx_L25_bool_binop_done:;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_10 = __pyx_t_2;
  if (!__pyx_t_10) {
  } else {
    __pyx_t_3 = __pyx_t_10;
    goto __pyx_L23_bool_binop_done;
  }
  __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_v_pymode, __pyx_mstate_global->__pyx_n_u_a, Py_EQ)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 508, __pyx_L1_error)
  if (__pyx_t_10) {
  } else {
    __pyx_t_3 = __pyx_t_10;
    goto __pyx_L23_bool_binop_done;
  }
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_v_exists); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 508, __pyx_L1_error)
  __pyx_t_3 = __pyx_t_10;
  __pyx_L23_bool_binop_done:;
  __pyx_t_7 = __Pyx_PyBool_FromLong((!__pyx_t_3)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 508, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_new, __pyx_t_7) < (0)) __PYX_ERR(0, 508, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "tables/hdf5extension.pyx":510
 *     self._v_new = not (pymode in ('r', 'r+') or (pymode == 'a' and exists))
 * 
 *     user_block_size = params.get("USER_BLOCK_SIZE", 0)             # <<<<<<<<<<<<<<
 *     if user_block_size and not self._v_new:
 *         warnings.warn("The HDF5 file already esists: the USER_BLOCK_SIZE "
*/
  __pyx_t_7 = __Pyx_PyDict_GetItemDefault(__pyx_v_params, __pyx_mstate_global->__pyx_n_u_USER_BLOCK_SIZE, __pyx_mstate_global->__pyx_int_0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_6 = __Pyx_PyLong_As_size_t(__pyx_t_7); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 510, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_user_block_size = __pyx_t_6;

  /* "tables/hdf5extension.pyx":511
 * 
 *     user_block_size = params.get("USER_BLOCK_SIZE", 0)
 *     if user_block_size and not self._v_new:             # <<<<<<<<<<<<<<
 *         warnings.warn("The HDF5 file already esists: the USER_BLOCK_SIZE "
 *                       "will be ignored")
*/
  __pyx_t_10 = (__pyx_v_user_block_size != 0);
  if (__pyx_t_10) {
  } else {
    __pyx_t_3 = __pyx_t_10;
    goto __pyx_L29_bool_binop_done;
  }
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_new); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 511, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 511, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_2 = (!__pyx_t_10);
  __pyx_t_3 = __pyx_t_2;
  __pyx_L29_bool_binop_done:;
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":512
 *     user_block_size = params.get("USER_BLOCK_SIZE", 0)
 *     if user_block_size and not self._v_new:
 *         warnings.warn("The HDF5 file already esists: the USER_BLOCK_SIZE "             # <<<<<<<<<<<<<<
 *                       "will be ignored")
 *     elif user_block_size:
*/
    __pyx_t_9 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_warnings); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 512, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_warn); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 512, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_9);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_mstate_global->__pyx_kp_u_The_HDF5_file_already_esists_the};
      __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 512, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "tables/hdf5extension.pyx":511
 * 
 *     user_block_size = params.get("USER_BLOCK_SIZE", 0)
 *     if user_block_size and not self._v_new:             # <<<<<<<<<<<<<<
 *         warnings.warn("The HDF5 file already esists: the USER_BLOCK_SIZE "
 *                       "will be ignored")
*/
    goto __pyx_L28;
  }

  /* "tables/hdf5extension.pyx":514
 *         warnings.warn("The HDF5 file already esists: the USER_BLOCK_SIZE "
 *                       "will be ignored")
 *     elif user_block_size:             # <<<<<<<<<<<<<<
 *       user_block_size = int(user_block_size)
 *       is_pow_of_2 = ((user_block_size & (user_block_size - 1)) == 0)
*/
  __pyx_t_3 = (__pyx_v_user_block_size != 0);
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":516
 *     elif user_block_size:
 *       user_block_size = int(user_block_size)
 *       is_pow_of_2 = ((user_block_size & (user_block_size - 1)) == 0)             # <<<<<<<<<<<<<<
 *       if user_block_size < 512 or not is_pow_of_2:
 *         raise ValueError("The USER_BLOCK_SIZE must be a power od 2 greather "
*/
    __pyx_t_7 = __Pyx_PyBool_FromLong(((__pyx_v_user_block_size & (__pyx_v_user_block_size - 1)) == 0)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 516, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_v_is_pow_of_2 = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "tables/hdf5extension.pyx":517
 *       user_block_size = int(user_block_size)
 *       is_pow_of_2 = ((user_block_size & (user_block_size - 1)) == 0)
 *       if user_block_size < 512 or not is_pow_of_2:             # <<<<<<<<<<<<<<
 *         raise ValueError("The USER_BLOCK_SIZE must be a power od 2 greather "
 *                          "than 512 or zero")
*/
    __pyx_t_2 = (__pyx_v_user_block_size < 0x200);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_3 = __pyx_t_2;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_is_pow_of_2); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 517, __pyx_L1_error)
    __pyx_t_10 = (!__pyx_t_2);
    __pyx_t_3 = __pyx_t_10;
    __pyx_L32_bool_binop_done:;
    if (unlikely(__pyx_t_3)) {

      /* "tables/hdf5extension.pyx":518
 *       is_pow_of_2 = ((user_block_size & (user_block_size - 1)) == 0)
 *       if user_block_size < 512 or not is_pow_of_2:
 *         raise ValueError("The USER_BLOCK_SIZE must be a power od 2 greather "             # <<<<<<<<<<<<<<
 *                          "than 512 or zero")
 * 
*/
      __pyx_t_5 = NULL;
      __pyx_t_6 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_The_USER_BLOCK_SIZE_must_be_a_po};
        __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 518, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
      }
      __Pyx_Raise(__pyx_t_7, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __PYX_ERR(0, 518, __pyx_L1_error)

      /* "tables/hdf5extension.pyx":517
 *       user_block_size = int(user_block_size)
 *       is_pow_of_2 = ((user_block_size & (user_block_size - 1)) == 0)
 *       if user_block_size < 512 or not is_pow_of_2:             # <<<<<<<<<<<<<<
 *         raise ValueError("The USER_BLOCK_SIZE must be a power od 2 greather "
 *                          "than 512 or zero")
*/
    }

    /* "tables/hdf5extension.pyx":522
 * 
 *       # File creation property list
 *       create_plist = H5Pcreate(H5P_FILE_CREATE)             # <<<<<<<<<<<<<<
 *       err = H5Pset_userblock(create_plist, user_block_size)
 *       if err < 0:
*/
    __pyx_v_create_plist = H5Pcreate(H5P_FILE_CREATE);

    /* "tables/hdf5extension.pyx":523
 *       # File creation property list
 *       create_plist = H5Pcreate(H5P_FILE_CREATE)
 *       err = H5Pset_userblock(create_plist, user_block_size)             # <<<<<<<<<<<<<<
 *       if err < 0:
 *         H5Pclose(create_plist)
*/
    __pyx_v_err = H5Pset_userblock(__pyx_v_create_plist, __pyx_v_user_block_size);

    /* "tables/hdf5extension.pyx":524
 *       create_plist = H5Pcreate(H5P_FILE_CREATE)
 *       err = H5Pset_userblock(create_plist, user_block_size)
 *       if err < 0:             # <<<<<<<<<<<<<<
 *         H5Pclose(create_plist)
 *         raise HDF5ExtError("Unable to set the user block size")
*/
    __pyx_t_3 = (__pyx_v_err < 0);
    if (unlikely(__pyx_t_3)) {

      /* "tables/hdf5extension.pyx":525
 *       err = H5Pset_userblock(create_plist, user_block_size)
 *       if err < 0:
 *         H5Pclose(create_plist)             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Unable to set the user block size")
 * 
*/
      (void)(H5Pclose(__pyx_v_create_plist));

      /* "tables/hdf5extension.pyx":526
 *       if err < 0:
 *         H5Pclose(create_plist)
 *         raise HDF5ExtError("Unable to set the user block size")             # <<<<<<<<<<<<<<
 * 
 *     # File access property list
*/
      __pyx_t_5 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 526, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_9))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_9);
        assert(__pyx_t_5);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_Unable_to_set_the_user_block_siz};
        __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 526, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
      }
      __Pyx_Raise(__pyx_t_7, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __PYX_ERR(0, 526, __pyx_L1_error)

      /* "tables/hdf5extension.pyx":524
 *       create_plist = H5Pcreate(H5P_FILE_CREATE)
 *       err = H5Pset_userblock(create_plist, user_block_size)
 *       if err < 0:             # <<<<<<<<<<<<<<
 *         H5Pclose(create_plist)
 *         raise HDF5ExtError("Unable to set the user block size")
*/
    }

    /* "tables/hdf5extension.pyx":514
 *         warnings.warn("The HDF5 file already esists: the USER_BLOCK_SIZE "
 *                       "will be ignored")
 *     elif user_block_size:             # <<<<<<<<<<<<<<
 *       user_block_size = int(user_block_size)
 *       is_pow_of_2 = ((user_block_size & (user_block_size - 1)) == 0)
*/
  }
  __pyx_L28:;

  /* "tables/hdf5extension.pyx":529
 * 
 *     # File access property list
 *     access_plist = H5Pcreate(H5P_FILE_ACCESS)             # <<<<<<<<<<<<<<
 * 
 *     # Set parameters for chunk cache
*/
  __pyx_v_access_plist = H5Pcreate(H5P_FILE_ACCESS);

  /* "tables/hdf5extension.pyx":533
 *     # Set parameters for chunk cache
 *     H5Pset_cache(access_plist, 0,
 *                  params["CHUNK_CACHE_NELMTS"],             # <<<<<<<<<<<<<<
 *                  params["CHUNK_CACHE_SIZE"],
 *                  params["CHUNK_CACHE_PREEMPT"])
*/
  __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_params, __pyx_mstate_global->__pyx_n_u_CHUNK_CACHE_NELMTS); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_11 = __Pyx_PyLong_As_int(__pyx_t_7); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "tables/hdf5extension.pyx":534
 *     H5Pset_cache(access_plist, 0,
 *                  params["CHUNK_CACHE_NELMTS"],
 *                  params["CHUNK_CACHE_SIZE"],             # <<<<<<<<<<<<<<
 *                  params["CHUNK_CACHE_PREEMPT"])
 * 
*/
  __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_params, __pyx_mstate_global->__pyx_n_u_CHUNK_CACHE_SIZE); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 534, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_6 = __Pyx_PyLong_As_size_t(__pyx_t_7); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 534, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "tables/hdf5extension.pyx":535
 *                  params["CHUNK_CACHE_NELMTS"],
 *                  params["CHUNK_CACHE_SIZE"],
 *                  params["CHUNK_CACHE_PREEMPT"])             # <<<<<<<<<<<<<<
 * 
 *     # Set the I/O driver
*/
  __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_params, __pyx_mstate_global->__pyx_n_u_CHUNK_CACHE_PREEMPT); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_12 = __Pyx_PyFloat_AsDouble(__pyx_t_7); if (unlikely((__pyx_t_12 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "tables/hdf5extension.pyx":532
 * 
 *     # Set parameters for chunk cache
 *     H5Pset_cache(access_plist, 0,             # <<<<<<<<<<<<<<
 *                  params["CHUNK_CACHE_NELMTS"],
 *                  params["CHUNK_CACHE_SIZE"],
*/
  (void)(H5Pset_cache(__pyx_v_access_plist, 0, __pyx_t_11, __pyx_t_6, __pyx_t_12));

  /* "tables/hdf5extension.pyx":538
 * 
 *     # Set the I/O driver
 *     if driver == "H5FD_SEC2":             # <<<<<<<<<<<<<<
 *       err = H5Pset_fapl_sec2(access_plist)
 *     elif driver == "H5FD_DIRECT":
*/
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_n_u_H5FD_SEC2, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 538, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":539
 *     # Set the I/O driver
 *     if driver == "H5FD_SEC2":
 *       err = H5Pset_fapl_sec2(access_plist)             # <<<<<<<<<<<<<<
 *     elif driver == "H5FD_DIRECT":
 *       if not H5_HAVE_DIRECT_DRIVER:
*/
    __pyx_v_err = H5Pset_fapl_sec2(__pyx_v_access_plist);

    /* "tables/hdf5extension.pyx":538
 * 
 *     # Set the I/O driver
 *     if driver == "H5FD_SEC2":             # <<<<<<<<<<<<<<
 *       err = H5Pset_fapl_sec2(access_plist)
 *     elif driver == "H5FD_DIRECT":
*/
    goto __pyx_L35;
  }

  /* "tables/hdf5extension.pyx":540
 *     if driver == "H5FD_SEC2":
 *       err = H5Pset_fapl_sec2(access_plist)
 *     elif driver == "H5FD_DIRECT":             # <<<<<<<<<<<<<<
 *       if not H5_HAVE_DIRECT_DRIVER:
 *         H5Pclose(create_plist)
*/
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_n_u_H5FD_DIRECT, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 540, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":541
 *       err = H5Pset_fapl_sec2(access_plist)
 *     elif driver == "H5FD_DIRECT":
 *       if not H5_HAVE_DIRECT_DRIVER:             # <<<<<<<<<<<<<<
 *         H5Pclose(create_plist)
 *         H5Pclose(access_plist)
*/
    __pyx_t_3 = (!(H5_HAVE_DIRECT_DRIVER != 0));
    if (unlikely(__pyx_t_3)) {

      /* "tables/hdf5extension.pyx":542
 *     elif driver == "H5FD_DIRECT":
 *       if not H5_HAVE_DIRECT_DRIVER:
 *         H5Pclose(create_plist)             # <<<<<<<<<<<<<<
 *         H5Pclose(access_plist)
 *         raise RuntimeError("The H5FD_DIRECT driver is not available")
*/
      (void)(H5Pclose(__pyx_v_create_plist));

      /* "tables/hdf5extension.pyx":543
 *       if not H5_HAVE_DIRECT_DRIVER:
 *         H5Pclose(create_plist)
 *         H5Pclose(access_plist)             # <<<<<<<<<<<<<<
 *         raise RuntimeError("The H5FD_DIRECT driver is not available")
 *       err = pt_H5Pset_fapl_direct(access_plist,
*/
      (void)(H5Pclose(__pyx_v_access_plist));

      /* "tables/hdf5extension.pyx":544
 *         H5Pclose(create_plist)
 *         H5Pclose(access_plist)
 *         raise RuntimeError("The H5FD_DIRECT driver is not available")             # <<<<<<<<<<<<<<
 *       err = pt_H5Pset_fapl_direct(access_plist,
 *                                   params["DRIVER_DIRECT_ALIGNMENT"],
*/
      __pyx_t_9 = NULL;
      __pyx_t_6 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_mstate_global->__pyx_kp_u_The_H5FD_DIRECT_driver_is_not_av};
        __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 544, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
      }
      __Pyx_Raise(__pyx_t_7, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __PYX_ERR(0, 544, __pyx_L1_error)

      /* "tables/hdf5extension.pyx":541
 *       err = H5Pset_fapl_sec2(access_plist)
 *     elif driver == "H5FD_DIRECT":
 *       if not H5_HAVE_DIRECT_DRIVER:             # <<<<<<<<<<<<<<
 *         H5Pclose(create_plist)
 *         H5Pclose(access_plist)
*/
    }

    /* "tables/hdf5extension.pyx":546
 *         raise RuntimeError("The H5FD_DIRECT driver is not available")
 *       err = pt_H5Pset_fapl_direct(access_plist,
 *                                   params["DRIVER_DIRECT_ALIGNMENT"],             # <<<<<<<<<<<<<<
 *                                   params["DRIVER_DIRECT_BLOCK_SIZE"],
 *                                   params["DRIVER_DIRECT_CBUF_SIZE"])
*/
    __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_params, __pyx_mstate_global->__pyx_n_u_DRIVER_DIRECT_ALIGNMENT); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 546, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = __Pyx_PyLong_As_size_t(__pyx_t_7); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 546, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "tables/hdf5extension.pyx":547
 *       err = pt_H5Pset_fapl_direct(access_plist,
 *                                   params["DRIVER_DIRECT_ALIGNMENT"],
 *                                   params["DRIVER_DIRECT_BLOCK_SIZE"],             # <<<<<<<<<<<<<<
 *                                   params["DRIVER_DIRECT_CBUF_SIZE"])
 *     #elif driver == "H5FD_LOG":
*/
    __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_params, __pyx_mstate_global->__pyx_n_u_DRIVER_DIRECT_BLOCK_SIZE); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 547, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_13 = __Pyx_PyLong_As_size_t(__pyx_t_7); if (unlikely((__pyx_t_13 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 547, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "tables/hdf5extension.pyx":548
 *                                   params["DRIVER_DIRECT_ALIGNMENT"],
 *                                   params["DRIVER_DIRECT_BLOCK_SIZE"],
 *                                   params["DRIVER_DIRECT_CBUF_SIZE"])             # <<<<<<<<<<<<<<
 *     #elif driver == "H5FD_LOG":
 *     #  if "DRIVER_LOG_FILE" not in params:
*/
    __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_params, __pyx_mstate_global->__pyx_n_u_DRIVER_DIRECT_CBUF_SIZE); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 548, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_14 = __Pyx_PyLong_As_size_t(__pyx_t_7); if (unlikely((__pyx_t_14 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 548, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "tables/hdf5extension.pyx":545
 *         H5Pclose(access_plist)
 *         raise RuntimeError("The H5FD_DIRECT driver is not available")
 *       err = pt_H5Pset_fapl_direct(access_plist,             # <<<<<<<<<<<<<<
 *                                   params["DRIVER_DIRECT_ALIGNMENT"],
 *                                   params["DRIVER_DIRECT_BLOCK_SIZE"],
*/
    __pyx_v_err = pt_H5Pset_fapl_direct(__pyx_v_access_plist, __pyx_t_6, __pyx_t_13, __pyx_t_14);

    /* "tables/hdf5extension.pyx":540
 *     if driver == "H5FD_SEC2":
 *       err = H5Pset_fapl_sec2(access_plist)
 *     elif driver == "H5FD_DIRECT":             # <<<<<<<<<<<<<<
 *       if not H5_HAVE_DIRECT_DRIVER:
 *         H5Pclose(create_plist)
*/
    goto __pyx_L35;
  }

  /* "tables/hdf5extension.pyx":559
 *     #                        params["DRIVER_LOG_FLAGS"],
 *     #                        params["DRIVER_LOG_BUF_SIZE"])
 *     elif driver == "H5FD_WINDOWS":             # <<<<<<<<<<<<<<
 *       if not H5_HAVE_WINDOWS_DRIVER:
 *         H5Pclose(access_plist)
*/
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_n_u_H5FD_WINDOWS, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 559, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":560
 *     #                        params["DRIVER_LOG_BUF_SIZE"])
 *     elif driver == "H5FD_WINDOWS":
 *       if not H5_HAVE_WINDOWS_DRIVER:             # <<<<<<<<<<<<<<
 *         H5Pclose(access_plist)
 *         H5Pclose(create_plist)
*/
    __pyx_t_3 = (!(H5_HAVE_WINDOWS_DRIVER != 0));
    if (unlikely(__pyx_t_3)) {

      /* "tables/hdf5extension.pyx":561
 *     elif driver == "H5FD_WINDOWS":
 *       if not H5_HAVE_WINDOWS_DRIVER:
 *         H5Pclose(access_plist)             # <<<<<<<<<<<<<<
 *         H5Pclose(create_plist)
 *         raise RuntimeError("The H5FD_WINDOWS driver is not available")
*/
      (void)(H5Pclose(__pyx_v_access_plist));

      /* "tables/hdf5extension.pyx":562
 *       if not H5_HAVE_WINDOWS_DRIVER:
 *         H5Pclose(access_plist)
 *         H5Pclose(create_plist)             # <<<<<<<<<<<<<<
 *         raise RuntimeError("The H5FD_WINDOWS driver is not available")
 *       err = pt_H5Pset_fapl_windows(access_plist)
*/
      (void)(H5Pclose(__pyx_v_create_plist));

      /* "tables/hdf5extension.pyx":563
 *         H5Pclose(access_plist)
 *         H5Pclose(create_plist)
 *         raise RuntimeError("The H5FD_WINDOWS driver is not available")             # <<<<<<<<<<<<<<
 *       err = pt_H5Pset_fapl_windows(access_plist)
 *     elif driver == "H5FD_STDIO":
*/
      __pyx_t_9 = NULL;
      __pyx_t_14 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_mstate_global->__pyx_kp_u_The_H5FD_WINDOWS_driver_is_not_a};
        __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_14, (2-__pyx_t_14) | (__pyx_t_14*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 563, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
      }
      __Pyx_Raise(__pyx_t_7, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __PYX_ERR(0, 563, __pyx_L1_error)

      /* "tables/hdf5extension.pyx":560
 *     #                        params["DRIVER_LOG_BUF_SIZE"])
 *     elif driver == "H5FD_WINDOWS":
 *       if not H5_HAVE_WINDOWS_DRIVER:             # <<<<<<<<<<<<<<
 *         H5Pclose(access_plist)
 *         H5Pclose(create_plist)
*/
    }

    /* "tables/hdf5extension.pyx":564
 *         H5Pclose(create_plist)
 *         raise RuntimeError("The H5FD_WINDOWS driver is not available")
 *       err = pt_H5Pset_fapl_windows(access_plist)             # <<<<<<<<<<<<<<
 *     elif driver == "H5FD_STDIO":
 *       err = H5Pset_fapl_stdio(access_plist)
*/
    __pyx_v_err = pt_H5Pset_fapl_windows(__pyx_v_access_plist);

    /* "tables/hdf5extension.pyx":559
 *     #                        params["DRIVER_LOG_FLAGS"],
 *     #                        params["DRIVER_LOG_BUF_SIZE"])
 *     elif driver == "H5FD_WINDOWS":             # <<<<<<<<<<<<<<
 *       if not H5_HAVE_WINDOWS_DRIVER:
 *         H5Pclose(access_plist)
*/
    goto __pyx_L35;
  }

  /* "tables/hdf5extension.pyx":565
 *         raise RuntimeError("The H5FD_WINDOWS driver is not available")
 *       err = pt_H5Pset_fapl_windows(access_plist)
 *     elif driver == "H5FD_STDIO":             # <<<<<<<<<<<<<<
 *       err = H5Pset_fapl_stdio(access_plist)
 *     elif driver == "H5FD_CORE":
*/
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_n_u_H5FD_STDIO, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 565, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":566
 *       err = pt_H5Pset_fapl_windows(access_plist)
 *     elif driver == "H5FD_STDIO":
 *       err = H5Pset_fapl_stdio(access_plist)             # <<<<<<<<<<<<<<
 *     elif driver == "H5FD_CORE":
 *       err = H5Pset_fapl_core(access_plist,
*/
    __pyx_v_err = H5Pset_fapl_stdio(__pyx_v_access_plist);

    /* "tables/hdf5extension.pyx":565
 *         raise RuntimeError("The H5FD_WINDOWS driver is not available")
 *       err = pt_H5Pset_fapl_windows(access_plist)
 *     elif driver == "H5FD_STDIO":             # <<<<<<<<<<<<<<
 *       err = H5Pset_fapl_stdio(access_plist)
 *     elif driver == "H5FD_CORE":
*/
    goto __pyx_L35;
  }

  /* "tables/hdf5extension.pyx":567
 *     elif driver == "H5FD_STDIO":
 *       err = H5Pset_fapl_stdio(access_plist)
 *     elif driver == "H5FD_CORE":             # <<<<<<<<<<<<<<
 *       err = H5Pset_fapl_core(access_plist,
 *                              params["DRIVER_CORE_INCREMENT"],
*/
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_n_u_H5FD_CORE, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 567, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":569
 *     elif driver == "H5FD_CORE":
 *       err = H5Pset_fapl_core(access_plist,
 *                              params["DRIVER_CORE_INCREMENT"],             # <<<<<<<<<<<<<<
 *                              backing_store)
 *       if image:
*/
    __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_params, __pyx_mstate_global->__pyx_n_u_DRIVER_CORE_INCREMENT); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 569, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_14 = __Pyx_PyLong_As_size_t(__pyx_t_7); if (unlikely((__pyx_t_14 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 569, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "tables/hdf5extension.pyx":570
 *       err = H5Pset_fapl_core(access_plist,
 *                              params["DRIVER_CORE_INCREMENT"],
 *                              backing_store)             # <<<<<<<<<<<<<<
 *       if image:
 *         img_buf_len = len(image)
*/
    if (unlikely(!__pyx_v_backing_store)) { __Pyx_RaiseUnboundLocalError("backing_store"); __PYX_ERR(0, 570, __pyx_L1_error) }
    __pyx_t_15 = __Pyx_PyLong_As_hbool_t(__pyx_v_backing_store); if (unlikely((__pyx_t_15 == ((hbool_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 570, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":568
 *       err = H5Pset_fapl_stdio(access_plist)
 *     elif driver == "H5FD_CORE":
 *       err = H5Pset_fapl_core(access_plist,             # <<<<<<<<<<<<<<
 *                              params["DRIVER_CORE_INCREMENT"],
 *                              backing_store)
*/
    __pyx_v_err = H5Pset_fapl_core(__pyx_v_access_plist, __pyx_t_14, __pyx_t_15);

    /* "tables/hdf5extension.pyx":571
 *                              params["DRIVER_CORE_INCREMENT"],
 *                              backing_store)
 *       if image:             # <<<<<<<<<<<<<<
 *         img_buf_len = len(image)
 *         img_buf_p = <void *>PyBytes_AsString(image)
*/
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_image); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 571, __pyx_L1_error)
    if (__pyx_t_3) {

      /* "tables/hdf5extension.pyx":572
 *                              backing_store)
 *       if image:
 *         img_buf_len = len(image)             # <<<<<<<<<<<<<<
 *         img_buf_p = <void *>PyBytes_AsString(image)
 *         err = H5Pset_file_image(access_plist, img_buf_p, img_buf_len)
*/
      __pyx_t_8 = PyObject_Length(__pyx_v_image); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 572, __pyx_L1_error)
      __pyx_v_img_buf_len = __pyx_t_8;

      /* "tables/hdf5extension.pyx":573
 *       if image:
 *         img_buf_len = len(image)
 *         img_buf_p = <void *>PyBytes_AsString(image)             # <<<<<<<<<<<<<<
 *         err = H5Pset_file_image(access_plist, img_buf_p, img_buf_len)
 *         if err < 0:
*/
      __pyx_t_16 = PyBytes_AsString(__pyx_v_image); if (unlikely(__pyx_t_16 == ((void *)NULL))) __PYX_ERR(0, 573, __pyx_L1_error)
      __pyx_v_img_buf_p = ((void *)__pyx_t_16);

      /* "tables/hdf5extension.pyx":574
 *         img_buf_len = len(image)
 *         img_buf_p = <void *>PyBytes_AsString(image)
 *         err = H5Pset_file_image(access_plist, img_buf_p, img_buf_len)             # <<<<<<<<<<<<<<
 *         if err < 0:
 *           H5Pclose(create_plist)
*/
      __pyx_v_err = H5Pset_file_image(__pyx_v_access_plist, __pyx_v_img_buf_p, __pyx_v_img_buf_len);

      /* "tables/hdf5extension.pyx":575
 *         img_buf_p = <void *>PyBytes_AsString(image)
 *         err = H5Pset_file_image(access_plist, img_buf_p, img_buf_len)
 *         if err < 0:             # <<<<<<<<<<<<<<
 *           H5Pclose(create_plist)
 *           H5Pclose(access_plist)
*/
      __pyx_t_3 = (__pyx_v_err < 0);
      if (unlikely(__pyx_t_3)) {

        /* "tables/hdf5extension.pyx":576
 *         err = H5Pset_file_image(access_plist, img_buf_p, img_buf_len)
 *         if err < 0:
 *           H5Pclose(create_plist)             # <<<<<<<<<<<<<<
 *           H5Pclose(access_plist)
 *           raise HDF5ExtError("Unable to set the file image")
*/
        (void)(H5Pclose(__pyx_v_create_plist));

        /* "tables/hdf5extension.pyx":577
 *         if err < 0:
 *           H5Pclose(create_plist)
 *           H5Pclose(access_plist)             # <<<<<<<<<<<<<<
 *           raise HDF5ExtError("Unable to set the file image")
 * 
*/
        (void)(H5Pclose(__pyx_v_access_plist));

        /* "tables/hdf5extension.pyx":578
 *           H5Pclose(create_plist)
 *           H5Pclose(access_plist)
 *           raise HDF5ExtError("Unable to set the file image")             # <<<<<<<<<<<<<<
 * 
 *     #elif driver == "H5FD_FAMILY":
*/
        __pyx_t_9 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 578, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_14 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_5);
          assert(__pyx_t_9);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
          __pyx_t_14 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_mstate_global->__pyx_kp_u_Unable_to_set_the_file_image};
          __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_14, (2-__pyx_t_14) | (__pyx_t_14*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 578, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
        }
        __Pyx_Raise(__pyx_t_7, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __PYX_ERR(0, 578, __pyx_L1_error)

        /* "tables/hdf5extension.pyx":575
 *         img_buf_p = <void *>PyBytes_AsString(image)
 *         err = H5Pset_file_image(access_plist, img_buf_p, img_buf_len)
 *         if err < 0:             # <<<<<<<<<<<<<<
 *           H5Pclose(create_plist)
 *           H5Pclose(access_plist)
*/
      }

      /* "tables/hdf5extension.pyx":571
 *                              params["DRIVER_CORE_INCREMENT"],
 *                              backing_store)
 *       if image:             # <<<<<<<<<<<<<<
 *         img_buf_len = len(image)
 *         img_buf_p = <void *>PyBytes_AsString(image)
*/
    }

    /* "tables/hdf5extension.pyx":567
 *     elif driver == "H5FD_STDIO":
 *       err = H5Pset_fapl_stdio(access_plist)
 *     elif driver == "H5FD_CORE":             # <<<<<<<<<<<<<<
 *       err = H5Pset_fapl_core(access_plist,
 *                              params["DRIVER_CORE_INCREMENT"],
*/
    goto __pyx_L35;
  }

  /* "tables/hdf5extension.pyx":587
 *     #  err = H5Pset_fapl_multi(access_plist, memb_map, memb_fapl, memb_name,
 *     #                          memb_addr, relax)
 *     elif driver == "H5FD_SPLIT":             # <<<<<<<<<<<<<<
 *       err = H5Pset_fapl_split(access_plist, enc_meta_ext, meta_plist_id,
 *                               enc_raw_ext, raw_plist_id)
*/
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_n_u_H5FD_SPLIT, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 587, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":588
 *     #                          memb_addr, relax)
 *     elif driver == "H5FD_SPLIT":
 *       err = H5Pset_fapl_split(access_plist, enc_meta_ext, meta_plist_id,             # <<<<<<<<<<<<<<
 *                               enc_raw_ext, raw_plist_id)
 *     if err < 0:
*/
    if (unlikely(!__pyx_v_enc_meta_ext)) { __Pyx_RaiseUnboundLocalError("enc_meta_ext"); __PYX_ERR(0, 588, __pyx_L1_error) }
    __pyx_t_16 = __Pyx_PyObject_AsWritableString(__pyx_v_enc_meta_ext); if (unlikely((!__pyx_t_16) && PyErr_Occurred())) __PYX_ERR(0, 588, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":589
 *     elif driver == "H5FD_SPLIT":
 *       err = H5Pset_fapl_split(access_plist, enc_meta_ext, meta_plist_id,
 *                               enc_raw_ext, raw_plist_id)             # <<<<<<<<<<<<<<
 *     if err < 0:
 *       e = HDF5ExtError("Unable to set the file access property list")
*/
    if (unlikely(!__pyx_v_enc_raw_ext)) { __Pyx_RaiseUnboundLocalError("enc_raw_ext"); __PYX_ERR(0, 589, __pyx_L1_error) }
    __pyx_t_17 = __Pyx_PyObject_AsWritableString(__pyx_v_enc_raw_ext); if (unlikely((!__pyx_t_17) && PyErr_Occurred())) __PYX_ERR(0, 589, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":588
 *     #                          memb_addr, relax)
 *     elif driver == "H5FD_SPLIT":
 *       err = H5Pset_fapl_split(access_plist, enc_meta_ext, meta_plist_id,             # <<<<<<<<<<<<<<
 *                               enc_raw_ext, raw_plist_id)
 *     if err < 0:
*/
    __pyx_v_err = H5Pset_fapl_split(__pyx_v_access_plist, __pyx_t_16, __pyx_v_meta_plist_id, __pyx_t_17, __pyx_v_raw_plist_id);

    /* "tables/hdf5extension.pyx":587
 *     #  err = H5Pset_fapl_multi(access_plist, memb_map, memb_fapl, memb_name,
 *     #                          memb_addr, relax)
 *     elif driver == "H5FD_SPLIT":             # <<<<<<<<<<<<<<
 *       err = H5Pset_fapl_split(access_plist, enc_meta_ext, meta_plist_id,
 *                               enc_raw_ext, raw_plist_id)
*/
  }
  __pyx_L35:;

  /* "tables/hdf5extension.pyx":590
 *       err = H5Pset_fapl_split(access_plist, enc_meta_ext, meta_plist_id,
 *                               enc_raw_ext, raw_plist_id)
 *     if err < 0:             # <<<<<<<<<<<<<<
 *       e = HDF5ExtError("Unable to set the file access property list")
 *       H5Pclose(create_plist)
*/
  __pyx_t_3 = (__pyx_v_err < 0);
  if (unlikely(__pyx_t_3)) {

    /* "tables/hdf5extension.pyx":591
 *                               enc_raw_ext, raw_plist_id)
 *     if err < 0:
 *       e = HDF5ExtError("Unable to set the file access property list")             # <<<<<<<<<<<<<<
 *       H5Pclose(create_plist)
 *       H5Pclose(access_plist)
*/
    __pyx_t_5 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 591, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_14 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_9);
      assert(__pyx_t_5);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
      __pyx_t_14 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_Unable_to_set_the_file_access_pr};
      __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_14, (2-__pyx_t_14) | (__pyx_t_14*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 591, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }
    __pyx_v_e = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "tables/hdf5extension.pyx":592
 *     if err < 0:
 *       e = HDF5ExtError("Unable to set the file access property list")
 *       H5Pclose(create_plist)             # <<<<<<<<<<<<<<
 *       H5Pclose(access_plist)
 *       raise e
*/
    (void)(H5Pclose(__pyx_v_create_plist));

    /* "tables/hdf5extension.pyx":593
 *       e = HDF5ExtError("Unable to set the file access property list")
 *       H5Pclose(create_plist)
 *       H5Pclose(access_plist)             # <<<<<<<<<<<<<<
 *       raise e
 * 
*/
    (void)(H5Pclose(__pyx_v_access_plist));

    /* "tables/hdf5extension.pyx":594
 *       H5Pclose(create_plist)
 *       H5Pclose(access_plist)
 *       raise e             # <<<<<<<<<<<<<<
 * 
 *     if pymode == 'r':
*/
    __Pyx_Raise(__pyx_v_e, 0, 0, 0);
    __PYX_ERR(0, 594, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":590
 *       err = H5Pset_fapl_split(access_plist, enc_meta_ext, meta_plist_id,
 *                               enc_raw_ext, raw_plist_id)
 *     if err < 0:             # <<<<<<<<<<<<<<
 *       e = HDF5ExtError("Unable to set the file access property list")
 *       H5Pclose(create_plist)
*/
  }

  /* "tables/hdf5extension.pyx":596
 *       raise e
 * 
 *     if pymode == 'r':             # <<<<<<<<<<<<<<
 *       self.file_id = H5Fopen(encname, H5F_ACC_RDONLY, access_plist)
 *     elif pymode == 'r+':
*/
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_pymode, __pyx_mstate_global->__pyx_n_u_r, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 596, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":597
 * 
 *     if pymode == 'r':
 *       self.file_id = H5Fopen(encname, H5F_ACC_RDONLY, access_plist)             # <<<<<<<<<<<<<<
 *     elif pymode == 'r+':
 *       self.file_id = H5Fopen(encname, H5F_ACC_RDWR, access_plist)
*/
    if (unlikely(__pyx_v_encname == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 597, __pyx_L1_error)
    }
    __pyx_t_17 = __Pyx_PyBytes_AsWritableString(__pyx_v_encname); if (unlikely((!__pyx_t_17) && PyErr_Occurred())) __PYX_ERR(0, 597, __pyx_L1_error)
    __pyx_v_self->file_id = H5Fopen(__pyx_t_17, H5F_ACC_RDONLY, __pyx_v_access_plist);

    /* "tables/hdf5extension.pyx":596
 *       raise e
 * 
 *     if pymode == 'r':             # <<<<<<<<<<<<<<
 *       self.file_id = H5Fopen(encname, H5F_ACC_RDONLY, access_plist)
 *     elif pymode == 'r+':
*/
    goto __pyx_L41;
  }

  /* "tables/hdf5extension.pyx":598
 *     if pymode == 'r':
 *       self.file_id = H5Fopen(encname, H5F_ACC_RDONLY, access_plist)
 *     elif pymode == 'r+':             # <<<<<<<<<<<<<<
 *       self.file_id = H5Fopen(encname, H5F_ACC_RDWR, access_plist)
 *     elif pymode == 'a':
*/
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_pymode, __pyx_mstate_global->__pyx_kp_u_r_2, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 598, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":599
 *       self.file_id = H5Fopen(encname, H5F_ACC_RDONLY, access_plist)
 *     elif pymode == 'r+':
 *       self.file_id = H5Fopen(encname, H5F_ACC_RDWR, access_plist)             # <<<<<<<<<<<<<<
 *     elif pymode == 'a':
 *       if exists:
*/
    if (unlikely(__pyx_v_encname == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 599, __pyx_L1_error)
    }
    __pyx_t_17 = __Pyx_PyBytes_AsWritableString(__pyx_v_encname); if (unlikely((!__pyx_t_17) && PyErr_Occurred())) __PYX_ERR(0, 599, __pyx_L1_error)
    __pyx_v_self->file_id = H5Fopen(__pyx_t_17, H5F_ACC_RDWR, __pyx_v_access_plist);

    /* "tables/hdf5extension.pyx":598
 *     if pymode == 'r':
 *       self.file_id = H5Fopen(encname, H5F_ACC_RDONLY, access_plist)
 *     elif pymode == 'r+':             # <<<<<<<<<<<<<<
 *       self.file_id = H5Fopen(encname, H5F_ACC_RDWR, access_plist)
 *     elif pymode == 'a':
*/
    goto __pyx_L41;
  }

  /* "tables/hdf5extension.pyx":600
 *     elif pymode == 'r+':
 *       self.file_id = H5Fopen(encname, H5F_ACC_RDWR, access_plist)
 *     elif pymode == 'a':             # <<<<<<<<<<<<<<
 *       if exists:
 *         # A test for logging.
*/
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_pymode, __pyx_mstate_global->__pyx_n_u_a, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 600, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":601
 *       self.file_id = H5Fopen(encname, H5F_ACC_RDWR, access_plist)
 *     elif pymode == 'a':
 *       if exists:             # <<<<<<<<<<<<<<
 *         # A test for logging.
 *         ## H5Pset_sieve_buf_size(access_plist, 0)
*/
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_exists); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 601, __pyx_L1_error)
    if (__pyx_t_3) {

      /* "tables/hdf5extension.pyx":605
 *         ## H5Pset_sieve_buf_size(access_plist, 0)
 *         ## H5Pset_fapl_log (access_plist, "test.log", H5FD_LOG_LOC_WRITE, 0)
 *         self.file_id = H5Fopen(encname, H5F_ACC_RDWR, access_plist)             # <<<<<<<<<<<<<<
 *       else:
 *         self.file_id = H5Fcreate(encname, H5F_ACC_TRUNC, create_plist,
*/
      if (unlikely(__pyx_v_encname == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
        __PYX_ERR(0, 605, __pyx_L1_error)
      }
      __pyx_t_17 = __Pyx_PyBytes_AsWritableString(__pyx_v_encname); if (unlikely((!__pyx_t_17) && PyErr_Occurred())) __PYX_ERR(0, 605, __pyx_L1_error)
      __pyx_v_self->file_id = H5Fopen(__pyx_t_17, H5F_ACC_RDWR, __pyx_v_access_plist);

      /* "tables/hdf5extension.pyx":601
 *       self.file_id = H5Fopen(encname, H5F_ACC_RDWR, access_plist)
 *     elif pymode == 'a':
 *       if exists:             # <<<<<<<<<<<<<<
 *         # A test for logging.
 *         ## H5Pset_sieve_buf_size(access_plist, 0)
*/
      goto __pyx_L42;
    }

    /* "tables/hdf5extension.pyx":607
 *         self.file_id = H5Fopen(encname, H5F_ACC_RDWR, access_plist)
 *       else:
 *         self.file_id = H5Fcreate(encname, H5F_ACC_TRUNC, create_plist,             # <<<<<<<<<<<<<<
 *                                  access_plist)
 *     elif pymode == 'w':
*/
    /*else*/ {
      if (unlikely(__pyx_v_encname == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
        __PYX_ERR(0, 607, __pyx_L1_error)
      }
      __pyx_t_17 = __Pyx_PyBytes_AsWritableString(__pyx_v_encname); if (unlikely((!__pyx_t_17) && PyErr_Occurred())) __PYX_ERR(0, 607, __pyx_L1_error)

      /* "tables/hdf5extension.pyx":608
 *       else:
 *         self.file_id = H5Fcreate(encname, H5F_ACC_TRUNC, create_plist,
 *                                  access_plist)             # <<<<<<<<<<<<<<
 *     elif pymode == 'w':
 *       self.file_id = H5Fcreate(encname, H5F_ACC_TRUNC, create_plist,
*/
      __pyx_v_self->file_id = H5Fcreate(__pyx_t_17, H5F_ACC_TRUNC, __pyx_v_create_plist, __pyx_v_access_plist);
    }
    __pyx_L42:;

    /* "tables/hdf5extension.pyx":600
 *     elif pymode == 'r+':
 *       self.file_id = H5Fopen(encname, H5F_ACC_RDWR, access_plist)
 *     elif pymode == 'a':             # <<<<<<<<<<<<<<
 *       if exists:
 *         # A test for logging.
*/
    goto __pyx_L41;
  }

  /* "tables/hdf5extension.pyx":609
 *         self.file_id = H5Fcreate(encname, H5F_ACC_TRUNC, create_plist,
 *                                  access_plist)
 *     elif pymode == 'w':             # <<<<<<<<<<<<<<
 *       self.file_id = H5Fcreate(encname, H5F_ACC_TRUNC, create_plist,
 *                                access_plist)
*/
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_pymode, __pyx_mstate_global->__pyx_n_u_w, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 609, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":610
 *                                  access_plist)
 *     elif pymode == 'w':
 *       self.file_id = H5Fcreate(encname, H5F_ACC_TRUNC, create_plist,             # <<<<<<<<<<<<<<
 *                                access_plist)
 * 
*/
    if (unlikely(__pyx_v_encname == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 610, __pyx_L1_error)
    }
    __pyx_t_17 = __Pyx_PyBytes_AsWritableString(__pyx_v_encname); if (unlikely((!__pyx_t_17) && PyErr_Occurred())) __PYX_ERR(0, 610, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":611
 *     elif pymode == 'w':
 *       self.file_id = H5Fcreate(encname, H5F_ACC_TRUNC, create_plist,
 *                                access_plist)             # <<<<<<<<<<<<<<
 * 
 *     if self.file_id < 0:
*/
    __pyx_v_self->file_id = H5Fcreate(__pyx_t_17, H5F_ACC_TRUNC, __pyx_v_create_plist, __pyx_v_access_plist);

    /* "tables/hdf5extension.pyx":609
 *         self.file_id = H5Fcreate(encname, H5F_ACC_TRUNC, create_plist,
 *                                  access_plist)
 *     elif pymode == 'w':             # <<<<<<<<<<<<<<
 *       self.file_id = H5Fcreate(encname, H5F_ACC_TRUNC, create_plist,
 *                                access_plist)
*/
  }
  __pyx_L41:;

  /* "tables/hdf5extension.pyx":613
 *                                access_plist)
 * 
 *     if self.file_id < 0:             # <<<<<<<<<<<<<<
 *         e = HDF5ExtError("Unable to open/create file '%s'" % name)
 *         H5Pclose(create_plist)
*/
  __pyx_t_3 = (__pyx_v_self->file_id < 0);
  if (unlikely(__pyx_t_3)) {

    /* "tables/hdf5extension.pyx":614
 * 
 *     if self.file_id < 0:
 *         e = HDF5ExtError("Unable to open/create file '%s'" % name)             # <<<<<<<<<<<<<<
 *         H5Pclose(create_plist)
 *         H5Pclose(access_plist)
*/
    __pyx_t_9 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 614, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Unable_to_open_create_file_s, __pyx_v_name); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 614, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_14 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_9);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_14 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_t_4};
      __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_14, (2-__pyx_t_14) | (__pyx_t_14*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 614, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }
    __pyx_v_e = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "tables/hdf5extension.pyx":615
 *     if self.file_id < 0:
 *         e = HDF5ExtError("Unable to open/create file '%s'" % name)
 *         H5Pclose(create_plist)             # <<<<<<<<<<<<<<
 *         H5Pclose(access_plist)
 *         raise e
*/
    (void)(H5Pclose(__pyx_v_create_plist));

    /* "tables/hdf5extension.pyx":616
 *         e = HDF5ExtError("Unable to open/create file '%s'" % name)
 *         H5Pclose(create_plist)
 *         H5Pclose(access_plist)             # <<<<<<<<<<<<<<
 *         raise e
 * 
*/
    (void)(H5Pclose(__pyx_v_access_plist));

    /* "tables/hdf5extension.pyx":617
 *         H5Pclose(create_plist)
 *         H5Pclose(access_plist)
 *         raise e             # <<<<<<<<<<<<<<
 * 
 *     H5Pclose(create_plist)
*/
    __Pyx_Raise(__pyx_v_e, 0, 0, 0);
    __PYX_ERR(0, 617, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":613
 *                                access_plist)
 * 
 *     if self.file_id < 0:             # <<<<<<<<<<<<<<
 *         e = HDF5ExtError("Unable to open/create file '%s'" % name)
 *         H5Pclose(create_plist)
*/
  }

  /* "tables/hdf5extension.pyx":619
 *         raise e
 * 
 *     H5Pclose(create_plist)             # <<<<<<<<<<<<<<
 *     H5Pclose(access_plist)
 * 
*/
  (void)(H5Pclose(__pyx_v_create_plist));

  /* "tables/hdf5extension.pyx":620
 * 
 *     H5Pclose(create_plist)
 *     H5Pclose(access_plist)             # <<<<<<<<<<<<<<
 * 
 *     # Set the cache size
*/
  (void)(H5Pclose(__pyx_v_access_plist));

  /* "tables/hdf5extension.pyx":623
 * 
 *     # Set the cache size
 *     set_cache_size(self.file_id, params["METADATA_CACHE_SIZE"])             # <<<<<<<<<<<<<<
 * 
 *     # Set the maximum number of threads for Blosc
*/
  __pyx_t_7 = __Pyx_PyDict_GetItem(__pyx_v_params, __pyx_mstate_global->__pyx_n_u_METADATA_CACHE_SIZE); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 623, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_14 = __Pyx_PyLong_As_size_t(__pyx_t_7); if (unlikely((__pyx_t_14 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 623, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  (void)(set_cache_size(__pyx_v_self->file_id, __pyx_t_14));

  /* "tables/hdf5extension.pyx":626
 * 
 *     # Set the maximum number of threads for Blosc
 *     set_blosc_max_threads(params["MAX_BLOSC_THREADS"])             # <<<<<<<<<<<<<<
 *     set_blosc2_max_threads(params["MAX_BLOSC_THREADS"])
 * 
*/
  __pyx_t_5 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_set_blosc_max_threads); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 626, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_9 = __Pyx_PyDict_GetItem(__pyx_v_params, __pyx_mstate_global->__pyx_n_u_MAX_BLOSC_THREADS); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 626, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_14 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_5);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_14 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_9};
    __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_14, (2-__pyx_t_14) | (__pyx_t_14*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 626, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "tables/hdf5extension.pyx":627
 *     # Set the maximum number of threads for Blosc
 *     set_blosc_max_threads(params["MAX_BLOSC_THREADS"])
 *     set_blosc2_max_threads(params["MAX_BLOSC_THREADS"])             # <<<<<<<<<<<<<<
 * 
 *   # XXX: add the possibility to pass a pre-allocated buffer
*/
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_set_blosc2_max_threads); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_5 = __Pyx_PyDict_GetItem(__pyx_v_params, __pyx_mstate_global->__pyx_n_u_MAX_BLOSC_THREADS); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_14 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_9);
    assert(__pyx_t_4);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
    __pyx_t_14 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_5};
    __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_14, (2-__pyx_t_14) | (__pyx_t_14*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 627, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "tables/hdf5extension.pyx":445
 *   cdef object  name
 * 
 *   def _g_new(self, name, pymode, **params):             # <<<<<<<<<<<<<<
 *     cdef herr_t err = 0
 *     cdef hid_t access_plist, create_plist = H5P_DEFAULT
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("tables.hdf5extension.File._g_new", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_encname);
  __Pyx_XDECREF(__pyx_v_driver);
  __Pyx_XDECREF(__pyx_v_meta_ext);
  __Pyx_XDECREF(__pyx_v_raw_ext);
  __Pyx_XDECREF(__pyx_v_meta_name);
  __Pyx_XDECREF(__pyx_v_raw_name);
  __Pyx_XDECREF(__pyx_v_enc_meta_ext);
  __Pyx_XDECREF(__pyx_v_enc_raw_ext);
  __Pyx_XDECREF(__pyx_v_image);
  __Pyx_XDECREF(__pyx_v_n);
  __Pyx_XDECREF(__pyx_v_backing_store);
  __Pyx_XDECREF(__pyx_v_exists);
  __Pyx_XDECREF(__pyx_v_is_pow_of_2);
  __Pyx_XDECREF(__pyx_v_e);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":630
 * 
 *   # XXX: add the possibility to pass a pre-allocated buffer
 *   def get_file_image(self):             # <<<<<<<<<<<<<<
 *     """Retrieves an in-memory image of an existing, open HDF5 file.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_3get_file_image(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_4File_2get_file_image, "Retrieves an in-memory image of an existing, open HDF5 file.\n\n    .. versionadded:: 3.0\n\n    ");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4File_3get_file_image = {"get_file_image", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_3get_file_image, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_4File_2get_file_image};
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_3get_file_image(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_file_image (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("get_file_image", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("get_file_image", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4File_2get_file_image(((struct __pyx_obj_6tables_13hdf5extension_File *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4File_2get_file_image(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self) {
  Py_ssize_t __pyx_v_size;
  size_t __pyx_v_buf_len;
  PyObject *__pyx_v_image = 0;
  char *__pyx_v_cimage;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  char *__pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_file_image", 0);

  /* "tables/hdf5extension.pyx":637
 *     """
 * 
 *     cdef ssize_t size = 0             # <<<<<<<<<<<<<<
 *     cdef size_t buf_len = 0
 *     cdef bytes image
*/
  __pyx_v_size = 0;

  /* "tables/hdf5extension.pyx":638
 * 
 *     cdef ssize_t size = 0
 *     cdef size_t buf_len = 0             # <<<<<<<<<<<<<<
 *     cdef bytes image
 *     cdef char* cimage
*/
  __pyx_v_buf_len = 0;

  /* "tables/hdf5extension.pyx":642
 *     cdef char* cimage
 * 
 *     self.flush()             # <<<<<<<<<<<<<<
 * 
 *     # retrieve the size of the buffer for the file image
*/
  __pyx_t_2 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_flush, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 642, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":645
 * 
 *     # retrieve the size of the buffer for the file image
 *     size = H5Fget_file_image(self.file_id, NULL, buf_len)             # <<<<<<<<<<<<<<
 *     if size < 0:
 *       raise HDF5ExtError("Unable to retrieve the size of the buffer for the "
*/
  __pyx_v_size = H5Fget_file_image(__pyx_v_self->file_id, NULL, __pyx_v_buf_len);

  /* "tables/hdf5extension.pyx":646
 *     # retrieve the size of the buffer for the file image
 *     size = H5Fget_file_image(self.file_id, NULL, buf_len)
 *     if size < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Unable to retrieve the size of the buffer for the "
 *                          "file image.  Plese note that not all drivers "
*/
  __pyx_t_4 = (__pyx_v_size < 0);
  if (unlikely(__pyx_t_4)) {

    /* "tables/hdf5extension.pyx":647
 *     size = H5Fget_file_image(self.file_id, NULL, buf_len)
 *     if size < 0:
 *       raise HDF5ExtError("Unable to retrieve the size of the buffer for the "             # <<<<<<<<<<<<<<
 *                          "file image.  Plese note that not all drivers "
 *                          "provide support for image files.")
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 647, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_Unable_to_retrieve_the_size_of_t};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 647, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 647, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":646
 *     # retrieve the size of the buffer for the file image
 *     size = H5Fget_file_image(self.file_id, NULL, buf_len)
 *     if size < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Unable to retrieve the size of the buffer for the "
 *                          "file image.  Plese note that not all drivers "
*/
  }

  /* "tables/hdf5extension.pyx":652
 * 
 *     # allocate the memory buffer
 *     image = PyBytes_FromStringAndSize(NULL, size)             # <<<<<<<<<<<<<<
 *     if not image:
 *       raise RuntimeError("Unable to allecote meomory fir the file image")
*/
  __pyx_t_1 = PyBytes_FromStringAndSize(NULL, __pyx_v_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 652, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_image = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":653
 *     # allocate the memory buffer
 *     image = PyBytes_FromStringAndSize(NULL, size)
 *     if not image:             # <<<<<<<<<<<<<<
 *       raise RuntimeError("Unable to allecote meomory fir the file image")
 * 
*/
  if (__pyx_v_image == Py_None) __pyx_t_4 = 0;
  else
  {
    Py_ssize_t __pyx_temp = __Pyx_PyBytes_GET_SIZE(__pyx_v_image);
    if (unlikely(((!CYTHON_ASSUME_SAFE_SIZE) && __pyx_temp < 0))) __PYX_ERR(0, 653, __pyx_L1_error)
    __pyx_t_4 = (__pyx_temp != 0);
  }

  __pyx_t_6 = (!__pyx_t_4);
  if (unlikely(__pyx_t_6)) {

    /* "tables/hdf5extension.pyx":654
 *     image = PyBytes_FromStringAndSize(NULL, size)
 *     if not image:
 *       raise RuntimeError("Unable to allecote meomory fir the file image")             # <<<<<<<<<<<<<<
 * 
 *     cimage = image
*/
    __pyx_t_5 = NULL;
    __pyx_t_3 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_Unable_to_allecote_meomory_fir_t};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 654, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 654, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":653
 *     # allocate the memory buffer
 *     image = PyBytes_FromStringAndSize(NULL, size)
 *     if not image:             # <<<<<<<<<<<<<<
 *       raise RuntimeError("Unable to allecote meomory fir the file image")
 * 
*/
  }

  /* "tables/hdf5extension.pyx":656
 *       raise RuntimeError("Unable to allecote meomory fir the file image")
 * 
 *     cimage = image             # <<<<<<<<<<<<<<
 *     buf_len = size
 *     size = H5Fget_file_image(self.file_id, <void*>cimage, buf_len)
*/
  if (unlikely(__pyx_v_image == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 656, __pyx_L1_error)
  }
  __pyx_t_7 = __Pyx_PyBytes_AsWritableString(__pyx_v_image); if (unlikely((!__pyx_t_7) && PyErr_Occurred())) __PYX_ERR(0, 656, __pyx_L1_error)
  __pyx_v_cimage = __pyx_t_7;

  /* "tables/hdf5extension.pyx":657
 * 
 *     cimage = image
 *     buf_len = size             # <<<<<<<<<<<<<<
 *     size = H5Fget_file_image(self.file_id, <void*>cimage, buf_len)
 *     if size < 0:
*/
  __pyx_v_buf_len = __pyx_v_size;

  /* "tables/hdf5extension.pyx":658
 *     cimage = image
 *     buf_len = size
 *     size = H5Fget_file_image(self.file_id, <void*>cimage, buf_len)             # <<<<<<<<<<<<<<
 *     if size < 0:
 *       raise HDF5ExtError("Unable to retrieve the file image. "
*/
  __pyx_v_size = H5Fget_file_image(__pyx_v_self->file_id, ((void *)__pyx_v_cimage), __pyx_v_buf_len);

  /* "tables/hdf5extension.pyx":659
 *     buf_len = size
 *     size = H5Fget_file_image(self.file_id, <void*>cimage, buf_len)
 *     if size < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Unable to retrieve the file image. "
 *                          "Plese note that not all drivers provide support "
*/
  __pyx_t_6 = (__pyx_v_size < 0);
  if (unlikely(__pyx_t_6)) {

    /* "tables/hdf5extension.pyx":660
 *     size = H5Fget_file_image(self.file_id, <void*>cimage, buf_len)
 *     if size < 0:
 *       raise HDF5ExtError("Unable to retrieve the file image. "             # <<<<<<<<<<<<<<
 *                          "Plese note that not all drivers provide support "
 *                          "for image files.")
*/
    __pyx_t_5 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 660, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      assert(__pyx_t_5);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_Unable_to_retrieve_the_file_imag};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 660, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 660, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":659
 *     buf_len = size
 *     size = H5Fget_file_image(self.file_id, <void*>cimage, buf_len)
 *     if size < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Unable to retrieve the file image. "
 *                          "Plese note that not all drivers provide support "
*/
  }

  /* "tables/hdf5extension.pyx":664
 *                          "for image files.")
 * 
 *     return image             # <<<<<<<<<<<<<<
 * 
 *   def get_filesize(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_image);
  __pyx_r = __pyx_v_image;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":630
 * 
 *   # XXX: add the possibility to pass a pre-allocated buffer
 *   def get_file_image(self):             # <<<<<<<<<<<<<<
 *     """Retrieves an in-memory image of an existing, open HDF5 file.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("tables.hdf5extension.File.get_file_image", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_image);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":666
 *     return image
 * 
 *   def get_filesize(self):             # <<<<<<<<<<<<<<
 *     """Returns the size of an HDF5 file.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_5get_filesize(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_4File_4get_filesize, "Returns the size of an HDF5 file.\n\n    The returned size is that of the entire file, as opposed to only\n    the HDF5 portion of the file. I.e., size includes the user block,\n    if any, the HDF5 portion of the file, and any data that may have\n    been appended beyond the data written through the HDF5 Library.\n\n    .. versionadded:: 3.0\n\n    ");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4File_5get_filesize = {"get_filesize", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_5get_filesize, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_4File_4get_filesize};
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_5get_filesize(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_filesize (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("get_filesize", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("get_filesize", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4File_4get_filesize(((struct __pyx_obj_6tables_13hdf5extension_File *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4File_4get_filesize(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self) {
  herr_t __pyx_v_err;
  hsize_t __pyx_v_size;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_filesize", 0);

  /* "tables/hdf5extension.pyx":678
 *     """
 * 
 *     cdef herr_t err = 0             # <<<<<<<<<<<<<<
 *     cdef hsize_t size = 0
 * 
*/
  __pyx_v_err = 0;

  /* "tables/hdf5extension.pyx":679
 * 
 *     cdef herr_t err = 0
 *     cdef hsize_t size = 0             # <<<<<<<<<<<<<<
 * 
 *     err = H5Fget_filesize(self.file_id, &size)
*/
  __pyx_v_size = 0;

  /* "tables/hdf5extension.pyx":681
 *     cdef hsize_t size = 0
 * 
 *     err = H5Fget_filesize(self.file_id, &size)             # <<<<<<<<<<<<<<
 *     if err < 0:
 *       raise HDF5ExtError("Unable to retrieve the HDF5 file size")
*/
  __pyx_v_err = H5Fget_filesize(__pyx_v_self->file_id, (&__pyx_v_size));

  /* "tables/hdf5extension.pyx":682
 * 
 *     err = H5Fget_filesize(self.file_id, &size)
 *     if err < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Unable to retrieve the HDF5 file size")
 * 
*/
  __pyx_t_1 = (__pyx_v_err < 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/hdf5extension.pyx":683
 *     err = H5Fget_filesize(self.file_id, &size)
 *     if err < 0:
 *       raise HDF5ExtError("Unable to retrieve the HDF5 file size")             # <<<<<<<<<<<<<<
 * 
 *     return size
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 683, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Unable_to_retrieve_the_HDF5_file};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 683, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 683, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":682
 * 
 *     err = H5Fget_filesize(self.file_id, &size)
 *     if err < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Unable to retrieve the HDF5 file size")
 * 
*/
  }

  /* "tables/hdf5extension.pyx":685
 *       raise HDF5ExtError("Unable to retrieve the HDF5 file size")
 * 
 *     return size             # <<<<<<<<<<<<<<
 * 
 *   def get_userblock_size(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyLong_From_hsize_t(__pyx_v_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 685, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":666
 *     return image
 * 
 *   def get_filesize(self):             # <<<<<<<<<<<<<<
 *     """Returns the size of an HDF5 file.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("tables.hdf5extension.File.get_filesize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":687
 *     return size
 * 
 *   def get_userblock_size(self):             # <<<<<<<<<<<<<<
 *     """Retrieves the size of a user block.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_7get_userblock_size(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_4File_6get_userblock_size, "Retrieves the size of a user block.\n\n    .. versionadded:: 3.0\n\n    ");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4File_7get_userblock_size = {"get_userblock_size", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_7get_userblock_size, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_4File_6get_userblock_size};
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_7get_userblock_size(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_userblock_size (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("get_userblock_size", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("get_userblock_size", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4File_6get_userblock_size(((struct __pyx_obj_6tables_13hdf5extension_File *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4File_6get_userblock_size(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self) {
  herr_t __pyx_v_err;
  hsize_t __pyx_v_size;
  hid_t __pyx_v_create_plist;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_userblock_size", 0);

  /* "tables/hdf5extension.pyx":694
 *     """
 * 
 *     cdef herr_t err = 0             # <<<<<<<<<<<<<<
 *     cdef hsize_t size = 0
 *     cdef hid_t create_plist
*/
  __pyx_v_err = 0;

  /* "tables/hdf5extension.pyx":695
 * 
 *     cdef herr_t err = 0
 *     cdef hsize_t size = 0             # <<<<<<<<<<<<<<
 *     cdef hid_t create_plist
 * 
*/
  __pyx_v_size = 0;

  /* "tables/hdf5extension.pyx":698
 *     cdef hid_t create_plist
 * 
 *     create_plist = H5Fget_create_plist(self.file_id)             # <<<<<<<<<<<<<<
 *     if create_plist < 0:
 *       raise HDF5ExtError("Unable to get the creation property list")
*/
  __pyx_v_create_plist = H5Fget_create_plist(__pyx_v_self->file_id);

  /* "tables/hdf5extension.pyx":699
 * 
 *     create_plist = H5Fget_create_plist(self.file_id)
 *     if create_plist < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Unable to get the creation property list")
 * 
*/
  __pyx_t_1 = (__pyx_v_create_plist < 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/hdf5extension.pyx":700
 *     create_plist = H5Fget_create_plist(self.file_id)
 *     if create_plist < 0:
 *       raise HDF5ExtError("Unable to get the creation property list")             # <<<<<<<<<<<<<<
 * 
 *     err = H5Pget_userblock(create_plist, &size)
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 700, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Unable_to_get_the_creation_prope};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 700, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 700, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":699
 * 
 *     create_plist = H5Fget_create_plist(self.file_id)
 *     if create_plist < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Unable to get the creation property list")
 * 
*/
  }

  /* "tables/hdf5extension.pyx":702
 *       raise HDF5ExtError("Unable to get the creation property list")
 * 
 *     err = H5Pget_userblock(create_plist, &size)             # <<<<<<<<<<<<<<
 *     if err < 0:
 *       H5Pclose(create_plist)
*/
  __pyx_v_err = H5Pget_userblock(__pyx_v_create_plist, (&__pyx_v_size));

  /* "tables/hdf5extension.pyx":703
 * 
 *     err = H5Pget_userblock(create_plist, &size)
 *     if err < 0:             # <<<<<<<<<<<<<<
 *       H5Pclose(create_plist)
 *       raise HDF5ExtError("unable to retrieve the user block size")
*/
  __pyx_t_1 = (__pyx_v_err < 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/hdf5extension.pyx":704
 *     err = H5Pget_userblock(create_plist, &size)
 *     if err < 0:
 *       H5Pclose(create_plist)             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("unable to retrieve the user block size")
 * 
*/
    (void)(H5Pclose(__pyx_v_create_plist));

    /* "tables/hdf5extension.pyx":705
 *     if err < 0:
 *       H5Pclose(create_plist)
 *       raise HDF5ExtError("unable to retrieve the user block size")             # <<<<<<<<<<<<<<
 * 
 *     H5Pclose(create_plist)
*/
    __pyx_t_4 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 705, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_unable_to_retrieve_the_user_bloc};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 705, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 705, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":703
 * 
 *     err = H5Pget_userblock(create_plist, &size)
 *     if err < 0:             # <<<<<<<<<<<<<<
 *       H5Pclose(create_plist)
 *       raise HDF5ExtError("unable to retrieve the user block size")
*/
  }

  /* "tables/hdf5extension.pyx":707
 *       raise HDF5ExtError("unable to retrieve the user block size")
 * 
 *     H5Pclose(create_plist)             # <<<<<<<<<<<<<<
 * 
 *     return size
*/
  (void)(H5Pclose(__pyx_v_create_plist));

  /* "tables/hdf5extension.pyx":709
 *     H5Pclose(create_plist)
 * 
 *     return size             # <<<<<<<<<<<<<<
 * 
 *   # Accessor definitions
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyLong_From_hsize_t(__pyx_v_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 709, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":687
 *     return size
 * 
 *   def get_userblock_size(self):             # <<<<<<<<<<<<<<
 *     """Retrieves the size of a user block.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("tables.hdf5extension.File.get_userblock_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":712
 * 
 *   # Accessor definitions
 *   def _get_file_id(self):             # <<<<<<<<<<<<<<
 *     return self.file_id
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_9_get_file_id(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4File_9_get_file_id = {"_get_file_id", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_9_get_file_id, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_9_get_file_id(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_file_id (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_get_file_id", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_get_file_id", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4File_8_get_file_id(((struct __pyx_obj_6tables_13hdf5extension_File *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4File_8_get_file_id(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_file_id", 0);

  /* "tables/hdf5extension.pyx":713
 *   # Accessor definitions
 *   def _get_file_id(self):
 *     return self.file_id             # <<<<<<<<<<<<<<
 * 
 *   def fileno(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_hid_t(__pyx_v_self->file_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 713, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":712
 * 
 *   # Accessor definitions
 *   def _get_file_id(self):             # <<<<<<<<<<<<<<
 *     return self.file_id
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.hdf5extension.File._get_file_id", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":715
 *     return self.file_id
 * 
 *   def fileno(self):             # <<<<<<<<<<<<<<
 *     """Return the underlying OS integer file descriptor.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_11fileno(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_4File_10fileno, "Return the underlying OS integer file descriptor.\n\n    This is needed for lower-level file interfaces, such as the ``fcntl``\n    module.\n\n    ");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4File_11fileno = {"fileno", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_11fileno, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_4File_10fileno};
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_11fileno(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fileno (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("fileno", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("fileno", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4File_10fileno(((struct __pyx_obj_6tables_13hdf5extension_File *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4File_10fileno(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self) {
  void *__pyx_v_file_handle;
  uintptr_t *__pyx_v_descriptor;
  herr_t __pyx_v_err;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("fileno", 0);

  /* "tables/hdf5extension.pyx":726
 *     cdef uintptr_t *descriptor
 *     cdef herr_t err
 *     err = H5Fget_vfd_handle(self.file_id, H5P_DEFAULT, &file_handle)             # <<<<<<<<<<<<<<
 *     if err < 0:
 *       raise HDF5ExtError(
*/
  __pyx_v_err = H5Fget_vfd_handle(__pyx_v_self->file_id, H5P_DEFAULT, (&__pyx_v_file_handle));

  /* "tables/hdf5extension.pyx":727
 *     cdef herr_t err
 *     err = H5Fget_vfd_handle(self.file_id, H5P_DEFAULT, &file_handle)
 *     if err < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError(
 *         "Problems getting file descriptor for file ``%s``" % self.name)
*/
  __pyx_t_1 = (__pyx_v_err < 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/hdf5extension.pyx":728
 *     err = H5Fget_vfd_handle(self.file_id, H5P_DEFAULT, &file_handle)
 *     if err < 0:
 *       raise HDF5ExtError(             # <<<<<<<<<<<<<<
 *         "Problems getting file descriptor for file ``%s``" % self.name)
 *     # Convert the 'void *file_handle' into an 'int *descriptor'
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 728, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);

    /* "tables/hdf5extension.pyx":729
 *     if err < 0:
 *       raise HDF5ExtError(
 *         "Problems getting file descriptor for file ``%s``" % self.name)             # <<<<<<<<<<<<<<
 *     # Convert the 'void *file_handle' into an 'int *descriptor'
 *     descriptor = <uintptr_t *>file_handle
*/
    __pyx_t_5 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Problems_getting_file_descriptor, __pyx_v_self->name); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 729, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 728, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 728, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":727
 *     cdef herr_t err
 *     err = H5Fget_vfd_handle(self.file_id, H5P_DEFAULT, &file_handle)
 *     if err < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError(
 *         "Problems getting file descriptor for file ``%s``" % self.name)
*/
  }

  /* "tables/hdf5extension.pyx":731
 *         "Problems getting file descriptor for file ``%s``" % self.name)
 *     # Convert the 'void *file_handle' into an 'int *descriptor'
 *     descriptor = <uintptr_t *>file_handle             # <<<<<<<<<<<<<<
 *     return descriptor[0]
 * 
*/
  __pyx_v_descriptor = ((uintptr_t *)__pyx_v_file_handle);

  /* "tables/hdf5extension.pyx":732
 *     # Convert the 'void *file_handle' into an 'int *descriptor'
 *     descriptor = <uintptr_t *>file_handle
 *     return descriptor[0]             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyLong_From_uintptr_t((__pyx_v_descriptor[0])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 732, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":715
 *     return self.file_id
 * 
 *   def fileno(self):             # <<<<<<<<<<<<<<
 *     """Return the underlying OS integer file descriptor.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("tables.hdf5extension.File.fileno", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":735
 * 
 * 
 *   def _flush_file(self, scope):             # <<<<<<<<<<<<<<
 *     # Close the file
 *     H5Fflush(self.file_id, scope)
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_13_flush_file(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4File_13_flush_file = {"_flush_file", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_13_flush_file, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_13_flush_file(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_scope = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_flush_file (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_scope,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 735, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 735, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_flush_file", 0) < (0)) __PYX_ERR(0, 735, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_flush_file", 1, 1, 1, i); __PYX_ERR(0, 735, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 735, __pyx_L3_error)
    }
    __pyx_v_scope = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_flush_file", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 735, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.File._flush_file", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4File_12_flush_file(((struct __pyx_obj_6tables_13hdf5extension_File *)__pyx_v_self), __pyx_v_scope);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4File_12_flush_file(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self, PyObject *__pyx_v_scope) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  enum H5F_scope_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_flush_file", 0);

  /* "tables/hdf5extension.pyx":737
 *   def _flush_file(self, scope):
 *     # Close the file
 *     H5Fflush(self.file_id, scope)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_1 = ((enum H5F_scope_t)__Pyx_PyLong_As_enum__H5F_scope_t(__pyx_v_scope)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 737, __pyx_L1_error)
  (void)(H5Fflush(__pyx_v_self->file_id, __pyx_t_1));

  /* "tables/hdf5extension.pyx":735
 * 
 * 
 *   def _flush_file(self, scope):             # <<<<<<<<<<<<<<
 *     # Close the file
 *     H5Fflush(self.file_id, scope)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("tables.hdf5extension.File._flush_file", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":740
 * 
 * 
 *   def _close_file(self):             # <<<<<<<<<<<<<<
 *     # Close the file
 *     H5Fclose( self.file_id )
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_15_close_file(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4File_15_close_file = {"_close_file", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_15_close_file, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_15_close_file(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_close_file (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_close_file", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_close_file", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4File_14_close_file(((struct __pyx_obj_6tables_13hdf5extension_File *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4File_14_close_file(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_close_file", 0);

  /* "tables/hdf5extension.pyx":742
 *   def _close_file(self):
 *     # Close the file
 *     H5Fclose( self.file_id )             # <<<<<<<<<<<<<<
 *     self.file_id = 0    # Means file closed
 * 
*/
  (void)(H5Fclose(__pyx_v_self->file_id));

  /* "tables/hdf5extension.pyx":743
 *     # Close the file
 *     H5Fclose( self.file_id )
 *     self.file_id = 0    # Means file closed             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_v_self->file_id = 0;

  /* "tables/hdf5extension.pyx":740
 * 
 * 
 *   def _close_file(self):             # <<<<<<<<<<<<<<
 *     # Close the file
 *     H5Fclose( self.file_id )
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":748
 *   # This method is moved out of scope, until we provide code to delete
 *   # the memory booked by this extension types
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     cdef int ret
 *     if self.file_id > 0:
*/

/* Python wrapper */
static void __pyx_pw_6tables_13hdf5extension_4File_17__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_6tables_13hdf5extension_4File_17__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_6tables_13hdf5extension_4File_16__dealloc__(((struct __pyx_obj_6tables_13hdf5extension_File *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_6tables_13hdf5extension_4File_16__dealloc__(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self) {
  int __pyx_v_ret;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "tables/hdf5extension.pyx":750
 *   def __dealloc__(self):
 *     cdef int ret
 *     if self.file_id > 0:             # <<<<<<<<<<<<<<
 *       # Close the HDF5 file because user didn't do that!
 *       ret = H5Fclose(self.file_id)
*/
  __pyx_t_1 = (__pyx_v_self->file_id > 0);
  if (__pyx_t_1) {

    /* "tables/hdf5extension.pyx":752
 *     if self.file_id > 0:
 *       # Close the HDF5 file because user didn't do that!
 *       ret = H5Fclose(self.file_id)             # <<<<<<<<<<<<<<
 *       if ret < 0:
 *         raise HDF5ExtError("Problems closing the file '%s'" % self.name)
*/
    __pyx_v_ret = H5Fclose(__pyx_v_self->file_id);

    /* "tables/hdf5extension.pyx":753
 *       # Close the HDF5 file because user didn't do that!
 *       ret = H5Fclose(self.file_id)
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Problems closing the file '%s'" % self.name)
 * 
*/
    __pyx_t_1 = (__pyx_v_ret < 0);
    if (unlikely(__pyx_t_1)) {

      /* "tables/hdf5extension.pyx":754
 *       ret = H5Fclose(self.file_id)
 *       if ret < 0:
 *         raise HDF5ExtError("Problems closing the file '%s'" % self.name)             # <<<<<<<<<<<<<<
 * 
 * 
*/
      __pyx_t_3 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 754, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Problems_closing_the_file_s, __pyx_v_self->name); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 754, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
        assert(__pyx_t_3);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_5};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 754, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 754, __pyx_L1_error)

      /* "tables/hdf5extension.pyx":753
 *       # Close the HDF5 file because user didn't do that!
 *       ret = H5Fclose(self.file_id)
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Problems closing the file '%s'" % self.name)
 * 
*/
    }

    /* "tables/hdf5extension.pyx":750
 *   def __dealloc__(self):
 *     cdef int ret
 *     if self.file_id > 0:             # <<<<<<<<<<<<<<
 *       # Close the HDF5 file because user didn't do that!
 *       ret = H5Fclose(self.file_id)
*/
  }

  /* "tables/hdf5extension.pyx":748
 *   # This method is moved out of scope, until we provide code to delete
 *   # the memory booked by this extension types
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     cdef int ret
 *     if self.file_id > 0:
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("tables.hdf5extension.File.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_19__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4File_19__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_19__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_19__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4File_18__reduce_cython__(((struct __pyx_obj_6tables_13hdf5extension_File *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4File_18__reduce_cython__(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.access_plist, self.file_id, self.name)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = __Pyx_PyLong_From_hid_t(__pyx_v_self->access_plist); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_From_hid_t(__pyx_v_self->file_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1) != (0)) __PYX_ERR(2, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2) != (0)) __PYX_ERR(2, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->name);
  __Pyx_GIVEREF(__pyx_v_self->name);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_self->name) != (0)) __PYX_ERR(2, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.access_plist, self.file_id, self.name)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self.access_plist, self.file_id, self.name)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  if (__pyx_t_5) {
  } else {
    __pyx_t_4 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(2, 7, __pyx_L1_error)
  __pyx_t_4 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_4) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict) != (0)) __PYX_ERR(2, 8, __pyx_L1_error);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.name is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.access_plist, self.file_id, self.name)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.name is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_File, (type(self), 0x3498a4d, None), state
*/
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->name != Py_None);
    __pyx_v_use_setstate = __pyx_t_4;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_File, (type(self), 0x3498a4d, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self.name is not None
 *     if use_setstate:
 *         return __pyx_unpickle_File, (type(self), 0x3498a4d, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_File, (type(self), 0x3498a4d, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_File); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_55151181);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_55151181);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_mstate_global->__pyx_int_55151181) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_File, (type(self), 0x3498a4d, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_File, (type(self), 0x3498a4d, None), state
 *     else:
 *         return __pyx_unpickle_File, (type(self), 0x3498a4d, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_File__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_File); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_55151181);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_55151181);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_mstate_global->__pyx_int_55151181) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("tables.hdf5extension.File.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_File, (type(self), 0x3498a4d, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_File__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_21__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4File_21__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_21__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_4File_21__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(2, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(2, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(2, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(2, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.File.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4File_20__setstate_cython__(((struct __pyx_obj_6tables_13hdf5extension_File *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4File_20__setstate_cython__(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_File, (type(self), 0x3498a4d, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_File__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(2, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(2, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_6tables_13hdf5extension___pyx_unpickle_File__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_File, (type(self), 0x3498a4d, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_File__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.hdf5extension.File.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":760
 *   cdef object name
 * 
 *   def _g_new(self, node):             # <<<<<<<<<<<<<<
 *     self.name = node._v_name
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_12AttributeSet_1_g_new(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_12AttributeSet_1_g_new = {"_g_new", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_12AttributeSet_1_g_new, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_12AttributeSet_1_g_new(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_node = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_new (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_node,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 760, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 760, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_new", 0) < (0)) __PYX_ERR(0, 760, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_new", 1, 1, 1, i); __PYX_ERR(0, 760, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 760, __pyx_L3_error)
    }
    __pyx_v_node = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_new", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 760, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.AttributeSet._g_new", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_12AttributeSet__g_new(((struct __pyx_obj_6tables_13hdf5extension_AttributeSet *)__pyx_v_self), __pyx_v_node);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_12AttributeSet__g_new(struct __pyx_obj_6tables_13hdf5extension_AttributeSet *__pyx_v_self, PyObject *__pyx_v_node) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_new", 0);

  /* "tables/hdf5extension.pyx":761
 * 
 *   def _g_new(self, node):
 *     self.name = node._v_name             # <<<<<<<<<<<<<<
 * 
 *   def _g_list_attr(self, node):
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_node, __pyx_mstate_global->__pyx_n_u_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 761, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":760
 *   cdef object name
 * 
 *   def _g_new(self, node):             # <<<<<<<<<<<<<<
 *     self.name = node._v_name
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.hdf5extension.AttributeSet._g_new", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":763
 *     self.name = node._v_name
 * 
 *   def _g_list_attr(self, node):             # <<<<<<<<<<<<<<
 *     """Return a tuple with the attribute list"""
 *     a = Aiterate(node._v_objectid)
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_12AttributeSet_3_g_list_attr(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_12AttributeSet_2_g_list_attr, "Return a tuple with the attribute list");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_12AttributeSet_3_g_list_attr = {"_g_list_attr", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_12AttributeSet_3_g_list_attr, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_12AttributeSet_2_g_list_attr};
static PyObject *__pyx_pw_6tables_13hdf5extension_12AttributeSet_3_g_list_attr(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_node = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_list_attr (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_node,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 763, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 763, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_list_attr", 0) < (0)) __PYX_ERR(0, 763, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_list_attr", 1, 1, 1, i); __PYX_ERR(0, 763, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 763, __pyx_L3_error)
    }
    __pyx_v_node = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_list_attr", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 763, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.AttributeSet._g_list_attr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_12AttributeSet_2_g_list_attr(((struct __pyx_obj_6tables_13hdf5extension_AttributeSet *)__pyx_v_self), __pyx_v_node);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_12AttributeSet_2_g_list_attr(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_AttributeSet *__pyx_v_self, PyObject *__pyx_v_node) {
  PyObject *__pyx_v_a = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  hid_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_list_attr", 0);

  /* "tables/hdf5extension.pyx":765
 *   def _g_list_attr(self, node):
 *     """Return a tuple with the attribute list"""
 *     a = Aiterate(node._v_objectid)             # <<<<<<<<<<<<<<
 *     return a
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_node, __pyx_mstate_global->__pyx_n_u_v_objectid); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 765, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_2 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 765, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = Aiterate(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 765, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_a = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":766
 *     """Return a tuple with the attribute list"""
 *     a = Aiterate(node._v_objectid)
 *     return a             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_a);
  __pyx_r = __pyx_v_a;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":763
 *     self.name = node._v_name
 * 
 *   def _g_list_attr(self, node):             # <<<<<<<<<<<<<<
 *     """Return a tuple with the attribute list"""
 *     a = Aiterate(node._v_objectid)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.hdf5extension.AttributeSet._g_list_attr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":769
 * 
 * 
 *   def _g_setattr(self, node, name, object value):             # <<<<<<<<<<<<<<
 *     """Save Python or NumPy objects as HDF5 attributes.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_12AttributeSet_5_g_setattr(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_12AttributeSet_4_g_setattr, "Save Python or NumPy objects as HDF5 attributes.\n\n    Scalar Python objects, scalar NumPy & 0-dim NumPy objects will all be\n    saved as H5T_SCALAR type.  N-dim NumPy objects will be saved as H5T_ARRAY\n    type.\n\n    ");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_12AttributeSet_5_g_setattr = {"_g_setattr", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_12AttributeSet_5_g_setattr, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_12AttributeSet_4_g_setattr};
static PyObject *__pyx_pw_6tables_13hdf5extension_12AttributeSet_5_g_setattr(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_node = 0;
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_value = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_setattr (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_node,&__pyx_mstate_global->__pyx_n_u_name,&__pyx_mstate_global->__pyx_n_u_value,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 769, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 769, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 769, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 769, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_setattr", 0) < (0)) __PYX_ERR(0, 769, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_setattr", 1, 3, 3, i); __PYX_ERR(0, 769, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 769, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 769, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 769, __pyx_L3_error)
    }
    __pyx_v_node = values[0];
    __pyx_v_name = values[1];
    __pyx_v_value = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_setattr", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 769, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.AttributeSet._g_setattr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_12AttributeSet_4_g_setattr(((struct __pyx_obj_6tables_13hdf5extension_AttributeSet *)__pyx_v_self), __pyx_v_node, __pyx_v_name, __pyx_v_value);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_12AttributeSet_4_g_setattr(struct __pyx_obj_6tables_13hdf5extension_AttributeSet *__pyx_v_self, PyObject *__pyx_v_node, PyObject *__pyx_v_name, PyObject *__pyx_v_value) {
  int __pyx_v_ret;
  hid_t __pyx_v_dset_id;
  hid_t __pyx_v_type_id;
  hsize_t *__pyx_v_dims;
  PyArrayObject *__pyx_v_ndv = 0;
  PyObject *__pyx_v_byteorder = 0;
  PyObject *__pyx_v_rabyteorder = 0;
  PyObject *__pyx_v_baseatom = 0;
  char *__pyx_v_cname;
  PyObject *__pyx_v_encoded_name = 0;
  int __pyx_v_cset;
  PyObject *__pyx_v_description = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  char *__pyx_t_4;
  hid_t __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *(*__pyx_t_13)(PyObject *);
  hsize_t *__pyx_t_14;
  PyObject *__pyx_t_15[5];
  Py_ssize_t __pyx_t_16;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_setattr", 0);
  __Pyx_INCREF(__pyx_v_value);

  /* "tables/hdf5extension.pyx":783
 *     cdef ndarray ndv
 *     cdef object byteorder, rabyteorder, baseatom
 *     cdef char* cname = NULL             # <<<<<<<<<<<<<<
 *     cdef bytes encoded_name
 *     cdef int cset = H5T_CSET_DEFAULT
*/
  __pyx_v_cname = NULL;

  /* "tables/hdf5extension.pyx":785
 *     cdef char* cname = NULL
 *     cdef bytes encoded_name
 *     cdef int cset = H5T_CSET_DEFAULT             # <<<<<<<<<<<<<<
 * 
 *     encoded_name = name.encode('utf-8')
*/
  __pyx_v_cset = __pyx_v_6tables_13hdf5extension_H5T_CSET_DEFAULT;

  /* "tables/hdf5extension.pyx":787
 *     cdef int cset = H5T_CSET_DEFAULT
 * 
 *     encoded_name = name.encode('utf-8')             # <<<<<<<<<<<<<<
 *     # get the C pointer
 *     cname = encoded_name
*/
  __pyx_t_2 = __pyx_v_name;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 787, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 787, __pyx_L1_error)
  __pyx_v_encoded_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":789
 *     encoded_name = name.encode('utf-8')
 *     # get the C pointer
 *     cname = encoded_name             # <<<<<<<<<<<<<<
 * 
 *     # The dataset id of the node
*/
  if (unlikely(__pyx_v_encoded_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 789, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_name); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 789, __pyx_L1_error)
  __pyx_v_cname = __pyx_t_4;

  /* "tables/hdf5extension.pyx":792
 * 
 *     # The dataset id of the node
 *     dset_id = node._v_objectid             # <<<<<<<<<<<<<<
 * 
 *     # Convert a NumPy scalar into a NumPy 0-dim ndarray
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_node, __pyx_mstate_global->__pyx_n_u_v_objectid); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 792, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyLong_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_5 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 792, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_dset_id = __pyx_t_5;

  /* "tables/hdf5extension.pyx":795
 * 
 *     # Convert a NumPy scalar into a NumPy 0-dim ndarray
 *     if isinstance(value, np.generic):             # <<<<<<<<<<<<<<
 *       value = np.array(value)
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 795, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_generic); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 795, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = PyObject_IsInstance(__pyx_v_value, __pyx_t_2); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 795, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_6) {

    /* "tables/hdf5extension.pyx":796
 *     # Convert a NumPy scalar into a NumPy 0-dim ndarray
 *     if isinstance(value, np.generic):
 *       value = np.array(value)             # <<<<<<<<<<<<<<
 * 
 *     # Check if value is a NumPy ndarray and of a supported type
*/
    __pyx_t_1 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 796, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_array); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 796, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_8);
      assert(__pyx_t_1);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_v_value};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 796, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "tables/hdf5extension.pyx":795
 * 
 *     # Convert a NumPy scalar into a NumPy 0-dim ndarray
 *     if isinstance(value, np.generic):             # <<<<<<<<<<<<<<
 *       value = np.array(value)
 * 
*/
  }

  /* "tables/hdf5extension.pyx":799
 * 
 *     # Check if value is a NumPy ndarray and of a supported type
 *     if (isinstance(value, np.ndarray) and             # <<<<<<<<<<<<<<
 *         value.dtype.kind in ('V', 'S', 'b', 'i', 'u', 'f', 'c')):
 *       # get a contiguous array: fixes #270 and gh-176
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 799, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_ndarray); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 799, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_9 = PyObject_IsInstance(__pyx_v_value, __pyx_t_8); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 799, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__pyx_t_9) {
  } else {
    __pyx_t_6 = __pyx_t_9;
    goto __pyx_L5_bool_binop_done;
  }

  /* "tables/hdf5extension.pyx":800
 *     # Check if value is a NumPy ndarray and of a supported type
 *     if (isinstance(value, np.ndarray) and
 *         value.dtype.kind in ('V', 'S', 'b', 'i', 'u', 'f', 'c')):             # <<<<<<<<<<<<<<
 *       # get a contiguous array: fixes #270 and gh-176
 *       #value = np.ascontiguousarray(value)
*/
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 800, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 800, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_V, Py_EQ)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 800, __pyx_L1_error)
  if (!__pyx_t_10) {
  } else {
    __pyx_t_9 = __pyx_t_10;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_S, Py_EQ)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 800, __pyx_L1_error)
  if (!__pyx_t_10) {
  } else {
    __pyx_t_9 = __pyx_t_10;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_b, Py_EQ)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 800, __pyx_L1_error)
  if (!__pyx_t_10) {
  } else {
    __pyx_t_9 = __pyx_t_10;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_i, Py_EQ)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 800, __pyx_L1_error)
  if (!__pyx_t_10) {
  } else {
    __pyx_t_9 = __pyx_t_10;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_u, Py_EQ)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 800, __pyx_L1_error)
  if (!__pyx_t_10) {
  } else {
    __pyx_t_9 = __pyx_t_10;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_f, Py_EQ)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 800, __pyx_L1_error)
  if (!__pyx_t_10) {
  } else {
    __pyx_t_9 = __pyx_t_10;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_c, Py_EQ)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 800, __pyx_L1_error)
  __pyx_t_9 = __pyx_t_10;
  __pyx_L7_bool_binop_done:;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_10 = __pyx_t_9;
  __pyx_t_6 = __pyx_t_10;
  __pyx_L5_bool_binop_done:;

  /* "tables/hdf5extension.pyx":799
 * 
 *     # Check if value is a NumPy ndarray and of a supported type
 *     if (isinstance(value, np.ndarray) and             # <<<<<<<<<<<<<<
 *         value.dtype.kind in ('V', 'S', 'b', 'i', 'u', 'f', 'c')):
 *       # get a contiguous array: fixes #270 and gh-176
*/
  if (__pyx_t_6) {

    /* "tables/hdf5extension.pyx":803
 *       # get a contiguous array: fixes #270 and gh-176
 *       #value = np.ascontiguousarray(value)
 *       value = value.copy()             # <<<<<<<<<<<<<<
 *       if value.dtype.kind == 'V':
 *         description, rabyteorder = descr_from_dtype(value.dtype, ptparams=node._v_file.params)
*/
    __pyx_t_8 = __pyx_v_value;
    __Pyx_INCREF(__pyx_t_8);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_8, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_copy, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 803, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "tables/hdf5extension.pyx":804
 *       #value = np.ascontiguousarray(value)
 *       value = value.copy()
 *       if value.dtype.kind == 'V':             # <<<<<<<<<<<<<<
 *         description, rabyteorder = descr_from_dtype(value.dtype, ptparams=node._v_file.params)
 *         byteorder = byteorders[rabyteorder]
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 804, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 804, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_V, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 804, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_6) {

      /* "tables/hdf5extension.pyx":805
 *       value = value.copy()
 *       if value.dtype.kind == 'V':
 *         description, rabyteorder = descr_from_dtype(value.dtype, ptparams=node._v_file.params)             # <<<<<<<<<<<<<<
 *         byteorder = byteorders[rabyteorder]
 *         type_id = create_nested_type(description, byteorder)
*/
      __pyx_t_2 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_descr_from_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 805, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 805, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_node, __pyx_mstate_global->__pyx_n_u_v_file); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 805, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_params); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 805, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_3 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
        assert(__pyx_t_2);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
        __pyx_t_3 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, __pyx_t_7};
        __pyx_t_11 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 805, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_ptparams, __pyx_t_12, __pyx_t_11, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 805, __pyx_L1_error)
        __pyx_t_8 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_11);
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 805, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_8))) || (PyList_CheckExact(__pyx_t_8))) {
        PyObject* sequence = __pyx_t_8;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 805, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0);
          __Pyx_INCREF(__pyx_t_1);
          __pyx_t_11 = PyTuple_GET_ITEM(sequence, 1);
          __Pyx_INCREF(__pyx_t_11);
        } else {
          __pyx_t_1 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 805, __pyx_L1_error)
          __Pyx_XGOTREF(__pyx_t_1);
          __pyx_t_11 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 805, __pyx_L1_error)
          __Pyx_XGOTREF(__pyx_t_11);
        }
        #else
        __pyx_t_1 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 805, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_11 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 805, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        #endif
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_12 = PyObject_GetIter(__pyx_t_8); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 805, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_13 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_12);
        index = 0; __pyx_t_1 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_1)) goto __pyx_L15_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_1);
        index = 1; __pyx_t_11 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_11)) goto __pyx_L15_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_11);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_13(__pyx_t_12), 2) < (0)) __PYX_ERR(0, 805, __pyx_L1_error)
        __pyx_t_13 = NULL;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        goto __pyx_L16_unpacking_done;
        __pyx_L15_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __pyx_t_13 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 805, __pyx_L1_error)
        __pyx_L16_unpacking_done:;
      }
      __pyx_v_description = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_v_rabyteorder = __pyx_t_11;
      __pyx_t_11 = 0;

      /* "tables/hdf5extension.pyx":806
 *       if value.dtype.kind == 'V':
 *         description, rabyteorder = descr_from_dtype(value.dtype, ptparams=node._v_file.params)
 *         byteorder = byteorders[rabyteorder]             # <<<<<<<<<<<<<<
 *         type_id = create_nested_type(description, byteorder)
 *         # Make sure the value is consistent with offsets of the description
*/
      __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_byteorders); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 806, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_11 = __Pyx_PyObject_GetItem(__pyx_t_8, __pyx_v_rabyteorder); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 806, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_v_byteorder = __pyx_t_11;
      __pyx_t_11 = 0;

      /* "tables/hdf5extension.pyx":807
 *         description, rabyteorder = descr_from_dtype(value.dtype, ptparams=node._v_file.params)
 *         byteorder = byteorders[rabyteorder]
 *         type_id = create_nested_type(description, byteorder)             # <<<<<<<<<<<<<<
 *         # Make sure the value is consistent with offsets of the description
 *         value = value.astype(description._v_dtype)
*/
      __pyx_t_8 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_create_nested_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 807, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_3 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_1);
        assert(__pyx_t_8);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
        __pyx_t_3 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_8, __pyx_v_description, __pyx_v_byteorder};
        __pyx_t_11 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 807, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
      }
      __pyx_t_5 = __Pyx_PyLong_As_hid_t(__pyx_t_11); if (unlikely((__pyx_t_5 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 807, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_v_type_id = __pyx_t_5;

      /* "tables/hdf5extension.pyx":809
 *         type_id = create_nested_type(description, byteorder)
 *         # Make sure the value is consistent with offsets of the description
 *         value = value.astype(description._v_dtype)             # <<<<<<<<<<<<<<
 *       else:
 *         # Get the associated native HDF5 type of the scalar type
*/
      __pyx_t_1 = __pyx_v_value;
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_description, __pyx_mstate_global->__pyx_n_u_v_dtype); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 809, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_3 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_t_8};
        __pyx_t_11 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_astype, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 809, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
      }
      __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_11);
      __pyx_t_11 = 0;

      /* "tables/hdf5extension.pyx":804
 *       #value = np.ascontiguousarray(value)
 *       value = value.copy()
 *       if value.dtype.kind == 'V':             # <<<<<<<<<<<<<<
 *         description, rabyteorder = descr_from_dtype(value.dtype, ptparams=node._v_file.params)
 *         byteorder = byteorders[rabyteorder]
*/
      goto __pyx_L14;
    }

    /* "tables/hdf5extension.pyx":812
 *       else:
 *         # Get the associated native HDF5 type of the scalar type
 *         baseatom = Atom.from_dtype(value.dtype.base)             # <<<<<<<<<<<<<<
 *         byteorder = byteorders[value.dtype.byteorder]
 *         type_id = atom_to_hdf5_type(baseatom, byteorder)
*/
    /*else*/ {
      __pyx_t_8 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_Atom); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 812, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_from_dtype); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 812, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 812, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_base); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 812, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_3 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_12))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_12);
        assert(__pyx_t_8);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_12);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_12, __pyx__function);
        __pyx_t_3 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_t_7};
        __pyx_t_11 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_12, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 812, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
      }
      __pyx_v_baseatom = __pyx_t_11;
      __pyx_t_11 = 0;

      /* "tables/hdf5extension.pyx":813
 *         # Get the associated native HDF5 type of the scalar type
 *         baseatom = Atom.from_dtype(value.dtype.base)
 *         byteorder = byteorders[value.dtype.byteorder]             # <<<<<<<<<<<<<<
 *         type_id = atom_to_hdf5_type(baseatom, byteorder)
 *       # Get dimensionality info
*/
      __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_byteorders); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 813, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 813, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_byteorder); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 813, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_12 = __Pyx_PyObject_GetItem(__pyx_t_11, __pyx_t_7); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 813, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_v_byteorder = __pyx_t_12;
      __pyx_t_12 = 0;

      /* "tables/hdf5extension.pyx":814
 *         baseatom = Atom.from_dtype(value.dtype.base)
 *         byteorder = byteorders[value.dtype.byteorder]
 *         type_id = atom_to_hdf5_type(baseatom, byteorder)             # <<<<<<<<<<<<<<
 *       # Get dimensionality info
 *       ndv = <ndarray>value
*/
      __pyx_t_7 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_atom_to_hdf5_type); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 814, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_3 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_11))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_11);
        assert(__pyx_t_7);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
        __pyx_t_3 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_7, __pyx_v_baseatom, __pyx_v_byteorder};
        __pyx_t_12 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 814, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
      }
      __pyx_t_5 = __Pyx_PyLong_As_hid_t(__pyx_t_12); if (unlikely((__pyx_t_5 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 814, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_v_type_id = __pyx_t_5;
    }
    __pyx_L14:;

    /* "tables/hdf5extension.pyx":816
 *         type_id = atom_to_hdf5_type(baseatom, byteorder)
 *       # Get dimensionality info
 *       ndv = <ndarray>value             # <<<<<<<<<<<<<<
 *       dims = npy_malloc_dims(PyArray_NDIM(ndv), PyArray_DIMS(ndv))
 *       # Actually write the attribute
*/
    __pyx_t_12 = __pyx_v_value;
    __Pyx_INCREF(__pyx_t_12);
    __pyx_v_ndv = ((PyArrayObject *)__pyx_t_12);
    __pyx_t_12 = 0;

    /* "tables/hdf5extension.pyx":817
 *       # Get dimensionality info
 *       ndv = <ndarray>value
 *       dims = npy_malloc_dims(PyArray_NDIM(ndv), PyArray_DIMS(ndv))             # <<<<<<<<<<<<<<
 *       # Actually write the attribute
 *       ret = H5ATTRset_attribute(dset_id, cname, type_id,
*/
    __pyx_t_14 = __pyx_f_6tables_13hdf5extension_npy_malloc_dims(PyArray_NDIM(__pyx_v_ndv), PyArray_DIMS(__pyx_v_ndv)); if (unlikely(__pyx_t_14 == ((void *)NULL) && PyErr_Occurred())) __PYX_ERR(0, 817, __pyx_L1_error)
    __pyx_v_dims = __pyx_t_14;

    /* "tables/hdf5extension.pyx":819
 *       dims = npy_malloc_dims(PyArray_NDIM(ndv), PyArray_DIMS(ndv))
 *       # Actually write the attribute
 *       ret = H5ATTRset_attribute(dset_id, cname, type_id,             # <<<<<<<<<<<<<<
 *                                 PyArray_NDIM(ndv), dims, PyArray_BYTES(ndv))
 *       if ret < 0:
*/
    __pyx_v_ret = H5ATTRset_attribute(__pyx_v_dset_id, __pyx_v_cname, __pyx_v_type_id, PyArray_NDIM(__pyx_v_ndv), __pyx_v_dims, PyArray_BYTES(__pyx_v_ndv));

    /* "tables/hdf5extension.pyx":821
 *       ret = H5ATTRset_attribute(dset_id, cname, type_id,
 *                                 PyArray_NDIM(ndv), dims, PyArray_BYTES(ndv))
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Can't set attribute '%s' in node:\n %s." %
 *                            (name, self._v_node))
*/
    __pyx_t_6 = (__pyx_v_ret < 0);
    if (unlikely(__pyx_t_6)) {

      /* "tables/hdf5extension.pyx":822
 *                                 PyArray_NDIM(ndv), dims, PyArray_BYTES(ndv))
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in node:\n %s." %             # <<<<<<<<<<<<<<
 *                            (name, self._v_node))
 *       # Release resources
*/
      __pyx_t_11 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 822, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);

      /* "tables/hdf5extension.pyx":823
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in node:\n %s." %
 *                            (name, self._v_node))             # <<<<<<<<<<<<<<
 *       # Release resources
 *       free(<void *>dims)
*/
      __pyx_t_8 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 823, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_node); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 823, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_t_1), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 823, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_15[0] = __pyx_mstate_global->__pyx_kp_u_Can_t_set_attribute;
      __pyx_t_15[1] = __pyx_t_8;
      __pyx_t_15[2] = __pyx_mstate_global->__pyx_kp_u_in_node;
      __pyx_t_15[3] = __pyx_t_2;
      __pyx_t_15[4] = __pyx_mstate_global->__pyx_kp_u__3;

      /* "tables/hdf5extension.pyx":822
 *                                 PyArray_NDIM(ndv), dims, PyArray_BYTES(ndv))
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in node:\n %s." %             # <<<<<<<<<<<<<<
 *                            (name, self._v_node))
 *       # Release resources
*/
      __pyx_t_1 = __Pyx_PyUnicode_Join(__pyx_t_15, 5, 21 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8) + 12 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2));
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 822, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_3 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_7);
        assert(__pyx_t_11);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
        __pyx_t_3 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_t_1};
        __pyx_t_12 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 822, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
      }
      __Pyx_Raise(__pyx_t_12, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __PYX_ERR(0, 822, __pyx_L1_error)

      /* "tables/hdf5extension.pyx":821
 *       ret = H5ATTRset_attribute(dset_id, cname, type_id,
 *                                 PyArray_NDIM(ndv), dims, PyArray_BYTES(ndv))
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Can't set attribute '%s' in node:\n %s." %
 *                            (name, self._v_node))
*/
    }

    /* "tables/hdf5extension.pyx":825
 *                            (name, self._v_node))
 *       # Release resources
 *       free(<void *>dims)             # <<<<<<<<<<<<<<
 *       H5Tclose(type_id)
 *     else:
*/
    free(((void *)__pyx_v_dims));

    /* "tables/hdf5extension.pyx":826
 *       # Release resources
 *       free(<void *>dims)
 *       H5Tclose(type_id)             # <<<<<<<<<<<<<<
 *     else:
 *       # Object cannot be natively represented in HDF5.
*/
    (void)(H5Tclose(__pyx_v_type_id));

    /* "tables/hdf5extension.pyx":799
 * 
 *     # Check if value is a NumPy ndarray and of a supported type
 *     if (isinstance(value, np.ndarray) and             # <<<<<<<<<<<<<<
 *         value.dtype.kind in ('V', 'S', 'b', 'i', 'u', 'f', 'c')):
 *       # get a contiguous array: fixes #270 and gh-176
*/
    goto __pyx_L4;
  }

  /* "tables/hdf5extension.pyx":829
 *     else:
 *       # Object cannot be natively represented in HDF5.
 *       if (isinstance(value, np.ndarray) and             # <<<<<<<<<<<<<<
 *           value.dtype.kind == 'U' and
 *           value.shape == ()):
*/
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 829, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_ndarray); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 829, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_t_10 = PyObject_IsInstance(__pyx_v_value, __pyx_t_7); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(0, 829, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (__pyx_t_10) {
    } else {
      __pyx_t_6 = __pyx_t_10;
      goto __pyx_L19_bool_binop_done;
    }

    /* "tables/hdf5extension.pyx":830
 *       # Object cannot be natively represented in HDF5.
 *       if (isinstance(value, np.ndarray) and
 *           value.dtype.kind == 'U' and             # <<<<<<<<<<<<<<
 *           value.shape == ()):
 *         value = value[()].encode('utf-8')
*/
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 830, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 830, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_U, Py_EQ)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 830, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (__pyx_t_10) {
    } else {
      __pyx_t_6 = __pyx_t_10;
      goto __pyx_L19_bool_binop_done;
    }

    /* "tables/hdf5extension.pyx":831
 *       if (isinstance(value, np.ndarray) and
 *           value.dtype.kind == 'U' and
 *           value.shape == ()):             # <<<<<<<<<<<<<<
 *         value = value[()].encode('utf-8')
 *         cset = H5T_CSET_UTF8
*/
    __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 831, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_7 = PyObject_RichCompare(__pyx_t_12, __pyx_mstate_global->__pyx_empty_tuple, Py_EQ); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 831, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 831, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_6 = __pyx_t_10;
    __pyx_L19_bool_binop_done:;

    /* "tables/hdf5extension.pyx":829
 *     else:
 *       # Object cannot be natively represented in HDF5.
 *       if (isinstance(value, np.ndarray) and             # <<<<<<<<<<<<<<
 *           value.dtype.kind == 'U' and
 *           value.shape == ()):
*/
    if (__pyx_t_6) {

      /* "tables/hdf5extension.pyx":832
 *           value.dtype.kind == 'U' and
 *           value.shape == ()):
 *         value = value[()].encode('utf-8')             # <<<<<<<<<<<<<<
 *         cset = H5T_CSET_UTF8
 *       else:
*/
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_value, __pyx_mstate_global->__pyx_empty_tuple); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 832, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_12 = __pyx_t_1;
      __Pyx_INCREF(__pyx_t_12);
      __pyx_t_3 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_mstate_global->__pyx_kp_u_utf_8};
        __pyx_t_7 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 832, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
      }
      __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_7);
      __pyx_t_7 = 0;

      /* "tables/hdf5extension.pyx":833
 *           value.shape == ()):
 *         value = value[()].encode('utf-8')
 *         cset = H5T_CSET_UTF8             # <<<<<<<<<<<<<<
 *       else:
 *         # Convert this object to a null-terminated string
*/
      __pyx_v_cset = H5T_CSET_UTF8;

      /* "tables/hdf5extension.pyx":829
 *     else:
 *       # Object cannot be natively represented in HDF5.
 *       if (isinstance(value, np.ndarray) and             # <<<<<<<<<<<<<<
 *           value.dtype.kind == 'U' and
 *           value.shape == ()):
*/
      goto __pyx_L18;
    }

    /* "tables/hdf5extension.pyx":837
 *         # Convert this object to a null-terminated string
 *         # (binary pickles are not supported at this moment)
 *         value = pickle.dumps(value, 0)             # <<<<<<<<<<<<<<
 * 
 *       ret = H5ATTRset_attribute_string(dset_id, cname, value, len(value), cset)
*/
    /*else*/ {
      __pyx_t_1 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_pickle); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 837, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_dumps); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 837, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_3 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_11))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_11);
        assert(__pyx_t_1);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
        __pyx_t_3 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_1, __pyx_v_value, __pyx_mstate_global->__pyx_int_0};
        __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 837, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
      }
      __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_7);
      __pyx_t_7 = 0;
    }
    __pyx_L18:;

    /* "tables/hdf5extension.pyx":839
 *         value = pickle.dumps(value, 0)
 * 
 *       ret = H5ATTRset_attribute_string(dset_id, cname, value, len(value), cset)             # <<<<<<<<<<<<<<
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in node:\n %s." %
*/
    __pyx_t_4 = __Pyx_PyObject_AsWritableString(__pyx_v_value); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 839, __pyx_L1_error)
    __pyx_t_16 = PyObject_Length(__pyx_v_value); if (unlikely(__pyx_t_16 == ((Py_ssize_t)-1))) __PYX_ERR(0, 839, __pyx_L1_error)
    __pyx_v_ret = H5ATTRset_attribute_string(__pyx_v_dset_id, __pyx_v_cname, __pyx_t_4, __pyx_t_16, __pyx_v_cset);

    /* "tables/hdf5extension.pyx":840
 * 
 *       ret = H5ATTRset_attribute_string(dset_id, cname, value, len(value), cset)
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Can't set attribute '%s' in node:\n %s." %
 *                            (name, self._v_node))
*/
    __pyx_t_6 = (__pyx_v_ret < 0);
    if (unlikely(__pyx_t_6)) {

      /* "tables/hdf5extension.pyx":841
 *       ret = H5ATTRset_attribute_string(dset_id, cname, value, len(value), cset)
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in node:\n %s." %             # <<<<<<<<<<<<<<
 *                            (name, self._v_node))
 * 
*/
      __pyx_t_11 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 841, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);

      /* "tables/hdf5extension.pyx":842
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in node:\n %s." %
 *                            (name, self._v_node))             # <<<<<<<<<<<<<<
 * 
 * 
*/
      __pyx_t_12 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 842, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_node); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 842, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_8 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_t_2), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 842, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_15[0] = __pyx_mstate_global->__pyx_kp_u_Can_t_set_attribute;
      __pyx_t_15[1] = __pyx_t_12;
      __pyx_t_15[2] = __pyx_mstate_global->__pyx_kp_u_in_node;
      __pyx_t_15[3] = __pyx_t_8;
      __pyx_t_15[4] = __pyx_mstate_global->__pyx_kp_u__3;

      /* "tables/hdf5extension.pyx":841
 *       ret = H5ATTRset_attribute_string(dset_id, cname, value, len(value), cset)
 *       if ret < 0:
 *         raise HDF5ExtError("Can't set attribute '%s' in node:\n %s." %             # <<<<<<<<<<<<<<
 *                            (name, self._v_node))
 * 
*/
      __pyx_t_2 = __Pyx_PyUnicode_Join(__pyx_t_15, 5, 21 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_12) + 12 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_12) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8));
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 841, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_3 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_1);
        assert(__pyx_t_11);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
        __pyx_t_3 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_t_2};
        __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 841, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
      }
      __Pyx_Raise(__pyx_t_7, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __PYX_ERR(0, 841, __pyx_L1_error)

      /* "tables/hdf5extension.pyx":840
 * 
 *       ret = H5ATTRset_attribute_string(dset_id, cname, value, len(value), cset)
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Can't set attribute '%s' in node:\n %s." %
 *                            (name, self._v_node))
*/
    }
  }
  __pyx_L4:;

  /* "tables/hdf5extension.pyx":769
 * 
 * 
 *   def _g_setattr(self, node, name, object value):             # <<<<<<<<<<<<<<
 *     """Save Python or NumPy objects as HDF5 attributes.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("tables.hdf5extension.AttributeSet._g_setattr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_ndv);
  __Pyx_XDECREF(__pyx_v_byteorder);
  __Pyx_XDECREF(__pyx_v_rabyteorder);
  __Pyx_XDECREF(__pyx_v_baseatom);
  __Pyx_XDECREF(__pyx_v_encoded_name);
  __Pyx_XDECREF(__pyx_v_description);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":846
 * 
 *   # Get attributes
 *   def _g_getattr(self, node, attrname):             # <<<<<<<<<<<<<<
 *     """Get HDF5 attributes and retrieve them as NumPy objects.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_12AttributeSet_7_g_getattr(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_12AttributeSet_6_g_getattr, "Get HDF5 attributes and retrieve them as NumPy objects.\n\n    H5T_SCALAR types will be retrieved as scalar NumPy.\n    H5T_ARRAY types will be retrieved as ndarray NumPy objects.\n\n    ");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_12AttributeSet_7_g_getattr = {"_g_getattr", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_12AttributeSet_7_g_getattr, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_12AttributeSet_6_g_getattr};
static PyObject *__pyx_pw_6tables_13hdf5extension_12AttributeSet_7_g_getattr(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_node = 0;
  PyObject *__pyx_v_attrname = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_getattr (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_node,&__pyx_mstate_global->__pyx_n_u_attrname,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 846, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 846, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 846, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_getattr", 0) < (0)) __PYX_ERR(0, 846, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_getattr", 1, 2, 2, i); __PYX_ERR(0, 846, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 846, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 846, __pyx_L3_error)
    }
    __pyx_v_node = values[0];
    __pyx_v_attrname = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_getattr", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 846, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.AttributeSet._g_getattr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_12AttributeSet_6_g_getattr(((struct __pyx_obj_6tables_13hdf5extension_AttributeSet *)__pyx_v_self), __pyx_v_node, __pyx_v_attrname);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_12AttributeSet_6_g_getattr(struct __pyx_obj_6tables_13hdf5extension_AttributeSet *__pyx_v_self, PyObject *__pyx_v_node, PyObject *__pyx_v_attrname) {
  hsize_t *__pyx_v_dims;
  enum H5T_class_t __pyx_v_class_id;
  size_t __pyx_v_type_size;
  hid_t __pyx_v_dset_id;
  hid_t __pyx_v_type_id;
  int __pyx_v_rank;
  int __pyx_v_ret;
  void *__pyx_v_rbuf;
  char *__pyx_v_str_value;
  char **__pyx_v_str_values;
  PyArrayObject *__pyx_v_ndvalue = 0;
  PyObject *__pyx_v_shape = 0;
  PyObject *__pyx_v_retvalue = 0;
  int __pyx_v_i;
  int __pyx_v_nelements;
  char *__pyx_v_cattrname;
  PyObject *__pyx_v_encoded_attrname = 0;
  int __pyx_v_cset;
  PyObject *__pyx_v_dtype_ = NULL;
  PyObject *__pyx_v_stype_ = NULL;
  PyObject *__pyx_v_shape_ = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  char *__pyx_t_4;
  hid_t __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10[5];
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  int __pyx_t_13;
  PyObject *__pyx_t_14[6];
  PyObject *__pyx_t_15 = NULL;
  int __pyx_t_16;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *(*__pyx_t_20)(PyObject *);
  int __pyx_t_21;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  int __pyx_t_24;
  int __pyx_t_25;
  PyObject *__pyx_t_26 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_getattr", 0);

  /* "tables/hdf5extension.pyx":861
 *     cdef void *rbuf
 *     cdef char *str_value
 *     cdef char **str_values = NULL             # <<<<<<<<<<<<<<
 *     cdef ndarray ndvalue
 *     cdef object shape, stype_atom, shape_atom, retvalue
*/
  __pyx_v_str_values = NULL;

  /* "tables/hdf5extension.pyx":865
 *     cdef object shape, stype_atom, shape_atom, retvalue
 *     cdef int i, nelements
 *     cdef char* cattrname = NULL             # <<<<<<<<<<<<<<
 *     cdef bytes encoded_attrname
 *     cdef int cset = H5T_CSET_DEFAULT
*/
  __pyx_v_cattrname = NULL;

  /* "tables/hdf5extension.pyx":867
 *     cdef char* cattrname = NULL
 *     cdef bytes encoded_attrname
 *     cdef int cset = H5T_CSET_DEFAULT             # <<<<<<<<<<<<<<
 * 
 *     encoded_attrname = attrname.encode('utf-8')
*/
  __pyx_v_cset = __pyx_v_6tables_13hdf5extension_H5T_CSET_DEFAULT;

  /* "tables/hdf5extension.pyx":869
 *     cdef int cset = H5T_CSET_DEFAULT
 * 
 *     encoded_attrname = attrname.encode('utf-8')             # <<<<<<<<<<<<<<
 *     # Get the C pointer
 *     cattrname = encoded_attrname
*/
  __pyx_t_2 = __pyx_v_attrname;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 869, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 869, __pyx_L1_error)
  __pyx_v_encoded_attrname = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":871
 *     encoded_attrname = attrname.encode('utf-8')
 *     # Get the C pointer
 *     cattrname = encoded_attrname             # <<<<<<<<<<<<<<
 * 
 *     # The dataset id of the node
*/
  if (unlikely(__pyx_v_encoded_attrname == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 871, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_attrname); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 871, __pyx_L1_error)
  __pyx_v_cattrname = __pyx_t_4;

  /* "tables/hdf5extension.pyx":874
 * 
 *     # The dataset id of the node
 *     dset_id = node._v_objectid             # <<<<<<<<<<<<<<
 *     dims = NULL
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_node, __pyx_mstate_global->__pyx_n_u_v_objectid); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 874, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyLong_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_5 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 874, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_dset_id = __pyx_t_5;

  /* "tables/hdf5extension.pyx":875
 *     # The dataset id of the node
 *     dset_id = node._v_objectid
 *     dims = NULL             # <<<<<<<<<<<<<<
 * 
 *     ret = H5ATTRget_type_ndims(dset_id, cattrname, &type_id, &class_id,
*/
  __pyx_v_dims = NULL;

  /* "tables/hdf5extension.pyx":877
 *     dims = NULL
 * 
 *     ret = H5ATTRget_type_ndims(dset_id, cattrname, &type_id, &class_id,             # <<<<<<<<<<<<<<
 *                                &type_size, &rank )
 *     if ret < 0:
*/
  __pyx_v_ret = H5ATTRget_type_ndims(__pyx_v_dset_id, __pyx_v_cattrname, (&__pyx_v_type_id), (&__pyx_v_class_id), (&__pyx_v_type_size), (&__pyx_v_rank));

  /* "tables/hdf5extension.pyx":879
 *     ret = H5ATTRget_type_ndims(dset_id, cattrname, &type_id, &class_id,
 *                                &type_size, &rank )
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Can't get type info on attribute %s in node %s." %
 *                          (attrname, self.name))
*/
  __pyx_t_6 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_6)) {

    /* "tables/hdf5extension.pyx":880
 *                                &type_size, &rank )
 *     if ret < 0:
 *       raise HDF5ExtError("Can't get type info on attribute %s in node %s." %             # <<<<<<<<<<<<<<
 *                          (attrname, self.name))
 * 
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 880, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);

    /* "tables/hdf5extension.pyx":881
 *     if ret < 0:
 *       raise HDF5ExtError("Can't get type info on attribute %s in node %s." %
 *                          (attrname, self.name))             # <<<<<<<<<<<<<<
 * 
 *     # Call a fast function for scalar values and typical class types
*/
    __pyx_t_8 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_attrname), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 881, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_self->name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 881, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u_Can_t_get_type_info_on_attribute;
    __pyx_t_10[1] = __pyx_t_8;
    __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u_in_node_2;
    __pyx_t_10[3] = __pyx_t_9;
    __pyx_t_10[4] = __pyx_mstate_global->__pyx_kp_u__3;

    /* "tables/hdf5extension.pyx":880
 *                                &type_size, &rank )
 *     if ret < 0:
 *       raise HDF5ExtError("Can't get type info on attribute %s in node %s." %             # <<<<<<<<<<<<<<
 *                          (attrname, self.name))
 * 
*/
    __pyx_t_11 = __Pyx_PyUnicode_Join(__pyx_t_10, 5, 33 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8) + 9 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_9));
    if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 880, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_11};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 880, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 880, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":879
 *     ret = H5ATTRget_type_ndims(dset_id, cattrname, &type_id, &class_id,
 *                                &type_size, &rank )
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Can't get type info on attribute %s in node %s." %
 *                          (attrname, self.name))
*/
  }

  /* "tables/hdf5extension.pyx":884
 * 
 *     # Call a fast function for scalar values and typical class types
 *     if (rank == 0 and class_id == H5T_STRING):             # <<<<<<<<<<<<<<
 *       type_size = H5ATTRget_attribute_string(dset_id, cattrname, &str_value,
 *                                              &cset)
*/
  __pyx_t_12 = (__pyx_v_rank == 0);
  if (__pyx_t_12) {
  } else {
    __pyx_t_6 = __pyx_t_12;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_12 = (__pyx_v_class_id == H5T_STRING);
  __pyx_t_6 = __pyx_t_12;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_6) {

    /* "tables/hdf5extension.pyx":885
 *     # Call a fast function for scalar values and typical class types
 *     if (rank == 0 and class_id == H5T_STRING):
 *       type_size = H5ATTRget_attribute_string(dset_id, cattrname, &str_value,             # <<<<<<<<<<<<<<
 *                                              &cset)
 *       if type_size == 0:
*/
    __pyx_v_type_size = H5ATTRget_attribute_string(__pyx_v_dset_id, __pyx_v_cattrname, (&__pyx_v_str_value), (&__pyx_v_cset));

    /* "tables/hdf5extension.pyx":887
 *       type_size = H5ATTRget_attribute_string(dset_id, cattrname, &str_value,
 *                                              &cset)
 *       if type_size == 0:             # <<<<<<<<<<<<<<
 *         if cset == H5T_CSET_UTF8:
 *           retvalue = np.str_('')
*/
    __pyx_t_6 = (__pyx_v_type_size == 0);
    if (__pyx_t_6) {

      /* "tables/hdf5extension.pyx":888
 *                                              &cset)
 *       if type_size == 0:
 *         if cset == H5T_CSET_UTF8:             # <<<<<<<<<<<<<<
 *           retvalue = np.str_('')
 *         else:
*/
      __pyx_t_6 = (__pyx_v_cset == H5T_CSET_UTF8);
      if (__pyx_t_6) {

        /* "tables/hdf5extension.pyx":889
 *       if type_size == 0:
 *         if cset == H5T_CSET_UTF8:
 *           retvalue = np.str_('')             # <<<<<<<<<<<<<<
 *         else:
 *           retvalue = np.bytes_(b'')
*/
        __pyx_t_7 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 889, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_str); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 889, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_3 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_2))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
          assert(__pyx_t_7);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
          __pyx_t_3 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_};
          __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 889, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
        }
        __pyx_v_retvalue = __pyx_t_1;
        __pyx_t_1 = 0;

        /* "tables/hdf5extension.pyx":888
 *                                              &cset)
 *       if type_size == 0:
 *         if cset == H5T_CSET_UTF8:             # <<<<<<<<<<<<<<
 *           retvalue = np.str_('')
 *         else:
*/
        goto __pyx_L8;
      }

      /* "tables/hdf5extension.pyx":891
 *           retvalue = np.str_('')
 *         else:
 *           retvalue = np.bytes_(b'')             # <<<<<<<<<<<<<<
 * 
 *       elif cset == H5T_CSET_UTF8:
*/
      /*else*/ {
        __pyx_t_2 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 891, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_bytes); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 891, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_3 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_11))) {
          __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_11);
          assert(__pyx_t_2);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
          __pyx_t_3 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_b_};
          __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 891, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
        }
        __pyx_v_retvalue = __pyx_t_1;
        __pyx_t_1 = 0;
      }
      __pyx_L8:;

      /* "tables/hdf5extension.pyx":887
 *       type_size = H5ATTRget_attribute_string(dset_id, cattrname, &str_value,
 *                                              &cset)
 *       if type_size == 0:             # <<<<<<<<<<<<<<
 *         if cset == H5T_CSET_UTF8:
 *           retvalue = np.str_('')
*/
      goto __pyx_L7;
    }

    /* "tables/hdf5extension.pyx":893
 *           retvalue = np.bytes_(b'')
 * 
 *       elif cset == H5T_CSET_UTF8:             # <<<<<<<<<<<<<<
 *         retvalue = PyUnicode_DecodeUTF8(str_value, type_size, NULL)
 *         retvalue = np.str_(retvalue)
*/
    __pyx_t_6 = (__pyx_v_cset == H5T_CSET_UTF8);
    if (__pyx_t_6) {

      /* "tables/hdf5extension.pyx":894
 * 
 *       elif cset == H5T_CSET_UTF8:
 *         retvalue = PyUnicode_DecodeUTF8(str_value, type_size, NULL)             # <<<<<<<<<<<<<<
 *         retvalue = np.str_(retvalue)
 *       else:
*/
      __pyx_t_1 = PyUnicode_DecodeUTF8(__pyx_v_str_value, __pyx_v_type_size, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 894, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_retvalue = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "tables/hdf5extension.pyx":895
 *       elif cset == H5T_CSET_UTF8:
 *         retvalue = PyUnicode_DecodeUTF8(str_value, type_size, NULL)
 *         retvalue = np.str_(retvalue)             # <<<<<<<<<<<<<<
 *       else:
 *         retvalue = PyBytes_FromStringAndSize(str_value, type_size)
*/
      __pyx_t_11 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 895, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_str); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 895, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_3 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_7);
        assert(__pyx_t_11);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
        __pyx_t_3 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_v_retvalue};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 895, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_DECREF_SET(__pyx_v_retvalue, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "tables/hdf5extension.pyx":893
 *           retvalue = np.bytes_(b'')
 * 
 *       elif cset == H5T_CSET_UTF8:             # <<<<<<<<<<<<<<
 *         retvalue = PyUnicode_DecodeUTF8(str_value, type_size, NULL)
 *         retvalue = np.str_(retvalue)
*/
      goto __pyx_L7;
    }

    /* "tables/hdf5extension.pyx":897
 *         retvalue = np.str_(retvalue)
 *       else:
 *         retvalue = PyBytes_FromStringAndSize(str_value, type_size)             # <<<<<<<<<<<<<<
 *         # AV: oct 2012
 *         # since now we use the string size got form HDF5 we have to strip
*/
    /*else*/ {
      __pyx_t_1 = PyBytes_FromStringAndSize(__pyx_v_str_value, __pyx_v_type_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 897, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_retvalue = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "tables/hdf5extension.pyx":905
 *         # strlen(attr_value) is the actual length of the attibute
 *         # and np.bytes_(attr_value) can give a truncated pickle sting
 *         retvalue = retvalue.rstrip(b'\x00')             # <<<<<<<<<<<<<<
 *         retvalue = np.bytes_(retvalue)     # bytes
 *       # Important to release attr_value, because it has been malloc'ed!
*/
      __pyx_t_7 = __pyx_v_retvalue;
      __Pyx_INCREF(__pyx_t_7);
      __pyx_t_3 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_b__2};
        __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_rstrip, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 905, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_DECREF_SET(__pyx_v_retvalue, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "tables/hdf5extension.pyx":906
 *         # and np.bytes_(attr_value) can give a truncated pickle sting
 *         retvalue = retvalue.rstrip(b'\x00')
 *         retvalue = np.bytes_(retvalue)     # bytes             # <<<<<<<<<<<<<<
 *       # Important to release attr_value, because it has been malloc'ed!
 *       if str_value:
*/
      __pyx_t_7 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 906, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_bytes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 906, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_3 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
        assert(__pyx_t_7);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
        __pyx_t_3 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_v_retvalue};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 906, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_DECREF_SET(__pyx_v_retvalue, __pyx_t_1);
      __pyx_t_1 = 0;
    }
    __pyx_L7:;

    /* "tables/hdf5extension.pyx":908
 *         retvalue = np.bytes_(retvalue)     # bytes
 *       # Important to release attr_value, because it has been malloc'ed!
 *       if str_value:             # <<<<<<<<<<<<<<
 *         free(str_value)
 *       H5Tclose(type_id)
*/
    __pyx_t_6 = (__pyx_v_str_value != 0);
    if (__pyx_t_6) {

      /* "tables/hdf5extension.pyx":909
 *       # Important to release attr_value, because it has been malloc'ed!
 *       if str_value:
 *         free(str_value)             # <<<<<<<<<<<<<<
 *       H5Tclose(type_id)
 *       return retvalue
*/
      free(__pyx_v_str_value);

      /* "tables/hdf5extension.pyx":908
 *         retvalue = np.bytes_(retvalue)     # bytes
 *       # Important to release attr_value, because it has been malloc'ed!
 *       if str_value:             # <<<<<<<<<<<<<<
 *         free(str_value)
 *       H5Tclose(type_id)
*/
    }

    /* "tables/hdf5extension.pyx":910
 *       if str_value:
 *         free(str_value)
 *       H5Tclose(type_id)             # <<<<<<<<<<<<<<
 *       return retvalue
 *     elif (rank == 0 and class_id in (H5T_BITFIELD, H5T_INTEGER, H5T_FLOAT)):
*/
    (void)(H5Tclose(__pyx_v_type_id));

    /* "tables/hdf5extension.pyx":911
 *         free(str_value)
 *       H5Tclose(type_id)
 *       return retvalue             # <<<<<<<<<<<<<<
 *     elif (rank == 0 and class_id in (H5T_BITFIELD, H5T_INTEGER, H5T_FLOAT)):
 *       dtype_ = get_dtype_scalar(type_id, class_id, type_size)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_retvalue);
    __pyx_r = __pyx_v_retvalue;
    goto __pyx_L0;

    /* "tables/hdf5extension.pyx":884
 * 
 *     # Call a fast function for scalar values and typical class types
 *     if (rank == 0 and class_id == H5T_STRING):             # <<<<<<<<<<<<<<
 *       type_size = H5ATTRget_attribute_string(dset_id, cattrname, &str_value,
 *                                              &cset)
*/
  }

  /* "tables/hdf5extension.pyx":912
 *       H5Tclose(type_id)
 *       return retvalue
 *     elif (rank == 0 and class_id in (H5T_BITFIELD, H5T_INTEGER, H5T_FLOAT)):             # <<<<<<<<<<<<<<
 *       dtype_ = get_dtype_scalar(type_id, class_id, type_size)
 *       if dtype_ is None:
*/
  __pyx_t_12 = (__pyx_v_rank == 0);
  if (__pyx_t_12) {
  } else {
    __pyx_t_6 = __pyx_t_12;
    goto __pyx_L10_bool_binop_done;
  }
  switch (__pyx_v_class_id) {
    case H5T_BITFIELD:
    case H5T_INTEGER:
    case H5T_FLOAT:
    __pyx_t_12 = 1;
    break;
    default:
    __pyx_t_12 = 0;
    break;
  }
  __pyx_t_13 = __pyx_t_12;
  __pyx_t_6 = __pyx_t_13;
  __pyx_L10_bool_binop_done:;
  if (__pyx_t_6) {

    /* "tables/hdf5extension.pyx":913
 *       return retvalue
 *     elif (rank == 0 and class_id in (H5T_BITFIELD, H5T_INTEGER, H5T_FLOAT)):
 *       dtype_ = get_dtype_scalar(type_id, class_id, type_size)             # <<<<<<<<<<<<<<
 *       if dtype_ is None:
 *         warnings.warn("Unsupported type for attribute '%s' in node '%s'. "
*/
    __pyx_t_1 = __pyx_f_6tables_13hdf5extension_get_dtype_scalar(__pyx_v_type_id, __pyx_v_class_id, __pyx_v_type_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 913, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_dtype_ = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":914
 *     elif (rank == 0 and class_id in (H5T_BITFIELD, H5T_INTEGER, H5T_FLOAT)):
 *       dtype_ = get_dtype_scalar(type_id, class_id, type_size)
 *       if dtype_ is None:             # <<<<<<<<<<<<<<
 *         warnings.warn("Unsupported type for attribute '%s' in node '%s'. "
 *                       "Offending HDF5 class: %d" % (attrname, self.name,
*/
    __pyx_t_6 = (__pyx_v_dtype_ == Py_None);
    if (__pyx_t_6) {

      /* "tables/hdf5extension.pyx":915
 *       dtype_ = get_dtype_scalar(type_id, class_id, type_size)
 *       if dtype_ is None:
 *         warnings.warn("Unsupported type for attribute '%s' in node '%s'. "             # <<<<<<<<<<<<<<
 *                       "Offending HDF5 class: %d" % (attrname, self.name,
 *                                                     class_id), DataTypeWarning)
*/
      __pyx_t_2 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_warnings); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 915, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_warn); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 915, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "tables/hdf5extension.pyx":916
 *       if dtype_ is None:
 *         warnings.warn("Unsupported type for attribute '%s' in node '%s'. "
 *                       "Offending HDF5 class: %d" % (attrname, self.name,             # <<<<<<<<<<<<<<
 *                                                     class_id), DataTypeWarning)
 *         self._v_unimplemented.append(attrname)
*/
      __pyx_t_7 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_attrname), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 916, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_9 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_self->name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 916, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);

      /* "tables/hdf5extension.pyx":917
 *         warnings.warn("Unsupported type for attribute '%s' in node '%s'. "
 *                       "Offending HDF5 class: %d" % (attrname, self.name,
 *                                                     class_id), DataTypeWarning)             # <<<<<<<<<<<<<<
 *         self._v_unimplemented.append(attrname)
 *         return None
*/
      __pyx_t_8 = __Pyx_PyUnicode_From_enum__H5T_class_t(__pyx_v_class_id, 0, ' ', 'd'); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 917, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_14[0] = __pyx_mstate_global->__pyx_kp_u_Unsupported_type_for_attribute;
      __pyx_t_14[1] = __pyx_t_7;
      __pyx_t_14[2] = __pyx_mstate_global->__pyx_kp_u_in_node_3;
      __pyx_t_14[3] = __pyx_t_9;
      __pyx_t_14[4] = __pyx_mstate_global->__pyx_kp_u_Offending_HDF5_class;
      __pyx_t_14[5] = __pyx_t_8;

      /* "tables/hdf5extension.pyx":915
 *       dtype_ = get_dtype_scalar(type_id, class_id, type_size)
 *       if dtype_ is None:
 *         warnings.warn("Unsupported type for attribute '%s' in node '%s'. "             # <<<<<<<<<<<<<<
 *                       "Offending HDF5 class: %d" % (attrname, self.name,
 *                                                     class_id), DataTypeWarning)
*/
      __pyx_t_15 = __Pyx_PyUnicode_Join(__pyx_t_14, 6, 32 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7) + 11 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9) + 25 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_9) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8));
      if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 915, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "tables/hdf5extension.pyx":917
 *         warnings.warn("Unsupported type for attribute '%s' in node '%s'. "
 *                       "Offending HDF5 class: %d" % (attrname, self.name,
 *                                                     class_id), DataTypeWarning)             # <<<<<<<<<<<<<<
 *         self._v_unimplemented.append(attrname)
 *         return None
*/
      __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_DataTypeWarning); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 917, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_3 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_11))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_11);
        assert(__pyx_t_2);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
        __pyx_t_3 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_t_15, __pyx_t_8};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 915, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "tables/hdf5extension.pyx":918
 *                       "Offending HDF5 class: %d" % (attrname, self.name,
 *                                                     class_id), DataTypeWarning)
 *         self._v_unimplemented.append(attrname)             # <<<<<<<<<<<<<<
 *         return None
 *       shape = ()
*/
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_unimplemented); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 918, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_16 = __Pyx_PyObject_Append(__pyx_t_1, __pyx_v_attrname); if (unlikely(__pyx_t_16 == ((int)-1))) __PYX_ERR(0, 918, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "tables/hdf5extension.pyx":919
 *                                                     class_id), DataTypeWarning)
 *         self._v_unimplemented.append(attrname)
 *         return None             # <<<<<<<<<<<<<<
 *       shape = ()
 *     else:
*/
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = Py_None; __Pyx_INCREF(Py_None);
      goto __pyx_L0;

      /* "tables/hdf5extension.pyx":914
 *     elif (rank == 0 and class_id in (H5T_BITFIELD, H5T_INTEGER, H5T_FLOAT)):
 *       dtype_ = get_dtype_scalar(type_id, class_id, type_size)
 *       if dtype_ is None:             # <<<<<<<<<<<<<<
 *         warnings.warn("Unsupported type for attribute '%s' in node '%s'. "
 *                       "Offending HDF5 class: %d" % (attrname, self.name,
*/
    }

    /* "tables/hdf5extension.pyx":920
 *         self._v_unimplemented.append(attrname)
 *         return None
 *       shape = ()             # <<<<<<<<<<<<<<
 *     else:
 *       # General case
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_empty_tuple);
    __pyx_v_shape = __pyx_mstate_global->__pyx_empty_tuple;

    /* "tables/hdf5extension.pyx":912
 *       H5Tclose(type_id)
 *       return retvalue
 *     elif (rank == 0 and class_id in (H5T_BITFIELD, H5T_INTEGER, H5T_FLOAT)):             # <<<<<<<<<<<<<<
 *       dtype_ = get_dtype_scalar(type_id, class_id, type_size)
 *       if dtype_ is None:
*/
    goto __pyx_L4;
  }

  /* "tables/hdf5extension.pyx":925
 * 
 *       # Get the dimensional info
 *       dims = <hsize_t *>malloc(rank * sizeof(hsize_t))             # <<<<<<<<<<<<<<
 *       ret = H5ATTRget_dims(dset_id, cattrname, dims)
 *       if ret < 0:
*/
  /*else*/ {
    __pyx_v_dims = ((hsize_t *)malloc((__pyx_v_rank * (sizeof(hsize_t)))));

    /* "tables/hdf5extension.pyx":926
 *       # Get the dimensional info
 *       dims = <hsize_t *>malloc(rank * sizeof(hsize_t))
 *       ret = H5ATTRget_dims(dset_id, cattrname, dims)             # <<<<<<<<<<<<<<
 *       if ret < 0:
 *         raise HDF5ExtError("Can't get dims info on attribute %s in node %s." %
*/
    __pyx_v_ret = H5ATTRget_dims(__pyx_v_dset_id, __pyx_v_cattrname, __pyx_v_dims);

    /* "tables/hdf5extension.pyx":927
 *       dims = <hsize_t *>malloc(rank * sizeof(hsize_t))
 *       ret = H5ATTRget_dims(dset_id, cattrname, dims)
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Can't get dims info on attribute %s in node %s." %
 *                            (attrname, self.name))
*/
    __pyx_t_6 = (__pyx_v_ret < 0);
    if (unlikely(__pyx_t_6)) {

      /* "tables/hdf5extension.pyx":928
 *       ret = H5ATTRget_dims(dset_id, cattrname, dims)
 *       if ret < 0:
 *         raise HDF5ExtError("Can't get dims info on attribute %s in node %s." %             # <<<<<<<<<<<<<<
 *                            (attrname, self.name))
 *       shape = getshape(rank, dims)
*/
      __pyx_t_11 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 928, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);

      /* "tables/hdf5extension.pyx":929
 *       if ret < 0:
 *         raise HDF5ExtError("Can't get dims info on attribute %s in node %s." %
 *                            (attrname, self.name))             # <<<<<<<<<<<<<<
 *       shape = getshape(rank, dims)
 *       # dims is not needed anymore
*/
      __pyx_t_15 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_attrname), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 929, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      __pyx_t_2 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_self->name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 929, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u_Can_t_get_dims_info_on_attribute;
      __pyx_t_10[1] = __pyx_t_15;
      __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u_in_node_2;
      __pyx_t_10[3] = __pyx_t_2;
      __pyx_t_10[4] = __pyx_mstate_global->__pyx_kp_u__3;

      /* "tables/hdf5extension.pyx":928
 *       ret = H5ATTRget_dims(dset_id, cattrname, dims)
 *       if ret < 0:
 *         raise HDF5ExtError("Can't get dims info on attribute %s in node %s." %             # <<<<<<<<<<<<<<
 *                            (attrname, self.name))
 *       shape = getshape(rank, dims)
*/
      __pyx_t_9 = __Pyx_PyUnicode_Join(__pyx_t_10, 5, 33 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_15) + 9 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_15) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2));
      if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 928, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_3 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_8);
        assert(__pyx_t_11);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
        __pyx_t_3 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_t_9};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 928, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 928, __pyx_L1_error)

      /* "tables/hdf5extension.pyx":927
 *       dims = <hsize_t *>malloc(rank * sizeof(hsize_t))
 *       ret = H5ATTRget_dims(dset_id, cattrname, dims)
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Can't get dims info on attribute %s in node %s." %
 *                            (attrname, self.name))
*/
    }

    /* "tables/hdf5extension.pyx":930
 *         raise HDF5ExtError("Can't get dims info on attribute %s in node %s." %
 *                            (attrname, self.name))
 *       shape = getshape(rank, dims)             # <<<<<<<<<<<<<<
 *       # dims is not needed anymore
 *       free(<void *> dims)
*/
    __pyx_t_1 = __pyx_f_6tables_13hdf5extension_getshape(__pyx_v_rank, __pyx_v_dims); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 930, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_shape = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":932
 *       shape = getshape(rank, dims)
 *       # dims is not needed anymore
 *       free(<void *> dims)             # <<<<<<<<<<<<<<
 * 
 *       # Get the NumPy dtype from the type_id
*/
    free(((void *)__pyx_v_dims));

    /* "tables/hdf5extension.pyx":935
 * 
 *       # Get the NumPy dtype from the type_id
 *       try:             # <<<<<<<<<<<<<<
 *         stype_, shape_ = hdf5_to_np_ext_type(type_id, pure_numpy_types=True, ptparams=node._v_file.params)
 *         dtype_ = np.dtype((stype_, shape_))
*/
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_17, &__pyx_t_18, &__pyx_t_19);
      __Pyx_XGOTREF(__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_18);
      __Pyx_XGOTREF(__pyx_t_19);
      /*try:*/ {

        /* "tables/hdf5extension.pyx":936
 *       # Get the NumPy dtype from the type_id
 *       try:
 *         stype_, shape_ = hdf5_to_np_ext_type(type_id, pure_numpy_types=True, ptparams=node._v_file.params)             # <<<<<<<<<<<<<<
 *         dtype_ = np.dtype((stype_, shape_))
 *       except TypeError:
*/
        __pyx_t_8 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_hdf5_to_np_ext_type); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 936, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_11 = __Pyx_PyLong_From_hid_t(__pyx_v_type_id); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 936, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_node, __pyx_mstate_global->__pyx_n_u_v_file); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 936, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_params); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 936, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_15);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_3 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_9))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_9);
          assert(__pyx_t_8);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
          __pyx_t_3 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_8, __pyx_t_11};
          __pyx_t_2 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 936, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_2);
          if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_pure_numpy_types, Py_True, __pyx_t_2, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 936, __pyx_L14_error)
          if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_ptparams, __pyx_t_15, __pyx_t_2, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 936, __pyx_L14_error)
          __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_2);
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 936, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_1);
        }
        if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
          PyObject* sequence = __pyx_t_1;
          Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
          if (unlikely(size != 2)) {
            if (size > 2) __Pyx_RaiseTooManyValuesError(2);
            else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
            __PYX_ERR(0, 936, __pyx_L14_error)
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          if (likely(PyTuple_CheckExact(sequence))) {
            __pyx_t_9 = PyTuple_GET_ITEM(sequence, 0);
            __Pyx_INCREF(__pyx_t_9);
            __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1);
            __Pyx_INCREF(__pyx_t_2);
          } else {
            __pyx_t_9 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
            if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 936, __pyx_L14_error)
            __Pyx_XGOTREF(__pyx_t_9);
            __pyx_t_2 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
            if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 936, __pyx_L14_error)
            __Pyx_XGOTREF(__pyx_t_2);
          }
          #else
          __pyx_t_9 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 936, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_2 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 936, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        } else {
          Py_ssize_t index = -1;
          __pyx_t_15 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 936, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_15);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_20 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_15);
          index = 0; __pyx_t_9 = __pyx_t_20(__pyx_t_15); if (unlikely(!__pyx_t_9)) goto __pyx_L20_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_9);
          index = 1; __pyx_t_2 = __pyx_t_20(__pyx_t_15); if (unlikely(!__pyx_t_2)) goto __pyx_L20_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_2);
          if (__Pyx_IternextUnpackEndCheck(__pyx_t_20(__pyx_t_15), 2) < (0)) __PYX_ERR(0, 936, __pyx_L14_error)
          __pyx_t_20 = NULL;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          goto __pyx_L21_unpacking_done;
          __pyx_L20_unpacking_failed:;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __pyx_t_20 = NULL;
          if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
          __PYX_ERR(0, 936, __pyx_L14_error)
          __pyx_L21_unpacking_done:;
        }
        __pyx_v_stype_ = __pyx_t_9;
        __pyx_t_9 = 0;
        __pyx_v_shape_ = __pyx_t_2;
        __pyx_t_2 = 0;

        /* "tables/hdf5extension.pyx":937
 *       try:
 *         stype_, shape_ = hdf5_to_np_ext_type(type_id, pure_numpy_types=True, ptparams=node._v_file.params)
 *         dtype_ = np.dtype((stype_, shape_))             # <<<<<<<<<<<<<<
 *       except TypeError:
 *         if class_id == H5T_STRING and H5Tis_variable_str(type_id):
*/
        __pyx_t_2 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 937, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 937, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_15);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 937, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_INCREF(__pyx_v_stype_);
        __Pyx_GIVEREF(__pyx_v_stype_);
        if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_v_stype_) != (0)) __PYX_ERR(0, 937, __pyx_L14_error);
        __Pyx_INCREF(__pyx_v_shape_);
        __Pyx_GIVEREF(__pyx_v_shape_);
        if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_shape_) != (0)) __PYX_ERR(0, 937, __pyx_L14_error);
        __pyx_t_3 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_15);
          assert(__pyx_t_2);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_15);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_15, __pyx__function);
          __pyx_t_3 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_9};
          __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_15, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 937, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_1);
        }
        __pyx_v_dtype_ = __pyx_t_1;
        __pyx_t_1 = 0;

        /* "tables/hdf5extension.pyx":935
 * 
 *       # Get the NumPy dtype from the type_id
 *       try:             # <<<<<<<<<<<<<<
 *         stype_, shape_ = hdf5_to_np_ext_type(type_id, pure_numpy_types=True, ptparams=node._v_file.params)
 *         dtype_ = np.dtype((stype_, shape_))
*/
      }
      __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
      __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
      __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
      goto __pyx_L19_try_end;
      __pyx_L14_error:;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "tables/hdf5extension.pyx":938
 *         stype_, shape_ = hdf5_to_np_ext_type(type_id, pure_numpy_types=True, ptparams=node._v_file.params)
 *         dtype_ = np.dtype((stype_, shape_))
 *       except TypeError:             # <<<<<<<<<<<<<<
 *         if class_id == H5T_STRING and H5Tis_variable_str(type_id):
 *           nelements = H5ATTRget_attribute_vlen_string_array(dset_id, cattrname,
*/
      __pyx_t_21 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_TypeError))));
      if (__pyx_t_21) {
        __Pyx_AddTraceback("tables.hdf5extension.AttributeSet._g_getattr", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_15, &__pyx_t_9) < 0) __PYX_ERR(0, 938, __pyx_L16_except_error)
        __Pyx_XGOTREF(__pyx_t_1);
        __Pyx_XGOTREF(__pyx_t_15);
        __Pyx_XGOTREF(__pyx_t_9);

        /* "tables/hdf5extension.pyx":939
 *         dtype_ = np.dtype((stype_, shape_))
 *       except TypeError:
 *         if class_id == H5T_STRING and H5Tis_variable_str(type_id):             # <<<<<<<<<<<<<<
 *           nelements = H5ATTRget_attribute_vlen_string_array(dset_id, cattrname,
 *                                                             &str_values, &cset)
*/
        __pyx_t_13 = (__pyx_v_class_id == H5T_STRING);
        if (__pyx_t_13) {
        } else {
          __pyx_t_6 = __pyx_t_13;
          goto __pyx_L25_bool_binop_done;
        }
        __pyx_t_13 = (H5Tis_variable_str(__pyx_v_type_id) != 0);
        __pyx_t_6 = __pyx_t_13;
        __pyx_L25_bool_binop_done:;
        if (__pyx_t_6) {

          /* "tables/hdf5extension.pyx":940
 *       except TypeError:
 *         if class_id == H5T_STRING and H5Tis_variable_str(type_id):
 *           nelements = H5ATTRget_attribute_vlen_string_array(dset_id, cattrname,             # <<<<<<<<<<<<<<
 *                                                             &str_values, &cset)
 *           if nelements < 0:
*/
          __pyx_v_nelements = H5ATTRget_attribute_vlen_string_array(__pyx_v_dset_id, __pyx_v_cattrname, (&__pyx_v_str_values), (&__pyx_v_cset));

          /* "tables/hdf5extension.pyx":942
 *           nelements = H5ATTRget_attribute_vlen_string_array(dset_id, cattrname,
 *                                                             &str_values, &cset)
 *           if nelements < 0:             # <<<<<<<<<<<<<<
 *             raise HDF5ExtError("Can't read attribute %s in node %s." %
 *                                (attrname, self.name))
*/
          __pyx_t_6 = (__pyx_v_nelements < 0);
          if (unlikely(__pyx_t_6)) {

            /* "tables/hdf5extension.pyx":943
 *                                                             &str_values, &cset)
 *           if nelements < 0:
 *             raise HDF5ExtError("Can't read attribute %s in node %s." %             # <<<<<<<<<<<<<<
 *                                (attrname, self.name))
 * 
*/
            __pyx_t_11 = NULL;
            __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 943, __pyx_L16_except_error)
            __Pyx_GOTREF(__pyx_t_8);

            /* "tables/hdf5extension.pyx":944
 *           if nelements < 0:
 *             raise HDF5ExtError("Can't read attribute %s in node %s." %
 *                                (attrname, self.name))             # <<<<<<<<<<<<<<
 * 
 *           # The following generator expressions do not work with Cython 0.15.1
*/
            __pyx_t_7 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_attrname), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 944, __pyx_L16_except_error)
            __Pyx_GOTREF(__pyx_t_7);
            __pyx_t_22 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_self->name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 944, __pyx_L16_except_error)
            __Pyx_GOTREF(__pyx_t_22);
            __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u_Can_t_read_attribute;
            __pyx_t_10[1] = __pyx_t_7;
            __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u_in_node_2;
            __pyx_t_10[3] = __pyx_t_22;
            __pyx_t_10[4] = __pyx_mstate_global->__pyx_kp_u__3;

            /* "tables/hdf5extension.pyx":943
 *                                                             &str_values, &cset)
 *           if nelements < 0:
 *             raise HDF5ExtError("Can't read attribute %s in node %s." %             # <<<<<<<<<<<<<<
 *                                (attrname, self.name))
 * 
*/
            __pyx_t_23 = __Pyx_PyUnicode_Join(__pyx_t_10, 5, 21 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7) + 9 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_22) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_22));
            if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 943, __pyx_L16_except_error)
            __Pyx_GOTREF(__pyx_t_23);
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
            __pyx_t_3 = 1;
            #if CYTHON_UNPACK_METHODS
            if (unlikely(PyMethod_Check(__pyx_t_8))) {
              __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_8);
              assert(__pyx_t_11);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
              __Pyx_INCREF(__pyx_t_11);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
              __pyx_t_3 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_t_23};
              __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
              __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 943, __pyx_L16_except_error)
              __Pyx_GOTREF(__pyx_t_2);
            }
            __Pyx_Raise(__pyx_t_2, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __PYX_ERR(0, 943, __pyx_L16_except_error)

            /* "tables/hdf5extension.pyx":942
 *           nelements = H5ATTRget_attribute_vlen_string_array(dset_id, cattrname,
 *                                                             &str_values, &cset)
 *           if nelements < 0:             # <<<<<<<<<<<<<<
 *             raise HDF5ExtError("Can't read attribute %s in node %s." %
 *                                (attrname, self.name))
*/
          }

          /* "tables/hdf5extension.pyx":947
 * 
 *           # The following generator expressions do not work with Cython 0.15.1
 *           if cset == H5T_CSET_UTF8:             # <<<<<<<<<<<<<<
 *             #retvalue = np.fromiter(
 *             #  PyUnicode_DecodeUTF8(<char*>str_values[i],
*/
          __pyx_t_6 = (__pyx_v_cset == H5T_CSET_UTF8);
          if (__pyx_t_6) {

            /* "tables/hdf5extension.pyx":953
 *             #                        NULL)
 *             #    for i in range(nelements), "O8")
 *             retvalue = np.array([             # <<<<<<<<<<<<<<
 *               PyUnicode_DecodeUTF8(<char*>str_values[i],
 *                                     strlen(<char*>str_values[i]),
*/
            __pyx_t_8 = NULL;
            __Pyx_GetModuleGlobalName(__pyx_t_23, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 953, __pyx_L16_except_error)
            __Pyx_GOTREF(__pyx_t_23);
            __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_23, __pyx_mstate_global->__pyx_n_u_array); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 953, __pyx_L16_except_error)
            __Pyx_GOTREF(__pyx_t_11);
            __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
            __pyx_t_23 = PyList_New(0); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 953, __pyx_L16_except_error)
            __Pyx_GOTREF(__pyx_t_23);

            /* "tables/hdf5extension.pyx":957
 *                                     strlen(<char*>str_values[i]),
 *                                     NULL)
 *                 for i in range(nelements)], "O8")             # <<<<<<<<<<<<<<
 * 
 *           else:
*/
            __pyx_t_21 = __pyx_v_nelements;
            __pyx_t_24 = __pyx_t_21;
            for (__pyx_t_25 = 0; __pyx_t_25 < __pyx_t_24; __pyx_t_25+=1) {
              __pyx_v_i = __pyx_t_25;

              /* "tables/hdf5extension.pyx":954
 *             #    for i in range(nelements), "O8")
 *             retvalue = np.array([
 *               PyUnicode_DecodeUTF8(<char*>str_values[i],             # <<<<<<<<<<<<<<
 *                                     strlen(<char*>str_values[i]),
 *                                     NULL)
*/
              __pyx_t_22 = PyUnicode_DecodeUTF8(((char *)(__pyx_v_str_values[__pyx_v_i])), strlen(((char *)(__pyx_v_str_values[__pyx_v_i]))), NULL); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 954, __pyx_L16_except_error)
              __Pyx_GOTREF(__pyx_t_22);
              if (unlikely(__Pyx_ListComp_Append(__pyx_t_23, (PyObject*)__pyx_t_22))) __PYX_ERR(0, 953, __pyx_L16_except_error)
              __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
            }
            __pyx_t_3 = 1;
            #if CYTHON_UNPACK_METHODS
            if (unlikely(PyMethod_Check(__pyx_t_11))) {
              __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_11);
              assert(__pyx_t_8);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
              __Pyx_INCREF(__pyx_t_8);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
              __pyx_t_3 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[3] = {__pyx_t_8, __pyx_t_23, __pyx_mstate_global->__pyx_n_u_O8};
              __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
              __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
              if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 953, __pyx_L16_except_error)
              __Pyx_GOTREF(__pyx_t_2);
            }
            __pyx_v_retvalue = __pyx_t_2;
            __pyx_t_2 = 0;

            /* "tables/hdf5extension.pyx":947
 * 
 *           # The following generator expressions do not work with Cython 0.15.1
 *           if cset == H5T_CSET_UTF8:             # <<<<<<<<<<<<<<
 *             #retvalue = np.fromiter(
 *             #  PyUnicode_DecodeUTF8(<char*>str_values[i],
*/
            goto __pyx_L28;
          }

          /* "tables/hdf5extension.pyx":962
 *             #retvalue = np.fromiter(
 *             #  <char*>str_values[i] for i in range(nelements), "O8")
 *             retvalue = np.array(             # <<<<<<<<<<<<<<
 *               [<char*>str_values[i] for i in range(nelements)], "O8")
 *           retvalue.shape = shape
*/
          /*else*/ {
            __pyx_t_11 = NULL;
            __Pyx_GetModuleGlobalName(__pyx_t_23, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 962, __pyx_L16_except_error)
            __Pyx_GOTREF(__pyx_t_23);
            __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_23, __pyx_mstate_global->__pyx_n_u_array); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 962, __pyx_L16_except_error)
            __Pyx_GOTREF(__pyx_t_8);
            __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;

            /* "tables/hdf5extension.pyx":963
 *             #  <char*>str_values[i] for i in range(nelements), "O8")
 *             retvalue = np.array(
 *               [<char*>str_values[i] for i in range(nelements)], "O8")             # <<<<<<<<<<<<<<
 *           retvalue.shape = shape
 * 
*/
            __pyx_t_23 = PyList_New(0); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 963, __pyx_L16_except_error)
            __Pyx_GOTREF(__pyx_t_23);
            __pyx_t_21 = __pyx_v_nelements;
            __pyx_t_24 = __pyx_t_21;
            for (__pyx_t_25 = 0; __pyx_t_25 < __pyx_t_24; __pyx_t_25+=1) {
              __pyx_v_i = __pyx_t_25;
              __pyx_t_22 = __Pyx_PyBytes_FromString(((char *)(__pyx_v_str_values[__pyx_v_i]))); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 963, __pyx_L16_except_error)
              __Pyx_GOTREF(__pyx_t_22);
              if (unlikely(__Pyx_ListComp_Append(__pyx_t_23, (PyObject*)__pyx_t_22))) __PYX_ERR(0, 963, __pyx_L16_except_error)
              __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
            }
            __pyx_t_3 = 1;
            #if CYTHON_UNPACK_METHODS
            if (unlikely(PyMethod_Check(__pyx_t_8))) {
              __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_8);
              assert(__pyx_t_11);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
              __Pyx_INCREF(__pyx_t_11);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
              __pyx_t_3 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[3] = {__pyx_t_11, __pyx_t_23, __pyx_mstate_global->__pyx_n_u_O8};
              __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
              __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 962, __pyx_L16_except_error)
              __Pyx_GOTREF(__pyx_t_2);
            }
            __pyx_v_retvalue = __pyx_t_2;
            __pyx_t_2 = 0;
          }
          __pyx_L28:;

          /* "tables/hdf5extension.pyx":964
 *             retvalue = np.array(
 *               [<char*>str_values[i] for i in range(nelements)], "O8")
 *           retvalue.shape = shape             # <<<<<<<<<<<<<<
 * 
 *           # Important to release attr_value, because it has been malloc'ed!
*/
          if (__Pyx_PyObject_SetAttrStr(__pyx_v_retvalue, __pyx_mstate_global->__pyx_n_u_shape, __pyx_v_shape) < (0)) __PYX_ERR(0, 964, __pyx_L16_except_error)

          /* "tables/hdf5extension.pyx":967
 * 
 *           # Important to release attr_value, because it has been malloc'ed!
 *           for i in range(nelements):             # <<<<<<<<<<<<<<
 *             free(str_values[i])
 *           free(str_values)
*/
          __pyx_t_21 = __pyx_v_nelements;
          __pyx_t_24 = __pyx_t_21;
          for (__pyx_t_25 = 0; __pyx_t_25 < __pyx_t_24; __pyx_t_25+=1) {
            __pyx_v_i = __pyx_t_25;

            /* "tables/hdf5extension.pyx":968
 *           # Important to release attr_value, because it has been malloc'ed!
 *           for i in range(nelements):
 *             free(str_values[i])             # <<<<<<<<<<<<<<
 *           free(str_values)
 * 
*/
            free((__pyx_v_str_values[__pyx_v_i]));
          }

          /* "tables/hdf5extension.pyx":969
 *           for i in range(nelements):
 *             free(str_values[i])
 *           free(str_values)             # <<<<<<<<<<<<<<
 * 
 *           return retvalue
*/
          free(__pyx_v_str_values);

          /* "tables/hdf5extension.pyx":971
 *           free(str_values)
 * 
 *           return retvalue             # <<<<<<<<<<<<<<
 * 
 *         # This class is not supported. Instead of raising a TypeError, issue a
*/
          __Pyx_XDECREF(__pyx_r);
          __Pyx_INCREF(__pyx_v_retvalue);
          __pyx_r = __pyx_v_retvalue;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          goto __pyx_L17_except_return;

          /* "tables/hdf5extension.pyx":939
 *         dtype_ = np.dtype((stype_, shape_))
 *       except TypeError:
 *         if class_id == H5T_STRING and H5Tis_variable_str(type_id):             # <<<<<<<<<<<<<<
 *           nelements = H5ATTRget_attribute_vlen_string_array(dset_id, cattrname,
 *                                                             &str_values, &cset)
*/
        }

        /* "tables/hdf5extension.pyx":976
 *         # warning explaining the problem. This will allow to continue browsing
 *         # native HDF5 files, while informing the user about the problem.
 *         warnings.warn("Unsupported type for attribute '%s' in node '%s'. "             # <<<<<<<<<<<<<<
 *                       "Offending HDF5 class: %d" % (attrname, self.name,
 *                                                     class_id), DataTypeWarning)
*/
        __pyx_t_8 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_23, __pyx_mstate_global->__pyx_n_u_warnings); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 976, __pyx_L16_except_error)
        __Pyx_GOTREF(__pyx_t_23);
        __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_23, __pyx_mstate_global->__pyx_n_u_warn); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 976, __pyx_L16_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;

        /* "tables/hdf5extension.pyx":977
 *         # native HDF5 files, while informing the user about the problem.
 *         warnings.warn("Unsupported type for attribute '%s' in node '%s'. "
 *                       "Offending HDF5 class: %d" % (attrname, self.name,             # <<<<<<<<<<<<<<
 *                                                     class_id), DataTypeWarning)
 *         self._v_unimplemented.append(attrname)
*/
        __pyx_t_23 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_attrname), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 977, __pyx_L16_except_error)
        __Pyx_GOTREF(__pyx_t_23);
        __pyx_t_22 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_self->name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 977, __pyx_L16_except_error)
        __Pyx_GOTREF(__pyx_t_22);

        /* "tables/hdf5extension.pyx":978
 *         warnings.warn("Unsupported type for attribute '%s' in node '%s'. "
 *                       "Offending HDF5 class: %d" % (attrname, self.name,
 *                                                     class_id), DataTypeWarning)             # <<<<<<<<<<<<<<
 *         self._v_unimplemented.append(attrname)
 *         return None
*/
        __pyx_t_7 = __Pyx_PyUnicode_From_enum__H5T_class_t(__pyx_v_class_id, 0, ' ', 'd'); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 978, __pyx_L16_except_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_14[0] = __pyx_mstate_global->__pyx_kp_u_Unsupported_type_for_attribute;
        __pyx_t_14[1] = __pyx_t_23;
        __pyx_t_14[2] = __pyx_mstate_global->__pyx_kp_u_in_node_3;
        __pyx_t_14[3] = __pyx_t_22;
        __pyx_t_14[4] = __pyx_mstate_global->__pyx_kp_u_Offending_HDF5_class;
        __pyx_t_14[5] = __pyx_t_7;

        /* "tables/hdf5extension.pyx":976
 *         # warning explaining the problem. This will allow to continue browsing
 *         # native HDF5 files, while informing the user about the problem.
 *         warnings.warn("Unsupported type for attribute '%s' in node '%s'. "             # <<<<<<<<<<<<<<
 *                       "Offending HDF5 class: %d" % (attrname, self.name,
 *                                                     class_id), DataTypeWarning)
*/
        __pyx_t_26 = __Pyx_PyUnicode_Join(__pyx_t_14, 6, 32 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_23) + 11 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_22) + 25 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_23) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_22) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7));
        if (unlikely(!__pyx_t_26)) __PYX_ERR(0, 976, __pyx_L16_except_error)
        __Pyx_GOTREF(__pyx_t_26);
        __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
        __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "tables/hdf5extension.pyx":978
 *         warnings.warn("Unsupported type for attribute '%s' in node '%s'. "
 *                       "Offending HDF5 class: %d" % (attrname, self.name,
 *                                                     class_id), DataTypeWarning)             # <<<<<<<<<<<<<<
 *         self._v_unimplemented.append(attrname)
 *         return None
*/
        __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_DataTypeWarning); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 978, __pyx_L16_except_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_3 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_11))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_11);
          assert(__pyx_t_8);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
          __pyx_t_3 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_8, __pyx_t_26, __pyx_t_7};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_26); __pyx_t_26 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 976, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "tables/hdf5extension.pyx":979
 *                       "Offending HDF5 class: %d" % (attrname, self.name,
 *                                                     class_id), DataTypeWarning)
 *         self._v_unimplemented.append(attrname)             # <<<<<<<<<<<<<<
 *         return None
 * 
*/
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_unimplemented); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 979, __pyx_L16_except_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_16 = __Pyx_PyObject_Append(__pyx_t_2, __pyx_v_attrname); if (unlikely(__pyx_t_16 == ((int)-1))) __PYX_ERR(0, 979, __pyx_L16_except_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "tables/hdf5extension.pyx":980
 *                                                     class_id), DataTypeWarning)
 *         self._v_unimplemented.append(attrname)
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     # Get the container for data
*/
        __Pyx_XDECREF(__pyx_r);
        __pyx_r = Py_None; __Pyx_INCREF(Py_None);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        goto __pyx_L17_except_return;
      }
      goto __pyx_L16_except_error;

      /* "tables/hdf5extension.pyx":935
 * 
 *       # Get the NumPy dtype from the type_id
 *       try:             # <<<<<<<<<<<<<<
 *         stype_, shape_ = hdf5_to_np_ext_type(type_id, pure_numpy_types=True, ptparams=node._v_file.params)
 *         dtype_ = np.dtype((stype_, shape_))
*/
      __pyx_L16_except_error:;
      __Pyx_XGIVEREF(__pyx_t_17);
      __Pyx_XGIVEREF(__pyx_t_18);
      __Pyx_XGIVEREF(__pyx_t_19);
      __Pyx_ExceptionReset(__pyx_t_17, __pyx_t_18, __pyx_t_19);
      goto __pyx_L1_error;
      __pyx_L17_except_return:;
      __Pyx_XGIVEREF(__pyx_t_17);
      __Pyx_XGIVEREF(__pyx_t_18);
      __Pyx_XGIVEREF(__pyx_t_19);
      __Pyx_ExceptionReset(__pyx_t_17, __pyx_t_18, __pyx_t_19);
      goto __pyx_L0;
      __pyx_L19_try_end:;
    }
  }
  __pyx_L4:;

  /* "tables/hdf5extension.pyx":983
 * 
 *     # Get the container for data
 *     ndvalue = np.empty(dtype=dtype_, shape=shape)             # <<<<<<<<<<<<<<
 *     # Get the pointer to the buffer data area
 *     rbuf = PyArray_DATA(ndvalue)
*/
  __pyx_t_15 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 983, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 983, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_15);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_15);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_15, NULL};
    __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 983, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_v_dtype_, __pyx_t_1, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 983, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_shape, __pyx_v_shape, __pyx_t_1, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 983, __pyx_L1_error)
    __pyx_t_9 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 983, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
  }
  if (!(likely(((__pyx_t_9) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_9, __pyx_mstate_global->__pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 983, __pyx_L1_error)
  __pyx_v_ndvalue = ((PyArrayObject *)__pyx_t_9);
  __pyx_t_9 = 0;

  /* "tables/hdf5extension.pyx":985
 *     ndvalue = np.empty(dtype=dtype_, shape=shape)
 *     # Get the pointer to the buffer data area
 *     rbuf = PyArray_DATA(ndvalue)             # <<<<<<<<<<<<<<
 *     # Actually read the attribute from disk
 *     ret = H5ATTRget_attribute(dset_id, cattrname, type_id, rbuf)
*/
  __pyx_v_rbuf = PyArray_DATA(__pyx_v_ndvalue);

  /* "tables/hdf5extension.pyx":987
 *     rbuf = PyArray_DATA(ndvalue)
 *     # Actually read the attribute from disk
 *     ret = H5ATTRget_attribute(dset_id, cattrname, type_id, rbuf)             # <<<<<<<<<<<<<<
 *     if ret < 0:
 *       raise HDF5ExtError("Attribute %s exists in node %s, but can't get it." %
*/
  __pyx_v_ret = H5ATTRget_attribute(__pyx_v_dset_id, __pyx_v_cattrname, __pyx_v_type_id, __pyx_v_rbuf);

  /* "tables/hdf5extension.pyx":988
 *     # Actually read the attribute from disk
 *     ret = H5ATTRget_attribute(dset_id, cattrname, type_id, rbuf)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Attribute %s exists in node %s, but can't get it." %
 *                          (attrname, self.name))
*/
  __pyx_t_6 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_6)) {

    /* "tables/hdf5extension.pyx":989
 *     ret = H5ATTRget_attribute(dset_id, cattrname, type_id, rbuf)
 *     if ret < 0:
 *       raise HDF5ExtError("Attribute %s exists in node %s, but can't get it." %             # <<<<<<<<<<<<<<
 *                          (attrname, self.name))
 *     H5Tclose(type_id)
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 989, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "tables/hdf5extension.pyx":990
 *     if ret < 0:
 *       raise HDF5ExtError("Attribute %s exists in node %s, but can't get it." %
 *                          (attrname, self.name))             # <<<<<<<<<<<<<<
 *     H5Tclose(type_id)
 * 
*/
    __pyx_t_15 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_attrname), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 990, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __pyx_t_11 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_self->name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 990, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u_Attribute;
    __pyx_t_10[1] = __pyx_t_15;
    __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u_exists_in_node;
    __pyx_t_10[3] = __pyx_t_11;
    __pyx_t_10[4] = __pyx_mstate_global->__pyx_kp_u_but_can_t_get_it;

    /* "tables/hdf5extension.pyx":989
 *     ret = H5ATTRget_attribute(dset_id, cattrname, type_id, rbuf)
 *     if ret < 0:
 *       raise HDF5ExtError("Attribute %s exists in node %s, but can't get it." %             # <<<<<<<<<<<<<<
 *                          (attrname, self.name))
 *     H5Tclose(type_id)
*/
    __pyx_t_7 = __Pyx_PyUnicode_Join(__pyx_t_10, 5, 10 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_15) + 16 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_11) + 19, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_15) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_11));
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 989, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_7};
      __pyx_t_9 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 989, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
    }
    __Pyx_Raise(__pyx_t_9, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __PYX_ERR(0, 989, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":988
 *     # Actually read the attribute from disk
 *     ret = H5ATTRget_attribute(dset_id, cattrname, type_id, rbuf)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Attribute %s exists in node %s, but can't get it." %
 *                          (attrname, self.name))
*/
  }

  /* "tables/hdf5extension.pyx":991
 *       raise HDF5ExtError("Attribute %s exists in node %s, but can't get it." %
 *                          (attrname, self.name))
 *     H5Tclose(type_id)             # <<<<<<<<<<<<<<
 * 
 *     if rank > 0:    # multidimensional case
*/
  (void)(H5Tclose(__pyx_v_type_id));

  /* "tables/hdf5extension.pyx":993
 *     H5Tclose(type_id)
 * 
 *     if rank > 0:    # multidimensional case             # <<<<<<<<<<<<<<
 *       retvalue = ndvalue
 *     else:
*/
  __pyx_t_6 = (__pyx_v_rank > 0);
  if (__pyx_t_6) {

    /* "tables/hdf5extension.pyx":994
 * 
 *     if rank > 0:    # multidimensional case
 *       retvalue = ndvalue             # <<<<<<<<<<<<<<
 *     else:
 *       retvalue = ndvalue[()]   # 0-dim ndarray becomes a NumPy scalar
*/
    __Pyx_INCREF((PyObject *)__pyx_v_ndvalue);
    __pyx_v_retvalue = ((PyObject *)__pyx_v_ndvalue);

    /* "tables/hdf5extension.pyx":993
 *     H5Tclose(type_id)
 * 
 *     if rank > 0:    # multidimensional case             # <<<<<<<<<<<<<<
 *       retvalue = ndvalue
 *     else:
*/
    goto __pyx_L36;
  }

  /* "tables/hdf5extension.pyx":996
 *       retvalue = ndvalue
 *     else:
 *       retvalue = ndvalue[()]   # 0-dim ndarray becomes a NumPy scalar             # <<<<<<<<<<<<<<
 * 
 *     return retvalue
*/
  /*else*/ {
    __pyx_t_9 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_ndvalue), __pyx_mstate_global->__pyx_empty_tuple); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 996, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_retvalue = __pyx_t_9;
    __pyx_t_9 = 0;
  }
  __pyx_L36:;

  /* "tables/hdf5extension.pyx":998
 *       retvalue = ndvalue[()]   # 0-dim ndarray becomes a NumPy scalar
 * 
 *     return retvalue             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_retvalue);
  __pyx_r = __pyx_v_retvalue;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":846
 * 
 *   # Get attributes
 *   def _g_getattr(self, node, attrname):             # <<<<<<<<<<<<<<
 *     """Get HDF5 attributes and retrieve them as NumPy objects.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_22);
  __Pyx_XDECREF(__pyx_t_23);
  __Pyx_XDECREF(__pyx_t_26);
  __Pyx_AddTraceback("tables.hdf5extension.AttributeSet._g_getattr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_ndvalue);
  __Pyx_XDECREF(__pyx_v_shape);
  __Pyx_XDECREF(__pyx_v_retvalue);
  __Pyx_XDECREF(__pyx_v_encoded_attrname);
  __Pyx_XDECREF(__pyx_v_dtype_);
  __Pyx_XDECREF(__pyx_v_stype_);
  __Pyx_XDECREF(__pyx_v_shape_);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1001
 * 
 * 
 *   def _g_remove(self, node, attrname):             # <<<<<<<<<<<<<<
 *     cdef int ret
 *     cdef hid_t dset_id
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_12AttributeSet_9_g_remove(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_12AttributeSet_9_g_remove = {"_g_remove", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_12AttributeSet_9_g_remove, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_12AttributeSet_9_g_remove(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_node = 0;
  PyObject *__pyx_v_attrname = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_remove (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_node,&__pyx_mstate_global->__pyx_n_u_attrname,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1001, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1001, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1001, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_remove", 0) < (0)) __PYX_ERR(0, 1001, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_remove", 1, 2, 2, i); __PYX_ERR(0, 1001, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1001, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1001, __pyx_L3_error)
    }
    __pyx_v_node = values[0];
    __pyx_v_attrname = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_remove", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1001, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.AttributeSet._g_remove", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_12AttributeSet_8_g_remove(((struct __pyx_obj_6tables_13hdf5extension_AttributeSet *)__pyx_v_self), __pyx_v_node, __pyx_v_attrname);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_12AttributeSet_8_g_remove(struct __pyx_obj_6tables_13hdf5extension_AttributeSet *__pyx_v_self, PyObject *__pyx_v_node, PyObject *__pyx_v_attrname) {
  int __pyx_v_ret;
  hid_t __pyx_v_dset_id;
  char *__pyx_v_cattrname;
  PyObject *__pyx_v_encoded_attrname = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  char *__pyx_t_4;
  hid_t __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10[5];
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_remove", 0);

  /* "tables/hdf5extension.pyx":1004
 *     cdef int ret
 *     cdef hid_t dset_id
 *     cdef char *cattrname = NULL             # <<<<<<<<<<<<<<
 *     cdef bytes encoded_attrname
 * 
*/
  __pyx_v_cattrname = NULL;

  /* "tables/hdf5extension.pyx":1007
 *     cdef bytes encoded_attrname
 * 
 *     encoded_attrname = attrname.encode('utf-8')             # <<<<<<<<<<<<<<
 *     # Get the C pointer
 *     cattrname = encoded_attrname
*/
  __pyx_t_2 = __pyx_v_attrname;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1007, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1007, __pyx_L1_error)
  __pyx_v_encoded_attrname = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1009
 *     encoded_attrname = attrname.encode('utf-8')
 *     # Get the C pointer
 *     cattrname = encoded_attrname             # <<<<<<<<<<<<<<
 * 
 *     # The dataset id of the node
*/
  if (unlikely(__pyx_v_encoded_attrname == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1009, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_attrname); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 1009, __pyx_L1_error)
  __pyx_v_cattrname = __pyx_t_4;

  /* "tables/hdf5extension.pyx":1012
 * 
 *     # The dataset id of the node
 *     dset_id = node._v_objectid             # <<<<<<<<<<<<<<
 * 
 *     ret = H5Adelete(dset_id, cattrname)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_node, __pyx_mstate_global->__pyx_n_u_v_objectid); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1012, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyLong_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_5 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1012, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_dset_id = __pyx_t_5;

  /* "tables/hdf5extension.pyx":1014
 *     dset_id = node._v_objectid
 * 
 *     ret = H5Adelete(dset_id, cattrname)             # <<<<<<<<<<<<<<
 *     if ret < 0:
 *       raise HDF5ExtError("Attribute '%s' exists in node '%s', but cannot be "
*/
  __pyx_v_ret = H5Adelete(__pyx_v_dset_id, __pyx_v_cattrname);

  /* "tables/hdf5extension.pyx":1015
 * 
 *     ret = H5Adelete(dset_id, cattrname)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Attribute '%s' exists in node '%s', but cannot be "
 *                          "deleted." % (attrname, self.name))
*/
  __pyx_t_6 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_6)) {

    /* "tables/hdf5extension.pyx":1016
 *     ret = H5Adelete(dset_id, cattrname)
 *     if ret < 0:
 *       raise HDF5ExtError("Attribute '%s' exists in node '%s', but cannot be "             # <<<<<<<<<<<<<<
 *                          "deleted." % (attrname, self.name))
 * 
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1016, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);

    /* "tables/hdf5extension.pyx":1017
 *     if ret < 0:
 *       raise HDF5ExtError("Attribute '%s' exists in node '%s', but cannot be "
 *                          "deleted." % (attrname, self.name))             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __pyx_t_8 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_attrname), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1017, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_self->name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1017, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u_Attribute_2;
    __pyx_t_10[1] = __pyx_t_8;
    __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u_exists_in_node_2;
    __pyx_t_10[3] = __pyx_t_9;
    __pyx_t_10[4] = __pyx_mstate_global->__pyx_kp_u_but_cannot_be_deleted;

    /* "tables/hdf5extension.pyx":1016
 *     ret = H5Adelete(dset_id, cattrname)
 *     if ret < 0:
 *       raise HDF5ExtError("Attribute '%s' exists in node '%s', but cannot be "             # <<<<<<<<<<<<<<
 *                          "deleted." % (attrname, self.name))
 * 
*/
    __pyx_t_11 = __Pyx_PyUnicode_Join(__pyx_t_10, 5, 11 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8) + 18 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9) + 25, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_9));
    if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1016, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_11};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1016, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1016, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1015
 * 
 *     ret = H5Adelete(dset_id, cattrname)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Attribute '%s' exists in node '%s', but cannot be "
 *                          "deleted." % (attrname, self.name))
*/
  }

  /* "tables/hdf5extension.pyx":1001
 * 
 * 
 *   def _g_remove(self, node, attrname):             # <<<<<<<<<<<<<<
 *     cdef int ret
 *     cdef hid_t dset_id
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("tables.hdf5extension.AttributeSet._g_remove", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_encoded_attrname);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_12AttributeSet_11__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_12AttributeSet_11__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_12AttributeSet_11__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_12AttributeSet_11__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_12AttributeSet_10__reduce_cython__(((struct __pyx_obj_6tables_13hdf5extension_AttributeSet *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_12AttributeSet_10__reduce_cython__(struct __pyx_obj_6tables_13hdf5extension_AttributeSet *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.name,)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->name);
  __Pyx_GIVEREF(__pyx_v_self->name);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->name) != (0)) __PYX_ERR(2, 5, __pyx_L1_error);
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.name,)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = (self.name,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_3 = (__pyx_v__dict != Py_None);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(2, 7, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict) != (0)) __PYX_ERR(2, 8, __pyx_L1_error);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.name is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.name,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.name is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_AttributeSet, (type(self), 0x82a3537, None), state
*/
  /*else*/ {
    __pyx_t_2 = (__pyx_v_self->name != Py_None);
    __pyx_v_use_setstate = __pyx_t_2;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_AttributeSet, (type(self), 0x82a3537, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self.name is not None
 *     if use_setstate:
 *         return __pyx_unpickle_AttributeSet, (type(self), 0x82a3537, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_AttributeSet, (type(self), 0x82a3537, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_AttributeSet); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_136983863);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_136983863);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_136983863) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_AttributeSet, (type(self), 0x82a3537, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_AttributeSet, (type(self), 0x82a3537, None), state
 *     else:
 *         return __pyx_unpickle_AttributeSet, (type(self), 0x82a3537, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_AttributeSet__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_AttributeSet); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_136983863);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_136983863);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_136983863) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("tables.hdf5extension.AttributeSet.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_AttributeSet, (type(self), 0x82a3537, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_AttributeSet__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_12AttributeSet_13__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_12AttributeSet_13__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_12AttributeSet_13__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_12AttributeSet_13__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(2, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(2, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(2, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(2, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.AttributeSet.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_12AttributeSet_12__setstate_cython__(((struct __pyx_obj_6tables_13hdf5extension_AttributeSet *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_12AttributeSet_12__setstate_cython__(struct __pyx_obj_6tables_13hdf5extension_AttributeSet *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_AttributeSet, (type(self), 0x82a3537, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_AttributeSet__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(2, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(2, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_6tables_13hdf5extension___pyx_unpickle_AttributeSet__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_AttributeSet, (type(self), 0x82a3537, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_AttributeSet__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.hdf5extension.AttributeSet.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1023
 *   # Instance variables declared in .pxd
 * 
 *   def _g_new(self, where, name, init):             # <<<<<<<<<<<<<<
 *     self.name = name
 *     # """The name of this node in its parent group."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4Node_1_g_new(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4Node_1_g_new = {"_g_new", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Node_1_g_new, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_4Node_1_g_new(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_where = 0;
  PyObject *__pyx_v_name = 0;
  CYTHON_UNUSED PyObject *__pyx_v_init = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_new (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_where,&__pyx_mstate_global->__pyx_n_u_name,&__pyx_mstate_global->__pyx_n_u_init,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1023, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1023, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1023, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1023, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_new", 0) < (0)) __PYX_ERR(0, 1023, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_new", 1, 3, 3, i); __PYX_ERR(0, 1023, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1023, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1023, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1023, __pyx_L3_error)
    }
    __pyx_v_where = values[0];
    __pyx_v_name = values[1];
    __pyx_v_init = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_new", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 1023, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Node._g_new", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4Node__g_new(((struct __pyx_obj_6tables_13hdf5extension_Node *)__pyx_v_self), __pyx_v_where, __pyx_v_name, __pyx_v_init);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4Node__g_new(struct __pyx_obj_6tables_13hdf5extension_Node *__pyx_v_self, PyObject *__pyx_v_where, PyObject *__pyx_v_name, CYTHON_UNUSED PyObject *__pyx_v_init) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  hid_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_new", 0);

  /* "tables/hdf5extension.pyx":1024
 * 
 *   def _g_new(self, where, name, init):
 *     self.name = name             # <<<<<<<<<<<<<<
 *     # """The name of this node in its parent group."""
 *     self.parent_id = where._v_objectid
*/
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = __pyx_v_name;

  /* "tables/hdf5extension.pyx":1026
 *     self.name = name
 *     # """The name of this node in its parent group."""
 *     self.parent_id = where._v_objectid             # <<<<<<<<<<<<<<
 *     # """The identifier of the parent group."""
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_where, __pyx_mstate_global->__pyx_n_u_v_objectid); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1026, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_2 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1026, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->parent_id = __pyx_t_2;

  /* "tables/hdf5extension.pyx":1023
 *   # Instance variables declared in .pxd
 * 
 *   def _g_new(self, where, name, init):             # <<<<<<<<<<<<<<
 *     self.name = name
 *     # """The name of this node in its parent group."""
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.hdf5extension.Node._g_new", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1029
 *     # """The identifier of the parent group."""
 * 
 *   def _g_delete(self, parent):             # <<<<<<<<<<<<<<
 *     cdef int ret
 *     cdef bytes encoded_name
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4Node_3_g_delete(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4Node_3_g_delete = {"_g_delete", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Node_3_g_delete, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_4Node_3_g_delete(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_parent = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_delete (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_parent,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1029, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1029, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_delete", 0) < (0)) __PYX_ERR(0, 1029, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_delete", 1, 1, 1, i); __PYX_ERR(0, 1029, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1029, __pyx_L3_error)
    }
    __pyx_v_parent = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_delete", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1029, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Node._g_delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4Node_2_g_delete(((struct __pyx_obj_6tables_13hdf5extension_Node *)__pyx_v_self), __pyx_v_parent);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4Node_2_g_delete(struct __pyx_obj_6tables_13hdf5extension_Node *__pyx_v_self, PyObject *__pyx_v_parent) {
  int __pyx_v_ret;
  PyObject *__pyx_v_encoded_name = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  hid_t __pyx_t_4;
  char *__pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_delete", 0);

  /* "tables/hdf5extension.pyx":1033
 *     cdef bytes encoded_name
 * 
 *     encoded_name = self.name.encode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     # Delete this node
*/
  __pyx_t_2 = __pyx_v_self->name;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1033, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1033, __pyx_L1_error)
  __pyx_v_encoded_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1036
 * 
 *     # Delete this node
 *     ret = H5Ldelete(parent._v_objectid, encoded_name, H5P_DEFAULT)             # <<<<<<<<<<<<<<
 *     if ret < 0:
 *       raise HDF5ExtError("problems deleting the node ``%s``" % self.name)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_parent, __pyx_mstate_global->__pyx_n_u_v_objectid); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1036, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyLong_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_4 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1036, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_v_encoded_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1036, __pyx_L1_error)
  }
  __pyx_t_5 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_name); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 1036, __pyx_L1_error)
  __pyx_v_ret = H5Ldelete(__pyx_t_4, __pyx_t_5, H5P_DEFAULT);

  /* "tables/hdf5extension.pyx":1037
 *     # Delete this node
 *     ret = H5Ldelete(parent._v_objectid, encoded_name, H5P_DEFAULT)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("problems deleting the node ``%s``" % self.name)
 *     return ret
*/
  __pyx_t_6 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_6)) {

    /* "tables/hdf5extension.pyx":1038
 *     ret = H5Ldelete(parent._v_objectid, encoded_name, H5P_DEFAULT)
 *     if ret < 0:
 *       raise HDF5ExtError("problems deleting the node ``%s``" % self.name)             # <<<<<<<<<<<<<<
 *     return ret
 * 
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1038, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_problems_deleting_the_node_s, __pyx_v_self->name); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1038, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_8};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1038, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1038, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1037
 *     # Delete this node
 *     ret = H5Ldelete(parent._v_objectid, encoded_name, H5P_DEFAULT)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("problems deleting the node ``%s``" % self.name)
 *     return ret
*/
  }

  /* "tables/hdf5extension.pyx":1039
 *     if ret < 0:
 *       raise HDF5ExtError("problems deleting the node ``%s``" % self.name)
 *     return ret             # <<<<<<<<<<<<<<
 * 
 *   def __dealloc__(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_ret); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1039, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1029
 *     # """The identifier of the parent group."""
 * 
 *   def _g_delete(self, parent):             # <<<<<<<<<<<<<<
 *     cdef int ret
 *     cdef bytes encoded_name
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("tables.hdf5extension.Node._g_delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_encoded_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1041
 *     return ret
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     self.parent_id = 0
 * 
*/

/* Python wrapper */
static void __pyx_pw_6tables_13hdf5extension_4Node_5__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_6tables_13hdf5extension_4Node_5__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_6tables_13hdf5extension_4Node_4__dealloc__(((struct __pyx_obj_6tables_13hdf5extension_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_6tables_13hdf5extension_4Node_4__dealloc__(struct __pyx_obj_6tables_13hdf5extension_Node *__pyx_v_self) {

  /* "tables/hdf5extension.pyx":1042
 * 
 *   def __dealloc__(self):
 *     self.parent_id = 0             # <<<<<<<<<<<<<<
 * 
 *   def _get_obj_info(self):
*/
  __pyx_v_self->parent_id = 0;

  /* "tables/hdf5extension.pyx":1041
 *     return ret
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     self.parent_id = 0
 * 
*/

  /* function exit code */
}

/* "tables/hdf5extension.pyx":1044
 *     self.parent_id = 0
 * 
 *   def _get_obj_info(self):             # <<<<<<<<<<<<<<
 *     cdef herr_t ret = 0
 *     cdef H5O_info_t oinfo
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4Node_7_get_obj_info(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4Node_7_get_obj_info = {"_get_obj_info", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Node_7_get_obj_info, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_4Node_7_get_obj_info(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_obj_info (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_get_obj_info", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_get_obj_info", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4Node_6_get_obj_info(((struct __pyx_obj_6tables_13hdf5extension_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4Node_6_get_obj_info(struct __pyx_obj_6tables_13hdf5extension_Node *__pyx_v_self) {
  herr_t __pyx_v_ret;
  H5O_info_t __pyx_v_oinfo;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  hid_t __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  size_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_obj_info", 0);

  /* "tables/hdf5extension.pyx":1045
 * 
 *   def _get_obj_info(self):
 *     cdef herr_t ret = 0             # <<<<<<<<<<<<<<
 *     cdef H5O_info_t oinfo
 * 
*/
  __pyx_v_ret = 0;

  /* "tables/hdf5extension.pyx":1048
 *     cdef H5O_info_t oinfo
 * 
 *     ret = H5Oget_info(self._v_objectid, &oinfo)             # <<<<<<<<<<<<<<
 *     if ret < 0:
 *       raise HDF5ExtError("Unable to get object info for '%s'" %
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_objectid); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1048, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_2 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1048, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ret = H5Oget_info(__pyx_t_2, (&__pyx_v_oinfo));

  /* "tables/hdf5extension.pyx":1049
 * 
 *     ret = H5Oget_info(self._v_objectid, &oinfo)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Unable to get object info for '%s'" %
 *                          self. _v_pathname)
*/
  __pyx_t_3 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_3)) {

    /* "tables/hdf5extension.pyx":1050
 *     ret = H5Oget_info(self._v_objectid, &oinfo)
 *     if ret < 0:
 *       raise HDF5ExtError("Unable to get object info for '%s'" %             # <<<<<<<<<<<<<<
 *                          self. _v_pathname)
 * 
*/
    __pyx_t_4 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1050, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);

    /* "tables/hdf5extension.pyx":1051
 *     if ret < 0:
 *       raise HDF5ExtError("Unable to get object info for '%s'" %
 *                          self. _v_pathname)             # <<<<<<<<<<<<<<
 * 
 *     return ObjInfo(oinfo.addr, oinfo.rc)
*/
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_pathname); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1051, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);

    /* "tables/hdf5extension.pyx":1050
 *     ret = H5Oget_info(self._v_objectid, &oinfo)
 *     if ret < 0:
 *       raise HDF5ExtError("Unable to get object info for '%s'" %             # <<<<<<<<<<<<<<
 *                          self. _v_pathname)
 * 
*/
    __pyx_t_7 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Unable_to_get_object_info_for_s, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1050, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_8 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_8 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_7};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1050, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1050, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1049
 * 
 *     ret = H5Oget_info(self._v_objectid, &oinfo)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Unable to get object info for '%s'" %
 *                          self. _v_pathname)
*/
  }

  /* "tables/hdf5extension.pyx":1053
 *                          self. _v_pathname)
 * 
 *     return ObjInfo(oinfo.addr, oinfo.rc)             # <<<<<<<<<<<<<<
 * 
 *   def _get_obj_timestamps(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ObjInfo); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1053, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = __Pyx_PyLong_From_haddr_t(__pyx_v_oinfo.addr); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1053, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyLong_From_unsigned_int(__pyx_v_oinfo.rc); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1053, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_8 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_5);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_8 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_8, (3-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1053, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1044
 *     self.parent_id = 0
 * 
 *   def _get_obj_info(self):             # <<<<<<<<<<<<<<
 *     cdef herr_t ret = 0
 *     cdef H5O_info_t oinfo
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("tables.hdf5extension.Node._get_obj_info", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1055
 *     return ObjInfo(oinfo.addr, oinfo.rc)
 * 
 *   def _get_obj_timestamps(self):             # <<<<<<<<<<<<<<
 *     cdef herr_t ret = 0
 *     cdef H5O_info_t oinfo
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4Node_9_get_obj_timestamps(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4Node_9_get_obj_timestamps = {"_get_obj_timestamps", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Node_9_get_obj_timestamps, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_4Node_9_get_obj_timestamps(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_obj_timestamps (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_get_obj_timestamps", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_get_obj_timestamps", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4Node_8_get_obj_timestamps(((struct __pyx_obj_6tables_13hdf5extension_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4Node_8_get_obj_timestamps(struct __pyx_obj_6tables_13hdf5extension_Node *__pyx_v_self) {
  herr_t __pyx_v_ret;
  H5O_info_t __pyx_v_oinfo;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  hid_t __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  size_t __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_obj_timestamps", 0);

  /* "tables/hdf5extension.pyx":1056
 * 
 *   def _get_obj_timestamps(self):
 *     cdef herr_t ret = 0             # <<<<<<<<<<<<<<
 *     cdef H5O_info_t oinfo
 * 
*/
  __pyx_v_ret = 0;

  /* "tables/hdf5extension.pyx":1059
 *     cdef H5O_info_t oinfo
 * 
 *     ret = H5Oget_info(self._v_objectid, &oinfo)             # <<<<<<<<<<<<<<
 *     if ret < 0:
 *       raise HDF5ExtError("Unable to get object info for '%s'" %
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_objectid); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1059, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_2 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1059, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ret = H5Oget_info(__pyx_t_2, (&__pyx_v_oinfo));

  /* "tables/hdf5extension.pyx":1060
 * 
 *     ret = H5Oget_info(self._v_objectid, &oinfo)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Unable to get object info for '%s'" %
 *                          self. _v_pathname)
*/
  __pyx_t_3 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_3)) {

    /* "tables/hdf5extension.pyx":1061
 *     ret = H5Oget_info(self._v_objectid, &oinfo)
 *     if ret < 0:
 *       raise HDF5ExtError("Unable to get object info for '%s'" %             # <<<<<<<<<<<<<<
 *                          self. _v_pathname)
 * 
*/
    __pyx_t_4 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1061, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);

    /* "tables/hdf5extension.pyx":1062
 *     if ret < 0:
 *       raise HDF5ExtError("Unable to get object info for '%s'" %
 *                          self. _v_pathname)             # <<<<<<<<<<<<<<
 * 
 *     return ObjTimestamps(oinfo.atime, oinfo.mtime, oinfo.ctime,
*/
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_pathname); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1062, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);

    /* "tables/hdf5extension.pyx":1061
 *     ret = H5Oget_info(self._v_objectid, &oinfo)
 *     if ret < 0:
 *       raise HDF5ExtError("Unable to get object info for '%s'" %             # <<<<<<<<<<<<<<
 *                          self. _v_pathname)
 * 
*/
    __pyx_t_7 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Unable_to_get_object_info_for_s, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1061, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_8 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_8 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_7};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1061, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1061, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1060
 * 
 *     ret = H5Oget_info(self._v_objectid, &oinfo)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Unable to get object info for '%s'" %
 *                          self. _v_pathname)
*/
  }

  /* "tables/hdf5extension.pyx":1064
 *                          self. _v_pathname)
 * 
 *     return ObjTimestamps(oinfo.atime, oinfo.mtime, oinfo.ctime,             # <<<<<<<<<<<<<<
 *                          oinfo.btime)
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ObjTimestamps); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1064, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = __Pyx_PyLong_From_time_t(__pyx_v_oinfo.atime); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1064, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyLong_From_time_t(__pyx_v_oinfo.mtime); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1064, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_9 = __Pyx_PyLong_From_time_t(__pyx_v_oinfo.ctime); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1064, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);

  /* "tables/hdf5extension.pyx":1065
 * 
 *     return ObjTimestamps(oinfo.atime, oinfo.mtime, oinfo.ctime,
 *                          oinfo.btime)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_10 = __Pyx_PyLong_From_time_t(__pyx_v_oinfo.btime); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1065, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_8 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_5);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_8 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[5] = {__pyx_t_5, __pyx_t_4, __pyx_t_6, __pyx_t_9, __pyx_t_10};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_8, (5-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1064, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1055
 *     return ObjInfo(oinfo.addr, oinfo.rc)
 * 
 *   def _get_obj_timestamps(self):             # <<<<<<<<<<<<<<
 *     cdef herr_t ret = 0
 *     cdef H5O_info_t oinfo
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("tables.hdf5extension.Node._get_obj_timestamps", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4Node_11__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4Node_11__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Node_11__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_4Node_11__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4Node_10__reduce_cython__(((struct __pyx_obj_6tables_13hdf5extension_Node *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4Node_10__reduce_cython__(struct __pyx_obj_6tables_13hdf5extension_Node *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.name, self.parent_id)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = __Pyx_PyLong_From_hid_t(__pyx_v_self->parent_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->name);
  __Pyx_GIVEREF(__pyx_v_self->name);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_self->name) != (0)) __PYX_ERR(2, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1) != (0)) __PYX_ERR(2, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.name, self.parent_id)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_2 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v__dict = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":7
 *     state = (self.name, self.parent_id)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  if (__pyx_t_4) {
  } else {
    __pyx_t_3 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(2, 7, __pyx_L1_error)
  __pyx_t_3 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v__dict) != (0)) __PYX_ERR(2, 8, __pyx_L1_error);
    __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.name is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.name, self.parent_id)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.name is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_Node, (type(self), 0x75c7861, None), state
*/
  /*else*/ {
    __pyx_t_3 = (__pyx_v_self->name != Py_None);
    __pyx_v_use_setstate = __pyx_t_3;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Node, (type(self), 0x75c7861, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self.name is not None
 *     if use_setstate:
 *         return __pyx_unpickle_Node, (type(self), 0x75c7861, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_Node, (type(self), 0x75c7861, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_Node); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_123500641);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_123500641);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_mstate_global->__pyx_int_123500641) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, Py_None) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Node, (type(self), 0x75c7861, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_Node, (type(self), 0x75c7861, None), state
 *     else:
 *         return __pyx_unpickle_Node, (type(self), 0x75c7861, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Node__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_Node); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_123500641);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_123500641);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_mstate_global->__pyx_int_123500641) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __pyx_t_5 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("tables.hdf5extension.Node.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Node, (type(self), 0x75c7861, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Node__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4Node_13__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4Node_13__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Node_13__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_4Node_13__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(2, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(2, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(2, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(2, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Node.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4Node_12__setstate_cython__(((struct __pyx_obj_6tables_13hdf5extension_Node *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4Node_12__setstate_cython__(struct __pyx_obj_6tables_13hdf5extension_Node *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_Node, (type(self), 0x75c7861, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Node__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(2, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(2, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_6tables_13hdf5extension___pyx_unpickle_Node__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Node, (type(self), 0x75c7861, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Node__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.hdf5extension.Node.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1071
 *   cdef hid_t   group_id
 * 
 *   def _g_create(self):             # <<<<<<<<<<<<<<
 *     cdef hid_t ret
 *     cdef bytes encoded_name
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_1_g_create(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Group_1_g_create = {"_g_create", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_1_g_create, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_1_g_create(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_create (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_g_create", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_g_create", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Group__g_create(((struct __pyx_obj_6tables_13hdf5extension_Group *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Group__g_create(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self) {
  hid_t __pyx_v_ret;
  PyObject *__pyx_v_encoded_name = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  char *__pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_create", 0);

  /* "tables/hdf5extension.pyx":1075
 *     cdef bytes encoded_name
 * 
 *     encoded_name = self.name.encode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     # @TODO: set property list --> utf-8
*/
  __pyx_t_2 = __pyx_v_self->__pyx_base.name;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1075, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1075, __pyx_L1_error)
  __pyx_v_encoded_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1080
 * 
 *     # Create a new group
 *     ret = H5Gcreate(self.parent_id, encoded_name, H5P_DEFAULT, H5P_DEFAULT,             # <<<<<<<<<<<<<<
 *                     H5P_DEFAULT)
 *     if ret < 0:
*/
  if (unlikely(__pyx_v_encoded_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1080, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_name); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 1080, __pyx_L1_error)

  /* "tables/hdf5extension.pyx":1081
 *     # Create a new group
 *     ret = H5Gcreate(self.parent_id, encoded_name, H5P_DEFAULT, H5P_DEFAULT,
 *                     H5P_DEFAULT)             # <<<<<<<<<<<<<<
 *     if ret < 0:
 *       raise HDF5ExtError("Can't create the group %s." % self.name)
*/
  __pyx_v_ret = H5Gcreate(__pyx_v_self->__pyx_base.parent_id, __pyx_t_4, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

  /* "tables/hdf5extension.pyx":1082
 *     ret = H5Gcreate(self.parent_id, encoded_name, H5P_DEFAULT, H5P_DEFAULT,
 *                     H5P_DEFAULT)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Can't create the group %s." % self.name)
 *     self.group_id = ret
*/
  __pyx_t_5 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_5)) {

    /* "tables/hdf5extension.pyx":1083
 *                     H5P_DEFAULT)
 *     if ret < 0:
 *       raise HDF5ExtError("Can't create the group %s." % self.name)             # <<<<<<<<<<<<<<
 *     self.group_id = ret
 *     return self.group_id
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1083, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Can_t_create_the_group_s, __pyx_v_self->__pyx_base.name); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1083, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_7};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1083, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1083, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1082
 *     ret = H5Gcreate(self.parent_id, encoded_name, H5P_DEFAULT, H5P_DEFAULT,
 *                     H5P_DEFAULT)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Can't create the group %s." % self.name)
 *     self.group_id = ret
*/
  }

  /* "tables/hdf5extension.pyx":1084
 *     if ret < 0:
 *       raise HDF5ExtError("Can't create the group %s." % self.name)
 *     self.group_id = ret             # <<<<<<<<<<<<<<
 *     return self.group_id
 * 
*/
  __pyx_v_self->group_id = __pyx_v_ret;

  /* "tables/hdf5extension.pyx":1085
 *       raise HDF5ExtError("Can't create the group %s." % self.name)
 *     self.group_id = ret
 *     return self.group_id             # <<<<<<<<<<<<<<
 * 
 *   def _g_open(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_hid_t(__pyx_v_self->group_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1085, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1071
 *   cdef hid_t   group_id
 * 
 *   def _g_create(self):             # <<<<<<<<<<<<<<
 *     cdef hid_t ret
 *     cdef bytes encoded_name
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("tables.hdf5extension.Group._g_create", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_encoded_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1087
 *     return self.group_id
 * 
 *   def _g_open(self):             # <<<<<<<<<<<<<<
 *     cdef hid_t ret
 *     cdef bytes encoded_name
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_3_g_open(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Group_3_g_open = {"_g_open", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_3_g_open, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_3_g_open(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_open (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_g_open", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_g_open", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Group_2_g_open(((struct __pyx_obj_6tables_13hdf5extension_Group *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_2_g_open(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self) {
  hid_t __pyx_v_ret;
  PyObject *__pyx_v_encoded_name = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  char *__pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_open", 0);

  /* "tables/hdf5extension.pyx":1091
 *     cdef bytes encoded_name
 * 
 *     encoded_name = self.name.encode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     ret = H5Gopen(self.parent_id, encoded_name, H5P_DEFAULT)
*/
  __pyx_t_2 = __pyx_v_self->__pyx_base.name;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1091, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1091, __pyx_L1_error)
  __pyx_v_encoded_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1093
 *     encoded_name = self.name.encode('utf-8')
 * 
 *     ret = H5Gopen(self.parent_id, encoded_name, H5P_DEFAULT)             # <<<<<<<<<<<<<<
 *     if ret < 0:
 *       raise HDF5ExtError("Can't open the group: '%s'." % self.name)
*/
  if (unlikely(__pyx_v_encoded_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1093, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_name); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 1093, __pyx_L1_error)
  __pyx_v_ret = H5Gopen(__pyx_v_self->__pyx_base.parent_id, __pyx_t_4, H5P_DEFAULT);

  /* "tables/hdf5extension.pyx":1094
 * 
 *     ret = H5Gopen(self.parent_id, encoded_name, H5P_DEFAULT)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Can't open the group: '%s'." % self.name)
 *     self.group_id = ret
*/
  __pyx_t_5 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_5)) {

    /* "tables/hdf5extension.pyx":1095
 *     ret = H5Gopen(self.parent_id, encoded_name, H5P_DEFAULT)
 *     if ret < 0:
 *       raise HDF5ExtError("Can't open the group: '%s'." % self.name)             # <<<<<<<<<<<<<<
 *     self.group_id = ret
 *     return self.group_id
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1095, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Can_t_open_the_group_s, __pyx_v_self->__pyx_base.name); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1095, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_7};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1095, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1095, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1094
 * 
 *     ret = H5Gopen(self.parent_id, encoded_name, H5P_DEFAULT)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Can't open the group: '%s'." % self.name)
 *     self.group_id = ret
*/
  }

  /* "tables/hdf5extension.pyx":1096
 *     if ret < 0:
 *       raise HDF5ExtError("Can't open the group: '%s'." % self.name)
 *     self.group_id = ret             # <<<<<<<<<<<<<<
 *     return self.group_id
 * 
*/
  __pyx_v_self->group_id = __pyx_v_ret;

  /* "tables/hdf5extension.pyx":1097
 *       raise HDF5ExtError("Can't open the group: '%s'." % self.name)
 *     self.group_id = ret
 *     return self.group_id             # <<<<<<<<<<<<<<
 * 
 *   def _g_get_objinfo(self, object h5name):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_hid_t(__pyx_v_self->group_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1097, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1087
 *     return self.group_id
 * 
 *   def _g_open(self):             # <<<<<<<<<<<<<<
 *     cdef hid_t ret
 *     cdef bytes encoded_name
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("tables.hdf5extension.Group._g_open", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_encoded_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1099
 *     return self.group_id
 * 
 *   def _g_get_objinfo(self, object h5name):             # <<<<<<<<<<<<<<
 *     """Check whether 'name' is a children of 'self' and return its type."""
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_5_g_get_objinfo(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_5Group_4_g_get_objinfo, "Check whether 'name' is a children of 'self' and return its type.");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Group_5_g_get_objinfo = {"_g_get_objinfo", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_5_g_get_objinfo, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Group_4_g_get_objinfo};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_5_g_get_objinfo(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_h5name = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_get_objinfo (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_h5name,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1099, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1099, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_get_objinfo", 0) < (0)) __PYX_ERR(0, 1099, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_get_objinfo", 1, 1, 1, i); __PYX_ERR(0, 1099, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1099, __pyx_L3_error)
    }
    __pyx_v_h5name = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_get_objinfo", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1099, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Group._g_get_objinfo", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Group_4_g_get_objinfo(((struct __pyx_obj_6tables_13hdf5extension_Group *)__pyx_v_self), __pyx_v_h5name);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_4_g_get_objinfo(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self, PyObject *__pyx_v_h5name) {
  int __pyx_v_ret;
  PyObject *__pyx_v_node_type = 0;
  PyObject *__pyx_v_encoded_name = 0;
  char *__pyx_v_cname;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  char *__pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_get_objinfo", 0);

  /* "tables/hdf5extension.pyx":1107
 *     cdef char *cname
 * 
 *     encoded_name = h5name.encode('utf-8')             # <<<<<<<<<<<<<<
 *     # Get the C pointer
 *     cname = encoded_name
*/
  __pyx_t_2 = __pyx_v_h5name;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1107, __pyx_L1_error)
  __pyx_v_encoded_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1109
 *     encoded_name = h5name.encode('utf-8')
 *     # Get the C pointer
 *     cname = encoded_name             # <<<<<<<<<<<<<<
 * 
 *     ret = get_linkinfo(self.group_id, cname)
*/
  if (unlikely(__pyx_v_encoded_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1109, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_name); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 1109, __pyx_L1_error)
  __pyx_v_cname = __pyx_t_4;

  /* "tables/hdf5extension.pyx":1111
 *     cname = encoded_name
 * 
 *     ret = get_linkinfo(self.group_id, cname)             # <<<<<<<<<<<<<<
 *     if ret == -2 or ret == H5L_TYPE_ERROR:
 *       node_type = "NoSuchNode"
*/
  __pyx_v_ret = get_linkinfo(__pyx_v_self->group_id, __pyx_v_cname);

  /* "tables/hdf5extension.pyx":1112
 * 
 *     ret = get_linkinfo(self.group_id, cname)
 *     if ret == -2 or ret == H5L_TYPE_ERROR:             # <<<<<<<<<<<<<<
 *       node_type = "NoSuchNode"
 *     elif ret == H5L_TYPE_SOFT:
*/
  switch (__pyx_v_ret) {
    case -2L:
    case H5L_TYPE_ERROR:

    /* "tables/hdf5extension.pyx":1113
 *     ret = get_linkinfo(self.group_id, cname)
 *     if ret == -2 or ret == H5L_TYPE_ERROR:
 *       node_type = "NoSuchNode"             # <<<<<<<<<<<<<<
 *     elif ret == H5L_TYPE_SOFT:
 *       node_type = "SoftLink"
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_NoSuchNode);
    __pyx_v_node_type = __pyx_mstate_global->__pyx_n_u_NoSuchNode;

    /* "tables/hdf5extension.pyx":1112
 * 
 *     ret = get_linkinfo(self.group_id, cname)
 *     if ret == -2 or ret == H5L_TYPE_ERROR:             # <<<<<<<<<<<<<<
 *       node_type = "NoSuchNode"
 *     elif ret == H5L_TYPE_SOFT:
*/
    break;
    case H5L_TYPE_SOFT:

    /* "tables/hdf5extension.pyx":1115
 *       node_type = "NoSuchNode"
 *     elif ret == H5L_TYPE_SOFT:
 *       node_type = "SoftLink"             # <<<<<<<<<<<<<<
 *     elif ret == H5L_TYPE_EXTERNAL:
 *       node_type = "ExternalLink"
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_SoftLink);
    __pyx_v_node_type = __pyx_mstate_global->__pyx_n_u_SoftLink;

    /* "tables/hdf5extension.pyx":1114
 *     if ret == -2 or ret == H5L_TYPE_ERROR:
 *       node_type = "NoSuchNode"
 *     elif ret == H5L_TYPE_SOFT:             # <<<<<<<<<<<<<<
 *       node_type = "SoftLink"
 *     elif ret == H5L_TYPE_EXTERNAL:
*/
    break;
    case H5L_TYPE_EXTERNAL:

    /* "tables/hdf5extension.pyx":1117
 *       node_type = "SoftLink"
 *     elif ret == H5L_TYPE_EXTERNAL:
 *       node_type = "ExternalLink"             # <<<<<<<<<<<<<<
 *     elif ret == H5L_TYPE_HARD:
 *         ret = get_objinfo(self.group_id, cname)
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_ExternalLink);
    __pyx_v_node_type = __pyx_mstate_global->__pyx_n_u_ExternalLink;

    /* "tables/hdf5extension.pyx":1116
 *     elif ret == H5L_TYPE_SOFT:
 *       node_type = "SoftLink"
 *     elif ret == H5L_TYPE_EXTERNAL:             # <<<<<<<<<<<<<<
 *       node_type = "ExternalLink"
 *     elif ret == H5L_TYPE_HARD:
*/
    break;
    case H5L_TYPE_HARD:

    /* "tables/hdf5extension.pyx":1119
 *       node_type = "ExternalLink"
 *     elif ret == H5L_TYPE_HARD:
 *         ret = get_objinfo(self.group_id, cname)             # <<<<<<<<<<<<<<
 *         if ret == -2:
 *           node_type = "NoSuchNode"
*/
    __pyx_v_ret = get_objinfo(__pyx_v_self->group_id, __pyx_v_cname);

    /* "tables/hdf5extension.pyx":1120
 *     elif ret == H5L_TYPE_HARD:
 *         ret = get_objinfo(self.group_id, cname)
 *         if ret == -2:             # <<<<<<<<<<<<<<
 *           node_type = "NoSuchNode"
 *         elif ret == H5O_TYPE_UNKNOWN:
*/
    switch (__pyx_v_ret) {
      case -2L:

      /* "tables/hdf5extension.pyx":1121
 *         ret = get_objinfo(self.group_id, cname)
 *         if ret == -2:
 *           node_type = "NoSuchNode"             # <<<<<<<<<<<<<<
 *         elif ret == H5O_TYPE_UNKNOWN:
 *           node_type = "Unknown"
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_NoSuchNode);
      __pyx_v_node_type = __pyx_mstate_global->__pyx_n_u_NoSuchNode;

      /* "tables/hdf5extension.pyx":1120
 *     elif ret == H5L_TYPE_HARD:
 *         ret = get_objinfo(self.group_id, cname)
 *         if ret == -2:             # <<<<<<<<<<<<<<
 *           node_type = "NoSuchNode"
 *         elif ret == H5O_TYPE_UNKNOWN:
*/
      break;
      case H5O_TYPE_UNKNOWN:

      /* "tables/hdf5extension.pyx":1123
 *           node_type = "NoSuchNode"
 *         elif ret == H5O_TYPE_UNKNOWN:
 *           node_type = "Unknown"             # <<<<<<<<<<<<<<
 *         elif ret == H5O_TYPE_GROUP:
 *           node_type = "Group"
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_Unknown);
      __pyx_v_node_type = __pyx_mstate_global->__pyx_n_u_Unknown;

      /* "tables/hdf5extension.pyx":1122
 *         if ret == -2:
 *           node_type = "NoSuchNode"
 *         elif ret == H5O_TYPE_UNKNOWN:             # <<<<<<<<<<<<<<
 *           node_type = "Unknown"
 *         elif ret == H5O_TYPE_GROUP:
*/
      break;
      case H5O_TYPE_GROUP:

      /* "tables/hdf5extension.pyx":1125
 *           node_type = "Unknown"
 *         elif ret == H5O_TYPE_GROUP:
 *           node_type = "Group"             # <<<<<<<<<<<<<<
 *         elif ret == H5O_TYPE_DATASET:
 *           node_type = "Leaf"
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_Group);
      __pyx_v_node_type = __pyx_mstate_global->__pyx_n_u_Group;

      /* "tables/hdf5extension.pyx":1124
 *         elif ret == H5O_TYPE_UNKNOWN:
 *           node_type = "Unknown"
 *         elif ret == H5O_TYPE_GROUP:             # <<<<<<<<<<<<<<
 *           node_type = "Group"
 *         elif ret == H5O_TYPE_DATASET:
*/
      break;
      case H5O_TYPE_DATASET:

      /* "tables/hdf5extension.pyx":1127
 *           node_type = "Group"
 *         elif ret == H5O_TYPE_DATASET:
 *           node_type = "Leaf"             # <<<<<<<<<<<<<<
 *         elif ret == H5O_TYPE_NAMED_DATATYPE:
 *           node_type = "NamedType"              # Not supported yet
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_Leaf);
      __pyx_v_node_type = __pyx_mstate_global->__pyx_n_u_Leaf;

      /* "tables/hdf5extension.pyx":1126
 *         elif ret == H5O_TYPE_GROUP:
 *           node_type = "Group"
 *         elif ret == H5O_TYPE_DATASET:             # <<<<<<<<<<<<<<
 *           node_type = "Leaf"
 *         elif ret == H5O_TYPE_NAMED_DATATYPE:
*/
      break;
      case H5O_TYPE_NAMED_DATATYPE:

      /* "tables/hdf5extension.pyx":1129
 *           node_type = "Leaf"
 *         elif ret == H5O_TYPE_NAMED_DATATYPE:
 *           node_type = "NamedType"              # Not supported yet             # <<<<<<<<<<<<<<
 *         #else H5O_TYPE_LINK:
 *         #    # symbolic link
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_NamedType);
      __pyx_v_node_type = __pyx_mstate_global->__pyx_n_u_NamedType;

      /* "tables/hdf5extension.pyx":1128
 *         elif ret == H5O_TYPE_DATASET:
 *           node_type = "Leaf"
 *         elif ret == H5O_TYPE_NAMED_DATATYPE:             # <<<<<<<<<<<<<<
 *           node_type = "NamedType"              # Not supported yet
 *         #else H5O_TYPE_LINK:
*/
      break;
      default:

      /* "tables/hdf5extension.pyx":1134
 *         #    raise RuntimeError('unexpected object type')
 *         else:
 *           node_type = "Unknown"             # <<<<<<<<<<<<<<
 *     return node_type
 * 
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_Unknown);
      __pyx_v_node_type = __pyx_mstate_global->__pyx_n_u_Unknown;
      break;
    }

    /* "tables/hdf5extension.pyx":1118
 *     elif ret == H5L_TYPE_EXTERNAL:
 *       node_type = "ExternalLink"
 *     elif ret == H5L_TYPE_HARD:             # <<<<<<<<<<<<<<
 *         ret = get_objinfo(self.group_id, cname)
 *         if ret == -2:
*/
    break;
    default: break;
  }

  /* "tables/hdf5extension.pyx":1135
 *         else:
 *           node_type = "Unknown"
 *     return node_type             # <<<<<<<<<<<<<<
 * 
 *   def _g_list_group(self, parent):
*/
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(!__pyx_v_node_type)) { __Pyx_RaiseUnboundLocalError("node_type"); __PYX_ERR(0, 1135, __pyx_L1_error) }
  __Pyx_INCREF(__pyx_v_node_type);
  __pyx_r = __pyx_v_node_type;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1099
 *     return self.group_id
 * 
 *   def _g_get_objinfo(self, object h5name):             # <<<<<<<<<<<<<<
 *     """Check whether 'name' is a children of 'self' and return its type."""
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.hdf5extension.Group._g_get_objinfo", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_node_type);
  __Pyx_XDECREF(__pyx_v_encoded_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1137
 *     return node_type
 * 
 *   def _g_list_group(self, parent):             # <<<<<<<<<<<<<<
 *     """Return a tuple with the groups and the leaves hanging from self."""
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_7_g_list_group(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_5Group_6_g_list_group, "Return a tuple with the groups and the leaves hanging from self.");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Group_7_g_list_group = {"_g_list_group", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_7_g_list_group, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Group_6_g_list_group};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_7_g_list_group(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_parent = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_list_group (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_parent,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1137, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1137, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_list_group", 0) < (0)) __PYX_ERR(0, 1137, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_list_group", 1, 1, 1, i); __PYX_ERR(0, 1137, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1137, __pyx_L3_error)
    }
    __pyx_v_parent = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_list_group", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1137, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Group._g_list_group", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Group_6_g_list_group(((struct __pyx_obj_6tables_13hdf5extension_Group *)__pyx_v_self), __pyx_v_parent);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_6_g_list_group(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self, PyObject *__pyx_v_parent) {
  PyObject *__pyx_v_encoded_name = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  hid_t __pyx_t_4;
  hid_t __pyx_t_5;
  char *__pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_list_group", 0);

  /* "tables/hdf5extension.pyx":1142
 *     cdef bytes encoded_name
 * 
 *     encoded_name = self.name.encode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     return Giterate(parent._v_objectid, self._v_objectid, encoded_name)
*/
  __pyx_t_2 = __pyx_v_self->__pyx_base.name;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1142, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1142, __pyx_L1_error)
  __pyx_v_encoded_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1144
 *     encoded_name = self.name.encode('utf-8')
 * 
 *     return Giterate(parent._v_objectid, self._v_objectid, encoded_name)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_parent, __pyx_mstate_global->__pyx_n_u_v_objectid); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyLong_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_4 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1144, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_objectid); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyLong_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_5 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1144, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_v_encoded_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1144, __pyx_L1_error)
  }
  __pyx_t_6 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_name); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 1144, __pyx_L1_error)
  __pyx_t_1 = Giterate(__pyx_t_4, __pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1137
 *     return node_type
 * 
 *   def _g_list_group(self, parent):             # <<<<<<<<<<<<<<
 *     """Return a tuple with the groups and the leaves hanging from self."""
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.hdf5extension.Group._g_list_group", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_encoded_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1147
 * 
 * 
 *   def _g_get_gchild_attr(self, group_name, attr_name):             # <<<<<<<<<<<<<<
 *     """Return an attribute of a child `Group`.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_9_g_get_gchild_attr(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_5Group_8_g_get_gchild_attr, "Return an attribute of a child `Group`.\n\n    If the attribute does not exist, ``None`` is returned.\n\n    ");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Group_9_g_get_gchild_attr = {"_g_get_gchild_attr", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_9_g_get_gchild_attr, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Group_8_g_get_gchild_attr};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_9_g_get_gchild_attr(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_group_name = 0;
  PyObject *__pyx_v_attr_name = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_get_gchild_attr (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_group_name,&__pyx_mstate_global->__pyx_n_u_attr_name,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1147, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1147, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1147, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_get_gchild_attr", 0) < (0)) __PYX_ERR(0, 1147, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_get_gchild_attr", 1, 2, 2, i); __PYX_ERR(0, 1147, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1147, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1147, __pyx_L3_error)
    }
    __pyx_v_group_name = values[0];
    __pyx_v_attr_name = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_get_gchild_attr", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1147, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Group._g_get_gchild_attr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Group_8_g_get_gchild_attr(((struct __pyx_obj_6tables_13hdf5extension_Group *)__pyx_v_self), __pyx_v_group_name, __pyx_v_attr_name);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_8_g_get_gchild_attr(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self, PyObject *__pyx_v_group_name, PyObject *__pyx_v_attr_name) {
  hid_t __pyx_v_gchild_id;
  PyObject *__pyx_v_retvalue = 0;
  PyObject *__pyx_v_encoded_group_name = 0;
  PyObject *__pyx_v_encoded_attr_name = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  char *__pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10[5];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_get_gchild_attr", 0);

  /* "tables/hdf5extension.pyx":1159
 *     cdef bytes encoded_attr_name
 * 
 *     encoded_group_name = group_name.encode('utf-8')             # <<<<<<<<<<<<<<
 *     encoded_attr_name = attr_name.encode('utf-8')
 * 
*/
  __pyx_t_2 = __pyx_v_group_name;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1159, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1159, __pyx_L1_error)
  __pyx_v_encoded_group_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1160
 * 
 *     encoded_group_name = group_name.encode('utf-8')
 *     encoded_attr_name = attr_name.encode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     # Open the group
*/
  __pyx_t_2 = __pyx_v_attr_name;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1160, __pyx_L1_error)
  __pyx_v_encoded_attr_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1163
 * 
 *     # Open the group
 *     retvalue = None  # Default value             # <<<<<<<<<<<<<<
 *     gchild_id = H5Gopen(self.group_id, encoded_group_name, H5P_DEFAULT)
 *     if gchild_id < 0:
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_retvalue = Py_None;

  /* "tables/hdf5extension.pyx":1164
 *     # Open the group
 *     retvalue = None  # Default value
 *     gchild_id = H5Gopen(self.group_id, encoded_group_name, H5P_DEFAULT)             # <<<<<<<<<<<<<<
 *     if gchild_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
*/
  if (unlikely(__pyx_v_encoded_group_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1164, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_group_name); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 1164, __pyx_L1_error)
  __pyx_v_gchild_id = H5Gopen(__pyx_v_self->group_id, __pyx_t_4, H5P_DEFAULT);

  /* "tables/hdf5extension.pyx":1165
 *     retvalue = None  # Default value
 *     gchild_id = H5Gopen(self.group_id, encoded_group_name, H5P_DEFAULT)
 *     if gchild_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
 *                          (group_name, self._v_pathname))
*/
  __pyx_t_5 = (__pyx_v_gchild_id < 0);
  if (unlikely(__pyx_t_5)) {

    /* "tables/hdf5extension.pyx":1166
 *     gchild_id = H5Gopen(self.group_id, encoded_group_name, H5P_DEFAULT)
 *     if gchild_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %             # <<<<<<<<<<<<<<
 *                          (group_name, self._v_pathname))
 *     retvalue = get_attribute_string_or_none(gchild_id, encoded_attr_name)
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1166, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);

    /* "tables/hdf5extension.pyx":1167
 *     if gchild_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
 *                          (group_name, self._v_pathname))             # <<<<<<<<<<<<<<
 *     retvalue = get_attribute_string_or_none(gchild_id, encoded_attr_name)
 *     # Close child group
*/
    __pyx_t_7 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_group_name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_pathname); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_t_8), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u_Non_existing_node;
    __pyx_t_10[1] = __pyx_t_7;
    __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u_under;
    __pyx_t_10[3] = __pyx_t_9;
    __pyx_t_10[4] = __pyx_mstate_global->__pyx_kp_u__4;

    /* "tables/hdf5extension.pyx":1166
 *     gchild_id = H5Gopen(self.group_id, encoded_group_name, H5P_DEFAULT)
 *     if gchild_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %             # <<<<<<<<<<<<<<
 *                          (group_name, self._v_pathname))
 *     retvalue = get_attribute_string_or_none(gchild_id, encoded_attr_name)
*/
    __pyx_t_8 = __Pyx_PyUnicode_Join(__pyx_t_10, 5, 20 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7) + 11 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9) + 2, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_9));
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1166, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_8};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1166, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1166, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1165
 *     retvalue = None  # Default value
 *     gchild_id = H5Gopen(self.group_id, encoded_group_name, H5P_DEFAULT)
 *     if gchild_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
 *                          (group_name, self._v_pathname))
*/
  }

  /* "tables/hdf5extension.pyx":1168
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
 *                          (group_name, self._v_pathname))
 *     retvalue = get_attribute_string_or_none(gchild_id, encoded_attr_name)             # <<<<<<<<<<<<<<
 *     # Close child group
 *     H5Gclose(gchild_id)
*/
  if (unlikely(__pyx_v_encoded_attr_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1168, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_attr_name); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 1168, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6tables_13hdf5extension_get_attribute_string_or_none(__pyx_v_gchild_id, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_retvalue, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1170
 *     retvalue = get_attribute_string_or_none(gchild_id, encoded_attr_name)
 *     # Close child group
 *     H5Gclose(gchild_id)             # <<<<<<<<<<<<<<
 * 
 *     return retvalue
*/
  (void)(H5Gclose(__pyx_v_gchild_id));

  /* "tables/hdf5extension.pyx":1172
 *     H5Gclose(gchild_id)
 * 
 *     return retvalue             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_retvalue);
  __pyx_r = __pyx_v_retvalue;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1147
 * 
 * 
 *   def _g_get_gchild_attr(self, group_name, attr_name):             # <<<<<<<<<<<<<<
 *     """Return an attribute of a child `Group`.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("tables.hdf5extension.Group._g_get_gchild_attr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_retvalue);
  __Pyx_XDECREF(__pyx_v_encoded_group_name);
  __Pyx_XDECREF(__pyx_v_encoded_attr_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1175
 * 
 * 
 *   def _g_get_lchild_attr(self, leaf_name, attr_name):             # <<<<<<<<<<<<<<
 *     """Return an attribute of a child `Leaf`.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_11_g_get_lchild_attr(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_5Group_10_g_get_lchild_attr, "Return an attribute of a child `Leaf`.\n\n    If the attribute does not exist, ``None`` is returned.\n\n    ");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Group_11_g_get_lchild_attr = {"_g_get_lchild_attr", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_11_g_get_lchild_attr, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Group_10_g_get_lchild_attr};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_11_g_get_lchild_attr(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_leaf_name = 0;
  PyObject *__pyx_v_attr_name = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_get_lchild_attr (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_leaf_name,&__pyx_mstate_global->__pyx_n_u_attr_name,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1175, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1175, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1175, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_get_lchild_attr", 0) < (0)) __PYX_ERR(0, 1175, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_get_lchild_attr", 1, 2, 2, i); __PYX_ERR(0, 1175, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1175, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1175, __pyx_L3_error)
    }
    __pyx_v_leaf_name = values[0];
    __pyx_v_attr_name = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_get_lchild_attr", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1175, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Group._g_get_lchild_attr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Group_10_g_get_lchild_attr(((struct __pyx_obj_6tables_13hdf5extension_Group *)__pyx_v_self), __pyx_v_leaf_name, __pyx_v_attr_name);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_10_g_get_lchild_attr(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self, PyObject *__pyx_v_leaf_name, PyObject *__pyx_v_attr_name) {
  hid_t __pyx_v_leaf_id;
  PyObject *__pyx_v_retvalue = 0;
  PyObject *__pyx_v_encoded_leaf_name = 0;
  PyObject *__pyx_v_encoded_attr_name = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  char *__pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10[5];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_get_lchild_attr", 0);

  /* "tables/hdf5extension.pyx":1187
 *     cdef bytes encoded_attr_name
 * 
 *     encoded_leaf_name = leaf_name.encode('utf-8')             # <<<<<<<<<<<<<<
 *     encoded_attr_name = attr_name.encode('utf-8')
 * 
*/
  __pyx_t_2 = __pyx_v_leaf_name;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1187, __pyx_L1_error)
  __pyx_v_encoded_leaf_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1188
 * 
 *     encoded_leaf_name = leaf_name.encode('utf-8')
 *     encoded_attr_name = attr_name.encode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     # Open the dataset
*/
  __pyx_t_2 = __pyx_v_attr_name;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1188, __pyx_L1_error)
  __pyx_v_encoded_attr_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1191
 * 
 *     # Open the dataset
 *     leaf_id = H5Dopen(self.group_id, encoded_leaf_name, H5P_DEFAULT)             # <<<<<<<<<<<<<<
 *     if leaf_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
*/
  if (unlikely(__pyx_v_encoded_leaf_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1191, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_leaf_name); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 1191, __pyx_L1_error)
  __pyx_v_leaf_id = H5Dopen(__pyx_v_self->group_id, __pyx_t_4, H5P_DEFAULT);

  /* "tables/hdf5extension.pyx":1192
 *     # Open the dataset
 *     leaf_id = H5Dopen(self.group_id, encoded_leaf_name, H5P_DEFAULT)
 *     if leaf_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
 *                          (leaf_name, self._v_pathname))
*/
  __pyx_t_5 = (__pyx_v_leaf_id < 0);
  if (unlikely(__pyx_t_5)) {

    /* "tables/hdf5extension.pyx":1193
 *     leaf_id = H5Dopen(self.group_id, encoded_leaf_name, H5P_DEFAULT)
 *     if leaf_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %             # <<<<<<<<<<<<<<
 *                          (leaf_name, self._v_pathname))
 *     retvalue = get_attribute_string_or_none(leaf_id, encoded_attr_name)
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);

    /* "tables/hdf5extension.pyx":1194
 *     if leaf_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
 *                          (leaf_name, self._v_pathname))             # <<<<<<<<<<<<<<
 *     retvalue = get_attribute_string_or_none(leaf_id, encoded_attr_name)
 *     # Close the dataset
*/
    __pyx_t_7 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_leaf_name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1194, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_pathname); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1194, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_t_8), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1194, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u_Non_existing_node;
    __pyx_t_10[1] = __pyx_t_7;
    __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u_under;
    __pyx_t_10[3] = __pyx_t_9;
    __pyx_t_10[4] = __pyx_mstate_global->__pyx_kp_u__4;

    /* "tables/hdf5extension.pyx":1193
 *     leaf_id = H5Dopen(self.group_id, encoded_leaf_name, H5P_DEFAULT)
 *     if leaf_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %             # <<<<<<<<<<<<<<
 *                          (leaf_name, self._v_pathname))
 *     retvalue = get_attribute_string_or_none(leaf_id, encoded_attr_name)
*/
    __pyx_t_8 = __Pyx_PyUnicode_Join(__pyx_t_10, 5, 20 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7) + 11 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9) + 2, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_9));
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_8};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1193, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1193, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1192
 *     # Open the dataset
 *     leaf_id = H5Dopen(self.group_id, encoded_leaf_name, H5P_DEFAULT)
 *     if leaf_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
 *                          (leaf_name, self._v_pathname))
*/
  }

  /* "tables/hdf5extension.pyx":1195
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
 *                          (leaf_name, self._v_pathname))
 *     retvalue = get_attribute_string_or_none(leaf_id, encoded_attr_name)             # <<<<<<<<<<<<<<
 *     # Close the dataset
 *     H5Dclose(leaf_id)
*/
  if (unlikely(__pyx_v_encoded_attr_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1195, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_attr_name); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 1195, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6tables_13hdf5extension_get_attribute_string_or_none(__pyx_v_leaf_id, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_retvalue = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1197
 *     retvalue = get_attribute_string_or_none(leaf_id, encoded_attr_name)
 *     # Close the dataset
 *     H5Dclose(leaf_id)             # <<<<<<<<<<<<<<
 *     return retvalue
 * 
*/
  (void)(H5Dclose(__pyx_v_leaf_id));

  /* "tables/hdf5extension.pyx":1198
 *     # Close the dataset
 *     H5Dclose(leaf_id)
 *     return retvalue             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_retvalue);
  __pyx_r = __pyx_v_retvalue;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1175
 * 
 * 
 *   def _g_get_lchild_attr(self, leaf_name, attr_name):             # <<<<<<<<<<<<<<
 *     """Return an attribute of a child `Leaf`.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("tables.hdf5extension.Group._g_get_lchild_attr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_retvalue);
  __Pyx_XDECREF(__pyx_v_encoded_leaf_name);
  __Pyx_XDECREF(__pyx_v_encoded_attr_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1201
 * 
 * 
 *   def _g_flush_group(self):             # <<<<<<<<<<<<<<
 *     # Close the group
 *     H5Fflush(self.group_id, H5F_SCOPE_GLOBAL)
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_13_g_flush_group(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Group_13_g_flush_group = {"_g_flush_group", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_13_g_flush_group, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_13_g_flush_group(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_flush_group (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_g_flush_group", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_g_flush_group", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Group_12_g_flush_group(((struct __pyx_obj_6tables_13hdf5extension_Group *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_12_g_flush_group(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_flush_group", 0);

  /* "tables/hdf5extension.pyx":1203
 *   def _g_flush_group(self):
 *     # Close the group
 *     H5Fflush(self.group_id, H5F_SCOPE_GLOBAL)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  (void)(H5Fflush(__pyx_v_self->group_id, H5F_SCOPE_GLOBAL));

  /* "tables/hdf5extension.pyx":1201
 * 
 * 
 *   def _g_flush_group(self):             # <<<<<<<<<<<<<<
 *     # Close the group
 *     H5Fflush(self.group_id, H5F_SCOPE_GLOBAL)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1206
 * 
 * 
 *   def _g_close_group(self):             # <<<<<<<<<<<<<<
 *     cdef int ret
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_15_g_close_group(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Group_15_g_close_group = {"_g_close_group", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_15_g_close_group, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_15_g_close_group(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_close_group (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_g_close_group", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_g_close_group", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Group_14_g_close_group(((struct __pyx_obj_6tables_13hdf5extension_Group *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_14_g_close_group(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self) {
  int __pyx_v_ret;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_close_group", 0);

  /* "tables/hdf5extension.pyx":1209
 *     cdef int ret
 * 
 *     ret = H5Gclose(self.group_id)             # <<<<<<<<<<<<<<
 *     if ret < 0:
 *       raise HDF5ExtError("Problems closing the Group %s" % self.name)
*/
  __pyx_v_ret = H5Gclose(__pyx_v_self->group_id);

  /* "tables/hdf5extension.pyx":1210
 * 
 *     ret = H5Gclose(self.group_id)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems closing the Group %s" % self.name)
 *     self.group_id = 0  # indicate that this group is closed
*/
  __pyx_t_1 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/hdf5extension.pyx":1211
 *     ret = H5Gclose(self.group_id)
 *     if ret < 0:
 *       raise HDF5ExtError("Problems closing the Group %s" % self.name)             # <<<<<<<<<<<<<<
 *     self.group_id = 0  # indicate that this group is closed
 * 
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1211, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Problems_closing_the_Group_s, __pyx_v_self->__pyx_base.name); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1211, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1211, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1211, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1210
 * 
 *     ret = H5Gclose(self.group_id)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems closing the Group %s" % self.name)
 *     self.group_id = 0  # indicate that this group is closed
*/
  }

  /* "tables/hdf5extension.pyx":1212
 *     if ret < 0:
 *       raise HDF5ExtError("Problems closing the Group %s" % self.name)
 *     self.group_id = 0  # indicate that this group is closed             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_v_self->group_id = 0;

  /* "tables/hdf5extension.pyx":1206
 * 
 * 
 *   def _g_close_group(self):             # <<<<<<<<<<<<<<
 *     cdef int ret
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("tables.hdf5extension.Group._g_close_group", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1215
 * 
 * 
 *   def _g_move_node(self, hid_t oldparent, oldname, hid_t newparent, newname,             # <<<<<<<<<<<<<<
 *                    oldpathname, newpathname):
 *     cdef int ret
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_17_g_move_node(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Group_17_g_move_node = {"_g_move_node", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_17_g_move_node, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_17_g_move_node(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  hid_t __pyx_v_oldparent;
  PyObject *__pyx_v_oldname = 0;
  hid_t __pyx_v_newparent;
  PyObject *__pyx_v_newname = 0;
  PyObject *__pyx_v_oldpathname = 0;
  PyObject *__pyx_v_newpathname = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[6] = {0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_move_node (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_oldparent,&__pyx_mstate_global->__pyx_n_u_oldname,&__pyx_mstate_global->__pyx_n_u_newparent,&__pyx_mstate_global->__pyx_n_u_newname,&__pyx_mstate_global->__pyx_n_u_oldpathname,&__pyx_mstate_global->__pyx_n_u_newpathname,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1215, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  6:
        values[5] = __Pyx_ArgRef_FASTCALL(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 1215, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 1215, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1215, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1215, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1215, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1215, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_move_node", 0) < (0)) __PYX_ERR(0, 1215, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 6; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_move_node", 1, 6, 6, i); __PYX_ERR(0, 1215, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 6)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1215, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1215, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1215, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1215, __pyx_L3_error)
      values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 1215, __pyx_L3_error)
      values[5] = __Pyx_ArgRef_FASTCALL(__pyx_args, 5);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 1215, __pyx_L3_error)
    }
    __pyx_v_oldparent = __Pyx_PyLong_As_hid_t(values[0]); if (unlikely((__pyx_v_oldparent == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1215, __pyx_L3_error)
    __pyx_v_oldname = values[1];
    __pyx_v_newparent = __Pyx_PyLong_As_hid_t(values[2]); if (unlikely((__pyx_v_newparent == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1215, __pyx_L3_error)
    __pyx_v_newname = values[3];
    __pyx_v_oldpathname = values[4];
    __pyx_v_newpathname = values[5];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_move_node", 1, 6, 6, __pyx_nargs); __PYX_ERR(0, 1215, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Group._g_move_node", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Group_16_g_move_node(((struct __pyx_obj_6tables_13hdf5extension_Group *)__pyx_v_self), __pyx_v_oldparent, __pyx_v_oldname, __pyx_v_newparent, __pyx_v_newname, __pyx_v_oldpathname, __pyx_v_newpathname);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_16_g_move_node(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self, hid_t __pyx_v_oldparent, PyObject *__pyx_v_oldname, hid_t __pyx_v_newparent, PyObject *__pyx_v_newname, PyObject *__pyx_v_oldpathname, PyObject *__pyx_v_newpathname) {
  int __pyx_v_ret;
  PyObject *__pyx_v_encoded_oldname = 0;
  PyObject *__pyx_v_encoded_newname = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  char *__pyx_t_4;
  char *__pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10[4];
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_move_node", 0);

  /* "tables/hdf5extension.pyx":1220
 *     cdef bytes encoded_oldname, encoded_newname
 * 
 *     encoded_oldname = oldname.encode('utf-8')             # <<<<<<<<<<<<<<
 *     encoded_newname = newname.encode('utf-8')
 * 
*/
  __pyx_t_2 = __pyx_v_oldname;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1220, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1220, __pyx_L1_error)
  __pyx_v_encoded_oldname = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1221
 * 
 *     encoded_oldname = oldname.encode('utf-8')
 *     encoded_newname = newname.encode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     ret = H5Lmove(oldparent, encoded_oldname, newparent, encoded_newname,
*/
  __pyx_t_2 = __pyx_v_newname;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1221, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1221, __pyx_L1_error)
  __pyx_v_encoded_newname = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1223
 *     encoded_newname = newname.encode('utf-8')
 * 
 *     ret = H5Lmove(oldparent, encoded_oldname, newparent, encoded_newname,             # <<<<<<<<<<<<<<
 *                   H5P_DEFAULT, H5P_DEFAULT)
 *     if ret < 0:
*/
  if (unlikely(__pyx_v_encoded_oldname == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1223, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_oldname); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 1223, __pyx_L1_error)
  if (unlikely(__pyx_v_encoded_newname == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1223, __pyx_L1_error)
  }
  __pyx_t_5 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_newname); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 1223, __pyx_L1_error)

  /* "tables/hdf5extension.pyx":1224
 * 
 *     ret = H5Lmove(oldparent, encoded_oldname, newparent, encoded_newname,
 *                   H5P_DEFAULT, H5P_DEFAULT)             # <<<<<<<<<<<<<<
 *     if ret < 0:
 *       raise HDF5ExtError("Problems moving the node %s to %s" %
*/
  __pyx_v_ret = H5Lmove(__pyx_v_oldparent, __pyx_t_4, __pyx_v_newparent, __pyx_t_5, H5P_DEFAULT, H5P_DEFAULT);

  /* "tables/hdf5extension.pyx":1225
 *     ret = H5Lmove(oldparent, encoded_oldname, newparent, encoded_newname,
 *                   H5P_DEFAULT, H5P_DEFAULT)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems moving the node %s to %s" %
 *                          (oldpathname, newpathname) )
*/
  __pyx_t_6 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_6)) {

    /* "tables/hdf5extension.pyx":1226
 *                   H5P_DEFAULT, H5P_DEFAULT)
 *     if ret < 0:
 *       raise HDF5ExtError("Problems moving the node %s to %s" %             # <<<<<<<<<<<<<<
 *                          (oldpathname, newpathname) )
 *     return ret
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1226, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);

    /* "tables/hdf5extension.pyx":1227
 *     if ret < 0:
 *       raise HDF5ExtError("Problems moving the node %s to %s" %
 *                          (oldpathname, newpathname) )             # <<<<<<<<<<<<<<
 *     return ret
 * 
*/
    __pyx_t_8 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_oldpathname), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1227, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_newpathname), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1227, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u_Problems_moving_the_node;
    __pyx_t_10[1] = __pyx_t_8;
    __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u_to;
    __pyx_t_10[3] = __pyx_t_9;

    /* "tables/hdf5extension.pyx":1226
 *                   H5P_DEFAULT, H5P_DEFAULT)
 *     if ret < 0:
 *       raise HDF5ExtError("Problems moving the node %s to %s" %             # <<<<<<<<<<<<<<
 *                          (oldpathname, newpathname) )
 *     return ret
*/
    __pyx_t_11 = __Pyx_PyUnicode_Join(__pyx_t_10, 4, 25 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8) + 4 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_9));
    if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1226, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_11};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1226, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1226, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1225
 *     ret = H5Lmove(oldparent, encoded_oldname, newparent, encoded_newname,
 *                   H5P_DEFAULT, H5P_DEFAULT)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems moving the node %s to %s" %
 *                          (oldpathname, newpathname) )
*/
  }

  /* "tables/hdf5extension.pyx":1228
 *       raise HDF5ExtError("Problems moving the node %s to %s" %
 *                          (oldpathname, newpathname) )
 *     return ret             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_ret); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1215
 * 
 * 
 *   def _g_move_node(self, hid_t oldparent, oldname, hid_t newparent, newname,             # <<<<<<<<<<<<<<
 *                    oldpathname, newpathname):
 *     cdef int ret
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("tables.hdf5extension.Group._g_move_node", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_encoded_oldname);
  __Pyx_XDECREF(__pyx_v_encoded_newname);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_19__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Group_19__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_19__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_19__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Group_18__reduce_cython__(((struct __pyx_obj_6tables_13hdf5extension_Group *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_18__reduce_cython__(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.group_id, self.name, self.parent_id)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = __Pyx_PyLong_From_hid_t(__pyx_v_self->group_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_From_hid_t(__pyx_v_self->__pyx_base.parent_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1) != (0)) __PYX_ERR(2, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.name);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.name);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_self->__pyx_base.name) != (0)) __PYX_ERR(2, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2) != (0)) __PYX_ERR(2, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.group_id, self.name, self.parent_id)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self.group_id, self.name, self.parent_id)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  if (__pyx_t_5) {
  } else {
    __pyx_t_4 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(2, 7, __pyx_L1_error)
  __pyx_t_4 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_4) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict) != (0)) __PYX_ERR(2, 8, __pyx_L1_error);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.name is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.group_id, self.name, self.parent_id)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.name is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_Group, (type(self), 0xd52d8f4, None), state
*/
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->__pyx_base.name != Py_None);
    __pyx_v_use_setstate = __pyx_t_4;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Group, (type(self), 0xd52d8f4, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self.name is not None
 *     if use_setstate:
 *         return __pyx_unpickle_Group, (type(self), 0xd52d8f4, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_Group, (type(self), 0xd52d8f4, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_Group); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_223533300);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_223533300);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_mstate_global->__pyx_int_223533300) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state) != (0)) __PYX_ERR(2, 13, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Group, (type(self), 0xd52d8f4, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_Group, (type(self), 0xd52d8f4, None), state
 *     else:
 *         return __pyx_unpickle_Group, (type(self), 0xd52d8f4, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Group__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_Group); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_223533300);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_223533300);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_mstate_global->__pyx_int_223533300) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3) != (0)) __PYX_ERR(2, 15, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("tables.hdf5extension.Group.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Group, (type(self), 0xd52d8f4, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Group__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_21__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Group_21__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_21__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Group_21__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(2, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(2, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(2, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(2, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Group.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Group_20__setstate_cython__(((struct __pyx_obj_6tables_13hdf5extension_Group *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Group_20__setstate_cython__(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_Group, (type(self), 0xd52d8f4, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Group__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(2, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(2, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_6tables_13hdf5extension___pyx_unpickle_Group__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Group, (type(self), 0xd52d8f4, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Group__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.hdf5extension.Group.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1235
 *   # Instance variables declared in .pxd
 * 
 *   def _get_storage_size(self):             # <<<<<<<<<<<<<<
 *       return H5Dget_storage_size(self.dataset_id)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_1_get_storage_size(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4Leaf_1_get_storage_size = {"_get_storage_size", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_1_get_storage_size, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_1_get_storage_size(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_storage_size (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_get_storage_size", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_get_storage_size", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4Leaf__get_storage_size(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf__get_storage_size(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_storage_size", 0);

  /* "tables/hdf5extension.pyx":1236
 * 
 *   def _get_storage_size(self):
 *       return H5Dget_storage_size(self.dataset_id)             # <<<<<<<<<<<<<<
 * 
 *   def _get_obj_track_times(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_hsize_t(H5Dget_storage_size(__pyx_v_self->dataset_id)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1235
 *   # Instance variables declared in .pxd
 * 
 *   def _get_storage_size(self):             # <<<<<<<<<<<<<<
 *       return H5Dget_storage_size(self.dataset_id)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.hdf5extension.Leaf._get_storage_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1238
 *       return H5Dget_storage_size(self.dataset_id)
 * 
 *   def _get_obj_track_times(self):             # <<<<<<<<<<<<<<
 *     """Get track_times boolean for dataset
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_3_get_obj_track_times(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_4Leaf_2_get_obj_track_times, "Get track_times boolean for dataset\n\n    Uses H5Pget_obj_track_times to determine if the dataset was\n    created with the track_times property.  If the leaf is not a\n    dataset, this will fail with HDF5ExtError.\n\n    The track times dataset creation property does not seem to survive\n    closing and reopening as of HDF5 1.8.17.  Currently, it may be\n    more accurate to test whether the ctime for the dataset is 0:\n    track_times = (leaf._get_obj_timestamps().ctime == 0)\n    ");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4Leaf_3_get_obj_track_times = {"_get_obj_track_times", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_3_get_obj_track_times, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_4Leaf_2_get_obj_track_times};
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_3_get_obj_track_times(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_obj_track_times (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_get_obj_track_times", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_get_obj_track_times", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4Leaf_2_get_obj_track_times(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_2_get_obj_track_times(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self) {
  hbool_t __pyx_v_track_times;
  hid_t __pyx_v_plist_id;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_t_9;
  char const *__pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_obj_track_times", 0);

  /* "tables/hdf5extension.pyx":1251
 *     """
 *     cdef:
 *       hbool_t track_times = True             # <<<<<<<<<<<<<<
 * 
 *     if self.dataset_id < 0:
*/
  __pyx_v_track_times = 1;

  /* "tables/hdf5extension.pyx":1253
 *       hbool_t track_times = True
 * 
 *     if self.dataset_id < 0:             # <<<<<<<<<<<<<<
 *       raise ValueError('Invalid dataset id %s' % self.dataset_id)
 * 
*/
  __pyx_t_1 = (__pyx_v_self->dataset_id < 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/hdf5extension.pyx":1254
 * 
 *     if self.dataset_id < 0:
 *       raise ValueError('Invalid dataset id %s' % self.dataset_id)             # <<<<<<<<<<<<<<
 * 
 *     plist_id = H5Dget_create_plist(self.dataset_id)
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = __Pyx_PyLong_From_hid_t(__pyx_v_self->dataset_id); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1254, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_Invalid_dataset_id_s, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1254, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1254, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1254, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1253
 *       hbool_t track_times = True
 * 
 *     if self.dataset_id < 0:             # <<<<<<<<<<<<<<
 *       raise ValueError('Invalid dataset id %s' % self.dataset_id)
 * 
*/
  }

  /* "tables/hdf5extension.pyx":1256
 *       raise ValueError('Invalid dataset id %s' % self.dataset_id)
 * 
 *     plist_id = H5Dget_create_plist(self.dataset_id)             # <<<<<<<<<<<<<<
 *     if plist_id < 0:
 *       raise HDF5ExtError("Could not get dataset creation property list "
*/
  __pyx_v_plist_id = H5Dget_create_plist(__pyx_v_self->dataset_id);

  /* "tables/hdf5extension.pyx":1257
 * 
 *     plist_id = H5Dget_create_plist(self.dataset_id)
 *     if plist_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Could not get dataset creation property list "
 *                          "from dataset id %s" % self.dataset_id)
*/
  __pyx_t_1 = (__pyx_v_plist_id < 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/hdf5extension.pyx":1258
 *     plist_id = H5Dget_create_plist(self.dataset_id)
 *     if plist_id < 0:
 *       raise HDF5ExtError("Could not get dataset creation property list "             # <<<<<<<<<<<<<<
 *                          "from dataset id %s" % self.dataset_id)
 * 
*/
    __pyx_t_5 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1258, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "tables/hdf5extension.pyx":1259
 *     if plist_id < 0:
 *       raise HDF5ExtError("Could not get dataset creation property list "
 *                          "from dataset id %s" % self.dataset_id)             # <<<<<<<<<<<<<<
 * 
 *     try:
*/
    __pyx_t_4 = __Pyx_PyLong_From_hid_t(__pyx_v_self->dataset_id); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1259, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_Could_not_get_dataset_creation_p, __pyx_t_4); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1259, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      assert(__pyx_t_5);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_7};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1258, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1258, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1257
 * 
 *     plist_id = H5Dget_create_plist(self.dataset_id)
 *     if plist_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Could not get dataset creation property list "
 *                          "from dataset id %s" % self.dataset_id)
*/
  }

  /* "tables/hdf5extension.pyx":1261
 *                          "from dataset id %s" % self.dataset_id)
 * 
 *     try:             # <<<<<<<<<<<<<<
 *       # Get track_times boolean for dataset
 *       if H5Pget_obj_track_times(plist_id, &track_times) < 0:
*/
  /*try:*/ {

    /* "tables/hdf5extension.pyx":1263
 *     try:
 *       # Get track_times boolean for dataset
 *       if H5Pget_obj_track_times(plist_id, &track_times) < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Could not get dataset track_times property "
 *                            "from dataset id %s" % self.dataset_id)
*/
    __pyx_t_1 = (H5Pget_obj_track_times(__pyx_v_plist_id, (&__pyx_v_track_times)) < 0);
    if (unlikely(__pyx_t_1)) {

      /* "tables/hdf5extension.pyx":1264
 *       # Get track_times boolean for dataset
 *       if H5Pget_obj_track_times(plist_id, &track_times) < 0:
 *         raise HDF5ExtError("Could not get dataset track_times property "             # <<<<<<<<<<<<<<
 *                            "from dataset id %s" % self.dataset_id)
 *     finally:
*/
      __pyx_t_3 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1264, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_7);

      /* "tables/hdf5extension.pyx":1265
 *       if H5Pget_obj_track_times(plist_id, &track_times) < 0:
 *         raise HDF5ExtError("Could not get dataset track_times property "
 *                            "from dataset id %s" % self.dataset_id)             # <<<<<<<<<<<<<<
 *     finally:
 *       H5Pclose(plist_id)
*/
      __pyx_t_5 = __Pyx_PyLong_From_hid_t(__pyx_v_self->dataset_id); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1265, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_Could_not_get_dataset_track_time, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1265, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_7);
        assert(__pyx_t_3);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_4};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1264, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 1264, __pyx_L6_error)

      /* "tables/hdf5extension.pyx":1263
 *     try:
 *       # Get track_times boolean for dataset
 *       if H5Pget_obj_track_times(plist_id, &track_times) < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Could not get dataset track_times property "
 *                            "from dataset id %s" % self.dataset_id)
*/
    }
  }

  /* "tables/hdf5extension.pyx":1267
 *                            "from dataset id %s" % self.dataset_id)
 *     finally:
 *       H5Pclose(plist_id)             # <<<<<<<<<<<<<<
 * 
 *     return bool(track_times)
*/
  /*finally:*/ {
    /*normal exit:*/{
      (void)(H5Pclose(__pyx_v_plist_id));
      goto __pyx_L7;
    }
    __pyx_L6_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16);
      if ( unlikely(__Pyx_GetException(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13) < 0)) __Pyx_ErrFetch(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __pyx_t_8 = __pyx_lineno; __pyx_t_9 = __pyx_clineno; __pyx_t_10 = __pyx_filename;
      {
        (void)(H5Pclose(__pyx_v_plist_id));
      }
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_ExceptionReset(__pyx_t_14, __pyx_t_15, __pyx_t_16);
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_ErrRestore(__pyx_t_11, __pyx_t_12, __pyx_t_13);
      __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0;
      __pyx_lineno = __pyx_t_8; __pyx_clineno = __pyx_t_9; __pyx_filename = __pyx_t_10;
      goto __pyx_L1_error;
    }
    __pyx_L7:;
  }

  /* "tables/hdf5extension.pyx":1269
 *       H5Pclose(plist_id)
 * 
 *     return bool(track_times)             # <<<<<<<<<<<<<<
 * 
 *   def _g_new(self, where, name, init):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyBool_FromLong((!(!(__pyx_v_track_times != 0)))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1238
 *       return H5Dget_storage_size(self.dataset_id)
 * 
 *   def _get_obj_track_times(self):             # <<<<<<<<<<<<<<
 *     """Get track_times boolean for dataset
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("tables.hdf5extension.Leaf._get_obj_track_times", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1271
 *     return bool(track_times)
 * 
 *   def _g_new(self, where, name, init):             # <<<<<<<<<<<<<<
 *     if init:
 *       # Put this info to 0 just when the class is initialized
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_5_g_new(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4Leaf_5_g_new = {"_g_new", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_5_g_new, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_5_g_new(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_where = 0;
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_init = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_new (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_where,&__pyx_mstate_global->__pyx_n_u_name,&__pyx_mstate_global->__pyx_n_u_init,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1271, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1271, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1271, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1271, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_new", 0) < (0)) __PYX_ERR(0, 1271, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_new", 1, 3, 3, i); __PYX_ERR(0, 1271, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1271, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1271, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1271, __pyx_L3_error)
    }
    __pyx_v_where = values[0];
    __pyx_v_name = values[1];
    __pyx_v_init = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_new", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 1271, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Leaf._g_new", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4Leaf_4_g_new(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self), __pyx_v_where, __pyx_v_name, __pyx_v_init);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_4_g_new(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self, PyObject *__pyx_v_where, PyObject *__pyx_v_name, PyObject *__pyx_v_init) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_new", 0);

  /* "tables/hdf5extension.pyx":1272
 * 
 *   def _g_new(self, where, name, init):
 *     if init:             # <<<<<<<<<<<<<<
 *       # Put this info to 0 just when the class is initialized
 *       self.dataset_id = -1
*/
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_init); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1272, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "tables/hdf5extension.pyx":1274
 *     if init:
 *       # Put this info to 0 just when the class is initialized
 *       self.dataset_id = -1             # <<<<<<<<<<<<<<
 *       self.type_id = -1
 *       self.base_type_id = -1
*/
    __pyx_v_self->dataset_id = -1LL;

    /* "tables/hdf5extension.pyx":1275
 *       # Put this info to 0 just when the class is initialized
 *       self.dataset_id = -1
 *       self.type_id = -1             # <<<<<<<<<<<<<<
 *       self.base_type_id = -1
 *       self.disk_type_id = -1
*/
    __pyx_v_self->type_id = -1LL;

    /* "tables/hdf5extension.pyx":1276
 *       self.dataset_id = -1
 *       self.type_id = -1
 *       self.base_type_id = -1             # <<<<<<<<<<<<<<
 *       self.disk_type_id = -1
 *     super()._g_new(where, name, init)
*/
    __pyx_v_self->base_type_id = -1LL;

    /* "tables/hdf5extension.pyx":1277
 *       self.type_id = -1
 *       self.base_type_id = -1
 *       self.disk_type_id = -1             # <<<<<<<<<<<<<<
 *     super()._g_new(where, name, init)
 * 
*/
    __pyx_v_self->disk_type_id = -1LL;

    /* "tables/hdf5extension.pyx":1272
 * 
 *   def _g_new(self, where, name, init):
 *     if init:             # <<<<<<<<<<<<<<
 *       # Put this info to 0 just when the class is initialized
 *       self.dataset_id = -1
*/
  }

  /* "tables/hdf5extension.pyx":1278
 *       self.base_type_id = -1
 *       self.disk_type_id = -1
 *     super()._g_new(where, name, init)             # <<<<<<<<<<<<<<
 * 
 *   def _g_chunk_info(self, ndarray coords):
*/
  __pyx_t_5 = NULL;
  __pyx_t_6 = 1;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_5, ((PyObject *)__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Leaf), ((PyObject *)__pyx_v_self)};
    __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_builtin_super, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1278, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
  }
  __pyx_t_3 = __pyx_t_4;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_6 = 0;
  {
    PyObject *__pyx_callargs[4] = {__pyx_t_3, __pyx_v_where, __pyx_v_name, __pyx_v_init};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_g_new, __pyx_callargs+__pyx_t_6, (4-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1278, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1271
 *     return bool(track_times)
 * 
 *   def _g_new(self, where, name, init):             # <<<<<<<<<<<<<<
 *     if init:
 *       # Put this info to 0 just when the class is initialized
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("tables.hdf5extension.Leaf._g_new", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1280
 *     super()._g_new(where, name, init)
 * 
 *   def _g_chunk_info(self, ndarray coords):             # <<<<<<<<<<<<<<
 *     """Get storage information about chunk at `coords`.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_7_g_chunk_info(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_4Leaf_6_g_chunk_info, "Get storage information about chunk at `coords`.\n\n    Return ``(filter_mask, offset, size)``, where items are ``None`` if the\n    chunk is missing.\n\n    ");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4Leaf_7_g_chunk_info = {"_g_chunk_info", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_7_g_chunk_info, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_4Leaf_6_g_chunk_info};
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_7_g_chunk_info(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyArrayObject *__pyx_v_coords = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_chunk_info (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_coords,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1280, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1280, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_chunk_info", 0) < (0)) __PYX_ERR(0, 1280, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_chunk_info", 1, 1, 1, i); __PYX_ERR(0, 1280, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1280, __pyx_L3_error)
    }
    __pyx_v_coords = ((PyArrayObject *)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_chunk_info", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1280, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Leaf._g_chunk_info", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_coords), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "coords", 0))) __PYX_ERR(0, 1280, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4Leaf_6_g_chunk_info(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self), __pyx_v_coords);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_6_g_chunk_info(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self, PyArrayObject *__pyx_v_coords) {
  herr_t __pyx_v_ret;
  hsize_t *__pyx_v_offset;
  unsigned int __pyx_v_filter_mask;
  haddr_t __pyx_v_addr;
  hsize_t __pyx_v_size;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_chunk_info", 0);

  /* "tables/hdf5extension.pyx":1294
 * 
 *     # Get the pointer to the buffer data area of the coords array
 *     with nogil:             # <<<<<<<<<<<<<<
 *       offset = <hsize_t *>PyArray_DATA(coords)
 *       ret = H5Dget_chunk_info_by_coord(self.dataset_id, offset,
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "tables/hdf5extension.pyx":1295
 *     # Get the pointer to the buffer data area of the coords array
 *     with nogil:
 *       offset = <hsize_t *>PyArray_DATA(coords)             # <<<<<<<<<<<<<<
 *       ret = H5Dget_chunk_info_by_coord(self.dataset_id, offset,
 *                                        &filter_mask, &addr, &size)
*/
        __pyx_v_offset = ((hsize_t *)PyArray_DATA(__pyx_v_coords));

        /* "tables/hdf5extension.pyx":1296
 *     with nogil:
 *       offset = <hsize_t *>PyArray_DATA(coords)
 *       ret = H5Dget_chunk_info_by_coord(self.dataset_id, offset,             # <<<<<<<<<<<<<<
 *                                        &filter_mask, &addr, &size)
 *     if ret < 0:
*/
        __pyx_v_ret = H5Dget_chunk_info_by_coord(__pyx_v_self->dataset_id, __pyx_v_offset, (&__pyx_v_filter_mask), (&__pyx_v_addr), (&__pyx_v_size));
      }

      /* "tables/hdf5extension.pyx":1294
 * 
 *     # Get the pointer to the buffer data area of the coords array
 *     with nogil:             # <<<<<<<<<<<<<<
 *       offset = <hsize_t *>PyArray_DATA(coords)
 *       ret = H5Dget_chunk_info_by_coord(self.dataset_id, offset,
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "tables/hdf5extension.pyx":1298
 *       ret = H5Dget_chunk_info_by_coord(self.dataset_id, offset,
 *                                        &filter_mask, &addr, &size)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems getting chunk info for ``%s``"
 *                          % self._v_pathname)
*/
  __pyx_t_1 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/hdf5extension.pyx":1299
 *                                        &filter_mask, &addr, &size)
 *     if ret < 0:
 *       raise HDF5ExtError("Problems getting chunk info for ``%s``"             # <<<<<<<<<<<<<<
 *                          % self._v_pathname)
 *     return ((filter_mask, addr, size) if addr != HADDR_UNDEF
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1299, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);

    /* "tables/hdf5extension.pyx":1300
 *     if ret < 0:
 *       raise HDF5ExtError("Problems getting chunk info for ``%s``"
 *                          % self._v_pathname)             # <<<<<<<<<<<<<<
 *     return ((filter_mask, addr, size) if addr != HADDR_UNDEF
 *             else (None, None, None))
*/
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_pathname); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1300, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Problems_getting_chunk_info_for, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1300, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_7 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_7 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1299, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1299, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1298
 *       ret = H5Dget_chunk_info_by_coord(self.dataset_id, offset,
 *                                        &filter_mask, &addr, &size)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems getting chunk info for ``%s``"
 *                          % self._v_pathname)
*/
  }

  /* "tables/hdf5extension.pyx":1301
 *       raise HDF5ExtError("Problems getting chunk info for ``%s``"
 *                          % self._v_pathname)
 *     return ((filter_mask, addr, size) if addr != HADDR_UNDEF             # <<<<<<<<<<<<<<
 *             else (None, None, None))
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = (__pyx_v_addr != HADDR_UNDEF);
  if (__pyx_t_1) {
    __pyx_t_4 = __Pyx_PyLong_From_unsigned_int(__pyx_v_filter_mask); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1301, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = __Pyx_PyLong_From_haddr_t(__pyx_v_addr); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1301, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_3 = __Pyx_PyLong_From_hsize_t(__pyx_v_size); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1301, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1301, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4) != (0)) __PYX_ERR(0, 1301, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_6);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6) != (0)) __PYX_ERR(0, 1301, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3) != (0)) __PYX_ERR(0, 1301, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_t_3 = 0;
    __pyx_t_2 = __pyx_t_5;
    __pyx_t_5 = 0;
  } else {

    /* "tables/hdf5extension.pyx":1302
 *                          % self._v_pathname)
 *     return ((filter_mask, addr, size) if addr != HADDR_UNDEF
 *             else (None, None, None))             # <<<<<<<<<<<<<<
 * 
 *   def _g_read_chunk(self, ndarray coords, ndarray out):
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_tuple[0]);
    __pyx_t_2 = __pyx_mstate_global->__pyx_tuple[0];
  }
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1280
 *     super()._g_new(where, name, init)
 * 
 *   def _g_chunk_info(self, ndarray coords):             # <<<<<<<<<<<<<<
 *     """Get storage information about chunk at `coords`.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("tables.hdf5extension.Leaf._g_chunk_info", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1304
 *             else (None, None, None))
 * 
 *   def _g_read_chunk(self, ndarray coords, ndarray out):             # <<<<<<<<<<<<<<
 *     """Read the raw chunk at `coords` (into `out`).
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_9_g_read_chunk(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_4Leaf_8_g_read_chunk, "Read the raw chunk at `coords` (into `out`).\n\n    Return a new array of bytes if `out` is ``None``, `out` itself otherwise.\n    Return ``None`` if the chunk is missing.\n\n    ");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4Leaf_9_g_read_chunk = {"_g_read_chunk", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_9_g_read_chunk, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_4Leaf_8_g_read_chunk};
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_9_g_read_chunk(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyArrayObject *__pyx_v_coords = 0;
  PyArrayObject *__pyx_v_out = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_read_chunk (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_coords,&__pyx_mstate_global->__pyx_n_u_out,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1304, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1304, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1304, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_read_chunk", 0) < (0)) __PYX_ERR(0, 1304, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_read_chunk", 1, 2, 2, i); __PYX_ERR(0, 1304, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1304, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1304, __pyx_L3_error)
    }
    __pyx_v_coords = ((PyArrayObject *)values[0]);
    __pyx_v_out = ((PyArrayObject *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_read_chunk", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1304, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Leaf._g_read_chunk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_coords), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "coords", 0))) __PYX_ERR(0, 1304, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_out), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "out", 0))) __PYX_ERR(0, 1304, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4Leaf_8_g_read_chunk(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self), __pyx_v_coords, __pyx_v_out);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_8_g_read_chunk(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self, PyArrayObject *__pyx_v_coords, PyArrayObject *__pyx_v_out) {
  PyArrayObject *__pyx_v_rarr = 0;
  herr_t __pyx_v_ret;
  hsize_t *__pyx_v_offset;
  uint32_t __pyx_v_filters;
  void *__pyx_v_rbuf;
  CYTHON_UNUSED PyObject *__pyx_v__ = NULL;
  PyObject *__pyx_v_addr = NULL;
  PyObject *__pyx_v_size = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *(*__pyx_t_7)(PyObject *);
  int __pyx_t_8;
  int __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11[4];
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_read_chunk", 0);

  /* "tables/hdf5extension.pyx":1314
 *     cdef herr_t ret
 *     cdef hsize_t *offset
 *     cdef uint32_t filters = 0             # <<<<<<<<<<<<<<
 *     cdef void *rbuf
 * 
*/
  __pyx_v_filters = 0;

  /* "tables/hdf5extension.pyx":1317
 *     cdef void *rbuf
 * 
 *     _, addr, size = self._g_chunk_info(coords)             # <<<<<<<<<<<<<<
 *     if addr is None:
 *       return None  # missing chunk
*/
  __pyx_t_2 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, ((PyObject *)__pyx_v_coords)};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_g_chunk_info, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1317, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 3)) {
      if (size > 3) __Pyx_RaiseTooManyValuesError(3);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 1317, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_2);
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_5 = PyTuple_GET_ITEM(sequence, 2);
      __Pyx_INCREF(__pyx_t_5);
    } else {
      __pyx_t_2 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1317, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1317, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyList_GetItemRefFast(sequence, 2, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1317, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_5);
    }
    #else
    __pyx_t_2 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1317, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1317, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1317, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_6 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1317, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_6);
    index = 0; __pyx_t_2 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_2)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 1; __pyx_t_4 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_4)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_4);
    index = 2; __pyx_t_5 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_5)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_5);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_6), 3) < (0)) __PYX_ERR(0, 1317, __pyx_L1_error)
    __pyx_t_7 = NULL;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 1317, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_v__ = __pyx_t_2;
  __pyx_t_2 = 0;
  __pyx_v_addr = __pyx_t_4;
  __pyx_t_4 = 0;
  __pyx_v_size = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "tables/hdf5extension.pyx":1318
 * 
 *     _, addr, size = self._g_chunk_info(coords)
 *     if addr is None:             # <<<<<<<<<<<<<<
 *       return None  # missing chunk
 *     if out is not None and len(out) < size:
*/
  __pyx_t_8 = (__pyx_v_addr == Py_None);
  if (__pyx_t_8) {

    /* "tables/hdf5extension.pyx":1319
 *     _, addr, size = self._g_chunk_info(coords)
 *     if addr is None:
 *       return None  # missing chunk             # <<<<<<<<<<<<<<
 *     if out is not None and len(out) < size:
 *       raise ValueError(f"Output buffer is too short: {len(out)} < {size}")
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "tables/hdf5extension.pyx":1318
 * 
 *     _, addr, size = self._g_chunk_info(coords)
 *     if addr is None:             # <<<<<<<<<<<<<<
 *       return None  # missing chunk
 *     if out is not None and len(out) < size:
*/
  }

  /* "tables/hdf5extension.pyx":1320
 *     if addr is None:
 *       return None  # missing chunk
 *     if out is not None and len(out) < size:             # <<<<<<<<<<<<<<
 *       raise ValueError(f"Output buffer is too short: {len(out)} < {size}")
 * 
*/
  __pyx_t_9 = (((PyObject *)__pyx_v_out) != Py_None);
  if (__pyx_t_9) {
  } else {
    __pyx_t_8 = __pyx_t_9;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_10 = PyObject_Length(((PyObject *)__pyx_v_out)); if (unlikely(__pyx_t_10 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1320, __pyx_L1_error)
  __pyx_t_1 = PyLong_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1320, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_1, __pyx_v_size, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1320, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 1320, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_8 = __pyx_t_9;
  __pyx_L7_bool_binop_done:;
  if (unlikely(__pyx_t_8)) {

    /* "tables/hdf5extension.pyx":1321
 *       return None  # missing chunk
 *     if out is not None and len(out) < size:
 *       raise ValueError(f"Output buffer is too short: {len(out)} < {size}")             # <<<<<<<<<<<<<<
 * 
 *     rarr = np.empty((size,), dtype='u1') if out is None else out
*/
    __pyx_t_1 = NULL;
    __pyx_t_10 = PyObject_Length(((PyObject *)__pyx_v_out)); if (unlikely(__pyx_t_10 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1321, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyUnicode_From_Py_ssize_t(__pyx_t_10, 0, ' ', 'd'); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1321, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_PyObject_FormatSimple(__pyx_v_size, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1321, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_11[0] = __pyx_mstate_global->__pyx_kp_u_Output_buffer_is_too_short;
    __pyx_t_11[1] = __pyx_t_4;
    __pyx_t_11[2] = __pyx_mstate_global->__pyx_kp_u__5;
    __pyx_t_11[3] = __pyx_t_2;
    __pyx_t_6 = __Pyx_PyUnicode_Join(__pyx_t_11, 4, 28 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4) + 3 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2));
    if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1321, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_3 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_t_6};
      __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1321, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 1321, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1320
 *     if addr is None:
 *       return None  # missing chunk
 *     if out is not None and len(out) < size:             # <<<<<<<<<<<<<<
 *       raise ValueError(f"Output buffer is too short: {len(out)} < {size}")
 * 
*/
  }

  /* "tables/hdf5extension.pyx":1323
 *       raise ValueError(f"Output buffer is too short: {len(out)} < {size}")
 * 
 *     rarr = np.empty((size,), dtype='u1') if out is None else out             # <<<<<<<<<<<<<<
 *     with nogil:
 *       rbuf = PyArray_DATA(rarr)
*/
  __pyx_t_8 = (((PyObject *)__pyx_v_out) == Py_None);
  if (__pyx_t_8) {
    __pyx_t_1 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1323, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1323, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1323, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_size);
    __Pyx_GIVEREF(__pyx_v_size);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_size) != (0)) __PYX_ERR(0, 1323, __pyx_L1_error);
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_1);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, __pyx_t_2};
      __pyx_t_12 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1323, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_mstate_global->__pyx_n_u_u1, __pyx_t_12, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 1323, __pyx_L1_error)
      __pyx_t_6 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_12);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1323, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
    }
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_mstate_global->__pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1323, __pyx_L1_error)
    __pyx_t_5 = __pyx_t_6;
    __pyx_t_6 = 0;
  } else {
    __Pyx_INCREF((PyObject *)__pyx_v_out);
    __pyx_t_5 = ((PyObject *)__pyx_v_out);
  }
  __pyx_v_rarr = ((PyArrayObject *)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "tables/hdf5extension.pyx":1324
 * 
 *     rarr = np.empty((size,), dtype='u1') if out is None else out
 *     with nogil:             # <<<<<<<<<<<<<<
 *       rbuf = PyArray_DATA(rarr)
 *       offset = <hsize_t *>PyArray_DATA(coords)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "tables/hdf5extension.pyx":1325
 *     rarr = np.empty((size,), dtype='u1') if out is None else out
 *     with nogil:
 *       rbuf = PyArray_DATA(rarr)             # <<<<<<<<<<<<<<
 *       offset = <hsize_t *>PyArray_DATA(coords)
 *       ret = H5Dread_chunk(self.dataset_id, H5P_DEFAULT, offset,
*/
        __pyx_v_rbuf = PyArray_DATA(__pyx_v_rarr);

        /* "tables/hdf5extension.pyx":1326
 *     with nogil:
 *       rbuf = PyArray_DATA(rarr)
 *       offset = <hsize_t *>PyArray_DATA(coords)             # <<<<<<<<<<<<<<
 *       ret = H5Dread_chunk(self.dataset_id, H5P_DEFAULT, offset,
 *                           &filters, rbuf)
*/
        __pyx_v_offset = ((hsize_t *)PyArray_DATA(__pyx_v_coords));

        /* "tables/hdf5extension.pyx":1327
 *       rbuf = PyArray_DATA(rarr)
 *       offset = <hsize_t *>PyArray_DATA(coords)
 *       ret = H5Dread_chunk(self.dataset_id, H5P_DEFAULT, offset,             # <<<<<<<<<<<<<<
 *                           &filters, rbuf)
 *     if ret < 0:
*/
        __pyx_v_ret = H5Dread_chunk(__pyx_v_self->dataset_id, H5P_DEFAULT, __pyx_v_offset, (&__pyx_v_filters), __pyx_v_rbuf);
      }

      /* "tables/hdf5extension.pyx":1324
 * 
 *     rarr = np.empty((size,), dtype='u1') if out is None else out
 *     with nogil:             # <<<<<<<<<<<<<<
 *       rbuf = PyArray_DATA(rarr)
 *       offset = <hsize_t *>PyArray_DATA(coords)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L11;
        }
        __pyx_L11:;
      }
  }

  /* "tables/hdf5extension.pyx":1329
 *       ret = H5Dread_chunk(self.dataset_id, H5P_DEFAULT, offset,
 *                           &filters, rbuf)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems reading chunk from ``%s``"
 *                          % self._v_pathname)
*/
  __pyx_t_8 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_8)) {

    /* "tables/hdf5extension.pyx":1330
 *                           &filters, rbuf)
 *     if ret < 0:
 *       raise HDF5ExtError("Problems reading chunk from ``%s``"             # <<<<<<<<<<<<<<
 *                          % self._v_pathname)
 *     return rarr
*/
    __pyx_t_6 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1330, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);

    /* "tables/hdf5extension.pyx":1331
 *     if ret < 0:
 *       raise HDF5ExtError("Problems reading chunk from ``%s``"
 *                          % self._v_pathname)             # <<<<<<<<<<<<<<
 *     return rarr
 * 
*/
    __pyx_t_12 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_pathname); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_2 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Problems_reading_chunk_from_s, __pyx_t_12); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_6);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_2};
      __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1330, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 1330, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1329
 *       ret = H5Dread_chunk(self.dataset_id, H5P_DEFAULT, offset,
 *                           &filters, rbuf)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems reading chunk from ``%s``"
 *                          % self._v_pathname)
*/
  }

  /* "tables/hdf5extension.pyx":1332
 *       raise HDF5ExtError("Problems reading chunk from ``%s``"
 *                          % self._v_pathname)
 *     return rarr             # <<<<<<<<<<<<<<
 * 
 *   def _g_write_chunk(self, ndarray coords, ndarray data, uint32_t filters):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_rarr);
  __pyx_r = ((PyObject *)__pyx_v_rarr);
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1304
 *             else (None, None, None))
 * 
 *   def _g_read_chunk(self, ndarray coords, ndarray out):             # <<<<<<<<<<<<<<
 *     """Read the raw chunk at `coords` (into `out`).
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("tables.hdf5extension.Leaf._g_read_chunk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rarr);
  __Pyx_XDECREF(__pyx_v__);
  __Pyx_XDECREF(__pyx_v_addr);
  __Pyx_XDECREF(__pyx_v_size);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1334
 *     return rarr
 * 
 *   def _g_write_chunk(self, ndarray coords, ndarray data, uint32_t filters):             # <<<<<<<<<<<<<<
 *     """Write the raw `data` to the chunk in `coords`.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_11_g_write_chunk(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_4Leaf_10_g_write_chunk, "Write the raw `data` to the chunk in `coords`.\n\n    The `filters` mask indicates which filters of the pipeline have not been\n    used to create the `data`.\n\n    ");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4Leaf_11_g_write_chunk = {"_g_write_chunk", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_11_g_write_chunk, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_4Leaf_10_g_write_chunk};
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_11_g_write_chunk(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyArrayObject *__pyx_v_coords = 0;
  PyArrayObject *__pyx_v_data = 0;
  uint32_t __pyx_v_filters;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_write_chunk (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_coords,&__pyx_mstate_global->__pyx_n_u_data,&__pyx_mstate_global->__pyx_n_u_filters,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1334, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1334, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1334, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1334, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_write_chunk", 0) < (0)) __PYX_ERR(0, 1334, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_write_chunk", 1, 3, 3, i); __PYX_ERR(0, 1334, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1334, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1334, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1334, __pyx_L3_error)
    }
    __pyx_v_coords = ((PyArrayObject *)values[0]);
    __pyx_v_data = ((PyArrayObject *)values[1]);
    __pyx_v_filters = __Pyx_PyLong_As_uint32_t(values[2]); if (unlikely((__pyx_v_filters == ((uint32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1334, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_write_chunk", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 1334, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Leaf._g_write_chunk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_coords), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "coords", 0))) __PYX_ERR(0, 1334, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "data", 0))) __PYX_ERR(0, 1334, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4Leaf_10_g_write_chunk(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self), __pyx_v_coords, __pyx_v_data, __pyx_v_filters);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_10_g_write_chunk(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self, PyArrayObject *__pyx_v_coords, PyArrayObject *__pyx_v_data, uint32_t __pyx_v_filters) {
  herr_t __pyx_v_ret;
  hsize_t *__pyx_v_offset;
  size_t __pyx_v_data_size;
  void *__pyx_v_wbuf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_write_chunk", 0);

  /* "tables/hdf5extension.pyx":1346
 *     cdef void *wbuf
 * 
 *     data_size = data.size             # <<<<<<<<<<<<<<
 *     with nogil:
 *       wbuf = PyArray_DATA(data)
*/
  __pyx_v_data_size = __pyx_f_5numpy_7ndarray_4size_size(__pyx_v_data);

  /* "tables/hdf5extension.pyx":1347
 * 
 *     data_size = data.size
 *     with nogil:             # <<<<<<<<<<<<<<
 *       wbuf = PyArray_DATA(data)
 *       offset = <hsize_t *>PyArray_DATA(coords)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "tables/hdf5extension.pyx":1348
 *     data_size = data.size
 *     with nogil:
 *       wbuf = PyArray_DATA(data)             # <<<<<<<<<<<<<<
 *       offset = <hsize_t *>PyArray_DATA(coords)
 *       ret = H5Dwrite_chunk(self.dataset_id, H5P_DEFAULT, filters,
*/
        __pyx_v_wbuf = PyArray_DATA(__pyx_v_data);

        /* "tables/hdf5extension.pyx":1349
 *     with nogil:
 *       wbuf = PyArray_DATA(data)
 *       offset = <hsize_t *>PyArray_DATA(coords)             # <<<<<<<<<<<<<<
 *       ret = H5Dwrite_chunk(self.dataset_id, H5P_DEFAULT, filters,
 *                            offset, data_size, wbuf)
*/
        __pyx_v_offset = ((hsize_t *)PyArray_DATA(__pyx_v_coords));

        /* "tables/hdf5extension.pyx":1350
 *       wbuf = PyArray_DATA(data)
 *       offset = <hsize_t *>PyArray_DATA(coords)
 *       ret = H5Dwrite_chunk(self.dataset_id, H5P_DEFAULT, filters,             # <<<<<<<<<<<<<<
 *                            offset, data_size, wbuf)
 *     if ret < 0:
*/
        __pyx_v_ret = H5Dwrite_chunk(__pyx_v_self->dataset_id, H5P_DEFAULT, __pyx_v_filters, __pyx_v_offset, __pyx_v_data_size, __pyx_v_wbuf);
      }

      /* "tables/hdf5extension.pyx":1347
 * 
 *     data_size = data.size
 *     with nogil:             # <<<<<<<<<<<<<<
 *       wbuf = PyArray_DATA(data)
 *       offset = <hsize_t *>PyArray_DATA(coords)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "tables/hdf5extension.pyx":1352
 *       ret = H5Dwrite_chunk(self.dataset_id, H5P_DEFAULT, filters,
 *                            offset, data_size, wbuf)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems writing chunk to ``%s``"
 *                          % self._v_pathname)
*/
  __pyx_t_1 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/hdf5extension.pyx":1353
 *                            offset, data_size, wbuf)
 *     if ret < 0:
 *       raise HDF5ExtError("Problems writing chunk to ``%s``"             # <<<<<<<<<<<<<<
 *                          % self._v_pathname)
 * 
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1353, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);

    /* "tables/hdf5extension.pyx":1354
 *     if ret < 0:
 *       raise HDF5ExtError("Problems writing chunk to ``%s``"
 *                          % self._v_pathname)             # <<<<<<<<<<<<<<
 * 
 *   cdef _get_type_ids(self):
*/
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_pathname); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1354, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Problems_writing_chunk_to_s, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1354, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_7 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_7 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1353, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1353, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1352
 *       ret = H5Dwrite_chunk(self.dataset_id, H5P_DEFAULT, filters,
 *                            offset, data_size, wbuf)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems writing chunk to ``%s``"
 *                          % self._v_pathname)
*/
  }

  /* "tables/hdf5extension.pyx":1334
 *     return rarr
 * 
 *   def _g_write_chunk(self, ndarray coords, ndarray data, uint32_t filters):             # <<<<<<<<<<<<<<
 *     """Write the raw `data` to the chunk in `coords`.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("tables.hdf5extension.Leaf._g_write_chunk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1356
 *                          % self._v_pathname)
 * 
 *   cdef _get_type_ids(self):             # <<<<<<<<<<<<<<
 *     """Get the disk and native HDF5 types associated with this leaf.
 * 
*/

static PyObject *__pyx_f_6tables_13hdf5extension_4Leaf__get_type_ids(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self) {
  hid_t __pyx_v_disk_type_id;
  hid_t __pyx_v_native_type_id;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  hid_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_type_ids", 0);

  /* "tables/hdf5extension.pyx":1366
 *     cdef hid_t disk_type_id, native_type_id
 * 
 *     disk_type_id = H5Dget_type(self.dataset_id)             # <<<<<<<<<<<<<<
 *     native_type_id = get_native_type(disk_type_id)
 *     return disk_type_id, native_type_id
*/
  __pyx_v_disk_type_id = H5Dget_type(__pyx_v_self->dataset_id);

  /* "tables/hdf5extension.pyx":1367
 * 
 *     disk_type_id = H5Dget_type(self.dataset_id)
 *     native_type_id = get_native_type(disk_type_id)             # <<<<<<<<<<<<<<
 *     return disk_type_id, native_type_id
 * 
*/
  __pyx_t_1 = __pyx_f_6tables_14utilsextension_get_native_type(__pyx_v_disk_type_id); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1367, __pyx_L1_error)
  __pyx_v_native_type_id = __pyx_t_1;

  /* "tables/hdf5extension.pyx":1368
 *     disk_type_id = H5Dget_type(self.dataset_id)
 *     native_type_id = get_native_type(disk_type_id)
 *     return disk_type_id, native_type_id             # <<<<<<<<<<<<<<
 * 
 *   cdef _convert_time64(self, ndarray nparr, int sense):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyLong_From_hid_t(__pyx_v_disk_type_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1368, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyLong_From_hid_t(__pyx_v_native_type_id); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1368, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1368, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2) != (0)) __PYX_ERR(0, 1368, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3) != (0)) __PYX_ERR(0, 1368, __pyx_L1_error);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1356
 *                          % self._v_pathname)
 * 
 *   cdef _get_type_ids(self):             # <<<<<<<<<<<<<<
 *     """Get the disk and native HDF5 types associated with this leaf.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("tables.hdf5extension.Leaf._get_type_ids", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1370
 *     return disk_type_id, native_type_id
 * 
 *   cdef _convert_time64(self, ndarray nparr, int sense):             # <<<<<<<<<<<<<<
 *     """Converts a NumPy of Time64 elements between NumPy and HDF5 formats.
 * 
*/

static PyObject *__pyx_f_6tables_13hdf5extension_4Leaf__convert_time64(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self, PyArrayObject *__pyx_v_nparr, int __pyx_v_sense) {
  void *__pyx_v_t64buf;
  long __pyx_v_byteoffset;
  long __pyx_v_bytestride;
  long __pyx_v_nelements;
  hsize_t __pyx_v_nrecords;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_convert_time64", 0);

  /* "tables/hdf5extension.pyx":1383
 *     cdef hsize_t nrecords
 * 
 *     byteoffset = 0   # NumPy objects doesn't have an offset             # <<<<<<<<<<<<<<
 *     if (<object>nparr).shape == ():
 *       # 0-dim array does contain *one* element
*/
  __pyx_v_byteoffset = 0;

  /* "tables/hdf5extension.pyx":1384
 * 
 *     byteoffset = 0   # NumPy objects doesn't have an offset
 *     if (<object>nparr).shape == ():             # <<<<<<<<<<<<<<
 *       # 0-dim array does contain *one* element
 *       nrecords = 1
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_nparr), __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1384, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_mstate_global->__pyx_empty_tuple, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1384, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1384, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":1386
 *     if (<object>nparr).shape == ():
 *       # 0-dim array does contain *one* element
 *       nrecords = 1             # <<<<<<<<<<<<<<
 *       bytestride = 8
 *     else:
*/
    __pyx_v_nrecords = 1;

    /* "tables/hdf5extension.pyx":1387
 *       # 0-dim array does contain *one* element
 *       nrecords = 1
 *       bytestride = 8             # <<<<<<<<<<<<<<
 *     else:
 *       nrecords = len(nparr)
*/
    __pyx_v_bytestride = 8;

    /* "tables/hdf5extension.pyx":1384
 * 
 *     byteoffset = 0   # NumPy objects doesn't have an offset
 *     if (<object>nparr).shape == ():             # <<<<<<<<<<<<<<
 *       # 0-dim array does contain *one* element
 *       nrecords = 1
*/
    goto __pyx_L3;
  }

  /* "tables/hdf5extension.pyx":1389
 *       bytestride = 8
 *     else:
 *       nrecords = len(nparr)             # <<<<<<<<<<<<<<
 *       bytestride = PyArray_STRIDE(nparr, 0)  # supports multi-dimensional recarray
 *     nelements = <size_t>nparr.size // nrecords
*/
  /*else*/ {
    __pyx_t_4 = PyObject_Length(((PyObject *)__pyx_v_nparr)); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1389, __pyx_L1_error)
    __pyx_v_nrecords = __pyx_t_4;

    /* "tables/hdf5extension.pyx":1390
 *     else:
 *       nrecords = len(nparr)
 *       bytestride = PyArray_STRIDE(nparr, 0)  # supports multi-dimensional recarray             # <<<<<<<<<<<<<<
 *     nelements = <size_t>nparr.size // nrecords
 *     t64buf = PyArray_DATA(nparr)
*/
    __pyx_v_bytestride = PyArray_STRIDE(__pyx_v_nparr, 0);
  }
  __pyx_L3:;

  /* "tables/hdf5extension.pyx":1391
 *       nrecords = len(nparr)
 *       bytestride = PyArray_STRIDE(nparr, 0)  # supports multi-dimensional recarray
 *     nelements = <size_t>nparr.size // nrecords             # <<<<<<<<<<<<<<
 *     t64buf = PyArray_DATA(nparr)
 * 
*/
  __pyx_t_5 = ((size_t)__pyx_f_5numpy_7ndarray_4size_size(__pyx_v_nparr));
  if (unlikely(__pyx_v_nrecords == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    __PYX_ERR(0, 1391, __pyx_L1_error)
  }
  __pyx_v_nelements = (__pyx_t_5 / __pyx_v_nrecords);

  /* "tables/hdf5extension.pyx":1392
 *       bytestride = PyArray_STRIDE(nparr, 0)  # supports multi-dimensional recarray
 *     nelements = <size_t>nparr.size // nrecords
 *     t64buf = PyArray_DATA(nparr)             # <<<<<<<<<<<<<<
 * 
 *     conv_float64_timeval32(
*/
  __pyx_v_t64buf = PyArray_DATA(__pyx_v_nparr);

  /* "tables/hdf5extension.pyx":1394
 *     t64buf = PyArray_DATA(nparr)
 * 
 *     conv_float64_timeval32(             # <<<<<<<<<<<<<<
 *       t64buf, byteoffset, bytestride, nrecords, nelements, sense)
 * 
*/
  conv_float64_timeval32(__pyx_v_t64buf, __pyx_v_byteoffset, __pyx_v_bytestride, __pyx_v_nrecords, __pyx_v_nelements, __pyx_v_sense);

  /* "tables/hdf5extension.pyx":1370
 *     return disk_type_id, native_type_id
 * 
 *   cdef _convert_time64(self, ndarray nparr, int sense):             # <<<<<<<<<<<<<<
 *     """Converts a NumPy of Time64 elements between NumPy and HDF5 formats.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("tables.hdf5extension.Leaf._convert_time64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1399
 *   # can't do since cdef'd
 * 
 *   def _g_truncate(self, hsize_t size):             # <<<<<<<<<<<<<<
 *     """Truncate a Leaf to `size` nrows."""
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_13_g_truncate(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_4Leaf_12_g_truncate, "Truncate a Leaf to `size` nrows.");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4Leaf_13_g_truncate = {"_g_truncate", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_13_g_truncate, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_4Leaf_12_g_truncate};
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_13_g_truncate(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  hsize_t __pyx_v_size;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_truncate (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1399, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1399, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_truncate", 0) < (0)) __PYX_ERR(0, 1399, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_truncate", 1, 1, 1, i); __PYX_ERR(0, 1399, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1399, __pyx_L3_error)
    }
    __pyx_v_size = __Pyx_PyLong_As_hsize_t(values[0]); if (unlikely((__pyx_v_size == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1399, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_truncate", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1399, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Leaf._g_truncate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4Leaf_12_g_truncate(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self), __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_12_g_truncate(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self, hsize_t __pyx_v_size) {
  hsize_t __pyx_v_ret;
  PyObject *__pyx_v_classname = NULL;
  PyObject *__pyx_v_shape = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_truncate", 0);

  /* "tables/hdf5extension.pyx":1404
 *     cdef hsize_t ret
 * 
 *     ret = truncate_dset(self.dataset_id, self.maindim, size)             # <<<<<<<<<<<<<<
 *     if ret < 0:
 *       raise HDF5ExtError("Problems truncating the leaf: %s" % self)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_maindim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1404, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1404, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ret = truncate_dset(__pyx_v_self->dataset_id, __pyx_t_2, __pyx_v_size);

  /* "tables/hdf5extension.pyx":1405
 * 
 *     ret = truncate_dset(self.dataset_id, self.maindim, size)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems truncating the leaf: %s" % self)
 * 
*/
  __pyx_t_3 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_3)) {

    /* "tables/hdf5extension.pyx":1406
 *     ret = truncate_dset(self.dataset_id, self.maindim, size)
 *     if ret < 0:
 *       raise HDF5ExtError("Problems truncating the leaf: %s" % self)             # <<<<<<<<<<<<<<
 * 
 *     classname = self.__class__.__name__
*/
    __pyx_t_4 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1406, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_Problems_truncating_the_leaf_s, ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1406, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_7 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_6};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1406, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1406, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1405
 * 
 *     ret = truncate_dset(self.dataset_id, self.maindim, size)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems truncating the leaf: %s" % self)
 * 
*/
  }

  /* "tables/hdf5extension.pyx":1408
 *       raise HDF5ExtError("Problems truncating the leaf: %s" % self)
 * 
 *     classname = self.__class__.__name__             # <<<<<<<<<<<<<<
 *     if classname in ('EArray', 'CArray'):
 *       # Update the new dimensionality
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_class); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_name_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_classname = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "tables/hdf5extension.pyx":1409
 * 
 *     classname = self.__class__.__name__
 *     if classname in ('EArray', 'CArray'):             # <<<<<<<<<<<<<<
 *       # Update the new dimensionality
 *       self.dims[self.maindim] = size
*/
  __Pyx_INCREF(__pyx_v_classname);
  __pyx_t_5 = __pyx_v_classname;
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_EArray, Py_EQ)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 1409, __pyx_L1_error)
  if (!__pyx_t_8) {
  } else {
    __pyx_t_3 = __pyx_t_8;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_CArray, Py_EQ)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 1409, __pyx_L1_error)
  __pyx_t_3 = __pyx_t_8;
  __pyx_L5_bool_binop_done:;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_8 = __pyx_t_3;
  if (__pyx_t_8) {

    /* "tables/hdf5extension.pyx":1411
 *     if classname in ('EArray', 'CArray'):
 *       # Update the new dimensionality
 *       self.dims[self.maindim] = size             # <<<<<<<<<<<<<<
 *       # Update the shape
 *       shape = list(self.shape)
*/
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_maindim); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_9 = __Pyx_PyIndex_AsSsize_t(__pyx_t_5); if (unlikely((__pyx_t_9 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1411, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    (__pyx_v_self->dims[__pyx_t_9]) = __pyx_v_size;

    /* "tables/hdf5extension.pyx":1413
 *       self.dims[self.maindim] = size
 *       # Update the shape
 *       shape = list(self.shape)             # <<<<<<<<<<<<<<
 *       shape[self.maindim] = SizeType(size)
 *       self.shape = tuple(shape)
*/
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1413, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = __Pyx_PySequence_ListKeepNew(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1413, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_shape = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":1414
 *       # Update the shape
 *       shape = list(self.shape)
 *       shape[self.maindim] = SizeType(size)             # <<<<<<<<<<<<<<
 *       self.shape = tuple(shape)
 *     elif classname in ('Table', 'VLArray'):
*/
    __pyx_t_5 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_SizeType); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1414, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_4 = __Pyx_PyLong_From_hsize_t(__pyx_v_size); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1414, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_5);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_7 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_4};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1414, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_maindim); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1414, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (unlikely((PyObject_SetItem(__pyx_v_shape, __pyx_t_6, __pyx_t_1) < 0))) __PYX_ERR(0, 1414, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":1415
 *       shape = list(self.shape)
 *       shape[self.maindim] = SizeType(size)
 *       self.shape = tuple(shape)             # <<<<<<<<<<<<<<
 *     elif classname in ('Table', 'VLArray'):
 *       self.nrows = size
*/
    __pyx_t_1 = PyList_AsTuple(__pyx_v_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1415, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_shape, __pyx_t_1) < (0)) __PYX_ERR(0, 1415, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":1409
 * 
 *     classname = self.__class__.__name__
 *     if classname in ('EArray', 'CArray'):             # <<<<<<<<<<<<<<
 *       # Update the new dimensionality
 *       self.dims[self.maindim] = size
*/
    goto __pyx_L4;
  }

  /* "tables/hdf5extension.pyx":1416
 *       shape[self.maindim] = SizeType(size)
 *       self.shape = tuple(shape)
 *     elif classname in ('Table', 'VLArray'):             # <<<<<<<<<<<<<<
 *       self.nrows = size
 *     else:
*/
  __Pyx_INCREF(__pyx_v_classname);
  __pyx_t_1 = __pyx_v_classname;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_Table, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1416, __pyx_L1_error)
  if (!__pyx_t_3) {
  } else {
    __pyx_t_8 = __pyx_t_3;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_VLArray, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1416, __pyx_L1_error)
  __pyx_t_8 = __pyx_t_3;
  __pyx_L7_bool_binop_done:;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __pyx_t_8;
  if (likely(__pyx_t_3)) {

    /* "tables/hdf5extension.pyx":1417
 *       self.shape = tuple(shape)
 *     elif classname in ('Table', 'VLArray'):
 *       self.nrows = size             # <<<<<<<<<<<<<<
 *     else:
 *       raise ValueError("Unexpected classname: %s" % classname)
*/
    __pyx_t_1 = __Pyx_PyLong_From_hsize_t(__pyx_v_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1417, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_nrows, __pyx_t_1) < (0)) __PYX_ERR(0, 1417, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":1416
 *       shape[self.maindim] = SizeType(size)
 *       self.shape = tuple(shape)
 *     elif classname in ('Table', 'VLArray'):             # <<<<<<<<<<<<<<
 *       self.nrows = size
 *     else:
*/
    goto __pyx_L4;
  }

  /* "tables/hdf5extension.pyx":1419
 *       self.nrows = size
 *     else:
 *       raise ValueError("Unexpected classname: %s" % classname)             # <<<<<<<<<<<<<<
 * 
 *   def _g_flush(self):
*/
  /*else*/ {
    __pyx_t_6 = NULL;
    __pyx_t_4 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Unexpected_classname_s, __pyx_v_classname); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1419, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_4};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1419, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1419, __pyx_L1_error)
  }
  __pyx_L4:;

  /* "tables/hdf5extension.pyx":1399
 *   # can't do since cdef'd
 * 
 *   def _g_truncate(self, hsize_t size):             # <<<<<<<<<<<<<<
 *     """Truncate a Leaf to `size` nrows."""
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("tables.hdf5extension.Leaf._g_truncate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_classname);
  __Pyx_XDECREF(__pyx_v_shape);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1421
 *       raise ValueError("Unexpected classname: %s" % classname)
 * 
 *   def _g_flush(self):             # <<<<<<<<<<<<<<
 *     # Flush the dataset (in fact, the entire buffers in file!)
 *     if self.dataset_id >= 0:
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_15_g_flush(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4Leaf_15_g_flush = {"_g_flush", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_15_g_flush, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_15_g_flush(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_flush (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_g_flush", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_g_flush", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4Leaf_14_g_flush(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_14_g_flush(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("_g_flush", 0);

  /* "tables/hdf5extension.pyx":1423
 *   def _g_flush(self):
 *     # Flush the dataset (in fact, the entire buffers in file!)
 *     if self.dataset_id >= 0:             # <<<<<<<<<<<<<<
 *         H5Fflush(self.dataset_id, H5F_SCOPE_GLOBAL)
 * 
*/
  __pyx_t_1 = (__pyx_v_self->dataset_id >= 0);
  if (__pyx_t_1) {

    /* "tables/hdf5extension.pyx":1424
 *     # Flush the dataset (in fact, the entire buffers in file!)
 *     if self.dataset_id >= 0:
 *         H5Fflush(self.dataset_id, H5F_SCOPE_GLOBAL)             # <<<<<<<<<<<<<<
 * 
 *   def _g_close(self):
*/
    (void)(H5Fflush(__pyx_v_self->dataset_id, H5F_SCOPE_GLOBAL));

    /* "tables/hdf5extension.pyx":1423
 *   def _g_flush(self):
 *     # Flush the dataset (in fact, the entire buffers in file!)
 *     if self.dataset_id >= 0:             # <<<<<<<<<<<<<<
 *         H5Fflush(self.dataset_id, H5F_SCOPE_GLOBAL)
 * 
*/
  }

  /* "tables/hdf5extension.pyx":1421
 *       raise ValueError("Unexpected classname: %s" % classname)
 * 
 *   def _g_flush(self):             # <<<<<<<<<<<<<<
 *     # Flush the dataset (in fact, the entire buffers in file!)
 *     if self.dataset_id >= 0:
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1426
 *         H5Fflush(self.dataset_id, H5F_SCOPE_GLOBAL)
 * 
 *   def _g_close(self):             # <<<<<<<<<<<<<<
 *     # Close dataset in HDF5 space
 *     # Release resources
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_17_g_close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4Leaf_17_g_close = {"_g_close", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_17_g_close, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_17_g_close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_close (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_g_close", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_g_close", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4Leaf_16_g_close(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_16_g_close(struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("_g_close", 0);

  /* "tables/hdf5extension.pyx":1429
 *     # Close dataset in HDF5 space
 *     # Release resources
 *     if self.type_id >= 0:             # <<<<<<<<<<<<<<
 *       H5Tclose(self.type_id)
 *     if self.disk_type_id >= 0:
*/
  __pyx_t_1 = (__pyx_v_self->type_id >= 0);
  if (__pyx_t_1) {

    /* "tables/hdf5extension.pyx":1430
 *     # Release resources
 *     if self.type_id >= 0:
 *       H5Tclose(self.type_id)             # <<<<<<<<<<<<<<
 *     if self.disk_type_id >= 0:
 *       H5Tclose(self.disk_type_id)
*/
    (void)(H5Tclose(__pyx_v_self->type_id));

    /* "tables/hdf5extension.pyx":1429
 *     # Close dataset in HDF5 space
 *     # Release resources
 *     if self.type_id >= 0:             # <<<<<<<<<<<<<<
 *       H5Tclose(self.type_id)
 *     if self.disk_type_id >= 0:
*/
  }

  /* "tables/hdf5extension.pyx":1431
 *     if self.type_id >= 0:
 *       H5Tclose(self.type_id)
 *     if self.disk_type_id >= 0:             # <<<<<<<<<<<<<<
 *       H5Tclose(self.disk_type_id)
 *     if self.base_type_id >= 0:
*/
  __pyx_t_1 = (__pyx_v_self->disk_type_id >= 0);
  if (__pyx_t_1) {

    /* "tables/hdf5extension.pyx":1432
 *       H5Tclose(self.type_id)
 *     if self.disk_type_id >= 0:
 *       H5Tclose(self.disk_type_id)             # <<<<<<<<<<<<<<
 *     if self.base_type_id >= 0:
 *       H5Tclose(self.base_type_id)
*/
    (void)(H5Tclose(__pyx_v_self->disk_type_id));

    /* "tables/hdf5extension.pyx":1431
 *     if self.type_id >= 0:
 *       H5Tclose(self.type_id)
 *     if self.disk_type_id >= 0:             # <<<<<<<<<<<<<<
 *       H5Tclose(self.disk_type_id)
 *     if self.base_type_id >= 0:
*/
  }

  /* "tables/hdf5extension.pyx":1433
 *     if self.disk_type_id >= 0:
 *       H5Tclose(self.disk_type_id)
 *     if self.base_type_id >= 0:             # <<<<<<<<<<<<<<
 *       H5Tclose(self.base_type_id)
 *     if self.dataset_id >= 0:
*/
  __pyx_t_1 = (__pyx_v_self->base_type_id >= 0);
  if (__pyx_t_1) {

    /* "tables/hdf5extension.pyx":1434
 *       H5Tclose(self.disk_type_id)
 *     if self.base_type_id >= 0:
 *       H5Tclose(self.base_type_id)             # <<<<<<<<<<<<<<
 *     if self.dataset_id >= 0:
 *       H5Dclose(self.dataset_id)
*/
    (void)(H5Tclose(__pyx_v_self->base_type_id));

    /* "tables/hdf5extension.pyx":1433
 *     if self.disk_type_id >= 0:
 *       H5Tclose(self.disk_type_id)
 *     if self.base_type_id >= 0:             # <<<<<<<<<<<<<<
 *       H5Tclose(self.base_type_id)
 *     if self.dataset_id >= 0:
*/
  }

  /* "tables/hdf5extension.pyx":1435
 *     if self.base_type_id >= 0:
 *       H5Tclose(self.base_type_id)
 *     if self.dataset_id >= 0:             # <<<<<<<<<<<<<<
 *       H5Dclose(self.dataset_id)
 * 
*/
  __pyx_t_1 = (__pyx_v_self->dataset_id >= 0);
  if (__pyx_t_1) {

    /* "tables/hdf5extension.pyx":1436
 *       H5Tclose(self.base_type_id)
 *     if self.dataset_id >= 0:
 *       H5Dclose(self.dataset_id)             # <<<<<<<<<<<<<<
 * 
 * 
*/
    (void)(H5Dclose(__pyx_v_self->dataset_id));

    /* "tables/hdf5extension.pyx":1435
 *     if self.base_type_id >= 0:
 *       H5Tclose(self.base_type_id)
 *     if self.dataset_id >= 0:             # <<<<<<<<<<<<<<
 *       H5Dclose(self.dataset_id)
 * 
*/
  }

  /* "tables/hdf5extension.pyx":1426
 *         H5Fflush(self.dataset_id, H5F_SCOPE_GLOBAL)
 * 
 *   def _g_close(self):             # <<<<<<<<<<<<<<
 *     # Close dataset in HDF5 space
 *     # Release resources
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_19__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4Leaf_19__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_19__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_19__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4Leaf_18__reduce_cython__(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_18__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self_dims_cannot_be_converted_to, 0, 0);
  __PYX_ERR(2, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("tables.hdf5extension.Leaf.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_21__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_4Leaf_21__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_21__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_4Leaf_21__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(2, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(2, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(2, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(2, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Leaf.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4Leaf_20__setstate_cython__(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4Leaf_20__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_Leaf *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self_dims_cannot_be_converted_to, 0, 0);
  __PYX_ERR(2, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("tables.hdf5extension.Leaf.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1439
 * 
 * 
 * cdef void* _array_data(ndarray arr):             # <<<<<<<<<<<<<<
 *     # When the object is not a 0-d ndarray and its strides == 0, that
 *     # means that the array does not contain actual data
*/

static void *__pyx_f_6tables_13hdf5extension__array_data(PyArrayObject *__pyx_v_arr) {
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_ndim;
  void *__pyx_r;
  int __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;
  npy_intp __pyx_t_4;

  /* "tables/hdf5extension.pyx":1444
 *     cdef npy_intp i, ndim
 * 
 *     ndim = PyArray_NDIM(arr)             # <<<<<<<<<<<<<<
 *     if ndim == 0:
 *         return PyArray_DATA(arr)
*/
  __pyx_v_ndim = PyArray_NDIM(__pyx_v_arr);

  /* "tables/hdf5extension.pyx":1445
 * 
 *     ndim = PyArray_NDIM(arr)
 *     if ndim == 0:             # <<<<<<<<<<<<<<
 *         return PyArray_DATA(arr)
 *     for i in range(ndim):
*/
  __pyx_t_1 = (__pyx_v_ndim == 0);
  if (__pyx_t_1) {

    /* "tables/hdf5extension.pyx":1446
 *     ndim = PyArray_NDIM(arr)
 *     if ndim == 0:
 *         return PyArray_DATA(arr)             # <<<<<<<<<<<<<<
 *     for i in range(ndim):
 *         if PyArray_STRIDE(arr, i) > 0:
*/
    __pyx_r = PyArray_DATA(__pyx_v_arr);
    goto __pyx_L0;

    /* "tables/hdf5extension.pyx":1445
 * 
 *     ndim = PyArray_NDIM(arr)
 *     if ndim == 0:             # <<<<<<<<<<<<<<
 *         return PyArray_DATA(arr)
 *     for i in range(ndim):
*/
  }

  /* "tables/hdf5extension.pyx":1447
 *     if ndim == 0:
 *         return PyArray_DATA(arr)
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if PyArray_STRIDE(arr, i) > 0:
 *             return PyArray_DATA(arr)
*/
  __pyx_t_2 = __pyx_v_ndim;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "tables/hdf5extension.pyx":1448
 *         return PyArray_DATA(arr)
 *     for i in range(ndim):
 *         if PyArray_STRIDE(arr, i) > 0:             # <<<<<<<<<<<<<<
 *             return PyArray_DATA(arr)
 *     return NULL
*/
    __pyx_t_1 = (PyArray_STRIDE(__pyx_v_arr, __pyx_v_i) > 0);
    if (__pyx_t_1) {

      /* "tables/hdf5extension.pyx":1449
 *     for i in range(ndim):
 *         if PyArray_STRIDE(arr, i) > 0:
 *             return PyArray_DATA(arr)             # <<<<<<<<<<<<<<
 *     return NULL
 * 
*/
      __pyx_r = PyArray_DATA(__pyx_v_arr);
      goto __pyx_L0;

      /* "tables/hdf5extension.pyx":1448
 *         return PyArray_DATA(arr)
 *     for i in range(ndim):
 *         if PyArray_STRIDE(arr, i) > 0:             # <<<<<<<<<<<<<<
 *             return PyArray_DATA(arr)
 *     return NULL
*/
    }
  }

  /* "tables/hdf5extension.pyx":1450
 *         if PyArray_STRIDE(arr, i) > 0:
 *             return PyArray_DATA(arr)
 *     return NULL             # <<<<<<<<<<<<<<
 * 
 * def _supports_opt_blosc2_read_write(byteorder, filter_list, file_mode):
*/
  __pyx_r = NULL;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1439
 * 
 * 
 * cdef void* _array_data(ndarray arr):             # <<<<<<<<<<<<<<
 *     # When the object is not a 0-d ndarray and its strides == 0, that
 *     # means that the array does not contain actual data
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1452
 *     return NULL
 * 
 * def _supports_opt_blosc2_read_write(byteorder, filter_list, file_mode):             # <<<<<<<<<<<<<<
 *     if len(filter_list) == 1:  # Blosc2 must be the only filter
 *       opt_write = ((byteorder == sys.byteorder)
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_1_supports_opt_blosc2_read_write(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_1_supports_opt_blosc2_read_write = {"_supports_opt_blosc2_read_write", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_1_supports_opt_blosc2_read_write, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_1_supports_opt_blosc2_read_write(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_byteorder = 0;
  PyObject *__pyx_v_filter_list = 0;
  PyObject *__pyx_v_file_mode = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_supports_opt_blosc2_read_write (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_byteorder,&__pyx_mstate_global->__pyx_n_u_filter_list,&__pyx_mstate_global->__pyx_n_u_file_mode,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1452, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1452, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1452, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1452, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_supports_opt_blosc2_read_write", 0) < (0)) __PYX_ERR(0, 1452, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_supports_opt_blosc2_read_write", 1, 3, 3, i); __PYX_ERR(0, 1452, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1452, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1452, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1452, __pyx_L3_error)
    }
    __pyx_v_byteorder = values[0];
    __pyx_v_filter_list = values[1];
    __pyx_v_file_mode = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_supports_opt_blosc2_read_write", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 1452, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension._supports_opt_blosc2_read_write", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension__supports_opt_blosc2_read_write(__pyx_self, __pyx_v_byteorder, __pyx_v_filter_list, __pyx_v_file_mode);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension__supports_opt_blosc2_read_write(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_byteorder, PyObject *__pyx_v_filter_list, PyObject *__pyx_v_file_mode) {
  PyObject *__pyx_v_opt_write = NULL;
  PyObject *__pyx_v_opt_read = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  size_t __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_supports_opt_blosc2_read_write", 0);

  /* "tables/hdf5extension.pyx":1453
 * 
 * def _supports_opt_blosc2_read_write(byteorder, filter_list, file_mode):
 *     if len(filter_list) == 1:  # Blosc2 must be the only filter             # <<<<<<<<<<<<<<
 *       opt_write = ((byteorder == sys.byteorder)
 *                    and ((filter_list[0] or "").startswith("blosc2")))
*/
  __pyx_t_1 = PyObject_Length(__pyx_v_filter_list); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1453, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_1 == 1);
  if (__pyx_t_2) {

    /* "tables/hdf5extension.pyx":1454
 * def _supports_opt_blosc2_read_write(byteorder, filter_list, file_mode):
 *     if len(filter_list) == 1:  # Blosc2 must be the only filter
 *       opt_write = ((byteorder == sys.byteorder)             # <<<<<<<<<<<<<<
 *                    and ((filter_list[0] or "").startswith("blosc2")))
 *     else:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_sys); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1454, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_byteorder); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1454, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyObject_RichCompare(__pyx_v_byteorder, __pyx_t_5, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1454, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 1454, __pyx_L1_error)
    if (__pyx_t_2) {
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_3 = __pyx_t_4;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      goto __pyx_L4_bool_binop_done;
    }

    /* "tables/hdf5extension.pyx":1455
 *     if len(filter_list) == 1:  # Blosc2 must be the only filter
 *       opt_write = ((byteorder == sys.byteorder)
 *                    and ((filter_list[0] or "").startswith("blosc2")))             # <<<<<<<<<<<<<<
 *     else:
 *       opt_write = False
*/
    __pyx_t_7 = __Pyx_GetItemInt(__pyx_v_filter_list, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1455, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 1455, __pyx_L1_error)
    if (!__pyx_t_2) {
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else {
      __Pyx_INCREF(__pyx_t_7);
      __pyx_t_6 = __pyx_t_7;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L6_bool_binop_done;
    }
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_);
    __pyx_t_6 = __pyx_mstate_global->__pyx_kp_u_;
    __pyx_L6_bool_binop_done:;
    __pyx_t_5 = __pyx_t_6;
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_8 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_n_u_blosc2};
      __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_startswith, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1455, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_3 = __pyx_t_4;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_L4_bool_binop_done:;
    __pyx_v_opt_write = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "tables/hdf5extension.pyx":1453
 * 
 * def _supports_opt_blosc2_read_write(byteorder, filter_list, file_mode):
 *     if len(filter_list) == 1:  # Blosc2 must be the only filter             # <<<<<<<<<<<<<<
 *       opt_write = ((byteorder == sys.byteorder)
 *                    and ((filter_list[0] or "").startswith("blosc2")))
*/
    goto __pyx_L3;
  }

  /* "tables/hdf5extension.pyx":1457
 *                    and ((filter_list[0] or "").startswith("blosc2")))
 *     else:
 *       opt_write = False             # <<<<<<<<<<<<<<
 *     # For reading, Windows does not support re-opening a file twice
 *     # in not read-only mode (for good reason), so we cannot use the
*/
  /*else*/ {
    __Pyx_INCREF(Py_False);
    __pyx_v_opt_write = Py_False;
  }
  __pyx_L3:;

  /* "tables/hdf5extension.pyx":1461
 *     # in not read-only mode (for good reason), so we cannot use the
 *     # blosc2 opt
 *     opt_read = (opt_write             # <<<<<<<<<<<<<<
 *                 and ((platform.system().lower() != 'windows') or
 *                      (file_mode == 'r')))
*/
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_opt_write); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 1461, __pyx_L1_error)
  if (__pyx_t_2) {
  } else {
    __Pyx_INCREF(__pyx_v_opt_write);
    __pyx_t_3 = __pyx_v_opt_write;
    goto __pyx_L8_bool_binop_done;
  }

  /* "tables/hdf5extension.pyx":1462
 *     # blosc2 opt
 *     opt_read = (opt_write
 *                 and ((platform.system().lower() != 'windows') or             # <<<<<<<<<<<<<<
 *                      (file_mode == 'r')))
 *     return (opt_read, opt_write)
*/
  __pyx_t_7 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_platform); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_system); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_8 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_10);
    assert(__pyx_t_7);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
    __Pyx_INCREF(__pyx_t_7);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
    __pyx_t_8 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_7, NULL};
    __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_8, (1-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1462, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
  }
  __pyx_t_6 = __pyx_t_5;
  __Pyx_INCREF(__pyx_t_6);
  __pyx_t_8 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_6, NULL};
    __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_lower, __pyx_callargs+__pyx_t_8, (1-__pyx_t_8) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1462, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
  }
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_windows, Py_NE); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1462, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 1462, __pyx_L1_error)
  if (!__pyx_t_2) {
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_3 = __pyx_t_5;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L8_bool_binop_done;
  }

  /* "tables/hdf5extension.pyx":1463
 *     opt_read = (opt_write
 *                 and ((platform.system().lower() != 'windows') or
 *                      (file_mode == 'r')))             # <<<<<<<<<<<<<<
 *     return (opt_read, opt_write)
 * 
*/
  __pyx_t_5 = PyObject_RichCompare(__pyx_v_file_mode, __pyx_mstate_global->__pyx_n_u_r, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1463, __pyx_L1_error)
  __Pyx_INCREF(__pyx_t_5);
  __pyx_t_3 = __pyx_t_5;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_L8_bool_binop_done:;
  __pyx_v_opt_read = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "tables/hdf5extension.pyx":1464
 *                 and ((platform.system().lower() != 'windows') or
 *                      (file_mode == 'r')))
 *     return (opt_read, opt_write)             # <<<<<<<<<<<<<<
 * 
 * cdef class Array(Leaf):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1464, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_opt_read);
  __Pyx_GIVEREF(__pyx_v_opt_read);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_opt_read) != (0)) __PYX_ERR(0, 1464, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_opt_write);
  __Pyx_GIVEREF(__pyx_v_opt_write);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_opt_write) != (0)) __PYX_ERR(0, 1464, __pyx_L1_error);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1452
 *     return NULL
 * 
 * def _supports_opt_blosc2_read_write(byteorder, filter_list, file_mode):             # <<<<<<<<<<<<<<
 *     if len(filter_list) == 1:  # Blosc2 must be the only filter
 *       opt_write = ((byteorder == sys.byteorder)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("tables.hdf5extension._supports_opt_blosc2_read_write", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_opt_write);
  __Pyx_XDECREF(__pyx_v_opt_read);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1469
 *   # Instance variables declared in .pxd
 * 
 *   def _create_array(self, ndarray nparr, object title, object atom):             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef herr_t ret
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_1_create_array(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Array_1_create_array = {"_create_array", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_1_create_array, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_1_create_array(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyArrayObject *__pyx_v_nparr = 0;
  PyObject *__pyx_v_title = 0;
  PyObject *__pyx_v_atom = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_create_array (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_nparr,&__pyx_mstate_global->__pyx_n_u_title,&__pyx_mstate_global->__pyx_n_u_atom,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1469, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1469, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1469, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1469, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_create_array", 0) < (0)) __PYX_ERR(0, 1469, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_create_array", 1, 3, 3, i); __PYX_ERR(0, 1469, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1469, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1469, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1469, __pyx_L3_error)
    }
    __pyx_v_nparr = ((PyArrayObject *)values[0]);
    __pyx_v_title = values[1];
    __pyx_v_atom = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_create_array", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 1469, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Array._create_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_nparr), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "nparr", 0))) __PYX_ERR(0, 1469, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Array__create_array(((struct __pyx_obj_6tables_13hdf5extension_Array *)__pyx_v_self), __pyx_v_nparr, __pyx_v_title, __pyx_v_atom);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Array__create_array(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyArrayObject *__pyx_v_nparr, PyObject *__pyx_v_title, PyObject *__pyx_v_atom) {
  void *__pyx_v_rbuf;
  PyObject *__pyx_v_complib = 0;
  PyObject *__pyx_v_version = 0;
  PyObject *__pyx_v_class_ = 0;
  PyObject *__pyx_v_dtype_ = 0;
  PyObject *__pyx_v_atom_ = 0;
  PyObject *__pyx_v_shape = 0;
  PyArrayObject *__pyx_v_dims = 0;
  PyObject *__pyx_v_encoded_title = 0;
  PyObject *__pyx_v_encoded_name = 0;
  enum H5T_cset_t __pyx_v_cset;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  hid_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11[4];
  PyObject *__pyx_t_12 = NULL;
  hsize_t *__pyx_t_13;
  void *__pyx_t_14;
  char *__pyx_t_15;
  char *__pyx_t_16;
  int __pyx_t_17;
  int __pyx_t_18;
  char *__pyx_t_19;
  int __pyx_t_20;
  int __pyx_t_21;
  hbool_t __pyx_t_22;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_create_array", 0);

  /* "tables/hdf5extension.pyx":1477
 *     cdef ndarray dims
 *     cdef bytes encoded_title, encoded_name
 *     cdef H5T_cset_t cset = H5T_CSET_ASCII             # <<<<<<<<<<<<<<
 * 
 *     encoded_title = title.encode('utf-8')
*/
  __pyx_v_cset = H5T_CSET_ASCII;

  /* "tables/hdf5extension.pyx":1479
 *     cdef H5T_cset_t cset = H5T_CSET_ASCII
 * 
 *     encoded_title = title.encode('utf-8')             # <<<<<<<<<<<<<<
 *     encoded_name = self.name.encode('utf-8')
 * 
*/
  __pyx_t_2 = __pyx_v_title;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1479, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1479, __pyx_L1_error)
  __pyx_v_encoded_title = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1480
 * 
 *     encoded_title = title.encode('utf-8')
 *     encoded_name = self.name.encode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     # Get the HDF5 type associated with this numpy type
*/
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.name;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1480, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1480, __pyx_L1_error)
  __pyx_v_encoded_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1483
 * 
 *     # Get the HDF5 type associated with this numpy type
 *     shape = (<object>nparr).shape             # <<<<<<<<<<<<<<
 *     if atom is None or atom.shape == ():
 *       dtype_ = nparr.dtype.base
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_nparr), __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_shape = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1484
 *     # Get the HDF5 type associated with this numpy type
 *     shape = (<object>nparr).shape
 *     if atom is None or atom.shape == ():             # <<<<<<<<<<<<<<
 *       dtype_ = nparr.dtype.base
 *       atom_ = Atom.from_dtype(dtype_)
*/
  __pyx_t_5 = (__pyx_v_atom == Py_None);
  if (!__pyx_t_5) {
  } else {
    __pyx_t_4 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1484, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_mstate_global->__pyx_empty_tuple, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1484, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 1484, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_4) {

    /* "tables/hdf5extension.pyx":1485
 *     shape = (<object>nparr).shape
 *     if atom is None or atom.shape == ():
 *       dtype_ = nparr.dtype.base             # <<<<<<<<<<<<<<
 *       atom_ = Atom.from_dtype(dtype_)
 *     else:
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_nparr), __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1485, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_base); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1485, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_dtype_ = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":1486
 *     if atom is None or atom.shape == ():
 *       dtype_ = nparr.dtype.base
 *       atom_ = Atom.from_dtype(dtype_)             # <<<<<<<<<<<<<<
 *     else:
 *       atom_ = atom
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_Atom); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1486, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_from_dtype); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1486, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_dtype_};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1486, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_v_atom_ = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":1484
 *     # Get the HDF5 type associated with this numpy type
 *     shape = (<object>nparr).shape
 *     if atom is None or atom.shape == ():             # <<<<<<<<<<<<<<
 *       dtype_ = nparr.dtype.base
 *       atom_ = Atom.from_dtype(dtype_)
*/
    goto __pyx_L3;
  }

  /* "tables/hdf5extension.pyx":1488
 *       atom_ = Atom.from_dtype(dtype_)
 *     else:
 *       atom_ = atom             # <<<<<<<<<<<<<<
 *       shape = shape[:-len(atom_.shape)]
 *     self.disk_type_id = atom_to_hdf5_type(atom_, self.byteorder)
*/
  /*else*/ {
    __Pyx_INCREF(__pyx_v_atom);
    __pyx_v_atom_ = __pyx_v_atom;

    /* "tables/hdf5extension.pyx":1489
 *     else:
 *       atom_ = atom
 *       shape = shape[:-len(atom_.shape)]             # <<<<<<<<<<<<<<
 *     self.disk_type_id = atom_to_hdf5_type(atom_, self.byteorder)
 *     if self.disk_type_id < 0:
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom_, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1489, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1489, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_v_shape, 0, (-__pyx_t_8), NULL, NULL, NULL, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1489, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_shape, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "tables/hdf5extension.pyx":1490
 *       atom_ = atom
 *       shape = shape[:-len(atom_.shape)]
 *     self.disk_type_id = atom_to_hdf5_type(atom_, self.byteorder)             # <<<<<<<<<<<<<<
 *     if self.disk_type_id < 0:
 *       raise HDF5ExtError(
*/
  __pyx_t_7 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_atom_to_hdf5_type); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1490, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_byteorder); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1490, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_7);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_7);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_7, __pyx_v_atom_, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1490, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_9 = __Pyx_PyLong_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_9 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1490, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->__pyx_base.disk_type_id = __pyx_t_9;

  /* "tables/hdf5extension.pyx":1491
 *       shape = shape[:-len(atom_.shape)]
 *     self.disk_type_id = atom_to_hdf5_type(atom_, self.byteorder)
 *     if self.disk_type_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError(
 *         "Problems creating the %s: invalid disk type ID for atom %s" % (
*/
  __pyx_t_4 = (__pyx_v_self->__pyx_base.disk_type_id < 0);
  if (unlikely(__pyx_t_4)) {

    /* "tables/hdf5extension.pyx":1492
 *     self.disk_type_id = atom_to_hdf5_type(atom_, self.byteorder)
 *     if self.disk_type_id < 0:
 *       raise HDF5ExtError(             # <<<<<<<<<<<<<<
 *         "Problems creating the %s: invalid disk type ID for atom %s" % (
 *             self.__class__.__name__, atom_))
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1492, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);

    /* "tables/hdf5extension.pyx":1494
 *       raise HDF5ExtError(
 *         "Problems creating the %s: invalid disk type ID for atom %s" % (
 *             self.__class__.__name__, atom_))             # <<<<<<<<<<<<<<
 * 
 *     # Allocate space for the dimension axis info and fill it
*/
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_class); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1494, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_name_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1494, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_t_10), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1494, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_atom_), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1494, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11[0] = __pyx_mstate_global->__pyx_kp_u_Problems_creating_the;
    __pyx_t_11[1] = __pyx_t_7;
    __pyx_t_11[2] = __pyx_mstate_global->__pyx_kp_u_invalid_disk_type_ID_for_atom;
    __pyx_t_11[3] = __pyx_t_10;

    /* "tables/hdf5extension.pyx":1493
 *     if self.disk_type_id < 0:
 *       raise HDF5ExtError(
 *         "Problems creating the %s: invalid disk type ID for atom %s" % (             # <<<<<<<<<<<<<<
 *             self.__class__.__name__, atom_))
 * 
*/
    __pyx_t_12 = __Pyx_PyUnicode_Join(__pyx_t_11, 4, 22 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7) + 32 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_10), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10));
    if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1493, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_12};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1492, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1492, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1491
 *       shape = shape[:-len(atom_.shape)]
 *     self.disk_type_id = atom_to_hdf5_type(atom_, self.byteorder)
 *     if self.disk_type_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError(
 *         "Problems creating the %s: invalid disk type ID for atom %s" % (
*/
  }

  /* "tables/hdf5extension.pyx":1497
 * 
 *     # Allocate space for the dimension axis info and fill it
 *     dims = np.array(shape, dtype=np.intp)             # <<<<<<<<<<<<<<
 *     self.rank = len(shape)
 *     self.dims = npy_malloc_dims(self.rank, <npy_intp *>PyArray_DATA(dims))
*/
  __pyx_t_6 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_array); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_intp); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_6);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_6);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_6, __pyx_v_shape};
    __pyx_t_12 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1497, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_10, __pyx_t_12, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 1497, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_12);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1497, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1497, __pyx_L1_error)
  __pyx_v_dims = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1498
 *     # Allocate space for the dimension axis info and fill it
 *     dims = np.array(shape, dtype=np.intp)
 *     self.rank = len(shape)             # <<<<<<<<<<<<<<
 *     self.dims = npy_malloc_dims(self.rank, <npy_intp *>PyArray_DATA(dims))
 *     rbuf = _array_data(nparr)
*/
  __pyx_t_8 = PyObject_Length(__pyx_v_shape); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1498, __pyx_L1_error)
  __pyx_v_self->rank = __pyx_t_8;

  /* "tables/hdf5extension.pyx":1499
 *     dims = np.array(shape, dtype=np.intp)
 *     self.rank = len(shape)
 *     self.dims = npy_malloc_dims(self.rank, <npy_intp *>PyArray_DATA(dims))             # <<<<<<<<<<<<<<
 *     rbuf = _array_data(nparr)
 * 
*/
  __pyx_t_13 = __pyx_f_6tables_13hdf5extension_npy_malloc_dims(__pyx_v_self->rank, ((npy_intp *)PyArray_DATA(__pyx_v_dims))); if (unlikely(__pyx_t_13 == ((void *)NULL) && PyErr_Occurred())) __PYX_ERR(0, 1499, __pyx_L1_error)
  __pyx_v_self->__pyx_base.dims = __pyx_t_13;

  /* "tables/hdf5extension.pyx":1500
 *     self.rank = len(shape)
 *     self.dims = npy_malloc_dims(self.rank, <npy_intp *>PyArray_DATA(dims))
 *     rbuf = _array_data(nparr)             # <<<<<<<<<<<<<<
 * 
 *     # Blosc2 optimized operations cannot be used (no chunking nor filters).
*/
  __pyx_t_14 = __pyx_f_6tables_13hdf5extension__array_data(__pyx_v_nparr); if (unlikely(__pyx_t_14 == ((void *)NULL) && PyErr_Occurred())) __PYX_ERR(0, 1500, __pyx_L1_error)
  __pyx_v_rbuf = __pyx_t_14;

  /* "tables/hdf5extension.pyx":1503
 * 
 *     # Blosc2 optimized operations cannot be used (no chunking nor filters).
 *     self.blosc2_support_read = False             # <<<<<<<<<<<<<<
 *     self.blosc2_support_wirte = False
 * 
*/
  __pyx_v_self->blosc2_support_read = 0;

  /* "tables/hdf5extension.pyx":1504
 *     # Blosc2 optimized operations cannot be used (no chunking nor filters).
 *     self.blosc2_support_read = False
 *     self.blosc2_support_wirte = False             # <<<<<<<<<<<<<<
 * 
 *     # Save the array
*/
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_blosc2_support_wirte, Py_False) < (0)) __PYX_ERR(0, 1504, __pyx_L1_error)

  /* "tables/hdf5extension.pyx":1507
 * 
 *     # Save the array
 *     complib = (self.filters.complib or '').encode('utf-8')             # <<<<<<<<<<<<<<
 *     version = self._v_version.encode('utf-8')
 *     class_ = self._c_classid.encode('utf-8')
*/
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_filters); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1507, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_complib); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1507, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 1507, __pyx_L1_error)
  if (!__pyx_t_4) {
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_6);
    __pyx_t_12 = __pyx_t_6;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L7_bool_binop_done;
  }
  __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_);
  __pyx_t_12 = __pyx_mstate_global->__pyx_kp_u_;
  __pyx_L7_bool_binop_done:;
  __pyx_t_2 = __pyx_t_12;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1507, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1507, __pyx_L1_error)
  __pyx_v_complib = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1508
 *     # Save the array
 *     complib = (self.filters.complib or '').encode('utf-8')
 *     version = self._v_version.encode('utf-8')             # <<<<<<<<<<<<<<
 *     class_ = self._c_classid.encode('utf-8')
 *     self.dataset_id = H5ARRAYmake(self.parent_id, encoded_name, version,
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_version); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1508, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_12 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_12);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1508, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1508, __pyx_L1_error)
  __pyx_v_version = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1509
 *     complib = (self.filters.complib or '').encode('utf-8')
 *     version = self._v_version.encode('utf-8')
 *     class_ = self._c_classid.encode('utf-8')             # <<<<<<<<<<<<<<
 *     self.dataset_id = H5ARRAYmake(self.parent_id, encoded_name, version,
 *                                   self.rank, self.dims,
*/
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_c_classid); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1509, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_2 = __pyx_t_12;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1509, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1509, __pyx_L1_error)
  __pyx_v_class_ = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1510
 *     version = self._v_version.encode('utf-8')
 *     class_ = self._c_classid.encode('utf-8')
 *     self.dataset_id = H5ARRAYmake(self.parent_id, encoded_name, version,             # <<<<<<<<<<<<<<
 *                                   self.rank, self.dims,
 *                                   self.extdim, self.disk_type_id, NULL, NULL,
*/
  if (unlikely(__pyx_v_encoded_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1510, __pyx_L1_error)
  }
  __pyx_t_15 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_name); if (unlikely((!__pyx_t_15) && PyErr_Occurred())) __PYX_ERR(0, 1510, __pyx_L1_error)
  if (unlikely(__pyx_v_version == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1510, __pyx_L1_error)
  }
  __pyx_t_16 = __Pyx_PyBytes_AsWritableString(__pyx_v_version); if (unlikely((!__pyx_t_16) && PyErr_Occurred())) __PYX_ERR(0, 1510, __pyx_L1_error)

  /* "tables/hdf5extension.pyx":1512
 *     self.dataset_id = H5ARRAYmake(self.parent_id, encoded_name, version,
 *                                   self.rank, self.dims,
 *                                   self.extdim, self.disk_type_id, NULL, NULL,             # <<<<<<<<<<<<<<
 *                                   self.filters.complevel, complib,
 *                                   self.filters.shuffle_bitshuffle,
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_extdim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_17 = __Pyx_PyLong_As_int(__pyx_t_1); if (unlikely((__pyx_t_17 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1512, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1513
 *                                   self.rank, self.dims,
 *                                   self.extdim, self.disk_type_id, NULL, NULL,
 *                                   self.filters.complevel, complib,             # <<<<<<<<<<<<<<
 *                                   self.filters.shuffle_bitshuffle,
 *                                   self.filters.fletcher32,
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_filters); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1513, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_complevel); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1513, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_18 = __Pyx_PyLong_As_int(__pyx_t_12); if (unlikely((__pyx_t_18 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1513, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (unlikely(__pyx_v_complib == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1513, __pyx_L1_error)
  }
  __pyx_t_19 = __Pyx_PyBytes_AsWritableString(__pyx_v_complib); if (unlikely((!__pyx_t_19) && PyErr_Occurred())) __PYX_ERR(0, 1513, __pyx_L1_error)

  /* "tables/hdf5extension.pyx":1514
 *                                   self.extdim, self.disk_type_id, NULL, NULL,
 *                                   self.filters.complevel, complib,
 *                                   self.filters.shuffle_bitshuffle,             # <<<<<<<<<<<<<<
 *                                   self.filters.fletcher32,
 *                                   self._want_track_times,
*/
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_filters); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1514, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_shuffle_bitshuffle); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1514, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_20 = __Pyx_PyLong_As_int(__pyx_t_1); if (unlikely((__pyx_t_20 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1514, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1515
 *                                   self.filters.complevel, complib,
 *                                   self.filters.shuffle_bitshuffle,
 *                                   self.filters.fletcher32,             # <<<<<<<<<<<<<<
 *                                   self._want_track_times,
 *                                   rbuf)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_filters); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1515, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_fletcher32); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1515, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_21 = __Pyx_PyLong_As_int(__pyx_t_12); if (unlikely((__pyx_t_21 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1515, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "tables/hdf5extension.pyx":1516
 *                                   self.filters.shuffle_bitshuffle,
 *                                   self.filters.fletcher32,
 *                                   self._want_track_times,             # <<<<<<<<<<<<<<
 *                                   rbuf)
 *     if self.dataset_id < 0:
*/
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_want_track_times); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1516, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_22 = __Pyx_PyLong_As_hbool_t(__pyx_t_12); if (unlikely((__pyx_t_22 == ((hbool_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1516, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "tables/hdf5extension.pyx":1510
 *     version = self._v_version.encode('utf-8')
 *     class_ = self._c_classid.encode('utf-8')
 *     self.dataset_id = H5ARRAYmake(self.parent_id, encoded_name, version,             # <<<<<<<<<<<<<<
 *                                   self.rank, self.dims,
 *                                   self.extdim, self.disk_type_id, NULL, NULL,
*/
  __pyx_v_self->__pyx_base.dataset_id = H5ARRAYmake(__pyx_v_self->__pyx_base.__pyx_base.parent_id, __pyx_t_15, __pyx_t_16, __pyx_v_self->rank, __pyx_v_self->__pyx_base.dims, __pyx_t_17, __pyx_v_self->__pyx_base.disk_type_id, NULL, NULL, __pyx_t_18, __pyx_t_19, __pyx_t_20, __pyx_t_21, __pyx_t_22, __pyx_v_rbuf);

  /* "tables/hdf5extension.pyx":1518
 *                                   self._want_track_times,
 *                                   rbuf)
 *     if self.dataset_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems creating the %s." % self.__class__.__name__)
 * 
*/
  __pyx_t_4 = (__pyx_v_self->__pyx_base.dataset_id < 0);
  if (unlikely(__pyx_t_4)) {

    /* "tables/hdf5extension.pyx":1519
 *                                   rbuf)
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Problems creating the %s." % self.__class__.__name__)             # <<<<<<<<<<<<<<
 * 
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:
*/
    __pyx_t_1 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1519, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_class); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1519, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_name_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1519, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Problems_creating_the_s, __pyx_t_10); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1519, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      assert(__pyx_t_1);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_t_6};
      __pyx_t_12 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1519, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
    }
    __Pyx_Raise(__pyx_t_12, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __PYX_ERR(0, 1519, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1518
 *                                   self._want_track_times,
 *                                   rbuf)
 *     if self.dataset_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems creating the %s." % self.__class__.__name__)
 * 
*/
  }

  /* "tables/hdf5extension.pyx":1521
 *       raise HDF5ExtError("Problems creating the %s." % self.__class__.__name__)
 * 
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:             # <<<<<<<<<<<<<<
 *       cset = H5T_CSET_UTF8
 *       # Set the conforming array attributes
*/
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_file); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_params); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_PYTABLES_SYS_ATTRS); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_12); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 1521, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (__pyx_t_4) {

    /* "tables/hdf5extension.pyx":1522
 * 
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:
 *       cset = H5T_CSET_UTF8             # <<<<<<<<<<<<<<
 *       # Set the conforming array attributes
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,
*/
    __pyx_v_cset = H5T_CSET_UTF8;

    /* "tables/hdf5extension.pyx":1524
 *       cset = H5T_CSET_UTF8
 *       # Set the conforming array attributes
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,             # <<<<<<<<<<<<<<
 *                                  len(class_), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,
*/
    if (unlikely(__pyx_v_class_ == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 1524, __pyx_L1_error)
    }
    __pyx_t_19 = __Pyx_PyBytes_AsWritableString(__pyx_v_class_); if (unlikely((!__pyx_t_19) && PyErr_Occurred())) __PYX_ERR(0, 1524, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1525
 *       # Set the conforming array attributes
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,
 *                                  len(class_), cset)             # <<<<<<<<<<<<<<
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,
 *                                  len(version), cset)
*/
    if (unlikely(__pyx_v_class_ == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 1525, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_PyBytes_GET_SIZE(__pyx_v_class_); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1525, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1524
 *       cset = H5T_CSET_UTF8
 *       # Set the conforming array attributes
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,             # <<<<<<<<<<<<<<
 *                                  len(class_), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,
*/
    (void)(H5ATTRset_attribute_string(__pyx_v_self->__pyx_base.dataset_id, ((char *)"CLASS"), __pyx_t_19, __pyx_t_8, __pyx_v_cset));

    /* "tables/hdf5extension.pyx":1526
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,
 *                                  len(class_), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,             # <<<<<<<<<<<<<<
 *                                  len(version), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,
*/
    if (unlikely(__pyx_v_version == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 1526, __pyx_L1_error)
    }
    __pyx_t_19 = __Pyx_PyBytes_AsWritableString(__pyx_v_version); if (unlikely((!__pyx_t_19) && PyErr_Occurred())) __PYX_ERR(0, 1526, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1527
 *                                  len(class_), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,
 *                                  len(version), cset)             # <<<<<<<<<<<<<<
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,
 *                                  len(encoded_title), cset)
*/
    if (unlikely(__pyx_v_version == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 1527, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_PyBytes_GET_SIZE(__pyx_v_version); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1527, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1526
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,
 *                                  len(class_), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,             # <<<<<<<<<<<<<<
 *                                  len(version), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,
*/
    (void)(H5ATTRset_attribute_string(__pyx_v_self->__pyx_base.dataset_id, ((char *)"VERSION"), __pyx_t_19, __pyx_t_8, __pyx_v_cset));

    /* "tables/hdf5extension.pyx":1528
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,
 *                                  len(version), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,             # <<<<<<<<<<<<<<
 *                                  len(encoded_title), cset)
 * 
*/
    if (unlikely(__pyx_v_encoded_title == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 1528, __pyx_L1_error)
    }
    __pyx_t_19 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_title); if (unlikely((!__pyx_t_19) && PyErr_Occurred())) __PYX_ERR(0, 1528, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1529
 *                                  len(version), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,
 *                                  len(encoded_title), cset)             # <<<<<<<<<<<<<<
 * 
 *     # Get the native type (so that it is HDF5 who is the responsible to deal
*/
    if (unlikely(__pyx_v_encoded_title == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 1529, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_PyBytes_GET_SIZE(__pyx_v_encoded_title); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1529, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1528
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,
 *                                  len(version), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,             # <<<<<<<<<<<<<<
 *                                  len(encoded_title), cset)
 * 
*/
    (void)(H5ATTRset_attribute_string(__pyx_v_self->__pyx_base.dataset_id, ((char *)"TITLE"), __pyx_t_19, __pyx_t_8, __pyx_v_cset));

    /* "tables/hdf5extension.pyx":1521
 *       raise HDF5ExtError("Problems creating the %s." % self.__class__.__name__)
 * 
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:             # <<<<<<<<<<<<<<
 *       cset = H5T_CSET_UTF8
 *       # Set the conforming array attributes
*/
  }

  /* "tables/hdf5extension.pyx":1533
 *     # Get the native type (so that it is HDF5 who is the responsible to deal
 *     # with non-native byteorders on-disk)
 *     self.type_id = get_native_type(self.disk_type_id)             # <<<<<<<<<<<<<<
 * 
 *     return self.dataset_id, shape, atom_
*/
  __pyx_t_9 = __pyx_f_6tables_14utilsextension_get_native_type(__pyx_v_self->__pyx_base.disk_type_id); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1533, __pyx_L1_error)
  __pyx_v_self->__pyx_base.type_id = __pyx_t_9;

  /* "tables/hdf5extension.pyx":1535
 *     self.type_id = get_native_type(self.disk_type_id)
 * 
 *     return self.dataset_id, shape, atom_             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_12 = __Pyx_PyLong_From_hid_t(__pyx_v_self->__pyx_base.dataset_id); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_12);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_12) != (0)) __PYX_ERR(0, 1535, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_shape);
  __Pyx_GIVEREF(__pyx_v_shape);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_shape) != (0)) __PYX_ERR(0, 1535, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_atom_);
  __Pyx_GIVEREF(__pyx_v_atom_);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_atom_) != (0)) __PYX_ERR(0, 1535, __pyx_L1_error);
  __pyx_t_12 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1469
 *   # Instance variables declared in .pxd
 * 
 *   def _create_array(self, ndarray nparr, object title, object atom):             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef herr_t ret
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("tables.hdf5extension.Array._create_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_complib);
  __Pyx_XDECREF(__pyx_v_version);
  __Pyx_XDECREF(__pyx_v_class_);
  __Pyx_XDECREF(__pyx_v_dtype_);
  __Pyx_XDECREF(__pyx_v_atom_);
  __Pyx_XDECREF(__pyx_v_shape);
  __Pyx_XDECREF((PyObject *)__pyx_v_dims);
  __Pyx_XDECREF(__pyx_v_encoded_title);
  __Pyx_XDECREF(__pyx_v_encoded_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1538
 * 
 * 
 *   def _create_carray(self, object title):             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef herr_t ret
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_3_create_carray(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Array_3_create_carray = {"_create_carray", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_3_create_carray, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_3_create_carray(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_title = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_create_carray (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_title,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1538, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1538, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_create_carray", 0) < (0)) __PYX_ERR(0, 1538, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_create_carray", 1, 1, 1, i); __PYX_ERR(0, 1538, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1538, __pyx_L3_error)
    }
    __pyx_v_title = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_create_carray", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1538, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Array._create_carray", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Array_2_create_carray(((struct __pyx_obj_6tables_13hdf5extension_Array *)__pyx_v_self), __pyx_v_title);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_2_create_carray(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyObject *__pyx_v_title) {
  void *__pyx_v_rbuf;
  PyObject *__pyx_v_complib = 0;
  PyObject *__pyx_v_version = 0;
  PyObject *__pyx_v_class_ = 0;
  PyArrayObject *__pyx_v_dflts = 0;
  void *__pyx_v_fill_data;
  PyArrayObject *__pyx_v_extdim = 0;
  PyObject *__pyx_v_atom = 0;
  PyObject *__pyx_v_encoded_title = 0;
  PyObject *__pyx_v_encoded_name = 0;
  hsize_t __pyx_v_blocksize;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  hid_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  hsize_t *__pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *(*__pyx_t_13)(PyObject *);
  hbool_t __pyx_t_14;
  hbool_t __pyx_t_15;
  int __pyx_t_16;
  hsize_t __pyx_t_17;
  char const *__pyx_t_18;
  char const *__pyx_t_19;
  int __pyx_t_20;
  int __pyx_t_21;
  char *__pyx_t_22;
  int __pyx_t_23;
  int __pyx_t_24;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_create_carray", 0);

  /* "tables/hdf5extension.pyx":1549
 *     cdef bytes encoded_title, encoded_name
 * 
 *     encoded_title = title.encode('utf-8')             # <<<<<<<<<<<<<<
 *     encoded_name = self.name.encode('utf-8')
 * 
*/
  __pyx_t_2 = __pyx_v_title;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1549, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1549, __pyx_L1_error)
  __pyx_v_encoded_title = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1550
 * 
 *     encoded_title = title.encode('utf-8')
 *     encoded_name = self.name.encode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     atom = self.atom
*/
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.name;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1550, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1550, __pyx_L1_error)
  __pyx_v_encoded_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1552
 *     encoded_name = self.name.encode('utf-8')
 * 
 *     atom = self.atom             # <<<<<<<<<<<<<<
 *     self.disk_type_id = atom_to_hdf5_type(atom, self.byteorder)
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_atom = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1553
 * 
 *     atom = self.atom
 *     self.disk_type_id = atom_to_hdf5_type(atom, self.byteorder)             # <<<<<<<<<<<<<<
 * 
 *     self.rank = len(self.shape)
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_atom_to_hdf5_type); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_byteorder); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_v_atom, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1553, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_6 = __Pyx_PyLong_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_6 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1553, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->__pyx_base.disk_type_id = __pyx_t_6;

  /* "tables/hdf5extension.pyx":1555
 *     self.disk_type_id = atom_to_hdf5_type(atom, self.byteorder)
 * 
 *     self.rank = len(self.shape)             # <<<<<<<<<<<<<<
 *     self.dims = malloc_dims(self.shape)
 *     if self.chunkshape:
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1555, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->rank = __pyx_t_7;

  /* "tables/hdf5extension.pyx":1556
 * 
 *     self.rank = len(self.shape)
 *     self.dims = malloc_dims(self.shape)             # <<<<<<<<<<<<<<
 *     if self.chunkshape:
 *       self.dims_chunk = malloc_dims(self.chunkshape)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1556, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = __pyx_f_6tables_14utilsextension_malloc_dims(__pyx_t_1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1556, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->__pyx_base.dims = __pyx_t_8;

  /* "tables/hdf5extension.pyx":1557
 *     self.rank = len(self.shape)
 *     self.dims = malloc_dims(self.shape)
 *     if self.chunkshape:             # <<<<<<<<<<<<<<
 *       self.dims_chunk = malloc_dims(self.chunkshape)
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_chunkshape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1557, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 1557, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_9) {

    /* "tables/hdf5extension.pyx":1558
 *     self.dims = malloc_dims(self.shape)
 *     if self.chunkshape:
 *       self.dims_chunk = malloc_dims(self.chunkshape)             # <<<<<<<<<<<<<<
 * 
 *     # Decide whether Blosc2 optimized operations can be used.
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_chunkshape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1558, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = __pyx_f_6tables_14utilsextension_malloc_dims(__pyx_t_1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1558, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_self->dims_chunk = __pyx_t_8;

    /* "tables/hdf5extension.pyx":1557
 *     self.rank = len(self.shape)
 *     self.dims = malloc_dims(self.shape)
 *     if self.chunkshape:             # <<<<<<<<<<<<<<
 *       self.dims_chunk = malloc_dims(self.chunkshape)
 * 
*/
  }

  /* "tables/hdf5extension.pyx":1562
 *     # Decide whether Blosc2 optimized operations can be used.
 *     (self.blosc2_support_read, self.blosc2_support_write) = (
 *         _supports_opt_blosc2_read_write(self.byteorder, [self.filters.complib],             # <<<<<<<<<<<<<<
 *                                         self._v_file.mode))
 * 
*/
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_supports_opt_blosc2_read_write); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1562, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_byteorder); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1562, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_filters); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1562, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_complib); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1562, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyList_New(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1562, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 0, __pyx_t_11) != (0)) __PYX_ERR(0, 1562, __pyx_L1_error);
  __pyx_t_11 = 0;

  /* "tables/hdf5extension.pyx":1563
 *     (self.blosc2_support_read, self.blosc2_support_write) = (
 *         _supports_opt_blosc2_read_write(self.byteorder, [self.filters.complib],
 *                                         self._v_file.mode))             # <<<<<<<<<<<<<<
 * 
 *     rbuf = NULL   # The data pointer. We don't have data to save initially
*/
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_file); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1563, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_mode); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1563, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
    assert(__pyx_t_4);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[4] = {__pyx_t_4, __pyx_t_2, __pyx_t_10, __pyx_t_12};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_3, (4-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1562, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 1561, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_5);
      __pyx_t_12 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_12);
    } else {
      __pyx_t_5 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1561, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __pyx_t_12 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1561, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_12);
    }
    #else
    __pyx_t_5 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1561, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_12 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1561, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_10 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1561, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_13 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_10);
    index = 0; __pyx_t_5 = __pyx_t_13(__pyx_t_10); if (unlikely(!__pyx_t_5)) goto __pyx_L4_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_5);
    index = 1; __pyx_t_12 = __pyx_t_13(__pyx_t_10); if (unlikely(!__pyx_t_12)) goto __pyx_L4_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_12);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_13(__pyx_t_10), 2) < (0)) __PYX_ERR(0, 1561, __pyx_L1_error)
    __pyx_t_13 = NULL;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    goto __pyx_L5_unpacking_done;
    __pyx_L4_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_13 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 1561, __pyx_L1_error)
    __pyx_L5_unpacking_done:;
  }

  /* "tables/hdf5extension.pyx":1561
 * 
 *     # Decide whether Blosc2 optimized operations can be used.
 *     (self.blosc2_support_read, self.blosc2_support_write) = (             # <<<<<<<<<<<<<<
 *         _supports_opt_blosc2_read_write(self.byteorder, [self.filters.complib],
 *                                         self._v_file.mode))
*/
  __pyx_t_14 = __Pyx_PyLong_As_hbool_t(__pyx_t_5); if (unlikely((__pyx_t_14 == ((hbool_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1561, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_15 = __Pyx_PyLong_As_hbool_t(__pyx_t_12); if (unlikely((__pyx_t_15 == ((hbool_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1561, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_self->blosc2_support_read = __pyx_t_14;
  __pyx_v_self->blosc2_support_write = __pyx_t_15;

  /* "tables/hdf5extension.pyx":1565
 *                                         self._v_file.mode))
 * 
 *     rbuf = NULL   # The data pointer. We don't have data to save initially             # <<<<<<<<<<<<<<
 *     # Encode strings
 *     complib = (self.filters.complib or '').encode('utf-8')
*/
  __pyx_v_rbuf = NULL;

  /* "tables/hdf5extension.pyx":1567
 *     rbuf = NULL   # The data pointer. We don't have data to save initially
 *     # Encode strings
 *     complib = (self.filters.complib or '').encode('utf-8')             # <<<<<<<<<<<<<<
 *     version = self._v_version.encode('utf-8')
 *     class_ = self._c_classid.encode('utf-8')
*/
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_filters); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1567, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_complib); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1567, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 1567, __pyx_L1_error)
  if (!__pyx_t_9) {
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_5 = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L6_bool_binop_done;
  }
  __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_);
  __pyx_t_5 = __pyx_mstate_global->__pyx_kp_u_;
  __pyx_L6_bool_binop_done:;
  __pyx_t_12 = __pyx_t_5;
  __Pyx_INCREF(__pyx_t_12);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1567, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1567, __pyx_L1_error)
  __pyx_v_complib = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1568
 *     # Encode strings
 *     complib = (self.filters.complib or '').encode('utf-8')
 *     version = self._v_version.encode('utf-8')             # <<<<<<<<<<<<<<
 *     class_ = self._c_classid.encode('utf-8')
 * 
*/
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_version); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_5 = __pyx_t_12;
  __Pyx_INCREF(__pyx_t_5);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1568, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1568, __pyx_L1_error)
  __pyx_v_version = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1569
 *     complib = (self.filters.complib or '').encode('utf-8')
 *     version = self._v_version.encode('utf-8')
 *     class_ = self._c_classid.encode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     # Get the fill values
*/
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_c_classid); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1569, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_12 = __pyx_t_5;
  __Pyx_INCREF(__pyx_t_12);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1569, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1569, __pyx_L1_error)
  __pyx_v_class_ = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1572
 * 
 *     # Get the fill values
 *     if isinstance(atom.dflt, np.ndarray) or atom.dflt:             # <<<<<<<<<<<<<<
 *       dflts = np.array(atom.dflt, dtype=atom.dtype)
 *       fill_data = PyArray_DATA(dflts)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_dflt); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1572, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1572, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_ndarray); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1572, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_16 = PyObject_IsInstance(__pyx_t_1, __pyx_t_12); if (unlikely(__pyx_t_16 == ((int)-1))) __PYX_ERR(0, 1572, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (!__pyx_t_16) {
  } else {
    __pyx_t_9 = __pyx_t_16;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_dflt); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1572, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_16 = __Pyx_PyObject_IsTrue(__pyx_t_12); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 1572, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_9 = __pyx_t_16;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_9) {

    /* "tables/hdf5extension.pyx":1573
 *     # Get the fill values
 *     if isinstance(atom.dflt, np.ndarray) or atom.dflt:
 *       dflts = np.array(atom.dflt, dtype=atom.dtype)             # <<<<<<<<<<<<<<
 *       fill_data = PyArray_DATA(dflts)
 *     else:
*/
    __pyx_t_1 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1573, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_array); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1573, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_dflt); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1573, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1573, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      assert(__pyx_t_1);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, __pyx_t_5};
      __pyx_t_4 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1573, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_10, __pyx_t_4, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 1573, __pyx_L1_error)
      __pyx_t_12 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1573, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
    }
    if (!(likely(((__pyx_t_12) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_12, __pyx_mstate_global->__pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1573, __pyx_L1_error)
    __pyx_v_dflts = ((PyArrayObject *)__pyx_t_12);
    __pyx_t_12 = 0;

    /* "tables/hdf5extension.pyx":1574
 *     if isinstance(atom.dflt, np.ndarray) or atom.dflt:
 *       dflts = np.array(atom.dflt, dtype=atom.dtype)
 *       fill_data = PyArray_DATA(dflts)             # <<<<<<<<<<<<<<
 *     else:
 *       dflts = np.zeros((), dtype=atom.dtype)
*/
    __pyx_v_fill_data = PyArray_DATA(__pyx_v_dflts);

    /* "tables/hdf5extension.pyx":1572
 * 
 *     # Get the fill values
 *     if isinstance(atom.dflt, np.ndarray) or atom.dflt:             # <<<<<<<<<<<<<<
 *       dflts = np.array(atom.dflt, dtype=atom.dtype)
 *       fill_data = PyArray_DATA(dflts)
*/
    goto __pyx_L8;
  }

  /* "tables/hdf5extension.pyx":1576
 *       fill_data = PyArray_DATA(dflts)
 *     else:
 *       dflts = np.zeros((), dtype=atom.dtype)             # <<<<<<<<<<<<<<
 *       fill_data = NULL
 *     if atom.shape == ():
*/
  /*else*/ {
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1576, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_zeros); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1576, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1576, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_10);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, __pyx_mstate_global->__pyx_empty_tuple};
      __pyx_t_5 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1576, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_4, __pyx_t_5, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 1576, __pyx_L1_error)
      __pyx_t_12 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1576, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
    }
    if (!(likely(((__pyx_t_12) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_12, __pyx_mstate_global->__pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1576, __pyx_L1_error)
    __pyx_v_dflts = ((PyArrayObject *)__pyx_t_12);
    __pyx_t_12 = 0;

    /* "tables/hdf5extension.pyx":1577
 *     else:
 *       dflts = np.zeros((), dtype=atom.dtype)
 *       fill_data = NULL             # <<<<<<<<<<<<<<
 *     if atom.shape == ():
 *       # The default is preferred as a scalar value instead of 0-dim array
*/
    __pyx_v_fill_data = NULL;
  }
  __pyx_L8:;

  /* "tables/hdf5extension.pyx":1578
 *       dflts = np.zeros((), dtype=atom.dtype)
 *       fill_data = NULL
 *     if atom.shape == ():             # <<<<<<<<<<<<<<
 *       # The default is preferred as a scalar value instead of 0-dim array
 *       atom.dflt = dflts[()]
*/
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1578, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_10 = PyObject_RichCompare(__pyx_t_12, __pyx_mstate_global->__pyx_empty_tuple, Py_EQ); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1578, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 1578, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (__pyx_t_9) {

    /* "tables/hdf5extension.pyx":1580
 *     if atom.shape == ():
 *       # The default is preferred as a scalar value instead of 0-dim array
 *       atom.dflt = dflts[()]             # <<<<<<<<<<<<<<
 *     else:
 *       atom.dflt = dflts
*/
    __pyx_t_10 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_dflts), __pyx_mstate_global->__pyx_empty_tuple); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1580, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_dflt, __pyx_t_10) < (0)) __PYX_ERR(0, 1580, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

    /* "tables/hdf5extension.pyx":1578
 *       dflts = np.zeros((), dtype=atom.dtype)
 *       fill_data = NULL
 *     if atom.shape == ():             # <<<<<<<<<<<<<<
 *       # The default is preferred as a scalar value instead of 0-dim array
 *       atom.dflt = dflts[()]
*/
    goto __pyx_L11;
  }

  /* "tables/hdf5extension.pyx":1582
 *       atom.dflt = dflts[()]
 *     else:
 *       atom.dflt = dflts             # <<<<<<<<<<<<<<
 * 
 *     cdef hsize_t blocksize = int(os.environ.get("PT_DEFAULT_B2_BLOCKSIZE", "0"))
*/
  /*else*/ {
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_dflt, ((PyObject *)__pyx_v_dflts)) < (0)) __PYX_ERR(0, 1582, __pyx_L1_error)
  }
  __pyx_L11:;

  /* "tables/hdf5extension.pyx":1584
 *       atom.dflt = dflts
 * 
 *     cdef hsize_t blocksize = int(os.environ.get("PT_DEFAULT_B2_BLOCKSIZE", "0"))             # <<<<<<<<<<<<<<
 *     # Create the CArray/EArray
 *     self.dataset_id = H5ARRAYOmake(self.parent_id, encoded_name, version,
*/
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_os); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_environ); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_get); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_mstate_global->__pyx_tuple[1], NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyNumber_Int(__pyx_t_12); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_17 = __Pyx_PyLong_As_hsize_t(__pyx_t_10); if (unlikely((__pyx_t_17 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1584, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_v_blocksize = __pyx_t_17;

  /* "tables/hdf5extension.pyx":1586
 *     cdef hsize_t blocksize = int(os.environ.get("PT_DEFAULT_B2_BLOCKSIZE", "0"))
 *     # Create the CArray/EArray
 *     self.dataset_id = H5ARRAYOmake(self.parent_id, encoded_name, version,             # <<<<<<<<<<<<<<
 *                                   self.rank, self.dims, self.extdim,
 *                                   self.disk_type_id, self.dims_chunk,
*/
  if (unlikely(__pyx_v_encoded_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1586, __pyx_L1_error)
  }
  __pyx_t_18 = __Pyx_PyBytes_AsString(__pyx_v_encoded_name); if (unlikely((!__pyx_t_18) && PyErr_Occurred())) __PYX_ERR(0, 1586, __pyx_L1_error)
  if (unlikely(__pyx_v_version == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1586, __pyx_L1_error)
  }
  __pyx_t_19 = __Pyx_PyBytes_AsString(__pyx_v_version); if (unlikely((!__pyx_t_19) && PyErr_Occurred())) __PYX_ERR(0, 1586, __pyx_L1_error)

  /* "tables/hdf5extension.pyx":1587
 *     # Create the CArray/EArray
 *     self.dataset_id = H5ARRAYOmake(self.parent_id, encoded_name, version,
 *                                   self.rank, self.dims, self.extdim,             # <<<<<<<<<<<<<<
 *                                   self.disk_type_id, self.dims_chunk,
 *                                   blocksize, fill_data,
*/
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_extdim); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_20 = __Pyx_PyLong_As_int(__pyx_t_10); if (unlikely((__pyx_t_20 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1587, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "tables/hdf5extension.pyx":1590
 *                                   self.disk_type_id, self.dims_chunk,
 *                                   blocksize, fill_data,
 *                                   self.filters.complevel, complib,             # <<<<<<<<<<<<<<
 *                                   self.filters.shuffle_bitshuffle,
 *                                   self.filters.fletcher32,
*/
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_filters); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1590, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_complevel); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1590, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_21 = __Pyx_PyLong_As_int(__pyx_t_12); if (unlikely((__pyx_t_21 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1590, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (unlikely(__pyx_v_complib == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1590, __pyx_L1_error)
  }
  __pyx_t_22 = __Pyx_PyBytes_AsWritableString(__pyx_v_complib); if (unlikely((!__pyx_t_22) && PyErr_Occurred())) __PYX_ERR(0, 1590, __pyx_L1_error)

  /* "tables/hdf5extension.pyx":1591
 *                                   blocksize, fill_data,
 *                                   self.filters.complevel, complib,
 *                                   self.filters.shuffle_bitshuffle,             # <<<<<<<<<<<<<<
 *                                   self.filters.fletcher32,
 *                                   self._want_track_times,
*/
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_filters); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1591, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_shuffle_bitshuffle); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1591, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_23 = __Pyx_PyLong_As_int(__pyx_t_10); if (unlikely((__pyx_t_23 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1591, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "tables/hdf5extension.pyx":1592
 *                                   self.filters.complevel, complib,
 *                                   self.filters.shuffle_bitshuffle,
 *                                   self.filters.fletcher32,             # <<<<<<<<<<<<<<
 *                                   self._want_track_times,
 *                                   rbuf)
*/
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_filters); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1592, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_fletcher32); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1592, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_24 = __Pyx_PyLong_As_int(__pyx_t_12); if (unlikely((__pyx_t_24 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1592, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "tables/hdf5extension.pyx":1593
 *                                   self.filters.shuffle_bitshuffle,
 *                                   self.filters.fletcher32,
 *                                   self._want_track_times,             # <<<<<<<<<<<<<<
 *                                   rbuf)
 *     if self.dataset_id < 0:
*/
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_want_track_times); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1593, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_15 = __Pyx_PyLong_As_hbool_t(__pyx_t_12); if (unlikely((__pyx_t_15 == ((hbool_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1593, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "tables/hdf5extension.pyx":1586
 *     cdef hsize_t blocksize = int(os.environ.get("PT_DEFAULT_B2_BLOCKSIZE", "0"))
 *     # Create the CArray/EArray
 *     self.dataset_id = H5ARRAYOmake(self.parent_id, encoded_name, version,             # <<<<<<<<<<<<<<
 *                                   self.rank, self.dims, self.extdim,
 *                                   self.disk_type_id, self.dims_chunk,
*/
  __pyx_v_self->__pyx_base.dataset_id = H5ARRAYOmake(__pyx_v_self->__pyx_base.__pyx_base.parent_id, __pyx_t_18, __pyx_t_19, __pyx_v_self->rank, __pyx_v_self->__pyx_base.dims, __pyx_t_20, __pyx_v_self->__pyx_base.disk_type_id, __pyx_v_self->dims_chunk, __pyx_v_blocksize, __pyx_v_fill_data, __pyx_t_21, __pyx_t_22, __pyx_t_23, __pyx_t_24, __pyx_t_15, __pyx_v_rbuf);

  /* "tables/hdf5extension.pyx":1595
 *                                   self._want_track_times,
 *                                   rbuf)
 *     if self.dataset_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems creating the %s." % self.__class__.__name__)
 * 
*/
  __pyx_t_9 = (__pyx_v_self->__pyx_base.dataset_id < 0);
  if (unlikely(__pyx_t_9)) {

    /* "tables/hdf5extension.pyx":1596
 *                                   rbuf)
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Problems creating the %s." % self.__class__.__name__)             # <<<<<<<<<<<<<<
 * 
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:
*/
    __pyx_t_10 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1596, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_class); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1596, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_name_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1596, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Problems_creating_the_s, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1596, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_10);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_t_4};
      __pyx_t_12 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1596, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
    }
    __Pyx_Raise(__pyx_t_12, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __PYX_ERR(0, 1596, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1595
 *                                   self._want_track_times,
 *                                   rbuf)
 *     if self.dataset_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems creating the %s." % self.__class__.__name__)
 * 
*/
  }

  /* "tables/hdf5extension.pyx":1598
 *       raise HDF5ExtError("Problems creating the %s." % self.__class__.__name__)
 * 
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:             # <<<<<<<<<<<<<<
 *       # Set the conforming array attributes
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,
*/
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_file); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1598, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_params); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1598, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Dict_GetItem(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_PYTABLES_SYS_ATTRS); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1598, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_12); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 1598, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (__pyx_t_9) {

    /* "tables/hdf5extension.pyx":1600
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:
 *       # Set the conforming array attributes
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,             # <<<<<<<<<<<<<<
 *                                  len(class_), H5T_CSET_ASCII)
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,
*/
    if (unlikely(__pyx_v_class_ == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 1600, __pyx_L1_error)
    }
    __pyx_t_22 = __Pyx_PyBytes_AsWritableString(__pyx_v_class_); if (unlikely((!__pyx_t_22) && PyErr_Occurred())) __PYX_ERR(0, 1600, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1601
 *       # Set the conforming array attributes
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,
 *                                  len(class_), H5T_CSET_ASCII)             # <<<<<<<<<<<<<<
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,
 *                                  len(version), H5T_CSET_ASCII)
*/
    if (unlikely(__pyx_v_class_ == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 1601, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_PyBytes_GET_SIZE(__pyx_v_class_); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1601, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1600
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:
 *       # Set the conforming array attributes
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,             # <<<<<<<<<<<<<<
 *                                  len(class_), H5T_CSET_ASCII)
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,
*/
    (void)(H5ATTRset_attribute_string(__pyx_v_self->__pyx_base.dataset_id, ((char *)"CLASS"), __pyx_t_22, __pyx_t_7, H5T_CSET_ASCII));

    /* "tables/hdf5extension.pyx":1602
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,
 *                                  len(class_), H5T_CSET_ASCII)
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,             # <<<<<<<<<<<<<<
 *                                  len(version), H5T_CSET_ASCII)
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,
*/
    if (unlikely(__pyx_v_version == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 1602, __pyx_L1_error)
    }
    __pyx_t_22 = __Pyx_PyBytes_AsWritableString(__pyx_v_version); if (unlikely((!__pyx_t_22) && PyErr_Occurred())) __PYX_ERR(0, 1602, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1603
 *                                  len(class_), H5T_CSET_ASCII)
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,
 *                                  len(version), H5T_CSET_ASCII)             # <<<<<<<<<<<<<<
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,
 *                                  len(encoded_title), H5T_CSET_ASCII)
*/
    if (unlikely(__pyx_v_version == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 1603, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_PyBytes_GET_SIZE(__pyx_v_version); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1603, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1602
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,
 *                                  len(class_), H5T_CSET_ASCII)
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,             # <<<<<<<<<<<<<<
 *                                  len(version), H5T_CSET_ASCII)
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,
*/
    (void)(H5ATTRset_attribute_string(__pyx_v_self->__pyx_base.dataset_id, ((char *)"VERSION"), __pyx_t_22, __pyx_t_7, H5T_CSET_ASCII));

    /* "tables/hdf5extension.pyx":1604
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,
 *                                  len(version), H5T_CSET_ASCII)
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,             # <<<<<<<<<<<<<<
 *                                  len(encoded_title), H5T_CSET_ASCII)
 *       if self.extdim >= 0:
*/
    if (unlikely(__pyx_v_encoded_title == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 1604, __pyx_L1_error)
    }
    __pyx_t_22 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_title); if (unlikely((!__pyx_t_22) && PyErr_Occurred())) __PYX_ERR(0, 1604, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1605
 *                                  len(version), H5T_CSET_ASCII)
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,
 *                                  len(encoded_title), H5T_CSET_ASCII)             # <<<<<<<<<<<<<<
 *       if self.extdim >= 0:
 *         extdim = <ndarray>np.array([self.extdim], dtype="int32")
*/
    if (unlikely(__pyx_v_encoded_title == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 1605, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_PyBytes_GET_SIZE(__pyx_v_encoded_title); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1605, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1604
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,
 *                                  len(version), H5T_CSET_ASCII)
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,             # <<<<<<<<<<<<<<
 *                                  len(encoded_title), H5T_CSET_ASCII)
 *       if self.extdim >= 0:
*/
    (void)(H5ATTRset_attribute_string(__pyx_v_self->__pyx_base.dataset_id, ((char *)"TITLE"), __pyx_t_22, __pyx_t_7, H5T_CSET_ASCII));

    /* "tables/hdf5extension.pyx":1606
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,
 *                                  len(encoded_title), H5T_CSET_ASCII)
 *       if self.extdim >= 0:             # <<<<<<<<<<<<<<
 *         extdim = <ndarray>np.array([self.extdim], dtype="int32")
 *         # Attach the EXTDIM attribute in case of enlargeable arrays
*/
    __pyx_t_12 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_extdim); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1606, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_5 = PyObject_RichCompare(__pyx_t_12, __pyx_mstate_global->__pyx_int_0, Py_GE); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1606, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 1606, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (__pyx_t_9) {

      /* "tables/hdf5extension.pyx":1607
 *                                  len(encoded_title), H5T_CSET_ASCII)
 *       if self.extdim >= 0:
 *         extdim = <ndarray>np.array([self.extdim], dtype="int32")             # <<<<<<<<<<<<<<
 *         # Attach the EXTDIM attribute in case of enlargeable arrays
 *         H5ATTRset_attribute(self.dataset_id, "EXTDIM", H5T_NATIVE_INT,
*/
      __pyx_t_12 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_array); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_extdim); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_4);
      if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_t_4) != (0)) __PYX_ERR(0, 1607, __pyx_L1_error);
      __pyx_t_4 = 0;
      __pyx_t_3 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_10);
        assert(__pyx_t_12);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_12);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
        __pyx_t_3 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_12, __pyx_t_2};
        __pyx_t_4 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1607, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_mstate_global->__pyx_n_u_int32, __pyx_t_4, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 1607, __pyx_L1_error)
        __pyx_t_5 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_4);
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1607, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
      }
      __pyx_t_10 = __pyx_t_5;
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_v_extdim = ((PyArrayObject *)__pyx_t_10);
      __pyx_t_10 = 0;

      /* "tables/hdf5extension.pyx":1609
 *         extdim = <ndarray>np.array([self.extdim], dtype="int32")
 *         # Attach the EXTDIM attribute in case of enlargeable arrays
 *         H5ATTRset_attribute(self.dataset_id, "EXTDIM", H5T_NATIVE_INT,             # <<<<<<<<<<<<<<
 *                             0, NULL, PyArray_BYTES(extdim))
 * 
*/
      (void)(H5ATTRset_attribute(__pyx_v_self->__pyx_base.dataset_id, ((char *)"EXTDIM"), H5T_NATIVE_INT, 0, NULL, PyArray_BYTES(__pyx_v_extdim)));

      /* "tables/hdf5extension.pyx":1606
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,
 *                                  len(encoded_title), H5T_CSET_ASCII)
 *       if self.extdim >= 0:             # <<<<<<<<<<<<<<
 *         extdim = <ndarray>np.array([self.extdim], dtype="int32")
 *         # Attach the EXTDIM attribute in case of enlargeable arrays
*/
    }

    /* "tables/hdf5extension.pyx":1598
 *       raise HDF5ExtError("Problems creating the %s." % self.__class__.__name__)
 * 
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:             # <<<<<<<<<<<<<<
 *       # Set the conforming array attributes
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,
*/
  }

  /* "tables/hdf5extension.pyx":1614
 *     # Get the native type (so that it is HDF5 who is the responsible to deal
 *     # with non-native byteorders on-disk)
 *     self.type_id = get_native_type(self.disk_type_id)             # <<<<<<<<<<<<<<
 * 
 *     return self.dataset_id
*/
  __pyx_t_6 = __pyx_f_6tables_14utilsextension_get_native_type(__pyx_v_self->__pyx_base.disk_type_id); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1614, __pyx_L1_error)
  __pyx_v_self->__pyx_base.type_id = __pyx_t_6;

  /* "tables/hdf5extension.pyx":1616
 *     self.type_id = get_native_type(self.disk_type_id)
 * 
 *     return self.dataset_id             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_10 = __Pyx_PyLong_From_hid_t(__pyx_v_self->__pyx_base.dataset_id); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1616, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_r = __pyx_t_10;
  __pyx_t_10 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1538
 * 
 * 
 *   def _create_carray(self, object title):             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef herr_t ret
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("tables.hdf5extension.Array._create_carray", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_complib);
  __Pyx_XDECREF(__pyx_v_version);
  __Pyx_XDECREF(__pyx_v_class_);
  __Pyx_XDECREF((PyObject *)__pyx_v_dflts);
  __Pyx_XDECREF((PyObject *)__pyx_v_extdim);
  __Pyx_XDECREF(__pyx_v_atom);
  __Pyx_XDECREF(__pyx_v_encoded_title);
  __Pyx_XDECREF(__pyx_v_encoded_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1619
 * 
 * 
 *   def _open_array(self):             # <<<<<<<<<<<<<<
 *     cdef size_t type_size, type_precision
 *     cdef H5T_class_t class_id
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_5_open_array(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Array_5_open_array = {"_open_array", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_5_open_array, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_5_open_array(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_open_array (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_open_array", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_open_array", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Array_4_open_array(((struct __pyx_obj_6tables_13hdf5extension_Array *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_4_open_array(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self) {
  enum H5T_class_t __pyx_v_class_id;
  char __pyx_v_cbyteorder[11];
  int __pyx_v_i;
  herr_t __pyx_v_ret;
  PyObject *__pyx_v_shape = 0;
  PyObject *__pyx_v_chunkshapes = 0;
  PyObject *__pyx_v_atom = 0;
  int __pyx_v_fill_status;
  PyArrayObject *__pyx_v_dflts = 0;
  void *__pyx_v_fill_data;
  PyObject *__pyx_v_encoded_name = 0;
  PyObject *__pyx_v_byteorder = 0;
  PyObject *__pyx_v_filters = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  char *__pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10[5];
  PyObject *(*__pyx_t_11)(PyObject *);
  hid_t __pyx_t_12;
  hid_t __pyx_t_13;
  int __pyx_t_14;
  hbool_t __pyx_t_15;
  hbool_t __pyx_t_16;
  int __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_open_array", 0);

  /* "tables/hdf5extension.pyx":1633
 *     cdef str byteorder
 * 
 *     encoded_name = self.name.encode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     # Open the dataset
*/
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.name;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1633, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 1633, __pyx_L1_error)
  __pyx_v_encoded_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1636
 * 
 *     # Open the dataset
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)             # <<<<<<<<<<<<<<
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
*/
  if (unlikely(__pyx_v_encoded_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1636, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_name); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 1636, __pyx_L1_error)
  __pyx_v_self->__pyx_base.dataset_id = H5Dopen(__pyx_v_self->__pyx_base.__pyx_base.parent_id, __pyx_t_4, H5P_DEFAULT);

  /* "tables/hdf5extension.pyx":1637
 *     # Open the dataset
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)
 *     if self.dataset_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
 *                          (self.name, self._v_parent._v_pathname))
*/
  __pyx_t_5 = (__pyx_v_self->__pyx_base.dataset_id < 0);
  if (unlikely(__pyx_t_5)) {

    /* "tables/hdf5extension.pyx":1638
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %             # <<<<<<<<<<<<<<
 *                          (self.name, self._v_parent._v_pathname))
 *     # Get the datatype handles
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1638, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);

    /* "tables/hdf5extension.pyx":1639
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
 *                          (self.name, self._v_parent._v_pathname))             # <<<<<<<<<<<<<<
 *     # Get the datatype handles
 *     self.disk_type_id, self.type_id = self._get_type_ids()
*/
    __pyx_t_7 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_self->__pyx_base.__pyx_base.name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1639, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_parent); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1639, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_v_pathname); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1639, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_t_9), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1639, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u_Non_existing_node;
    __pyx_t_10[1] = __pyx_t_7;
    __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u_under;
    __pyx_t_10[3] = __pyx_t_8;
    __pyx_t_10[4] = __pyx_mstate_global->__pyx_kp_u__4;

    /* "tables/hdf5extension.pyx":1638
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %             # <<<<<<<<<<<<<<
 *                          (self.name, self._v_parent._v_pathname))
 *     # Get the datatype handles
*/
    __pyx_t_9 = __Pyx_PyUnicode_Join(__pyx_t_10, 5, 20 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7) + 11 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8) + 2, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8));
    if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1638, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_9};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1638, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1638, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1637
 *     # Open the dataset
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)
 *     if self.dataset_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
 *                          (self.name, self._v_parent._v_pathname))
*/
  }

  /* "tables/hdf5extension.pyx":1641
 *                          (self.name, self._v_parent._v_pathname))
 *     # Get the datatype handles
 *     self.disk_type_id, self.type_id = self._get_type_ids()             # <<<<<<<<<<<<<<
 *     # Get the atom for this type
 *     atom = atom_from_hdf5_type(self.type_id)
*/
  __pyx_t_1 = ((struct __pyx_vtabstruct_6tables_13hdf5extension_Array *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get_type_ids(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1641, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 1641, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_6 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_6);
      __pyx_t_9 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_9);
    } else {
      __pyx_t_6 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1641, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_6);
      __pyx_t_9 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1641, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_9);
    }
    #else
    __pyx_t_6 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1641, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1641, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1641, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_11 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_2);
    index = 0; __pyx_t_6 = __pyx_t_11(__pyx_t_2); if (unlikely(!__pyx_t_6)) goto __pyx_L4_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_6);
    index = 1; __pyx_t_9 = __pyx_t_11(__pyx_t_2); if (unlikely(!__pyx_t_9)) goto __pyx_L4_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_9);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_11(__pyx_t_2), 2) < (0)) __PYX_ERR(0, 1641, __pyx_L1_error)
    __pyx_t_11 = NULL;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L5_unpacking_done;
    __pyx_L4_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_11 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 1641, __pyx_L1_error)
    __pyx_L5_unpacking_done:;
  }
  __pyx_t_12 = __Pyx_PyLong_As_hid_t(__pyx_t_6); if (unlikely((__pyx_t_12 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1641, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_13 = __Pyx_PyLong_As_hid_t(__pyx_t_9); if (unlikely((__pyx_t_13 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1641, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_self->__pyx_base.disk_type_id = __pyx_t_12;
  __pyx_v_self->__pyx_base.type_id = __pyx_t_13;

  /* "tables/hdf5extension.pyx":1643
 *     self.disk_type_id, self.type_id = self._get_type_ids()
 *     # Get the atom for this type
 *     atom = atom_from_hdf5_type(self.type_id)             # <<<<<<<<<<<<<<
 * 
 *     # Get the rank for this array object
*/
  __pyx_t_9 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_atom_from_hdf5_type); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1643, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_2 = __Pyx_PyLong_From_hid_t(__pyx_v_self->__pyx_base.type_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1643, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_6);
    assert(__pyx_t_9);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
    __Pyx_INCREF(__pyx_t_9);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_t_2};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1643, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_atom = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1646
 * 
 *     # Get the rank for this array object
 *     if H5ARRAYget_ndims(self.dataset_id, &self.rank) < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems getting ndims!")
 *     # Allocate space for the dimension axis info
*/
  __pyx_t_5 = (H5ARRAYget_ndims(__pyx_v_self->__pyx_base.dataset_id, (&__pyx_v_self->rank)) < 0);
  if (unlikely(__pyx_t_5)) {

    /* "tables/hdf5extension.pyx":1647
 *     # Get the rank for this array object
 *     if H5ARRAYget_ndims(self.dataset_id, &self.rank) < 0:
 *       raise HDF5ExtError("Problems getting ndims!")             # <<<<<<<<<<<<<<
 *     # Allocate space for the dimension axis info
 *     self.dims = <hsize_t *>malloc(self.rank * sizeof(hsize_t))
*/
    __pyx_t_6 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1647, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
      assert(__pyx_t_6);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u_Problems_getting_ndims};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1647, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1647, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1646
 * 
 *     # Get the rank for this array object
 *     if H5ARRAYget_ndims(self.dataset_id, &self.rank) < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems getting ndims!")
 *     # Allocate space for the dimension axis info
*/
  }

  /* "tables/hdf5extension.pyx":1649
 *       raise HDF5ExtError("Problems getting ndims!")
 *     # Allocate space for the dimension axis info
 *     self.dims = <hsize_t *>malloc(self.rank * sizeof(hsize_t))             # <<<<<<<<<<<<<<
 *     self.maxdims = <hsize_t *>malloc(self.rank * sizeof(hsize_t))
 *     # Get info on dimensions, class and type (of base class)
*/
  __pyx_v_self->__pyx_base.dims = ((hsize_t *)malloc((__pyx_v_self->rank * (sizeof(hsize_t)))));

  /* "tables/hdf5extension.pyx":1650
 *     # Allocate space for the dimension axis info
 *     self.dims = <hsize_t *>malloc(self.rank * sizeof(hsize_t))
 *     self.maxdims = <hsize_t *>malloc(self.rank * sizeof(hsize_t))             # <<<<<<<<<<<<<<
 *     # Get info on dimensions, class and type (of base class)
 *     ret = H5ARRAYget_info(self.dataset_id, self.disk_type_id,
*/
  __pyx_v_self->maxdims = ((hsize_t *)malloc((__pyx_v_self->rank * (sizeof(hsize_t)))));

  /* "tables/hdf5extension.pyx":1652
 *     self.maxdims = <hsize_t *>malloc(self.rank * sizeof(hsize_t))
 *     # Get info on dimensions, class and type (of base class)
 *     ret = H5ARRAYget_info(self.dataset_id, self.disk_type_id,             # <<<<<<<<<<<<<<
 *                           self.dims, self.maxdims,
 *                           &class_id, cbyteorder)
*/
  __pyx_v_ret = H5ARRAYget_info(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.disk_type_id, __pyx_v_self->__pyx_base.dims, __pyx_v_self->maxdims, (&__pyx_v_class_id), __pyx_v_cbyteorder);

  /* "tables/hdf5extension.pyx":1655
 *                           self.dims, self.maxdims,
 *                           &class_id, cbyteorder)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Unable to get array info.")
 * 
*/
  __pyx_t_5 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_5)) {

    /* "tables/hdf5extension.pyx":1656
 *                           &class_id, cbyteorder)
 *     if ret < 0:
 *       raise HDF5ExtError("Unable to get array info.")             # <<<<<<<<<<<<<<
 * 
 *     byteorder = cstr_to_pystr(cbyteorder)
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1656, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_Unable_to_get_array_info};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1656, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1656, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1655
 *                           self.dims, self.maxdims,
 *                           &class_id, cbyteorder)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Unable to get array info.")
 * 
*/
  }

  /* "tables/hdf5extension.pyx":1658
 *       raise HDF5ExtError("Unable to get array info.")
 * 
 *     byteorder = cstr_to_pystr(cbyteorder)             # <<<<<<<<<<<<<<
 * 
 *     # Get the extendable dimension (if any)
*/
  __pyx_t_1 = __pyx_f_6tables_14utilsextension_cstr_to_pystr(__pyx_v_cbyteorder); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1658, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_byteorder = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1661
 * 
 *     # Get the extendable dimension (if any)
 *     self.extdim = -1  # default is non-extensible Array             # <<<<<<<<<<<<<<
 *     for i from 0 <= i < self.rank:
 *       if self.maxdims[i] == <hsize_t>-1:
*/
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_extdim, __pyx_mstate_global->__pyx_int_neg_1) < (0)) __PYX_ERR(0, 1661, __pyx_L1_error)

  /* "tables/hdf5extension.pyx":1662
 *     # Get the extendable dimension (if any)
 *     self.extdim = -1  # default is non-extensible Array
 *     for i from 0 <= i < self.rank:             # <<<<<<<<<<<<<<
 *       if self.maxdims[i] == <hsize_t>-1:
 *         self.extdim = i
*/
  __pyx_t_14 = __pyx_v_self->rank;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_14; __pyx_v_i++) {

    /* "tables/hdf5extension.pyx":1663
 *     self.extdim = -1  # default is non-extensible Array
 *     for i from 0 <= i < self.rank:
 *       if self.maxdims[i] == <hsize_t>-1:             # <<<<<<<<<<<<<<
 *         self.extdim = i
 *         break
*/
    __pyx_t_5 = ((__pyx_v_self->maxdims[__pyx_v_i]) == ((hsize_t)-1L));
    if (__pyx_t_5) {

      /* "tables/hdf5extension.pyx":1664
 *     for i from 0 <= i < self.rank:
 *       if self.maxdims[i] == <hsize_t>-1:
 *         self.extdim = i             # <<<<<<<<<<<<<<
 *         break
 * 
*/
      __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1664, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_extdim, __pyx_t_1) < (0)) __PYX_ERR(0, 1664, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "tables/hdf5extension.pyx":1665
 *       if self.maxdims[i] == <hsize_t>-1:
 *         self.extdim = i
 *         break             # <<<<<<<<<<<<<<
 * 
 *     # Get the shape as a python tuple
*/
      goto __pyx_L9_break;

      /* "tables/hdf5extension.pyx":1663
 *     self.extdim = -1  # default is non-extensible Array
 *     for i from 0 <= i < self.rank:
 *       if self.maxdims[i] == <hsize_t>-1:             # <<<<<<<<<<<<<<
 *         self.extdim = i
 *         break
*/
    }
  }
  __pyx_L9_break:;

  /* "tables/hdf5extension.pyx":1668
 * 
 *     # Get the shape as a python tuple
 *     shape = getshape(self.rank, self.dims)             # <<<<<<<<<<<<<<
 * 
 *     # Allocate space for the dimension chunking info
*/
  __pyx_t_1 = __pyx_f_6tables_13hdf5extension_getshape(__pyx_v_self->rank, __pyx_v_self->__pyx_base.dims); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1668, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_shape = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1671
 * 
 *     # Allocate space for the dimension chunking info
 *     self.dims_chunk = <hsize_t *>malloc(self.rank * sizeof(hsize_t))             # <<<<<<<<<<<<<<
 *     if H5ARRAYget_chunkshape(self.dataset_id, self.rank, self.dims_chunk) < 0:
 *       # The Array class is not chunked!
*/
  __pyx_v_self->dims_chunk = ((hsize_t *)malloc((__pyx_v_self->rank * (sizeof(hsize_t)))));

  /* "tables/hdf5extension.pyx":1672
 *     # Allocate space for the dimension chunking info
 *     self.dims_chunk = <hsize_t *>malloc(self.rank * sizeof(hsize_t))
 *     if H5ARRAYget_chunkshape(self.dataset_id, self.rank, self.dims_chunk) < 0:             # <<<<<<<<<<<<<<
 *       # The Array class is not chunked!
 *       chunkshapes = None
*/
  __pyx_t_5 = (H5ARRAYget_chunkshape(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->rank, __pyx_v_self->dims_chunk) < 0);
  if (__pyx_t_5) {

    /* "tables/hdf5extension.pyx":1674
 *     if H5ARRAYget_chunkshape(self.dataset_id, self.rank, self.dims_chunk) < 0:
 *       # The Array class is not chunked!
 *       chunkshapes = None             # <<<<<<<<<<<<<<
 *       # Blosc2 optimized operations cannot be used (no chunking nor filters).
 *       self.blosc2_support_read = False
*/
    __Pyx_INCREF(Py_None);
    __pyx_v_chunkshapes = Py_None;

    /* "tables/hdf5extension.pyx":1676
 *       chunkshapes = None
 *       # Blosc2 optimized operations cannot be used (no chunking nor filters).
 *       self.blosc2_support_read = False             # <<<<<<<<<<<<<<
 *       self.blosc2_support_write = False
 *     else:
*/
    __pyx_v_self->blosc2_support_read = 0;

    /* "tables/hdf5extension.pyx":1677
 *       # Blosc2 optimized operations cannot be used (no chunking nor filters).
 *       self.blosc2_support_read = False
 *       self.blosc2_support_write = False             # <<<<<<<<<<<<<<
 *     else:
 *       # Get the chunkshape as a python tuple
*/
    __pyx_v_self->blosc2_support_write = 0;

    /* "tables/hdf5extension.pyx":1672
 *     # Allocate space for the dimension chunking info
 *     self.dims_chunk = <hsize_t *>malloc(self.rank * sizeof(hsize_t))
 *     if H5ARRAYget_chunkshape(self.dataset_id, self.rank, self.dims_chunk) < 0:             # <<<<<<<<<<<<<<
 *       # The Array class is not chunked!
 *       chunkshapes = None
*/
    goto __pyx_L11;
  }

  /* "tables/hdf5extension.pyx":1680
 *     else:
 *       # Get the chunkshape as a python tuple
 *       chunkshapes = getshape(self.rank, self.dims_chunk)             # <<<<<<<<<<<<<<
 *       # Decide whether Blosc2 optimized operations can be used.
 *       filters = get_filters(self.parent_id, self.name) or {}
*/
  /*else*/ {
    __pyx_t_1 = __pyx_f_6tables_13hdf5extension_getshape(__pyx_v_self->rank, __pyx_v_self->dims_chunk); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1680, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_chunkshapes = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":1682
 *       chunkshapes = getshape(self.rank, self.dims_chunk)
 *       # Decide whether Blosc2 optimized operations can be used.
 *       filters = get_filters(self.parent_id, self.name) or {}             # <<<<<<<<<<<<<<
 *       (self.blosc2_support_read, self.blosc2_support_write) = (
 *           _supports_opt_blosc2_read_write(byteorder, list(filters),
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_get_filters); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1682, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyLong_From_hid_t(__pyx_v_self->__pyx_base.__pyx_base.parent_id); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1682, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_9);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_t_8, __pyx_v_self->__pyx_base.__pyx_base.name};
      __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1682, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
    }
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 1682, __pyx_L1_error)
    if (!__pyx_t_5) {
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else {
      __Pyx_INCREF(__pyx_t_6);
      __pyx_t_1 = __pyx_t_6;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      goto __pyx_L12_bool_binop_done;
    }
    __pyx_t_6 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1682, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_t_6);
    __pyx_t_1 = __pyx_t_6;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_L12_bool_binop_done:;
    __pyx_v_filters = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":1684
 *       filters = get_filters(self.parent_id, self.name) or {}
 *       (self.blosc2_support_read, self.blosc2_support_write) = (
 *           _supports_opt_blosc2_read_write(byteorder, list(filters),             # <<<<<<<<<<<<<<
 *                                           self._v_file.mode))
 * 
*/
    __pyx_t_6 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_supports_opt_blosc2_read_write); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1684, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = PySequence_List(__pyx_v_filters); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1684, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);

    /* "tables/hdf5extension.pyx":1685
 *       (self.blosc2_support_read, self.blosc2_support_write) = (
 *           _supports_opt_blosc2_read_write(byteorder, list(filters),
 *                                           self._v_file.mode))             # <<<<<<<<<<<<<<
 * 
 *     # object arrays should not be read directly into memory
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_file); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1685, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_mode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1685, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_9);
      assert(__pyx_t_6);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[4] = {__pyx_t_6, __pyx_v_byteorder, __pyx_t_8, __pyx_t_7};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_3, (4-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1684, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
      PyObject* sequence = __pyx_t_1;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 1683, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 0);
        __Pyx_INCREF(__pyx_t_9);
        __pyx_t_7 = PyTuple_GET_ITEM(sequence, 1);
        __Pyx_INCREF(__pyx_t_7);
      } else {
        __pyx_t_9 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
        if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1683, __pyx_L1_error)
        __Pyx_XGOTREF(__pyx_t_9);
        __pyx_t_7 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1683, __pyx_L1_error)
        __Pyx_XGOTREF(__pyx_t_7);
      }
      #else
      __pyx_t_9 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1683, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_7 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1683, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_8 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1683, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_11 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_8);
      index = 0; __pyx_t_9 = __pyx_t_11(__pyx_t_8); if (unlikely(!__pyx_t_9)) goto __pyx_L14_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_9);
      index = 1; __pyx_t_7 = __pyx_t_11(__pyx_t_8); if (unlikely(!__pyx_t_7)) goto __pyx_L14_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_7);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_11(__pyx_t_8), 2) < (0)) __PYX_ERR(0, 1683, __pyx_L1_error)
      __pyx_t_11 = NULL;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      goto __pyx_L15_unpacking_done;
      __pyx_L14_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_11 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 1683, __pyx_L1_error)
      __pyx_L15_unpacking_done:;
    }

    /* "tables/hdf5extension.pyx":1683
 *       # Decide whether Blosc2 optimized operations can be used.
 *       filters = get_filters(self.parent_id, self.name) or {}
 *       (self.blosc2_support_read, self.blosc2_support_write) = (             # <<<<<<<<<<<<<<
 *           _supports_opt_blosc2_read_write(byteorder, list(filters),
 *                                           self._v_file.mode))
*/
    __pyx_t_15 = __Pyx_PyLong_As_hbool_t(__pyx_t_9); if (unlikely((__pyx_t_15 == ((hbool_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1683, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_16 = __Pyx_PyLong_As_hbool_t(__pyx_t_7); if (unlikely((__pyx_t_16 == ((hbool_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1683, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_self->blosc2_support_read = __pyx_t_15;
    __pyx_v_self->blosc2_support_write = __pyx_t_16;
  }
  __pyx_L11:;

  /* "tables/hdf5extension.pyx":1688
 * 
 *     # object arrays should not be read directly into memory
 *     if atom.dtype != object:             # <<<<<<<<<<<<<<
 *       # Get the fill value
 *       dflts = np.zeros((), dtype=atom.dtype)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1688, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = PyObject_RichCompare(__pyx_t_1, __pyx_builtin_object, Py_NE); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1688, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 1688, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__pyx_t_5) {

    /* "tables/hdf5extension.pyx":1690
 *     if atom.dtype != object:
 *       # Get the fill value
 *       dflts = np.zeros((), dtype=atom.dtype)             # <<<<<<<<<<<<<<
 *       fill_data = PyArray_DATA(dflts)
 *       H5ARRAYget_fill_value(self.dataset_id, self.type_id,
*/
    __pyx_t_1 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1690, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_zeros); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1690, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1690, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_8);
      assert(__pyx_t_1);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, __pyx_mstate_global->__pyx_empty_tuple};
      __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1690, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_9, __pyx_t_6, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 1690, __pyx_L1_error)
      __pyx_t_7 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1690, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }
    if (!(likely(((__pyx_t_7) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_7, __pyx_mstate_global->__pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1690, __pyx_L1_error)
    __pyx_v_dflts = ((PyArrayObject *)__pyx_t_7);
    __pyx_t_7 = 0;

    /* "tables/hdf5extension.pyx":1691
 *       # Get the fill value
 *       dflts = np.zeros((), dtype=atom.dtype)
 *       fill_data = PyArray_DATA(dflts)             # <<<<<<<<<<<<<<
 *       H5ARRAYget_fill_value(self.dataset_id, self.type_id,
 *                             &fill_status, fill_data);
*/
    __pyx_v_fill_data = PyArray_DATA(__pyx_v_dflts);

    /* "tables/hdf5extension.pyx":1692
 *       dflts = np.zeros((), dtype=atom.dtype)
 *       fill_data = PyArray_DATA(dflts)
 *       H5ARRAYget_fill_value(self.dataset_id, self.type_id,             # <<<<<<<<<<<<<<
 *                             &fill_status, fill_data);
 *       if fill_status == H5D_FILL_VALUE_UNDEFINED:
*/
    (void)(H5ARRAYget_fill_value(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, (&__pyx_v_fill_status), __pyx_v_fill_data));

    /* "tables/hdf5extension.pyx":1694
 *       H5ARRAYget_fill_value(self.dataset_id, self.type_id,
 *                             &fill_status, fill_data);
 *       if fill_status == H5D_FILL_VALUE_UNDEFINED:             # <<<<<<<<<<<<<<
 *         # This can only happen with datasets created with other libraries
 *         # than PyTables.
*/
    __pyx_t_5 = (__pyx_v_fill_status == H5D_FILL_VALUE_UNDEFINED);
    if (__pyx_t_5) {

      /* "tables/hdf5extension.pyx":1697
 *         # This can only happen with datasets created with other libraries
 *         # than PyTables.
 *         dflts = None             # <<<<<<<<<<<<<<
 *       if dflts is not None and atom.shape == ():
 *         # The default is preferred as a scalar value instead of 0-dim array
*/
      __Pyx_INCREF(Py_None);
      __Pyx_DECREF_SET(__pyx_v_dflts, ((PyArrayObject *)Py_None));

      /* "tables/hdf5extension.pyx":1694
 *       H5ARRAYget_fill_value(self.dataset_id, self.type_id,
 *                             &fill_status, fill_data);
 *       if fill_status == H5D_FILL_VALUE_UNDEFINED:             # <<<<<<<<<<<<<<
 *         # This can only happen with datasets created with other libraries
 *         # than PyTables.
*/
    }

    /* "tables/hdf5extension.pyx":1698
 *         # than PyTables.
 *         dflts = None
 *       if dflts is not None and atom.shape == ():             # <<<<<<<<<<<<<<
 *         # The default is preferred as a scalar value instead of 0-dim array
 *         atom.dflt = dflts[()]
*/
    __pyx_t_17 = (((PyObject *)__pyx_v_dflts) != Py_None);
    if (__pyx_t_17) {
    } else {
      __pyx_t_5 = __pyx_t_17;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1698, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = PyObject_RichCompare(__pyx_t_7, __pyx_mstate_global->__pyx_empty_tuple, Py_EQ); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1698, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_17 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_17 < 0))) __PYX_ERR(0, 1698, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_5 = __pyx_t_17;
    __pyx_L19_bool_binop_done:;
    if (__pyx_t_5) {

      /* "tables/hdf5extension.pyx":1700
 *       if dflts is not None and atom.shape == ():
 *         # The default is preferred as a scalar value instead of 0-dim array
 *         atom.dflt = dflts[()]             # <<<<<<<<<<<<<<
 *       else:
 *         atom.dflt = dflts
*/
      __pyx_t_8 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_dflts), __pyx_mstate_global->__pyx_empty_tuple); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1700, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__Pyx_PyObject_SetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_dflt, __pyx_t_8) < (0)) __PYX_ERR(0, 1700, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "tables/hdf5extension.pyx":1698
 *         # than PyTables.
 *         dflts = None
 *       if dflts is not None and atom.shape == ():             # <<<<<<<<<<<<<<
 *         # The default is preferred as a scalar value instead of 0-dim array
 *         atom.dflt = dflts[()]
*/
      goto __pyx_L18;
    }

    /* "tables/hdf5extension.pyx":1702
 *         atom.dflt = dflts[()]
 *       else:
 *         atom.dflt = dflts             # <<<<<<<<<<<<<<
 * 
 *     # Get the byteorder
*/
    /*else*/ {
      if (__Pyx_PyObject_SetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_dflt, ((PyObject *)__pyx_v_dflts)) < (0)) __PYX_ERR(0, 1702, __pyx_L1_error)
    }
    __pyx_L18:;

    /* "tables/hdf5extension.pyx":1688
 * 
 *     # object arrays should not be read directly into memory
 *     if atom.dtype != object:             # <<<<<<<<<<<<<<
 *       # Get the fill value
 *       dflts = np.zeros((), dtype=atom.dtype)
*/
  }

  /* "tables/hdf5extension.pyx":1705
 * 
 *     # Get the byteorder
 *     self.byteorder = correct_byteorder(atom.type, byteorder)             # <<<<<<<<<<<<<<
 * 
 *     return self.dataset_id, atom, shape, chunkshapes
*/
  __pyx_t_7 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_correct_byteorder); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
    assert(__pyx_t_7);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
    __Pyx_INCREF(__pyx_t_7);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_7, __pyx_t_9, __pyx_v_byteorder};
    __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1705, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
  }
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_byteorder, __pyx_t_8) < (0)) __PYX_ERR(0, 1705, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "tables/hdf5extension.pyx":1707
 *     self.byteorder = correct_byteorder(atom.type, byteorder)
 * 
 *     return self.dataset_id, atom, shape, chunkshapes             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_8 = __Pyx_PyLong_From_hid_t(__pyx_v_self->__pyx_base.dataset_id); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1707, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_6 = PyTuple_New(4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1707, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_8);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_8) != (0)) __PYX_ERR(0, 1707, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_atom);
  __Pyx_GIVEREF(__pyx_v_atom);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_atom) != (0)) __PYX_ERR(0, 1707, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_shape);
  __Pyx_GIVEREF(__pyx_v_shape);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_shape) != (0)) __PYX_ERR(0, 1707, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_chunkshapes);
  __Pyx_GIVEREF(__pyx_v_chunkshapes);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_v_chunkshapes) != (0)) __PYX_ERR(0, 1707, __pyx_L1_error);
  __pyx_t_8 = 0;
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1619
 * 
 * 
 *   def _open_array(self):             # <<<<<<<<<<<<<<
 *     cdef size_t type_size, type_precision
 *     cdef H5T_class_t class_id
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("tables.hdf5extension.Array._open_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_shape);
  __Pyx_XDECREF(__pyx_v_chunkshapes);
  __Pyx_XDECREF(__pyx_v_atom);
  __Pyx_XDECREF((PyObject *)__pyx_v_dflts);
  __Pyx_XDECREF(__pyx_v_encoded_name);
  __Pyx_XDECREF(__pyx_v_byteorder);
  __Pyx_XDECREF(__pyx_v_filters);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1710
 * 
 * 
 *   def _append(self, ndarray nparr):             # <<<<<<<<<<<<<<
 *     cdef int ret, extdim
 *     cdef hsize_t *dims_arr
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_7_append(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Array_7_append = {"_append", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_7_append, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_7_append(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyArrayObject *__pyx_v_nparr = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_append (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_nparr,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1710, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1710, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_append", 0) < (0)) __PYX_ERR(0, 1710, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_append", 1, 1, 1, i); __PYX_ERR(0, 1710, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1710, __pyx_L3_error)
    }
    __pyx_v_nparr = ((PyArrayObject *)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_append", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1710, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Array._append", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_nparr), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "nparr", 0))) __PYX_ERR(0, 1710, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Array_6_append(((struct __pyx_obj_6tables_13hdf5extension_Array *)__pyx_v_self), __pyx_v_nparr);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_6_append(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyArrayObject *__pyx_v_nparr) {
  int __pyx_v_ret;
  int __pyx_v_extdim;
  hsize_t *__pyx_v_dims_arr;
  void *__pyx_v_rbuf;
  PyObject *__pyx_v_shape = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  size_t __pyx_t_4;
  hsize_t *__pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  Py_ssize_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_append", 0);

  /* "tables/hdf5extension.pyx":1716
 *     cdef object shape
 * 
 *     if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *       raise ValueError("Cannot append to the reference types")
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_reference, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1716, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(__pyx_t_3)) {

    /* "tables/hdf5extension.pyx":1717
 * 
 *     if self.atom.kind == "reference":
 *       raise ValueError("Cannot append to the reference types")             # <<<<<<<<<<<<<<
 * 
 *     # Allocate space for the dimension axis info
*/
    __pyx_t_1 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_Cannot_append_to_the_reference_t};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1717, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1717, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1716
 *     cdef object shape
 * 
 *     if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *       raise ValueError("Cannot append to the reference types")
 * 
*/
  }

  /* "tables/hdf5extension.pyx":1720
 * 
 *     # Allocate space for the dimension axis info
 *     dims_arr = npy_malloc_dims(self.rank, PyArray_DIMS(nparr))             # <<<<<<<<<<<<<<
 *     # Get the pointer to the buffer data area
 *     rbuf = PyArray_DATA(nparr)
*/
  __pyx_t_5 = __pyx_f_6tables_13hdf5extension_npy_malloc_dims(__pyx_v_self->rank, PyArray_DIMS(__pyx_v_nparr)); if (unlikely(__pyx_t_5 == ((void *)NULL) && PyErr_Occurred())) __PYX_ERR(0, 1720, __pyx_L1_error)
  __pyx_v_dims_arr = __pyx_t_5;

  /* "tables/hdf5extension.pyx":1722
 *     dims_arr = npy_malloc_dims(self.rank, PyArray_DIMS(nparr))
 *     # Get the pointer to the buffer data area
 *     rbuf = PyArray_DATA(nparr)             # <<<<<<<<<<<<<<
 *     # Convert some NumPy types to HDF5 before storing.
 *     if self.atom.type == 'time64':
*/
  __pyx_v_rbuf = PyArray_DATA(__pyx_v_nparr);

  /* "tables/hdf5extension.pyx":1724
 *     rbuf = PyArray_DATA(nparr)
 *     # Convert some NumPy types to HDF5 before storing.
 *     if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *       self._convert_time64(nparr, 0)
 * 
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1724, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1724, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_time64, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1724, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":1725
 *     # Convert some NumPy types to HDF5 before storing.
 *     if self.atom.type == 'time64':
 *       self._convert_time64(nparr, 0)             # <<<<<<<<<<<<<<
 * 
 *     # Append the records
*/
    __pyx_t_1 = ((struct __pyx_vtabstruct_6tables_13hdf5extension_Array *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._convert_time64(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self), __pyx_v_nparr, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1725, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":1724
 *     rbuf = PyArray_DATA(nparr)
 *     # Convert some NumPy types to HDF5 before storing.
 *     if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *       self._convert_time64(nparr, 0)
 * 
*/
  }

  /* "tables/hdf5extension.pyx":1728
 * 
 *     # Append the records
 *     extdim = self.extdim             # <<<<<<<<<<<<<<
 *     with nogil:
 *         ret = H5ARRAYappend_records(self.dataset_id, self.type_id, self.rank,
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_extdim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyLong_As_int(__pyx_t_1); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1728, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_extdim = __pyx_t_6;

  /* "tables/hdf5extension.pyx":1729
 *     # Append the records
 *     extdim = self.extdim
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5ARRAYappend_records(self.dataset_id, self.type_id, self.rank,
 *                                     self.dims, dims_arr, extdim, rbuf)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "tables/hdf5extension.pyx":1730
 *     extdim = self.extdim
 *     with nogil:
 *         ret = H5ARRAYappend_records(self.dataset_id, self.type_id, self.rank,             # <<<<<<<<<<<<<<
 *                                     self.dims, dims_arr, extdim, rbuf)
 * 
*/
        __pyx_v_ret = H5ARRAYappend_records(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_self->rank, __pyx_v_self->__pyx_base.dims, __pyx_v_dims_arr, __pyx_v_extdim, __pyx_v_rbuf);
      }

      /* "tables/hdf5extension.pyx":1729
 *     # Append the records
 *     extdim = self.extdim
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5ARRAYappend_records(self.dataset_id, self.type_id, self.rank,
 *                                     self.dims, dims_arr, extdim, rbuf)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "tables/hdf5extension.pyx":1733
 *                                     self.dims, dims_arr, extdim, rbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems appending the elements")
 * 
*/
  __pyx_t_3 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_3)) {

    /* "tables/hdf5extension.pyx":1734
 * 
 *     if ret < 0:
 *       raise HDF5ExtError("Problems appending the elements")             # <<<<<<<<<<<<<<
 * 
 *     free(dims_arr)
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1734, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_Problems_appending_the_elements};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1734, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1734, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":1733
 *                                     self.dims, dims_arr, extdim, rbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems appending the elements")
 * 
*/
  }

  /* "tables/hdf5extension.pyx":1736
 *       raise HDF5ExtError("Problems appending the elements")
 * 
 *     free(dims_arr)             # <<<<<<<<<<<<<<
 *     # Update the new dimensionality
 *     shape = list(self.shape)
*/
  free(__pyx_v_dims_arr);

  /* "tables/hdf5extension.pyx":1738
 *     free(dims_arr)
 *     # Update the new dimensionality
 *     shape = list(self.shape)             # <<<<<<<<<<<<<<
 *     shape[self.extdim] = SizeType(self.dims[self.extdim])
 *     self.shape = tuple(shape)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1738, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PySequence_ListKeepNew(__pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1738, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_shape = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "tables/hdf5extension.pyx":1739
 *     # Update the new dimensionality
 *     shape = list(self.shape)
 *     shape[self.extdim] = SizeType(self.dims[self.extdim])             # <<<<<<<<<<<<<<
 *     self.shape = tuple(shape)
 * 
*/
  __pyx_t_1 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_SizeType); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_extdim); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_9 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1739, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyLong_From_hsize_t((__pyx_v_self->__pyx_base.dims[__pyx_t_9])); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_1);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_t_8};
    __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1739, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_extdim); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (unlikely((PyObject_SetItem(__pyx_v_shape, __pyx_t_2, __pyx_t_7) < 0))) __PYX_ERR(0, 1739, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "tables/hdf5extension.pyx":1740
 *     shape = list(self.shape)
 *     shape[self.extdim] = SizeType(self.dims[self.extdim])
 *     self.shape = tuple(shape)             # <<<<<<<<<<<<<<
 * 
 *   def _read_array(self, hsize_t start, hsize_t stop, hsize_t step,
*/
  __pyx_t_7 = __Pyx_PySequence_Tuple(__pyx_v_shape); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1740, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_shape, __pyx_t_7) < (0)) __PYX_ERR(0, 1740, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "tables/hdf5extension.pyx":1710
 * 
 * 
 *   def _append(self, ndarray nparr):             # <<<<<<<<<<<<<<
 *     cdef int ret, extdim
 *     cdef hsize_t *dims_arr
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("tables.hdf5extension.Array._append", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_shape);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1742
 *     self.shape = tuple(shape)
 * 
 *   def _read_array(self, hsize_t start, hsize_t stop, hsize_t step,             # <<<<<<<<<<<<<<
 *                  ndarray nparr):
 *     cdef herr_t ret
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_9_read_array(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Array_9_read_array = {"_read_array", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_9_read_array, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_9_read_array(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  hsize_t __pyx_v_start;
  hsize_t __pyx_v_stop;
  hsize_t __pyx_v_step;
  PyArrayObject *__pyx_v_nparr = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_read_array (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_start,&__pyx_mstate_global->__pyx_n_u_stop,&__pyx_mstate_global->__pyx_n_u_step,&__pyx_mstate_global->__pyx_n_u_nparr,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1742, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1742, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1742, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1742, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1742, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_read_array", 0) < (0)) __PYX_ERR(0, 1742, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 4; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_read_array", 1, 4, 4, i); __PYX_ERR(0, 1742, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 4)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1742, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1742, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1742, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1742, __pyx_L3_error)
    }
    __pyx_v_start = __Pyx_PyLong_As_hsize_t(values[0]); if (unlikely((__pyx_v_start == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1742, __pyx_L3_error)
    __pyx_v_stop = __Pyx_PyLong_As_hsize_t(values[1]); if (unlikely((__pyx_v_stop == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1742, __pyx_L3_error)
    __pyx_v_step = __Pyx_PyLong_As_hsize_t(values[2]); if (unlikely((__pyx_v_step == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1742, __pyx_L3_error)
    __pyx_v_nparr = ((PyArrayObject *)values[3]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_read_array", 1, 4, 4, __pyx_nargs); __PYX_ERR(0, 1742, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Array._read_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_nparr), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "nparr", 0))) __PYX_ERR(0, 1743, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Array_8_read_array(((struct __pyx_obj_6tables_13hdf5extension_Array *)__pyx_v_self), __pyx_v_start, __pyx_v_stop, __pyx_v_step, __pyx_v_nparr);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_8_read_array(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, hsize_t __pyx_v_start, hsize_t __pyx_v_stop, hsize_t __pyx_v_step, PyArrayObject *__pyx_v_nparr) {
  herr_t __pyx_v_ret;
  void *__pyx_v_rbuf;
  hsize_t __pyx_v_nrows;
  int __pyx_v_extdim;
  size_t __pyx_v_item_size;
  void *__pyx_v_refbuf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_t_7;
  char const *__pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_read_array", 0);

  /* "tables/hdf5extension.pyx":1748
 *     cdef hsize_t nrows
 *     cdef int extdim
 *     cdef size_t item_size = H5Tget_size(self.type_id)             # <<<<<<<<<<<<<<
 *     cdef void * refbuf = NULL
 * 
*/
  __pyx_v_item_size = H5Tget_size(__pyx_v_self->__pyx_base.type_id);

  /* "tables/hdf5extension.pyx":1749
 *     cdef int extdim
 *     cdef size_t item_size = H5Tget_size(self.type_id)
 *     cdef void * refbuf = NULL             # <<<<<<<<<<<<<<
 * 
 *     # Number of rows to read
*/
  __pyx_v_refbuf = NULL;

  /* "tables/hdf5extension.pyx":1752
 * 
 *     # Number of rows to read
 *     nrows = get_len_of_range(start, stop, step)             # <<<<<<<<<<<<<<
 * 
 *     # Get the pointer to the buffer data area
*/
  __pyx_v_nrows = get_len_of_range(__pyx_v_start, __pyx_v_stop, __pyx_v_step);

  /* "tables/hdf5extension.pyx":1755
 * 
 *     # Get the pointer to the buffer data area
 *     if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *       refbuf = malloc(nrows * item_size)
 *       rbuf = refbuf
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1755, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1755, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_reference, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1755, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":1756
 *     # Get the pointer to the buffer data area
 *     if self.atom.kind == "reference":
 *       refbuf = malloc(nrows * item_size)             # <<<<<<<<<<<<<<
 *       rbuf = refbuf
 *     else:
*/
    __pyx_v_refbuf = malloc((__pyx_v_nrows * __pyx_v_item_size));

    /* "tables/hdf5extension.pyx":1757
 *     if self.atom.kind == "reference":
 *       refbuf = malloc(nrows * item_size)
 *       rbuf = refbuf             # <<<<<<<<<<<<<<
 *     else:
 *       rbuf = PyArray_DATA(nparr)
*/
    __pyx_v_rbuf = __pyx_v_refbuf;

    /* "tables/hdf5extension.pyx":1755
 * 
 *     # Get the pointer to the buffer data area
 *     if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *       refbuf = malloc(nrows * item_size)
 *       rbuf = refbuf
*/
    goto __pyx_L3;
  }

  /* "tables/hdf5extension.pyx":1759
 *       rbuf = refbuf
 *     else:
 *       rbuf = PyArray_DATA(nparr)             # <<<<<<<<<<<<<<
 * 
 *     if hasattr(self, "extdim"):
*/
  /*else*/ {
    __pyx_v_rbuf = PyArray_DATA(__pyx_v_nparr);
  }
  __pyx_L3:;

  /* "tables/hdf5extension.pyx":1761
 *       rbuf = PyArray_DATA(nparr)
 * 
 *     if hasattr(self, "extdim"):             # <<<<<<<<<<<<<<
 *       extdim = self.extdim
 *     else:
*/
  __pyx_t_3 = __Pyx_HasAttr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_extdim); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 1761, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":1762
 * 
 *     if hasattr(self, "extdim"):
 *       extdim = self.extdim             # <<<<<<<<<<<<<<
 *     else:
 *       extdim = -1
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_extdim); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1762, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyLong_As_int(__pyx_t_2); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1762, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_extdim = __pyx_t_4;

    /* "tables/hdf5extension.pyx":1761
 *       rbuf = PyArray_DATA(nparr)
 * 
 *     if hasattr(self, "extdim"):             # <<<<<<<<<<<<<<
 *       extdim = self.extdim
 *     else:
*/
    goto __pyx_L4;
  }

  /* "tables/hdf5extension.pyx":1764
 *       extdim = self.extdim
 *     else:
 *       extdim = -1             # <<<<<<<<<<<<<<
 * 
 *     # Do the physical read
*/
  /*else*/ {
    __pyx_v_extdim = -1;
  }
  __pyx_L4:;

  /* "tables/hdf5extension.pyx":1767
 * 
 *     # Do the physical read
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5ARRAYread(self.dataset_id, self.type_id, start, nrows, step,
 *                           extdim, rbuf)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "tables/hdf5extension.pyx":1768
 *     # Do the physical read
 *     with nogil:
 *         ret = H5ARRAYread(self.dataset_id, self.type_id, start, nrows, step,             # <<<<<<<<<<<<<<
 *                           extdim, rbuf)
 * 
*/
        __pyx_v_ret = H5ARRAYread(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_start, __pyx_v_nrows, __pyx_v_step, __pyx_v_extdim, __pyx_v_rbuf);
      }

      /* "tables/hdf5extension.pyx":1767
 * 
 *     # Do the physical read
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5ARRAYread(self.dataset_id, self.type_id, start, nrows, step,
 *                           extdim, rbuf)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "tables/hdf5extension.pyx":1771
 *                           extdim, rbuf)
 * 
 *     try:             # <<<<<<<<<<<<<<
 *       if ret < 0:
 *         raise HDF5ExtError("Problems reading the array data.")
*/
  /*try:*/ {

    /* "tables/hdf5extension.pyx":1772
 * 
 *     try:
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Problems reading the array data.")
 * 
*/
    __pyx_t_3 = (__pyx_v_ret < 0);
    if (unlikely(__pyx_t_3)) {

      /* "tables/hdf5extension.pyx":1773
 *     try:
 *       if ret < 0:
 *         raise HDF5ExtError("Problems reading the array data.")             # <<<<<<<<<<<<<<
 * 
 *       # Get the pointer to the buffer data area
*/
      __pyx_t_1 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1773, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
        assert(__pyx_t_1);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_Problems_reading_the_array_data};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1773, __pyx_L9_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 1773, __pyx_L9_error)

      /* "tables/hdf5extension.pyx":1772
 * 
 *     try:
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Problems reading the array data.")
 * 
*/
    }

    /* "tables/hdf5extension.pyx":1776
 * 
 *       # Get the pointer to the buffer data area
 *       if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1776, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1776, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_reference, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1776, __pyx_L9_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (__pyx_t_3) {

      /* "tables/hdf5extension.pyx":1777
 *       # Get the pointer to the buffer data area
 *       if self.atom.kind == "reference":
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)             # <<<<<<<<<<<<<<
 *     finally:
 *       if refbuf:
*/
      __pyx_t_4 = __pyx_f_6tables_14utilsextension_load_reference(__pyx_v_self->__pyx_base.dataset_id, ((hobj_ref_t *)__pyx_v_rbuf), __pyx_v_item_size, __pyx_v_nparr); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 1777, __pyx_L9_error)

      /* "tables/hdf5extension.pyx":1776
 * 
 *       # Get the pointer to the buffer data area
 *       if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
*/
    }
  }

  /* "tables/hdf5extension.pyx":1779
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
 *       if refbuf:             # <<<<<<<<<<<<<<
 *         free(refbuf)
 *         refbuf = NULL
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_3 = (__pyx_v_refbuf != 0);
      if (__pyx_t_3) {

        /* "tables/hdf5extension.pyx":1780
 *     finally:
 *       if refbuf:
 *         free(refbuf)             # <<<<<<<<<<<<<<
 *         refbuf = NULL
 * 
*/
        free(__pyx_v_refbuf);

        /* "tables/hdf5extension.pyx":1781
 *       if refbuf:
 *         free(refbuf)
 *         refbuf = NULL             # <<<<<<<<<<<<<<
 * 
 *     if self.atom.kind == 'time':
*/
        __pyx_v_refbuf = NULL;

        /* "tables/hdf5extension.pyx":1779
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
 *       if refbuf:             # <<<<<<<<<<<<<<
 *         free(refbuf)
 *         refbuf = NULL
*/
      }
      goto __pyx_L10;
    }
    __pyx_L9_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14);
      if ( unlikely(__Pyx_GetException(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11) < 0)) __Pyx_ErrFetch(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      __pyx_t_4 = __pyx_lineno; __pyx_t_7 = __pyx_clineno; __pyx_t_8 = __pyx_filename;
      {
        __pyx_t_3 = (__pyx_v_refbuf != 0);
        if (__pyx_t_3) {

          /* "tables/hdf5extension.pyx":1780
 *     finally:
 *       if refbuf:
 *         free(refbuf)             # <<<<<<<<<<<<<<
 *         refbuf = NULL
 * 
*/
          free(__pyx_v_refbuf);

          /* "tables/hdf5extension.pyx":1781
 *       if refbuf:
 *         free(refbuf)
 *         refbuf = NULL             # <<<<<<<<<<<<<<
 * 
 *     if self.atom.kind == 'time':
*/
          __pyx_v_refbuf = NULL;

          /* "tables/hdf5extension.pyx":1779
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
 *       if refbuf:             # <<<<<<<<<<<<<<
 *         free(refbuf)
 *         refbuf = NULL
*/
        }
      }
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_13, __pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_XGIVEREF(__pyx_t_10);
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_ErrRestore(__pyx_t_9, __pyx_t_10, __pyx_t_11);
      __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0;
      __pyx_lineno = __pyx_t_4; __pyx_clineno = __pyx_t_7; __pyx_filename = __pyx_t_8;
      goto __pyx_L1_error;
    }
    __pyx_L10:;
  }

  /* "tables/hdf5extension.pyx":1783
 *         refbuf = NULL
 * 
 *     if self.atom.kind == 'time':             # <<<<<<<<<<<<<<
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:
*/
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1783, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1783, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_time, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1783, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":1785
 *     if self.atom.kind == 'time':
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:             # <<<<<<<<<<<<<<
 *         nparr.byteswap(True)
 * 
*/
    __pyx_t_2 = __Pyx_PyLong_From_enum__H5T_order_t(H5Tget_order(__pyx_v_self->__pyx_base.type_id)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1785, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_platform_byteorder); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1785, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyObject_RichCompare(__pyx_t_2, __pyx_t_5, Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1785, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1785, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_3) {

      /* "tables/hdf5extension.pyx":1786
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:
 *         nparr.byteswap(True)             # <<<<<<<<<<<<<<
 * 
 *     # Convert some HDF5 types to NumPy after reading.
*/
      __pyx_t_5 = ((PyObject *)__pyx_v_nparr);
      __Pyx_INCREF(__pyx_t_5);
      __pyx_t_6 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_5, Py_True};
        __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_byteswap, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1786, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "tables/hdf5extension.pyx":1785
 *     if self.atom.kind == 'time':
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:             # <<<<<<<<<<<<<<
 *         nparr.byteswap(True)
 * 
*/
    }

    /* "tables/hdf5extension.pyx":1783
 *         refbuf = NULL
 * 
 *     if self.atom.kind == 'time':             # <<<<<<<<<<<<<<
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:
*/
  }

  /* "tables/hdf5extension.pyx":1789
 * 
 *     # Convert some HDF5 types to NumPy after reading.
 *     if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *       self._convert_time64(nparr, 1)
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1789, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1789, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_time64, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1789, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":1790
 *     # Convert some HDF5 types to NumPy after reading.
 *     if self.atom.type == 'time64':
 *       self._convert_time64(nparr, 1)             # <<<<<<<<<<<<<<
 * 
 *     return
*/
    __pyx_t_5 = ((struct __pyx_vtabstruct_6tables_13hdf5extension_Array *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._convert_time64(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self), __pyx_v_nparr, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1790, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "tables/hdf5extension.pyx":1789
 * 
 *     # Convert some HDF5 types to NumPy after reading.
 *     if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *       self._convert_time64(nparr, 1)
 * 
*/
  }

  /* "tables/hdf5extension.pyx":1792
 *       self._convert_time64(nparr, 1)
 * 
 *     return             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1742
 *     self.shape = tuple(shape)
 * 
 *   def _read_array(self, hsize_t start, hsize_t stop, hsize_t step,             # <<<<<<<<<<<<<<
 *                  ndarray nparr):
 *     cdef herr_t ret
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("tables.hdf5extension.Array._read_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1795
 * 
 * 
 *   def _g_read_slice(self, ndarray startl, ndarray stopl, ndarray stepl,             # <<<<<<<<<<<<<<
 *                    ndarray nparr):
 *     cdef herr_t ret
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_11_g_read_slice(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Array_11_g_read_slice = {"_g_read_slice", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_11_g_read_slice, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_11_g_read_slice(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyArrayObject *__pyx_v_startl = 0;
  PyArrayObject *__pyx_v_stopl = 0;
  PyArrayObject *__pyx_v_stepl = 0;
  PyArrayObject *__pyx_v_nparr = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_read_slice (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_startl,&__pyx_mstate_global->__pyx_n_u_stopl,&__pyx_mstate_global->__pyx_n_u_stepl,&__pyx_mstate_global->__pyx_n_u_nparr,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1795, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1795, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1795, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1795, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1795, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_read_slice", 0) < (0)) __PYX_ERR(0, 1795, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 4; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_read_slice", 1, 4, 4, i); __PYX_ERR(0, 1795, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 4)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1795, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1795, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1795, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1795, __pyx_L3_error)
    }
    __pyx_v_startl = ((PyArrayObject *)values[0]);
    __pyx_v_stopl = ((PyArrayObject *)values[1]);
    __pyx_v_stepl = ((PyArrayObject *)values[2]);
    __pyx_v_nparr = ((PyArrayObject *)values[3]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_read_slice", 1, 4, 4, __pyx_nargs); __PYX_ERR(0, 1795, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Array._g_read_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_startl), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "startl", 0))) __PYX_ERR(0, 1795, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_stopl), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "stopl", 0))) __PYX_ERR(0, 1795, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_stepl), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "stepl", 0))) __PYX_ERR(0, 1795, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_nparr), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "nparr", 0))) __PYX_ERR(0, 1796, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Array_10_g_read_slice(((struct __pyx_obj_6tables_13hdf5extension_Array *)__pyx_v_self), __pyx_v_startl, __pyx_v_stopl, __pyx_v_stepl, __pyx_v_nparr);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_10_g_read_slice(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyArrayObject *__pyx_v_startl, PyArrayObject *__pyx_v_stopl, PyArrayObject *__pyx_v_stepl, PyArrayObject *__pyx_v_nparr) {
  herr_t __pyx_v_ret;
  hsize_t *__pyx_v_start;
  hsize_t *__pyx_v_stop;
  hsize_t *__pyx_v_step;
  void *__pyx_v_rbuf;
  size_t __pyx_v_item_size;
  void *__pyx_v_refbuf;
  PyObject *__pyx_v_fname = 0;
  char *__pyx_v_filename;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  char *__pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  char const *__pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_read_slice", 0);

  /* "tables/hdf5extension.pyx":1802
 *     cdef hsize_t *step
 *     cdef void *rbuf
 *     cdef size_t item_size = H5Tget_size(self.type_id)             # <<<<<<<<<<<<<<
 *     cdef void * refbuf = NULL
 * 
*/
  __pyx_v_item_size = H5Tget_size(__pyx_v_self->__pyx_base.type_id);

  /* "tables/hdf5extension.pyx":1803
 *     cdef void *rbuf
 *     cdef size_t item_size = H5Tget_size(self.type_id)
 *     cdef void * refbuf = NULL             # <<<<<<<<<<<<<<
 * 
 *     # Get the pointer to the buffer data area of startl, stopl and stepl arrays
*/
  __pyx_v_refbuf = NULL;

  /* "tables/hdf5extension.pyx":1806
 * 
 *     # Get the pointer to the buffer data area of startl, stopl and stepl arrays
 *     start = <hsize_t *>PyArray_DATA(startl)             # <<<<<<<<<<<<<<
 *     stop = <hsize_t *>PyArray_DATA(stopl)
 *     step = <hsize_t *>PyArray_DATA(stepl)
*/
  __pyx_v_start = ((hsize_t *)PyArray_DATA(__pyx_v_startl));

  /* "tables/hdf5extension.pyx":1807
 *     # Get the pointer to the buffer data area of startl, stopl and stepl arrays
 *     start = <hsize_t *>PyArray_DATA(startl)
 *     stop = <hsize_t *>PyArray_DATA(stopl)             # <<<<<<<<<<<<<<
 *     step = <hsize_t *>PyArray_DATA(stepl)
 * 
*/
  __pyx_v_stop = ((hsize_t *)PyArray_DATA(__pyx_v_stopl));

  /* "tables/hdf5extension.pyx":1808
 *     start = <hsize_t *>PyArray_DATA(startl)
 *     stop = <hsize_t *>PyArray_DATA(stopl)
 *     step = <hsize_t *>PyArray_DATA(stepl)             # <<<<<<<<<<<<<<
 * 
 *     # Get the pointer to the buffer data area
*/
  __pyx_v_step = ((hsize_t *)PyArray_DATA(__pyx_v_stepl));

  /* "tables/hdf5extension.pyx":1811
 * 
 *     # Get the pointer to the buffer data area
 *     if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *       refbuf = malloc(nparr.size * item_size)
 *       rbuf = refbuf
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1811, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1811, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_reference, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1811, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":1812
 *     # Get the pointer to the buffer data area
 *     if self.atom.kind == "reference":
 *       refbuf = malloc(nparr.size * item_size)             # <<<<<<<<<<<<<<
 *       rbuf = refbuf
 *     else:
*/
    __pyx_v_refbuf = malloc((__pyx_f_5numpy_7ndarray_4size_size(__pyx_v_nparr) * __pyx_v_item_size));

    /* "tables/hdf5extension.pyx":1813
 *     if self.atom.kind == "reference":
 *       refbuf = malloc(nparr.size * item_size)
 *       rbuf = refbuf             # <<<<<<<<<<<<<<
 *     else:
 *       rbuf = PyArray_DATA(nparr)
*/
    __pyx_v_rbuf = __pyx_v_refbuf;

    /* "tables/hdf5extension.pyx":1811
 * 
 *     # Get the pointer to the buffer data area
 *     if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *       refbuf = malloc(nparr.size * item_size)
 *       rbuf = refbuf
*/
    goto __pyx_L3;
  }

  /* "tables/hdf5extension.pyx":1815
 *       rbuf = refbuf
 *     else:
 *       rbuf = PyArray_DATA(nparr)             # <<<<<<<<<<<<<<
 * 
 *     cdef bytes fname = self._v_file.filename.encode('utf8')
*/
  /*else*/ {
    __pyx_v_rbuf = PyArray_DATA(__pyx_v_nparr);
  }
  __pyx_L3:;

  /* "tables/hdf5extension.pyx":1817
 *       rbuf = PyArray_DATA(nparr)
 * 
 *     cdef bytes fname = self._v_file.filename.encode('utf8')             # <<<<<<<<<<<<<<
 *     cdef char *filename = fname
 *     # Do the physical read
*/
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_file); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1817, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_filename); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1817, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_1 = __pyx_t_5;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_6 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_mstate_global->__pyx_n_u_utf8};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1817, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_2))) __PYX_ERR(0, 1817, __pyx_L1_error)
  __pyx_v_fname = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1818
 * 
 *     cdef bytes fname = self._v_file.filename.encode('utf8')
 *     cdef char *filename = fname             # <<<<<<<<<<<<<<
 *     # Do the physical read
 *     with nogil:
*/
  if (unlikely(__pyx_v_fname == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1818, __pyx_L1_error)
  }
  __pyx_t_7 = __Pyx_PyBytes_AsWritableString(__pyx_v_fname); if (unlikely((!__pyx_t_7) && PyErr_Occurred())) __PYX_ERR(0, 1818, __pyx_L1_error)
  __pyx_v_filename = __pyx_t_7;

  /* "tables/hdf5extension.pyx":1820
 *     cdef char *filename = fname
 *     # Do the physical read
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5ARRAYOreadSlice(filename, self.blosc2_support_read, self.dataset_id, self.type_id,
 *                                 start, stop, step, rbuf)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "tables/hdf5extension.pyx":1821
 *     # Do the physical read
 *     with nogil:
 *         ret = H5ARRAYOreadSlice(filename, self.blosc2_support_read, self.dataset_id, self.type_id,             # <<<<<<<<<<<<<<
 *                                 start, stop, step, rbuf)
 *     try:
*/
        __pyx_v_ret = H5ARRAYOreadSlice(__pyx_v_filename, __pyx_v_self->blosc2_support_read, __pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_start, __pyx_v_stop, __pyx_v_step, __pyx_v_rbuf);
      }

      /* "tables/hdf5extension.pyx":1820
 *     cdef char *filename = fname
 *     # Do the physical read
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5ARRAYOreadSlice(filename, self.blosc2_support_read, self.dataset_id, self.type_id,
 *                                 start, stop, step, rbuf)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "tables/hdf5extension.pyx":1823
 *         ret = H5ARRAYOreadSlice(filename, self.blosc2_support_read, self.dataset_id, self.type_id,
 *                                 start, stop, step, rbuf)
 *     try:             # <<<<<<<<<<<<<<
 *       if ret < 0:
 *         raise HDF5ExtError("Internal error reading the elements "
*/
  /*try:*/ {

    /* "tables/hdf5extension.pyx":1824
 *                                 start, stop, step, rbuf)
 *     try:
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Internal error reading the elements "
 *                            "(H5ARRAYOreadSlice returned errorcode %i)" % ret)
*/
    __pyx_t_3 = (__pyx_v_ret < 0);
    if (unlikely(__pyx_t_3)) {

      /* "tables/hdf5extension.pyx":1825
 *     try:
 *       if ret < 0:
 *         raise HDF5ExtError("Internal error reading the elements "             # <<<<<<<<<<<<<<
 *                            "(H5ARRAYOreadSlice returned errorcode %i)" % ret)
 * 
*/
      __pyx_t_5 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1825, __pyx_L8_error)
      __Pyx_GOTREF(__pyx_t_1);

      /* "tables/hdf5extension.pyx":1826
 *       if ret < 0:
 *         raise HDF5ExtError("Internal error reading the elements "
 *                            "(H5ARRAYOreadSlice returned errorcode %i)" % ret)             # <<<<<<<<<<<<<<
 * 
 *       # Get the pointer to the buffer data area
*/
      __pyx_t_4 = __Pyx_PyLong_From_herr_t(__pyx_v_ret); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1826, __pyx_L8_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_Internal_error_reading_the_eleme, __pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1826, __pyx_L8_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
        assert(__pyx_t_5);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_8};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1825, __pyx_L8_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 1825, __pyx_L8_error)

      /* "tables/hdf5extension.pyx":1824
 *                                 start, stop, step, rbuf)
 *     try:
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Internal error reading the elements "
 *                            "(H5ARRAYOreadSlice returned errorcode %i)" % ret)
*/
    }

    /* "tables/hdf5extension.pyx":1829
 * 
 *       # Get the pointer to the buffer data area
 *       if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1829, __pyx_L8_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1829, __pyx_L8_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_reference, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1829, __pyx_L8_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_3) {

      /* "tables/hdf5extension.pyx":1830
 *       # Get the pointer to the buffer data area
 *       if self.atom.kind == "reference":
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)             # <<<<<<<<<<<<<<
 *     finally:
 *       if refbuf:
*/
      __pyx_t_9 = __pyx_f_6tables_14utilsextension_load_reference(__pyx_v_self->__pyx_base.dataset_id, ((hobj_ref_t *)__pyx_v_rbuf), __pyx_v_item_size, __pyx_v_nparr); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 1830, __pyx_L8_error)

      /* "tables/hdf5extension.pyx":1829
 * 
 *       # Get the pointer to the buffer data area
 *       if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
*/
    }
  }

  /* "tables/hdf5extension.pyx":1832
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
 *       if refbuf:             # <<<<<<<<<<<<<<
 *         free(refbuf)
 *         refbuf = NULL
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_3 = (__pyx_v_refbuf != 0);
      if (__pyx_t_3) {

        /* "tables/hdf5extension.pyx":1833
 *     finally:
 *       if refbuf:
 *         free(refbuf)             # <<<<<<<<<<<<<<
 *         refbuf = NULL
 * 
*/
        free(__pyx_v_refbuf);

        /* "tables/hdf5extension.pyx":1834
 *       if refbuf:
 *         free(refbuf)
 *         refbuf = NULL             # <<<<<<<<<<<<<<
 * 
 *     if self.atom.kind == 'time':
*/
        __pyx_v_refbuf = NULL;

        /* "tables/hdf5extension.pyx":1832
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
 *       if refbuf:             # <<<<<<<<<<<<<<
 *         free(refbuf)
 *         refbuf = NULL
*/
      }
      goto __pyx_L9;
    }
    __pyx_L8_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17);
      if ( unlikely(__Pyx_GetException(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14) < 0)) __Pyx_ErrFetch(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __pyx_t_9 = __pyx_lineno; __pyx_t_10 = __pyx_clineno; __pyx_t_11 = __pyx_filename;
      {
        __pyx_t_3 = (__pyx_v_refbuf != 0);
        if (__pyx_t_3) {

          /* "tables/hdf5extension.pyx":1833
 *     finally:
 *       if refbuf:
 *         free(refbuf)             # <<<<<<<<<<<<<<
 *         refbuf = NULL
 * 
*/
          free(__pyx_v_refbuf);

          /* "tables/hdf5extension.pyx":1834
 *       if refbuf:
 *         free(refbuf)
 *         refbuf = NULL             # <<<<<<<<<<<<<<
 * 
 *     if self.atom.kind == 'time':
*/
          __pyx_v_refbuf = NULL;

          /* "tables/hdf5extension.pyx":1832
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
 *       if refbuf:             # <<<<<<<<<<<<<<
 *         free(refbuf)
 *         refbuf = NULL
*/
        }
      }
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_XGIVEREF(__pyx_t_17);
      __Pyx_ExceptionReset(__pyx_t_15, __pyx_t_16, __pyx_t_17);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_ErrRestore(__pyx_t_12, __pyx_t_13, __pyx_t_14);
      __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0;
      __pyx_lineno = __pyx_t_9; __pyx_clineno = __pyx_t_10; __pyx_filename = __pyx_t_11;
      goto __pyx_L1_error;
    }
    __pyx_L9:;
  }

  /* "tables/hdf5extension.pyx":1836
 *         refbuf = NULL
 * 
 *     if self.atom.kind == 'time':             # <<<<<<<<<<<<<<
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1836, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1836, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_time, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1836, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":1838
 *     if self.atom.kind == 'time':
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:             # <<<<<<<<<<<<<<
 *         nparr.byteswap(True)
 * 
*/
    __pyx_t_2 = __Pyx_PyLong_From_enum__H5T_order_t(H5Tget_order(__pyx_v_self->__pyx_base.type_id)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1838, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_platform_byteorder); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1838, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = PyObject_RichCompare(__pyx_t_2, __pyx_t_1, Py_NE); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1838, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1838, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_3) {

      /* "tables/hdf5extension.pyx":1839
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:
 *         nparr.byteswap(True)             # <<<<<<<<<<<<<<
 * 
 *     # Convert some HDF5 types to NumPy after reading
*/
      __pyx_t_1 = ((PyObject *)__pyx_v_nparr);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_6 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_1, Py_True};
        __pyx_t_8 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_byteswap, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1839, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "tables/hdf5extension.pyx":1838
 *     if self.atom.kind == 'time':
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:             # <<<<<<<<<<<<<<
 *         nparr.byteswap(True)
 * 
*/
    }

    /* "tables/hdf5extension.pyx":1836
 *         refbuf = NULL
 * 
 *     if self.atom.kind == 'time':             # <<<<<<<<<<<<<<
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:
*/
  }

  /* "tables/hdf5extension.pyx":1842
 * 
 *     # Convert some HDF5 types to NumPy after reading
 *     if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *       self._convert_time64(nparr, 1)
 * 
*/
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1842, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1842, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_time64, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1842, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":1843
 *     # Convert some HDF5 types to NumPy after reading
 *     if self.atom.type == 'time64':
 *       self._convert_time64(nparr, 1)             # <<<<<<<<<<<<<<
 * 
 *     return
*/
    __pyx_t_1 = ((struct __pyx_vtabstruct_6tables_13hdf5extension_Array *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._convert_time64(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self), __pyx_v_nparr, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1843, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":1842
 * 
 *     # Convert some HDF5 types to NumPy after reading
 *     if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *       self._convert_time64(nparr, 1)
 * 
*/
  }

  /* "tables/hdf5extension.pyx":1845
 *       self._convert_time64(nparr, 1)
 * 
 *     return             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1795
 * 
 * 
 *   def _g_read_slice(self, ndarray startl, ndarray stopl, ndarray stepl,             # <<<<<<<<<<<<<<
 *                    ndarray nparr):
 *     cdef herr_t ret
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("tables.hdf5extension.Array._g_read_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_fname);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1848
 * 
 * 
 *   def _g_read_coords(self, ndarray coords, ndarray nparr):             # <<<<<<<<<<<<<<
 *     """Read coordinates in an already created NumPy array."""
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_13_g_read_coords(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_5Array_12_g_read_coords, "Read coordinates in an already created NumPy array.");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Array_13_g_read_coords = {"_g_read_coords", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_13_g_read_coords, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Array_12_g_read_coords};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_13_g_read_coords(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyArrayObject *__pyx_v_coords = 0;
  PyArrayObject *__pyx_v_nparr = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_read_coords (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_coords,&__pyx_mstate_global->__pyx_n_u_nparr,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1848, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1848, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1848, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_read_coords", 0) < (0)) __PYX_ERR(0, 1848, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_read_coords", 1, 2, 2, i); __PYX_ERR(0, 1848, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1848, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1848, __pyx_L3_error)
    }
    __pyx_v_coords = ((PyArrayObject *)values[0]);
    __pyx_v_nparr = ((PyArrayObject *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_read_coords", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1848, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Array._g_read_coords", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_coords), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "coords", 0))) __PYX_ERR(0, 1848, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_nparr), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "nparr", 0))) __PYX_ERR(0, 1848, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Array_12_g_read_coords(((struct __pyx_obj_6tables_13hdf5extension_Array *)__pyx_v_self), __pyx_v_coords, __pyx_v_nparr);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_12_g_read_coords(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyArrayObject *__pyx_v_coords, PyArrayObject *__pyx_v_nparr) {
  herr_t __pyx_v_ret;
  hid_t __pyx_v_space_id;
  hid_t __pyx_v_mem_space_id;
  hsize_t __pyx_v_size;
  void *__pyx_v_rbuf;
  size_t __pyx_v_item_size;
  void *__pyx_v_refbuf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  char const *__pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_read_coords", 0);

  /* "tables/hdf5extension.pyx":1857
 *     cdef void *rbuf
 *     cdef object mode
 *     cdef size_t item_size = H5Tget_size(self.type_id)             # <<<<<<<<<<<<<<
 *     cdef void * refbuf = NULL
 * 
*/
  __pyx_v_item_size = H5Tget_size(__pyx_v_self->__pyx_base.type_id);

  /* "tables/hdf5extension.pyx":1858
 *     cdef object mode
 *     cdef size_t item_size = H5Tget_size(self.type_id)
 *     cdef void * refbuf = NULL             # <<<<<<<<<<<<<<
 * 
 *     # Get the dataspace handle
*/
  __pyx_v_refbuf = NULL;

  /* "tables/hdf5extension.pyx":1861
 * 
 *     # Get the dataspace handle
 *     space_id = H5Dget_space(self.dataset_id)             # <<<<<<<<<<<<<<
 *     # Create a memory dataspace handle
 *     size = nparr.size
*/
  __pyx_v_space_id = H5Dget_space(__pyx_v_self->__pyx_base.dataset_id);

  /* "tables/hdf5extension.pyx":1863
 *     space_id = H5Dget_space(self.dataset_id)
 *     # Create a memory dataspace handle
 *     size = nparr.size             # <<<<<<<<<<<<<<
 *     mem_space_id = H5Screate_simple(1, &size, NULL)
 * 
*/
  __pyx_v_size = __pyx_f_5numpy_7ndarray_4size_size(__pyx_v_nparr);

  /* "tables/hdf5extension.pyx":1864
 *     # Create a memory dataspace handle
 *     size = nparr.size
 *     mem_space_id = H5Screate_simple(1, &size, NULL)             # <<<<<<<<<<<<<<
 * 
 *     # Select the dataspace to be read
*/
  __pyx_v_mem_space_id = H5Screate_simple(1, (&__pyx_v_size), NULL);

  /* "tables/hdf5extension.pyx":1867
 * 
 *     # Select the dataspace to be read
 *     H5Sselect_elements(space_id, H5S_SELECT_SET,             # <<<<<<<<<<<<<<
 *                        <size_t>size, <hsize_t *>PyArray_DATA(coords))
 * 
*/
  (void)(H5Sselect_elements(__pyx_v_space_id, H5S_SELECT_SET, ((size_t)__pyx_v_size), ((hsize_t *)PyArray_DATA(__pyx_v_coords))));

  /* "tables/hdf5extension.pyx":1871
 * 
 *     # Get the pointer to the buffer data area
 *     if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *       refbuf = malloc(nparr.size * item_size)
 *       rbuf = refbuf
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1871, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1871, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_reference, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1871, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":1872
 *     # Get the pointer to the buffer data area
 *     if self.atom.kind == "reference":
 *       refbuf = malloc(nparr.size * item_size)             # <<<<<<<<<<<<<<
 *       rbuf = refbuf
 *     else:
*/
    __pyx_v_refbuf = malloc((__pyx_f_5numpy_7ndarray_4size_size(__pyx_v_nparr) * __pyx_v_item_size));

    /* "tables/hdf5extension.pyx":1873
 *     if self.atom.kind == "reference":
 *       refbuf = malloc(nparr.size * item_size)
 *       rbuf = refbuf             # <<<<<<<<<<<<<<
 *     else:
 *       rbuf = PyArray_DATA(nparr)
*/
    __pyx_v_rbuf = __pyx_v_refbuf;

    /* "tables/hdf5extension.pyx":1871
 * 
 *     # Get the pointer to the buffer data area
 *     if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *       refbuf = malloc(nparr.size * item_size)
 *       rbuf = refbuf
*/
    goto __pyx_L3;
  }

  /* "tables/hdf5extension.pyx":1875
 *       rbuf = refbuf
 *     else:
 *       rbuf = PyArray_DATA(nparr)             # <<<<<<<<<<<<<<
 * 
 *     # Do the actual read
*/
  /*else*/ {
    __pyx_v_rbuf = PyArray_DATA(__pyx_v_nparr);
  }
  __pyx_L3:;

  /* "tables/hdf5extension.pyx":1878
 * 
 *     # Do the actual read
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5Dread(self.dataset_id, self.type_id, mem_space_id, space_id,
 *                       H5P_DEFAULT, rbuf)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "tables/hdf5extension.pyx":1879
 *     # Do the actual read
 *     with nogil:
 *         ret = H5Dread(self.dataset_id, self.type_id, mem_space_id, space_id,             # <<<<<<<<<<<<<<
 *                       H5P_DEFAULT, rbuf)
 * 
*/
        __pyx_v_ret = H5Dread(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_mem_space_id, __pyx_v_space_id, H5P_DEFAULT, __pyx_v_rbuf);
      }

      /* "tables/hdf5extension.pyx":1878
 * 
 *     # Do the actual read
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5Dread(self.dataset_id, self.type_id, mem_space_id, space_id,
 *                       H5P_DEFAULT, rbuf)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "tables/hdf5extension.pyx":1882
 *                       H5P_DEFAULT, rbuf)
 * 
 *     try:             # <<<<<<<<<<<<<<
 *       if ret < 0:
 *         raise HDF5ExtError("Problems reading the array data.")
*/
  /*try:*/ {

    /* "tables/hdf5extension.pyx":1883
 * 
 *     try:
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Problems reading the array data.")
 * 
*/
    __pyx_t_3 = (__pyx_v_ret < 0);
    if (unlikely(__pyx_t_3)) {

      /* "tables/hdf5extension.pyx":1884
 *     try:
 *       if ret < 0:
 *         raise HDF5ExtError("Problems reading the array data.")             # <<<<<<<<<<<<<<
 * 
 *       # Get the pointer to the buffer data area
*/
      __pyx_t_1 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1884, __pyx_L8_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
        assert(__pyx_t_1);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
        __pyx_t_5 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_Problems_reading_the_array_data};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1884, __pyx_L8_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 1884, __pyx_L8_error)

      /* "tables/hdf5extension.pyx":1883
 * 
 *     try:
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Problems reading the array data.")
 * 
*/
    }

    /* "tables/hdf5extension.pyx":1887
 * 
 *       # Get the pointer to the buffer data area
 *       if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1887, __pyx_L8_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1887, __pyx_L8_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_reference, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1887, __pyx_L8_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__pyx_t_3) {

      /* "tables/hdf5extension.pyx":1888
 *       # Get the pointer to the buffer data area
 *       if self.atom.kind == "reference":
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)             # <<<<<<<<<<<<<<
 *     finally:
 *       if refbuf:
*/
      __pyx_t_6 = __pyx_f_6tables_14utilsextension_load_reference(__pyx_v_self->__pyx_base.dataset_id, ((hobj_ref_t *)__pyx_v_rbuf), __pyx_v_item_size, __pyx_v_nparr); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 1888, __pyx_L8_error)

      /* "tables/hdf5extension.pyx":1887
 * 
 *       # Get the pointer to the buffer data area
 *       if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
*/
    }
  }

  /* "tables/hdf5extension.pyx":1890
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
 *       if refbuf:             # <<<<<<<<<<<<<<
 *         free(refbuf)
 *         refbuf = NULL
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_3 = (__pyx_v_refbuf != 0);
      if (__pyx_t_3) {

        /* "tables/hdf5extension.pyx":1891
 *     finally:
 *       if refbuf:
 *         free(refbuf)             # <<<<<<<<<<<<<<
 *         refbuf = NULL
 * 
*/
        free(__pyx_v_refbuf);

        /* "tables/hdf5extension.pyx":1892
 *       if refbuf:
 *         free(refbuf)
 *         refbuf = NULL             # <<<<<<<<<<<<<<
 * 
 *     # Terminate access to the memory dataspace
*/
        __pyx_v_refbuf = NULL;

        /* "tables/hdf5extension.pyx":1890
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
 *       if refbuf:             # <<<<<<<<<<<<<<
 *         free(refbuf)
 *         refbuf = NULL
*/
      }
      goto __pyx_L9;
    }
    __pyx_L8_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14);
      if ( unlikely(__Pyx_GetException(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11) < 0)) __Pyx_ErrFetch(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      __pyx_t_6 = __pyx_lineno; __pyx_t_7 = __pyx_clineno; __pyx_t_8 = __pyx_filename;
      {
        __pyx_t_3 = (__pyx_v_refbuf != 0);
        if (__pyx_t_3) {

          /* "tables/hdf5extension.pyx":1891
 *     finally:
 *       if refbuf:
 *         free(refbuf)             # <<<<<<<<<<<<<<
 *         refbuf = NULL
 * 
*/
          free(__pyx_v_refbuf);

          /* "tables/hdf5extension.pyx":1892
 *       if refbuf:
 *         free(refbuf)
 *         refbuf = NULL             # <<<<<<<<<<<<<<
 * 
 *     # Terminate access to the memory dataspace
*/
          __pyx_v_refbuf = NULL;

          /* "tables/hdf5extension.pyx":1890
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
 *       if refbuf:             # <<<<<<<<<<<<<<
 *         free(refbuf)
 *         refbuf = NULL
*/
        }
      }
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_13, __pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_XGIVEREF(__pyx_t_10);
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_ErrRestore(__pyx_t_9, __pyx_t_10, __pyx_t_11);
      __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0;
      __pyx_lineno = __pyx_t_6; __pyx_clineno = __pyx_t_7; __pyx_filename = __pyx_t_8;
      goto __pyx_L1_error;
    }
    __pyx_L9:;
  }

  /* "tables/hdf5extension.pyx":1895
 * 
 *     # Terminate access to the memory dataspace
 *     H5Sclose(mem_space_id)             # <<<<<<<<<<<<<<
 *     # Terminate access to the dataspace
 *     H5Sclose(space_id)
*/
  (void)(H5Sclose(__pyx_v_mem_space_id));

  /* "tables/hdf5extension.pyx":1897
 *     H5Sclose(mem_space_id)
 *     # Terminate access to the dataspace
 *     H5Sclose(space_id)             # <<<<<<<<<<<<<<
 * 
 *     if self.atom.kind == 'time':
*/
  (void)(H5Sclose(__pyx_v_space_id));

  /* "tables/hdf5extension.pyx":1899
 *     H5Sclose(space_id)
 * 
 *     if self.atom.kind == 'time':             # <<<<<<<<<<<<<<
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:
*/
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1899, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1899, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_time, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1899, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":1901
 *     if self.atom.kind == 'time':
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:             # <<<<<<<<<<<<<<
 *         nparr.byteswap(True)
 * 
*/
    __pyx_t_2 = __Pyx_PyLong_From_enum__H5T_order_t(H5Tget_order(__pyx_v_self->__pyx_base.type_id)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1901, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_platform_byteorder); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1901, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyObject_RichCompare(__pyx_t_2, __pyx_t_4, Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1901, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1901, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_3) {

      /* "tables/hdf5extension.pyx":1902
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:
 *         nparr.byteswap(True)             # <<<<<<<<<<<<<<
 * 
 *     # Convert some HDF5 types to NumPy after reading
*/
      __pyx_t_4 = ((PyObject *)__pyx_v_nparr);
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_5 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, Py_True};
        __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_byteswap, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1902, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "tables/hdf5extension.pyx":1901
 *     if self.atom.kind == 'time':
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:             # <<<<<<<<<<<<<<
 *         nparr.byteswap(True)
 * 
*/
    }

    /* "tables/hdf5extension.pyx":1899
 *     H5Sclose(space_id)
 * 
 *     if self.atom.kind == 'time':             # <<<<<<<<<<<<<<
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:
*/
  }

  /* "tables/hdf5extension.pyx":1905
 * 
 *     # Convert some HDF5 types to NumPy after reading
 *     if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *       self._convert_time64(nparr, 1)
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1905, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1905, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_time64, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1905, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":1906
 *     # Convert some HDF5 types to NumPy after reading
 *     if self.atom.type == 'time64':
 *       self._convert_time64(nparr, 1)             # <<<<<<<<<<<<<<
 * 
 *     return
*/
    __pyx_t_4 = ((struct __pyx_vtabstruct_6tables_13hdf5extension_Array *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._convert_time64(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self), __pyx_v_nparr, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1906, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "tables/hdf5extension.pyx":1905
 * 
 *     # Convert some HDF5 types to NumPy after reading
 *     if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *       self._convert_time64(nparr, 1)
 * 
*/
  }

  /* "tables/hdf5extension.pyx":1908
 *       self._convert_time64(nparr, 1)
 * 
 *     return             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1848
 * 
 * 
 *   def _g_read_coords(self, ndarray coords, ndarray nparr):             # <<<<<<<<<<<<<<
 *     """Read coordinates in an already created NumPy array."""
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("tables.hdf5extension.Array._g_read_coords", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1911
 * 
 * 
 *   def perform_selection(self, space_id, start, count, step, idx, mode):             # <<<<<<<<<<<<<<
 *     """Performs a selection using start/count/step in the given axis.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_15perform_selection(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_5Array_14perform_selection, "Performs a selection using start/count/step in the given axis.\n\n    All other axes have their full range selected.  The selection is\n    added to the current `space_id` selection using the given mode.\n\n    Note: This is a backport from the h5py project.\n\n    ");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Array_15perform_selection = {"perform_selection", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_15perform_selection, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Array_14perform_selection};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_15perform_selection(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_space_id = 0;
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_count = 0;
  PyObject *__pyx_v_step = 0;
  PyObject *__pyx_v_idx = 0;
  PyObject *__pyx_v_mode = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[6] = {0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("perform_selection (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_space_id,&__pyx_mstate_global->__pyx_n_u_start,&__pyx_mstate_global->__pyx_n_u_count,&__pyx_mstate_global->__pyx_n_u_step,&__pyx_mstate_global->__pyx_n_u_idx,&__pyx_mstate_global->__pyx_n_u_mode,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1911, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  6:
        values[5] = __Pyx_ArgRef_FASTCALL(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 1911, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 1911, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1911, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1911, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1911, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1911, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "perform_selection", 0) < (0)) __PYX_ERR(0, 1911, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 6; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("perform_selection", 1, 6, 6, i); __PYX_ERR(0, 1911, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 6)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1911, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1911, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1911, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1911, __pyx_L3_error)
      values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 1911, __pyx_L3_error)
      values[5] = __Pyx_ArgRef_FASTCALL(__pyx_args, 5);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 1911, __pyx_L3_error)
    }
    __pyx_v_space_id = values[0];
    __pyx_v_start = values[1];
    __pyx_v_count = values[2];
    __pyx_v_step = values[3];
    __pyx_v_idx = values[4];
    __pyx_v_mode = values[5];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("perform_selection", 1, 6, 6, __pyx_nargs); __PYX_ERR(0, 1911, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Array.perform_selection", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Array_14perform_selection(((struct __pyx_obj_6tables_13hdf5extension_Array *)__pyx_v_self), __pyx_v_space_id, __pyx_v_start, __pyx_v_count, __pyx_v_step, __pyx_v_idx, __pyx_v_mode);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_14perform_selection(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyObject *__pyx_v_space_id, PyObject *__pyx_v_start, PyObject *__pyx_v_count, PyObject *__pyx_v_step, PyObject *__pyx_v_idx, PyObject *__pyx_v_mode) {
  int __pyx_v_select_mode;
  PyArrayObject *__pyx_v_start_ = 0;
  PyArrayObject *__pyx_v_count_ = 0;
  PyArrayObject *__pyx_v_step_ = 0;
  hsize_t *__pyx_v_startp;
  hsize_t *__pyx_v_countp;
  hsize_t *__pyx_v_stepp;
  PyObject *__pyx_v_startl = NULL;
  PyObject *__pyx_v_countl = NULL;
  PyObject *__pyx_v_stepl = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_x = NULL;
  PyObject *__pyx_v_select_modes = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  int __pyx_t_10;
  hid_t __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("perform_selection", 0);

  /* "tables/hdf5extension.pyx":1928
 * 
 *     # Build arrays for the selection parameters
 *     startl, countl, stepl = [], [], []             # <<<<<<<<<<<<<<
 *     for i, x in enumerate(self.shape):
 *       if i != idx:
*/
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1928, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1928, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1928, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_startl = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_v_countl = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_v_stepl = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "tables/hdf5extension.pyx":1929
 *     # Build arrays for the selection parameters
 *     startl, countl, stepl = [], [], []
 *     for i, x in enumerate(self.shape):             # <<<<<<<<<<<<<<
 *       if i != idx:
 *         startl.append(0)
*/
  __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
  __pyx_t_3 = __pyx_mstate_global->__pyx_int_0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1929, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
    __pyx_t_1 = __pyx_t_2; __Pyx_INCREF(__pyx_t_1);
    __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1929, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1929, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1929, __pyx_L1_error)
          #endif
          if (__pyx_t_4 >= __pyx_temp) break;
        }
        __pyx_t_2 = __Pyx_PyList_GetItemRefFast(__pyx_t_1, __pyx_t_4, __Pyx_ReferenceSharing_OwnStrongReference);
        ++__pyx_t_4;
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1929, __pyx_L1_error)
          #endif
          if (__pyx_t_4 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_4));
        #else
        __pyx_t_2 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_4);
        #endif
        ++__pyx_t_4;
      }
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1929, __pyx_L1_error)
    } else {
      __pyx_t_2 = __pyx_t_5(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 1929, __pyx_L1_error)
          PyErr_Clear();
        }
        break;
      }
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_x, __pyx_t_2);
    __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_3);
    __pyx_t_2 = __Pyx_PyLong_AddObjC(__pyx_t_3, __pyx_mstate_global->__pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1929, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3);
    __pyx_t_3 = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "tables/hdf5extension.pyx":1930
 *     startl, countl, stepl = [], [], []
 *     for i, x in enumerate(self.shape):
 *       if i != idx:             # <<<<<<<<<<<<<<
 *         startl.append(0)
 *         countl.append(x)
*/
    __pyx_t_2 = PyObject_RichCompare(__pyx_v_i, __pyx_v_idx, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1930, __pyx_L1_error)
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1930, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_6) {

      /* "tables/hdf5extension.pyx":1931
 *     for i, x in enumerate(self.shape):
 *       if i != idx:
 *         startl.append(0)             # <<<<<<<<<<<<<<
 *         countl.append(x)
 *         stepl.append(1)
*/
      __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_startl, __pyx_mstate_global->__pyx_int_0); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1931, __pyx_L1_error)

      /* "tables/hdf5extension.pyx":1932
 *       if i != idx:
 *         startl.append(0)
 *         countl.append(x)             # <<<<<<<<<<<<<<
 *         stepl.append(1)
 *       else:
*/
      __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_countl, __pyx_v_x); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1932, __pyx_L1_error)

      /* "tables/hdf5extension.pyx":1933
 *         startl.append(0)
 *         countl.append(x)
 *         stepl.append(1)             # <<<<<<<<<<<<<<
 *       else:
 *         startl.append(start)
*/
      __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_stepl, __pyx_mstate_global->__pyx_int_1); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1933, __pyx_L1_error)

      /* "tables/hdf5extension.pyx":1930
 *     startl, countl, stepl = [], [], []
 *     for i, x in enumerate(self.shape):
 *       if i != idx:             # <<<<<<<<<<<<<<
 *         startl.append(0)
 *         countl.append(x)
*/
      goto __pyx_L5;
    }

    /* "tables/hdf5extension.pyx":1935
 *         stepl.append(1)
 *       else:
 *         startl.append(start)             # <<<<<<<<<<<<<<
 *         countl.append(count)
 *         stepl.append(step)
*/
    /*else*/ {
      __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_startl, __pyx_v_start); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1935, __pyx_L1_error)

      /* "tables/hdf5extension.pyx":1936
 *       else:
 *         startl.append(start)
 *         countl.append(count)             # <<<<<<<<<<<<<<
 *         stepl.append(step)
 *     start_ = np.array(startl, dtype="i8")
*/
      __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_countl, __pyx_v_count); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1936, __pyx_L1_error)

      /* "tables/hdf5extension.pyx":1937
 *         startl.append(start)
 *         countl.append(count)
 *         stepl.append(step)             # <<<<<<<<<<<<<<
 *     start_ = np.array(startl, dtype="i8")
 *     count_ = np.array(countl, dtype="i8")
*/
      __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_stepl, __pyx_v_step); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1937, __pyx_L1_error)
    }
    __pyx_L5:;

    /* "tables/hdf5extension.pyx":1929
 *     # Build arrays for the selection parameters
 *     startl, countl, stepl = [], [], []
 *     for i, x in enumerate(self.shape):             # <<<<<<<<<<<<<<
 *       if i != idx:
 *         startl.append(0)
*/
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "tables/hdf5extension.pyx":1938
 *         countl.append(count)
 *         stepl.append(step)
 *     start_ = np.array(startl, dtype="i8")             # <<<<<<<<<<<<<<
 *     count_ = np.array(countl, dtype="i8")
 *     step_ = np.array(stepl, dtype="i8")
*/
  __pyx_t_1 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1938, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_array); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1938, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_9 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_8);
    assert(__pyx_t_1);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
    __pyx_t_9 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, __pyx_v_startl};
    __pyx_t_2 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1938, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_mstate_global->__pyx_n_u_i8, __pyx_t_2, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 1938, __pyx_L1_error)
    __pyx_t_3 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_2);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1938, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1938, __pyx_L1_error)
  __pyx_v_start_ = ((PyArrayObject *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "tables/hdf5extension.pyx":1939
 *         stepl.append(step)
 *     start_ = np.array(startl, dtype="i8")
 *     count_ = np.array(countl, dtype="i8")             # <<<<<<<<<<<<<<
 *     step_ = np.array(stepl, dtype="i8")
 * 
*/
  __pyx_t_8 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1939, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_array); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1939, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_9 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_1);
    assert(__pyx_t_8);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_8);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
    __pyx_t_9 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_8, __pyx_v_countl};
    __pyx_t_2 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1939, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_mstate_global->__pyx_n_u_i8, __pyx_t_2, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 1939, __pyx_L1_error)
    __pyx_t_3 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_2);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1939, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1939, __pyx_L1_error)
  __pyx_v_count_ = ((PyArrayObject *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "tables/hdf5extension.pyx":1940
 *     start_ = np.array(startl, dtype="i8")
 *     count_ = np.array(countl, dtype="i8")
 *     step_ = np.array(stepl, dtype="i8")             # <<<<<<<<<<<<<<
 * 
 *     # Get the pointers to array data
*/
  __pyx_t_1 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1940, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_array); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1940, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_9 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_8);
    assert(__pyx_t_1);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
    __pyx_t_9 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, __pyx_v_stepl};
    __pyx_t_2 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1940, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_mstate_global->__pyx_n_u_i8, __pyx_t_2, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 1940, __pyx_L1_error)
    __pyx_t_3 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_2);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1940, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1940, __pyx_L1_error)
  __pyx_v_step_ = ((PyArrayObject *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "tables/hdf5extension.pyx":1943
 * 
 *     # Get the pointers to array data
 *     startp = <hsize_t *>PyArray_DATA(start_)             # <<<<<<<<<<<<<<
 *     countp = <hsize_t *>PyArray_DATA(count_)
 *     stepp = <hsize_t *>PyArray_DATA(step_)
*/
  __pyx_v_startp = ((hsize_t *)PyArray_DATA(__pyx_v_start_));

  /* "tables/hdf5extension.pyx":1944
 *     # Get the pointers to array data
 *     startp = <hsize_t *>PyArray_DATA(start_)
 *     countp = <hsize_t *>PyArray_DATA(count_)             # <<<<<<<<<<<<<<
 *     stepp = <hsize_t *>PyArray_DATA(step_)
 * 
*/
  __pyx_v_countp = ((hsize_t *)PyArray_DATA(__pyx_v_count_));

  /* "tables/hdf5extension.pyx":1945
 *     startp = <hsize_t *>PyArray_DATA(start_)
 *     countp = <hsize_t *>PyArray_DATA(count_)
 *     stepp = <hsize_t *>PyArray_DATA(step_)             # <<<<<<<<<<<<<<
 * 
 *     # Do the actual selection
*/
  __pyx_v_stepp = ((hsize_t *)PyArray_DATA(__pyx_v_step_));

  /* "tables/hdf5extension.pyx":1948
 * 
 *     # Do the actual selection
 *     select_modes = {"AND": H5S_SELECT_AND, "NOTB": H5S_SELECT_NOTB}             # <<<<<<<<<<<<<<
 *     assert mode in select_modes
 *     select_mode = select_modes[mode]
*/
  __pyx_t_3 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1948, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_8 = __Pyx_PyLong_From_enum__H5S_seloper_t(H5S_SELECT_AND); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1948, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_AND, __pyx_t_8) < (0)) __PYX_ERR(0, 1948, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyLong_From_enum__H5S_seloper_t(H5S_SELECT_NOTB); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1948, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_NOTB, __pyx_t_8) < (0)) __PYX_ERR(0, 1948, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_select_modes = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "tables/hdf5extension.pyx":1949
 *     # Do the actual selection
 *     select_modes = {"AND": H5S_SELECT_AND, "NOTB": H5S_SELECT_NOTB}
 *     assert mode in select_modes             # <<<<<<<<<<<<<<
 *     select_mode = select_modes[mode]
 *     H5Sselect_hyperslab(space_id, <H5S_seloper_t>select_mode,
*/
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(__pyx_assertions_enabled())) {
    __pyx_t_6 = (__Pyx_PyDict_ContainsTF(__pyx_v_mode, __pyx_v_select_modes, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1949, __pyx_L1_error)
    if (unlikely(!__pyx_t_6)) {
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
      __PYX_ERR(0, 1949, __pyx_L1_error)
    }
  }
  #else
  if ((1)); else __PYX_ERR(0, 1949, __pyx_L1_error)
  #endif

  /* "tables/hdf5extension.pyx":1950
 *     select_modes = {"AND": H5S_SELECT_AND, "NOTB": H5S_SELECT_NOTB}
 *     assert mode in select_modes
 *     select_mode = select_modes[mode]             # <<<<<<<<<<<<<<
 *     H5Sselect_hyperslab(space_id, <H5S_seloper_t>select_mode,
 *                         startp, stepp, countp, NULL)
*/
  __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_select_modes, __pyx_v_mode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1950, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_10 = __Pyx_PyLong_As_int(__pyx_t_3); if (unlikely((__pyx_t_10 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1950, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_select_mode = __pyx_t_10;

  /* "tables/hdf5extension.pyx":1951
 *     assert mode in select_modes
 *     select_mode = select_modes[mode]
 *     H5Sselect_hyperslab(space_id, <H5S_seloper_t>select_mode,             # <<<<<<<<<<<<<<
 *                         startp, stepp, countp, NULL)
 * 
*/
  __pyx_t_11 = __Pyx_PyLong_As_hid_t(__pyx_v_space_id); if (unlikely((__pyx_t_11 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1951, __pyx_L1_error)

  /* "tables/hdf5extension.pyx":1952
 *     select_mode = select_modes[mode]
 *     H5Sselect_hyperslab(space_id, <H5S_seloper_t>select_mode,
 *                         startp, stepp, countp, NULL)             # <<<<<<<<<<<<<<
 * 
 *   def _g_read_selection(self, object selection, ndarray nparr):
*/
  (void)(H5Sselect_hyperslab(__pyx_t_11, ((enum H5S_seloper_t)__pyx_v_select_mode), __pyx_v_startp, __pyx_v_stepp, __pyx_v_countp, NULL));

  /* "tables/hdf5extension.pyx":1911
 * 
 * 
 *   def perform_selection(self, space_id, start, count, step, idx, mode):             # <<<<<<<<<<<<<<
 *     """Performs a selection using start/count/step in the given axis.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("tables.hdf5extension.Array.perform_selection", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_start_);
  __Pyx_XDECREF((PyObject *)__pyx_v_count_);
  __Pyx_XDECREF((PyObject *)__pyx_v_step_);
  __Pyx_XDECREF(__pyx_v_startl);
  __Pyx_XDECREF(__pyx_v_countl);
  __Pyx_XDECREF(__pyx_v_stepl);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_x);
  __Pyx_XDECREF(__pyx_v_select_modes);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":1954
 *                         startp, stepp, countp, NULL)
 * 
 *   def _g_read_selection(self, object selection, ndarray nparr):             # <<<<<<<<<<<<<<
 *     """Read a selection in an already created NumPy array."""
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_17_g_read_selection(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_5Array_16_g_read_selection, "Read a selection in an already created NumPy array.");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Array_17_g_read_selection = {"_g_read_selection", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_17_g_read_selection, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Array_16_g_read_selection};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_17_g_read_selection(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_selection = 0;
  PyArrayObject *__pyx_v_nparr = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_read_selection (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_selection,&__pyx_mstate_global->__pyx_n_u_nparr,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1954, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1954, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1954, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_read_selection", 0) < (0)) __PYX_ERR(0, 1954, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_read_selection", 1, 2, 2, i); __PYX_ERR(0, 1954, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1954, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1954, __pyx_L3_error)
    }
    __pyx_v_selection = values[0];
    __pyx_v_nparr = ((PyArrayObject *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_read_selection", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1954, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Array._g_read_selection", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_nparr), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "nparr", 0))) __PYX_ERR(0, 1954, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Array_16_g_read_selection(((struct __pyx_obj_6tables_13hdf5extension_Array *)__pyx_v_self), __pyx_v_selection, __pyx_v_nparr);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_16_g_read_selection(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyObject *__pyx_v_selection, PyArrayObject *__pyx_v_nparr) {
  herr_t __pyx_v_ret;
  hid_t __pyx_v_space_id;
  hid_t __pyx_v_mem_space_id;
  hsize_t __pyx_v_size;
  void *__pyx_v_rbuf;
  size_t __pyx_v_item_size;
  void *__pyx_v_refbuf;
  PyObject *__pyx_v_args = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  size_t __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  char const *__pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_read_selection", 0);

  /* "tables/hdf5extension.pyx":1963
 *     cdef void *rbuf
 *     cdef object mode
 *     cdef size_t item_size = H5Tget_size(self.type_id)             # <<<<<<<<<<<<<<
 *     cdef void * refbuf = NULL
 * 
*/
  __pyx_v_item_size = H5Tget_size(__pyx_v_self->__pyx_base.type_id);

  /* "tables/hdf5extension.pyx":1964
 *     cdef object mode
 *     cdef size_t item_size = H5Tget_size(self.type_id)
 *     cdef void * refbuf = NULL             # <<<<<<<<<<<<<<
 * 
 *     # Get the dataspace handle
*/
  __pyx_v_refbuf = NULL;

  /* "tables/hdf5extension.pyx":1967
 * 
 *     # Get the dataspace handle
 *     space_id = H5Dget_space(self.dataset_id)             # <<<<<<<<<<<<<<
 *     # Create a memory dataspace handle
 *     size = nparr.size
*/
  __pyx_v_space_id = H5Dget_space(__pyx_v_self->__pyx_base.dataset_id);

  /* "tables/hdf5extension.pyx":1969
 *     space_id = H5Dget_space(self.dataset_id)
 *     # Create a memory dataspace handle
 *     size = nparr.size             # <<<<<<<<<<<<<<
 *     mem_space_id = H5Screate_simple(1, &size, NULL)
 * 
*/
  __pyx_v_size = __pyx_f_5numpy_7ndarray_4size_size(__pyx_v_nparr);

  /* "tables/hdf5extension.pyx":1970
 *     # Create a memory dataspace handle
 *     size = nparr.size
 *     mem_space_id = H5Screate_simple(1, &size, NULL)             # <<<<<<<<<<<<<<
 * 
 *     # Select the dataspace to be read
*/
  __pyx_v_mem_space_id = H5Screate_simple(1, (&__pyx_v_size), NULL);

  /* "tables/hdf5extension.pyx":1974
 *     # Select the dataspace to be read
 *     # Start by selecting everything
 *     H5Sselect_all(space_id)             # <<<<<<<<<<<<<<
 *     # Now refine with outstanding selections
 *     for args in selection:
*/
  (void)(H5Sselect_all(__pyx_v_space_id));

  /* "tables/hdf5extension.pyx":1976
 *     H5Sselect_all(space_id)
 *     # Now refine with outstanding selections
 *     for args in selection:             # <<<<<<<<<<<<<<
 *       self.perform_selection(space_id, *args)
 * 
*/
  if (likely(PyList_CheckExact(__pyx_v_selection)) || PyTuple_CheckExact(__pyx_v_selection)) {
    __pyx_t_1 = __pyx_v_selection; __Pyx_INCREF(__pyx_t_1);
    __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_selection); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1976, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1976, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1976, __pyx_L1_error)
          #endif
          if (__pyx_t_2 >= __pyx_temp) break;
        }
        __pyx_t_4 = __Pyx_PyList_GetItemRefFast(__pyx_t_1, __pyx_t_2, __Pyx_ReferenceSharing_OwnStrongReference);
        ++__pyx_t_2;
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1976, __pyx_L1_error)
          #endif
          if (__pyx_t_2 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2));
        #else
        __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2);
        #endif
        ++__pyx_t_2;
      }
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1976, __pyx_L1_error)
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 1976, __pyx_L1_error)
          PyErr_Clear();
        }
        break;
      }
    }
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_XDECREF_SET(__pyx_v_args, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "tables/hdf5extension.pyx":1977
 *     # Now refine with outstanding selections
 *     for args in selection:
 *       self.perform_selection(space_id, *args)             # <<<<<<<<<<<<<<
 * 
 *     # Get the pointer to the buffer data area
*/
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_perform_selection); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1977, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyLong_From_hid_t(__pyx_v_space_id); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1977, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1977, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5) != (0)) __PYX_ERR(0, 1977, __pyx_L1_error);
    __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PySequence_Tuple(__pyx_v_args); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1977, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = PyNumber_Add(__pyx_t_6, __pyx_t_5); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1977, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1977, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "tables/hdf5extension.pyx":1976
 *     H5Sselect_all(space_id)
 *     # Now refine with outstanding selections
 *     for args in selection:             # <<<<<<<<<<<<<<
 *       self.perform_selection(space_id, *args)
 * 
*/
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":1980
 * 
 *     # Get the pointer to the buffer data area
 *     if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *       refbuf = malloc(nparr.size * item_size)
 *       rbuf = refbuf
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1980, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1980, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_reference, Py_EQ)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 1980, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__pyx_t_8) {

    /* "tables/hdf5extension.pyx":1981
 *     # Get the pointer to the buffer data area
 *     if self.atom.kind == "reference":
 *       refbuf = malloc(nparr.size * item_size)             # <<<<<<<<<<<<<<
 *       rbuf = refbuf
 *     else:
*/
    __pyx_v_refbuf = malloc((__pyx_f_5numpy_7ndarray_4size_size(__pyx_v_nparr) * __pyx_v_item_size));

    /* "tables/hdf5extension.pyx":1982
 *     if self.atom.kind == "reference":
 *       refbuf = malloc(nparr.size * item_size)
 *       rbuf = refbuf             # <<<<<<<<<<<<<<
 *     else:
 *       rbuf = PyArray_DATA(nparr)
*/
    __pyx_v_rbuf = __pyx_v_refbuf;

    /* "tables/hdf5extension.pyx":1980
 * 
 *     # Get the pointer to the buffer data area
 *     if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *       refbuf = malloc(nparr.size * item_size)
 *       rbuf = refbuf
*/
    goto __pyx_L6;
  }

  /* "tables/hdf5extension.pyx":1984
 *       rbuf = refbuf
 *     else:
 *       rbuf = PyArray_DATA(nparr)             # <<<<<<<<<<<<<<
 * 
 *     # Do the actual read
*/
  /*else*/ {
    __pyx_v_rbuf = PyArray_DATA(__pyx_v_nparr);
  }
  __pyx_L6:;

  /* "tables/hdf5extension.pyx":1987
 * 
 *     # Do the actual read
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5Dread(self.dataset_id, self.type_id, mem_space_id, space_id,
 *                       H5P_DEFAULT, rbuf)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "tables/hdf5extension.pyx":1988
 *     # Do the actual read
 *     with nogil:
 *         ret = H5Dread(self.dataset_id, self.type_id, mem_space_id, space_id,             # <<<<<<<<<<<<<<
 *                       H5P_DEFAULT, rbuf)
 * 
*/
        __pyx_v_ret = H5Dread(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_mem_space_id, __pyx_v_space_id, H5P_DEFAULT, __pyx_v_rbuf);
      }

      /* "tables/hdf5extension.pyx":1987
 * 
 *     # Do the actual read
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5Dread(self.dataset_id, self.type_id, mem_space_id, space_id,
 *                       H5P_DEFAULT, rbuf)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "tables/hdf5extension.pyx":1991
 *                       H5P_DEFAULT, rbuf)
 * 
 *     try:             # <<<<<<<<<<<<<<
 *       if ret < 0:
 *         raise HDF5ExtError("Problems reading the array data.")
*/
  /*try:*/ {

    /* "tables/hdf5extension.pyx":1992
 * 
 *     try:
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Problems reading the array data.")
 * 
*/
    __pyx_t_8 = (__pyx_v_ret < 0);
    if (unlikely(__pyx_t_8)) {

      /* "tables/hdf5extension.pyx":1993
 *     try:
 *       if ret < 0:
 *         raise HDF5ExtError("Problems reading the array data.")             # <<<<<<<<<<<<<<
 * 
 *       # Get the pointer to the buffer data area
*/
      __pyx_t_1 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1993, __pyx_L11_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_9 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_7);
        assert(__pyx_t_1);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
        __pyx_t_9 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_Problems_reading_the_array_data};
        __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1993, __pyx_L11_error)
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_Raise(__pyx_t_5, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __PYX_ERR(0, 1993, __pyx_L11_error)

      /* "tables/hdf5extension.pyx":1992
 * 
 *     try:
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         raise HDF5ExtError("Problems reading the array data.")
 * 
*/
    }

    /* "tables/hdf5extension.pyx":1996
 * 
 *       # Get the pointer to the buffer data area
 *       if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
*/
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1996, __pyx_L11_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1996, __pyx_L11_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_reference, Py_EQ)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 1996, __pyx_L11_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (__pyx_t_8) {

      /* "tables/hdf5extension.pyx":1997
 *       # Get the pointer to the buffer data area
 *       if self.atom.kind == "reference":
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)             # <<<<<<<<<<<<<<
 *     finally:
 *       if refbuf:
*/
      __pyx_t_10 = __pyx_f_6tables_14utilsextension_load_reference(__pyx_v_self->__pyx_base.dataset_id, ((hobj_ref_t *)__pyx_v_rbuf), __pyx_v_item_size, __pyx_v_nparr); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(0, 1997, __pyx_L11_error)

      /* "tables/hdf5extension.pyx":1996
 * 
 *       # Get the pointer to the buffer data area
 *       if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
*/
    }
  }

  /* "tables/hdf5extension.pyx":1999
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
 *       if refbuf:             # <<<<<<<<<<<<<<
 *         free(refbuf)
 *         refbuf = NULL
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_8 = (__pyx_v_refbuf != 0);
      if (__pyx_t_8) {

        /* "tables/hdf5extension.pyx":2000
 *     finally:
 *       if refbuf:
 *         free(refbuf)             # <<<<<<<<<<<<<<
 *         refbuf = NULL
 * 
*/
        free(__pyx_v_refbuf);

        /* "tables/hdf5extension.pyx":2001
 *       if refbuf:
 *         free(refbuf)
 *         refbuf = NULL             # <<<<<<<<<<<<<<
 * 
 *     # Terminate access to the memory dataspace
*/
        __pyx_v_refbuf = NULL;

        /* "tables/hdf5extension.pyx":1999
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
 *       if refbuf:             # <<<<<<<<<<<<<<
 *         free(refbuf)
 *         refbuf = NULL
*/
      }
      goto __pyx_L12;
    }
    __pyx_L11_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
      if ( unlikely(__Pyx_GetException(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15) < 0)) __Pyx_ErrFetch(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_18);
      __pyx_t_10 = __pyx_lineno; __pyx_t_11 = __pyx_clineno; __pyx_t_12 = __pyx_filename;
      {
        __pyx_t_8 = (__pyx_v_refbuf != 0);
        if (__pyx_t_8) {

          /* "tables/hdf5extension.pyx":2000
 *     finally:
 *       if refbuf:
 *         free(refbuf)             # <<<<<<<<<<<<<<
 *         refbuf = NULL
 * 
*/
          free(__pyx_v_refbuf);

          /* "tables/hdf5extension.pyx":2001
 *       if refbuf:
 *         free(refbuf)
 *         refbuf = NULL             # <<<<<<<<<<<<<<
 * 
 *     # Terminate access to the memory dataspace
*/
          __pyx_v_refbuf = NULL;

          /* "tables/hdf5extension.pyx":1999
 *         load_reference(self.dataset_id, <hobj_ref_t *>rbuf, item_size, nparr)
 *     finally:
 *       if refbuf:             # <<<<<<<<<<<<<<
 *         free(refbuf)
 *         refbuf = NULL
*/
        }
      }
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_XGIVEREF(__pyx_t_17);
      __Pyx_XGIVEREF(__pyx_t_18);
      __Pyx_ExceptionReset(__pyx_t_16, __pyx_t_17, __pyx_t_18);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_ErrRestore(__pyx_t_13, __pyx_t_14, __pyx_t_15);
      __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
      __pyx_lineno = __pyx_t_10; __pyx_clineno = __pyx_t_11; __pyx_filename = __pyx_t_12;
      goto __pyx_L1_error;
    }
    __pyx_L12:;
  }

  /* "tables/hdf5extension.pyx":2004
 * 
 *     # Terminate access to the memory dataspace
 *     H5Sclose(mem_space_id)             # <<<<<<<<<<<<<<
 *     # Terminate access to the dataspace
 *     H5Sclose(space_id)
*/
  (void)(H5Sclose(__pyx_v_mem_space_id));

  /* "tables/hdf5extension.pyx":2006
 *     H5Sclose(mem_space_id)
 *     # Terminate access to the dataspace
 *     H5Sclose(space_id)             # <<<<<<<<<<<<<<
 * 
 *     if self.atom.kind == 'time':
*/
  (void)(H5Sclose(__pyx_v_space_id));

  /* "tables/hdf5extension.pyx":2008
 *     H5Sclose(space_id)
 * 
 *     if self.atom.kind == 'time':             # <<<<<<<<<<<<<<
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:
*/
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2008, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2008, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_time, Py_EQ)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 2008, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__pyx_t_8) {

    /* "tables/hdf5extension.pyx":2010
 *     if self.atom.kind == 'time':
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:             # <<<<<<<<<<<<<<
 *         nparr.byteswap(True)
 * 
*/
    __pyx_t_5 = __Pyx_PyLong_From_enum__H5T_order_t(H5Tget_order(__pyx_v_self->__pyx_base.type_id)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2010, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_platform_byteorder); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2010, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_1 = PyObject_RichCompare(__pyx_t_5, __pyx_t_7, Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2010, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 2010, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_8) {

      /* "tables/hdf5extension.pyx":2011
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:
 *         nparr.byteswap(True)             # <<<<<<<<<<<<<<
 * 
 *     # Convert some HDF5 types to NumPy after reading
*/
      __pyx_t_7 = ((PyObject *)__pyx_v_nparr);
      __Pyx_INCREF(__pyx_t_7);
      __pyx_t_9 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_7, Py_True};
        __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_byteswap, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2011, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "tables/hdf5extension.pyx":2010
 *     if self.atom.kind == 'time':
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:             # <<<<<<<<<<<<<<
 *         nparr.byteswap(True)
 * 
*/
    }

    /* "tables/hdf5extension.pyx":2008
 *     H5Sclose(space_id)
 * 
 *     if self.atom.kind == 'time':             # <<<<<<<<<<<<<<
 *       # Swap the byteorder by hand (this is not currently supported by HDF5)
 *       if H5Tget_order(self.type_id) != platform_byteorder:
*/
  }

  /* "tables/hdf5extension.pyx":2014
 * 
 *     # Convert some HDF5 types to NumPy after reading
 *     if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *       self._convert_time64(nparr, 1)
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2014, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2014, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_time64, Py_EQ)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 2014, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__pyx_t_8) {

    /* "tables/hdf5extension.pyx":2015
 *     # Convert some HDF5 types to NumPy after reading
 *     if self.atom.type == 'time64':
 *       self._convert_time64(nparr, 1)             # <<<<<<<<<<<<<<
 * 
 *     return
*/
    __pyx_t_7 = ((struct __pyx_vtabstruct_6tables_13hdf5extension_Array *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._convert_time64(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self), __pyx_v_nparr, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2015, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "tables/hdf5extension.pyx":2014
 * 
 *     # Convert some HDF5 types to NumPy after reading
 *     if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *       self._convert_time64(nparr, 1)
 * 
*/
  }

  /* "tables/hdf5extension.pyx":2017
 *       self._convert_time64(nparr, 1)
 * 
 *     return             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":1954
 *                         startp, stepp, countp, NULL)
 * 
 *   def _g_read_selection(self, object selection, ndarray nparr):             # <<<<<<<<<<<<<<
 *     """Read a selection in an already created NumPy array."""
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("tables.hdf5extension.Array._g_read_selection", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":2020
 * 
 * 
 *   def _g_write_slice(self, ndarray startl, ndarray stepl, ndarray countl,             # <<<<<<<<<<<<<<
 *                     ndarray nparr):
 *     """Write a slice in an already created NumPy array."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_19_g_write_slice(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_5Array_18_g_write_slice, "Write a slice in an already created NumPy array.");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Array_19_g_write_slice = {"_g_write_slice", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_19_g_write_slice, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Array_18_g_write_slice};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_19_g_write_slice(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyArrayObject *__pyx_v_startl = 0;
  PyArrayObject *__pyx_v_stepl = 0;
  PyArrayObject *__pyx_v_countl = 0;
  PyArrayObject *__pyx_v_nparr = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_write_slice (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_startl,&__pyx_mstate_global->__pyx_n_u_stepl,&__pyx_mstate_global->__pyx_n_u_countl,&__pyx_mstate_global->__pyx_n_u_nparr,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2020, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 2020, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2020, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2020, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2020, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_write_slice", 0) < (0)) __PYX_ERR(0, 2020, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 4; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_write_slice", 1, 4, 4, i); __PYX_ERR(0, 2020, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 4)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2020, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2020, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2020, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 2020, __pyx_L3_error)
    }
    __pyx_v_startl = ((PyArrayObject *)values[0]);
    __pyx_v_stepl = ((PyArrayObject *)values[1]);
    __pyx_v_countl = ((PyArrayObject *)values[2]);
    __pyx_v_nparr = ((PyArrayObject *)values[3]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_write_slice", 1, 4, 4, __pyx_nargs); __PYX_ERR(0, 2020, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Array._g_write_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_startl), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "startl", 0))) __PYX_ERR(0, 2020, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_stepl), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "stepl", 0))) __PYX_ERR(0, 2020, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_countl), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "countl", 0))) __PYX_ERR(0, 2020, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_nparr), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "nparr", 0))) __PYX_ERR(0, 2021, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Array_18_g_write_slice(((struct __pyx_obj_6tables_13hdf5extension_Array *)__pyx_v_self), __pyx_v_startl, __pyx_v_stepl, __pyx_v_countl, __pyx_v_nparr);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_18_g_write_slice(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyArrayObject *__pyx_v_startl, PyArrayObject *__pyx_v_stepl, PyArrayObject *__pyx_v_countl, PyArrayObject *__pyx_v_nparr) {
  int __pyx_v_ret;
  void *__pyx_v_rbuf;
  hsize_t *__pyx_v_start;
  hsize_t *__pyx_v_step;
  hsize_t *__pyx_v_count;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_write_slice", 0);

  /* "tables/hdf5extension.pyx":2031
 *     cdef hsize_t *count
 * 
 *     if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *       raise ValueError("Cannot write reference types yet")
 *     # Get the pointer to the buffer data area
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2031, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2031, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_reference, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 2031, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(__pyx_t_3)) {

    /* "tables/hdf5extension.pyx":2032
 * 
 *     if self.atom.kind == "reference":
 *       raise ValueError("Cannot write reference types yet")             # <<<<<<<<<<<<<<
 *     # Get the pointer to the buffer data area
 *     rbuf = PyArray_DATA(nparr)
*/
    __pyx_t_1 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_Cannot_write_reference_types_yet};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2032, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 2032, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2031
 *     cdef hsize_t *count
 * 
 *     if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *       raise ValueError("Cannot write reference types yet")
 *     # Get the pointer to the buffer data area
*/
  }

  /* "tables/hdf5extension.pyx":2034
 *       raise ValueError("Cannot write reference types yet")
 *     # Get the pointer to the buffer data area
 *     rbuf = PyArray_DATA(nparr)             # <<<<<<<<<<<<<<
 *     # Get the start, step and count values
 *     start = <hsize_t *>PyArray_DATA(startl)
*/
  __pyx_v_rbuf = PyArray_DATA(__pyx_v_nparr);

  /* "tables/hdf5extension.pyx":2036
 *     rbuf = PyArray_DATA(nparr)
 *     # Get the start, step and count values
 *     start = <hsize_t *>PyArray_DATA(startl)             # <<<<<<<<<<<<<<
 *     step = <hsize_t *>PyArray_DATA(stepl)
 *     count = <hsize_t *>PyArray_DATA(countl)
*/
  __pyx_v_start = ((hsize_t *)PyArray_DATA(__pyx_v_startl));

  /* "tables/hdf5extension.pyx":2037
 *     # Get the start, step and count values
 *     start = <hsize_t *>PyArray_DATA(startl)
 *     step = <hsize_t *>PyArray_DATA(stepl)             # <<<<<<<<<<<<<<
 *     count = <hsize_t *>PyArray_DATA(countl)
 * 
*/
  __pyx_v_step = ((hsize_t *)PyArray_DATA(__pyx_v_stepl));

  /* "tables/hdf5extension.pyx":2038
 *     start = <hsize_t *>PyArray_DATA(startl)
 *     step = <hsize_t *>PyArray_DATA(stepl)
 *     count = <hsize_t *>PyArray_DATA(countl)             # <<<<<<<<<<<<<<
 * 
 *     # Convert some NumPy types to HDF5 before storing.
*/
  __pyx_v_count = ((hsize_t *)PyArray_DATA(__pyx_v_countl));

  /* "tables/hdf5extension.pyx":2041
 * 
 *     # Convert some NumPy types to HDF5 before storing.
 *     if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *       self._convert_time64(nparr, 0)
 * 
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2041, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2041, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_time64, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 2041, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":2042
 *     # Convert some NumPy types to HDF5 before storing.
 *     if self.atom.type == 'time64':
 *       self._convert_time64(nparr, 0)             # <<<<<<<<<<<<<<
 * 
 *     # Modify the elements:
*/
    __pyx_t_1 = ((struct __pyx_vtabstruct_6tables_13hdf5extension_Array *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._convert_time64(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self), __pyx_v_nparr, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2042, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":2041
 * 
 *     # Convert some NumPy types to HDF5 before storing.
 *     if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *       self._convert_time64(nparr, 0)
 * 
*/
  }

  /* "tables/hdf5extension.pyx":2045
 * 
 *     # Modify the elements:
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5ARRAYwrite_records(self.dataset_id, self.type_id, self.rank,
 *                                     start, step, count, rbuf)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "tables/hdf5extension.pyx":2046
 *     # Modify the elements:
 *     with nogil:
 *         ret = H5ARRAYwrite_records(self.dataset_id, self.type_id, self.rank,             # <<<<<<<<<<<<<<
 *                                     start, step, count, rbuf)
 * 
*/
        __pyx_v_ret = H5ARRAYwrite_records(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_self->rank, __pyx_v_start, __pyx_v_step, __pyx_v_count, __pyx_v_rbuf);
      }

      /* "tables/hdf5extension.pyx":2045
 * 
 *     # Modify the elements:
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5ARRAYwrite_records(self.dataset_id, self.type_id, self.rank,
 *                                     start, step, count, rbuf)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "tables/hdf5extension.pyx":2049
 *                                     start, step, count, rbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Internal error modifying the elements "
 *                          "(H5ARRAYwrite_records returned errorcode %i)" % ret)
*/
  __pyx_t_3 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_3)) {

    /* "tables/hdf5extension.pyx":2050
 * 
 *     if ret < 0:
 *       raise HDF5ExtError("Internal error modifying the elements "             # <<<<<<<<<<<<<<
 *                          "(H5ARRAYwrite_records returned errorcode %i)" % ret)
 * 
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2050, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);

    /* "tables/hdf5extension.pyx":2051
 *     if ret < 0:
 *       raise HDF5ExtError("Internal error modifying the elements "
 *                          "(H5ARRAYwrite_records returned errorcode %i)" % ret)             # <<<<<<<<<<<<<<
 * 
 *     return
*/
    __pyx_t_6 = __Pyx_PyLong_From_int(__pyx_v_ret); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2051, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_Internal_error_modifying_the_ele, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2051, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_7};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2050, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 2050, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2049
 *                                     start, step, count, rbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Internal error modifying the elements "
 *                          "(H5ARRAYwrite_records returned errorcode %i)" % ret)
*/
  }

  /* "tables/hdf5extension.pyx":2053
 *                          "(H5ARRAYwrite_records returned errorcode %i)" % ret)
 * 
 *     return             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":2020
 * 
 * 
 *   def _g_write_slice(self, ndarray startl, ndarray stepl, ndarray countl,             # <<<<<<<<<<<<<<
 *                     ndarray nparr):
 *     """Write a slice in an already created NumPy array."""
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("tables.hdf5extension.Array._g_write_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":2056
 * 
 * 
 *   def _g_write_coords(self, ndarray coords, ndarray nparr):             # <<<<<<<<<<<<<<
 *     """Write a selection in an already created NumPy array."""
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_21_g_write_coords(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_5Array_20_g_write_coords, "Write a selection in an already created NumPy array.");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Array_21_g_write_coords = {"_g_write_coords", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_21_g_write_coords, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Array_20_g_write_coords};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_21_g_write_coords(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyArrayObject *__pyx_v_coords = 0;
  PyArrayObject *__pyx_v_nparr = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_write_coords (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_coords,&__pyx_mstate_global->__pyx_n_u_nparr,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2056, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2056, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2056, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_write_coords", 0) < (0)) __PYX_ERR(0, 2056, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_write_coords", 1, 2, 2, i); __PYX_ERR(0, 2056, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2056, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2056, __pyx_L3_error)
    }
    __pyx_v_coords = ((PyArrayObject *)values[0]);
    __pyx_v_nparr = ((PyArrayObject *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_write_coords", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 2056, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Array._g_write_coords", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_coords), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "coords", 0))) __PYX_ERR(0, 2056, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_nparr), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "nparr", 0))) __PYX_ERR(0, 2056, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Array_20_g_write_coords(((struct __pyx_obj_6tables_13hdf5extension_Array *)__pyx_v_self), __pyx_v_coords, __pyx_v_nparr);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_20_g_write_coords(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyArrayObject *__pyx_v_coords, PyArrayObject *__pyx_v_nparr) {
  herr_t __pyx_v_ret;
  hid_t __pyx_v_space_id;
  hid_t __pyx_v_mem_space_id;
  hsize_t __pyx_v_size;
  void *__pyx_v_rbuf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_write_coords", 0);

  /* "tables/hdf5extension.pyx":2066
 *     cdef object mode
 * 
 *     if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *       raise ValueError("Cannot write reference types yet")
 *     # Get the dataspace handle
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2066, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2066, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_reference, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 2066, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(__pyx_t_3)) {

    /* "tables/hdf5extension.pyx":2067
 * 
 *     if self.atom.kind == "reference":
 *       raise ValueError("Cannot write reference types yet")             # <<<<<<<<<<<<<<
 *     # Get the dataspace handle
 *     space_id = H5Dget_space(self.dataset_id)
*/
    __pyx_t_1 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_Cannot_write_reference_types_yet};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2067, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 2067, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2066
 *     cdef object mode
 * 
 *     if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *       raise ValueError("Cannot write reference types yet")
 *     # Get the dataspace handle
*/
  }

  /* "tables/hdf5extension.pyx":2069
 *       raise ValueError("Cannot write reference types yet")
 *     # Get the dataspace handle
 *     space_id = H5Dget_space(self.dataset_id)             # <<<<<<<<<<<<<<
 *     # Create a memory dataspace handle
 *     size = nparr.size
*/
  __pyx_v_space_id = H5Dget_space(__pyx_v_self->__pyx_base.dataset_id);

  /* "tables/hdf5extension.pyx":2071
 *     space_id = H5Dget_space(self.dataset_id)
 *     # Create a memory dataspace handle
 *     size = nparr.size             # <<<<<<<<<<<<<<
 *     mem_space_id = H5Screate_simple(1, &size, NULL)
 * 
*/
  __pyx_v_size = __pyx_f_5numpy_7ndarray_4size_size(__pyx_v_nparr);

  /* "tables/hdf5extension.pyx":2072
 *     # Create a memory dataspace handle
 *     size = nparr.size
 *     mem_space_id = H5Screate_simple(1, &size, NULL)             # <<<<<<<<<<<<<<
 * 
 *     # Select the dataspace to be written
*/
  __pyx_v_mem_space_id = H5Screate_simple(1, (&__pyx_v_size), NULL);

  /* "tables/hdf5extension.pyx":2075
 * 
 *     # Select the dataspace to be written
 *     H5Sselect_elements(space_id, H5S_SELECT_SET,             # <<<<<<<<<<<<<<
 *                        <size_t>size, <hsize_t *>PyArray_DATA(coords))
 * 
*/
  (void)(H5Sselect_elements(__pyx_v_space_id, H5S_SELECT_SET, ((size_t)__pyx_v_size), ((hsize_t *)PyArray_DATA(__pyx_v_coords))));

  /* "tables/hdf5extension.pyx":2079
 * 
 *     # Get the pointer to the buffer data area
 *     rbuf = PyArray_DATA(nparr)             # <<<<<<<<<<<<<<
 * 
 *     # Convert some NumPy types to HDF5 before storing.
*/
  __pyx_v_rbuf = PyArray_DATA(__pyx_v_nparr);

  /* "tables/hdf5extension.pyx":2082
 * 
 *     # Convert some NumPy types to HDF5 before storing.
 *     if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *       self._convert_time64(nparr, 0)
 * 
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2082, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2082, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_time64, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 2082, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":2083
 *     # Convert some NumPy types to HDF5 before storing.
 *     if self.atom.type == 'time64':
 *       self._convert_time64(nparr, 0)             # <<<<<<<<<<<<<<
 * 
 *     # Do the actual write
*/
    __pyx_t_1 = ((struct __pyx_vtabstruct_6tables_13hdf5extension_Array *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._convert_time64(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self), __pyx_v_nparr, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2083, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":2082
 * 
 *     # Convert some NumPy types to HDF5 before storing.
 *     if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *       self._convert_time64(nparr, 0)
 * 
*/
  }

  /* "tables/hdf5extension.pyx":2086
 * 
 *     # Do the actual write
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5Dwrite(self.dataset_id, self.type_id, mem_space_id, space_id,
 *                        H5P_DEFAULT, rbuf)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "tables/hdf5extension.pyx":2087
 *     # Do the actual write
 *     with nogil:
 *         ret = H5Dwrite(self.dataset_id, self.type_id, mem_space_id, space_id,             # <<<<<<<<<<<<<<
 *                        H5P_DEFAULT, rbuf)
 * 
*/
        __pyx_v_ret = H5Dwrite(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_mem_space_id, __pyx_v_space_id, H5P_DEFAULT, __pyx_v_rbuf);
      }

      /* "tables/hdf5extension.pyx":2086
 * 
 *     # Do the actual write
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5Dwrite(self.dataset_id, self.type_id, mem_space_id, space_id,
 *                        H5P_DEFAULT, rbuf)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "tables/hdf5extension.pyx":2090
 *                        H5P_DEFAULT, rbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems writing the array data.")
 * 
*/
  __pyx_t_3 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_3)) {

    /* "tables/hdf5extension.pyx":2091
 * 
 *     if ret < 0:
 *       raise HDF5ExtError("Problems writing the array data.")             # <<<<<<<<<<<<<<
 * 
 *     # Terminate access to the memory dataspace
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2091, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_Problems_writing_the_array_data};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2091, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 2091, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2090
 *                        H5P_DEFAULT, rbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems writing the array data.")
 * 
*/
  }

  /* "tables/hdf5extension.pyx":2094
 * 
 *     # Terminate access to the memory dataspace
 *     H5Sclose(mem_space_id)             # <<<<<<<<<<<<<<
 *     # Terminate access to the dataspace
 *     H5Sclose(space_id)
*/
  (void)(H5Sclose(__pyx_v_mem_space_id));

  /* "tables/hdf5extension.pyx":2096
 *     H5Sclose(mem_space_id)
 *     # Terminate access to the dataspace
 *     H5Sclose(space_id)             # <<<<<<<<<<<<<<
 * 
 *     return
*/
  (void)(H5Sclose(__pyx_v_space_id));

  /* "tables/hdf5extension.pyx":2098
 *     H5Sclose(space_id)
 * 
 *     return             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":2056
 * 
 * 
 *   def _g_write_coords(self, ndarray coords, ndarray nparr):             # <<<<<<<<<<<<<<
 *     """Write a selection in an already created NumPy array."""
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("tables.hdf5extension.Array._g_write_coords", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":2101
 * 
 * 
 *   def _g_write_selection(self, object selection, ndarray nparr):             # <<<<<<<<<<<<<<
 *     """Write a selection in an already created NumPy array."""
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_23_g_write_selection(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_5Array_22_g_write_selection, "Write a selection in an already created NumPy array.");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Array_23_g_write_selection = {"_g_write_selection", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_23_g_write_selection, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Array_22_g_write_selection};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_23_g_write_selection(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_selection = 0;
  PyArrayObject *__pyx_v_nparr = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_write_selection (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_selection,&__pyx_mstate_global->__pyx_n_u_nparr,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2101, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2101, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2101, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_g_write_selection", 0) < (0)) __PYX_ERR(0, 2101, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_g_write_selection", 1, 2, 2, i); __PYX_ERR(0, 2101, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2101, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2101, __pyx_L3_error)
    }
    __pyx_v_selection = values[0];
    __pyx_v_nparr = ((PyArrayObject *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_g_write_selection", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 2101, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Array._g_write_selection", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_nparr), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "nparr", 0))) __PYX_ERR(0, 2101, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Array_22_g_write_selection(((struct __pyx_obj_6tables_13hdf5extension_Array *)__pyx_v_self), __pyx_v_selection, __pyx_v_nparr);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_22_g_write_selection(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, PyObject *__pyx_v_selection, PyArrayObject *__pyx_v_nparr) {
  herr_t __pyx_v_ret;
  hid_t __pyx_v_space_id;
  hid_t __pyx_v_mem_space_id;
  hsize_t __pyx_v_size;
  void *__pyx_v_rbuf;
  PyObject *__pyx_v_args = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_g_write_selection", 0);

  /* "tables/hdf5extension.pyx":2111
 *     cdef object mode
 * 
 *     if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *       raise ValueError("Cannot write reference types yet")
 *     # Get the dataspace handle
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_reference, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 2111, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(__pyx_t_3)) {

    /* "tables/hdf5extension.pyx":2112
 * 
 *     if self.atom.kind == "reference":
 *       raise ValueError("Cannot write reference types yet")             # <<<<<<<<<<<<<<
 *     # Get the dataspace handle
 *     space_id = H5Dget_space(self.dataset_id)
*/
    __pyx_t_1 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_Cannot_write_reference_types_yet};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2112, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 2112, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2111
 *     cdef object mode
 * 
 *     if self.atom.kind == "reference":             # <<<<<<<<<<<<<<
 *       raise ValueError("Cannot write reference types yet")
 *     # Get the dataspace handle
*/
  }

  /* "tables/hdf5extension.pyx":2114
 *       raise ValueError("Cannot write reference types yet")
 *     # Get the dataspace handle
 *     space_id = H5Dget_space(self.dataset_id)             # <<<<<<<<<<<<<<
 *     # Create a memory dataspace handle
 *     size = nparr.size
*/
  __pyx_v_space_id = H5Dget_space(__pyx_v_self->__pyx_base.dataset_id);

  /* "tables/hdf5extension.pyx":2116
 *     space_id = H5Dget_space(self.dataset_id)
 *     # Create a memory dataspace handle
 *     size = nparr.size             # <<<<<<<<<<<<<<
 *     mem_space_id = H5Screate_simple(1, &size, NULL)
 * 
*/
  __pyx_v_size = __pyx_f_5numpy_7ndarray_4size_size(__pyx_v_nparr);

  /* "tables/hdf5extension.pyx":2117
 *     # Create a memory dataspace handle
 *     size = nparr.size
 *     mem_space_id = H5Screate_simple(1, &size, NULL)             # <<<<<<<<<<<<<<
 * 
 *     # Select the dataspace to be written
*/
  __pyx_v_mem_space_id = H5Screate_simple(1, (&__pyx_v_size), NULL);

  /* "tables/hdf5extension.pyx":2121
 *     # Select the dataspace to be written
 *     # Start by selecting everything
 *     H5Sselect_all(space_id)             # <<<<<<<<<<<<<<
 *     # Now refine with outstanding selections
 *     for args in selection:
*/
  (void)(H5Sselect_all(__pyx_v_space_id));

  /* "tables/hdf5extension.pyx":2123
 *     H5Sselect_all(space_id)
 *     # Now refine with outstanding selections
 *     for args in selection:             # <<<<<<<<<<<<<<
 *       self.perform_selection(space_id, *args)
 * 
*/
  if (likely(PyList_CheckExact(__pyx_v_selection)) || PyTuple_CheckExact(__pyx_v_selection)) {
    __pyx_t_2 = __pyx_v_selection; __Pyx_INCREF(__pyx_t_2);
    __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_selection); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2123, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2123, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_2);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 2123, __pyx_L1_error)
          #endif
          if (__pyx_t_5 >= __pyx_temp) break;
        }
        __pyx_t_1 = __Pyx_PyList_GetItemRefFast(__pyx_t_2, __pyx_t_5, __Pyx_ReferenceSharing_OwnStrongReference);
        ++__pyx_t_5;
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_2);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 2123, __pyx_L1_error)
          #endif
          if (__pyx_t_5 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_5));
        #else
        __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_5);
        #endif
        ++__pyx_t_5;
      }
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2123, __pyx_L1_error)
    } else {
      __pyx_t_1 = __pyx_t_6(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 2123, __pyx_L1_error)
          PyErr_Clear();
        }
        break;
      }
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_args, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":2124
 *     # Now refine with outstanding selections
 *     for args in selection:
 *       self.perform_selection(space_id, *args)             # <<<<<<<<<<<<<<
 * 
 *     # Get the pointer to the buffer data area
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_perform_selection); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = __Pyx_PyLong_From_hid_t(__pyx_v_space_id); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_7);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7) != (0)) __PYX_ERR(0, 2124, __pyx_L1_error);
    __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PySequence_Tuple(__pyx_v_args); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = PyNumber_Add(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_9, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "tables/hdf5extension.pyx":2123
 *     H5Sselect_all(space_id)
 *     # Now refine with outstanding selections
 *     for args in selection:             # <<<<<<<<<<<<<<
 *       self.perform_selection(space_id, *args)
 * 
*/
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":2127
 * 
 *     # Get the pointer to the buffer data area
 *     rbuf = PyArray_DATA(nparr)             # <<<<<<<<<<<<<<
 * 
 *     # Convert some NumPy types to HDF5 before storing.
*/
  __pyx_v_rbuf = PyArray_DATA(__pyx_v_nparr);

  /* "tables/hdf5extension.pyx":2130
 * 
 *     # Convert some NumPy types to HDF5 before storing.
 *     if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *       self._convert_time64(nparr, 0)
 * 
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_time64, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 2130, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":2131
 *     # Convert some NumPy types to HDF5 before storing.
 *     if self.atom.type == 'time64':
 *       self._convert_time64(nparr, 0)             # <<<<<<<<<<<<<<
 * 
 *     # Do the actual write
*/
    __pyx_t_7 = ((struct __pyx_vtabstruct_6tables_13hdf5extension_Array *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._convert_time64(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self), __pyx_v_nparr, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2131, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "tables/hdf5extension.pyx":2130
 * 
 *     # Convert some NumPy types to HDF5 before storing.
 *     if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *       self._convert_time64(nparr, 0)
 * 
*/
  }

  /* "tables/hdf5extension.pyx":2134
 * 
 *     # Do the actual write
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5Dwrite(self.dataset_id, self.type_id, mem_space_id, space_id,
 *                        H5P_DEFAULT, rbuf)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "tables/hdf5extension.pyx":2135
 *     # Do the actual write
 *     with nogil:
 *         ret = H5Dwrite(self.dataset_id, self.type_id, mem_space_id, space_id,             # <<<<<<<<<<<<<<
 *                        H5P_DEFAULT, rbuf)
 * 
*/
        __pyx_v_ret = H5Dwrite(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_mem_space_id, __pyx_v_space_id, H5P_DEFAULT, __pyx_v_rbuf);
      }

      /* "tables/hdf5extension.pyx":2134
 * 
 *     # Do the actual write
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5Dwrite(self.dataset_id, self.type_id, mem_space_id, space_id,
 *                        H5P_DEFAULT, rbuf)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L10;
        }
        __pyx_L10:;
      }
  }

  /* "tables/hdf5extension.pyx":2138
 *                        H5P_DEFAULT, rbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems writing the array data.")
 * 
*/
  __pyx_t_3 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_3)) {

    /* "tables/hdf5extension.pyx":2139
 * 
 *     if ret < 0:
 *       raise HDF5ExtError("Problems writing the array data.")             # <<<<<<<<<<<<<<
 * 
 *     # Terminate access to the memory dataspace
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_9);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_Problems_writing_the_array_data};
      __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2139, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }
    __Pyx_Raise(__pyx_t_7, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __PYX_ERR(0, 2139, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2138
 *                        H5P_DEFAULT, rbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems writing the array data.")
 * 
*/
  }

  /* "tables/hdf5extension.pyx":2142
 * 
 *     # Terminate access to the memory dataspace
 *     H5Sclose(mem_space_id)             # <<<<<<<<<<<<<<
 *     # Terminate access to the dataspace
 *     H5Sclose(space_id)
*/
  (void)(H5Sclose(__pyx_v_mem_space_id));

  /* "tables/hdf5extension.pyx":2144
 *     H5Sclose(mem_space_id)
 *     # Terminate access to the dataspace
 *     H5Sclose(space_id)             # <<<<<<<<<<<<<<
 * 
 *     return
*/
  (void)(H5Sclose(__pyx_v_space_id));

  /* "tables/hdf5extension.pyx":2146
 *     H5Sclose(space_id)
 * 
 *     return             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":2101
 * 
 * 
 *   def _g_write_selection(self, object selection, ndarray nparr):             # <<<<<<<<<<<<<<
 *     """Write a selection in an already created NumPy array."""
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("tables.hdf5extension.Array._g_write_selection", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":2149
 * 
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     if self.dims:
 *       free(<void *>self.dims)
*/

/* Python wrapper */
static void __pyx_pw_6tables_13hdf5extension_5Array_25__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_6tables_13hdf5extension_5Array_25__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_6tables_13hdf5extension_5Array_24__dealloc__(((struct __pyx_obj_6tables_13hdf5extension_Array *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_6tables_13hdf5extension_5Array_24__dealloc__(struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self) {
  int __pyx_t_1;

  /* "tables/hdf5extension.pyx":2150
 * 
 *   def __dealloc__(self):
 *     if self.dims:             # <<<<<<<<<<<<<<
 *       free(<void *>self.dims)
 *     if self.maxdims:
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base.dims != 0);
  if (__pyx_t_1) {

    /* "tables/hdf5extension.pyx":2151
 *   def __dealloc__(self):
 *     if self.dims:
 *       free(<void *>self.dims)             # <<<<<<<<<<<<<<
 *     if self.maxdims:
 *       free(<void *>self.maxdims)
*/
    free(((void *)__pyx_v_self->__pyx_base.dims));

    /* "tables/hdf5extension.pyx":2150
 * 
 *   def __dealloc__(self):
 *     if self.dims:             # <<<<<<<<<<<<<<
 *       free(<void *>self.dims)
 *     if self.maxdims:
*/
  }

  /* "tables/hdf5extension.pyx":2152
 *     if self.dims:
 *       free(<void *>self.dims)
 *     if self.maxdims:             # <<<<<<<<<<<<<<
 *       free(<void *>self.maxdims)
 *     if self.dims_chunk:
*/
  __pyx_t_1 = (__pyx_v_self->maxdims != 0);
  if (__pyx_t_1) {

    /* "tables/hdf5extension.pyx":2153
 *       free(<void *>self.dims)
 *     if self.maxdims:
 *       free(<void *>self.maxdims)             # <<<<<<<<<<<<<<
 *     if self.dims_chunk:
 *       free(self.dims_chunk)
*/
    free(((void *)__pyx_v_self->maxdims));

    /* "tables/hdf5extension.pyx":2152
 *     if self.dims:
 *       free(<void *>self.dims)
 *     if self.maxdims:             # <<<<<<<<<<<<<<
 *       free(<void *>self.maxdims)
 *     if self.dims_chunk:
*/
  }

  /* "tables/hdf5extension.pyx":2154
 *     if self.maxdims:
 *       free(<void *>self.maxdims)
 *     if self.dims_chunk:             # <<<<<<<<<<<<<<
 *       free(self.dims_chunk)
 * 
*/
  __pyx_t_1 = (__pyx_v_self->dims_chunk != 0);
  if (__pyx_t_1) {

    /* "tables/hdf5extension.pyx":2155
 *       free(<void *>self.maxdims)
 *     if self.dims_chunk:
 *       free(self.dims_chunk)             # <<<<<<<<<<<<<<
 * 
 * 
*/
    free(__pyx_v_self->dims_chunk);

    /* "tables/hdf5extension.pyx":2154
 *     if self.maxdims:
 *       free(<void *>self.maxdims)
 *     if self.dims_chunk:             # <<<<<<<<<<<<<<
 *       free(self.dims_chunk)
 * 
*/
  }

  /* "tables/hdf5extension.pyx":2149
 * 
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     if self.dims:
 *       free(<void *>self.dims)
*/

  /* function exit code */
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_27__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Array_27__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_27__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_27__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Array_26__reduce_cython__(((struct __pyx_obj_6tables_13hdf5extension_Array *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_26__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self_dims_self_dims_chunk_self_m, 0, 0);
  __PYX_ERR(2, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("tables.hdf5extension.Array.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_29__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5Array_29__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_29__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_5Array_29__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(2, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(2, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(2, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(2, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.Array.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_5Array_28__setstate_cython__(((struct __pyx_obj_6tables_13hdf5extension_Array *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_5Array_28__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_Array *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self_dims_self_dims_chunk_self_m, 0, 0);
  __PYX_ERR(2, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("tables.hdf5extension.Array.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":2162
 *   cdef hsize_t nrecords
 * 
 *   def _create_array(self, object title):             # <<<<<<<<<<<<<<
 *     cdef int rank
 *     cdef hsize_t *dims
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_7VLArray_1_create_array(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_7VLArray_1_create_array = {"_create_array", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7VLArray_1_create_array, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_7VLArray_1_create_array(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_title = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_create_array (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_title,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2162, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2162, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_create_array", 0) < (0)) __PYX_ERR(0, 2162, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_create_array", 1, 1, 1, i); __PYX_ERR(0, 2162, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2162, __pyx_L3_error)
    }
    __pyx_v_title = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_create_array", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 2162, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.VLArray._create_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_7VLArray__create_array(((struct __pyx_obj_6tables_13hdf5extension_VLArray *)__pyx_v_self), __pyx_v_title);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_7VLArray__create_array(struct __pyx_obj_6tables_13hdf5extension_VLArray *__pyx_v_self, PyObject *__pyx_v_title) {
  int __pyx_v_rank;
  hsize_t *__pyx_v_dims;
  void *__pyx_v_rbuf;
  PyObject *__pyx_v_complib = 0;
  PyObject *__pyx_v_version = 0;
  PyObject *__pyx_v_class_ = 0;
  PyObject *__pyx_v_atom = 0;
  PyObject *__pyx_v_scatom = 0;
  PyObject *__pyx_v_encoded_title = 0;
  PyObject *__pyx_v_encoded_name = 0;
  enum H5T_cset_t __pyx_v_cset;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  hid_t __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10[4];
  PyObject *__pyx_t_11 = NULL;
  Py_ssize_t __pyx_t_12;
  hsize_t *__pyx_t_13;
  char *__pyx_t_14;
  char *__pyx_t_15;
  hsize_t __pyx_t_16;
  int __pyx_t_17;
  char *__pyx_t_18;
  int __pyx_t_19;
  int __pyx_t_20;
  hbool_t __pyx_t_21;
  PyObject *(*__pyx_t_22)(PyObject *);
  hid_t __pyx_t_23;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_create_array", 0);

  /* "tables/hdf5extension.pyx":2170
 *     cdef object type_, itemsize, atom, scatom
 *     cdef bytes encoded_title, encoded_name
 *     cdef H5T_cset_t cset = H5T_CSET_ASCII             # <<<<<<<<<<<<<<
 * 
 *     encoded_title = title.encode('utf-8')
*/
  __pyx_v_cset = H5T_CSET_ASCII;

  /* "tables/hdf5extension.pyx":2172
 *     cdef H5T_cset_t cset = H5T_CSET_ASCII
 * 
 *     encoded_title = title.encode('utf-8')             # <<<<<<<<<<<<<<
 *     encoded_name = self.name.encode('utf-8')
 * 
*/
  __pyx_t_2 = __pyx_v_title;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2172, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 2172, __pyx_L1_error)
  __pyx_v_encoded_title = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2173
 * 
 *     encoded_title = title.encode('utf-8')
 *     encoded_name = self.name.encode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     atom = self.atom
*/
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.name;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2173, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 2173, __pyx_L1_error)
  __pyx_v_encoded_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2175
 *     encoded_name = self.name.encode('utf-8')
 * 
 *     atom = self.atom             # <<<<<<<<<<<<<<
 *     if not hasattr(atom, 'size'):  # it is a pseudo-atom
 *       atom = atom.base
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_atom = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2176
 * 
 *     atom = self.atom
 *     if not hasattr(atom, 'size'):  # it is a pseudo-atom             # <<<<<<<<<<<<<<
 *       atom = atom.base
 * 
*/
  __pyx_t_4 = __Pyx_HasAttr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_size); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 2176, __pyx_L1_error)
  __pyx_t_5 = (!__pyx_t_4);
  if (__pyx_t_5) {

    /* "tables/hdf5extension.pyx":2177
 *     atom = self.atom
 *     if not hasattr(atom, 'size'):  # it is a pseudo-atom
 *       atom = atom.base             # <<<<<<<<<<<<<<
 * 
 *     # Get the HDF5 type of the *scalar* atom
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_base); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2177, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_atom, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":2176
 * 
 *     atom = self.atom
 *     if not hasattr(atom, 'size'):  # it is a pseudo-atom             # <<<<<<<<<<<<<<
 *       atom = atom.base
 * 
*/
  }

  /* "tables/hdf5extension.pyx":2180
 * 
 *     # Get the HDF5 type of the *scalar* atom
 *     scatom = atom.copy(shape=())             # <<<<<<<<<<<<<<
 *     self.base_type_id = atom_to_hdf5_type(scatom, self.byteorder)
 *     if self.base_type_id < 0:
*/
  __pyx_t_2 = __pyx_v_atom;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, NULL};
    __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2180, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_shape, __pyx_mstate_global->__pyx_empty_tuple, __pyx_t_6, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 2180, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_copy, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2180, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_scatom = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2181
 *     # Get the HDF5 type of the *scalar* atom
 *     scatom = atom.copy(shape=())
 *     self.base_type_id = atom_to_hdf5_type(scatom, self.byteorder)             # <<<<<<<<<<<<<<
 *     if self.base_type_id < 0:
 *       raise HDF5ExtError(
*/
  __pyx_t_6 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_atom_to_hdf5_type); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_byteorder); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_6);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_6);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_6, __pyx_v_scatom, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2181, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_8 = __Pyx_PyLong_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_8 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2181, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->__pyx_base.base_type_id = __pyx_t_8;

  /* "tables/hdf5extension.pyx":2182
 *     scatom = atom.copy(shape=())
 *     self.base_type_id = atom_to_hdf5_type(scatom, self.byteorder)
 *     if self.base_type_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError(
 *         "Problems creating the %s: invalid base type ID for atom %s" % (
*/
  __pyx_t_5 = (__pyx_v_self->__pyx_base.base_type_id < 0);
  if (unlikely(__pyx_t_5)) {

    /* "tables/hdf5extension.pyx":2183
 *     self.base_type_id = atom_to_hdf5_type(scatom, self.byteorder)
 *     if self.base_type_id < 0:
 *       raise HDF5ExtError(             # <<<<<<<<<<<<<<
 *         "Problems creating the %s: invalid base type ID for atom %s" % (
 *             self.__class__.__name__, scatom))
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);

    /* "tables/hdf5extension.pyx":2185
 *       raise HDF5ExtError(
 *         "Problems creating the %s: invalid base type ID for atom %s" % (
 *             self.__class__.__name__, scatom))             # <<<<<<<<<<<<<<
 * 
 *     # Allocate space for the dimension axis info
*/
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_class); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2185, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_name_2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2185, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_t_9), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2185, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_scatom), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2185, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u_Problems_creating_the;
    __pyx_t_10[1] = __pyx_t_6;
    __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u_invalid_base_type_ID_for_atom;
    __pyx_t_10[3] = __pyx_t_9;

    /* "tables/hdf5extension.pyx":2184
 *     if self.base_type_id < 0:
 *       raise HDF5ExtError(
 *         "Problems creating the %s: invalid base type ID for atom %s" % (             # <<<<<<<<<<<<<<
 *             self.__class__.__name__, scatom))
 * 
*/
    __pyx_t_11 = __Pyx_PyUnicode_Join(__pyx_t_10, 4, 22 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6) + 32 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_9));
    if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 2184, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_11};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2183, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 2183, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2182
 *     scatom = atom.copy(shape=())
 *     self.base_type_id = atom_to_hdf5_type(scatom, self.byteorder)
 *     if self.base_type_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError(
 *         "Problems creating the %s: invalid base type ID for atom %s" % (
*/
  }

  /* "tables/hdf5extension.pyx":2188
 * 
 *     # Allocate space for the dimension axis info
 *     rank = len(atom.shape)             # <<<<<<<<<<<<<<
 *     dims = malloc_dims(atom.shape)
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_12 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2188, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_rank = __pyx_t_12;

  /* "tables/hdf5extension.pyx":2189
 *     # Allocate space for the dimension axis info
 *     rank = len(atom.shape)
 *     dims = malloc_dims(atom.shape)             # <<<<<<<<<<<<<<
 * 
 *     rbuf = NULL   # We don't have data to save initially
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_13 = __pyx_f_6tables_14utilsextension_malloc_dims(__pyx_t_1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 2189, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_dims = __pyx_t_13;

  /* "tables/hdf5extension.pyx":2191
 *     dims = malloc_dims(atom.shape)
 * 
 *     rbuf = NULL   # We don't have data to save initially             # <<<<<<<<<<<<<<
 * 
 *     # Encode strings
*/
  __pyx_v_rbuf = NULL;

  /* "tables/hdf5extension.pyx":2194
 * 
 *     # Encode strings
 *     complib = (self.filters.complib or '').encode('utf-8')             # <<<<<<<<<<<<<<
 *     version = self._v_version.encode('utf-8')
 *     class_ = self._c_classid.encode('utf-8')
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_filters); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_complib); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 2194, __pyx_L1_error)
  if (!__pyx_t_5) {
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_9);
    __pyx_t_11 = __pyx_t_9;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    goto __pyx_L5_bool_binop_done;
  }
  __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_);
  __pyx_t_11 = __pyx_mstate_global->__pyx_kp_u_;
  __pyx_L5_bool_binop_done:;
  __pyx_t_7 = __pyx_t_11;
  __Pyx_INCREF(__pyx_t_7);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2194, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 2194, __pyx_L1_error)
  __pyx_v_complib = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2195
 *     # Encode strings
 *     complib = (self.filters.complib or '').encode('utf-8')
 *     version = self._v_version.encode('utf-8')             # <<<<<<<<<<<<<<
 *     class_ = self._c_classid.encode('utf-8')
 * 
*/
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_version); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_11 = __pyx_t_7;
  __Pyx_INCREF(__pyx_t_11);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2195, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 2195, __pyx_L1_error)
  __pyx_v_version = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2196
 *     complib = (self.filters.complib or '').encode('utf-8')
 *     version = self._v_version.encode('utf-8')
 *     class_ = self._c_classid.encode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     # Create the vlarray
*/
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_c_classid); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 2196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_7 = __pyx_t_11;
  __Pyx_INCREF(__pyx_t_7);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2196, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 2196, __pyx_L1_error)
  __pyx_v_class_ = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2199
 * 
 *     # Create the vlarray
 *     self.dataset_id = H5VLARRAYmake(self.parent_id, encoded_name, version,             # <<<<<<<<<<<<<<
 *                                     rank, dims, self.base_type_id,
 *                                     self.chunkshape[0], rbuf,
*/
  if (unlikely(__pyx_v_encoded_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 2199, __pyx_L1_error)
  }
  __pyx_t_14 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_name); if (unlikely((!__pyx_t_14) && PyErr_Occurred())) __PYX_ERR(0, 2199, __pyx_L1_error)
  if (unlikely(__pyx_v_version == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 2199, __pyx_L1_error)
  }
  __pyx_t_15 = __Pyx_PyBytes_AsWritableString(__pyx_v_version); if (unlikely((!__pyx_t_15) && PyErr_Occurred())) __PYX_ERR(0, 2199, __pyx_L1_error)

  /* "tables/hdf5extension.pyx":2201
 *     self.dataset_id = H5VLARRAYmake(self.parent_id, encoded_name, version,
 *                                     rank, dims, self.base_type_id,
 *                                     self.chunkshape[0], rbuf,             # <<<<<<<<<<<<<<
 *                                     self.filters.complevel, complib,
 *                                     self.filters.shuffle_bitshuffle,
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_chunkshape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_11 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 2201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_16 = __Pyx_PyLong_As_hsize_t(__pyx_t_11); if (unlikely((__pyx_t_16 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2201, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "tables/hdf5extension.pyx":2202
 *                                     rank, dims, self.base_type_id,
 *                                     self.chunkshape[0], rbuf,
 *                                     self.filters.complevel, complib,             # <<<<<<<<<<<<<<
 *                                     self.filters.shuffle_bitshuffle,
 *                                     self.filters.fletcher32,
*/
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_filters); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 2202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_complevel); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_17 = __Pyx_PyLong_As_int(__pyx_t_1); if (unlikely((__pyx_t_17 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2202, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_v_complib == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 2202, __pyx_L1_error)
  }
  __pyx_t_18 = __Pyx_PyBytes_AsWritableString(__pyx_v_complib); if (unlikely((!__pyx_t_18) && PyErr_Occurred())) __PYX_ERR(0, 2202, __pyx_L1_error)

  /* "tables/hdf5extension.pyx":2203
 *                                     self.chunkshape[0], rbuf,
 *                                     self.filters.complevel, complib,
 *                                     self.filters.shuffle_bitshuffle,             # <<<<<<<<<<<<<<
 *                                     self.filters.fletcher32,
 *                                     self._want_track_times, rbuf)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_filters); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_shuffle_bitshuffle); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 2203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_19 = __Pyx_PyLong_As_int(__pyx_t_11); if (unlikely((__pyx_t_19 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2203, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "tables/hdf5extension.pyx":2204
 *                                     self.filters.complevel, complib,
 *                                     self.filters.shuffle_bitshuffle,
 *                                     self.filters.fletcher32,             # <<<<<<<<<<<<<<
 *                                     self._want_track_times, rbuf)
 *     if dims:
*/
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_filters); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 2204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_fletcher32); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_20 = __Pyx_PyLong_As_int(__pyx_t_1); if (unlikely((__pyx_t_20 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2204, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2205
 *                                     self.filters.shuffle_bitshuffle,
 *                                     self.filters.fletcher32,
 *                                     self._want_track_times, rbuf)             # <<<<<<<<<<<<<<
 *     if dims:
 *       free(<void *>dims)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_want_track_times); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_21 = __Pyx_PyLong_As_hbool_t(__pyx_t_1); if (unlikely((__pyx_t_21 == ((hbool_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2205, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2199
 * 
 *     # Create the vlarray
 *     self.dataset_id = H5VLARRAYmake(self.parent_id, encoded_name, version,             # <<<<<<<<<<<<<<
 *                                     rank, dims, self.base_type_id,
 *                                     self.chunkshape[0], rbuf,
*/
  __pyx_v_self->__pyx_base.dataset_id = H5VLARRAYmake(__pyx_v_self->__pyx_base.__pyx_base.parent_id, __pyx_t_14, __pyx_t_15, __pyx_v_rank, __pyx_v_dims, __pyx_v_self->__pyx_base.base_type_id, __pyx_t_16, __pyx_v_rbuf, __pyx_t_17, __pyx_t_18, __pyx_t_19, __pyx_t_20, __pyx_t_21, __pyx_v_rbuf);

  /* "tables/hdf5extension.pyx":2206
 *                                     self.filters.fletcher32,
 *                                     self._want_track_times, rbuf)
 *     if dims:             # <<<<<<<<<<<<<<
 *       free(<void *>dims)
 *     if self.dataset_id < 0:
*/
  __pyx_t_5 = (__pyx_v_dims != 0);
  if (__pyx_t_5) {

    /* "tables/hdf5extension.pyx":2207
 *                                     self._want_track_times, rbuf)
 *     if dims:
 *       free(<void *>dims)             # <<<<<<<<<<<<<<
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Problems creating the VLArray.")
*/
    free(((void *)__pyx_v_dims));

    /* "tables/hdf5extension.pyx":2206
 *                                     self.filters.fletcher32,
 *                                     self._want_track_times, rbuf)
 *     if dims:             # <<<<<<<<<<<<<<
 *       free(<void *>dims)
 *     if self.dataset_id < 0:
*/
  }

  /* "tables/hdf5extension.pyx":2208
 *     if dims:
 *       free(<void *>dims)
 *     if self.dataset_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems creating the VLArray.")
 *     self.nrecords = 0  # Initialize the number of records saved
*/
  __pyx_t_5 = (__pyx_v_self->__pyx_base.dataset_id < 0);
  if (unlikely(__pyx_t_5)) {

    /* "tables/hdf5extension.pyx":2209
 *       free(<void *>dims)
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Problems creating the VLArray.")             # <<<<<<<<<<<<<<
 *     self.nrecords = 0  # Initialize the number of records saved
 * 
*/
    __pyx_t_11 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2209, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_7);
      assert(__pyx_t_11);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_mstate_global->__pyx_kp_u_Problems_creating_the_VLArray};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2209, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 2209, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2208
 *     if dims:
 *       free(<void *>dims)
 *     if self.dataset_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems creating the VLArray.")
 *     self.nrecords = 0  # Initialize the number of records saved
*/
  }

  /* "tables/hdf5extension.pyx":2210
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Problems creating the VLArray.")
 *     self.nrecords = 0  # Initialize the number of records saved             # <<<<<<<<<<<<<<
 * 
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:
*/
  __pyx_v_self->nrecords = 0;

  /* "tables/hdf5extension.pyx":2212
 *     self.nrecords = 0  # Initialize the number of records saved
 * 
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:             # <<<<<<<<<<<<<<
 *       cset = H5T_CSET_UTF8
 *       # Set the conforming array attributes
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_file); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_params); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_PYTABLES_SYS_ATTRS); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 2212, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "tables/hdf5extension.pyx":2213
 * 
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:
 *       cset = H5T_CSET_UTF8             # <<<<<<<<<<<<<<
 *       # Set the conforming array attributes
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,
*/
    __pyx_v_cset = H5T_CSET_UTF8;

    /* "tables/hdf5extension.pyx":2215
 *       cset = H5T_CSET_UTF8
 *       # Set the conforming array attributes
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,             # <<<<<<<<<<<<<<
 *                                  len(class_), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,
*/
    if (unlikely(__pyx_v_class_ == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 2215, __pyx_L1_error)
    }
    __pyx_t_18 = __Pyx_PyBytes_AsWritableString(__pyx_v_class_); if (unlikely((!__pyx_t_18) && PyErr_Occurred())) __PYX_ERR(0, 2215, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2216
 *       # Set the conforming array attributes
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,
 *                                  len(class_), cset)             # <<<<<<<<<<<<<<
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,
 *                                  len(version), cset)
*/
    if (unlikely(__pyx_v_class_ == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 2216, __pyx_L1_error)
    }
    __pyx_t_12 = __Pyx_PyBytes_GET_SIZE(__pyx_v_class_); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2216, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2215
 *       cset = H5T_CSET_UTF8
 *       # Set the conforming array attributes
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,             # <<<<<<<<<<<<<<
 *                                  len(class_), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,
*/
    (void)(H5ATTRset_attribute_string(__pyx_v_self->__pyx_base.dataset_id, ((char *)"CLASS"), __pyx_t_18, __pyx_t_12, __pyx_v_cset));

    /* "tables/hdf5extension.pyx":2217
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,
 *                                  len(class_), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,             # <<<<<<<<<<<<<<
 *                                  len(version), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,
*/
    if (unlikely(__pyx_v_version == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 2217, __pyx_L1_error)
    }
    __pyx_t_18 = __Pyx_PyBytes_AsWritableString(__pyx_v_version); if (unlikely((!__pyx_t_18) && PyErr_Occurred())) __PYX_ERR(0, 2217, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2218
 *                                  len(class_), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,
 *                                  len(version), cset)             # <<<<<<<<<<<<<<
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,
 *                                  len(encoded_title), cset)
*/
    if (unlikely(__pyx_v_version == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 2218, __pyx_L1_error)
    }
    __pyx_t_12 = __Pyx_PyBytes_GET_SIZE(__pyx_v_version); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2218, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2217
 *       H5ATTRset_attribute_string(self.dataset_id, "CLASS", class_,
 *                                  len(class_), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,             # <<<<<<<<<<<<<<
 *                                  len(version), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,
*/
    (void)(H5ATTRset_attribute_string(__pyx_v_self->__pyx_base.dataset_id, ((char *)"VERSION"), __pyx_t_18, __pyx_t_12, __pyx_v_cset));

    /* "tables/hdf5extension.pyx":2219
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,
 *                                  len(version), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,             # <<<<<<<<<<<<<<
 *                                  len(encoded_title), cset)
 * 
*/
    if (unlikely(__pyx_v_encoded_title == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 2219, __pyx_L1_error)
    }
    __pyx_t_18 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_title); if (unlikely((!__pyx_t_18) && PyErr_Occurred())) __PYX_ERR(0, 2219, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2220
 *                                  len(version), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,
 *                                  len(encoded_title), cset)             # <<<<<<<<<<<<<<
 * 
 *     # Get the datatype handles
*/
    if (unlikely(__pyx_v_encoded_title == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 2220, __pyx_L1_error)
    }
    __pyx_t_12 = __Pyx_PyBytes_GET_SIZE(__pyx_v_encoded_title); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2220, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2219
 *       H5ATTRset_attribute_string(self.dataset_id, "VERSION", version,
 *                                  len(version), cset)
 *       H5ATTRset_attribute_string(self.dataset_id, "TITLE", encoded_title,             # <<<<<<<<<<<<<<
 *                                  len(encoded_title), cset)
 * 
*/
    (void)(H5ATTRset_attribute_string(__pyx_v_self->__pyx_base.dataset_id, ((char *)"TITLE"), __pyx_t_18, __pyx_t_12, __pyx_v_cset));

    /* "tables/hdf5extension.pyx":2212
 *     self.nrecords = 0  # Initialize the number of records saved
 * 
 *     if self._v_file.params['PYTABLES_SYS_ATTRS']:             # <<<<<<<<<<<<<<
 *       cset = H5T_CSET_UTF8
 *       # Set the conforming array attributes
*/
  }

  /* "tables/hdf5extension.pyx":2223
 * 
 *     # Get the datatype handles
 *     self.disk_type_id, self.type_id = self._get_type_ids()             # <<<<<<<<<<<<<<
 * 
 *     return self.dataset_id
*/
  __pyx_t_1 = ((struct __pyx_vtabstruct_6tables_13hdf5extension_VLArray *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get_type_ids(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2223, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 2223, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_7 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_7);
      __pyx_t_11 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_11);
    } else {
      __pyx_t_7 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2223, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_7);
      __pyx_t_11 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 2223, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_11);
    }
    #else
    __pyx_t_7 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2223, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_11 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 2223, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2223, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_22 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_9);
    index = 0; __pyx_t_7 = __pyx_t_22(__pyx_t_9); if (unlikely(!__pyx_t_7)) goto __pyx_L10_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_7);
    index = 1; __pyx_t_11 = __pyx_t_22(__pyx_t_9); if (unlikely(!__pyx_t_11)) goto __pyx_L10_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_11);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_22(__pyx_t_9), 2) < (0)) __PYX_ERR(0, 2223, __pyx_L1_error)
    __pyx_t_22 = NULL;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    goto __pyx_L11_unpacking_done;
    __pyx_L10_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_22 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 2223, __pyx_L1_error)
    __pyx_L11_unpacking_done:;
  }
  __pyx_t_8 = __Pyx_PyLong_As_hid_t(__pyx_t_7); if (unlikely((__pyx_t_8 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2223, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_23 = __Pyx_PyLong_As_hid_t(__pyx_t_11); if (unlikely((__pyx_t_23 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2223, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_v_self->__pyx_base.disk_type_id = __pyx_t_8;
  __pyx_v_self->__pyx_base.type_id = __pyx_t_23;

  /* "tables/hdf5extension.pyx":2225
 *     self.disk_type_id, self.type_id = self._get_type_ids()
 * 
 *     return self.dataset_id             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_hid_t(__pyx_v_self->__pyx_base.dataset_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2225, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":2162
 *   cdef hsize_t nrecords
 * 
 *   def _create_array(self, object title):             # <<<<<<<<<<<<<<
 *     cdef int rank
 *     cdef hsize_t *dims
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("tables.hdf5extension.VLArray._create_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_complib);
  __Pyx_XDECREF(__pyx_v_version);
  __Pyx_XDECREF(__pyx_v_class_);
  __Pyx_XDECREF(__pyx_v_atom);
  __Pyx_XDECREF(__pyx_v_scatom);
  __Pyx_XDECREF(__pyx_v_encoded_title);
  __Pyx_XDECREF(__pyx_v_encoded_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":2228
 * 
 * 
 *   def _open_array(self):             # <<<<<<<<<<<<<<
 *     cdef char cbyteorder[11]  # "irrelevant" fits easily here
 *     cdef int i, enumtype
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_7VLArray_3_open_array(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_7VLArray_3_open_array = {"_open_array", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7VLArray_3_open_array, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_7VLArray_3_open_array(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_open_array (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_open_array", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_open_array", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_7VLArray_2_open_array(((struct __pyx_obj_6tables_13hdf5extension_VLArray *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_7VLArray_2_open_array(struct __pyx_obj_6tables_13hdf5extension_VLArray *__pyx_v_self) {
  char __pyx_v_cbyteorder[11];
  hsize_t __pyx_v_nrecords;
  hsize_t __pyx_v_chunksize;
  PyObject *__pyx_v_encoded_name = 0;
  PyObject *__pyx_v_byteorder = 0;
  PyObject *__pyx_v_atom = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  char *__pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10[5];
  PyObject *(*__pyx_t_11)(PyObject *);
  hid_t __pyx_t_12;
  hid_t __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_open_array", 0);

  /* "tables/hdf5extension.pyx":2238
 *     cdef str byteorder
 * 
 *     encoded_name = self.name.encode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     # Open the dataset
*/
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.name;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2238, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 2238, __pyx_L1_error)
  __pyx_v_encoded_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2241
 * 
 *     # Open the dataset
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)             # <<<<<<<<<<<<<<
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
*/
  if (unlikely(__pyx_v_encoded_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 2241, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_name); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 2241, __pyx_L1_error)
  __pyx_v_self->__pyx_base.dataset_id = H5Dopen(__pyx_v_self->__pyx_base.__pyx_base.parent_id, __pyx_t_4, H5P_DEFAULT);

  /* "tables/hdf5extension.pyx":2242
 *     # Open the dataset
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)
 *     if self.dataset_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
 *                          (self.name, self._v_parent._v_pathname))
*/
  __pyx_t_5 = (__pyx_v_self->__pyx_base.dataset_id < 0);
  if (unlikely(__pyx_t_5)) {

    /* "tables/hdf5extension.pyx":2243
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %             # <<<<<<<<<<<<<<
 *                          (self.name, self._v_parent._v_pathname))
 *     # Get the datatype handles
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2243, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);

    /* "tables/hdf5extension.pyx":2244
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
 *                          (self.name, self._v_parent._v_pathname))             # <<<<<<<<<<<<<<
 *     # Get the datatype handles
 *     self.disk_type_id, self.type_id = self._get_type_ids()
*/
    __pyx_t_7 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_self->__pyx_base.__pyx_base.name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2244, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_v_parent); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2244, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_v_pathname); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2244, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_t_9), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2244, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u_Non_existing_node;
    __pyx_t_10[1] = __pyx_t_7;
    __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u_under;
    __pyx_t_10[3] = __pyx_t_8;
    __pyx_t_10[4] = __pyx_mstate_global->__pyx_kp_u__4;

    /* "tables/hdf5extension.pyx":2243
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)
 *     if self.dataset_id < 0:
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %             # <<<<<<<<<<<<<<
 *                          (self.name, self._v_parent._v_pathname))
 *     # Get the datatype handles
*/
    __pyx_t_9 = __Pyx_PyUnicode_Join(__pyx_t_10, 5, 20 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7) + 11 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8) + 2, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8));
    if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2243, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_9};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2243, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 2243, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2242
 *     # Open the dataset
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)
 *     if self.dataset_id < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Non-existing node ``%s`` under ``%s``" %
 *                          (self.name, self._v_parent._v_pathname))
*/
  }

  /* "tables/hdf5extension.pyx":2246
 *                          (self.name, self._v_parent._v_pathname))
 *     # Get the datatype handles
 *     self.disk_type_id, self.type_id = self._get_type_ids()             # <<<<<<<<<<<<<<
 *     # Get the atom for this type
 *     atom = atom_from_hdf5_type(self.type_id)
*/
  __pyx_t_1 = ((struct __pyx_vtabstruct_6tables_13hdf5extension_VLArray *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get_type_ids(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 2246, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_6 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_6);
      __pyx_t_9 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_9);
    } else {
      __pyx_t_6 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2246, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_6);
      __pyx_t_9 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2246, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_9);
    }
    #else
    __pyx_t_6 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2246, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2246, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2246, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_11 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_2);
    index = 0; __pyx_t_6 = __pyx_t_11(__pyx_t_2); if (unlikely(!__pyx_t_6)) goto __pyx_L4_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_6);
    index = 1; __pyx_t_9 = __pyx_t_11(__pyx_t_2); if (unlikely(!__pyx_t_9)) goto __pyx_L4_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_9);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_11(__pyx_t_2), 2) < (0)) __PYX_ERR(0, 2246, __pyx_L1_error)
    __pyx_t_11 = NULL;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L5_unpacking_done;
    __pyx_L4_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_11 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 2246, __pyx_L1_error)
    __pyx_L5_unpacking_done:;
  }
  __pyx_t_12 = __Pyx_PyLong_As_hid_t(__pyx_t_6); if (unlikely((__pyx_t_12 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2246, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_13 = __Pyx_PyLong_As_hid_t(__pyx_t_9); if (unlikely((__pyx_t_13 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2246, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_self->__pyx_base.disk_type_id = __pyx_t_12;
  __pyx_v_self->__pyx_base.type_id = __pyx_t_13;

  /* "tables/hdf5extension.pyx":2248
 *     self.disk_type_id, self.type_id = self._get_type_ids()
 *     # Get the atom for this type
 *     atom = atom_from_hdf5_type(self.type_id)             # <<<<<<<<<<<<<<
 * 
 *     # Get info on dimensions & types (of base class)
*/
  __pyx_t_9 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_atom_from_hdf5_type); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_2 = __Pyx_PyLong_From_hid_t(__pyx_v_self->__pyx_base.type_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_6);
    assert(__pyx_t_9);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
    __Pyx_INCREF(__pyx_t_9);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_t_2};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2248, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_atom = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2251
 * 
 *     # Get info on dimensions & types (of base class)
 *     H5VLARRAYget_info(self.dataset_id, self.disk_type_id, &nrecords,             # <<<<<<<<<<<<<<
 *                       cbyteorder)
 * 
*/
  (void)(H5VLARRAYget_info(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.disk_type_id, (&__pyx_v_nrecords), __pyx_v_cbyteorder));

  /* "tables/hdf5extension.pyx":2254
 *                       cbyteorder)
 * 
 *     byteorder = cstr_to_pystr(cbyteorder)             # <<<<<<<<<<<<<<
 * 
 *     # Get some properties of the atomic type
*/
  __pyx_t_1 = __pyx_f_6tables_14utilsextension_cstr_to_pystr(__pyx_v_cbyteorder); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_byteorder = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2257
 * 
 *     # Get some properties of the atomic type
 *     self._atomicdtype = atom.dtype             # <<<<<<<<<<<<<<
 *     self._atomictype = atom.type
 *     self._atomicshape = atom.shape
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atomicdtype, __pyx_t_1) < (0)) __PYX_ERR(0, 2257, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2258
 *     # Get some properties of the atomic type
 *     self._atomicdtype = atom.dtype
 *     self._atomictype = atom.type             # <<<<<<<<<<<<<<
 *     self._atomicshape = atom.shape
 *     self._atomicsize = atom.size
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atomictype, __pyx_t_1) < (0)) __PYX_ERR(0, 2258, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2259
 *     self._atomicdtype = atom.dtype
 *     self._atomictype = atom.type
 *     self._atomicshape = atom.shape             # <<<<<<<<<<<<<<
 *     self._atomicsize = atom.size
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atomicshape, __pyx_t_1) < (0)) __PYX_ERR(0, 2259, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2260
 *     self._atomictype = atom.type
 *     self._atomicshape = atom.shape
 *     self._atomicsize = atom.size             # <<<<<<<<<<<<<<
 * 
 *     # Get the byteorder
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atomicsize, __pyx_t_1) < (0)) __PYX_ERR(0, 2260, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2263
 * 
 *     # Get the byteorder
 *     self.byteorder = correct_byteorder(atom.type, byteorder)             # <<<<<<<<<<<<<<
 * 
 *     # Get the chunkshape (VLArrays are unidimensional entities)
*/
  __pyx_t_6 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_correct_byteorder); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_atom, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_6);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_6);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_6, __pyx_t_9, __pyx_v_byteorder};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2263, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_byteorder, __pyx_t_1) < (0)) __PYX_ERR(0, 2263, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2266
 * 
 *     # Get the chunkshape (VLArrays are unidimensional entities)
 *     H5ARRAYget_chunkshape(self.dataset_id, 1, &chunksize)             # <<<<<<<<<<<<<<
 * 
 *     self.nrecords = nrecords  # Initialize the number of records saved
*/
  (void)(H5ARRAYget_chunkshape(__pyx_v_self->__pyx_base.dataset_id, 1, (&__pyx_v_chunksize)));

  /* "tables/hdf5extension.pyx":2268
 *     H5ARRAYget_chunkshape(self.dataset_id, 1, &chunksize)
 * 
 *     self.nrecords = nrecords  # Initialize the number of records saved             # <<<<<<<<<<<<<<
 *     return self.dataset_id, SizeType(nrecords), (SizeType(chunksize),), atom
 * 
*/
  __pyx_v_self->nrecords = __pyx_v_nrecords;

  /* "tables/hdf5extension.pyx":2269
 * 
 *     self.nrecords = nrecords  # Initialize the number of records saved
 *     return self.dataset_id, SizeType(nrecords), (SizeType(chunksize),), atom             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_hid_t(__pyx_v_self->__pyx_base.dataset_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_SizeType); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_8 = __Pyx_PyLong_From_hsize_t(__pyx_v_nrecords); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_6);
    assert(__pyx_t_9);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
    __Pyx_INCREF(__pyx_t_9);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_t_8};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2269, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_t_8 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_SizeType); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_7 = __Pyx_PyLong_From_hsize_t(__pyx_v_chunksize); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_9);
    assert(__pyx_t_8);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
    __Pyx_INCREF(__pyx_t_8);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_t_7};
    __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2269, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
  }
  __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_6);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_6) != (0)) __PYX_ERR(0, 2269, __pyx_L1_error);
  __pyx_t_6 = 0;
  __pyx_t_6 = PyTuple_New(4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1) != (0)) __PYX_ERR(0, 2269, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_2) != (0)) __PYX_ERR(0, 2269, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_9);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_9) != (0)) __PYX_ERR(0, 2269, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_atom);
  __Pyx_GIVEREF(__pyx_v_atom);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_v_atom) != (0)) __PYX_ERR(0, 2269, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_9 = 0;
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":2228
 * 
 * 
 *   def _open_array(self):             # <<<<<<<<<<<<<<
 *     cdef char cbyteorder[11]  # "irrelevant" fits easily here
 *     cdef int i, enumtype
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("tables.hdf5extension.VLArray._open_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_encoded_name);
  __Pyx_XDECREF(__pyx_v_byteorder);
  __Pyx_XDECREF(__pyx_v_atom);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":2272
 * 
 * 
 *   def _append(self, ndarray nparr, int nobjects):             # <<<<<<<<<<<<<<
 *     cdef int ret
 *     cdef void *rbuf
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_7VLArray_5_append(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_7VLArray_5_append = {"_append", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7VLArray_5_append, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_7VLArray_5_append(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyArrayObject *__pyx_v_nparr = 0;
  int __pyx_v_nobjects;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_append (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_nparr,&__pyx_mstate_global->__pyx_n_u_nobjects,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2272, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2272, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2272, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_append", 0) < (0)) __PYX_ERR(0, 2272, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_append", 1, 2, 2, i); __PYX_ERR(0, 2272, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2272, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2272, __pyx_L3_error)
    }
    __pyx_v_nparr = ((PyArrayObject *)values[0]);
    __pyx_v_nobjects = __Pyx_PyLong_As_int(values[1]); if (unlikely((__pyx_v_nobjects == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2272, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_append", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 2272, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.VLArray._append", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_nparr), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "nparr", 0))) __PYX_ERR(0, 2272, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_13hdf5extension_7VLArray_4_append(((struct __pyx_obj_6tables_13hdf5extension_VLArray *)__pyx_v_self), __pyx_v_nparr, __pyx_v_nobjects);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_7VLArray_4_append(struct __pyx_obj_6tables_13hdf5extension_VLArray *__pyx_v_self, PyArrayObject *__pyx_v_nparr, int __pyx_v_nobjects) {
  int __pyx_v_ret;
  void *__pyx_v_rbuf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_append", 0);

  /* "tables/hdf5extension.pyx":2277
 * 
 *     # Get the pointer to the buffer data area
 *     if nobjects:             # <<<<<<<<<<<<<<
 *       rbuf = PyArray_DATA(nparr)
 *       # Convert some NumPy types to HDF5 before storing.
*/
  __pyx_t_1 = (__pyx_v_nobjects != 0);
  if (__pyx_t_1) {

    /* "tables/hdf5extension.pyx":2278
 *     # Get the pointer to the buffer data area
 *     if nobjects:
 *       rbuf = PyArray_DATA(nparr)             # <<<<<<<<<<<<<<
 *       # Convert some NumPy types to HDF5 before storing.
 *       if self.atom.type == 'time64':
*/
    __pyx_v_rbuf = PyArray_DATA(__pyx_v_nparr);

    /* "tables/hdf5extension.pyx":2280
 *       rbuf = PyArray_DATA(nparr)
 *       # Convert some NumPy types to HDF5 before storing.
 *       if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *         self._convert_time64(nparr, 0)
 *     else:
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2280, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2280, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_time64, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 2280, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_1) {

      /* "tables/hdf5extension.pyx":2281
 *       # Convert some NumPy types to HDF5 before storing.
 *       if self.atom.type == 'time64':
 *         self._convert_time64(nparr, 0)             # <<<<<<<<<<<<<<
 *     else:
 *       rbuf = NULL
*/
      __pyx_t_3 = ((struct __pyx_vtabstruct_6tables_13hdf5extension_VLArray *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._convert_time64(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self), __pyx_v_nparr, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2281, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "tables/hdf5extension.pyx":2280
 *       rbuf = PyArray_DATA(nparr)
 *       # Convert some NumPy types to HDF5 before storing.
 *       if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *         self._convert_time64(nparr, 0)
 *     else:
*/
    }

    /* "tables/hdf5extension.pyx":2277
 * 
 *     # Get the pointer to the buffer data area
 *     if nobjects:             # <<<<<<<<<<<<<<
 *       rbuf = PyArray_DATA(nparr)
 *       # Convert some NumPy types to HDF5 before storing.
*/
    goto __pyx_L3;
  }

  /* "tables/hdf5extension.pyx":2283
 *         self._convert_time64(nparr, 0)
 *     else:
 *       rbuf = NULL             # <<<<<<<<<<<<<<
 * 
 *     # Append the records:
*/
  /*else*/ {
    __pyx_v_rbuf = NULL;
  }
  __pyx_L3:;

  /* "tables/hdf5extension.pyx":2286
 * 
 *     # Append the records:
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5VLARRAYappend_records(self.dataset_id, self.type_id,
 *                                       nobjects, self.nrecords, rbuf)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "tables/hdf5extension.pyx":2287
 *     # Append the records:
 *     with nogil:
 *         ret = H5VLARRAYappend_records(self.dataset_id, self.type_id,             # <<<<<<<<<<<<<<
 *                                       nobjects, self.nrecords, rbuf)
 * 
*/
        __pyx_v_ret = H5VLARRAYappend_records(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_nobjects, __pyx_v_self->nrecords, __pyx_v_rbuf);
      }

      /* "tables/hdf5extension.pyx":2286
 * 
 *     # Append the records:
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5VLARRAYappend_records(self.dataset_id, self.type_id,
 *                                       nobjects, self.nrecords, rbuf)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "tables/hdf5extension.pyx":2290
 *                                       nobjects, self.nrecords, rbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems appending the records.")
 * 
*/
  __pyx_t_1 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/hdf5extension.pyx":2291
 * 
 *     if ret < 0:
 *       raise HDF5ExtError("Problems appending the records.")             # <<<<<<<<<<<<<<
 * 
 *     self.nrecords = self.nrecords + 1
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2291, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_Problems_appending_the_records};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2291, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 2291, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2290
 *                                       nobjects, self.nrecords, rbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems appending the records.")
 * 
*/
  }

  /* "tables/hdf5extension.pyx":2293
 *       raise HDF5ExtError("Problems appending the records.")
 * 
 *     self.nrecords = self.nrecords + 1             # <<<<<<<<<<<<<<
 * 
 *   def _modify(self, hsize_t nrow, ndarray nparr, int nobjects):
*/
  __pyx_v_self->nrecords = (__pyx_v_self->nrecords + 1);

  /* "tables/hdf5extension.pyx":2272
 * 
 * 
 *   def _append(self, ndarray nparr, int nobjects):             # <<<<<<<<<<<<<<
 *     cdef int ret
 *     cdef void *rbuf
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("tables.hdf5extension.VLArray._append", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":2295
 *     self.nrecords = self.nrecords + 1
 * 
 *   def _modify(self, hsize_t nrow, ndarray nparr, int nobjects):             # <<<<<<<<<<<<<<
 *     cdef int ret
 *     cdef void *rbuf
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_7VLArray_7_modify(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_7VLArray_7_modify = {"_modify", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7VLArray_7_modify, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_7VLArray_7_modify(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  hsize_t __pyx_v_nrow;
  PyArrayObject *__pyx_v_nparr = 0;
  int __pyx_v_nobjects;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_modify (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_nrow,&__pyx_mstate_global->__pyx_n_u_nparr,&__pyx_mstate_global->__pyx_n_u_nobjects,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2295, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2295, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2295, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2295, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_modify", 0) < (0)) __PYX_ERR(0, 2295, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_modify", 1, 3, 3, i); __PYX_ERR(0, 2295, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2295, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2295, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2295, __pyx_L3_error)
    }
    __pyx_v_nrow = __Pyx_PyLong_As_hsize_t(values[0]); if (unlikely((__pyx_v_nrow == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2295, __pyx_L3_error)
    __pyx_v_nparr = ((PyArrayObject *)values[1]);
    __pyx_v_nobjects = __Pyx_PyLong_As_int(values[2]); if (unlikely((__pyx_v_nobjects == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2295, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_modify", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 2295, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.VLArray._modify", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_nparr), __pyx_mstate_global->__pyx_ptype_5numpy_ndarray, 1, "nparr", 0))) __PYX_ERR(0, 2295, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_13hdf5extension_7VLArray_6_modify(((struct __pyx_obj_6tables_13hdf5extension_VLArray *)__pyx_v_self), __pyx_v_nrow, __pyx_v_nparr, __pyx_v_nobjects);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_7VLArray_6_modify(struct __pyx_obj_6tables_13hdf5extension_VLArray *__pyx_v_self, hsize_t __pyx_v_nrow, PyArrayObject *__pyx_v_nparr, int __pyx_v_nobjects) {
  int __pyx_v_ret;
  void *__pyx_v_rbuf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_modify", 0);

  /* "tables/hdf5extension.pyx":2300
 * 
 *     # Get the pointer to the buffer data area
 *     rbuf = PyArray_DATA(nparr)             # <<<<<<<<<<<<<<
 *     if nobjects:
 *       # Convert some NumPy types to HDF5 before storing.
*/
  __pyx_v_rbuf = PyArray_DATA(__pyx_v_nparr);

  /* "tables/hdf5extension.pyx":2301
 *     # Get the pointer to the buffer data area
 *     rbuf = PyArray_DATA(nparr)
 *     if nobjects:             # <<<<<<<<<<<<<<
 *       # Convert some NumPy types to HDF5 before storing.
 *       if self.atom.type == 'time64':
*/
  __pyx_t_1 = (__pyx_v_nobjects != 0);
  if (__pyx_t_1) {

    /* "tables/hdf5extension.pyx":2303
 *     if nobjects:
 *       # Convert some NumPy types to HDF5 before storing.
 *       if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *         self._convert_time64(nparr, 0)
 * 
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2303, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2303, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_time64, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 2303, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_1) {

      /* "tables/hdf5extension.pyx":2304
 *       # Convert some NumPy types to HDF5 before storing.
 *       if self.atom.type == 'time64':
 *         self._convert_time64(nparr, 0)             # <<<<<<<<<<<<<<
 * 
 *     # Append the records:
*/
      __pyx_t_3 = ((struct __pyx_vtabstruct_6tables_13hdf5extension_VLArray *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._convert_time64(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self), __pyx_v_nparr, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2304, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "tables/hdf5extension.pyx":2303
 *     if nobjects:
 *       # Convert some NumPy types to HDF5 before storing.
 *       if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *         self._convert_time64(nparr, 0)
 * 
*/
    }

    /* "tables/hdf5extension.pyx":2301
 *     # Get the pointer to the buffer data area
 *     rbuf = PyArray_DATA(nparr)
 *     if nobjects:             # <<<<<<<<<<<<<<
 *       # Convert some NumPy types to HDF5 before storing.
 *       if self.atom.type == 'time64':
*/
  }

  /* "tables/hdf5extension.pyx":2307
 * 
 *     # Append the records:
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5VLARRAYmodify_records(self.dataset_id, self.type_id,
 *                                       nrow, nobjects, rbuf)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "tables/hdf5extension.pyx":2308
 *     # Append the records:
 *     with nogil:
 *         ret = H5VLARRAYmodify_records(self.dataset_id, self.type_id,             # <<<<<<<<<<<<<<
 *                                       nrow, nobjects, rbuf)
 * 
*/
        __pyx_v_ret = H5VLARRAYmodify_records(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_nrow, __pyx_v_nobjects, __pyx_v_rbuf);
      }

      /* "tables/hdf5extension.pyx":2307
 * 
 *     # Append the records:
 *     with nogil:             # <<<<<<<<<<<<<<
 *         ret = H5VLARRAYmodify_records(self.dataset_id, self.type_id,
 *                                       nrow, nobjects, rbuf)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "tables/hdf5extension.pyx":2311
 *                                       nrow, nobjects, rbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems modifying the record.")
 * 
*/
  __pyx_t_1 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_1)) {

    /* "tables/hdf5extension.pyx":2312
 * 
 *     if ret < 0:
 *       raise HDF5ExtError("Problems modifying the record.")             # <<<<<<<<<<<<<<
 * 
 *     return nobjects
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2312, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_Problems_modifying_the_record};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2312, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 2312, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2311
 *                                       nrow, nobjects, rbuf)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("Problems modifying the record.")
 * 
*/
  }

  /* "tables/hdf5extension.pyx":2314
 *       raise HDF5ExtError("Problems modifying the record.")
 * 
 *     return nobjects             # <<<<<<<<<<<<<<
 * 
 *   # Because the size of each "row" is unknown, there is no easy way to
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyLong_From_int(__pyx_v_nobjects); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":2295
 *     self.nrecords = self.nrecords + 1
 * 
 *   def _modify(self, hsize_t nrow, ndarray nparr, int nobjects):             # <<<<<<<<<<<<<<
 *     cdef int ret
 *     cdef void *rbuf
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("tables.hdf5extension.VLArray._modify", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":2318
 *   # Because the size of each "row" is unknown, there is no easy way to
 *   # calculate this value
 *   def _get_memory_size(self):             # <<<<<<<<<<<<<<
 *     cdef hid_t space_id
 *     cdef hsize_t size
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_7VLArray_9_get_memory_size(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_7VLArray_9_get_memory_size = {"_get_memory_size", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7VLArray_9_get_memory_size, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_7VLArray_9_get_memory_size(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_memory_size (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_get_memory_size", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_get_memory_size", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_7VLArray_8_get_memory_size(((struct __pyx_obj_6tables_13hdf5extension_VLArray *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_7VLArray_8_get_memory_size(struct __pyx_obj_6tables_13hdf5extension_VLArray *__pyx_v_self) {
  hid_t __pyx_v_space_id;
  hsize_t __pyx_v_size;
  herr_t __pyx_v_ret;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_memory_size", 0);

  /* "tables/hdf5extension.pyx":2323
 *     cdef herr_t ret
 * 
 *     if self.nrows == 0:             # <<<<<<<<<<<<<<
 *       size = 0
 *     else:
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_nrows); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__Pyx_PyLong_BoolEqObjC(__pyx_t_1, __pyx_mstate_global->__pyx_int_0, 0, 0)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 2323, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "tables/hdf5extension.pyx":2324
 * 
 *     if self.nrows == 0:
 *       size = 0             # <<<<<<<<<<<<<<
 *     else:
 *       # Get the dataspace handle
*/
    __pyx_v_size = 0;

    /* "tables/hdf5extension.pyx":2323
 *     cdef herr_t ret
 * 
 *     if self.nrows == 0:             # <<<<<<<<<<<<<<
 *       size = 0
 *     else:
*/
    goto __pyx_L3;
  }

  /* "tables/hdf5extension.pyx":2327
 *     else:
 *       # Get the dataspace handle
 *       space_id = H5Dget_space(self.dataset_id)             # <<<<<<<<<<<<<<
 *       # Return the size of the entire dataset
 *       ret = H5Dvlen_get_buf_size(self.dataset_id, self.type_id, space_id,
*/
  /*else*/ {
    __pyx_v_space_id = H5Dget_space(__pyx_v_self->__pyx_base.dataset_id);

    /* "tables/hdf5extension.pyx":2329
 *       space_id = H5Dget_space(self.dataset_id)
 *       # Return the size of the entire dataset
 *       ret = H5Dvlen_get_buf_size(self.dataset_id, self.type_id, space_id,             # <<<<<<<<<<<<<<
 *                                  &size)
 *       if ret < 0:
*/
    __pyx_v_ret = H5Dvlen_get_buf_size(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_space_id, (&__pyx_v_size));

    /* "tables/hdf5extension.pyx":2331
 *       ret = H5Dvlen_get_buf_size(self.dataset_id, self.type_id, space_id,
 *                                  &size)
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         size = -1
 * 
*/
    __pyx_t_2 = (__pyx_v_ret < 0);
    if (__pyx_t_2) {

      /* "tables/hdf5extension.pyx":2332
 *                                  &size)
 *       if ret < 0:
 *         size = -1             # <<<<<<<<<<<<<<
 * 
 *       # Terminate access to the dataspace
*/
      __pyx_v_size = -1LL;

      /* "tables/hdf5extension.pyx":2331
 *       ret = H5Dvlen_get_buf_size(self.dataset_id, self.type_id, space_id,
 *                                  &size)
 *       if ret < 0:             # <<<<<<<<<<<<<<
 *         size = -1
 * 
*/
    }

    /* "tables/hdf5extension.pyx":2335
 * 
 *       # Terminate access to the dataspace
 *       H5Sclose(space_id)             # <<<<<<<<<<<<<<
 * 
 *     return size
*/
    (void)(H5Sclose(__pyx_v_space_id));
  }
  __pyx_L3:;

  /* "tables/hdf5extension.pyx":2337
 *       H5Sclose(space_id)
 * 
 *     return size             # <<<<<<<<<<<<<<
 * 
 *   def _read_array(self, hsize_t start, hsize_t stop, hsize_t step):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_hsize_t(__pyx_v_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":2318
 *   # Because the size of each "row" is unknown, there is no easy way to
 *   # calculate this value
 *   def _get_memory_size(self):             # <<<<<<<<<<<<<<
 *     cdef hid_t space_id
 *     cdef hsize_t size
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.hdf5extension.VLArray._get_memory_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":2339
 *     return size
 * 
 *   def _read_array(self, hsize_t start, hsize_t stop, hsize_t step):             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef size_t vllen
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_7VLArray_11_read_array(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_7VLArray_11_read_array = {"_read_array", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7VLArray_11_read_array, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_7VLArray_11_read_array(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  hsize_t __pyx_v_start;
  hsize_t __pyx_v_stop;
  hsize_t __pyx_v_step;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_read_array (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_start,&__pyx_mstate_global->__pyx_n_u_stop,&__pyx_mstate_global->__pyx_n_u_step,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2339, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2339, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2339, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2339, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_read_array", 0) < (0)) __PYX_ERR(0, 2339, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_read_array", 1, 3, 3, i); __PYX_ERR(0, 2339, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2339, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2339, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2339, __pyx_L3_error)
    }
    __pyx_v_start = __Pyx_PyLong_As_hsize_t(values[0]); if (unlikely((__pyx_v_start == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2339, __pyx_L3_error)
    __pyx_v_stop = __Pyx_PyLong_As_hsize_t(values[1]); if (unlikely((__pyx_v_stop == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2339, __pyx_L3_error)
    __pyx_v_step = __Pyx_PyLong_As_hsize_t(values[2]); if (unlikely((__pyx_v_step == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2339, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_read_array", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 2339, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.VLArray._read_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_7VLArray_10_read_array(((struct __pyx_obj_6tables_13hdf5extension_VLArray *)__pyx_v_self), __pyx_v_start, __pyx_v_stop, __pyx_v_step);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_7VLArray_10_read_array(struct __pyx_obj_6tables_13hdf5extension_VLArray *__pyx_v_self, hsize_t __pyx_v_start, hsize_t __pyx_v_stop, hsize_t __pyx_v_step) {
  int __pyx_v_i;
  size_t __pyx_v_vllen;
  herr_t __pyx_v_ret;
  hvl_t *__pyx_v_rdata;
  hsize_t __pyx_v_nrows;
  hid_t __pyx_v_space_id;
  hid_t __pyx_v_mem_space_id;
  PyObject *__pyx_v_buf = 0;
  PyObject *__pyx_v_nparr = 0;
  PyObject *__pyx_v_shape = 0;
  PyObject *__pyx_v_datalist = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PY_LONG_LONG __pyx_t_7;
  PY_LONG_LONG __pyx_t_8;
  int __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_read_array", 0);

  /* "tables/hdf5extension.pyx":2350
 * 
 *     # Compute the number of rows to read
 *     nrows = get_len_of_range(start, stop, step)             # <<<<<<<<<<<<<<
 *     if start + nrows > self.nrows:
 *       raise HDF5ExtError(
*/
  __pyx_v_nrows = get_len_of_range(__pyx_v_start, __pyx_v_stop, __pyx_v_step);

  /* "tables/hdf5extension.pyx":2351
 *     # Compute the number of rows to read
 *     nrows = get_len_of_range(start, stop, step)
 *     if start + nrows > self.nrows:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError(
 *         "Asking for a range of rows exceeding the available ones!.",
*/
  __pyx_t_1 = __Pyx_PyLong_From_hsize_t((__pyx_v_start + __pyx_v_nrows)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_nrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2351, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 2351, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(__pyx_t_4)) {

    /* "tables/hdf5extension.pyx":2352
 *     nrows = get_len_of_range(start, stop, step)
 *     if start + nrows > self.nrows:
 *       raise HDF5ExtError(             # <<<<<<<<<<<<<<
 *         "Asking for a range of rows exceeding the available ones!.",
 *         h5bt=False)
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2352, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "tables/hdf5extension.pyx":2354
 *       raise HDF5ExtError(
 *         "Asking for a range of rows exceeding the available ones!.",
 *         h5bt=False)             # <<<<<<<<<<<<<<
 * 
 *     # Now, read the chunk of rows
*/
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_Asking_for_a_range_of_rows_excee};
      __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2352, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_h5bt, Py_False, __pyx_t_6, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 2352, __pyx_L1_error)
      __pyx_t_3 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2352, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 2352, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2351
 *     # Compute the number of rows to read
 *     nrows = get_len_of_range(start, stop, step)
 *     if start + nrows > self.nrows:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError(
 *         "Asking for a range of rows exceeding the available ones!.",
*/
  }

  /* "tables/hdf5extension.pyx":2357
 * 
 *     # Now, read the chunk of rows
 *     with nogil:             # <<<<<<<<<<<<<<
 *         # Allocate the necessary memory for keeping the row handlers
 *         rdata = <hvl_t *>malloc(<size_t>nrows*sizeof(hvl_t))
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "tables/hdf5extension.pyx":2359
 *     with nogil:
 *         # Allocate the necessary memory for keeping the row handlers
 *         rdata = <hvl_t *>malloc(<size_t>nrows*sizeof(hvl_t))             # <<<<<<<<<<<<<<
 *         # Get the dataspace handle
 *         space_id = H5Dget_space(self.dataset_id)
*/
        __pyx_v_rdata = ((hvl_t *)malloc((((size_t)__pyx_v_nrows) * (sizeof(hvl_t)))));

        /* "tables/hdf5extension.pyx":2361
 *         rdata = <hvl_t *>malloc(<size_t>nrows*sizeof(hvl_t))
 *         # Get the dataspace handle
 *         space_id = H5Dget_space(self.dataset_id)             # <<<<<<<<<<<<<<
 *         # Create a memory dataspace handle
 *         mem_space_id = H5Screate_simple(1, &nrows, NULL)
*/
        __pyx_v_space_id = H5Dget_space(__pyx_v_self->__pyx_base.dataset_id);

        /* "tables/hdf5extension.pyx":2363
 *         space_id = H5Dget_space(self.dataset_id)
 *         # Create a memory dataspace handle
 *         mem_space_id = H5Screate_simple(1, &nrows, NULL)             # <<<<<<<<<<<<<<
 *         # Select the data to be read
 *         H5Sselect_hyperslab(space_id, H5S_SELECT_SET, &start, &step, &nrows,
*/
        __pyx_v_mem_space_id = H5Screate_simple(1, (&__pyx_v_nrows), NULL);

        /* "tables/hdf5extension.pyx":2365
 *         mem_space_id = H5Screate_simple(1, &nrows, NULL)
 *         # Select the data to be read
 *         H5Sselect_hyperslab(space_id, H5S_SELECT_SET, &start, &step, &nrows,             # <<<<<<<<<<<<<<
 *                             NULL)
 *         # Do the actual read
*/
        (void)(H5Sselect_hyperslab(__pyx_v_space_id, H5S_SELECT_SET, (&__pyx_v_start), (&__pyx_v_step), (&__pyx_v_nrows), NULL));

        /* "tables/hdf5extension.pyx":2368
 *                             NULL)
 *         # Do the actual read
 *         ret = H5Dread(self.dataset_id, self.type_id, mem_space_id, space_id,             # <<<<<<<<<<<<<<
 *                       H5P_DEFAULT, rdata)
 * 
*/
        __pyx_v_ret = H5Dread(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_mem_space_id, __pyx_v_space_id, H5P_DEFAULT, __pyx_v_rdata);
      }

      /* "tables/hdf5extension.pyx":2357
 * 
 *     # Now, read the chunk of rows
 *     with nogil:             # <<<<<<<<<<<<<<
 *         # Allocate the necessary memory for keeping the row handlers
 *         rdata = <hvl_t *>malloc(<size_t>nrows*sizeof(hvl_t))
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "tables/hdf5extension.pyx":2371
 *                       H5P_DEFAULT, rdata)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError(
 *         "VLArray._read_array: Problems reading the array data.")
*/
  __pyx_t_4 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_4)) {

    /* "tables/hdf5extension.pyx":2372
 * 
 *     if ret < 0:
 *       raise HDF5ExtError(             # <<<<<<<<<<<<<<
 *         "VLArray._read_array: Problems reading the array data.")
 * 
*/
    __pyx_t_1 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2372, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_1);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_VLArray__read_array_Problems_rea};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2372, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 2372, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2371
 *                       H5P_DEFAULT, rdata)
 * 
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError(
 *         "VLArray._read_array: Problems reading the array data.")
*/
  }

  /* "tables/hdf5extension.pyx":2375
 *         "VLArray._read_array: Problems reading the array data.")
 * 
 *     datalist = []             # <<<<<<<<<<<<<<
 *     for i in range(<long long>nrows):
 *       # Number of atoms in row
*/
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2375, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_datalist = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "tables/hdf5extension.pyx":2376
 * 
 *     datalist = []
 *     for i in range(<long long>nrows):             # <<<<<<<<<<<<<<
 *       # Number of atoms in row
 *       vllen = rdata[i].len
*/
  __pyx_t_7 = ((PY_LONG_LONG)__pyx_v_nrows);
  __pyx_t_8 = __pyx_t_7;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_i = __pyx_t_9;

    /* "tables/hdf5extension.pyx":2378
 *     for i in range(<long long>nrows):
 *       # Number of atoms in row
 *       vllen = rdata[i].len             # <<<<<<<<<<<<<<
 *       # Get the pointer to the buffer data area
 *       if vllen > 0:
*/
    __pyx_t_5 = (__pyx_v_rdata[__pyx_v_i]).len;
    __pyx_v_vllen = __pyx_t_5;

    /* "tables/hdf5extension.pyx":2380
 *       vllen = rdata[i].len
 *       # Get the pointer to the buffer data area
 *       if vllen > 0:             # <<<<<<<<<<<<<<
 *         # Create a buffer to keep this info. It is important to do a
 *         # copy, because we will dispose the buffer memory later on by
*/
    __pyx_t_4 = (__pyx_v_vllen > 0);
    if (__pyx_t_4) {

      /* "tables/hdf5extension.pyx":2385
 *         # calling the H5Dvlen_reclaim. PyByteArray_FromStringAndSize does this.
 *         buf = PyByteArray_FromStringAndSize(<char *>rdata[i].p,
 *                                             vllen*self._atomicsize)             # <<<<<<<<<<<<<<
 *       else:
 *         # Case where there is info with zero lentgh
*/
      __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_vllen); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2385, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atomicsize); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2385, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_1 = PyNumber_Multiply(__pyx_t_3, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2385, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 2385, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "tables/hdf5extension.pyx":2384
 *         # copy, because we will dispose the buffer memory later on by
 *         # calling the H5Dvlen_reclaim. PyByteArray_FromStringAndSize does this.
 *         buf = PyByteArray_FromStringAndSize(<char *>rdata[i].p,             # <<<<<<<<<<<<<<
 *                                             vllen*self._atomicsize)
 *       else:
*/
      __pyx_t_1 = PyByteArray_FromStringAndSize(((char *)(__pyx_v_rdata[__pyx_v_i]).p), __pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2384, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_buf, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "tables/hdf5extension.pyx":2380
 *       vllen = rdata[i].len
 *       # Get the pointer to the buffer data area
 *       if vllen > 0:             # <<<<<<<<<<<<<<
 *         # Create a buffer to keep this info. It is important to do a
 *         # copy, because we will dispose the buffer memory later on by
*/
      goto __pyx_L10;
    }

    /* "tables/hdf5extension.pyx":2388
 *       else:
 *         # Case where there is info with zero lentgh
 *         buf = None             # <<<<<<<<<<<<<<
 *       # Compute the shape for the read array
 *       shape = list(self._atomicshape)
*/
    /*else*/ {
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_buf, Py_None);
    }
    __pyx_L10:;

    /* "tables/hdf5extension.pyx":2390
 *         buf = None
 *       # Compute the shape for the read array
 *       shape = list(self._atomicshape)             # <<<<<<<<<<<<<<
 *       shape.insert(0, vllen)  # put the length at the beginning of the shape
 *       nparr = np.ndarray(
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atomicshape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2390, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PySequence_ListKeepNew(__pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2390, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF_SET(__pyx_v_shape, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "tables/hdf5extension.pyx":2391
 *       # Compute the shape for the read array
 *       shape = list(self._atomicshape)
 *       shape.insert(0, vllen)  # put the length at the beginning of the shape             # <<<<<<<<<<<<<<
 *       nparr = np.ndarray(
 *         buffer=buf, dtype=self._atomicdtype.base, shape=shape)
*/
    __pyx_t_1 = __pyx_v_shape;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_vllen); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2391, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_1, __pyx_mstate_global->__pyx_int_0, __pyx_t_3};
      __pyx_t_6 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_insert, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2391, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "tables/hdf5extension.pyx":2392
 *       shape = list(self._atomicshape)
 *       shape.insert(0, vllen)  # put the length at the beginning of the shape
 *       nparr = np.ndarray(             # <<<<<<<<<<<<<<
 *         buffer=buf, dtype=self._atomicdtype.base, shape=shape)
 *       # Set the writeable flag for this ndarray object
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2392, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ndarray); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2392, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "tables/hdf5extension.pyx":2393
 *       shape.insert(0, vllen)  # put the length at the beginning of the shape
 *       nparr = np.ndarray(
 *         buffer=buf, dtype=self._atomicdtype.base, shape=shape)             # <<<<<<<<<<<<<<
 *       # Set the writeable flag for this ndarray object
 *       nparr.flags.writeable = True
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atomicdtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_base); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 2393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 3 : 0)] = {__pyx_t_3, NULL};
      __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2392, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_buffer, __pyx_v_buf, __pyx_t_1, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 2392, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_11, __pyx_t_1, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 2392, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_shape, __pyx_v_shape, __pyx_t_1, __pyx_callargs+1, 2) < (0)) __PYX_ERR(0, 2392, __pyx_L1_error)
      __pyx_t_6 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2392, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
    }
    __Pyx_XDECREF_SET(__pyx_v_nparr, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "tables/hdf5extension.pyx":2395
 *         buffer=buf, dtype=self._atomicdtype.base, shape=shape)
 *       # Set the writeable flag for this ndarray object
 *       nparr.flags.writeable = True             # <<<<<<<<<<<<<<
 *       if self.atom.kind == 'time':
 *         # Swap the byteorder by hand (this is not currently supported by HDF5)
*/
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_nparr, __pyx_mstate_global->__pyx_n_u_flags); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2395, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__Pyx_PyObject_SetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_writeable, Py_True) < (0)) __PYX_ERR(0, 2395, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "tables/hdf5extension.pyx":2396
 *       # Set the writeable flag for this ndarray object
 *       nparr.flags.writeable = True
 *       if self.atom.kind == 'time':             # <<<<<<<<<<<<<<
 *         # Swap the byteorder by hand (this is not currently supported by HDF5)
 *         if H5Tget_order(self.type_id) != platform_byteorder:
*/
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2396, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2396, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_time, Py_EQ)); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 2396, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_4) {

      /* "tables/hdf5extension.pyx":2398
 *       if self.atom.kind == 'time':
 *         # Swap the byteorder by hand (this is not currently supported by HDF5)
 *         if H5Tget_order(self.type_id) != platform_byteorder:             # <<<<<<<<<<<<<<
 *           nparr.byteswap(True)
 *       # Convert some HDF5 types to NumPy after reading.
*/
      __pyx_t_2 = __Pyx_PyLong_From_enum__H5T_order_t(H5Tget_order(__pyx_v_self->__pyx_base.type_id)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2398, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_platform_byteorder); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2398, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_1 = PyObject_RichCompare(__pyx_t_2, __pyx_t_6, Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2398, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 2398, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (__pyx_t_4) {

        /* "tables/hdf5extension.pyx":2399
 *         # Swap the byteorder by hand (this is not currently supported by HDF5)
 *         if H5Tget_order(self.type_id) != platform_byteorder:
 *           nparr.byteswap(True)             # <<<<<<<<<<<<<<
 *       # Convert some HDF5 types to NumPy after reading.
 *       if self.atom.type == 'time64':
*/
        __pyx_t_6 = __pyx_v_nparr;
        __Pyx_INCREF(__pyx_t_6);
        __pyx_t_5 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_6, Py_True};
          __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_byteswap, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2399, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
        }
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "tables/hdf5extension.pyx":2398
 *       if self.atom.kind == 'time':
 *         # Swap the byteorder by hand (this is not currently supported by HDF5)
 *         if H5Tget_order(self.type_id) != platform_byteorder:             # <<<<<<<<<<<<<<
 *           nparr.byteswap(True)
 *       # Convert some HDF5 types to NumPy after reading.
*/
      }

      /* "tables/hdf5extension.pyx":2396
 *       # Set the writeable flag for this ndarray object
 *       nparr.flags.writeable = True
 *       if self.atom.kind == 'time':             # <<<<<<<<<<<<<<
 *         # Swap the byteorder by hand (this is not currently supported by HDF5)
 *         if H5Tget_order(self.type_id) != platform_byteorder:
*/
    }

    /* "tables/hdf5extension.pyx":2401
 *           nparr.byteswap(True)
 *       # Convert some HDF5 types to NumPy after reading.
 *       if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *         self._convert_time64(nparr, 1)
 *       # Append this array to the output list
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_atom); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2401, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2401, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_time64, Py_EQ)); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 2401, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (__pyx_t_4) {

      /* "tables/hdf5extension.pyx":2402
 *       # Convert some HDF5 types to NumPy after reading.
 *       if self.atom.type == 'time64':
 *         self._convert_time64(nparr, 1)             # <<<<<<<<<<<<<<
 *       # Append this array to the output list
 *       datalist.append(nparr)
*/
      if (!(likely(((__pyx_v_nparr) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_nparr, __pyx_mstate_global->__pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2402, __pyx_L1_error)
      __pyx_t_6 = ((struct __pyx_vtabstruct_6tables_13hdf5extension_VLArray *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._convert_time64(((struct __pyx_obj_6tables_13hdf5extension_Leaf *)__pyx_v_self), ((PyArrayObject *)__pyx_v_nparr), 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2402, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "tables/hdf5extension.pyx":2401
 *           nparr.byteswap(True)
 *       # Convert some HDF5 types to NumPy after reading.
 *       if self.atom.type == 'time64':             # <<<<<<<<<<<<<<
 *         self._convert_time64(nparr, 1)
 *       # Append this array to the output list
*/
    }

    /* "tables/hdf5extension.pyx":2404
 *         self._convert_time64(nparr, 1)
 *       # Append this array to the output list
 *       datalist.append(nparr)             # <<<<<<<<<<<<<<
 * 
 *     # Release resources
*/
    __pyx_t_12 = __Pyx_PyObject_Append(__pyx_v_datalist, __pyx_v_nparr); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 2404, __pyx_L1_error)
  }

  /* "tables/hdf5extension.pyx":2408
 *     # Release resources
 *     # Reclaim all the (nested) VL data
 *     ret = H5Dvlen_reclaim(self.type_id, mem_space_id, H5P_DEFAULT, rdata)             # <<<<<<<<<<<<<<
 *     if ret < 0:
 *       raise HDF5ExtError("VLArray._read_array: error freeing the data buffer.")
*/
  __pyx_v_ret = H5Dvlen_reclaim(__pyx_v_self->__pyx_base.type_id, __pyx_v_mem_space_id, H5P_DEFAULT, __pyx_v_rdata);

  /* "tables/hdf5extension.pyx":2409
 *     # Reclaim all the (nested) VL data
 *     ret = H5Dvlen_reclaim(self.type_id, mem_space_id, H5P_DEFAULT, rdata)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("VLArray._read_array: error freeing the data buffer.")
 *     # Terminate access to the memory dataspace
*/
  __pyx_t_4 = (__pyx_v_ret < 0);
  if (unlikely(__pyx_t_4)) {

    /* "tables/hdf5extension.pyx":2410
 *     ret = H5Dvlen_reclaim(self.type_id, mem_space_id, H5P_DEFAULT, rdata)
 *     if ret < 0:
 *       raise HDF5ExtError("VLArray._read_array: error freeing the data buffer.")             # <<<<<<<<<<<<<<
 *     # Terminate access to the memory dataspace
 *     H5Sclose(mem_space_id)
*/
    __pyx_t_1 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2410, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      assert(__pyx_t_1);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_VLArray__read_array_error_freein};
      __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2410, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
    }
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 2410, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2409
 *     # Reclaim all the (nested) VL data
 *     ret = H5Dvlen_reclaim(self.type_id, mem_space_id, H5P_DEFAULT, rdata)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError("VLArray._read_array: error freeing the data buffer.")
 *     # Terminate access to the memory dataspace
*/
  }

  /* "tables/hdf5extension.pyx":2412
 *       raise HDF5ExtError("VLArray._read_array: error freeing the data buffer.")
 *     # Terminate access to the memory dataspace
 *     H5Sclose(mem_space_id)             # <<<<<<<<<<<<<<
 *     # Terminate access to the dataspace
 *     H5Sclose(space_id)
*/
  (void)(H5Sclose(__pyx_v_mem_space_id));

  /* "tables/hdf5extension.pyx":2414
 *     H5Sclose(mem_space_id)
 *     # Terminate access to the dataspace
 *     H5Sclose(space_id)             # <<<<<<<<<<<<<<
 *     # Free the amount of row pointers to VL row data
 *     free(rdata)
*/
  (void)(H5Sclose(__pyx_v_space_id));

  /* "tables/hdf5extension.pyx":2416
 *     H5Sclose(space_id)
 *     # Free the amount of row pointers to VL row data
 *     free(rdata)             # <<<<<<<<<<<<<<
 * 
 *     return datalist
*/
  free(__pyx_v_rdata);

  /* "tables/hdf5extension.pyx":2418
 *     free(rdata)
 * 
 *     return datalist             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_datalist);
  __pyx_r = __pyx_v_datalist;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":2339
 *     return size
 * 
 *   def _read_array(self, hsize_t start, hsize_t stop, hsize_t step):             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef size_t vllen
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("tables.hdf5extension.VLArray._read_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_buf);
  __Pyx_XDECREF(__pyx_v_nparr);
  __Pyx_XDECREF(__pyx_v_shape);
  __Pyx_XDECREF(__pyx_v_datalist);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":2421
 * 
 * 
 *   def get_row_size(self, row):             # <<<<<<<<<<<<<<
 *     """Return the total size in bytes of all the elements contained in a given row."""
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_7VLArray_13get_row_size(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6tables_13hdf5extension_7VLArray_12get_row_size, "Return the total size in bytes of all the elements contained in a given row.");
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_7VLArray_13get_row_size = {"get_row_size", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7VLArray_13get_row_size, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_7VLArray_12get_row_size};
static PyObject *__pyx_pw_6tables_13hdf5extension_7VLArray_13get_row_size(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_row = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_row_size (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_row,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2421, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2421, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_row_size", 0) < (0)) __PYX_ERR(0, 2421, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_row_size", 1, 1, 1, i); __PYX_ERR(0, 2421, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2421, __pyx_L3_error)
    }
    __pyx_v_row = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_row_size", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 2421, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.VLArray.get_row_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_7VLArray_12get_row_size(((struct __pyx_obj_6tables_13hdf5extension_VLArray *)__pyx_v_self), __pyx_v_row);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_7VLArray_12get_row_size(struct __pyx_obj_6tables_13hdf5extension_VLArray *__pyx_v_self, PyObject *__pyx_v_row) {
  hid_t __pyx_v_space_id;
  hsize_t __pyx_v_size;
  herr_t __pyx_v_ret;
  hsize_t __pyx_v_offset[1];
  hsize_t __pyx_v_count[1];
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  hsize_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_row_size", 0);

  /* "tables/hdf5extension.pyx":2431
 *     cdef hsize_t count[1]
 * 
 *     if row >= self.nrows:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError(
 *         "Asking for a range of rows exceeding the available ones!.",
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_nrows); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2431, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_row, __pyx_t_1, Py_GE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2431, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 2431, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(__pyx_t_3)) {

    /* "tables/hdf5extension.pyx":2432
 * 
 *     if row >= self.nrows:
 *       raise HDF5ExtError(             # <<<<<<<<<<<<<<
 *         "Asking for a range of rows exceeding the available ones!.",
 *         h5bt=False)
*/
    __pyx_t_1 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_HDF5ExtError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2432, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);

    /* "tables/hdf5extension.pyx":2434
 *       raise HDF5ExtError(
 *         "Asking for a range of rows exceeding the available ones!.",
 *         h5bt=False)             # <<<<<<<<<<<<<<
 * 
 *     # Get the dataspace handle
*/
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_1);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_Asking_for_a_range_of_rows_excee};
      __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2432, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_h5bt, Py_False, __pyx_t_6, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 2432, __pyx_L1_error)
      __pyx_t_2 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2432, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 2432, __pyx_L1_error)

    /* "tables/hdf5extension.pyx":2431
 *     cdef hsize_t count[1]
 * 
 *     if row >= self.nrows:             # <<<<<<<<<<<<<<
 *       raise HDF5ExtError(
 *         "Asking for a range of rows exceeding the available ones!.",
*/
  }

  /* "tables/hdf5extension.pyx":2437
 * 
 *     # Get the dataspace handle
 *     space_id = H5Dget_space(self.dataset_id)             # <<<<<<<<<<<<<<
 * 
 *     offset[0] = row
*/
  __pyx_v_space_id = H5Dget_space(__pyx_v_self->__pyx_base.dataset_id);

  /* "tables/hdf5extension.pyx":2439
 *     space_id = H5Dget_space(self.dataset_id)
 * 
 *     offset[0] = row             # <<<<<<<<<<<<<<
 *     count[0] = 1
 * 
*/
  __pyx_t_7 = __Pyx_PyLong_As_hsize_t(__pyx_v_row); if (unlikely((__pyx_t_7 == ((hsize_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2439, __pyx_L1_error)
  (__pyx_v_offset[0]) = __pyx_t_7;

  /* "tables/hdf5extension.pyx":2440
 * 
 *     offset[0] = row
 *     count[0] = 1             # <<<<<<<<<<<<<<
 * 
 *     ret = H5Sselect_hyperslab(space_id, H5S_SELECT_SET, offset, NULL, count, NULL);
*/
  (__pyx_v_count[0]) = 1;

  /* "tables/hdf5extension.pyx":2442
 *     count[0] = 1
 * 
 *     ret = H5Sselect_hyperslab(space_id, H5S_SELECT_SET, offset, NULL, count, NULL);             # <<<<<<<<<<<<<<
 *     if ret < 0:
 *       size = -1
*/
  __pyx_v_ret = H5Sselect_hyperslab(__pyx_v_space_id, H5S_SELECT_SET, __pyx_v_offset, NULL, __pyx_v_count, NULL);

  /* "tables/hdf5extension.pyx":2443
 * 
 *     ret = H5Sselect_hyperslab(space_id, H5S_SELECT_SET, offset, NULL, count, NULL);
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       size = -1
 * 
*/
  __pyx_t_3 = (__pyx_v_ret < 0);
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":2444
 *     ret = H5Sselect_hyperslab(space_id, H5S_SELECT_SET, offset, NULL, count, NULL);
 *     if ret < 0:
 *       size = -1             # <<<<<<<<<<<<<<
 * 
 *     ret = H5Dvlen_get_buf_size(self.dataset_id, self.type_id, space_id, &size)
*/
    __pyx_v_size = -1LL;

    /* "tables/hdf5extension.pyx":2443
 * 
 *     ret = H5Sselect_hyperslab(space_id, H5S_SELECT_SET, offset, NULL, count, NULL);
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       size = -1
 * 
*/
  }

  /* "tables/hdf5extension.pyx":2446
 *       size = -1
 * 
 *     ret = H5Dvlen_get_buf_size(self.dataset_id, self.type_id, space_id, &size)             # <<<<<<<<<<<<<<
 *     if ret < 0:
 *       size = -1
*/
  __pyx_v_ret = H5Dvlen_get_buf_size(__pyx_v_self->__pyx_base.dataset_id, __pyx_v_self->__pyx_base.type_id, __pyx_v_space_id, (&__pyx_v_size));

  /* "tables/hdf5extension.pyx":2447
 * 
 *     ret = H5Dvlen_get_buf_size(self.dataset_id, self.type_id, space_id, &size)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       size = -1
 * 
*/
  __pyx_t_3 = (__pyx_v_ret < 0);
  if (__pyx_t_3) {

    /* "tables/hdf5extension.pyx":2448
 *     ret = H5Dvlen_get_buf_size(self.dataset_id, self.type_id, space_id, &size)
 *     if ret < 0:
 *       size = -1             # <<<<<<<<<<<<<<
 * 
 *     # Terminate access to the dataspace
*/
    __pyx_v_size = -1LL;

    /* "tables/hdf5extension.pyx":2447
 * 
 *     ret = H5Dvlen_get_buf_size(self.dataset_id, self.type_id, space_id, &size)
 *     if ret < 0:             # <<<<<<<<<<<<<<
 *       size = -1
 * 
*/
  }

  /* "tables/hdf5extension.pyx":2451
 * 
 *     # Terminate access to the dataspace
 *     H5Sclose(space_id)             # <<<<<<<<<<<<<<
 * 
 *     return size
*/
  (void)(H5Sclose(__pyx_v_space_id));

  /* "tables/hdf5extension.pyx":2453
 *     H5Sclose(space_id)
 * 
 *     return size             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyLong_From_hsize_t(__pyx_v_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":2421
 * 
 * 
 *   def get_row_size(self, row):             # <<<<<<<<<<<<<<
 *     """Return the total size in bytes of all the elements contained in a given row."""
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("tables.hdf5extension.VLArray.get_row_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_7VLArray_15__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_7VLArray_15__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7VLArray_15__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_7VLArray_15__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_7VLArray_14__reduce_cython__(((struct __pyx_obj_6tables_13hdf5extension_VLArray *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_7VLArray_14__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_VLArray *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self_dims_cannot_be_converted_to, 0, 0);
  __PYX_ERR(2, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("tables.hdf5extension.VLArray.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_7VLArray_17__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_7VLArray_17__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7VLArray_17__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_7VLArray_17__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(2, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(2, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(2, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(2, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.VLArray.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_7VLArray_16__setstate_cython__(((struct __pyx_obj_6tables_13hdf5extension_VLArray *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_7VLArray_16__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_VLArray *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self_dims_cannot_be_converted_to, 0, 0);
  __PYX_ERR(2, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("tables.hdf5extension.VLArray.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":2458
 * cdef class UnImplemented(Leaf):
 * 
 *   def _open_unimplemented(self):             # <<<<<<<<<<<<<<
 *     cdef object shape
 *     cdef char cbyteorder[11]  # "irrelevant" fits easily here
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_13UnImplemented_1_open_unimplemented(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_13UnImplemented_1_open_unimplemented = {"_open_unimplemented", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_13UnImplemented_1_open_unimplemented, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_13UnImplemented_1_open_unimplemented(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_open_unimplemented (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_open_unimplemented", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_open_unimplemented", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_13UnImplemented__open_unimplemented(((struct __pyx_obj_6tables_13hdf5extension_UnImplemented *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_13UnImplemented__open_unimplemented(struct __pyx_obj_6tables_13hdf5extension_UnImplemented *__pyx_v_self) {
  PyObject *__pyx_v_shape = 0;
  char __pyx_v_cbyteorder[11];
  PyObject *__pyx_v_encoded_name = 0;
  PyObject *__pyx_v_byteorder = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  char *__pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_open_unimplemented", 0);

  /* "tables/hdf5extension.pyx":2464
 *     cdef str byteorder
 * 
 *     encoded_name = self.name.encode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     # Get info on dimensions
*/
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.name;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_utf_8};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2464, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 2464, __pyx_L1_error)
  __pyx_v_encoded_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2467
 * 
 *     # Get info on dimensions
 *     shape = H5UIget_info(self.parent_id, encoded_name, cbyteorder)             # <<<<<<<<<<<<<<
 *     shape = tuple(map(SizeType, shape))
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)
*/
  if (unlikely(__pyx_v_encoded_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 2467, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_name); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 2467, __pyx_L1_error)
  __pyx_t_1 = H5UIget_info(__pyx_v_self->__pyx_base.__pyx_base.parent_id, __pyx_t_4, __pyx_v_cbyteorder); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_shape = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "tables/hdf5extension.pyx":2468
 *     # Get info on dimensions
 *     shape = H5UIget_info(self.parent_id, encoded_name, cbyteorder)
 *     shape = tuple(map(SizeType, shape))             # <<<<<<<<<<<<<<
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)
 *     byteorder = cstr_to_pystr(cbyteorder)
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_SizeType); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = 1;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_t_5, __pyx_v_shape};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_builtin_map, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2468, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_5 = __Pyx_PySequence_Tuple(__pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF_SET(__pyx_v_shape, __pyx_t_5);
  __pyx_t_5 = 0;

  /* "tables/hdf5extension.pyx":2469
 *     shape = H5UIget_info(self.parent_id, encoded_name, cbyteorder)
 *     shape = tuple(map(SizeType, shape))
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)             # <<<<<<<<<<<<<<
 *     byteorder = cstr_to_pystr(cbyteorder)
 * 
*/
  if (unlikely(__pyx_v_encoded_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 2469, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_name); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 2469, __pyx_L1_error)
  __pyx_v_self->__pyx_base.dataset_id = H5Dopen(__pyx_v_self->__pyx_base.__pyx_base.parent_id, __pyx_t_4, H5P_DEFAULT);

  /* "tables/hdf5extension.pyx":2470
 *     shape = tuple(map(SizeType, shape))
 *     self.dataset_id = H5Dopen(self.parent_id, encoded_name, H5P_DEFAULT)
 *     byteorder = cstr_to_pystr(cbyteorder)             # <<<<<<<<<<<<<<
 * 
 *     return (shape, byteorder, self.dataset_id)
*/
  __pyx_t_5 = __pyx_f_6tables_14utilsextension_cstr_to_pystr(__pyx_v_cbyteorder); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_v_byteorder = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "tables/hdf5extension.pyx":2472
 *     byteorder = cstr_to_pystr(cbyteorder)
 * 
 *     return (shape, byteorder, self.dataset_id)             # <<<<<<<<<<<<<<
 * 
 *   def _g_close(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __Pyx_PyLong_From_hid_t(__pyx_v_self->__pyx_base.dataset_id); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2472, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2472, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_shape);
  __Pyx_GIVEREF(__pyx_v_shape);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_shape) != (0)) __PYX_ERR(0, 2472, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_byteorder);
  __Pyx_GIVEREF(__pyx_v_byteorder);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_byteorder) != (0)) __PYX_ERR(0, 2472, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_5);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_t_5) != (0)) __PYX_ERR(0, 2472, __pyx_L1_error);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "tables/hdf5extension.pyx":2458
 * cdef class UnImplemented(Leaf):
 * 
 *   def _open_unimplemented(self):             # <<<<<<<<<<<<<<
 *     cdef object shape
 *     cdef char cbyteorder[11]  # "irrelevant" fits easily here
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("tables.hdf5extension.UnImplemented._open_unimplemented", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_shape);
  __Pyx_XDECREF(__pyx_v_encoded_name);
  __Pyx_XDECREF(__pyx_v_byteorder);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "tables/hdf5extension.pyx":2474
 *     return (shape, byteorder, self.dataset_id)
 * 
 *   def _g_close(self):             # <<<<<<<<<<<<<<
 *     H5Dclose(self.dataset_id)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_13UnImplemented_3_g_close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_13UnImplemented_3_g_close = {"_g_close", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_13UnImplemented_3_g_close, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_13UnImplemented_3_g_close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_close (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_g_close", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_g_close", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_13UnImplemented_2_g_close(((struct __pyx_obj_6tables_13hdf5extension_UnImplemented *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_13UnImplemented_2_g_close(struct __pyx_obj_6tables_13hdf5extension_UnImplemented *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_g_close", 0);

  /* "tables/hdf5extension.pyx":2475
 * 
 *   def _g_close(self):
 *     H5Dclose(self.dataset_id)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  (void)(H5Dclose(__pyx_v_self->__pyx_base.dataset_id));

  /* "tables/hdf5extension.pyx":2474
 *     return (shape, byteorder, self.dataset_id)
 * 
 *   def _g_close(self):             # <<<<<<<<<<<<<<
 *     H5Dclose(self.dataset_id)
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_13UnImplemented_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_13UnImplemented_5__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_13UnImplemented_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_13UnImplemented_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6tables_13hdf5extension_13UnImplemented_4__reduce_cython__(((struct __pyx_obj_6tables_13hdf5extension_UnImplemented *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_13UnImplemented_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_UnImplemented *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self_dims_cannot_be_converted_to, 0, 0);
  __PYX_ERR(2, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("tables.hdf5extension.UnImplemented.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_13UnImplemented_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_13UnImplemented_7__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_13UnImplemented_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_13UnImplemented_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(2, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(2, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(2, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(2, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.UnImplemented.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6tables_13hdf5extension_13UnImplemented_6__setstate_cython__(((struct __pyx_obj_6tables_13hdf5extension_UnImplemented *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_13UnImplemented_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6tables_13hdf5extension_UnImplemented *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self_dims_cannot_be_converted_to, 0, 0);
  __PYX_ERR(2, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("tables.hdf5extension.UnImplemented.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_File(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x3498a4d, 0x296486f, 0xfba3a2d, b'access_plist, file_id, name')
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_3__pyx_unpickle_File(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_3__pyx_unpickle_File = {"__pyx_unpickle_File", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_3__pyx_unpickle_File, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_3__pyx_unpickle_File(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_File (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(2, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(2, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(2, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_File", 0) < (0)) __PYX_ERR(2, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_File", 1, 3, 3, i); __PYX_ERR(2, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(2, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(2, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_File", 1, 3, 3, __pyx_nargs); __PYX_ERR(2, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.__pyx_unpickle_File", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(2, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_13hdf5extension_2__pyx_unpickle_File(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_2__pyx_unpickle_File(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_File", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_File(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x3498a4d, 0x296486f, 0xfba3a2d, b'access_plist, file_id, name')             # <<<<<<<<<<<<<<
 *     __pyx_result = File.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0x3498a4d, 0x296486f, 0xfba3a2d, __pyx_k_access_plist_file_id_name); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(2, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x3498a4d, 0x296486f, 0xfba3a2d, b'access_plist, file_id, name')
 *     __pyx_result = File.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_File__set_state(<File> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_File);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x3498a4d, 0x296486f, 0xfba3a2d, b'access_plist, file_id, name')
 *     __pyx_result = File.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_File__set_state(<File> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = File.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_File__set_state(<File> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_File__set_state(File __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(2, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_6tables_13hdf5extension___pyx_unpickle_File__set_state(((struct __pyx_obj_6tables_13hdf5extension_File *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x3498a4d, 0x296486f, 0xfba3a2d, b'access_plist, file_id, name')
 *     __pyx_result = File.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_File__set_state(<File> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_File__set_state(<File> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_File__set_state(File __pyx_result, __pyx_state: tuple):
 *     __pyx_result.access_plist = __pyx_state[0]; __pyx_result.file_id = __pyx_state[1]; __pyx_result.name = __pyx_state[2]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_File(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x3498a4d, 0x296486f, 0xfba3a2d, b'access_plist, file_id, name')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("tables.hdf5extension.__pyx_unpickle_File", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_File__set_state(<File> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_File__set_state(File __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.access_plist = __pyx_state[0]; __pyx_result.file_id = __pyx_state[1]; __pyx_result.name = __pyx_state[2]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)
*/

static PyObject *__pyx_f_6tables_13hdf5extension___pyx_unpickle_File__set_state(struct __pyx_obj_6tables_13hdf5extension_File *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  hid_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_File__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_File__set_state(File __pyx_result, __pyx_state: tuple):
 *     __pyx_result.access_plist = __pyx_state[0]; __pyx_result.file_id = __pyx_state[1]; __pyx_result.name = __pyx_state[2]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_2 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->access_plist = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_2 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->file_id = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->name);
  __Pyx_DECREF(__pyx_v___pyx_result->name);
  __pyx_v___pyx_result->name = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_File__set_state(File __pyx_result, __pyx_state: tuple):
 *     __pyx_result.access_plist = __pyx_state[0]; __pyx_result.file_id = __pyx_state[1]; __pyx_result.name = __pyx_state[2]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)             # <<<<<<<<<<<<<<
*/
  __pyx_t_3 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 3); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_File__set_state(<File> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_File__set_state(File __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.access_plist = __pyx_state[0]; __pyx_result.file_id = __pyx_state[1]; __pyx_result.name = __pyx_state[2]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.hdf5extension.__pyx_unpickle_File__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_AttributeSet(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x82a3537, 0x6ae9995, 0xb068931, b'name')
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_5__pyx_unpickle_AttributeSet(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_5__pyx_unpickle_AttributeSet = {"__pyx_unpickle_AttributeSet", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5__pyx_unpickle_AttributeSet, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_5__pyx_unpickle_AttributeSet(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_AttributeSet (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(2, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(2, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(2, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_AttributeSet", 0) < (0)) __PYX_ERR(2, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_AttributeSet", 1, 3, 3, i); __PYX_ERR(2, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(2, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(2, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_AttributeSet", 1, 3, 3, __pyx_nargs); __PYX_ERR(2, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.__pyx_unpickle_AttributeSet", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(2, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_13hdf5extension_4__pyx_unpickle_AttributeSet(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_4__pyx_unpickle_AttributeSet(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_AttributeSet", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_AttributeSet(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x82a3537, 0x6ae9995, 0xb068931, b'name')             # <<<<<<<<<<<<<<
 *     __pyx_result = AttributeSet.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0x82a3537, 0x6ae9995, 0xb068931, __pyx_k_name); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(2, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x82a3537, 0x6ae9995, 0xb068931, b'name')
 *     __pyx_result = AttributeSet.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_AttributeSet__set_state(<AttributeSet> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_AttributeSet);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x82a3537, 0x6ae9995, 0xb068931, b'name')
 *     __pyx_result = AttributeSet.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_AttributeSet__set_state(<AttributeSet> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = AttributeSet.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_AttributeSet__set_state(<AttributeSet> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_AttributeSet__set_state(AttributeSet __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(2, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_6tables_13hdf5extension___pyx_unpickle_AttributeSet__set_state(((struct __pyx_obj_6tables_13hdf5extension_AttributeSet *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x82a3537, 0x6ae9995, 0xb068931, b'name')
 *     __pyx_result = AttributeSet.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_AttributeSet__set_state(<AttributeSet> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_AttributeSet__set_state(<AttributeSet> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_AttributeSet__set_state(AttributeSet __pyx_result, __pyx_state: tuple):
 *     __pyx_result.name = __pyx_state[0]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_AttributeSet(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x82a3537, 0x6ae9995, 0xb068931, b'name')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("tables.hdf5extension.__pyx_unpickle_AttributeSet", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_AttributeSet__set_state(<AttributeSet> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_AttributeSet__set_state(AttributeSet __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.name = __pyx_state[0]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 1)
*/

static PyObject *__pyx_f_6tables_13hdf5extension___pyx_unpickle_AttributeSet__set_state(struct __pyx_obj_6tables_13hdf5extension_AttributeSet *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_AttributeSet__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_AttributeSet__set_state(AttributeSet __pyx_result, __pyx_state: tuple):
 *     __pyx_result.name = __pyx_state[0]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 1)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->name);
  __Pyx_DECREF(__pyx_v___pyx_result->name);
  __pyx_v___pyx_result->name = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_AttributeSet__set_state(AttributeSet __pyx_result, __pyx_state: tuple):
 *     __pyx_result.name = __pyx_state[0]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 1)             # <<<<<<<<<<<<<<
*/
  __pyx_t_2 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_AttributeSet__set_state(<AttributeSet> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_AttributeSet__set_state(AttributeSet __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.name = __pyx_state[0]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 1)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.hdf5extension.__pyx_unpickle_AttributeSet__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_Node(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x75c7861, 0x87bccfa, 0x381b6b9, b'name, parent_id')
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_7__pyx_unpickle_Node(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_7__pyx_unpickle_Node = {"__pyx_unpickle_Node", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7__pyx_unpickle_Node, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_7__pyx_unpickle_Node(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_Node (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(2, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(2, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(2, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_Node", 0) < (0)) __PYX_ERR(2, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Node", 1, 3, 3, i); __PYX_ERR(2, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(2, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(2, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Node", 1, 3, 3, __pyx_nargs); __PYX_ERR(2, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.__pyx_unpickle_Node", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(2, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_13hdf5extension_6__pyx_unpickle_Node(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_6__pyx_unpickle_Node(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Node", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_Node(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x75c7861, 0x87bccfa, 0x381b6b9, b'name, parent_id')             # <<<<<<<<<<<<<<
 *     __pyx_result = Node.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0x75c7861, 0x87bccfa, 0x381b6b9, __pyx_k_name_parent_id); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(2, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x75c7861, 0x87bccfa, 0x381b6b9, b'name, parent_id')
 *     __pyx_result = Node.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Node__set_state(<Node> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Node);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x75c7861, 0x87bccfa, 0x381b6b9, b'name, parent_id')
 *     __pyx_result = Node.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Node__set_state(<Node> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = Node.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Node__set_state(<Node> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_Node__set_state(Node __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(2, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_6tables_13hdf5extension___pyx_unpickle_Node__set_state(((struct __pyx_obj_6tables_13hdf5extension_Node *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x75c7861, 0x87bccfa, 0x381b6b9, b'name, parent_id')
 *     __pyx_result = Node.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Node__set_state(<Node> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Node__set_state(<Node> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_Node__set_state(Node __pyx_result, __pyx_state: tuple):
 *     __pyx_result.name = __pyx_state[0]; __pyx_result.parent_id = __pyx_state[1]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_Node(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x75c7861, 0x87bccfa, 0x381b6b9, b'name, parent_id')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("tables.hdf5extension.__pyx_unpickle_Node", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_Node__set_state(<Node> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Node__set_state(Node __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.name = __pyx_state[0]; __pyx_result.parent_id = __pyx_state[1]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)
*/

static PyObject *__pyx_f_6tables_13hdf5extension___pyx_unpickle_Node__set_state(struct __pyx_obj_6tables_13hdf5extension_Node *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  hid_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Node__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_Node__set_state(Node __pyx_result, __pyx_state: tuple):
 *     __pyx_result.name = __pyx_state[0]; __pyx_result.parent_id = __pyx_state[1]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->name);
  __Pyx_DECREF(__pyx_v___pyx_result->name);
  __pyx_v___pyx_result->name = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_2 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->parent_id = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_Node__set_state(Node __pyx_result, __pyx_state: tuple):
 *     __pyx_result.name = __pyx_state[0]; __pyx_result.parent_id = __pyx_state[1]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)             # <<<<<<<<<<<<<<
*/
  __pyx_t_3 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 2); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_Node__set_state(<Node> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Node__set_state(Node __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.name = __pyx_state[0]; __pyx_result.parent_id = __pyx_state[1]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.hdf5extension.__pyx_unpickle_Node__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_Group(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xd52d8f4, 0x98ddd7b, 0x1360bff, b'group_id, name, parent_id')
*/

/* Python wrapper */
static PyObject *__pyx_pw_6tables_13hdf5extension_9__pyx_unpickle_Group(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6tables_13hdf5extension_9__pyx_unpickle_Group = {"__pyx_unpickle_Group", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_9__pyx_unpickle_Group, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6tables_13hdf5extension_9__pyx_unpickle_Group(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_Group (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(2, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(2, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(2, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_Group", 0) < (0)) __PYX_ERR(2, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Group", 1, 3, 3, i); __PYX_ERR(2, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(2, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(2, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(2, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Group", 1, 3, 3, __pyx_nargs); __PYX_ERR(2, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("tables.hdf5extension.__pyx_unpickle_Group", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(2, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_6tables_13hdf5extension_8__pyx_unpickle_Group(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6tables_13hdf5extension_8__pyx_unpickle_Group(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Group", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_Group(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xd52d8f4, 0x98ddd7b, 0x1360bff, b'group_id, name, parent_id')             # <<<<<<<<<<<<<<
 *     __pyx_result = Group.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0xd52d8f4, 0x98ddd7b, 0x1360bff, __pyx_k_group_id_name_parent_id); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(2, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xd52d8f4, 0x98ddd7b, 0x1360bff, b'group_id, name, parent_id')
 *     __pyx_result = Group.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Group__set_state(<Group> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Group);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xd52d8f4, 0x98ddd7b, 0x1360bff, b'group_id, name, parent_id')
 *     __pyx_result = Group.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Group__set_state(<Group> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = Group.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Group__set_state(<Group> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_Group__set_state(Group __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(2, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_6tables_13hdf5extension___pyx_unpickle_Group__set_state(((struct __pyx_obj_6tables_13hdf5extension_Group *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xd52d8f4, 0x98ddd7b, 0x1360bff, b'group_id, name, parent_id')
 *     __pyx_result = Group.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Group__set_state(<Group> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Group__set_state(<Group> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_Group__set_state(Group __pyx_result, __pyx_state: tuple):
 *     __pyx_result.group_id = __pyx_state[0]; __pyx_result.name = __pyx_state[1]; __pyx_result.parent_id = __pyx_state[2]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_Group(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xd52d8f4, 0x98ddd7b, 0x1360bff, b'group_id, name, parent_id')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("tables.hdf5extension.__pyx_unpickle_Group", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_Group__set_state(<Group> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Group__set_state(Group __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.group_id = __pyx_state[0]; __pyx_result.name = __pyx_state[1]; __pyx_result.parent_id = __pyx_state[2]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)
*/

static PyObject *__pyx_f_6tables_13hdf5extension___pyx_unpickle_Group__set_state(struct __pyx_obj_6tables_13hdf5extension_Group *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  hid_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Group__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_Group__set_state(Group __pyx_result, __pyx_state: tuple):
 *     __pyx_result.group_id = __pyx_state[0]; __pyx_result.name = __pyx_state[1]; __pyx_result.parent_id = __pyx_state[2]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_2 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->group_id = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.name);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.name);
  __pyx_v___pyx_result->__pyx_base.name = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_hid_t(__pyx_t_1); if (unlikely((__pyx_t_2 == ((hid_t)-1)) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.parent_id = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_Group__set_state(Group __pyx_result, __pyx_state: tuple):
 *     __pyx_result.group_id = __pyx_state[0]; __pyx_result.name = __pyx_state[1]; __pyx_result.parent_id = __pyx_state[2]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)             # <<<<<<<<<<<<<<
*/
  __pyx_t_3 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 3); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_Group__set_state(<Group> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Group__set_state(Group __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.group_id = __pyx_state[0]; __pyx_result.name = __pyx_state[1]; __pyx_result.parent_id = __pyx_state[2]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 3)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("tables.hdf5extension.__pyx_unpickle_Group__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
/* #### Code section: module_exttypes ### */

static PyObject *__pyx_tp_new_6tables_13hdf5extension_Node(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6tables_13hdf5extension_Node *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6tables_13hdf5extension_Node *)o);
  p->name = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6tables_13hdf5extension_Node(PyObject *o) {
  struct __pyx_obj_6tables_13hdf5extension_Node *p = (struct __pyx_obj_6tables_13hdf5extension_Node *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_6tables_13hdf5extension_Node) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_DeallocKeepAliveBegin(o);
    __pyx_pw_6tables_13hdf5extension_4Node_5__dealloc__(o);
    __Pyx_DeallocKeepAliveEnd(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->name);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_6tables_13hdf5extension_Node(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6tables_13hdf5extension_Node *p = (struct __pyx_obj_6tables_13hdf5extension_Node *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->name) {
    e = (*v)(p->name, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6tables_13hdf5extension_Node(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6tables_13hdf5extension_Node *p = (struct __pyx_obj_6tables_13hdf5extension_Node *)o;
  tmp = ((PyObject*)p->name);
  p->name = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_6tables_13hdf5extension_Node[] = {
  {"_g_new", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Node_1_g_new, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_g_delete", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Node_3_g_delete, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_get_obj_info", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Node_7_get_obj_info, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_get_obj_timestamps", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Node_9_get_obj_timestamps, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Node_11__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Node_13__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_6tables_13hdf5extension_Node_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_6tables_13hdf5extension_Node},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_6tables_13hdf5extension_Node},
  {Py_tp_clear, (void *)__pyx_tp_clear_6tables_13hdf5extension_Node},
  {Py_tp_methods, (void *)__pyx_methods_6tables_13hdf5extension_Node},
  {Py_tp_new, (void *)__pyx_tp_new_6tables_13hdf5extension_Node},
  {0, 0},
};
static PyType_Spec __pyx_type_6tables_13hdf5extension_Node_spec = {
  "tables.hdf5extension.Node",
  sizeof(struct __pyx_obj_6tables_13hdf5extension_Node),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_6tables_13hdf5extension_Node_slots,
};
#else

static PyTypeObject __pyx_type_6tables_13hdf5extension_Node = {
  PyVarObject_HEAD_INIT(0, 0)
  "tables.hdf5extension.""Node", /*tp_name*/
  sizeof(struct __pyx_obj_6tables_13hdf5extension_Node), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6tables_13hdf5extension_Node, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6tables_13hdf5extension_Node, /*tp_traverse*/
  __pyx_tp_clear_6tables_13hdf5extension_Node, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6tables_13hdf5extension_Node, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6tables_13hdf5extension_Node, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf __pyx_vtable_6tables_13hdf5extension_Leaf;

static PyObject *__pyx_tp_new_6tables_13hdf5extension_Leaf(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6tables_13hdf5extension_Leaf *p;
  PyObject *o = __pyx_tp_new_6tables_13hdf5extension_Node(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6tables_13hdf5extension_Leaf *)o);
  p->__pyx_vtab = __pyx_vtabptr_6tables_13hdf5extension_Leaf;
  return o;
}

static PyMethodDef __pyx_methods_6tables_13hdf5extension_Leaf[] = {
  {"_get_storage_size", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_1_get_storage_size, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_get_obj_track_times", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_3_get_obj_track_times, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_4Leaf_2_get_obj_track_times},
  {"_g_new", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_5_g_new, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_g_chunk_info", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_7_g_chunk_info, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_4Leaf_6_g_chunk_info},
  {"_g_read_chunk", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_9_g_read_chunk, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_4Leaf_8_g_read_chunk},
  {"_g_write_chunk", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_11_g_write_chunk, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_4Leaf_10_g_write_chunk},
  {"_g_truncate", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_13_g_truncate, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_4Leaf_12_g_truncate},
  {"_g_flush", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_15_g_flush, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_g_close", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_17_g_close, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_19__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4Leaf_21__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_6tables_13hdf5extension_Leaf_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_6tables_13hdf5extension_Node},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_6tables_13hdf5extension_Node},
  {Py_tp_clear, (void *)__pyx_tp_clear_6tables_13hdf5extension_Node},
  {Py_tp_methods, (void *)__pyx_methods_6tables_13hdf5extension_Leaf},
  {Py_tp_new, (void *)__pyx_tp_new_6tables_13hdf5extension_Leaf},
  {0, 0},
};
static PyType_Spec __pyx_type_6tables_13hdf5extension_Leaf_spec = {
  "tables.hdf5extension.Leaf",
  sizeof(struct __pyx_obj_6tables_13hdf5extension_Leaf),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_6tables_13hdf5extension_Leaf_slots,
};
#else

static PyTypeObject __pyx_type_6tables_13hdf5extension_Leaf = {
  PyVarObject_HEAD_INIT(0, 0)
  "tables.hdf5extension.""Leaf", /*tp_name*/
  sizeof(struct __pyx_obj_6tables_13hdf5extension_Leaf), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6tables_13hdf5extension_Node, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6tables_13hdf5extension_Node, /*tp_traverse*/
  __pyx_tp_clear_6tables_13hdf5extension_Node, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6tables_13hdf5extension_Leaf, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6tables_13hdf5extension_Leaf, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_6tables_13hdf5extension_Array __pyx_vtable_6tables_13hdf5extension_Array;

static PyObject *__pyx_tp_new_6tables_13hdf5extension_Array(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6tables_13hdf5extension_Array *p;
  PyObject *o = __pyx_tp_new_6tables_13hdf5extension_Leaf(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6tables_13hdf5extension_Array *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf*)__pyx_vtabptr_6tables_13hdf5extension_Array;
  return o;
}

static void __pyx_tp_dealloc_6tables_13hdf5extension_Array(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_6tables_13hdf5extension_Array) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_DeallocKeepAliveBegin(o);
    __pyx_pw_6tables_13hdf5extension_5Array_25__dealloc__(o);
    __Pyx_DeallocKeepAliveEnd(o);
    PyErr_Restore(etype, eval, etb);
  }
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_6tables_13hdf5extension_Node(o);
}

static PyMethodDef __pyx_methods_6tables_13hdf5extension_Array[] = {
  {"_create_array", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_1_create_array, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_create_carray", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_3_create_carray, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_open_array", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_5_open_array, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_append", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_7_append, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_read_array", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_9_read_array, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_g_read_slice", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_11_g_read_slice, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_g_read_coords", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_13_g_read_coords, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Array_12_g_read_coords},
  {"perform_selection", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_15perform_selection, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Array_14perform_selection},
  {"_g_read_selection", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_17_g_read_selection, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Array_16_g_read_selection},
  {"_g_write_slice", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_19_g_write_slice, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Array_18_g_write_slice},
  {"_g_write_coords", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_21_g_write_coords, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Array_20_g_write_coords},
  {"_g_write_selection", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_23_g_write_selection, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Array_22_g_write_selection},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_27__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Array_29__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_6tables_13hdf5extension_Array_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_6tables_13hdf5extension_Array},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_6tables_13hdf5extension_Node},
  {Py_tp_clear, (void *)__pyx_tp_clear_6tables_13hdf5extension_Node},
  {Py_tp_methods, (void *)__pyx_methods_6tables_13hdf5extension_Array},
  {Py_tp_new, (void *)__pyx_tp_new_6tables_13hdf5extension_Array},
  {0, 0},
};
static PyType_Spec __pyx_type_6tables_13hdf5extension_Array_spec = {
  "tables.hdf5extension.Array",
  sizeof(struct __pyx_obj_6tables_13hdf5extension_Array),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_6tables_13hdf5extension_Array_slots,
};
#else

static PyTypeObject __pyx_type_6tables_13hdf5extension_Array = {
  PyVarObject_HEAD_INIT(0, 0)
  "tables.hdf5extension.""Array", /*tp_name*/
  sizeof(struct __pyx_obj_6tables_13hdf5extension_Array), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6tables_13hdf5extension_Array, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6tables_13hdf5extension_Node, /*tp_traverse*/
  __pyx_tp_clear_6tables_13hdf5extension_Node, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6tables_13hdf5extension_Array, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6tables_13hdf5extension_Array, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_6tables_13hdf5extension_File(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6tables_13hdf5extension_File *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6tables_13hdf5extension_File *)o);
  p->name = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6tables_13hdf5extension_File(PyObject *o) {
  struct __pyx_obj_6tables_13hdf5extension_File *p = (struct __pyx_obj_6tables_13hdf5extension_File *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_6tables_13hdf5extension_File) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_DeallocKeepAliveBegin(o);
    __pyx_pw_6tables_13hdf5extension_4File_17__dealloc__(o);
    __Pyx_DeallocKeepAliveEnd(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->name);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_6tables_13hdf5extension_File(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6tables_13hdf5extension_File *p = (struct __pyx_obj_6tables_13hdf5extension_File *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->name) {
    e = (*v)(p->name, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6tables_13hdf5extension_File(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6tables_13hdf5extension_File *p = (struct __pyx_obj_6tables_13hdf5extension_File *)o;
  tmp = ((PyObject*)p->name);
  p->name = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_6tables_13hdf5extension_File[] = {
  {"_g_new", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_1_g_new, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"get_file_image", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_3get_file_image, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_4File_2get_file_image},
  {"get_filesize", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_5get_filesize, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_4File_4get_filesize},
  {"get_userblock_size", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_7get_userblock_size, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_4File_6get_userblock_size},
  {"_get_file_id", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_9_get_file_id, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"fileno", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_11fileno, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_4File_10fileno},
  {"_flush_file", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_13_flush_file, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_close_file", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_15_close_file, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_19__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_4File_21__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_6tables_13hdf5extension_File_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_6tables_13hdf5extension_File},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_6tables_13hdf5extension_File},
  {Py_tp_clear, (void *)__pyx_tp_clear_6tables_13hdf5extension_File},
  {Py_tp_methods, (void *)__pyx_methods_6tables_13hdf5extension_File},
  {Py_tp_new, (void *)__pyx_tp_new_6tables_13hdf5extension_File},
  {0, 0},
};
static PyType_Spec __pyx_type_6tables_13hdf5extension_File_spec = {
  "tables.hdf5extension.File",
  sizeof(struct __pyx_obj_6tables_13hdf5extension_File),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_6tables_13hdf5extension_File_slots,
};
#else

static PyTypeObject __pyx_type_6tables_13hdf5extension_File = {
  PyVarObject_HEAD_INIT(0, 0)
  "tables.hdf5extension.""File", /*tp_name*/
  sizeof(struct __pyx_obj_6tables_13hdf5extension_File), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6tables_13hdf5extension_File, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6tables_13hdf5extension_File, /*tp_traverse*/
  __pyx_tp_clear_6tables_13hdf5extension_File, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6tables_13hdf5extension_File, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6tables_13hdf5extension_File, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_6tables_13hdf5extension_AttributeSet(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6tables_13hdf5extension_AttributeSet *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6tables_13hdf5extension_AttributeSet *)o);
  p->name = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6tables_13hdf5extension_AttributeSet(PyObject *o) {
  struct __pyx_obj_6tables_13hdf5extension_AttributeSet *p = (struct __pyx_obj_6tables_13hdf5extension_AttributeSet *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_6tables_13hdf5extension_AttributeSet) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->name);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_6tables_13hdf5extension_AttributeSet(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6tables_13hdf5extension_AttributeSet *p = (struct __pyx_obj_6tables_13hdf5extension_AttributeSet *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->name) {
    e = (*v)(p->name, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6tables_13hdf5extension_AttributeSet(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6tables_13hdf5extension_AttributeSet *p = (struct __pyx_obj_6tables_13hdf5extension_AttributeSet *)o;
  tmp = ((PyObject*)p->name);
  p->name = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_6tables_13hdf5extension_AttributeSet[] = {
  {"_g_new", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_12AttributeSet_1_g_new, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_g_list_attr", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_12AttributeSet_3_g_list_attr, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_12AttributeSet_2_g_list_attr},
  {"_g_setattr", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_12AttributeSet_5_g_setattr, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_12AttributeSet_4_g_setattr},
  {"_g_getattr", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_12AttributeSet_7_g_getattr, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_12AttributeSet_6_g_getattr},
  {"_g_remove", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_12AttributeSet_9_g_remove, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_12AttributeSet_11__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_12AttributeSet_13__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_6tables_13hdf5extension_AttributeSet_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_6tables_13hdf5extension_AttributeSet},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_6tables_13hdf5extension_AttributeSet},
  {Py_tp_clear, (void *)__pyx_tp_clear_6tables_13hdf5extension_AttributeSet},
  {Py_tp_methods, (void *)__pyx_methods_6tables_13hdf5extension_AttributeSet},
  {Py_tp_new, (void *)__pyx_tp_new_6tables_13hdf5extension_AttributeSet},
  {0, 0},
};
static PyType_Spec __pyx_type_6tables_13hdf5extension_AttributeSet_spec = {
  "tables.hdf5extension.AttributeSet",
  sizeof(struct __pyx_obj_6tables_13hdf5extension_AttributeSet),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_6tables_13hdf5extension_AttributeSet_slots,
};
#else

static PyTypeObject __pyx_type_6tables_13hdf5extension_AttributeSet = {
  PyVarObject_HEAD_INIT(0, 0)
  "tables.hdf5extension.""AttributeSet", /*tp_name*/
  sizeof(struct __pyx_obj_6tables_13hdf5extension_AttributeSet), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6tables_13hdf5extension_AttributeSet, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6tables_13hdf5extension_AttributeSet, /*tp_traverse*/
  __pyx_tp_clear_6tables_13hdf5extension_AttributeSet, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6tables_13hdf5extension_AttributeSet, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6tables_13hdf5extension_AttributeSet, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyMethodDef __pyx_methods_6tables_13hdf5extension_Group[] = {
  {"_g_create", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_1_g_create, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_g_open", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_3_g_open, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_g_get_objinfo", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_5_g_get_objinfo, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Group_4_g_get_objinfo},
  {"_g_list_group", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_7_g_list_group, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Group_6_g_list_group},
  {"_g_get_gchild_attr", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_9_g_get_gchild_attr, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Group_8_g_get_gchild_attr},
  {"_g_get_lchild_attr", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_11_g_get_lchild_attr, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_5Group_10_g_get_lchild_attr},
  {"_g_flush_group", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_13_g_flush_group, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_g_close_group", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_15_g_close_group, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_g_move_node", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_17_g_move_node, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_19__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_5Group_21__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_6tables_13hdf5extension_Group_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_6tables_13hdf5extension_Node},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_6tables_13hdf5extension_Node},
  {Py_tp_clear, (void *)__pyx_tp_clear_6tables_13hdf5extension_Node},
  {Py_tp_methods, (void *)__pyx_methods_6tables_13hdf5extension_Group},
  {Py_tp_new, (void *)__pyx_tp_new_6tables_13hdf5extension_Node},
  {0, 0},
};
static PyType_Spec __pyx_type_6tables_13hdf5extension_Group_spec = {
  "tables.hdf5extension.Group",
  sizeof(struct __pyx_obj_6tables_13hdf5extension_Group),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_6tables_13hdf5extension_Group_slots,
};
#else

static PyTypeObject __pyx_type_6tables_13hdf5extension_Group = {
  PyVarObject_HEAD_INIT(0, 0)
  "tables.hdf5extension.""Group", /*tp_name*/
  sizeof(struct __pyx_obj_6tables_13hdf5extension_Group), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6tables_13hdf5extension_Node, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6tables_13hdf5extension_Node, /*tp_traverse*/
  __pyx_tp_clear_6tables_13hdf5extension_Node, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6tables_13hdf5extension_Group, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6tables_13hdf5extension_Node, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_6tables_13hdf5extension_VLArray __pyx_vtable_6tables_13hdf5extension_VLArray;

static PyObject *__pyx_tp_new_6tables_13hdf5extension_VLArray(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6tables_13hdf5extension_VLArray *p;
  PyObject *o = __pyx_tp_new_6tables_13hdf5extension_Leaf(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6tables_13hdf5extension_VLArray *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf*)__pyx_vtabptr_6tables_13hdf5extension_VLArray;
  return o;
}

static PyMethodDef __pyx_methods_6tables_13hdf5extension_VLArray[] = {
  {"_create_array", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7VLArray_1_create_array, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_open_array", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7VLArray_3_open_array, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_append", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7VLArray_5_append, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_modify", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7VLArray_7_modify, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_get_memory_size", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7VLArray_9_get_memory_size, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_read_array", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7VLArray_11_read_array, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"get_row_size", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7VLArray_13get_row_size, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6tables_13hdf5extension_7VLArray_12get_row_size},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7VLArray_15__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_7VLArray_17__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_6tables_13hdf5extension_VLArray_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_6tables_13hdf5extension_Node},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_6tables_13hdf5extension_Node},
  {Py_tp_clear, (void *)__pyx_tp_clear_6tables_13hdf5extension_Node},
  {Py_tp_methods, (void *)__pyx_methods_6tables_13hdf5extension_VLArray},
  {Py_tp_new, (void *)__pyx_tp_new_6tables_13hdf5extension_VLArray},
  {0, 0},
};
static PyType_Spec __pyx_type_6tables_13hdf5extension_VLArray_spec = {
  "tables.hdf5extension.VLArray",
  sizeof(struct __pyx_obj_6tables_13hdf5extension_VLArray),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_6tables_13hdf5extension_VLArray_slots,
};
#else

static PyTypeObject __pyx_type_6tables_13hdf5extension_VLArray = {
  PyVarObject_HEAD_INIT(0, 0)
  "tables.hdf5extension.""VLArray", /*tp_name*/
  sizeof(struct __pyx_obj_6tables_13hdf5extension_VLArray), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6tables_13hdf5extension_Node, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6tables_13hdf5extension_Node, /*tp_traverse*/
  __pyx_tp_clear_6tables_13hdf5extension_Node, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6tables_13hdf5extension_VLArray, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6tables_13hdf5extension_VLArray, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_6tables_13hdf5extension_UnImplemented __pyx_vtable_6tables_13hdf5extension_UnImplemented;

static PyObject *__pyx_tp_new_6tables_13hdf5extension_UnImplemented(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6tables_13hdf5extension_UnImplemented *p;
  PyObject *o = __pyx_tp_new_6tables_13hdf5extension_Leaf(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6tables_13hdf5extension_UnImplemented *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6tables_13hdf5extension_Leaf*)__pyx_vtabptr_6tables_13hdf5extension_UnImplemented;
  return o;
}

static PyMethodDef __pyx_methods_6tables_13hdf5extension_UnImplemented[] = {
  {"_open_unimplemented", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_13UnImplemented_1_open_unimplemented, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"_g_close", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_13UnImplemented_3_g_close, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_13UnImplemented_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6tables_13hdf5extension_13UnImplemented_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_6tables_13hdf5extension_UnImplemented_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_6tables_13hdf5extension_Node},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_6tables_13hdf5extension_Node},
  {Py_tp_clear, (void *)__pyx_tp_clear_6tables_13hdf5extension_Node},
  {Py_tp_methods, (void *)__pyx_methods_6tables_13hdf5extension_UnImplemented},
  {Py_tp_new, (void *)__pyx_tp_new_6tables_13hdf5extension_UnImplemented},
  {0, 0},
};
static PyType_Spec __pyx_type_6tables_13hdf5extension_UnImplemented_spec = {
  "tables.hdf5extension.UnImplemented",
  sizeof(struct __pyx_obj_6tables_13hdf5extension_UnImplemented),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_6tables_13hdf5extension_UnImplemented_slots,
};
#else

static PyTypeObject __pyx_type_6tables_13hdf5extension_UnImplemented = {
  PyVarObject_HEAD_INIT(0, 0)
  "tables.hdf5extension.""UnImplemented", /*tp_name*/
  sizeof(struct __pyx_obj_6tables_13hdf5extension_UnImplemented), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6tables_13hdf5extension_Node, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6tables_13hdf5extension_Node, /*tp_traverse*/
  __pyx_tp_clear_6tables_13hdf5extension_Node, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6tables_13hdf5extension_UnImplemented, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6tables_13hdf5extension_UnImplemented, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};
/* #### Code section: initfunc_declarations ### */
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate); /*proto*/
/* #### Code section: init_module ### */

static int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Node = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_6tables_13hdf5extension_Node_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Node)) __PYX_ERR(0, 1020, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_6tables_13hdf5extension_Node_spec, __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Node) < (0)) __PYX_ERR(0, 1020, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Node = &__pyx_type_6tables_13hdf5extension_Node;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Node) < (0)) __PYX_ERR(0, 1020, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Node);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Node->tp_dictoffset && __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Node->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Node->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Node, (PyObject *) __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Node) < (0)) __PYX_ERR(0, 1020, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Node) < (0)) __PYX_ERR(0, 1020, __pyx_L1_error)
  __pyx_vtabptr_6tables_13hdf5extension_Leaf = &__pyx_vtable_6tables_13hdf5extension_Leaf;
  __pyx_vtable_6tables_13hdf5extension_Leaf._get_type_ids = (PyObject *(*)(struct __pyx_obj_6tables_13hdf5extension_Leaf *))__pyx_f_6tables_13hdf5extension_4Leaf__get_type_ids;
  __pyx_vtable_6tables_13hdf5extension_Leaf._convert_time64 = (PyObject *(*)(struct __pyx_obj_6tables_13hdf5extension_Leaf *, PyArrayObject *, int))__pyx_f_6tables_13hdf5extension_4Leaf__convert_time64;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_1 = PyTuple_Pack(1, (PyObject *)__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Node); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Leaf = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_6tables_13hdf5extension_Leaf_spec, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Leaf)) __PYX_ERR(0, 1232, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_6tables_13hdf5extension_Leaf_spec, __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Leaf) < (0)) __PYX_ERR(0, 1232, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Leaf = &__pyx_type_6tables_13hdf5extension_Leaf;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Leaf->tp_base = __pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Node;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Leaf) < (0)) __PYX_ERR(0, 1232, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Leaf);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Leaf->tp_dictoffset && __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Leaf->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Leaf->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Leaf, __pyx_vtabptr_6tables_13hdf5extension_Leaf) < (0)) __PYX_ERR(0, 1232, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Leaf) < (0)) __PYX_ERR(0, 1232, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Leaf, (PyObject *) __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Leaf) < (0)) __PYX_ERR(0, 1232, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Leaf) < (0)) __PYX_ERR(0, 1232, __pyx_L1_error)
  __pyx_vtabptr_6tables_13hdf5extension_Array = &__pyx_vtable_6tables_13hdf5extension_Array;
  __pyx_vtable_6tables_13hdf5extension_Array.__pyx_base = *__pyx_vtabptr_6tables_13hdf5extension_Leaf;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_1 = PyTuple_Pack(1, (PyObject *)__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Leaf); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Array = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_6tables_13hdf5extension_Array_spec, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Array)) __PYX_ERR(0, 1466, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_6tables_13hdf5extension_Array_spec, __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Array) < (0)) __PYX_ERR(0, 1466, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Array = &__pyx_type_6tables_13hdf5extension_Array;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Array->tp_base = __pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Leaf;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Array) < (0)) __PYX_ERR(0, 1466, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Array);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Array->tp_dictoffset && __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Array->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Array->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Array, __pyx_vtabptr_6tables_13hdf5extension_Array) < (0)) __PYX_ERR(0, 1466, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Array) < (0)) __PYX_ERR(0, 1466, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Array, (PyObject *) __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Array) < (0)) __PYX_ERR(0, 1466, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Array) < (0)) __PYX_ERR(0, 1466, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_File = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_6tables_13hdf5extension_File_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_File)) __PYX_ERR(0, 440, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_6tables_13hdf5extension_File_spec, __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_File) < (0)) __PYX_ERR(0, 440, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_File = &__pyx_type_6tables_13hdf5extension_File;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_File) < (0)) __PYX_ERR(0, 440, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_File);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_File->tp_dictoffset && __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_File->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_File->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_File, (PyObject *) __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_File) < (0)) __PYX_ERR(0, 440, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_File) < (0)) __PYX_ERR(0, 440, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_AttributeSet = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_6tables_13hdf5extension_AttributeSet_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_AttributeSet)) __PYX_ERR(0, 757, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_6tables_13hdf5extension_AttributeSet_spec, __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_AttributeSet) < (0)) __PYX_ERR(0, 757, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_AttributeSet = &__pyx_type_6tables_13hdf5extension_AttributeSet;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_AttributeSet) < (0)) __PYX_ERR(0, 757, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_AttributeSet);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_AttributeSet->tp_dictoffset && __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_AttributeSet->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_AttributeSet->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_AttributeSet, (PyObject *) __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_AttributeSet) < (0)) __PYX_ERR(0, 757, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_AttributeSet) < (0)) __PYX_ERR(0, 757, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_1 = PyTuple_Pack(1, (PyObject *)__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Node); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1068, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Group = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_6tables_13hdf5extension_Group_spec, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Group)) __PYX_ERR(0, 1068, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_6tables_13hdf5extension_Group_spec, __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Group) < (0)) __PYX_ERR(0, 1068, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Group = &__pyx_type_6tables_13hdf5extension_Group;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Group->tp_base = __pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Node;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Group) < (0)) __PYX_ERR(0, 1068, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Group);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Group->tp_dictoffset && __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Group->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Group->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Group, (PyObject *) __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Group) < (0)) __PYX_ERR(0, 1068, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_Group) < (0)) __PYX_ERR(0, 1068, __pyx_L1_error)
  __pyx_vtabptr_6tables_13hdf5extension_VLArray = &__pyx_vtable_6tables_13hdf5extension_VLArray;
  __pyx_vtable_6tables_13hdf5extension_VLArray.__pyx_base = *__pyx_vtabptr_6tables_13hdf5extension_Leaf;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_1 = PyTuple_Pack(1, (PyObject *)__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Leaf); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_VLArray = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_6tables_13hdf5extension_VLArray_spec, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_VLArray)) __PYX_ERR(0, 2158, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_6tables_13hdf5extension_VLArray_spec, __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_VLArray) < (0)) __PYX_ERR(0, 2158, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_VLArray = &__pyx_type_6tables_13hdf5extension_VLArray;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_VLArray->tp_base = __pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Leaf;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_VLArray) < (0)) __PYX_ERR(0, 2158, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_VLArray);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_VLArray->tp_dictoffset && __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_VLArray->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_VLArray->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_VLArray, __pyx_vtabptr_6tables_13hdf5extension_VLArray) < (0)) __PYX_ERR(0, 2158, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_VLArray) < (0)) __PYX_ERR(0, 2158, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_VLArray, (PyObject *) __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_VLArray) < (0)) __PYX_ERR(0, 2158, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_VLArray) < (0)) __PYX_ERR(0, 2158, __pyx_L1_error)
  __pyx_vtabptr_6tables_13hdf5extension_UnImplemented = &__pyx_vtable_6tables_13hdf5extension_UnImplemented;
  __pyx_vtable_6tables_13hdf5extension_UnImplemented.__pyx_base = *__pyx_vtabptr_6tables_13hdf5extension_Leaf;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_1 = PyTuple_Pack(1, (PyObject *)__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Leaf); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2456, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_UnImplemented = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_6tables_13hdf5extension_UnImplemented_spec, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_UnImplemented)) __PYX_ERR(0, 2456, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_6tables_13hdf5extension_UnImplemented_spec, __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_UnImplemented) < (0)) __PYX_ERR(0, 2456, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_UnImplemented = &__pyx_type_6tables_13hdf5extension_UnImplemented;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_UnImplemented->tp_base = __pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Leaf;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_UnImplemented) < (0)) __PYX_ERR(0, 2456, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_UnImplemented);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_UnImplemented->tp_dictoffset && __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_UnImplemented->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_UnImplemented->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_UnImplemented, __pyx_vtabptr_6tables_13hdf5extension_UnImplemented) < (0)) __PYX_ERR(0, 2456, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_6tables_13hdf5extension_UnImplemented) < (0)) __PYX_ERR(0, 2456, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_UnImplemented, (PyObject *) __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_UnImplemented) < (0)) __PYX_ERR(0, 2456, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_6tables_13hdf5extension_UnImplemented) < (0)) __PYX_ERR(0, 2456, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_7cpython_4type_type = __Pyx_ImportType_3_2_8(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyTypeObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  0, 0,
  #else
  sizeof(PyHeapTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_8); if (!__pyx_mstate->__pyx_ptype_7cpython_4type_type) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 229, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_5numpy_dtype = __Pyx_ImportType_3_2_8(__pyx_t_1, "numpy", "dtype",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyArray_Descr), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyArray_Descr),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyArray_Descr), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyArray_Descr),
  #else
  sizeof(PyArray_Descr), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyArray_Descr),
  #endif
  __Pyx_ImportType_CheckSize_Ignore_3_2_8); if (!__pyx_mstate->__pyx_ptype_5numpy_dtype) __PYX_ERR(1, 229, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_5numpy_flatiter = __Pyx_ImportType_3_2_8(__pyx_t_1, "numpy", "flatiter",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyArrayIterObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyArrayIterObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyArrayIterObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyArrayIterObject),
  #else
  sizeof(PyArrayIterObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyArrayIterObject),
  #endif
  __Pyx_ImportType_CheckSize_Ignore_3_2_8); if (!__pyx_mstate->__pyx_ptype_5numpy_flatiter) __PYX_ERR(1, 286, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_5numpy_broadcast = __Pyx_ImportType_3_2_8(__pyx_t_1, "numpy", "broadcast",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyArrayMultiIterObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyArrayMultiIterObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyArrayMultiIterObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyArrayMultiIterObject),
  #else
  sizeof(PyArrayMultiIterObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyArrayMultiIterObject),
  #endif
  __Pyx_ImportType_CheckSize_Ignore_3_2_8); if (!__pyx_mstate->__pyx_ptype_5numpy_broadcast) __PYX_ERR(1, 290, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_5numpy_ndarray = __Pyx_ImportType_3_2_8(__pyx_t_1, "numpy", "ndarray",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyArrayObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyArrayObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyArrayObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyArrayObject),
  #else
  sizeof(PyArrayObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyArrayObject),
  #endif
  __Pyx_ImportType_CheckSize_Ignore_3_2_8); if (!__pyx_mstate->__pyx_ptype_5numpy_ndarray) __PYX_ERR(1, 329, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_5numpy_generic = __Pyx_ImportType_3_2_8(__pyx_t_1, "numpy", "generic",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #else
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_8); if (!__pyx_mstate->__pyx_ptype_5numpy_generic) __PYX_ERR(1, 847, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_5numpy_number = __Pyx_ImportType_3_2_8(__pyx_t_1, "numpy", "number",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #else
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_8); if (!__pyx_mstate->__pyx_ptype_5numpy_number) __PYX_ERR(1, 849, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_5numpy_integer = __Pyx_ImportType_3_2_8(__pyx_t_1, "numpy", "integer",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #else
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_8); if (!__pyx_mstate->__pyx_ptype_5numpy_integer) __PYX_ERR(1, 851, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_5numpy_signedinteger = __Pyx_ImportType_3_2_8(__pyx_t_1, "numpy", "signedinteger",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #else
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_8); if (!__pyx_mstate->__pyx_ptype_5numpy_signedinteger) __PYX_ERR(1, 853, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_5numpy_unsignedinteger = __Pyx_ImportType_3_2_8(__pyx_t_1, "numpy", "unsignedinteger",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #else
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_8); if (!__pyx_mstate->__pyx_ptype_5numpy_unsignedinteger) __PYX_ERR(1, 855, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_5numpy_inexact = __Pyx_ImportType_3_2_8(__pyx_t_1, "numpy", "inexact",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #else
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_8); if (!__pyx_mstate->__pyx_ptype_5numpy_inexact) __PYX_ERR(1, 857, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_5numpy_floating = __Pyx_ImportType_3_2_8(__pyx_t_1, "numpy", "floating",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #else
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_8); if (!__pyx_mstate->__pyx_ptype_5numpy_floating) __PYX_ERR(1, 859, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_5numpy_complexfloating = __Pyx_ImportType_3_2_8(__pyx_t_1, "numpy", "complexfloating",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #else
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_8); if (!__pyx_mstate->__pyx_ptype_5numpy_complexfloating) __PYX_ERR(1, 861, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_5numpy_flexible = __Pyx_ImportType_3_2_8(__pyx_t_1, "numpy", "flexible",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #else
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_8); if (!__pyx_mstate->__pyx_ptype_5numpy_flexible) __PYX_ERR(1, 863, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_5numpy_character = __Pyx_ImportType_3_2_8(__pyx_t_1, "numpy", "character",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #else
  sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_8); if (!__pyx_mstate->__pyx_ptype_5numpy_character) __PYX_ERR(1, 865, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_5numpy_ufunc = __Pyx_ImportType_3_2_8(__pyx_t_1, "numpy", "ufunc",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyUFuncObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyUFuncObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyUFuncObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyUFuncObject),
  #else
  sizeof(PyUFuncObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(PyUFuncObject),
  #endif
  __Pyx_ImportType_CheckSize_Ignore_3_2_8); if (!__pyx_mstate->__pyx_ptype_5numpy_ufunc) __PYX_ERR(1, 946, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  {
    __pyx_t_1 = PyImport_ImportModule("tables.utilsextension"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_PyObject_char_const_hid_t_hid_t);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 112;
    void (**const __pyx_import_pointers[])(void) = {(void (**)(void))&__pyx_f_6tables_14utilsextension_cstr_to_pystr, (void (**)(void))&__pyx_f_6tables_14utilsextension_get_native_type, (void (**)(void))&__pyx_f_6tables_14utilsextension_malloc_dims, (void (**)(void))&__pyx_f_6tables_14utilsextension_load_reference, (void (**)(void)) NULL};
    void (**const *__pyx_import_pointer)(void) = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportFunction_3_2_8(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_hdf5extension(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_hdf5extension},
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  {Py_mod_gil, __Pyx_FREETHREADING_COMPATIBLE},
  #endif
  #if PY_VERSION_HEX >= 0x030C0000 && CYTHON_USE_MODULE_STATE
  {Py_mod_multiple_interpreters, Py_MOD_MULTIPLE_INTERPRETERS_NOT_SUPPORTED},
  #endif
  {0, NULL}
};
#endif

#ifdef __cplusplus
namespace {
  struct PyModuleDef __pyx_moduledef =
  #else
  static struct PyModuleDef __pyx_moduledef =
  #endif
  {
      PyModuleDef_HEAD_INIT,
      "hdf5extension",
      __pyx_k_Cython_interface_between_several, /* m_doc */
    #if CYTHON_USE_MODULE_STATE
      sizeof(__pyx_mstatetype), /* m_size */
    #else
      (CYTHON_PEP489_MULTI_PHASE_INIT) ? 0 : -1, /* m_size */
    #endif
      __pyx_methods /* m_methods */,
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      __pyx_moduledef_slots, /* m_slots */
    #else
      NULL, /* m_reload */
    #endif
    #if CYTHON_USE_MODULE_STATE
      __pyx_m_traverse, /* m_traverse */
      __pyx_m_clear, /* m_clear */
      NULL /* m_free */
    #else
      NULL, /* m_traverse */
      NULL, /* m_clear */
      NULL /* m_free */
    #endif
  };
  #ifdef __cplusplus
} /* anonymous namespace */
#endif

/* PyModInitFuncType */
#ifndef CYTHON_NO_PYINIT_EXPORT
  #define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#else
  #ifdef __cplusplus
  #define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
  #else
  #define __Pyx_PyMODINIT_FUNC PyObject *
  #endif
#endif

__Pyx_PyMODINIT_FUNC PyInit_hdf5extension(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_hdf5extension(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
/* ModuleCreationPEP489 */
#if CYTHON_COMPILING_IN_LIMITED_API && (__PYX_LIMITED_VERSION_HEX < 0x03090000\
      || ((defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)) && __PYX_LIMITED_VERSION_HEX < 0x030A0000))
static PY_INT64_T __Pyx_GetCurrentInterpreterId(void) {
    {
        PyObject *module = PyImport_ImportModule("_interpreters"); // 3.13+ I think
        if (!module) {
            PyErr_Clear(); // just try the 3.8-3.12 version
            module = PyImport_ImportModule("_xxsubinterpreters");
            if (!module) goto bad;
        }
        PyObject *current = PyObject_CallMethod(module, "get_current", NULL);
        Py_DECREF(module);
        if (!current) goto bad;
        if (PyTuple_Check(current)) {
            PyObject *new_current = PySequence_GetItem(current, 0);
            Py_DECREF(current);
            current = new_current;
            if (!new_current) goto bad;
        }
        long long as_c_int = PyLong_AsLongLong(current);
        Py_DECREF(current);
        return as_c_int;
    }
  bad:
    PySys_WriteStderr("__Pyx_GetCurrentInterpreterId failed. Try setting the C define CYTHON_PEP489_MULTI_PHASE_INIT=0\n");
    return -1;
}
#endif
#if !CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    static PY_INT64_T main_interpreter_id = -1;
#if CYTHON_COMPILING_IN_GRAAL && defined(GRAALPY_VERSION_NUM) && GRAALPY_VERSION_NUM > 0x19000000
    PY_INT64_T current_id = GraalPyInterpreterState_GetIDFromThreadState(PyThreadState_Get());
#elif CYTHON_COMPILING_IN_GRAAL
    PY_INT64_T current_id = PyInterpreterState_GetIDFromThreadState(PyThreadState_Get());
#elif CYTHON_COMPILING_IN_LIMITED_API && (__PYX_LIMITED_VERSION_HEX < 0x03090000\
      || ((defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)) && __PYX_LIMITED_VERSION_HEX < 0x030A0000))
    PY_INT64_T current_id = __Pyx_GetCurrentInterpreterId();
#elif CYTHON_COMPILING_IN_LIMITED_API
    PY_INT64_T current_id = PyInterpreterState_GetID(PyInterpreterState_Get());
#else
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
#endif
    if (unlikely(current_id == -1)) {
        return -1;
    }
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return 0;
    } else if (unlikely(main_interpreter_id != current_id)) {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
#endif
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none)
{
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    CYTHON_UNUSED_VAR(def);
    #if !CYTHON_USE_MODULE_STATE
    if (__Pyx_check_single_interpreter())
        return NULL;
    #endif
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_hdf5extension(PyObject *__pyx_pyinit_module)
#endif
{
  int stringtab_initialized = 0;
  #if CYTHON_USE_MODULE_STATE
  int pystate_addmodule_run = 0;
  #endif
  __pyx_mstatetype *__pyx_mstate = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'hdf5extension' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #else
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_t_1 = __pyx_pyinit_module;
  Py_INCREF(__pyx_t_1);
  #else
  __pyx_t_1 = PyModule_Create(&__pyx_moduledef); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #if CYTHON_USE_MODULE_STATE
  {
    int add_module_result = __Pyx_State_AddModule(__pyx_t_1, &__pyx_moduledef);
    __pyx_t_1 = 0; /* transfer ownership from __pyx_t_1 to "hdf5extension" pseudovariable */
    if (unlikely((add_module_result < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    pystate_addmodule_run = 1;
  }
  #else
  __pyx_m = __pyx_t_1;
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  PyUnstable_Module_SetGIL(__pyx_m, Py_MOD_GIL_NOT_USED);
  #endif
  __pyx_mstate = __pyx_mstate_global;
  CYTHON_UNUSED_VAR(__pyx_t_1);
  __pyx_mstate->__pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_mstate->__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_mstate->__pyx_d);
  __pyx_mstate->__pyx_b = __Pyx_PyImport_AddModuleRef(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_mstate->__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_cython_runtime = __Pyx_PyImport_AddModuleRef("cython_runtime"); if (unlikely(!__pyx_mstate->__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_mstate->__pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /* ImportRefnannyAPI */
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
    PyErr_Clear();
    __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
    if (!__Pyx_RefNanny)
        Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  
__Pyx_RefNannySetupContext("PyInit_hdf5extension", 0);
  __Pyx_init_runtime_version();
  if (__Pyx_check_binary_version(__PYX_LIMITED_VERSION_HEX, __Pyx_get_runtime_version(), CYTHON_COMPILING_IN_LIMITED_API) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_mstate->__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Library function declarations ---*/
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitConstants(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  stringtab_initialized = 1;
  if (__Pyx_InitGlobals() < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__pyx_module_is_main_tables__hdf5extension) {
    if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_name_2, __pyx_mstate_global->__pyx_n_u_main) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "tables.hdf5extension")) {
      if (unlikely((PyDict_SetItemString(modules, "tables.hdf5extension", __pyx_m) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_CreateCodeObjects(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code(__pyx_mstate);
  (void)__Pyx_modinit_variable_export_code(__pyx_mstate);
  (void)__Pyx_modinit_function_export_code(__pyx_mstate);
  if (unlikely((__Pyx_modinit_type_init_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_type_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code(__pyx_mstate);
  if (unlikely((__Pyx_modinit_function_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Execution code ---*/

  /* "tables/hdf5extension.pyx":30
 * """
 * 
 * import os             # <<<<<<<<<<<<<<
 * import sys
 * import platform
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_os, 0, 0, __pyx_mstate_global->__pyx_kp_u_tables_os, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 30, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_os, __pyx_t_2) < (0)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":31
 * 
 * import os
 * import sys             # <<<<<<<<<<<<<<
 * import platform
 * import warnings
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_sys, 0, 0, __pyx_mstate_global->__pyx_kp_u_tables_sys, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_sys, __pyx_t_2) < (0)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":32
 * import os
 * import sys
 * import platform             # <<<<<<<<<<<<<<
 * import warnings
 * from collections import namedtuple
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_platform, 0, 0, __pyx_mstate_global->__pyx_kp_u_tables_platform, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 32, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_platform, __pyx_t_2) < (0)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":33
 * import sys
 * import platform
 * import warnings             # <<<<<<<<<<<<<<
 * from collections import namedtuple
 * 
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_warnings, 0, 0, __pyx_mstate_global->__pyx_kp_u_tables_warnings, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 33, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_warnings, __pyx_t_2) < (0)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":34
 * import platform
 * import warnings
 * from collections import namedtuple             # <<<<<<<<<<<<<<
 * 
 * ObjInfo = namedtuple('ObjInfo', ['addr', 'rc'])
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_namedtuple};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_collections, __pyx_imported_names, 1, __pyx_mstate_global->__pyx_kp_u_tables_collections, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 34, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_namedtuple};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 34, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 34, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":36
 * from collections import namedtuple
 * 
 * ObjInfo = namedtuple('ObjInfo', ['addr', 'rc'])             # <<<<<<<<<<<<<<
 * ObjTimestamps = namedtuple('ObjTimestamps', ['atime', 'mtime',
 *                                              'ctime', 'btime'])
*/
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_namedtuple); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyList_Pack(2, __pyx_mstate_global->__pyx_n_u_addr, __pyx_mstate_global->__pyx_n_u_rc); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = 1;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_4, __pyx_mstate_global->__pyx_n_u_ObjInfo, __pyx_t_6};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 36, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_ObjInfo, __pyx_t_2) < (0)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":37
 * 
 * ObjInfo = namedtuple('ObjInfo', ['addr', 'rc'])
 * ObjTimestamps = namedtuple('ObjTimestamps', ['atime', 'mtime',             # <<<<<<<<<<<<<<
 *                                              'ctime', 'btime'])
 * 
*/
  __pyx_t_5 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_namedtuple); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyList_Pack(4, __pyx_mstate_global->__pyx_n_u_atime, __pyx_mstate_global->__pyx_n_u_mtime, __pyx_mstate_global->__pyx_n_u_ctime, __pyx_mstate_global->__pyx_n_u_btime); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_7 = 1;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_mstate_global->__pyx_n_u_ObjTimestamps, __pyx_t_4};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 37, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_ObjTimestamps, __pyx_t_2) < (0)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":40
 *                                              'ctime', 'btime'])
 * 
 * import pickle             # <<<<<<<<<<<<<<
 * 
 * import numpy as np
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_pickle, 0, 0, __pyx_mstate_global->__pyx_kp_u_tables_pickle, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 40, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pickle, __pyx_t_2) < (0)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":42
 * import pickle
 * 
 * import numpy as np             # <<<<<<<<<<<<<<
 * 
 * from .atom import Atom
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_numpy, 0, 0, __pyx_mstate_global->__pyx_kp_u_tables_numpy, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 42, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_np, __pyx_t_2) < (0)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":44
 * import numpy as np
 * 
 * from .atom import Atom             # <<<<<<<<<<<<<<
 * from .utils import check_file_access, byteorders, correct_byteorder, SizeType
 * from .exceptions import HDF5ExtError, DataTypeWarning
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_Atom};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_atom, __pyx_imported_names, 1, __pyx_mstate_global->__pyx_kp_u_tables_atom, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 44, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_Atom};
    __pyx_t_3 = 0; {
      __pyx_t_6 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 44, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_6) < (0)) __PYX_ERR(0, 44, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":45
 * 
 * from .atom import Atom
 * from .utils import check_file_access, byteorders, correct_byteorder, SizeType             # <<<<<<<<<<<<<<
 * from .exceptions import HDF5ExtError, DataTypeWarning
 * from .description import descr_from_dtype
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_check_file_access,__pyx_mstate_global->__pyx_n_u_byteorders,__pyx_mstate_global->__pyx_n_u_correct_byteorder,__pyx_mstate_global->__pyx_n_u_SizeType};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_utils, __pyx_imported_names, 4, __pyx_mstate_global->__pyx_kp_u_tables_utils, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 45, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_check_file_access,__pyx_mstate_global->__pyx_n_u_byteorders,__pyx_mstate_global->__pyx_n_u_correct_byteorder,__pyx_mstate_global->__pyx_n_u_SizeType};
    for (__pyx_t_3=0; __pyx_t_3 < 4; __pyx_t_3++) {
      __pyx_t_6 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 45, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_6) < (0)) __PYX_ERR(0, 45, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":46
 * from .atom import Atom
 * from .utils import check_file_access, byteorders, correct_byteorder, SizeType
 * from .exceptions import HDF5ExtError, DataTypeWarning             # <<<<<<<<<<<<<<
 * from .description import descr_from_dtype
 * from .utilsextension import (
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_HDF5ExtError,__pyx_mstate_global->__pyx_n_u_DataTypeWarning};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_exceptions, __pyx_imported_names, 2, __pyx_mstate_global->__pyx_kp_u_tables_exceptions, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 46, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_HDF5ExtError,__pyx_mstate_global->__pyx_n_u_DataTypeWarning};
    for (__pyx_t_3=0; __pyx_t_3 < 2; __pyx_t_3++) {
      __pyx_t_6 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 46, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_6) < (0)) __PYX_ERR(0, 46, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":47
 * from .utils import check_file_access, byteorders, correct_byteorder, SizeType
 * from .exceptions import HDF5ExtError, DataTypeWarning
 * from .description import descr_from_dtype             # <<<<<<<<<<<<<<
 * from .utilsextension import (
 *     encode_filename,
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_descr_from_dtype};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_description, __pyx_imported_names, 1, __pyx_mstate_global->__pyx_kp_u_tables_description, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 47, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_descr_from_dtype};
    __pyx_t_3 = 0; {
      __pyx_t_6 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 47, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_6) < (0)) __PYX_ERR(0, 47, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":48
 * from .exceptions import HDF5ExtError, DataTypeWarning
 * from .description import descr_from_dtype
 * from .utilsextension import (             # <<<<<<<<<<<<<<
 *     encode_filename,
 *     set_blosc_max_threads,
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_encode_filename,__pyx_mstate_global->__pyx_n_u_set_blosc_max_threads,__pyx_mstate_global->__pyx_n_u_set_blosc2_max_threads,__pyx_mstate_global->__pyx_n_u_atom_to_hdf5_type,__pyx_mstate_global->__pyx_n_u_atom_from_hdf5_type,__pyx_mstate_global->__pyx_n_u_hdf5_to_np_ext_type,__pyx_mstate_global->__pyx_n_u_create_nested_type,__pyx_mstate_global->__pyx_n_u_pttype_to_hdf5,__pyx_mstate_global->__pyx_n_u_pt_special_kinds,__pyx_mstate_global->__pyx_n_u_npext_prefixes_to_ptkinds,__pyx_mstate_global->__pyx_n_u_hdf5_class_to_string,__pyx_mstate_global->__pyx_n_u_platform_byteorder,__pyx_mstate_global->__pyx_n_u_get_filters};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_utilsextension, __pyx_imported_names, 13, __pyx_mstate_global->__pyx_kp_u_tables_utilsextension, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 48, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_encode_filename,__pyx_mstate_global->__pyx_n_u_set_blosc_max_threads,__pyx_mstate_global->__pyx_n_u_set_blosc2_max_threads,__pyx_mstate_global->__pyx_n_u_atom_to_hdf5_type,__pyx_mstate_global->__pyx_n_u_atom_from_hdf5_type,__pyx_mstate_global->__pyx_n_u_hdf5_to_np_ext_type,__pyx_mstate_global->__pyx_n_u_create_nested_type,__pyx_mstate_global->__pyx_n_u_pttype_to_hdf5,__pyx_mstate_global->__pyx_n_u_pt_special_kinds,__pyx_mstate_global->__pyx_n_u_npext_prefixes_to_ptkinds,__pyx_mstate_global->__pyx_n_u_hdf5_class_to_string,__pyx_mstate_global->__pyx_n_u_platform_byteorder,__pyx_mstate_global->__pyx_n_u_get_filters};
    for (__pyx_t_3=0; __pyx_t_3 < 13; __pyx_t_3++) {
      __pyx_t_6 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 48, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_6) < (0)) __PYX_ERR(0, 48, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":205
 * 
 * 
 * cdef int H5T_CSET_DEFAULT = 16             # <<<<<<<<<<<<<<
 * 
 * from .utilsextension cimport (
*/
  __pyx_v_6tables_13hdf5extension_H5T_CSET_DEFAULT = 16;

  /* "tables/hdf5extension.pyx":311
 * # The numpy API requires this function to be called before
 * # using any numpy facilities in an extension module.
 * import_array()             # <<<<<<<<<<<<<<
 * 
 * #---------------------------------------------------------------------------
*/
  __pyx_t_8 = __pyx_f_5numpy_import_array(); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 311, __pyx_L1_error)

  /* "tables/hdf5extension.pyx":420
 * 
 * _supported_drivers = (
 *     "H5FD_SEC2",             # <<<<<<<<<<<<<<
 *     "H5FD_DIRECT",
 *     #"H5FD_LOG",
*/
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_supported_drivers, __pyx_mstate_global->__pyx_tuple[2]) < (0)) __PYX_ERR(0, 419, __pyx_L1_error)

  /* "tables/hdf5extension.pyx":434
 * )
 * 
 * HAVE_DIRECT_DRIVER = bool(H5_HAVE_DIRECT_DRIVER)             # <<<<<<<<<<<<<<
 * HAVE_WINDOWS_DRIVER = bool(H5_HAVE_WINDOWS_DRIVER)
 * 
*/
  __pyx_t_2 = __Pyx_PyBool_FromLong((!(!(H5_HAVE_DIRECT_DRIVER != 0)))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 434, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_HAVE_DIRECT_DRIVER, __pyx_t_2) < (0)) __PYX_ERR(0, 434, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":435
 * 
 * HAVE_DIRECT_DRIVER = bool(H5_HAVE_DIRECT_DRIVER)
 * HAVE_WINDOWS_DRIVER = bool(H5_HAVE_WINDOWS_DRIVER)             # <<<<<<<<<<<<<<
 * 
 * # Type extensions declarations (these are subclassed by PyTables
*/
  __pyx_t_2 = __Pyx_PyBool_FromLong((!(!(H5_HAVE_WINDOWS_DRIVER != 0)))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_HAVE_WINDOWS_DRIVER, __pyx_t_2) < (0)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":445
 *   cdef object  name
 * 
 *   def _g_new(self, name, pymode, **params):             # <<<<<<<<<<<<<<
 *     cdef herr_t err = 0
 *     cdef hid_t access_plist, create_plist = H5P_DEFAULT
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4File_1_g_new, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_File__g_new, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[0])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_File, __pyx_mstate_global->__pyx_n_u_g_new, __pyx_t_2) < (0)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":630
 * 
 *   # XXX: add the possibility to pass a pre-allocated buffer
 *   def get_file_image(self):             # <<<<<<<<<<<<<<
 *     """Retrieves an in-memory image of an existing, open HDF5 file.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4File_3get_file_image, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_File_get_file_image, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[1])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 630, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_File, __pyx_mstate_global->__pyx_n_u_get_file_image, __pyx_t_2) < (0)) __PYX_ERR(0, 630, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":666
 *     return image
 * 
 *   def get_filesize(self):             # <<<<<<<<<<<<<<
 *     """Returns the size of an HDF5 file.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4File_5get_filesize, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_File_get_filesize, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[2])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 666, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_File, __pyx_mstate_global->__pyx_n_u_get_filesize, __pyx_t_2) < (0)) __PYX_ERR(0, 666, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":687
 *     return size
 * 
 *   def get_userblock_size(self):             # <<<<<<<<<<<<<<
 *     """Retrieves the size of a user block.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4File_7get_userblock_size, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_File_get_userblock_size, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[3])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 687, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_File, __pyx_mstate_global->__pyx_n_u_get_userblock_size, __pyx_t_2) < (0)) __PYX_ERR(0, 687, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":712
 * 
 *   # Accessor definitions
 *   def _get_file_id(self):             # <<<<<<<<<<<<<<
 *     return self.file_id
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4File_9_get_file_id, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_File__get_file_id, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[4])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 712, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_File, __pyx_mstate_global->__pyx_n_u_get_file_id, __pyx_t_2) < (0)) __PYX_ERR(0, 712, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":715
 *     return self.file_id
 * 
 *   def fileno(self):             # <<<<<<<<<<<<<<
 *     """Return the underlying OS integer file descriptor.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4File_11fileno, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_File_fileno, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[5])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 715, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_File, __pyx_mstate_global->__pyx_n_u_fileno, __pyx_t_2) < (0)) __PYX_ERR(0, 715, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":735
 * 
 * 
 *   def _flush_file(self, scope):             # <<<<<<<<<<<<<<
 *     # Close the file
 *     H5Fflush(self.file_id, scope)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4File_13_flush_file, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_File__flush_file, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[6])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 735, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_File, __pyx_mstate_global->__pyx_n_u_flush_file, __pyx_t_2) < (0)) __PYX_ERR(0, 735, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":740
 * 
 * 
 *   def _close_file(self):             # <<<<<<<<<<<<<<
 *     # Close the file
 *     H5Fclose( self.file_id )
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4File_15_close_file, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_File__close_file, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[7])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 740, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_File, __pyx_mstate_global->__pyx_n_u_close_file, __pyx_t_2) < (0)) __PYX_ERR(0, 740, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4File_19__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_File___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[8])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_File, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < (0)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_File, (type(self), 0x3498a4d, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_File__set_state(self, __pyx_state)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4File_21__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_File___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[9])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_File, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < (0)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":760
 *   cdef object name
 * 
 *   def _g_new(self, node):             # <<<<<<<<<<<<<<
 *     self.name = node._v_name
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_12AttributeSet_1_g_new, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_AttributeSet__g_new, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[10])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 760, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_AttributeSet, __pyx_mstate_global->__pyx_n_u_g_new, __pyx_t_2) < (0)) __PYX_ERR(0, 760, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":763
 *     self.name = node._v_name
 * 
 *   def _g_list_attr(self, node):             # <<<<<<<<<<<<<<
 *     """Return a tuple with the attribute list"""
 *     a = Aiterate(node._v_objectid)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_12AttributeSet_3_g_list_attr, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_AttributeSet__g_list_attr, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[11])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 763, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_AttributeSet, __pyx_mstate_global->__pyx_n_u_g_list_attr, __pyx_t_2) < (0)) __PYX_ERR(0, 763, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":769
 * 
 * 
 *   def _g_setattr(self, node, name, object value):             # <<<<<<<<<<<<<<
 *     """Save Python or NumPy objects as HDF5 attributes.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_12AttributeSet_5_g_setattr, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_AttributeSet__g_setattr, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[12])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 769, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_AttributeSet, __pyx_mstate_global->__pyx_n_u_g_setattr, __pyx_t_2) < (0)) __PYX_ERR(0, 769, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":846
 * 
 *   # Get attributes
 *   def _g_getattr(self, node, attrname):             # <<<<<<<<<<<<<<
 *     """Get HDF5 attributes and retrieve them as NumPy objects.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_12AttributeSet_7_g_getattr, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_AttributeSet__g_getattr, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[13])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_AttributeSet, __pyx_mstate_global->__pyx_n_u_g_getattr, __pyx_t_2) < (0)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1001
 * 
 * 
 *   def _g_remove(self, node, attrname):             # <<<<<<<<<<<<<<
 *     cdef int ret
 *     cdef hid_t dset_id
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_12AttributeSet_9_g_remove, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_AttributeSet__g_remove, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[14])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1001, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_AttributeSet, __pyx_mstate_global->__pyx_n_u_g_remove, __pyx_t_2) < (0)) __PYX_ERR(0, 1001, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_12AttributeSet_11__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_AttributeSet___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[15])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_AttributeSet, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < (0)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_AttributeSet, (type(self), 0x82a3537, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_AttributeSet__set_state(self, __pyx_state)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_12AttributeSet_13__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_AttributeSet___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[16])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_AttributeSet, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < (0)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1023
 *   # Instance variables declared in .pxd
 * 
 *   def _g_new(self, where, name, init):             # <<<<<<<<<<<<<<
 *     self.name = name
 *     # """The name of this node in its parent group."""
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4Node_1_g_new, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node__g_new, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[17])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1023, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Node, __pyx_mstate_global->__pyx_n_u_g_new, __pyx_t_2) < (0)) __PYX_ERR(0, 1023, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1029
 *     # """The identifier of the parent group."""
 * 
 *   def _g_delete(self, parent):             # <<<<<<<<<<<<<<
 *     cdef int ret
 *     cdef bytes encoded_name
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4Node_3_g_delete, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node__g_delete, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[18])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1029, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Node, __pyx_mstate_global->__pyx_n_u_g_delete, __pyx_t_2) < (0)) __PYX_ERR(0, 1029, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1044
 *     self.parent_id = 0
 * 
 *   def _get_obj_info(self):             # <<<<<<<<<<<<<<
 *     cdef herr_t ret = 0
 *     cdef H5O_info_t oinfo
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4Node_7_get_obj_info, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node__get_obj_info, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[19])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1044, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Node, __pyx_mstate_global->__pyx_n_u_get_obj_info, __pyx_t_2) < (0)) __PYX_ERR(0, 1044, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1055
 *     return ObjInfo(oinfo.addr, oinfo.rc)
 * 
 *   def _get_obj_timestamps(self):             # <<<<<<<<<<<<<<
 *     cdef herr_t ret = 0
 *     cdef H5O_info_t oinfo
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4Node_9_get_obj_timestamps, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node__get_obj_timestamps, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[20])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1055, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Node, __pyx_mstate_global->__pyx_n_u_get_obj_timestamps, __pyx_t_2) < (0)) __PYX_ERR(0, 1055, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4Node_11__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[21])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Node, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < (0)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Node, (type(self), 0x75c7861, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Node__set_state(self, __pyx_state)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4Node_13__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Node___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[22])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Node, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < (0)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1071
 *   cdef hid_t   group_id
 * 
 *   def _g_create(self):             # <<<<<<<<<<<<<<
 *     cdef hid_t ret
 *     cdef bytes encoded_name
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Group_1_g_create, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Group__g_create, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[23])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1071, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Group, __pyx_mstate_global->__pyx_n_u_g_create, __pyx_t_2) < (0)) __PYX_ERR(0, 1071, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1087
 *     return self.group_id
 * 
 *   def _g_open(self):             # <<<<<<<<<<<<<<
 *     cdef hid_t ret
 *     cdef bytes encoded_name
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Group_3_g_open, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Group__g_open, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[24])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1087, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Group, __pyx_mstate_global->__pyx_n_u_g_open, __pyx_t_2) < (0)) __PYX_ERR(0, 1087, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1099
 *     return self.group_id
 * 
 *   def _g_get_objinfo(self, object h5name):             # <<<<<<<<<<<<<<
 *     """Check whether 'name' is a children of 'self' and return its type."""
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Group_5_g_get_objinfo, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Group__g_get_objinfo, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[25])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1099, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Group, __pyx_mstate_global->__pyx_n_u_g_get_objinfo, __pyx_t_2) < (0)) __PYX_ERR(0, 1099, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1137
 *     return node_type
 * 
 *   def _g_list_group(self, parent):             # <<<<<<<<<<<<<<
 *     """Return a tuple with the groups and the leaves hanging from self."""
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Group_7_g_list_group, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Group__g_list_group, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[26])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Group, __pyx_mstate_global->__pyx_n_u_g_list_group, __pyx_t_2) < (0)) __PYX_ERR(0, 1137, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1147
 * 
 * 
 *   def _g_get_gchild_attr(self, group_name, attr_name):             # <<<<<<<<<<<<<<
 *     """Return an attribute of a child `Group`.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Group_9_g_get_gchild_attr, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Group__g_get_gchild_attr, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[27])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1147, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Group, __pyx_mstate_global->__pyx_n_u_g_get_gchild_attr, __pyx_t_2) < (0)) __PYX_ERR(0, 1147, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1175
 * 
 * 
 *   def _g_get_lchild_attr(self, leaf_name, attr_name):             # <<<<<<<<<<<<<<
 *     """Return an attribute of a child `Leaf`.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Group_11_g_get_lchild_attr, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Group__g_get_lchild_attr, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[28])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Group, __pyx_mstate_global->__pyx_n_u_g_get_lchild_attr, __pyx_t_2) < (0)) __PYX_ERR(0, 1175, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1201
 * 
 * 
 *   def _g_flush_group(self):             # <<<<<<<<<<<<<<
 *     # Close the group
 *     H5Fflush(self.group_id, H5F_SCOPE_GLOBAL)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Group_13_g_flush_group, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Group__g_flush_group, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[29])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Group, __pyx_mstate_global->__pyx_n_u_g_flush_group, __pyx_t_2) < (0)) __PYX_ERR(0, 1201, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1206
 * 
 * 
 *   def _g_close_group(self):             # <<<<<<<<<<<<<<
 *     cdef int ret
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Group_15_g_close_group, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Group__g_close_group, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[30])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Group, __pyx_mstate_global->__pyx_n_u_g_close_group, __pyx_t_2) < (0)) __PYX_ERR(0, 1206, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1215
 * 
 * 
 *   def _g_move_node(self, hid_t oldparent, oldname, hid_t newparent, newname,             # <<<<<<<<<<<<<<
 *                    oldpathname, newpathname):
 *     cdef int ret
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Group_17_g_move_node, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Group__g_move_node, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[31])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Group, __pyx_mstate_global->__pyx_n_u_g_move_node, __pyx_t_2) < (0)) __PYX_ERR(0, 1215, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Group_19__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Group___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[32])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Group, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < (0)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Group, (type(self), 0xd52d8f4, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Group__set_state(self, __pyx_state)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Group_21__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Group___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[33])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Group, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < (0)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1235
 *   # Instance variables declared in .pxd
 * 
 *   def _get_storage_size(self):             # <<<<<<<<<<<<<<
 *       return H5Dget_storage_size(self.dataset_id)
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4Leaf_1_get_storage_size, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Leaf__get_storage_size, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[34])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Leaf, __pyx_mstate_global->__pyx_n_u_get_storage_size, __pyx_t_2) < (0)) __PYX_ERR(0, 1235, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1238
 *       return H5Dget_storage_size(self.dataset_id)
 * 
 *   def _get_obj_track_times(self):             # <<<<<<<<<<<<<<
 *     """Get track_times boolean for dataset
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4Leaf_3_get_obj_track_times, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Leaf__get_obj_track_times, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[35])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1238, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Leaf, __pyx_mstate_global->__pyx_n_u_get_obj_track_times, __pyx_t_2) < (0)) __PYX_ERR(0, 1238, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1271
 *     return bool(track_times)
 * 
 *   def _g_new(self, where, name, init):             # <<<<<<<<<<<<<<
 *     if init:
 *       # Put this info to 0 just when the class is initialized
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4Leaf_5_g_new, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Leaf__g_new, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[36])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1271, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Leaf, __pyx_mstate_global->__pyx_n_u_g_new, __pyx_t_2) < (0)) __PYX_ERR(0, 1271, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1280
 *     super()._g_new(where, name, init)
 * 
 *   def _g_chunk_info(self, ndarray coords):             # <<<<<<<<<<<<<<
 *     """Get storage information about chunk at `coords`.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4Leaf_7_g_chunk_info, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Leaf__g_chunk_info, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[37])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1280, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Leaf, __pyx_mstate_global->__pyx_n_u_g_chunk_info, __pyx_t_2) < (0)) __PYX_ERR(0, 1280, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1304
 *             else (None, None, None))
 * 
 *   def _g_read_chunk(self, ndarray coords, ndarray out):             # <<<<<<<<<<<<<<
 *     """Read the raw chunk at `coords` (into `out`).
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4Leaf_9_g_read_chunk, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Leaf__g_read_chunk, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[38])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Leaf, __pyx_mstate_global->__pyx_n_u_g_read_chunk, __pyx_t_2) < (0)) __PYX_ERR(0, 1304, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1334
 *     return rarr
 * 
 *   def _g_write_chunk(self, ndarray coords, ndarray data, uint32_t filters):             # <<<<<<<<<<<<<<
 *     """Write the raw `data` to the chunk in `coords`.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4Leaf_11_g_write_chunk, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Leaf__g_write_chunk, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[39])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1334, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Leaf, __pyx_mstate_global->__pyx_n_u_g_write_chunk, __pyx_t_2) < (0)) __PYX_ERR(0, 1334, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1399
 *   # can't do since cdef'd
 * 
 *   def _g_truncate(self, hsize_t size):             # <<<<<<<<<<<<<<
 *     """Truncate a Leaf to `size` nrows."""
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4Leaf_13_g_truncate, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Leaf__g_truncate, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[40])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1399, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Leaf, __pyx_mstate_global->__pyx_n_u_g_truncate, __pyx_t_2) < (0)) __PYX_ERR(0, 1399, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1421
 *       raise ValueError("Unexpected classname: %s" % classname)
 * 
 *   def _g_flush(self):             # <<<<<<<<<<<<<<
 *     # Flush the dataset (in fact, the entire buffers in file!)
 *     if self.dataset_id >= 0:
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4Leaf_15_g_flush, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Leaf__g_flush, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[41])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1421, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Leaf, __pyx_mstate_global->__pyx_n_u_g_flush, __pyx_t_2) < (0)) __PYX_ERR(0, 1421, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1426
 *         H5Fflush(self.dataset_id, H5F_SCOPE_GLOBAL)
 * 
 *   def _g_close(self):             # <<<<<<<<<<<<<<
 *     # Close dataset in HDF5 space
 *     # Release resources
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4Leaf_17_g_close, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Leaf__g_close, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[42])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Leaf, __pyx_mstate_global->__pyx_n_u_g_close, __pyx_t_2) < (0)) __PYX_ERR(0, 1426, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4Leaf_19__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Leaf___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[43])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < (0)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_4Leaf_21__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Leaf___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[44])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < (0)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1452
 *     return NULL
 * 
 * def _supports_opt_blosc2_read_write(byteorder, filter_list, file_mode):             # <<<<<<<<<<<<<<
 *     if len(filter_list) == 1:  # Blosc2 must be the only filter
 *       opt_write = ((byteorder == sys.byteorder)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_1_supports_opt_blosc2_read_write, 0, __pyx_mstate_global->__pyx_n_u_supports_opt_blosc2_read_write, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[45])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_supports_opt_blosc2_read_write, __pyx_t_2) < (0)) __PYX_ERR(0, 1452, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1469
 *   # Instance variables declared in .pxd
 * 
 *   def _create_array(self, ndarray nparr, object title, object atom):             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef herr_t ret
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Array_1_create_array, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Array__create_array, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[46])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Array, __pyx_mstate_global->__pyx_n_u_create_array, __pyx_t_2) < (0)) __PYX_ERR(0, 1469, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1538
 * 
 * 
 *   def _create_carray(self, object title):             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef herr_t ret
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Array_3_create_carray, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Array__create_carray, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[47])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1538, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Array, __pyx_mstate_global->__pyx_n_u_create_carray, __pyx_t_2) < (0)) __PYX_ERR(0, 1538, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1619
 * 
 * 
 *   def _open_array(self):             # <<<<<<<<<<<<<<
 *     cdef size_t type_size, type_precision
 *     cdef H5T_class_t class_id
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Array_5_open_array, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Array__open_array, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[48])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1619, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Array, __pyx_mstate_global->__pyx_n_u_open_array, __pyx_t_2) < (0)) __PYX_ERR(0, 1619, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1710
 * 
 * 
 *   def _append(self, ndarray nparr):             # <<<<<<<<<<<<<<
 *     cdef int ret, extdim
 *     cdef hsize_t *dims_arr
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Array_7_append, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Array__append, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[49])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1710, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Array, __pyx_mstate_global->__pyx_n_u_append_2, __pyx_t_2) < (0)) __PYX_ERR(0, 1710, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1742
 *     self.shape = tuple(shape)
 * 
 *   def _read_array(self, hsize_t start, hsize_t stop, hsize_t step,             # <<<<<<<<<<<<<<
 *                  ndarray nparr):
 *     cdef herr_t ret
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Array_9_read_array, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Array__read_array, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[50])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1742, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Array, __pyx_mstate_global->__pyx_n_u_read_array, __pyx_t_2) < (0)) __PYX_ERR(0, 1742, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1795
 * 
 * 
 *   def _g_read_slice(self, ndarray startl, ndarray stopl, ndarray stepl,             # <<<<<<<<<<<<<<
 *                    ndarray nparr):
 *     cdef herr_t ret
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Array_11_g_read_slice, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Array__g_read_slice, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[51])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1795, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Array, __pyx_mstate_global->__pyx_n_u_g_read_slice, __pyx_t_2) < (0)) __PYX_ERR(0, 1795, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1848
 * 
 * 
 *   def _g_read_coords(self, ndarray coords, ndarray nparr):             # <<<<<<<<<<<<<<
 *     """Read coordinates in an already created NumPy array."""
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Array_13_g_read_coords, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Array__g_read_coords, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[52])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1848, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Array, __pyx_mstate_global->__pyx_n_u_g_read_coords, __pyx_t_2) < (0)) __PYX_ERR(0, 1848, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1911
 * 
 * 
 *   def perform_selection(self, space_id, start, count, step, idx, mode):             # <<<<<<<<<<<<<<
 *     """Performs a selection using start/count/step in the given axis.
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Array_15perform_selection, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Array_perform_selection, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[53])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1911, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Array, __pyx_mstate_global->__pyx_n_u_perform_selection, __pyx_t_2) < (0)) __PYX_ERR(0, 1911, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1954
 *                         startp, stepp, countp, NULL)
 * 
 *   def _g_read_selection(self, object selection, ndarray nparr):             # <<<<<<<<<<<<<<
 *     """Read a selection in an already created NumPy array."""
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Array_17_g_read_selection, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Array__g_read_selection, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[54])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1954, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Array, __pyx_mstate_global->__pyx_n_u_g_read_selection, __pyx_t_2) < (0)) __PYX_ERR(0, 1954, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":2020
 * 
 * 
 *   def _g_write_slice(self, ndarray startl, ndarray stepl, ndarray countl,             # <<<<<<<<<<<<<<
 *                     ndarray nparr):
 *     """Write a slice in an already created NumPy array."""
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Array_19_g_write_slice, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Array__g_write_slice, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[55])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2020, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Array, __pyx_mstate_global->__pyx_n_u_g_write_slice, __pyx_t_2) < (0)) __PYX_ERR(0, 2020, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":2056
 * 
 * 
 *   def _g_write_coords(self, ndarray coords, ndarray nparr):             # <<<<<<<<<<<<<<
 *     """Write a selection in an already created NumPy array."""
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Array_21_g_write_coords, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Array__g_write_coords, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[56])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2056, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Array, __pyx_mstate_global->__pyx_n_u_g_write_coords, __pyx_t_2) < (0)) __PYX_ERR(0, 2056, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":2101
 * 
 * 
 *   def _g_write_selection(self, object selection, ndarray nparr):             # <<<<<<<<<<<<<<
 *     """Write a selection in an already created NumPy array."""
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Array_23_g_write_selection, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Array__g_write_selection, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[57])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_Array, __pyx_mstate_global->__pyx_n_u_g_write_selection, __pyx_t_2) < (0)) __PYX_ERR(0, 2101, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Array_27__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Array___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[58])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < (0)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for pickling"
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5Array_29__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Array___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[59])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < (0)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":2162
 *   cdef hsize_t nrecords
 * 
 *   def _create_array(self, object title):             # <<<<<<<<<<<<<<
 *     cdef int rank
 *     cdef hsize_t *dims
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_7VLArray_1_create_array, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_VLArray__create_array, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[60])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2162, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_VLArray, __pyx_mstate_global->__pyx_n_u_create_array, __pyx_t_2) < (0)) __PYX_ERR(0, 2162, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":2228
 * 
 * 
 *   def _open_array(self):             # <<<<<<<<<<<<<<
 *     cdef char cbyteorder[11]  # "irrelevant" fits easily here
 *     cdef int i, enumtype
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_7VLArray_3_open_array, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_VLArray__open_array, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[61])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_VLArray, __pyx_mstate_global->__pyx_n_u_open_array, __pyx_t_2) < (0)) __PYX_ERR(0, 2228, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":2272
 * 
 * 
 *   def _append(self, ndarray nparr, int nobjects):             # <<<<<<<<<<<<<<
 *     cdef int ret
 *     cdef void *rbuf
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_7VLArray_5_append, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_VLArray__append, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[62])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_VLArray, __pyx_mstate_global->__pyx_n_u_append_2, __pyx_t_2) < (0)) __PYX_ERR(0, 2272, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":2295
 *     self.nrecords = self.nrecords + 1
 * 
 *   def _modify(self, hsize_t nrow, ndarray nparr, int nobjects):             # <<<<<<<<<<<<<<
 *     cdef int ret
 *     cdef void *rbuf
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_7VLArray_7_modify, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_VLArray__modify, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[63])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2295, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_VLArray, __pyx_mstate_global->__pyx_n_u_modify, __pyx_t_2) < (0)) __PYX_ERR(0, 2295, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":2318
 *   # Because the size of each "row" is unknown, there is no easy way to
 *   # calculate this value
 *   def _get_memory_size(self):             # <<<<<<<<<<<<<<
 *     cdef hid_t space_id
 *     cdef hsize_t size
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_7VLArray_9_get_memory_size, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_VLArray__get_memory_size, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[64])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_VLArray, __pyx_mstate_global->__pyx_n_u_get_memory_size, __pyx_t_2) < (0)) __PYX_ERR(0, 2318, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":2339
 *     return size
 * 
 *   def _read_array(self, hsize_t start, hsize_t stop, hsize_t step):             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef size_t vllen
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_7VLArray_11_read_array, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_VLArray__read_array, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[65])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2339, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_VLArray, __pyx_mstate_global->__pyx_n_u_read_array, __pyx_t_2) < (0)) __PYX_ERR(0, 2339, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":2421
 * 
 * 
 *   def get_row_size(self, row):             # <<<<<<<<<<<<<<
 *     """Return the total size in bytes of all the elements contained in a given row."""
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_7VLArray_13get_row_size, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_VLArray_get_row_size, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[66])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2421, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_VLArray, __pyx_mstate_global->__pyx_n_u_get_row_size, __pyx_t_2) < (0)) __PYX_ERR(0, 2421, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_7VLArray_15__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_VLArray___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[67])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < (0)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_7VLArray_17__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_VLArray___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[68])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < (0)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":2458
 * cdef class UnImplemented(Leaf):
 * 
 *   def _open_unimplemented(self):             # <<<<<<<<<<<<<<
 *     cdef object shape
 *     cdef char cbyteorder[11]  # "irrelevant" fits easily here
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_13UnImplemented_1_open_unimplemented, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_UnImplemented__open_unimplemente, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[69])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_UnImplemented, __pyx_mstate_global->__pyx_n_u_open_unimplemented, __pyx_t_2) < (0)) __PYX_ERR(0, 2458, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":2474
 *     return (shape, byteorder, self.dataset_id)
 * 
 *   def _g_close(self):             # <<<<<<<<<<<<<<
 *     H5Dclose(self.dataset_id)
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_13UnImplemented_3_g_close, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_UnImplemented__g_close, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[70])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6tables_13hdf5extension_UnImplemented, __pyx_mstate_global->__pyx_n_u_g_close, __pyx_t_2) < (0)) __PYX_ERR(0, 2474, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_13UnImplemented_5__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_UnImplemented___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[71])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < (0)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.dims cannot be converted to a Python object for pickling"
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_13UnImplemented_7__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_UnImplemented___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[72])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < (0)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_File(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x3498a4d, 0x296486f, 0xfba3a2d, b'access_plist, file_id, name')
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_3__pyx_unpickle_File, 0, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_File, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[73])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_File, __pyx_t_2) < (0)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * cdef extern from *:             # <<<<<<<<<<<<<<
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_5__pyx_unpickle_AttributeSet, 0, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_AttributeSet, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[74])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_AttributeSet, __pyx_t_2) < (0)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_Node(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x75c7861, 0x87bccfa, 0x381b6b9, b'name, parent_id')
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_7__pyx_unpickle_Node, 0, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_Node, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[75])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_Node, __pyx_t_2) < (0)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * cdef extern from *:             # <<<<<<<<<<<<<<
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6tables_13hdf5extension_9__pyx_unpickle_Group, 0, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_Group, NULL, __pyx_mstate_global->__pyx_n_u_tables_hdf5extension, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[76])); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_Group, __pyx_t_2) < (0)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "tables/hdf5extension.pyx":1
 * ########################################################################             # <<<<<<<<<<<<<<
 * #
 * # License: BSD
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_test, __pyx_t_2) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  if (__pyx_m) {
    if (__pyx_mstate->__pyx_d && stringtab_initialized) {
      __Pyx_AddTraceback("init tables.hdf5extension", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    #if !CYTHON_USE_MODULE_STATE
    Py_CLEAR(__pyx_m);
    #else
    Py_DECREF(__pyx_m);
    if (pystate_addmodule_run) {
      PyObject *tp, *value, *tb;
      PyErr_Fetch(&tp, &value, &tb);
      PyState_RemoveModule(&__pyx_moduledef);
      PyErr_Restore(tp, value, tb);
    }
    #endif
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init tables.hdf5extension");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #else
  return __pyx_m;
  #endif
}
/* #### Code section: pystring_table ### */
/* #### Code section: cached_builtins ### */

static int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_super); if (!__pyx_builtin_super) __PYX_ERR(0, 1278, __pyx_L1_error)
  __pyx_builtin_object = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_object); if (!__pyx_builtin_object) __PYX_ERR(0, 1688, __pyx_L1_error)
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_enumerate); if (!__pyx_builtin_enumerate) __PYX_ERR(0, 1929, __pyx_L1_error)
  __pyx_builtin_map = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_map); if (!__pyx_builtin_map) __PYX_ERR(0, 2468, __pyx_L1_error)

  /* Cached unbound methods */
  __pyx_mstate->__pyx_umethod_PyDict_Type_get.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_get.method_name = &__pyx_mstate->__pyx_n_u_get;
  __pyx_mstate->__pyx_umethod_PyDict_Type_items.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_items.method_name = &__pyx_mstate->__pyx_n_u_items;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.method_name = &__pyx_mstate->__pyx_n_u_pop;
  __pyx_mstate->__pyx_umethod_PyDict_Type_values.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_values.method_name = &__pyx_mstate->__pyx_n_u_values;
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cached_constants ### */

static int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "tables/hdf5extension.pyx":1302
 *                          % self._v_pathname)
 *     return ((filter_mask, addr, size) if addr != HADDR_UNDEF
 *             else (None, None, None))             # <<<<<<<<<<<<<<
 * 
 *   def _g_read_chunk(self, ndarray coords, ndarray out):
*/
  __pyx_mstate_global->__pyx_tuple[0] = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_tuple[0])) __PYX_ERR(0, 1302, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[0]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[0]);

  /* "tables/hdf5extension.pyx":1584
 *       atom.dflt = dflts
 * 
 *     cdef hsize_t blocksize = int(os.environ.get("PT_DEFAULT_B2_BLOCKSIZE", "0"))             # <<<<<<<<<<<<<<
 *     # Create the CArray/EArray
 *     self.dataset_id = H5ARRAYOmake(self.parent_id, encoded_name, version,
*/
  __pyx_mstate_global->__pyx_tuple[1] = PyTuple_Pack(2, __pyx_mstate_global->__pyx_n_u_PT_DEFAULT_B2_BLOCKSIZE, __pyx_mstate_global->__pyx_kp_u_0); if (unlikely(!__pyx_mstate_global->__pyx_tuple[1])) __PYX_ERR(0, 1584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[1]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[1]);

  /* "tables/hdf5extension.pyx":420
 * 
 * _supported_drivers = (
 *     "H5FD_SEC2",             # <<<<<<<<<<<<<<
 *     "H5FD_DIRECT",
 *     #"H5FD_LOG",
*/
  __pyx_mstate_global->__pyx_tuple[2] = PyTuple_Pack(6, __pyx_mstate_global->__pyx_n_u_H5FD_SEC2, __pyx_mstate_global->__pyx_n_u_H5FD_DIRECT, __pyx_mstate_global->__pyx_n_u_H5FD_WINDOWS, __pyx_mstate_global->__pyx_n_u_H5FD_STDIO, __pyx_mstate_global->__pyx_n_u_H5FD_CORE, __pyx_mstate_global->__pyx_n_u_H5FD_SPLIT); if (unlikely(!__pyx_mstate_global->__pyx_tuple[2])) __PYX_ERR(0, 420, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[2]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[2]);
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_tuple;
    for (Py_ssize_t i=0; i<3; ++i) {
      #if PY_VERSION_HEX >= 0x030F0000
      PyUnstable_SetImmortal(table[i]);
      #elif CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      if ((PY_SSIZE_T_MAX <= _Py_IMMORTAL_REFCNT_LOCAL)) break;
      #if PY_VERSION_HEX < 0x030E0000
      if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
      #else
      if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
      #endif
      {
        Py_SET_REFCNT(table[i], ((Py_ssize_t)_Py_IMMORTAL_REFCNT_LOCAL + 1));
      }
      #else
      if ((PY_SSIZE_T_MAX < _Py_IMMORTAL_INITIAL_REFCNT)) break;
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}
/* #### Code section: init_constants ### */

static int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  {
    const struct { const unsigned int length: 11; } index[] = {{0},{1},{57},{10},{11},{26},{33},{33},{27},{21},{21},{36},{32},{63},{61},{82},{77},{21},{37},{20},{179},{25},{28},{31},{31},{29},{30},{22},{30},{25},{38},{48},{23},{30},{25},{34},{32},{32},{32},{32},{3},{47},{66},{39},{40},{65},{66},{45},{25},{34},{40},{31},{37},{99},{127},{43},{28},{33},{24},{32},{53},{51},{1},{2},{3},{1},{8},{104},{19},{25},{7},{6},{16},{18},{3},{2},{3},{12},{9},{11},{32},{32},{9},{5},{39},{34},{33},{2},{5},{2},{61},{90},{11},{18},{18},{17},{24},{12},{9},{13},{15},{10},{12},{21},{15},{4},{15},{3},{38},{11},{5},{45},{3},{5},{23},{25},{13},{19},{20},{20},{23},{19},{21},{24},{20},{17},{17},{23},{4},{12},{30},{32},{23},{25},{19},{22},{23},{6},{18},{19},{16},{6},{25},{17},{21},{23},{24},{23},{21},{20},{15},{6},{12},{4},{22},{24},{16},{16},{11},{17},{11},{19},{17},{23},{5},{23},{25},{20},{15},{20},{24},{24},{20},{19},{18},{13},{9},{11},{9},{10},{10},{12},{18},{19},{12},{4},{22},{24},{18},{13},{13},{11},{18},{16},{19},{25},{22},{17},{19},{4},{9},{10},{4},{22},{24},{14},{11},{18},{24},{2},{7},{13},{23},{18},{20},{1},{8},{8},{5},{1},{15},{13},{31},{33},{22},{33},{7},{1},{7},{25},{27},{15},{21},{24},{15},{19},{19},{20},{1},{1},{12},{4},{12},{6},{7},{4},{5},{6},{18},{5},{4},{5},{19},{17},{12},{12},{11},{11},{9},{8},{1},{2},{13},{4},{8},{9},{6},{20},{5},{3},{7},{6},{9},{10},{6},{8},{1},{10},{9},{10},{17},{10},{11},{9},{6},{9},{6},{17},{8},{9},{18},{11},{5},{11},{9},{7},{6},{4},{17},{5},{6},{6},{6},{13},{14},{18},{12},{4},{5},{1},{4},{9},{8},{16},{11},{10},{4},{5},{8},{5},{4},{8},{6},{7},{5},{6},{5},{1},{5},{12},{11},{7},{6},{15},{17},{16},{18},{17},{12},{15},{15},{13},{9},{8},{7},{3},{10},{6},{6},{1},{11},{9},{8},{6},{9},{11},{11},{11},{7},{5},{10},{5},{11},{5},{10},{8},{13},{8},{14},{9},{9},{8},{14},{18},{18},{14},{10},{12},{13},{12},{6},{7},{13},{14},{17},{13},{9},{10},{11},{14},{15},{18},{14},{9},{7},{3},{12},{14},{12},{11},{16},{13},{19},{20},{12},{17},{18},{12},{10},{4},{6},{20},{19},{1},{2},{3},{5},{11},{9},{4},{6},{5},{4},{9},{13},{11},{9},{5},{8},{4},{7},{9},{5},{8},{7},{3},{12},{8},{8},{9},{13},{4},{7},{10},{5},{1},{4},{8},{10},{11},{7},{3},{7},{9},{7},{7},{9},{11},{8},{4},{9},{2},{5},{25},{8},{4},{5},{5},{6},{6},{5},{7},{9},{11},{11},{19},{8},{9},{2},{3},{6},{6},{4},{17},{6},{8},{18},{8},{3},{16},{8},{14},{16},{6},{14},{12},{11},{10},{27},{19},{20},{19},{14},{12},{1},{11},{4},{4},{7},{8},{12},{4},{2},{5},{11},{10},{17},{13},{6},{9},{3},{8},{3},{6},{6},{5},{11},{12},{9},{4},{22},{21},{12},{10},{12},{19},{5},{6},{10},{18},{4},{8},{5},{6},{6},{6},{10},{5},{4},{5},{5},{5},{4},{5},{4},{9},{10},{6},{10},{5},{18},{31},{3},{6},{20},{4},{8},{4},{6},{5},{11},{4},{5},{7},{14},{9},{1},{2},{6},{12},{15},{4},{5},{14},{8},{7},{7},{6},{7},{11},{9},{11},{16},{10},{5},{6},{7},{5},{1},{17},{4},{8},{4},{5},{7},{9},{1},{5},{0},{168},{1},{63},{11},{418},{487},{72},{9},{47},{78},{64},{34},{86},{562},{228},{71},{11},{11},{173},{98},{11},{11},{13},{13},{430},{81},{126},{181},{18},{568},{227},{310},{306},{269},{298},{14},{18},{837},{9},{101},{105},{97},{105},{18},{142},{1161},{65},{74},{97},{52},{94},{93},{75},{55},{55},{55},{445},{144},{109},{52},{94},{90},{146},{192},{188},{192},{155},{84},{84},{24}};
    #if (CYTHON_COMPRESS_STRINGS) == 3 && __PYX_LIMITED_VERSION_HEX >= 0x030e0000 /* compression: zstd (8057 bytes) */
const char* const cstring = "(\265/\375`\306K}\373\000J\336\244/H\360\352\352\001\314\300D\310O\207-\023\017\004lH\2214`C\n\013\007\263tA\300\212vJ\272\343E\314>\243\355`\010\002\340\021-\233\230\030\272;N\020\264\311\213\006M\265\010\270?5d\342\211'6;\367}s\325\221\010\272\002\306\002\002\003?\351y\003\004\203\365\320zD\031m\002l\024i0\t#\255\346Fg&\263\003\023\2033Ss#\2164\034\230\004\230,\006\2076\033gpjlbD\030\232\216\004\030L\2463\243I\270\201\341\2140\232\0301f\264\301\331\241\211\211\351{\363\335NG\277g\273\253\244\215\364\343H\216\233d\313\027\216\270\373m\243'\327:\027\024\323\263\273LY\266\257X\0036\2757\367%)\327;\364\364,\257{\215n\025\210\267\266z\272=\320^1\313\033\202\265\253d\253{ \215\033O\320\037\362\\u@\355S\225l\231\350\212\243\230\360\025\307\342N\223\305\265\366\212c\323\225[\013\207R\366\363\347US{\001M/'i\353+\223\265\373Y\266\366\256\261Z\376<\351\234\355z\243+\310\300\033(\031\367\215,\367>\307\262\324\020\300\006`\206\014\255\326\336\334\004_\3061\335I\311\275\002\355\037REk\235\355}\223d\253\357\262oT\007l\355\rg\337\233#\250\251\255m\323\270\223\363\270\212S7\216\334(\252\323\035\225\357\r)\256\225\233\rL\001Ku\247\2564\030\205\232\331h\023\223s\0033\333\351\033\344\241z\326\223\363\302\251\351\241\305\300d5\257\361%\263BSn\256\246\266\023E\317\335\306i\256\327\355y\235\305\201\260\031QB\315\010\223\345\304\2142\030O\215\354E\353\253\231\352_L\275Q2e\271\327\272V\230r\325\257''\373.3d\350\253\237\224\275\221\253\026Os.\336\360l\3644g\272\332\3525\225\353\200/\267\016\252\334\263\253gs\300\027pE\346E\223n\323\314\001[\254\326\356\250\226\341\254\221\225v\275\356\235\202\254\312\003\315\032\260\205\236l\255\357\3365J\326\272'\267\2062\325\240\212\213\n\212\016L\007L\311\016\362\320\3555\337\265\023\335%\330\317\233\347\305\200^\2538\0312\244@\033%Uq\370\326\235|{\256\256+\333\313\023\325\272M\236\350,\307\300\271\002w\251\222\345N9\353{\311Ur\363\321G\224\311\304\025\232\022\260\373t\3436\272y\346Y\2528\3167\210\352\224o\237\326\364\215;\375\370Cj\356\313\023\247r""\023\364\375DA\236\347|=w\327^\203%\312\2521~\363@\177\362]\337d\272\3716U\017\226\253\232\352^\262\372\212\005\276\244{U\367\312\363\032T\315\364\004;\356,A\277m\255\276\002WV\345\266\355\365\346$\350Y\251\216a\201\333\263\265\223{/\266\302\276S\264\246\321S\3534\270o\276\334\233\004\332`\337\251\0344Up\333R\315A\rr\325\004m\351\371\323\305\004\0019\230BL6\361i\330G\025\372\\\262\201\370\302\233\001J\374b\241\333\362Q-I\304\177\003\2276lM\022\276\\\203\2229$\312\250\230\024\222_\261+>\301%*\237\030\213b\022\370\200\312\027\366pb\024\374\302}\212\266\004\362\212o\217k\362\313\226B\344\013\343T\314\202\373\033\363.p],\\\236\201\353\373V\267\nZT\3545qM\337\002\345+\271\334\013\367\201\216Uq\t\234\202\rt\314\035\244\231$\362\305\\\335\327_\203\"\002\200D\210\324\247\265\023\022\276\224\313\362\303\337\222kU~\342\214\313~\225\217!\362\245\336\320\313\341\302~\206\350\347rC\"\000X\204\260\314\353z\013\200\320?\366_\332\223\310\247]\221\237\022\375R=\232\234\"3 \371B\356M\213\222\360\305\332\224\326\203\030\211|!9\027\243\300\246\367\345\272\\\020\323|\212\000\260\021\"\004\001@E\001\214\250\306\001\000\300#u\300\223\033x\222\236%\240\277\0010\320\271\276\221%\007rx:\035h\267\371\276\355vz7\356P]y\226\352n\353t\243\213\351\331\221\227+\353b\331\223<UX\306\355;\257\367qd\373\310Ut\\\267\3577\177\356Vt\364\334\237\265[\037\273\3567\215\0337\331W\356GNr\316\177RT\263\271Q\207\303z\224\355\233\343\330\276\216\274\265\217\233\263\253_GT\237}U\327\336U\277\252\252\337\3354\325\236\246\335\257\033\35047_\327\224\247k\277y\256\3271\335,\252ueZ\246+\353\313t\243\354H\352\025\267j\\z\216\350Y\366\224\321T\327\361<S\024\370\364\\G\033\356\004*O\227\363\\\323\225\233\257\255\334\310s\333\312sd\315q4S\024p\277m\0325\373O\3278\276}gGT\335hir\276S\225\237:\034[\242\352n\217\245\272\323\234L7\317\257\270\275{Y\226\0365K\322\357N\256\350\350SV\0359o\255\235\350\326\251$\271Q\334H\222dy\216\344H\256z7\366>\326\266\347\347n\034S\310\276I]s\325j#\367\372\326K\362T\375\310\315Q\367\255\236d""\312\353\272\353n\343z\034\371:n\034O\272S\323\2373e\313\275\342j\322\235\035\2327\325\331lfn\231\007\3328\243'\317\027\020Og\034g2\032M\034g>\263q\006\273\201\021i>:)7;\242\014L\207V\243\241\226\343\222\323W\034\316\215dg*N\271q\032\007\206\226\203#\322\304\327\354%\302d4\2641f6A\224\300U\337\307\321f\352}\003\203\361\210495/ZJ\014l6\242%z\336H\233\3156\252iy\372}\353,jn\231\0270\320;z\301\016\004\257I\264\022\273\300& \363L\207o\203r+\364\2307q\302wQ\372+\255\000\021\315\210+\016u.\201\020\n$S8\342z\374\324,\006\243\250\034\251\336\203\270\003\032d\223`v\t\2753W$\2642\221`\300\033\\\231k\333\340:)\361\305z\023B\362\r\353\365'\267\345z\221h\270`\304\220\204\230\374\022\013X\360\362\270\023~J/\354\225\220\261~a^V\301*G\321\023\372\2716%\001\032\312\001d\007\246\2674G$h\205?\300\347\215\3748\214\200c\376\211\256\310\010\300\007\210\207\206\031\274\005@|)Y\002\354!tqN\\\016\026L%_\252\r\3414\2344\257\2550\221\r~$,\277\220LBRp\222([\225\353d\004\026\323P\314\025\245\324\347\037f3\330i#\332\n\321\257\364\353a\210_l?\335\213\365t\262\226w\261\216YH\346\010\244\031\001,\216\231\203\324\340c\220~\032\216\301\0210i\303-}\013\322\357d\"\371/\373\272V\326\352zP\363\204\350r?~\237&~Q.\266\201]\260I\305\\Ar\3617\310\014X\263\315(\227\202C%\267\243\345\375\350\3408\211\251\211\r\023\351\347\247\245 \315\332\211\222/\024\200\361\271\376',\251\230\005\016\341\003)\230\345\215\361\270%\024\267^\364\253X\241\240VGl\201\346\227\2035\254\340\203c#\224!\240\270\345FF\nf\2752a\334\002\373Wc\236\010a(\322/\006\027\200[T\334\375w\355F\350\247J\356\354\207\204\360\005W$d\342\344_H\260H\374\232\216op\353@J\246\305&\246\2147\034\365x\024\277`\224\005\007\315\327\030\275#\241i6\375\014\211\237\0304\022tA\316\220Yq\t\213\2744WtCFs<\022\313\344\336\r\021\355\213\237\246\364#\264(R\004\027#\243-I\370D^;\372\275/.H\016V\201q\256\nA*\346\360\206\370\232'\r\303\300\346\351\227\263\301*\t\256\310\315}\216\321Lz\275\253\335\001\247\231\024\177\300\264\327\346\016\370(\2050\3703\023)|\362\226\266\320""\253\241\300O}X\213\212\007\300\026\334\214\t\224\214\003\213\\L=/\313d\007:\246\200[\022\242,\222k1\213F\006J\214\363\242\334\227)#'\253\310\027\362\202\\\370K\332\023\010\027\253\306@k\022\375\302\212o\241\337\235\013\364;$<\344^\2257|!\374\023)\326{\221},\213W<\346\002:\224Q\370\026D\322\203\020{\370L\3054\330\366\222\214\370B\275V\214\276\2527\365\376\020 \3159\261\347\352u\210\033\3003\257\350\315\271\025\242\200Or\275\027/\213;\363]\237'\026g\344\334\225\233r\311\224\277\365O\2731\t_\350\312T\274\005\336\226\230\317\2661-\353\362\374\t!\271\005\227/\203\212X\274\262\232}\001\341{!t\273\350b\204\032bU\314\r\371\027#\237)(\002\r\013l\242R\365L\362+\366\200\322\370%\027\274;T\257\324\006x\224\353\251\364>d\332k\224\213u|\002\267|@Ep\370\304^*\272Ud\244\202BA\032\344\010\016bA\266\236D\217$\273\224\314\032\024\010x\301\r\001\361Lv\373\033\026\262\202\031\210\353\201\201\361\222\016z\004\370\004\325\"\032y\375r\262\r\211\217Q\3402\245d\000\365_\255+\364\336\344(\374\014\221\226a\344\006\031(aP\t9\212yf\212\334)py\276\016\346@{\372\215\364tODf\211y@\216\237\2051\366k\"\260+\034J=?R\204\036\236\371\026\244\n\362\350u\024\2301|\215\002f\253!\357X#\206\301\354\016\313\304H\216\225\013J\345\017\260\206[@\0241\211\264k\371\035\200d\261\371\247\307@>\305\t\300\371\242%2\233l\330bq\r\320j6\251\360\350\270 a\r\270t\301\234\200KX!\032\302\007\0360\346\214\016\017_H^\331\357-\344:\020G\036&,$\241\r\030\032&\027W\370\214\204\010`\017\036O\214\323\327\247\205M\006\370\322\013\324\267\010\345\037\230\"\0262\323\374\214\r\337\000\221\002\"\256\2220\026\343H\rH-Ha\360\371\033#\r\375\302\365\350\006f\362%\351\370\251x\001\272w\365\226\253\2378\023\263\260\014\316\200\1774>/\275$\r\264\374 \234\315\277Q\300]\221N\017\276\022\231\330\205\300\366j\250~K\370M*)g\374\032\021\002\340\225\216\260\027\240\267\"c\330I\221\000\231C\267\264\010\2317R\237L\365\270\014\241Ge\243P0`\036@`&p\"B\205r\211\313|c\357\312\367q\232u+\021r=&Ba\\\302\021\010\314\364@PR?\261\327\343\032\201\21032\010\355]""\221`'\034!\020\261o\322\2328X\037\207\367\024\t\301<\357\004\223\311E\255\374\335V\234\014\374\013S\030\357\274^\214w\327VP\370\264\253a\341[\035\010c\261\030\037\240\360\205\370\357\230\330i)N*\370\000\210\242\320N\230|aP*\310\202\354\240A\215\2741\247\\Hf\325\313\341\307PL\270\270\027\027dm\256Q\230)\360\314>A\272\336\014\313\3541\227H\243\005\277[\210\215\304('\222\223!\250\370\302^)?P\211\340(\365\034\304\030J#\277\367\270\374\214RH\256\260\243e\006$\213\323\356\027c\277\354k\230H\037p\013%\325\373\211}`\021>\271\230\303\267\273g\200\010\325\307\351\005\t\370\264\034\223kx\242; \002\300\200\330\317\370\023s*\250f\306\307\003\353^\026\350\243\3403\373\354\314\023A\257\336\2535\253(fS\351\303\205\003\245\350\215z\034\362\206\237\220nw\332\205qMTL\256\300\261\201#\014\335\017\237\003!\243\211\\\317F6\035l\230r\021\276-Q!\024^\300\0221\030s\350\327\301\365\260\t\211\020\25040o3\252\"\224jhF\000\000\006\005c00\200\220H$\226\317\207\225z$(\317I\024\2028\024b$\346\014\204\206\210P@\000\020\002\000\202\002\204\000~\351~\2304@\236\307wh\244\352\273\030\375\306\371?\231^\313\302`\322\000\023titHy\267\207\0349U<*c\252\013\315K\323JtV\251\350[n\210\307\262)Us\307\207\265kr\351p.\024\226k&\034F&-\031h\250D\034\354\353\177\372\312\252\221\3620V\24247\010\212w\220\222\366\274K3=\010x&\353\020\221/h\314(\342\346D\372\330m\305;I\302\266\244\024\323\202\363\003\230\223\032\250\255\200X\3507\364\025+\336\350\020nl\244F(\317-<5A\371\020\237\205#~\331F\375h\210\237\330\2453\301n\035oU\244kx%vl[\037.\340\340\213G\231\324]\3779r\\R\2741=\302 \016\324\302,\350\260\307\325=\035\rC\007IZ\3072\327\014\245O\177\331\311;U\201\210\274\227M\225\233\324\341\000\342m\243a\234*4\207O\3429\306\231\0176+\346\364\271\21644?|t\336\205\226o.\221$\311=C\311\370\224UK\032\264o\237\214\321\265\222\343\030!\307Gy\277\350\264\300\224\n\030\302L(zu`\025\345\231\nFR\341\275!\353;;\\\010\235\tI\343\036d\2453\021\312\373\026\023\222 \353F\006=^\031^\366b\221hK2\201\201\363\332N\332\035\201\331\324\341\035""\361\227\243\360\032\274\0261\202E\305gP\346+'J6W\271t\004\017\230\002;\306\337 \022\261 w\305\220\231\000\317\007\367:\372\343\370{\243\231y_w\226\260\276\303\371\237\323\360\202#N%\035zR\233%X\323\213\"\n\377\n\371\233\302\345\377\351\236\211\321\242YmA\377\263V>\037v\2228\236\224d\002\002\202\321M4\377\336\350\005\212\205\035A\356\222\177\275\252\345\036|\325\036\214\313|L\255\356=\301\241\014j\220\340\032>\361\376\372\302\006\330\n\346A\274\313\357\303\2044Y\023\211A\3650=\362aM\r\213c\256\363G\314\034+\253\031\336-\0053{\246\307\322~\023\315=\304Y/$\317xc\026\245\333\336\305PW\276\005\365\315\026\363\331!\327\246\001&Rp\312\360P\321\260x2Nte\226r\0035\034-q\260sb\275\232fQ@\207\000\303\030'<\255\203\001\350J\2579@\22567\204%\232\327@`\"\214\323\246\354 \243c\007?6d\232\303\221\016d\2435\351\353+\350U<\213\346F\225\220\352\334\250\023|\004\227;a\014\273\217L_a\364z\\\000K\305(zW\326blYBi\246\211i\374\336\323Y\320\330\036\001\321\3069;\216F\020\301\226\237g-\206a\211G\002\342D\005Ri\031\021`g3b-\035b\020n\305\234\210\206A\374\371N`\223\360\261\031\323bbW\307\000$\220\274\345%\022\202T\226\357%\240!\314~\261\210\037\022!\235\243z\352\307\010S\245`<\316\003W\340*\035\270#@\355\023\210~\342\317>\002k\364\252\002\036\266\254\340\230\n\036\321\2441r/j\270\220\256\264\376hVf3\003\215\321\264sf\364\031\370\005\260\252\016(\262S\255:\247jLS0n\371\206\031\034\365\221\251\235\000\322\220*\240\323\242\325\236\036\326\324\300\003\220\270AC\027\256\306\221b8\365&\241\374c\313\345L\010\323\353\003\256\0239z\n\t\2121\007H\265-F\276\247i\2130\371Z\365>\014\324)\2746\034\262\007\013Xz\210\013\014\247^\314'\203\353_w\036\017\252\017S.O\005\327+\376=\367xI\034\330\226\343\266z\362\275/\231ES\\`/\242\274\221l\257\004\262\246\254C\333\rk\343\207B\2630\277\024\351h\337\216\356\376:B\204\334\316BM\272z\013\261\331]\"\211i\335\001\021J\024i*\344\267\362:=\377\366?\350\036j\013[#@6e\270\241%t\252U\206\032\021\236&,\231\245\n7!\2203.0~\333 \341\324\367z\350\026\307ocs\t+\304{{\027\343`d""\325b\024+T<C\311#\263\206\224\276\363\307\311&\254\337l\335\344M\231\030\216\350\225j\261\"\376k\030\361\323\301\303SY\321\020\230p\2602\351n\305\333\307\226\036\233\267:P\365\206\251\367\201\201q\272\260C\233\333\353\010V>\356\330\213\254I\010\n\324\341\006\016\305\221M\002D\014kj\245\234b,f/\3550\001\262\231\004]\224\214\250\362\016\005\267\241\031\025\207YDX@\313sGU\267\340\323\017\3753\263\331\360\310H9N\016\264)\213\262^\n;\226h\221\210\340T\007\270\231z\366\345\300wfD\014z\373\003\030\245u8\270\214\200\326p;\317\272\207)\2542\356\226xM4\010\375\004\224\223\271q\02370\333\020\035\235\317.L\263'\"\024\224bAw-lsY\335S\240?\376\360\237\024G\224\335\030\326\225\320\2119\232\204\231\t\254\213F\262: SV\301\342\321\251\314ko_\006\314|\364N\344V\201`\251\027+\365\3269k\030Z,%\202-\360\366\013\265R\332(\236\232\242\367i\234\360\n\262\364\320U}\332\272\207\364\025\232CF\375C\205&\301\306\"y\324\214\203Y\375\212\3070h\225\322\215\2571\267\265$\030\310\215]]j\257P\216\000\216\252Rz\333GM\224E\264K\016F^\020yu\"\350\366f\021t*\351{\034\365\311X\320\014\325\005L\261\377ax#u\306\354Xe  AF\200\270z\265\314\027t\246\tVn+U\026J\221\225\304\\\327\016\344C?\336$\376I>\355\354\202\351\322\324%\0352\005[.Mc\266\370\350XT\376\000\216F;\370\302b\314\256\027\007\377\037@|o\254\245\254=[L\037\245]\253\303w\033`\206^\376\274\343\"\346\211\221\255H\n%\324}\n\277[\324\213_\2468nN\302\230\037\016Dz\031\020:\362,\217j\364\240\376\310\343P\243\326\037M\032V\014\224e\300\001\035\022\270\302\215\273\320\304N\\\023#\026l\2008D\250D\371\021\226\315q\317\034\231\333\213\303x\344-6\364\211\354E\364?\321W\220\326gC\224V\007p\236\265\n\014=/\0316/x\"O\256\227\211\331\256l\032G\352^\332\346.k\324\367H\267.C\3627KJG\376\031\254\227\305\tB\334P\205\006td\345\233\223}Wzd\032\256\300\377f\215\362K<\014\273\253\300G\266\340\312\307\227\3319M\002\235\"n\223*\223\304\372\2656\026:j\312\320\250E\035M\212\302\004\363\035UX\200\014\213\202\025\320\352\250r\227w\325u\226\271\014\314\311\022\337\245ms\327w\340""\331\322vM\212]j%\331\001\323\025\357\216\206\205\254\2316s\000\225\201\377\364\303\3449\0276\300\343N\303\340\320\253\rq\232\304p?B\361A&ypvW2\250xS\365\225\203 \344\253b\2426\014$\315F#U\3476\250'\315\3724\236\246\375< \027cz\241\272\315\200\214e\346\251M}\332\265\344\333\237@\217\354$\035\240\353m\221\300N\2021\235\323T\362\317D\204\210\004\312.\001\243\007\356\273\356f\316\000\231\3241P9G\027\300N(\241pB\035W\346\230cH\250\202\277\032ce\342\313\307$x!\026KP\n\360\253\274\277\2214\214J\352\206\244tW\020\277=qn\324\225J[i:\217\213Y&P&@\306\352\002\361\250\017D\301@,\221\224\316\267\305.C\014\210\014]U6\323\2333\271\353 w\243\240\005`\342\\\3323f\266\273?\2030\312\337\250\222>\242V\277\0045V7\212fa\031\236g\210<\210\000\214d\247w\315e\216\227\227e\034\230\233\241\2327K\305d\326\265P\376\242\326Y\352\216\310\346i\244B)\010\213D\002^\035\"\200\225\217\240thmx<\352^\241\002\251\317\224(uQm\366}\202L\372A7\340\3423\305O\0241j\321\230\211m\323\321t\314\r\001JR_l\356AQ\237\226`;\212>y\314\200\027UU\231*B)\202\020\023?}\303\035\354\340&\364\376\n@\326=\217\357W\273a\235\311\205I8\010\217\230;\001\237|\217\222\341\360\375\363y^y\216y\320\3577\265\372\372[\376\326\375\034W\236\034\277`\201\037b+.\313\335n\013\336xN\233\340me#\3676\377gS\224\033\243\206\371@\317\321\305\307Up\377\031W\347]\344`G\260\003H\013l\\<\215\023d\344\336\036\301<\307\025\334\254\201\006\203!\006\007\024\310\361=\327z\r\013\005\345\350\t\367jr\034\337\"k8\210#\231B\314\361\211\333\364=\237|\226#nn\365\351/(\222\372\253\236b\2257\2635\030:\271\277n\230yw\331~\030Z\241CN\024JFr\325\362p'1\310\177\254\276\322\021\007\273\266\353\261i\346\344&\217\202Q\244\374\224\033\022\277NH\033\220\301\266\311B\322\031d\263\240?\256\367Ri\320\265\254@\303\350\023C/j|wD0\331\003*\253\234I0G\277\200\\\204\250\254P\2656H\270\3212\264\221G<\314\210X\320\237\276\375\216\002\257F`\020it0\267]T\251\234\322T\332\310\241\"\265\032\332\373u\311q\264\352T\303As\354}m\224\205\260\0106\227\030\177\375\261v\031XA\033\2723'\004\270""\277\016\023*\357\360\342\341\200l\261\310\201\210pS\007\000\250\214\343\310\375\303\215\002\000\356<\271j\345\366\034R\016\016\026\255\005\365\271\311e|\255\330\016~\372\366\013\313\276\333\341\310a\203K-\026\374\036\036n(\302Z\014\374\177\0206y\213\347\276z\350\365 =\336\246\237\313AO=s8\000\210\216\374<\216\324\254`,/\322U\r\372}\002B\361\316\367\222\354tr{\000*\004\"[\034\373\320C\235\033\362<\254>a\352\250\345$\010\022\022\025\017Jii\311mI\221\r\227r\203>R\003q9\222\214\237\350\016\320s\223\226(\352\361!\332\362\366\016\327\267\272XRx\334\3636\030#,\235\331\0379\241\244\031\305q[wX\226\036P\206CX\256\030\211\273\001\310\025\2146\205\304y\022\261\000R\251\020.\343j\275\322\203\277\262\342U\210\206\364\215\333(p\236\352\366\343'2\313\0332\r\307\273{9\300\204\254z\2068\026\332?\"\247\354\312\035bUZ\273R\346\"q\364QV\302\313\226\342F\352'd\003i\003\265h\306gSv\333\021\345\234X1y\020`\261+\277\302b\002\013\361j\216\027\263\221\000\252\235\252\021\024\374kA\004\243\201\023\352kh\363G\3750>.\342\0300[\202\326\335I\355\351\274\321\000\024\033(\314\361'\212\306N\346\251]z\271qz{\024\206\310\370`\367\223[\017\211pbP\030\341\215PA\250\355\246\323\304\220,\344\240&\335\246r\225$to\375\243\364z5H\261\345\201e\222\03369;\230\326h\025\204t\210\260\315U\307\034&\215\036\027J\177\354\303{\214\335\322\241\256\202IH\202=\241\004q<\037\237<3\315r\035\243F\252\003\205+\337\022\265\333\262x:D\315\2026H\335\276L\336\377\236\251\307L&b\206\030\360\207\231\213\207'\336Q8\227\254\340\0220\325\322n\273\311\2606\244+1\210\252\345yF\336.S\221\020\n\267j\376\250\226\030\210\377n\t\312\357\017\224\027\"\223\364\2245\217\366t\007t\307\352\361J\244\017\214\226\244\351\301w\247\301\251zL\266T\305\256\316\004$\023rTO\036\360km0\375/\007T\374\321Jy\257*\324\220\355\227$ 4\003%\014\267\273:\037*('\347\255\237\256?\356Hh\340G\rvx\257Z*vL\t\030\344t\273\316v\021{\372<'\313(\304S\270$\240\007Up\213\213u\010jz7\021\305\205\370J\220M\356\010\341\200\370\340\314\355l\023E\343Hv\215d\241\313\373 \371""\353\022\221'\234{\311\033qiq\3058\253\243[\300f\373\266\267\317\371>o\325z2\370\311Rg\330\344\223h P&?ij[\020\257\014\277(\030\317\320\244\345\r\371N\234\033.cLS\203\266[\"9\220\014'\237LC\244*\003Z\376w\323a\006\277\321\005\001\317*\324|\370BO\013\315\"\222\232/\371\002L\330\2649\021\330\203\3206\000\341\372\214X\300\336\353\235 \227\337\315!\010\226\256)u\361w\032\035\322,\344\226\275,\336\227O\217\261\312\241#AP\252\034\252\370R-\222\320\370\037\332\237X;\021\231'\240\363\031J\005\344\324\300\270\353\330\330]\273i\266\235\315\327\277Y\004\370\000\240\323Kb^`#\014\315\362mD\342\310\225E\021\\\024\332\027?\020\243-/\234\010\332C.F\370E\355-\232\346\365\316[G|\373\275\350`\"y\305b\254\321\0070D\212\331\202\217\217a#\253\\\3007-\021'\312\234\014/T\367\305\304\237\316\340\000\316\031\354\331_<\231C\235\251\324!\310\030Zk\365\250\314@\036Y\336\347G\213\332P\251\214\032\337\265\n\231\250\3661!\304\033Y\276\330\325\3030\265w\247\327\240\n>C\250M\277=T\332\314\347$\374\276\366\276y!\177\344\214\343\325\270Po\270\342+\232G\035\020\370\206\006\230O\250\240\267\0215\252\36084\242\307\323\246\005\247h\257\364\372\305\030\005\210\016\357y\220\n-\255\0246\202\r\261\203\263H\230\2577q\255\013^<E\327\312\236\245*\017\264\235\036\250\261\370Ag\346.\332\317?\227J\362\023\363\275\224\213K\n)W!= 8\262\254\031\354\324@T\337\254W\310\335\214)RV\320\351,i9U\342\021\013\274\333D\270S\336.\232*=;\276I\331L\230\202]\361R\330SoHN\311t\375<\354\362\017\202V\250\002\241\325\t\322:\337\270\274\003\315Q\n\264`;Qfn\243a\347Mpd|\330\344\024\0170\316if\260\326[\227\\Z\3301[q\365\306\201^/\363@\206+\2502f\371S\355\247\225\335T=\3720\356*i\027x\261\325\027E>l\013\n\216\r\n~\022\002\300/\275\266=0l\323A\023\371b\236\r\014u\360&\267\247\311\002\303 \217\031\006*.\030\230\340\301`\350A\227W+A\260r\364/o\017\273\340\367\37718k\002\225!p\372\336+\363r\236\310\3301\020\267Z>\t\357\270\336\376\333g\354\270\244\021\367\336|Tng!\321\016JC\241\344Q\254h.t\"\025\016M\200\010\177H0\374J?\341,\225\256y\310\246I""\027(+*\205\372\004\365\010\3147\241\003\365%&\330\010c*\267\263P\255\214\326q\250\351t'j\330\275\005\ni\3012\247l\206'\260\325\350\005\177\2521\r\277\216|\023\351\022OX\277\343:\202\307\336o\263\362\242U\013sp\365\204\032\222\314\313\353FzC\353M\313\322r\321\242\242\242\315f\002\215K\364\234\213\277B\351\277\347P\026\243\313k;\345|\260\322s0\000}\316/K\301sf[\205~\213[\331\313s\226\316EUl\306\357|\216\007\231a\240\336\031\023.Hj\355\031I\257\031\006\026P\345\n>\nx\334\025\320\023tS\334GK*\354\315~!o\321\207|\214(T`\266S\210T\222`3\n_\310T0\007\276\2606\361Db\226h\244|\314\332\3019\202\271\354 \332x#\023q\261 -T\222\330\007\311\257\2049\243KO>\241Al\362\260{V0\235\0131\320\023\272-f*:5\207z\002\247\032\211\320]\340\017\334@\001\202w\177#\333\306\034\301\224Tu\347\230d\222\3072\014\000rrb\255\004\326\3713\007\200\270\t\033\026\226\311\033|G\203-\234\263\034TyR\361@~n\321Tg\335\311\252\206\373(\234\357\377\207\200\016>E\377\343~-59\"\246\316\201\371\222\217i\344\355x;\272\266I\256\005\234P\"\002\225?\220\246\324\037\212\3269\303\007\321+\232\224\200\330\344k\242\303\033(\260\025M\216\010\230\353\311\034\006\275{;T\323QH\332\365\017@\3413\303\267'\305\353g\254\226\252\025X\341\ru\251\260\216Y\322\237\022\343Wi%\360\362\274\357%'\025\326\246J\177\230\325o[\345Z\223\267\177\314\302\026\306\367Z\225\017\250\227 \3428N3\303E\030\361\362\205\3766\315\226\275\220\321\360V\007H\3218-\n;X\336\265\272\205\206\006\023!\022<\371,~\037\211\305\324\313\250c \207\215\"\326\037j\230my`\231TP\310o.\347,\035H'\341\\\277\037B\256\246)H\017@\343\221ht\355-\207A]\200\365\276\\<\347m*\026\347C\320\000\316\202T\315\334 t\002\001KR&b\353\247\313\357L\313\\h\340\r\025\377M\371\333hz\320N\333\234\264 \225k\021\274\274\212\t\n\264\037\250u\r17\317\345x\001\332\004s+\031y\20122\014\373&\006\\\376\262\242\004\240^\351t\266vK\227\001\241\321\313H\233*\272\201\237S\3415DB\317\260\344\257E\024\\\320RS\022\"\327i\350Y\001\327X\231\213\372\210)\313\306\340Y\204}\214ig\t /S\030\251\316\350=\200j\033j""\214?N \222\246\225\034&\016\301\376\277\315\320\006\370u\232\000\312H\177:\307^\332W,\177\322l \252\207t\366\013\265\302\270\001\210;\255q\242\370\232?\246K\244`Z\232\013\371\262O4+$\333\225<\301\036\0068\302T&\177\377\341t\334\200\200\250\007\307f\323o\233!\n\314\313Z,O\0073\3768\341\n7\036\237a\351\205\224\202J\321\352\020K=\017\317\325p\241\020\324?u\247\263\030>e\236YP\0332\026\007\221\023V\254\234\024f`\225\204O\342m\005\362\331\373\031@\272\035\353]i'\224\330JP\210&\271\271a\314p\263rf4\374\201\202gF\234\306\243w\300\034\363\242\350f\356T\261\300\224;b\300>\274\207\200\006%\010\375\264\034\230;3@\202\2736g\302\240F\021\001\t\204q\350\333\242\367B0\211\222d\002l\267\206\302\307\314/a\347{!)\214\344n9F\020D\270qS\204\324\226\256\n\310|\321\344\304\2554r\366M\273\211-\301\201T0E\350\"\314s0\350\337\021\364\t\254\221W\374W\232\001\rj\353\375?1\301\347p.;&\305\247\242xR\t\3057\223\225\233V\320\375\353.E\340\230\004\224\257\331dh\364:$6/\321\333\334\177\020z\n\240\254\363\007";
    PyObject *data = __Pyx_DecompressString(cstring, 8057, 3);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #elif (CYTHON_COMPRESS_STRINGS) == 2 /* compression: bz2 (7684 bytes) */
const char* const cstring = "BZh91AY&SY\033r\240\035\000\010\307\377\377\377\377\377\377\377\377\377\377\277\377\377\377\377\377\377\376\300@@@@@@@@@@@@\000@\000`\"=\256!\364\352\300\025\326)}\201\276\362\303\267v\316\335\255s\267J\252\350\334\267\336\367v\014\007#-\263\035p\351\326\005\000\240\000\355\225\221\256\273\265SX \006\340\001\332\324(-\3566\312\200>\246\270\036\200\001\220\211\250\232i=\032jzi\250O(\362zScB\2310\304d#3&\214\233I\205&\3242x$\365=\003M@\000\001\240\222\020\t\244h\232F\204\321\224\364'\250\320\r\032d\014\206\215\003C \321\221\220\302i\246\203@\032h\323A\240\004\246@\224\211M4`\232\r\032\001\221\246\200\032\r\000hh\032\001\246\232\006\206\200\000\003@\014\215\002MHBFR\231\224\320\r\000d\320h\320\017P6\246\236\223G\244\0172M\240L\324\361(\365\003a\020\361OHyC@\032HS\320\232i\246&A\246\203A\246&M\030\236\246\201\246\232i\241\221\220\000i\210\031\006M\032\032\000\320\000\t\022\020\001\002d\320\031\010\304\004a&\215\032\236\247\250\310i\241\241\241\243@\000\000\0004\000h\r\254\375\353'\276\243\215`\2735\260PFB\230\326\033K0>J]\264\234d\356\000\375F\035\243\257h\263s\320\263\005\271^\362\2118\325\251\027\235\372gu\2116\220\323\313imU\253n\240k\322\214\265>wm\235\316\306\370>\357\217\305\342\020\2369\000L\020\177\250\017,\312 \242\201\377\271\362\360wu;\226\302\346\023\244_\230\312\201(\311\272\243L\263\216R*\361\222\301\017\220\234\244\316$\\L\311-\267ar.t\230\002\342\r\330\341\276lG\232\201D\002\006#\"\301\203\030$R,\210\204F1\024P\025\021D@AH1Tb(\3062((\221\005\001A\021b\252\n\242\220`\214F \262 \305\021\010\254a\230f\006p\316C8Vu\200\200\355J\342\301TRh\377\344\204+\321x\223L\033C\004V(\200\351A)\024X,!\021\213\001b(\221`\240\210\212\250\262\n\242\301Ab\306L+Y!h]\031\250\250\302\377\311\260\222\021k[R\372\230\251\224#l\206\266\010\251\001b\310DERDd\212\260\214\214\241\307\034tw\242\024\320\006R3\234\3447V\250\36774\033{&4\232l\301T\001Q\274\006\320\2268\230\206*\242\204\372\205\232\025\302/I\230@6,\004\037a\221\220er\030\306\306\301\261\217%y\221\222\321\242\244\025\260`""\317\024d_\224b\215+&\026 ,`\264\214\007\004\020\3207\2323Y\347\232\326\255\255]\316#Xh\014\222\n\222\250\305\014K&\n\010\206\004\020\241\000CB\310Y\031,\262\310-\254\2614\306+\321\231\205\022\322\312b\322\255\216\t\251B\222Y\304\200\310\311bbZ\251Cn\205\226)\260\250^fjv#\026\214Y\223\021\t\220\r\241\264\225\351j\313$\265P^|Aa\253\020\312Pc^,fNNL$y\034)\225\346\215y_UV\211\214\324\032\214\022\220\035t\"\252\322\276sD\235\311\221\231+,i\301\3653\226\260\316ha\2117\000\204\372a.d#\231\027\244`-\310\336m\232s\225*\256\251&\340\222\010`C\033C\256y\242B\324\\\267N\344\327\203\230\265\021R\265\3215\025\203\213Q\2525\312E\032(\3061\326Ac*\2349\353ZM\252\243Q\267Q\247H0\207\245\2716ME\301\034K\023\327T\263#\010\344\316\330F\026\r\244\0236F6B0\226\031\232\206{NYA\220\310\262\222\311\311oX\336j\347\036&s\220\311\234\347\275\342\216(\334n7n\3246\336\0143\276\366\341\266\313cj[P\325\032\255M\315zVyf\020\302\014\214\205\220\031jZ\353\356\326\0324,\267\217\023]\207t\214\367\353\251E\240\264H\301\005\203\247\357\261\303\255\266\022:XW\351\240b\306\373\234\226\373\357-\307a\314|\025\3332\214\363\006\206\313\"\311\2626\016\340\352\235GF\361\263b\323\255\024\250y\031\301\341i\263)e\206\027\335\211\201\245_\263/m\305\331\374\033\370\225\256~Y\373\rx\243\002\262\354y\233\250\216j\017#\00391\022\327\003\320S0#\302\362QN\237^\245H\t\314\024\030\273\314\033\033\256\376\034Ho\243\374\010U\340\372\341\3248\356\306\204?\221\034T'\230\315\251\264\211\016\310\376\244\311F\236\306\245^W\033U\366g\"\367)\t\234\366\323\357(\362\230\317\365%\353z\317\033\312.g$b\363\244\177H2\332\356pv'\310\227\240J\233\301\235\275\250\255\\\030m\027\335\251\312I\313\014\024)\321\271\332\230\212c\201\3230\271\355B\211\270\316\307\244p\376\370\n\220\367(X\265\024\311\304\3513x\323z\360\304u\356~\036\266\231\031>\035O@\321\256\022~\265-0\221\307\235\270F\235\030 \271\273R\027\013M\341\2716\366X\274\352\204;+q\031\315\025Jn\370q\225\244\003\033iJ\034\334\2479\312\267~\202N\013\344\010$\247\366\363t\210\031}\315z\342""\221\"\373G(*\350\367:\355\333#<\002\342A\300\357\374\014\"4\217\004\351\357\271\341_U\271$F\004\270\370\234\236a|)\025|&\226\002\245\0166j\341\343P\351\006\352\330\343\211\225Z\355s0X[o\010Q\314I\003V` 3\204\275=>\312X\324\302\322\022\360\004\314\030s\360\010\300\032\235\360%\340j\364/\311\313q\370\306\321\034\370\243\021\031\212\346\276\374\332\\+\215\201\317\262s\330\037\031\361\377\253D%\365\037\244^?O\351l[\026\316c\217\020\331\244\373[\315J\320-\016\257\313.\363>\223\257\023\036\331\210\313z\350:=\376<\303b\327\363\222\211ld\201\257r\\\246\336;\r\272\253\321\240\r\n\374LJ\217\2665\212\324\267\243\205\327xrR\323\332\357\355>\334\360G!\360/F\th\027\nv\003\276\335\245\037\205\273\373\013!s\265\330<\233\373\334a\026\267\016\247C\364\270\313\301\336\254\026K\265\205O_\032|\373\230\306o;\327\233\323\355^\302\301\003'\242/\216.\236%\025\242\230\323:\321E\222\272\2113\2725\030+\245u\370}\263\345\2277n\356\032\201I4L\260\261\neK\270\241\216)2\010Ta\353\260\265\321R*U<\276\264\352\234\024f\347&\266\355\017_\334\362\374\235;\326\374\370\345\337\337\201\325\221#\241\320\243\211\217H\366\277M\033\362s'\250\226\205\220\224 \261\005\210&X>\027\217\352{p\355\317q\311\332\256\201sS\322\213\312\253Cs\223\023,\352\326\3348FB\032J\2202D\301\214\026lT0\0211\357\260\332\316L\304Xq\312=*=\031\330,\322m\376w\251\352{A\347\237{D\222\334\027\353y\372\235\276\372\252\002\002h\362i\351o\345{\300o_\002o\201\002\370\234B%\034\216q\243\306l\321\344\014\200\014\360&\306<\001,\021y\273\367>f}\361\307/g\242\353\203Z\025\372\327\234\272\352\320V\005R*\025J\236g\355\372\034:\270\026\003\013\0303\267\211P\255QS_G\206[\344v\273\376\357\302=]S\313\035Kv\362\332#J\2504\375\213\376\326\277\222\362\365\227\223\004\226\013-\266\314\310YP'\273(D\004Y/\035 \373\371BK\206;\367t\000\320[\274\247B\206\210\335l\314\301\365\301\004\221\354\307H#\177\202\344v|\336X\211\222jlxi\321)\375\n\371~mx\235\347\374\343\365g\344\367\367G\\>\216\316\321\02100\"\240\203\273\202\362\t\205-2\350\247.\307\340\324r\342\325""\350j\244\204\262[\013\301ai2\257\225M\270\250Q{\316Q\304c%fv)s\3650\272\335\025EU\213\026\323\242\341\301\264\016\217/X\370\210\230\277}\205W\253\227x\334us9k5\255I\037\324e\206=\241\242\005%\234\035\227\222(1\327\334\000Y\262\304\260g\202p\t\236\005\021\207\213\371s\004\231\342\240\240\2629B\344\244\344\232\352Cm~\352\2651\036\024'\274\262\2149\202:|\334\274\274\2506\3555\006\232\300\373\275=\371\214\360\363\375\231\037n\2343\315\007\005\3004\366/\177w^\217\243\200\362~&\210\221<\221\226BYdn\363\016\377N\035\201\2372\217?\205\032\235,\206)\234\006,Jf\201~&\214 \262\006T\340?\003\367\344\022\343\212\026@W\251\037\357\360\374\303\317\034\271\271\275t\021@\324\272\n\325~\211\362\035\211\205\202\220\020\204/\200\353\2774\203\330\245\240q\230\034\315^\220\312\245\341\247\232\311C3\217\340\236A\"\214\374\343'\006\217f\211w[\312[n\346\253|\213\256\325\272\230\214{6\306\331\374\177\265\231\327\231\327\277Z4\371\270\212[\262\215L\037\204\373\377\232tq\027\242gvf\007\274\256\212$\366Y\343\272\254\310\257f\220\335f\rP\345\313\304\346\204\222\355IB\347\231O\211\322\237/UT\245\223n\250a\016\216\004)\014\034J\212\3433\010\250\032r\346)_\014\004#\024aN\004\315\312\010\224\025\312Q:TRUT\302\2217w\260\327\353\340/\213\324\370\264P\353B\361;d\205\346\314\026K-:\264\255G\371/\213\243\014\371=\n\220/\211\352k\341k\272\232\367\371\320y>H\360\334\033\215\333\267sc\2073\331\317\227\230\3772Vul\253\177\215\365z\031c\260\366\370\036\335\307/\320q\3677\241o\240\005\002\005\000r\016GVm#\2262e\345\363\250\206\3671\326\226wQ\031\203(\206$\340D\343:t\356\252\0271\307\340p4|\266\375\356x\036\027#`\330\333m\215Wz\266\335d\254+![\251\236\221j\252\324\244\273\276\316\350\250T\256\026\347o\177\325\351\201{Z\312\246\356\334\326\024\311\302L\033\001\240\347\000\\\252\376\221|\3202\351a0\030m\207\245\226\031:\224\314\367Sy\n\241L%u=W\227\261\324\020U\035\262'/\275*]yG\363\304K\001\375\034}V\204\312\3408\204\302\024\032\312|\3366\357\237{\271r\355VR\371\371\226\230\373\225\236\252\027\3609y\300v\323""\305\224\007M\232\216\273\027\256\211\306\333\033\336\251#l\022\037bc\341\212\363u\014\305\241\232\246\003\346\326\014\3747U\2534\213i876\370\240\203\303s&\230\335\367\235e\336\375m\313\346\320f\026NnK\014\347\013]\371Q\263\334\354x\270m\330\344\353k\206\225\262\030u\276S\261\3446p\256m8p\343aN\300\253\271;M\022\036NFc5w}\245+ng\255)\035\246\204\310\317\305f\3329L\255MCmF\345\335\324{\271\301\351Q\211r\2646q\243\277?=\261\200\273b\361\"#\214PRb\263\256J;\364\271\254g7\225\231\261\241\224\033\364\013d\r\253j\240\342\333\350tP\224\251\316\356\361A\204@\334\357\305\366v\314\2537\2633E\314s\362\203<\374m\264si.N\355\303&\345\304b%\276\235\027\303t\\\337\274\256l\355\211\342G+\2356\260Pz.Gd\3421+\256\252\257\001\325\347\362\374\251-\372~|~\202R\006\335\023-i\210\331\243\354\363\300&w\322V@\225\272\355\267\245\364\223\375\006\207RGSE\237\350\340\004\371\200\250c\313z\352&f\264\315\025\346\251\367#\315AV\205\3630E\366&w\030\370\036F\027N\204\246\236\317\247\300\250Y\331@d\266\250a\327\237\246\305E\255\330\260`sE\222\267-\010\321\275B3\246\3416\216\332\264\253\203\314\304IahB\032\007\014\274\224\335\212\352\\\030\302\261\225r\031\225\347D\302\246p\256\033\006\334\342\244E%>j\006!bi\002\343\354\366\303\356\022\204%\t\004\201{Oi\356\371\352\n\253H0\234\271\327\014\232\233V;\231*\031I \227u\231\201Q\253\014\t\250\3157kw/\023\233s\014\242\212R\211}\317g\342\371\307\035\247G\321\232Q\257\345;\322i\201\340\2044\201\244\250k)\261\237\r\361\353\215\322\"\265\317e;f\261\000s\347\260\302S\351\237 \267}y\206\035uj\234\313\002d)q\321\200eF\306mHr_\0042z\002T\372\276\275Xv|>F\336\2054\264f\347\006/I\263\216\234[\200x\265\221\334\214\374e\t@\366\324a\017\336gZ*\205\344P!S\267\033\361;\326<\274M+Z\021@\266\343\370\314N2ff\206Bz\2109\203\265\002\362y|\275]\362\327j\333\2720\030\323\330=\262\023k\221\242%I\004\216\013\t\030!\250\356\024\301p\304\235\235\250l\347\236\313\203\272\341\224\236\0040\302\0273T\252|`S\267E'C\335\245Z\334\2710\212E\273\253\312:\035R\301`\340\252\216.l\314\252\251""\310\236n\240\305\230@V\302\266\342\226S\256%\"\311$e*\312W\312\221\312\013\225.D\034\246\343nY\033X\365\021\261J \034E\212TS\037pH\310b\023p\225\250=\031\235\244\230`T\274gk\334\034\3632\314W\326\353\256.;\341(\220\214\303\016\023Z\222\022I\010:\036\262V\234\270\336\256\255}%\365\324\245\000\360\245y)\025Ps\013\253\234j4\266/k\230\261\211\211d\265\211\232\252\252\252\253j\320\315\270\347\244\351f\305\232\316\335\034\336\035}S\252:>\261H\272\305\251q\364\344\253\313\200\276zE\031\2245d\200ja\3371(\265\\\241\022\325J\315\000\363\360\005\020D:\010t\3257\027rf\316;\"vsQ\230Li\030\254\242(\n\371|c\025@\366\267\307\255\031\023zD=\266\311\005\2439\235\233mPw\ny\010CcrcPYu\202ZM u\265\020J\323\016\315*\r1\024\027\203<\272/\264m\341\024\260'k\001\246\026\252A0!\315\232J\360/\330n\336\372\220\362%9\231\323\244\322-ug\014b\307\014.c\205\311s#\005\032N\341\245\201\262\317V\330\001\245C\334NVE\007z=\344\264\215B\231\301ew\265\274\232\265\236F\301\214EA\344\311\345+b\304\024\026I\200\331Vfv\031@\321\017d<0\206N\263cS\035\200\266R\253\007gh\355yPcW\277\033&\317D\016\013\273\203\271\035\247\177x|\376\376O\024\213C\235)H\355\2567\014r\3268|P\303\340\220t2\272\2224\205l+`\371\\\311\024\267e\363\371*\256\226\217\022\245\360\265M\350U\202\267\234dd\333T\250\323~R\253\2409NP\364-\346\364f$\266\226\200(`\261\327\310M\210\331\244\n\215\262u\222;\030zX\023\266\010\244\005j\333t\022S\002f\327F\264\210\322\006\224i^\237w)\226Q\031\351b\221\224\nRT\355\241\353\256(\276\220\332ci%Zc\364r{\t\"A\316\302FYM\234\222\027\025\305q\342.>->\331\321\321\032wD\000u\235\021\345\002G\307\322NZ\225H\333\353V\353\221\353\032\017\261h\\\026\235<\025\310\356H\372i\014m&\324\206\203\240f \303\272\300\301\232\343\341\251\007[\014q\356\206\254!\003D\360\261\370b\315\224*G\272\264\337\205\272\335J\306I\355d\3331~\3079\356s\2376\213m#&\347\357,\356~}+l\214\230\233\001\242\344r\343\236\313\0273\330k\252W\306\201.U\203\246\t\030\345mS0f\252\217sR\003EA\356hG\177\177\177}\250\014\032\245\n/O\305\030\233\007sD&\014""\010\267.IM\005\001\267r\270\260\361W\372\226\320\342\233~\013]\311\035K#F\216\342\264v\003\026\306\014l\023Bk\2109\311\\\300\347\351\271\337<p\001\267\005\246\025J\240(\023H \212\331\240\266\216PK\306\034\322Mp\323\025\204;uz\003\2024\332d\262\013\021H\300\201\217I\302\016:rH\275\006T,Y\336F\263L\0048\240\232UoH\336\216\215\346\204{\335WGS\006H\207x\215f\257\030\256\250=QV`66\r\244\264s\305C\233\370/\005\034\267E\000\252\233\034\346\027\003\257\276\353\021!\322\260\202Q\237\265\026*\206Z\257\251\242P1\240l\006\023n\t\220EVU\225\246\316(\343qp\335\350ay\022JD\232\177\204\217$\220\024s\006C\201\271\201{Q\212\341\335\237[\360m\320\247\\\253sk\r\224\247]=\354\252\332\344Tb\306\230ni\032X\245\243O$\324Q\256\312\3432\250\250\254\213DK~\311\201\275\244\005\354T0W)@66\025R\025!&\\\005\244=q\314L\332\306\2304\312v\276\214.\014\355\270\331#'0k)I\325\224\250J\000\214\311Ht\204M\362i\306\376\212Wk\035\212M\343\317\307\210t\235\370\213\240hcAk\"\375\320\037\025\360\250.\275'S\016d\372-\202\304\202\244\251\326Q.7\212\006C\224\244\310A\020\322\230Wt\374\261,\266J\301\336\232cm)H\200\260\305f`\302\004\r\025\316\002\246\223\035Y\314\024uD`M\372E]\331[\021X\305\254i\271\021]dK;\020(}\305Q\013:&`7\253\354%\216\213\244\255\263\256?(\220XB\352\340\034w\300\357\262A\247\313\277\245 \263UrP\315\027\233&rp\276\226TJ\224\253\256\335@\240`\330\016\330\363\264r\240\324\210c\253n\352\004\325\220\340\310T\310\226:4Np\006\030\2373*c\030\341\2516E\206\256*\226\000\245\346mAYu\032)U\254n\222\035l\322W\302\310U\316*m\006~]\241\327\345\343%\000\020#z\210\356\334]57b\356\325\222\212\227\336\357\3468^-52\325\211\362\366\031\201q\026\010\264\212\276p\247Uf\245\234Z\311\026\207))\014\210\0246g\231\251\2036\301\r4\306\323\036\270\217\222<\314\251\023\215\354\3434@\323k\241\212\207u\313\236\236\025\234\262\262P\220&\\\325\243\266.iQ\2457s:o\370\357\022\200|N\353\222<\315\026q\256\267\331\216\312\212 \3525i\222\260{\274\034\003\200DC\212\217\206&@\003\260\214\207x\222\346\003(\232\242\326V\254s\364\245""\203H\304Y\310*\0254\376\000&\332m\027\2244\032\252\362j\254\351\256\036\231\010\212\010$4\2753\226\224\226I\307\240\263\261\230\323\224\r\262\003J\222\347\027B\016\303\263\267\311X\315l\013\360\2051\251\035g\253\354\312\216\347*\371\246D\213\334\304_&\253\211\200_(\232\205\333\361]\035\232\233T\221[M\255\005\357R\2559V,)bS\302-Ys\351\244!\330\342u\354\227<\213-\250\334)\030~\231\206V\242(o\270\030>ygr\351sh:\235\2334\232so\345\302Iix\354\217\267>\233s\233\203\233q\212\331\3568\313F;\020d\211\261G\202\315\235R\014L\271\016]\024\266\350\244H\265\2646\333\325\004\332\227\233V\263)\261\264\243\034\364\211Mk\232c\251\"L\203\231\236\245oIZS$IN|\251.I\233\236\301-z*\250|\232\265\314\255\351paZ\333\243\222\371\370\343\337\303\201\336\274W\0369\351{\213\234c\320\262\031A\256\225\365&\034\305\205U\024TSu\216Jd\"\246E\226\263\232iM\364\230\365\376\235I\326z\013S\017\272U\306ABP\202DA\226\305\306eA\022\236(\2303\331\315\321u\221o\\\354\234\024v60\032\036\206\204\350\020\322w\033[k\305+[fN\255$\016\202@\302!\223\212[\350\310\300Q\207\303\225*;\021qdQ\001khT\253\225:\374q\030sSG\r\375\375\222\021\014\312\250E\036\305\257\254\303Z^[X\n\324\356n\306\267P\330}\240\302\033\231\3233\023\307\330\224\311\361\261n0\374\004\213\212\330\235\316\355\220\251\223F\031\246n\207\301\022\334\t\203J\261\206\317\311\212\000Q\301\220\352e.H\325\306\207|\263\244\255\004\332\310\026lm4\rg\344\231'@\236\030\005P;\234;\345\215\223\225/r5@X\235\037\004\204\034\004p\020\270%\301\007\207\020\342u}n\177\372\304\320\221\357\347\027X\036#\013JU\317\311\036^j1K\311\322\267\357X!0\274\022\372\265\364\260:\240\352\260u\222\322\226eZD\036\312\365]\250\371r^\312\270#\202\340\221\3008p\326#\025\275\034\334\223/\236P\035\271\203!\201M5\024L\262y\2134\031\254\326k\207\273\024h\033\301\2620\277\035r\202\207O1]t\244\206\311\314>\273\n2E\003@\264-\001\240\321\332x\340\326\201\212\024l\331${\032\243J\032\276\345u\326\253\037\036Xk\257\246Iu\014B\3676\333l\334\306\325d\341\272\366\024'\266\221:#\345LIQL\265\007\307q""\342 s\021\247q\250\324\265mI\031z!V\025*\270\226\004\213(\244\211\004\267\316aQ8\253\310xl\376\351/\232w\300\333~#\375\363V\343W\224\360\316U\223\305M\353\331T\356\362\374\016\\k}*W|\351\211 *\013 \301V\n@X*\202$g}X,\210\212\211\025\004U\024@\3240\016\3202D<\275t\263\370\2421Q\"\212$\366\220\237\034c\t*\236\325\255\001CbJ\010{\225*\354\303l\025\024\251,\304h\006\324\026$\357(091\021\362\031,\213 \253$\213 <q\346HJO\tAd\221\221e@NS=\375\375\262\311CQ\263\200\034\213\314\241r\366\202\254\2314\035\246O\337\351\327X[Q~\375\022\234\002b\330K\021-\336\264,\3142\177&B\336\2419\215>\037\220z\t~\234\007+J_\077\077\334\332^\025Z.\224M\0144d\237/UL\305M%|\306N\217\231G\312\334\203c\325\225W\024\364oc\0270\202\366F\325{\337D\246ra\220B\254\332\324\030WS+\355\371\247+\253\315[\375\343\321OBK[d*\253&\025r\317\241\213\230)\033\260\325n\315\357s\372\363\271gz\307\327|\312L\366Y\322\320{5e\340\316\312\336s\305\332\361j\001+\027\007p\360<\037O\343}&\362\274`\372\016\323\330\325\204qE \360EX\272v\207L\362\224x\024(\022P\336\341\022lV\230_\361L\214&}u\276\254\347\261\341\372g\253\353\371>)5K\003\345W\332\377\255V\361\232\317\303\245\265\350\206\243\224\352\326\345%\314\261d\326\257\271\375\370C\325\007\325\356[\276\027\017\022\247\216/\347\203\352\277\253'#\353<p3\245)\256gX\320\326\244k-M\017#\374\277\217\371a~ \334\361\331\251\177\332m\206\225\177P\314\031\214\267y\370kX`\031M\n9\200s\320\241d)\233-W6\371\361\306\335\3606\316\331\033#ws\301\240\316rWZ\267m\374\034\267c\320g\212\341RfOn\345\036F\367G*\331\245\2045U7\2111\264B\243_\317;e\265{d&\226\016\357@H\031\260\351\263\332O\360\220\352t\3746\222<\3175\350\267\260\232\035\353od\0103\232\004{z\026\314\207\237\255B\017\364y\254\272\351\247\257\340;\333\225^\346\227\313\255\225\302o\223~\241r\314\t@\230\006{\247\r\262\343\334\313\214\027\316\23566\230\307\007\033f\337W\272u\306\rP\177L@^\277\027\027\260S\377uJ\014\221\332\266W6\222RY?\346\240\342\035S \242&\256\243\324\353\303\374\3237\370\032\036\023\200\035 T-\271*\315\2333\261""\223|\3553\267c#W\025\245\305\016\306\000\243!\316\362\365b\025\\[f\377_\213\203\330\304\340a\220\217z\270\003\322=\023\375\225\253\020+j\341\370\347\270\226\340f\031\203X\3054\266\200\353\317\177C\316\365\334\354\356,K\311\373\277s\310\017%\226*\226y\213m\206}\376\304]\212\310\362z26\334\217\237{\2017\323\312\210\361\355N\010D\317\206\373\267\345o'\003\036\004/YM%\013p\231FM\363\326JR\260z\275\276\016\036\032\377p8\\\216\020\276\272;\t\331\325\221\235\312N\313-6\314Z~\267-\376/bRp\366<\022\034[\363\206\366\253C\365i\221k\350\373\313^\330B\326eoz=j\336\365\341w\2667\007\250\224^\210u\255\214\255,\035\\3\326\350s\371\375S\243\351S\376\206\374U\0369\274\320\251|\227\0362x\2627\353s8\343\247O\244x\307\302`\327\370D4`.\317\376\030\335o\032)\272\250\227\236U\202\020\3107YQ\3767\304\351\333\233\227\n%\337\022\316\227O\224L!\324\350\364\246\016\235`CI\212\202\302\235\261\016\340i2-\232\031d\347g-{\234\271\334\335\223\221r\242\357\033\325\224*\223\2424M\341\\\244&zn\233#L\033,\005\201hA\341\025H\333\021\027F\313\033\310\013mv\243\340\363\200\253\350\004\346\363j\245\202\305\025\035\324\355\305\370[\256w\303\353hn{';\272\"\207\351\346\277\320\250\312\253o\322\240N\202\372\202_^\223\007\001(\004e\274\314\311\023\240\005\372\355\341y\375<~\244\264\002\r\302\251r\3631\027\235\013\317@\202E\230\365Db\315\267\265\344W\244V\332(*\213\245\001z#\240G%\256K]\320+\210\375\261\333\212\322\207\333\221 9\234]%\346\344J\371\013\336\"\315z\3405\205\036w\276;6c\203>J\004`\251!?oe\216\365\250\354\306\341\350s$\026\006'\254<\335\255\270\327h\361\306\005\253L#Ai\334n\221*\342\311\024\005\304\271^\347\230\254w}\233\227_r)u\305\253Am\333\377\022\322\2725.\272 -Z9\325\203Y\303a\314\361\022\363A\342\273\300\257\265R\261\254\344\365X\274\257{\353\301\301\270\236\n\204)\315\346\253\267M,Xk\010\201\373.~\314\023m\374U\370m\323\\j\225N\373VU\332G\n\365|\233\n\243c\225\230WAs\232\034'\342\247\341\326Z\212\332\352\352\333\010\360\035\262\260\021\324\365\353\257.\275l\372\262\004\003\333\2277\227\211qj""\325J\333&\250\252\347}\250\262!\341\336\327\330\365\267\t\246FM$GF4\320\245\023\303HtcM\354B \317!\233+\220\303,\340P\307\022\002\210\213\026\240g\027@\342\331\360\034\335\201\254\201\032\265G\310\013 \017\006\275J\226\365\236\\\007\016{\335\023'\227\342\344\240\020kp<\361\016\226\234\026\303'(\344\0204%b\206D\031p1_\341\254~\370\311$\222X\022\006\343\003\366'\273xO\216\302\033n\235=\023\362\220\235E\343\301\312\347Vu\3615\321n\340\005\315\233\307[2\"\313\233G{\364\025*c\217)\215u\372\361\316p\n\2374\207\342Hr\350\030u\306\002\217\337'J\023\231;a\351&\017d\373\254\300H\2252\004\030^\3609\205&\224\202N\036\304$m\345\262f\335\246B\nQ\202U\373\307\215\331+\223+XAz\035w/\333K\311\357\251\371\033f\341<\311\253\252\336\037\244s\214\275\331\232\n\336f\\\332\233\213x\225R\311W\267\270\343l\264\252T\326:\271\247\255\"\251\264'\227y\211<\324]\"*)]]G\207\032\031\033\2743x\rl\215\305z\032h\346\236 L\326\364\341\271f\023\322\363%6\320\013(\233\205\004\353\310B\0263Q\213\020\325(\222\307\021u\014\262\356\245\002\211pUw\205\3166+\235&\205\\\346%Ug\002\216\363kn\342\223\031\355(\213J\356\026T\252\036\303v\335\016]P\354\220\234$\353@\321\362h\270\350\232\216\255\266\206VX8S\030\372Ew\266\035\316\260\335\352\341>\014\335\007K\210h\332\354\256\241\316w\353zn\356\362k\005,\260\233\267\331\n\370\034\347\352M\026t\234\305\024P\202K\035\241(\211w\027\323\245\337p\270s\221\260\243\002\251\232M\327)\304\355X\224\336i[7x\253,\222\346\244\225\tIy{\264]\336\245H\375\317\222\233\251\211|Y\260\2531\315\2139\264m&\226Q\226I\256C*mM\013T\023\246\211\266U\276\275\303Q\227\353WJ\300\351+\244r\302q\361\207j\356\261\275\232\345\362\306\024c\311(\305\315yHR\355{\332-_i\223\320/\244\326_\022\266\010\314\251\274W)\334\316Ysh2\256\352J\014g@\354+X\233\334&\232\\\344\226\311\253|\026\322U\265\033Osh`\226\321Wv\352\266\302\344QU\033&\242u\231*\2461\021\250j\002\321\007\377\027rE8P\220\033r\240\035";
    PyObject *data = __Pyx_DecompressString(cstring, 7684, 2);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #elif (CYTHON_COMPRESS_STRINGS) != 0 /* compression: zlib (8159 bytes) */
const char* const cstring = "x\332\345Z\313_\333\326\266\016'\004\31431!\220w\355\004\002y\321\222@\336m\177\016\220\204<Hx$iz\332\252\302\026\340`l#\311\020r\332\337\355\320C\r5\324PC\r=\364\220\241\207\036\372O8\177\302\375\326\332[\262lC{No\317\275\203\373k\261\266\266\266\366c=\276\365\255\245|\22506\323\331\365\330ZN\217\2511]\315\256k\261\334ZL\317\355\0321\355SR\323R\364\324\334\320b\352\216\232\316\250\253\031<\317jF|\"a\232zz\265`j\261zklF\315\216\231\261\244\256\251\270\243\267\326\365\\!\037\0335&\304\223u\315\214\245\322[F,\235]\313a\242\230\032\274[\037`\356\345\265C\007\344\362Z\266>\363\203\330\330\2501&'\307\252\251\226\361\006&T\0336\230\315\241'\217iR13\307S\351\332\232\246k\331\244\306K\033r\314\256\2366[\236\305\3664s&W\310\244b4\204\217\243\232*-\302\207Nc\307y\035{\324\315\275X&m\230\2615=\267\025\214I\247 \212\203_7u5\271\251\230\351-\254\021\314\320\372\362|\326\324\364\254\232\211i\272\016\225m\345R\351\265=_EZF\333\322\262\246\021\033\1776\235XZJ|\340#(\272\226\314\351)\003G1\013zVK\211w\223\271\224\026\033M_m\232\221\204x\350|\257\351\351r&\235\324\016\235lG\315`\243M{\026\235\230\235Nm\024\362\371\234n\342\325\224\236\336\321t\241\303\205\\\366\246\366\t\022\243\305\2634\335\317?/\344\330\212T36\263gn@\264i#\226\3222\351UM\207}e\366b\006\364\232\304\366iP6\366f\356\315\315\251{S1\025\232\325\265\217Z\022;7\n\253\311\214j\030\220*\314z\265\220\316`\001\241\312\211\330\374Zl/W\210ea\345d\ny\214\013\277\000\021d\331~H\026cl\024\254a\005\257c\227c\260d\210\326\304\031\350\355'j\306\320&\306&b\257\327\326`[t\214g\263O\246c<\333\203\330\353\202\231/\230\330\001\236\352t\0203\227\213\031\033\020\304\203\330\033=\007\277\202W\010\263l\026\377!\217\245Z'\202\307\311L\316\360\037>\225~w\340\303\2654\274\230\204^\177\312\306+\037\037\334\373\356eB\327\325\275\211\203\237\216\206\366\001\263\346\376\344F!\273)\034\231\340\345\347\237G\215\237\177n\031\305\233IiFRO\347M\014\243\241\334w\310\370,\341G<\350n\364\000!\223\211\320\323\035\377\021\333T\360\3007s\261G\366\263\246\365""\302\216\240\322\311\331\250\353S\233z!\233\254\013 \243\251k\017\302\002'\337\253\257\000\003i\232\337\177\3364\377\362\250\261\202\236\331\245\371wsK\312\314\353\2459e\376U\342\351\\l\253\0004Y\305X\266z\274H\366\274gj\207\014\317\253\272\272\245\221o\354\2463\031z1\275\236\315\351\260\364\325=a\320\320\277\364@\232\341\331\364\223Yev~infE\366\222\2212V\372\300\037\014{?\2770\373\372\375\362\357\216#\323g5\252\031\222\344^L3\340\334p\004Z\373\35526\373\370\345\353\231\027\312\362\374\367s\315[\\9`H\375\370\371\334.\026\315\245b\267\020\003`\202\033\276\377OO\336\"\214\371\254\351\271\267Y\216T\220\272\232\311\300&\200#[Zn+\247\003S\323z\335\t\322[\352\272V\037L\200,tAV;\321\370 \267J\220R\267g\022`\343\010\232\366\3400P\037G\361\353K\031 \003?\254?\006\254\352imG\004\317\272\020\215\364g\355\220A\365sL\304\336d4\203,\335\207MV\013D+\024\305\201e'\r7\220\020\314\307\340Wy\026c\342\220%hu27jK\000\2437\233W\377\013\227\367QWXP2\251\031\306a\002m\030\332\244O\377Y\301\300\226W3\271\344\246\224\244\366)\017]\302\031\030\235\263\360\024\362\335\267\331zlb\022\302\254\250N\036|\010T\310\242\0256\224\007\261?\002\214\003_\022\261vM\3274\377\r\032+\205;1\361\363\317\261G\261o\325TJ!a\302\264\3232\204n\021\210I\377\027x\302Q\213\366\273A\210\231\334\320\300\"H\024\212\220\332\370U\014\001H\230\017cy@\224A\020\311\262778\006\211\335\026\020]u\343\006\326\007\217\txX\332\234\030\013\372H\227\253\004\324\031\200Jj\"\2256H\310\232\0205\007n\"u\002e\307Z:\326F\215\365d\232\020Gv=\350\216\205\206\007\255\007\301IWi\257\254\204\371Y\251\007`t\37596\260\331\372<m\210\035\245nnMlLg\013[y\010\036!A\233\330* \362\013\265\254\001\247D\304Oo\221,\302\303\n[\000\224\226\021y_\307|\374\206\210\"\264\240_\277\251c\301Q\250\"\2636\301\014\267.\263d.\013\3637\305\204j\354\215\3402\022Nh\353\371tr3\203Y\203\227o\004-\205\303\207\270\337R?\375\331\231M\022\2121A2\222\315d\216\200\221\220\312\220=~\014F\227\354\241\004 \037\032\362\345FjmZ\373djY\003\235\023\371\275Or KP\266s\376|\274\266\346""\337dT\023\033\362W7\366\374Q\0053\235ih\007\363\313\316]U\317\222l\350\210\343&<\006n\243\256\0231\272Z\0305\n\007#V\223\303\303\007\n\331\224F\034\244`\256\335\274\267\243\352i~/\243A6\033\322\243@\260t\255\211\242\202\356'\026f\331\203\245\033\303\217S\205\244\246$Y\330\212\342w\003kp\016\263\345\201 m\362F\000\277@\201\306\256d\270o]\200E2G\004\257\261\017\226 \324\326\324M\244<\350\022\274\277\351u\321\331\372\276\354\017M@1\252a\217u\350\022\035\300aRfh2\030V\220\006.Cf\241\366\001\"k|\332*\271\206\347\353\n\300\210P\270\271\233\342\200r\320\203\254\266\333\334\245k\240\202Zs\257!&\236\341S\315<{\273\360B\231I\314<\233S\026\346^\276ZY\016\367\274Y\232\233{\365f%\334E\274D\020\2570\375z\234\230y1\277\360TY^\301]\013/k\350X\230Y\232{5\267\260\";\005\373R\022/\347\237.\264v\327\271Pc\377\314\343\267O\302\335\313o^\316\257(\257\346V\022\312\334w+\r\235K\211\367\334\207P\263\002\344|/<k\216O?\367I\244\201/\323\331\315'@?\372k\325\234\354m\321\230\350\247\034C\343\350#\356\3272\005c#t\317\232\221M\315\024a*\235\342\016jgs\334\254?\242`\336\320E\276\034t\220\217\263\213+\324\315\t\017\377\264\356\332\357n\331\266|\260.w\276\036\232c]:fp+\016\3238\204\266\261\236\334HgRl\207\r\375\231C\372\201\316D!\203>\266\342\306i\311T\025\212.A\017y$So2\234\020U\347\346\362\334\314-\321 -\213\326\312\354\374\3530W\177\226x7\347[\214\260\t\356\221O\375.\360M\330\301\034\221\223\227\310g\350\257U\232\262\267E\230\242\177]D,\205\316\030\364\220t\375\033\226\243\177\003{\360\233\002\361\350]\277GfX\301\233\022\325BC\2048\225P\355\244\376\300@2\t\373a\343x\225\370\216\374gyFYy\2664\227\230]&\367\230M\300E\352\236\274\360z\345\361\002\210`\212\\c!\267\\Hn,@\005\376_\253\034do\213\034D\377\272\"\250\222\177\207\223\312\246\3344\t\250\261\207\017`\252[y\343\365\275\327\253\037\3471\000\227\225\240\367\rT7\367$\361\366%\300\340\226\300\003\332\370\233\017+\211\307/\347\226\225\345\017\313JbeeiYQ\336\354}\302\337l:i*\013\010\250K\332\332\3622\344@G[\316\255\231\344\344+\024\000\3376\245Yo""\263\363[yQv\320R\r7\255\347o~\334\"\210\246\001\322\016\232z9\326\024\262\351\360\262\233\331\334n\366\235$\316\001\177n^\277\376\240e\345\340\221\010\276\301m8\374\006\235$\376-\215rC\266\225\240_T\026\202\333zP<\200\321\373]4\227\236\333\345\211\024UPp%O.\0162\257+\212,\"i\212\334\231\274\250\372\272\301\363\250\006QZ\325\330\313&\3239p4\370?\350\246f\250d\033\304\336\350O\341\037\252Y(D\310\024~\205\272\314\\\275\203\007\245\223\251P\333\330P\353m\336 7\305\353\246\256P\032D\r\272\256\256N\256\302\251\020\036\330\2214\242\343\364G\2570\350\322\004h\030\311[\342W\221\234I\331M\2038\255\322~\221\315\340\177\005\300.\362\032\252V\200\221\204\032\006\0270\024\376\335U\363I\005\377QB\226N%\375\215$\203\301-\271\r#\001\037\252\336\222\235\330\\\222\303\207\"&\014.\362\n=\001K\266\374[\254\347\347\201I\320e\204\236,\203\212F2\010\205\265$\217\2503\347d\216\314vG\313p#\275*\030W2\227\337\203\352\2506\250\324w\237+dM\376Q\3707\303\277\371\006\223l\344\202\362\006\332\207S\260Ne\017\333S\022F\237$)\247(s\244?6:j\320c&\363\302F\330\004B\344\276^kK\255eL\372\203\\R\004\025\342B9\006\247\036\330\205\310\334SX\0132\022\023\261A\245@\370\rM\333\312\233{Z6\t\367\301\362L\334\223\212\256\356\312&\t\013\027`\234\370e\021\326;ElT\232;\302\367\034\026\033FP\231\255\241\243\241\255\355\206os\231T\370\026\032\247\204\265\260\305\325cj\320Q\264\354NZ\317e\221\220\327\263\035\221\275\342\020\020\303\032\033\334\206\232Me4nR\n\356\037D\260\026\374f\024\022<7\010\211\n\006\232\340S\034\334esK56E\323X\313\250\353\370\201\00274\375\366-I\223\002\262\264\306S\007\252S\2245\304A('\034Z\353\234%\314]\002\326\022\204\237:q\t\023\230V\352\322JZ\032\351J\235\205\207\211w\003}\t\023\027\021\360$qi\010\356\215\271MKV\323\220\317\004\274\275N\325C\264\240\221\0214e=\255\371Nc\246#\217\236N\255k\320}:\311\342\250s\322F\016\032\246\237\262Mjl\216\034\r\241\375\200\250~\020_i\010\030\315\264\245\225\345*4F\304\272\220slL\257\232\033\323\334\"\370\027\230\206` \322Z\021\022rJ6O~\311@\222N\337K\247>""\361\331\322[\353\212Di\277\231Og\2514\215\205\361k\336\276\225&\234J\203~<a\211\030J\020\227\320\316c\363\2715\345\026\303)m\221\032\206\370\371\254!\200\244\330e\323\251\300s3T\277U\340\017\000Y\376\205\227m\251\371-z?\017\314\305Xj\323>}\\\341+\275\313\r\306?\032E\206&\242\264B\327\002v\247l\221X\2634VQ\352\277)\263\220'\227\245\2175<3\305\\`l6\265\203\377\325L\001V \277\266(l\271>\226\340\222Wu\364s\303\334\340>Q`1\310\320\331\330y\302|\026\003\365l\236d\234\327\001\r\2374VB\336$!\030Y\371\271&\013m\323\237\3015\0231Snm\r\346\235#\263\221\230\205\213X\226\033b\331\020\0039\2005\345\362&{\016]\331\316s\006t\304_\001\0141\025\315\323\222\270\213\"\215_\235\361\257\365\260\345\313:\237\313cb\0031(\255f\024>Q\336\237\335d\224\222\344#_\320\001\001t6\026\213\221\337c5)y0R\216\340FaK\334\351\232Q\310\230\242-M\232\232\002\362\250U\220\273k\250\0074=b\223l\354\342\244\251\251o!\330\303\016\227\225\200\246\312vA\315\010\013\201H\203\003\353jvS\207\220e\030\243\013\r\242\253/\013\035N\242'u\216\271u\n\030\360S\245\205\251\006\035\332'j\022+\n>+S\331J3\331\004a\025:\271l\336H\022\315\"\036\241\tEq\320\t7\003\375Qa\220\242\263$a[*\004\270A\2332\202\336p'se\351\027h!\222\251\254\203\200A+\007\260iA\254\230<\326)\244\261\001N\007A\256\246M\331$\177\367\035\030\023\350&\377(\374\233\341\337\274\350\336M\233\033\274\002\210M\236\376\024\372\311\320\017ns\374\207;\235\360Ka\311\324\033\314\217\025#`\265`\235\000\223\240^\247\310O\r~\217\241\220?H\341\260\256\3307\214=\203*\220\232_\216l\250k\242\033\346C\273S\030\243\351\357\316\024\263\207\020p\323\016\204\335\323OZDj\370}2\315s\360.!\220Ba\262\220\207\245h\300\361@\262\204\351J\035\324\013\346\332=.}6\326?\225\035\031\377wD(\332\021z\333a|\332\021avG\021\010\222N\241)\334\234\033\0221v\0321\002\367$\034\314\315\302\024\022\365{@j\263\273\312\256\212\031B\307\244\352\253_\201\335\205\335\356\202\261h\273p\177 \030\213\222\004\370\211>\271\031o\366^\213\302\363\265\361\344\206\252Sq\332\300\315\325#\033\330\235\031\033\347\013""\3568\216\321\250\372\013W\217 \306\310\0217b\033\024#\341 4\350FL\014\277\021{#j\217\365W\222d\025\004\260\320\244~d\235\355:\300\367#[j\006\002V\210\310\036\311\344\240\370\300\343\216\374\026\377\347\320\221c=\225\356s\366\244=\353t\2737\275\355R[\265\275\3637\263x\253\270X\355\350-\276\264\333\252\221\207\245[\245\225r\264\326\036\265.\330j\265\275\247\370\300Z\264\324\3370\364\2623\351<q\343\230\351\304\221c\347\355\311Z\373\2405g\017;mN\264\332~\322\232\262\n\366S\014I\324\360\322Tq[\314}\327\212[w\354h\265\243\257\270bE\377\331q\344X?\032\247,\325\332\261\027\253\355\221J\344t\345\364U7\352\216{#\236*\336\371\306\036\260'\203\035uU{OX\335\366ug\221\336\356-\336.\252E\323\232\244\255=\3044&m\244\2478I\017O\340\301=\373\2763\343\030\356\230\027\365\342\325\366\376b\312\272n\277w\022\016V\243=\234\023\233\244\341]\305~k\0137\263n\257\267P\216WG\256\271W\274v/Q\035\031w{\274Io\026\002B3\342\365\226\022\242\321V\277T\306\037\241\027\373\335\256vt\027\343\305{\264%\332\375#K\247\363\362\356\321h\217\024{-\310\204\036\335\263\356\332q:Z_q\261\326\021\203\344\006\235\037\275\253\245\311j|\004;\374\340\252\325\240\367ziQ\364~\357nW[\306VF\247\275Eq\206\343\020\301C'\345N\224\332\244\344\377PCB\r\337Y\357l\336s\2418S,X3\230\347\256s\031r\3528\216\333'v\274\332!\244\267h\25354U\272\327\254[\260\207\244\035\265G\355\355\260\376\306\274Ao\361_\321\337\255\342;k\322zj\337q\006\234I\022O\304:\212\216\204h\036\267\267\235v\347\251\373\310\373\245<YN\370Z\336\346\263V\"_\310E!\217\177S\3414}\277\265\351D\235\021'\343M\224\333\252\227\2568\353\356\212\027\245F\3325\275o\313W\366\271w\303\315\224\342\242\241\206.\370\271\334\252\342\312\231io\3113K\017\312\213\377\031%GH\311=\326\267N\334\231r\266\205\212\177\2453A\315\035\254f\272\234\266>\331\273\216J\003NX,\233\343\020D\274\326\336]\274\n\351\316\333\213\315\276\216\023\214M\222\322~\213W\003\273\301{=\305q\370\354\224\265\335\374B\345\314\375\322@i\252\264-m\032\303\261\267\301\220Yu\323\333\264\334\254\335\353,x71t\360T""\3534w\274\325R{)!\247\251/\3364[\207?\333\023{\302\355\367\301!<\321\327\245\245\222Yf( it7\274O\222\272'\337nw\027J\004\\\275-+\364\261(\006\355\037\334\213\036\016\326[\234\263\242\326e\230\370G\247M\230\314:\354?\345\334p\263\264\343\023V\037\273\003\006>\265\036\242\237f=\3372kW\363{\007\000\303\331\213\366\272\263\342\236\365\340\246\221&/\256\364\\\264U\333tX\265\221J\347\260\265m\267\333\257\340f#\356:\314m;\230\246\306^\323\003o\"\303^s\037\303\274\320\325\203\315\255\301\256V\334!W\367\242d\006_\302gG\354-w\n\246&\327\306&\316\331\017\234\003L\243\346\237T\234\246\307bO\355,\356a\033\211jGWq\020\360A8p\322~\016\307\210\364\026\237[\270t\t\221>\263\022\326\np\247\275.\215Mg\310\331\206\036\236x\227<\222G\245\363\274\235\260W\234\013n\312\033+\r\226~\334\277\264\237\250u\014\002*;\272+\335q \026]/9d\223\235\247\254g<z\0303`\300\t\034\230\242X\257;\353uz\006\234\265\023\"\213\303y\026\253\335\004F\327\275\225R\024\335\327&\334\r\217\317\020\341X\004\305\022\222\255Z\035\0349\356\360\033\035\003\326\rH\0332\025\261\261\217A\341\374\005\373G7NG=o-\333\344Q]Q\253\215\356\207\212\273\226fOA=w\335\313\356b\r\207\177\006\230\215\332q\321\224g\356\253\234\002f8\246{\327\213\373^\373\304\031s\243\260\230\301\377\254\305\014A5)\373&\326\376\266\024\007\230\014\r7\352\023\240\315z\224\263L\n\034o\350j=\003\365\234\2256t\333U\335\355Z\330\177\177\265\237\301\334~\360NC\356\023\373m-\336L\0268\006\327\032\2612\316\204\327\326\352\313w=\035f\260\330\004\n\030\010\023p\356@jh^s\343\356\035/*\342\333 x\005\2076rF\236\377\006+\365\241#\221b\252\250['\001EG\t\214\007\254x\265\253\257\270L*<\244\035\351/n\362\3716\341\030\021Z\237hF\367\020\014\273/\212\231\022\377r\033H\3243l\251>.\235&\301\235\205\265\304]\210\372\264\335\005;ns\243\"\200m\023\250t\262\177\277\250\214=,qD\376l\rX\223!\305_\201\236\201\322\307+\307\257\273\223\356\013o\233\317\214\370\"V\020\241\263\235 \271\312\017@\265~\213\327d\363\254=Y\2779g'\004\025\354\253\364\222\007\276'JGK\356\024\227\212;\326\022""\233C=tG\020\304\260\317\356\236Z\344\204\325e\017!:G\334c\356\242\253\326\"'\255G\034\254\023\265\310pex\014\356}\024\233{\n\353\214\220#=\257\\\271\343\251\336nI-\355\224\027\313*<\252\006\021\257\007\266\371\300\373\t\330=\004ZYk\241\241]\265\366\343E\0164\307\212o\021\216\036\n\n\242\001\317\037[h\222\027\352\340\223\221#]\307+\375$\224g\336\034L\357\022\\\347\306$\230\300\010\273}\327\361\"\277\227\222\021\255\243\207\t\351m\213\271\314\252\025\241\265\004\333!\263\276\353]\2013a\320\035&\277\014so\255+dA5\030\304#\326\322s'\t\373\350>\016,$\240\014\017\002\016\372RM\320\262tV\225\034\241G\242\357G\304\261\255\362\344Ax+\365)/\221b[1Z\253\333O\224\264[\355\352&V\361\003A\r\221\264\177\3307(\342\323$\013\240\034S\366\257\356\253R\264z\356\262\363\320U\017\010\367g\316a\014\005\332\263G:/\330\222\223\006\030\003\000\257\014\177\211h\001JP#S\273$\030\007M\264g\375\r~\022L4\361\025\321\344Z\3731\234\276\322\005N\206|\"\356~]\372\033 \027*\3160\377\371\352\026)\201\211cQ\322\226m\253M\200\037V\027\240\274S|\017\017\355\347cH&\272\315\314\311\317\036Fa\220\267\201l\005\3471\374\332\337&x\221\227\204\272\307\313\3612\323\177\"\225\267\254\357\354w\316}\304\204\243\336\003 \311\341\322z%a\354k\020|D\300\373N\342PiI\037&\305<&3\022\301uR\370\320\251\337\317\177\352\344\370\212\220\227\317ug\355\266\252\214\333\024p$\032NR\"1a\367pFB\001\007\001\020\302\005\246W#c\034<_\226\332\200\221\337\225\027\253\3437`\360\274\322$y\312\277\225\366D\000/\004\304\353\366\222\375\331=\ty\265UCA\361\256s\205\242l\020\026\017\014\230q_?\034Q\273A\031G\341\034m\242\ti]\260\227!\227\001\347\036\321\215\312W/\3669d\365\003\374\311\246~\361\276)G}\376=\314\\\02079\n\332\325K7\335\305\277\214\204\377\005\374\333';\313\004\025Q\353\214}\tQ<\316:\352f\270\210\370\254\341{\320\262\363_\330;\230!\356M\036\310\333\311\371.\t\206\364\014\357\264\213\264\344hq\032\330J&\311\2303@)nUBe\270\321\007gI@\336\207?\"]\023\214/Z\033D\377Zn\311\210\3270\360=\235Ad\201\177\007U\227V\0224{\213/\254_|s9e=\255""\234\273\356\336\363\230gO\223\001W\333}\312\324N\314\264\333\371\022\204\351t\314\351t>\271t\304\001\362\360\027l\262X\364\214-C\335/\302\332k<\r\214\376d\345$\325\024f0\260\303e\234\034\200a\306\205\237\2641\377#0\375;;\0033\270i\330a\222\264~\234O\245\311$R\244Z\322\3279\\U\017\2154\307\204B\227\340\240>T\021\303\354\016\243\370\020\033\373\327\000\353\265\322\213\3626a|\2248.\240\231l *Xn\030\257;\353\233\2531e\244\245\257\321\321H7\037\374\014\266\341\0148\336Y,\177\327\301\211\217\375i\251a\014\250\206\365\031 4L\306\367w\252V\224\026\377_H\254\367 \211\205\211\315!RX\263\037\223\253\205e@\033\332E\244\302c\032\005\346GiF[ \232\315 {\372\\\032.w\226\221C]\342m\374\005\302\371k\204\321\177\2400|O\026\205\024\202\337\311P\363\337\261\222\032|\335\264\357!\300\202\342Q\016\207\325k\210\261\221\376J\277\240\000#\225\211\331\362\354~_\345\355\207\312\207\357\253\2611g\307}G\260q\200\220\252O\347\367\243\3777\222\302\261*}\002\353\027\177\363\321\306\032$\364\024|\353\342\221c\027m.\274\310\032Z\202\316~`}\245*\"0V\240\364k\217\313\206\363\245\305\222Z\375\"\346\364\200\226\036\304,\316\216\310\000\217\000q\014\251\307\337\020\214\323\266\341`g\003\225\201\033n\302\375\340m\226\243\345x\365\346\204\313!\341,S\326\210\017\277\335\247@\324\0130\362\305\0327w9N\367\234\340\207\021\000\004\321\205\036\357!\266\001\226~\313z\313\261\220\036\320\344\177/\265!\177\352\032\004g\270\313\366\300c\336\363ta\201\013\346&\330\030\350\256a]\002q\351\202\324~\362\276.\003\236A\021I\357]\224\220\322&\207\305\362'\254c\034\223\246\276)\245p\204\251oK\333\314\213/H:\022!bOs#K\352\244\370\323\021\246wDd\357;/(\034\267\230\314y\221\006\341\335\rQ?\256q\345\365>g\334\235\021Z\272\253r\206\345W\271\363\264\274\263\377\241\362\356\373\312\367\253\225\325$Y\317\000\022\306\001\030\360d\265\257\237dy\337\232\261\376\341\214T\256\334\003\255\002\243\037\252\014!\375*\315\203^=\372\272\364\303~t?^\355\356+~\317<\243\327\217x\261k\356\010\303nO?\354\026\364\007(\324w\032c(7j\253\366\023k\276""\353^\363\022\240\016#\224\260\336\365\256\021 WGF\253'\250\034{\n\323\374\352R\261\314\177\255\332\177\274\270K\005'\347$\014\343\230\373\026\374\201$\334=`]e8\3519\216\314g\232m\247\027\326G\031\035R\365n\301\254\273\t^\273\251:\361\307r\027v+\330\334\216\375\235\363\316e\366\374\210\271\033Y\361\005{\321N;\377`\nt\230\341V\353\020^\000\237\200\332\3103\3613,2\031r\"\014\352.^\267\026\177k#7\242\222\377\260(/P\335\214jr7D\226N\374\177\316\036\264\261=\"\225\021\320\td\223\3144\207\3356Q^\217\300\320F\251Z\005r{\256\364\024\204\277\261'Q_\344\034\"e'\302\327\344\237^\210M\014:\273^z_N\324\032{B\247\211\376\351\005nRu\311\033+\235+?\335\027'\251\367\204N\362\005\310\341s7\345\305\377C\"\2235\2036\202\277>\002\010\021A\316Pf\002\260\024\354\265\243\310\374\242\267\322#\212ug\251\372R\255\353]\332\005\025^\021uG\235Eg\315\007\274\311Pj\313\351\034}?\2402\225?\323]o\274\024?`\256\010}#\340\232\006\025U\332o\320\204_T\276\340\302\310\305\312\305\333\024O\300\270i\315~\376\222\300\013\356\202\273\274E\200^w\336S\352\"\323\304oKz9*\036'\2412\230\350\020\3257*\347\231\327\237\200\033\244\354x\345\034\315\331A\276v\013.\361\326\275\354\376T\232.\377\r>\204\027N3dL\273G\221,\036+-\301\247:NY\n;T\307\240\365\272^\276\213\370Q!W/\233v[qYG\263\344\307\217\273\200z\212\256\247\220\372,\326\276\234\3648\363\177\202^U\024\334\267\017@\322q\300H\025\030\360+A4\301a\232\374\225$\025\361O\326]\034E\202\013k@\334\267/\0232\016\201\345\2006V\342\337\224T\177\316G\034dx\314\025G\222\352m\n\372\333\325.\021<\320\326\255S\\\211\334t\007\335%\267\340=\005&&j\241\007\\`\356F\302\267k\247\234Q\304x\312\341\216\226\246\250\024_k\017\326\235\222u^2\315)\221\305\312\003\021k\3508M\021\237j\311\200\335+N\262rU|\\0H\354]\225\2563\220\373\024\266;\342p\232\363Q\332F\347\220\365A\324je\245\345'w2\024)\010\034\243a\"vl\320\272OR\243f\024\017/\342\325(\345,m\215\027!\013)L9\365b \024\232\1771\230\275q\021\036\036w\242\020\354\245q*\3535^\3769t\004\000\375\257\315r\211\243\261\277(C7I;\324!\266u\34123""\272\013\027\253\"\\\017\361\3678(v\320zV9O\361o1`e\375\356fIV3u\254\207\220\022'\002!/'\254^\226\316@\260I\271j\226>\230T/^w\303a@TX\267\003b /\\}\355\254[\342u\346\022\037\270\374\027l>\350v\262\260\213\272`\271\204G,u\227\371S\277\247R\021\375\205\265\207hx\306\375\022{\217_j\230\344\003\217;\216\\\373\213\230\240\203\367\251$H\244\205\013\001\364\001\373\000)\2532\361\220\301\213\364}\222\213\202\237\371\303D\233_\335\276\"\3129\304\027\351\223J\\\002\221\370\030u\235\331\346\004\253\267\245\200M~v\006\262\243\000HE\306i\273\023\264&\361\347&\370\032\366~\312\336p4\344\335\323\245\266\352\031\320K\201\320\\W>-,v\260rR\224\347YK\277\262\034\352\260+i\3577\300\374xh\265@k\215\205\307\240\324\350/P\377f\364\361\240\375\332Ay\362<\245H\r\005\266J\257\330\026\325\327\306\306\335>o\021|\2368\320\341U6\332\305\217\220~\227{\312M\"\310\014\234&^(vt\306\257\2307\025\317\217\313\204\365ze\364\236\370\267\014\237\330\310\353|\205#^P7\237\367\026\003+\010\276\030\016\003\323d\235l\310o\372\037+\256W.L\260\377\014\360w\347V>4\346\310\002!\244\205!\225\330W^O\351ay\263\262\270$J\202\237\230\024\342\315_\250 U\215\220\\v\305\007\236\303_\233\266#\302\n\203\327\210\317G\274\376\222\372{\257=\262?\221%\207^C\366\340\376W)\273\237\250\006\252:v\032tA\2444\264?\2219\220P\202\224\246^\"\034\023\337\361\375\362\337v\020\3240\373%\361\335\257\017!\370\016\310\310[\004\202\202;\347\235\362\264\022'\300\222\261\222\213\237*\256\201\275\\\246\320u\312\372\251r\031\341\326\233+\235.\237\331\037\331\337\250\274{O\270\365\234A=\202\364\207\252q?z\361\032\007\213\367A\371\247\213\276%\375\003\0009MU\017\377\225\002\177\275\240\327\010\022\237\273j\r\272\231\247\024\276\257\250\340\335\037\270\366\312\005\221\032\231\245\310W\206\275\266j\214`\362\235w\247t\241\254\226\017L6.Q\354\251\005\377\266C\370\r\035\255\247x\215Kh\217\3111(3\332\261\336\331\267\351[0\335\354r\316)C]R\226\207\231{G\210\243w <\312\357\003\322M\366\274No\267\224D\2767\276\037?t#\302X{\2127\345\227\340A.7\266`\310\243""\322\343\222J)j;\311G|J\032\264\276\247*.Q\204m\364\277t\332\232>\211L\026g\021\007\236\tRC\377^@\244\032]d \300a\3217Y\363Y\325-\372\204+\366\303\364\301\257\017\034\376%\304\377\210\304Q\211hb\312\275\351\231`tW\350C\302\213\322v\355\000p\241\357\200\263v\267sI`(\250\021\235\341KN\350)\020q^\177\216\014T^\333\251tz\227\255\023(.6\330+r\033\371\372!\033<do7\334\347\210F\207\300\036Oi\3006\276\026\337\027\311\036\223\310\337\022\325\223g\354s\020\3452(\343m\002\217\022$\203pKt\215> \237\030\2626@S\227\300\233\226\275\243\234\243\017\305\234\243\016\247,\220\257\370@B\365\213\333\\`h\371\026\027.\t\222qrQ\020\201\224\324\033\344p\037+\343\367J\221rGy}\177\261\021\373\203b\205\351|\r\341?/}\334o\333\2176\216\tU\353\004n\0146UTx\247\004$\307\377\207E]\251\237\337)\331\004\025\270\rY\033j\207Z\262\345Dux\304I\264j\247\265\320x\354\177\264\377\337)\257\376/\355\275\367\340\275\013K\370/V\344]\367\206\247z\333\241T\276\363\320\r\311\364U\026\356.\363\207\214>o\245t\266<\005\034\034\031w;\335\317\000\347\350\301.\311_%{\201\027\262`Fv7R\334\264O\363\077\077\321\020d\351k\022\006v\361\277\361\333\026\337\311\016\226}\260\331\336\003\213\224\r\037!\376@\300\225~\361\261\362Wo\266\324]\036)\253\325\221\353\310U\207Km\255\307\250>{\276/K\246\335\241\177\027R\225(\3259`\235\347\177\207\021g\204\366\277\t\215x\037\313m\340\242N\2741\335\020\261N~\341\224\377$+\302\244t\306g\312\246\377/\003\277\261OR\302\363\207}\217,C\374\343;\356\373-^\013\3652\243\035\264~t\342\377\r\344\206\007~";
    PyObject *data = __Pyx_DecompressString(cstring, 8159, 1);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #else /* compression: none (19654 bytes) */
const char* const bytes = "0Asking for a range of rows exceeding the available ones!.Attribute Attribute 'Can't create the group %s.Can't get dims info on attribute Can't get type info on attribute Can't open the group: '%s'.Can't read attribute Can't set attribute 'Cannot append to the reference typesCannot write reference types yetCould not get dataset creation property list from dataset id %sCould not get dataset track_times property from dataset id %sInternal error modifying the elements (H5ARRAYwrite_records returned errorcode %i)Internal error reading the elements (H5ARRAYOreadSlice returned errorcode %i)Invalid dataset id %sInvalid or not supported driver: '%s'Non-existing node ``Note that Cython is deliberately stricter than PEP-484 and rejects subclasses of builtin types. If you need to pass subclasses then set the 'annotation_typing' directive to False.'. Offending HDF5 class: Output buffer is too short: Problems appending the elementsProblems appending the records.Problems closing the Group %sProblems closing the file '%s'Problems creating the Problems creating the VLArray.Problems creating the %s.Problems getting chunk info for ``%s``Problems getting file descriptor for file ``%s``Problems getting ndims!Problems modifying the record.Problems moving the node Problems reading chunk from ``%s``Problems reading the array data.Problems truncating the leaf: %sProblems writing chunk to ``%s``Problems writing the array data.S%sThe DRIVER_CORE_IMAGE must be a string of bytesThe DRIVER_CORE_IMAGE parameter will be ignored by the '%s' driverThe H5FD_DIRECT driver is not availableThe H5FD_WINDOWS driver is not availableThe HDF5 file already esists: the USER_BLOCK_SIZE will be ignoredThe USER_BLOCK_SIZE must be a power od 2 greather than 512 or zeroUnable to allecote meomory fir the file imageUnable to get array info.Unable to get object info for '%s'Unable to get the creation property listUnable to open/create file '%s'Unable to retrieve the HDF5 file sizeUnable to retrieve the file image. Pl""ese note that not all drivers provide support for image files.Unable to retrieve the size of the buffer for the file image.  Plese note that not all drivers provide support for image files.Unable to set the file access property listUnable to set the file imageUnable to set the user block sizeUnexpected classname: %sUnsupported type for attribute 'VLArray._read_array: Problems reading the array data.VLArray._read_array: error freeing the data buffer..`` < ?add_notean invalid mode string ``%s`` passed the ``check_file_access()`` test; please report this to the authors, but can't get it.', but cannot be deleted.disableenable exists in node ' exists in node 'f%sgci%s' in node:\n  in node ' in node ': invalid base type ID for atom : invalid disk type ID for atom isenabled-m.h5numpy._core.multiarray failed to importnumpy._core.umath failed to importproblems deleting the node ``%s``r+-r.h5%sself.dims cannot be converted to a Python object for picklingself.dims,self.dims_chunk,self.maxdims cannot be converted to a Python object for picklingtables.atomtables.collectionstables.descriptiontables.exceptionstables/hdf5extension.pyxtables.numpytables.ostables.pickletables.platformtables.systables.utilstables.utilsextensiontables.warnings to (tree fragment)u%sunable to retrieve the user block size`` under ``utf-8variable length strings are not supported yetANDArrayArray.__reduce_cython__Array.__setstate_cython__Array._appendArray._create_arrayArray._create_carrayArray._g_read_coordsArray._g_read_selectionArray._g_read_sliceArray._g_write_coordsArray._g_write_selectionArray._g_write_sliceArray._open_arrayArray._read_arrayArray.perform_selectionAtomAttributeSetAttributeSet.__reduce_cython__AttributeSet.__setstate_cython__AttributeSet._g_getattrAttributeSet._g_list_attrAttributeSet._g_newAttributeSet._g_removeAttributeSet._g_setattrCArrayCHUNK_CACHE_NELMTSCHUNK_CACHE_PREEMPTCHUNK_CACHE_SIZEDRIVERDRIVER_CORE_BACKING_STOREDRIVER_CORE_IMAGEDRIVER_CORE_INCREMENTDRIVER_DIRECT_ALIGNMENTDRI""VER_DIRECT_BLOCK_SIZEDRIVER_DIRECT_CBUF_SIZEDRIVER_SPLIT_META_EXTDRIVER_SPLIT_RAW_EXTDataTypeWarningEArrayExternalLinkFileFile.__reduce_cython__File.__setstate_cython__File._close_fileFile._flush_fileFile._g_newFile._get_file_idFile.filenoFile.get_file_imageFile.get_filesizeFile.get_userblock_sizeGroupGroup.__reduce_cython__Group.__setstate_cython__Group._g_close_groupGroup._g_createGroup._g_flush_groupGroup._g_get_gchild_attrGroup._g_get_lchild_attrGroup._g_get_objinfoGroup._g_list_groupGroup._g_move_nodeGroup._g_openH5FD_COREH5FD_DIRECTH5FD_SEC2H5FD_SPLITH5FD_STDIOH5FD_WINDOWSHAVE_DIRECT_DRIVERHAVE_WINDOWS_DRIVERHDF5ExtErrorLeafLeaf.__reduce_cython__Leaf.__setstate_cython__Leaf._g_chunk_infoLeaf._g_closeLeaf._g_flushLeaf._g_newLeaf._g_read_chunkLeaf._g_truncateLeaf._g_write_chunkLeaf._get_obj_track_timesLeaf._get_storage_sizeMAX_BLOSC_THREADSMETADATA_CACHE_SIZENOTBNamedTypeNoSuchNodeNodeNode.__reduce_cython__Node.__setstate_cython__Node._g_deleteNode._g_newNode._get_obj_infoNode._get_obj_timestampsO8ObjInfoObjTimestampsPT_DEFAULT_B2_BLOCKSIZEPYTABLES_SYS_ATTRS__Pyx_PyDict_NextRefSSizeTypeSoftLinkTableUUSER_BLOCK_SIZEUnImplementedUnImplemented.__reduce_cython__UnImplemented.__setstate_cython__UnImplemented._g_closeUnImplemented._open_unimplementedUnknownVVLArrayVLArray.__reduce_cython__VLArray.__setstate_cython__VLArray._appendVLArray._create_arrayVLArray._get_memory_sizeVLArray._modifyVLArray._open_arrayVLArray._read_arrayVLArray.get_row_size_aaccess_plistaddr__annotate__append_appendargsarrayastypeasyncio.coroutinesatimeatomatom_atom_from_hdf5_typeatom_to_hdf5_type_atomicdtype_atomicshape_atomicsize_atomictypeattr_nameattrnamebb1backing_storebasebaseatomblocksizeblosc2blosc2_support_wirtebtimebufbuf_lenbufferbyteorderbyteordersbytes_byteswapc_c_classidcattrnamecbyteordercheck_file_accesschunkshapechunkshapeschunksizecimage__class__class___class_getitem__class_idclassnamecline_in_traceback_close_filecnamecollectionscomplevelcomplibcoordscopycorrect_byteordercountc""ount_countlcountp_create_array_create_carraycreate_nested_typecreate_plistcsetctimeddatadata_sizedatalistdescr_from_dtypedescriptiondescriptordfltdflts__dict___dictdimsdims_arrdriverdset_iddtypedtype_dumpseemptyenc_meta_extenc_raw_extencnameencodeencode_filenameencoded_attr_nameencoded_attrnameencoded_group_nameencoded_leaf_nameencoded_nameencoded_newnameencoded_oldnameencoded_titleenumerateenumtypeenvironerrexceptionsexistsextdimffile_handlefile_modefilenamefilenofill_datafill_statusfilter_listfilter_maskfiltersflagsfletcher32flush_flush_filefnamefrom_dtype__func___g_chunk_info_g_close_g_close_group_g_create_g_delete_g_flush_g_flush_group_g_get_gchild_attr_g_get_lchild_attr_g_get_objinfo_g_getattr_g_list_attr_g_list_group_g_move_node_g_new_g_open_g_read_chunk_g_read_coords_g_read_selection_g_read_slice_g_remove_g_setattr_g_truncate_g_write_chunk_g_write_coords_g_write_selection_g_write_slicegchild_idgenericget_get_file_idget_file_imageget_filesizeget_filters_get_memory_size_get_obj_info_get_obj_timestamps_get_obj_track_timesget_row_size_get_storage_sizeget_userblock_size__getstate__group_nameh5bth5namehdf5_class_to_stringhdf5_to_np_ext_typeii8idximageimg_buf_lenimg_buf_pinitinsertint32intp_isPTFile_is_coroutineis_pow_of_2item_sizeitemsitemsizekindleaf_idleaf_namelower__main__maindimmapmem_space_idmem_typemeta_extmeta_namemeta_plist_idmode_modify__module__mtimenname__name__namedtuplenative_typendarrayndvndvaluenelements__new__newnamenewparentnewpathnamenobjectsnodenode_typenpnparrnpext_prefixes_to_ptkindsnrecordsnrownrowsnumpyobjectoffsetoinfooldnameoldparentoldpathname_open_array_open_unimplementedopt_readopt_writeosoutparamsparentpathperform_selectionpickleplatformplatform_byteorderplist_idpoppt_special_kindsptparamspttype_to_hdf5pure_numpy_typespymode__pyx_checksum__pyx_result__pyx_state__pyx_type__pyx_unpickle_AttributeSet__pyx_unpickle_File__pyx_unpickle_Group__pyx_unpickle_Node__pyx_vtable____qualname__rrabyteorderrankrarrraw_extraw_nameraw_plist_idrbufrcrdata""_read_array__reduce____reduce_cython____reduce_ex__refbufreferenceretretvaluerowrstripscatomscopeselect_modeselect_modesselectionselfset_blosc2_max_threadsset_blosc_max_threads__set_name__setdefault__setstate____setstate_cython__shapeshape_shape_atomshuffle_bitshufflesizespace_idstartstart_startlstartpstartswithstatestepstep_steplsteppstopstoplstr_str_valuestr_valuesstype_stype_atomsuper_supported_drivers_supports_opt_blosc2_read_writesyssystemtables.hdf5extensiontemp__test__timetime64titletrack_timestypetype_type_idtype_precisiontype_sizeuu1updateuse_setstateuser_block_sizeutf8utilsutilsextension_v_dtype_v_file_v_name_v_new_v_node_v_objectid_v_parent_v_pathname_v_unimplemented_v_versionvaluevaluesversionvllenw_want_track_timeswarnwarningswbufwherewindowswriteablexzerosPyObject *(char const *)\000hid_t (hid_t)\000hsize_t *(PyObject *)\000int (hid_t, hobj_ref_t *, size_t, PyArrayObject *)\000cstr_to_pystr\000get_native_type\000malloc_dims\000load_reference\000\200!\360\026\000\005\013\320\n\033\2301\230D\240\n\250-\260q\270\001\330\004\007\200t\2102\210Q\330\006\014\210L\230\001\330\010;\2702\270T\300\021\340\004\021\220\035\230a\330\004\013\210:\220Q\220a\200\001\330\004#\2401\240F\250!\200!\360\020\000\005\034\2301\340\004\024\220E\230\027\240\001\240\021\330\004\023\2204\220u\230G\2401\240A\340\004\013\2104\210q\330\004\007\200t\2107\220!\2206\230\021\330\006\r\210T\220\021\360\006\000\005\016\210T\220\025\220a\220v\230Q\330\004\010\320\010\030\320\030)\250\021\250(\260$\260a\330\004\007\200t\210>\230\022\2301\330\006\014\210L\230\001\330\010\t\330\014\020\220\n\230+\240Q\360\006\000\005\014\2103\210a\210t\2201\330\004\013\210;\220a\220t\2301\340\004\013\2101\360\006\000\005\020\210t\2208\2309\240C\240s\250'\260\021\260!\330\004\016\210d\220+\230W\240A\240Q\330\004\r\210T\220\033\230G\2401\240A\360\006\000\005\t\210\016\220m\2401\240D\250\014\260N\300!\330$*\250&\260\004\260A\330$(\250\013\2601\260D\270\001\330$(\250\010\260\014\270A\330$(\250\010\260\001\330$(""\250\010\260\001\330$(\320(<\270A\330\004\007\200q\330\006\n\210!\2108\2201\330\004\007\200t\210<\220r\230\021\330\006\014\210L\230\001\230\021\330\004\010\210\014\220A\340\004\007\200t\2108\2207\230!\2301\330\006\r\210Q\340\006 \240\001\240\024\240]\260)\2701\330!$\240A\240Y\250a\330\006 \240\001\240\024\240]\260+\270Q\330!$\240A\240Z\250q\330\006 \240\001\240\024\240]\260)\2701\330!$\240A\320%5\260Q\360\006\000\005\t\210\017\220t\230;\240d\250.\270\001\340\004\013\2104\210q\200!\360\020\000\005\034\2301\340\004\024\220E\230\027\240\001\240\021\330\004\023\2204\220u\230G\2401\240A\360\006\000\005\016\210X\220V\2301\330\004\007\200u\210C\210u\220C\220t\2307\240#\240Q\330\006\017\210u\220F\230!\330\006\016\210d\220+\230Q\230a\340\006\016\210a\330\006\016\210e\2202\220Q\220c\230\021\230%\230q\330\004\010\320\010\030\320\030)\250\021\250'\260\024\260Q\330\004\007\200t\210>\230\022\2301\330\006\014\210L\230\001\330\010\t\330\014\020\220\n\230+\240Q\360\006\000\005\014\2102\210V\2201\220G\2306\240\022\2401\330\004\010\210\010\220\003\2201\220A\330\004\010\210\010\220\017\230q\240\004\240G\250<\260|\3001\300A\330\004\013\210;\220a\220q\360\006\000\005\t\320\010\037\230q\330\004\010\320\010 \240\001\360\006\000\005\020\210t\2208\2309\240C\240s\250'\260\021\260!\330\004\016\210d\220+\230W\240A\240Q\330\004\r\210T\220\033\230G\2401\240A\330\004\010\210\016\220k\240\021\240$\240l\260.\300\001\330\"&\240g\250T\260\021\330\"&\240i\250t\260?\300&\310\001\330\"&\240h\250l\270!\330\"&\240h\250a\330\"&\240h\250a\330\"&\240a\330\"#\330\004\007\200t\210<\220r\230\021\330\006\014\210L\230\001\320\0315\260R\260t\270:\300Q\340\004\007\200t\2108\2207\230!\2301\330\006\r\210Q\340\006 \240\001\240\024\240]\260)\2701\330!$\240A\240Y\250a\330\006 \240\001\240\024\240]\260+\270Q\330!$\240A\240Z\250q\330\006 \240\001\240\024\240]\260)\2701\330!$\240A\320%5\260Q\360\010\000\005\t\210\013\220?\240!\2404\240q\340\004\013\2104\210}\230G\2401\200!\360\006\000\005\034\2301\360\006\000\005\030\220x""\230w\240a\240q\340\004\020\220\001\360\006\000\005\017\210d\220!\340\004\n\210)\2201\220I\230Q\330\004\007\200t\2102\210Q\330\006\014\210L\230\001\230\021\330'1\260\024\260Q\200!\330\004\013\2104\210q\200!\360\006\000\005\013\210(\220!\2204\220q\330\004\007\200t\2102\210Q\330\006\014\210L\230\001\320\0319\270\022\2704\270q\330\004\010\210\014\220A\200!\360\010\000\005\024\2204\220u\230G\2401\240A\360\n\000\005\013\210)\2201\220D\230\014\240N\260-\270q\330\024\025\330\004\007\200t\2102\210Q\330\006\014\210L\230\001\320\0316\260b\270\004\270A\330\004\010\210\014\220A\330\004\013\2104\210q\200!\360\010\000\005\024\2204\220u\230G\2401\240A\360\006\000\005\013\210)\2201\220F\230.\250\016\260a\330\004\007\200t\2102\210Q\330\006\014\210L\230\001\320\031=\270R\270t\3001\330\004\013\2101\200!\360\n\000\005\024\2204\220u\230G\2401\240A\340\004\013\2108\2201\220F\230.\250\004\250N\270!\200!\360\014\000\005\024\2204\220u\230G\2401\240A\360\006\000\005\r\210L\230\001\230\024\230\\\250\036\260q\330\004\014\210E\220\021\220#\220Q\220j\240\001\330\004\010\210\016\220g\230Q\230d\240,\250n\270A\330\004\020\220\r\230Q\230a\340\004\014\210G\220;\230d\240!\200!\360\034\000\005\024\2204\220u\230G\2401\240A\360\006\000\005\t\210\016\220g\230Q\230d\240,\250n\270A\330\004\007\200t\210<\220r\230\021\330\006\014\210L\230\001\230\021\330\032\036\230g\240T\250\032\2601\340\004\010\210\017\220t\230;\240d\250.\270\001\340\004\013\320\013\036\230a\230t\2401\360\006\000\005\010\320\007\027\220q\230\004\230M\250\021\250$\250g\260R\260q\330\006\014\210L\230\001\230\021\340\004\010\210\010\220\013\2306\240\021\240$\240f\250B\250a\330\004\010\210\013\220;\230f\240A\240T\250\026\250r\260\021\340\004\n\210/\230\021\230$\230m\2504\250q\330\032\036\230g\240T\250\021\330\032\033\230:\240Q\330\004\007\200t\2102\210Q\330\006\014\210L\230\001\230\021\340\004\020\220\r\230Q\230a\360\006\000\005\t\210\013\2201\330\004\010\210\007\210y\230\004\230A\330\006\t\210\024\210X\220Q\220c\230\023\230J\240a\330\010\014""\210J\220a\330\010\t\360\006\000\005\r\210H\220A\220T\230\027\240\004\240A\360\006\000\005\t\210\016\220k\240\026\240q\250\004\250F\260\"\260A\330\004\007\320\007\034\230A\230T\240\035\250d\260'\270\024\270]\310\"\310A\340\006\024\220A\340\006\n\320\n!\240\021\330\006\n\320\n\"\240!\360\006\000\007\025\220H\230A\230T\240\027\250\004\250A\340\006\020\220\013\2301\230D\240\014\250D\260\007\260s\270!\330\007\013\320\013!\240\024\240Q\330\n)\250\021\250+\260T\270\021\270!\330*.\250h\260a\360\006\000\005\010\200t\2107\220#\220Q\340\006\016\210b\220\006\220a\220t\2306\240\024\240Q\330\006\022\220,\230a\230q\330\006\033\2301\230D\240\r\250T\260\021\330\034\035\230]\250!\330\006\t\210\034\220S\230\001\360\006\000\t\021\220\001\330\006\t\210\026\210w\220e\2304\230t\2407\250#\250Q\340\010\014\210H\220E\230\021\230!\340\010\014\210H\220A\360\006\000\005\t\210\r\320\025&\240a\240t\2507\260!\340\004\013\2104\210}\230F\240'\250\021\200!\360\024\000\005\024\2204\220u\230G\2401\240A\360\006\000\005\t\210\016\220g\230Q\230d\240,\250n\270A\330\004\007\200t\210<\220r\230\021\330\006\014\210L\230\001\230\021\330\032\036\230g\240T\250\032\2601\340\004\010\210\017\220t\230;\240d\250.\270\001\340\004\013\320\013\036\230a\230t\2401\360\006\000\005\026\220Q\220d\230-\240t\250?\270!\2701\330\026\027\340\004\020\220\r\230Q\230a\360\006\000\005\t\320\010\030\230\004\230A\330\004\010\210\017\220t\2301\330\004\010\320\010\030\230\004\230A\330\004\010\210\017\220t\2301\360\006\000\005\t\210\r\320\025&\240a\240t\2507\260!\360\006\000\005\032\230\021\230$\230m\2503\250a\250q\340\004\010\210\014\220A\330\004\013\2104\210}\230H\240A\240\\\260\030\270\021\270.\310\001\200!\360\010\000\005\024\2204\220u\230G\2401\240A\340\004\n\210'\220\021\220$\220l\240.\260\001\330\004\007\200t\2102\210Q\330\006\014\210L\230\001\320\0317\260r\270\024\270Q\330\004\010\210\014\220A\330\004\013\2104\210q\200\001\330\004\"\240!\2406\250\021\200\001\330\004*\250!\2506\260\021\200!\360\020\000\005\024\2206\230\027\240\001""\240\021\340\004\014\210A\340\004\n\210,\220a\220t\230;\240a\330\004\007\200t\2104\210r\220\023\220D\230\003\2301\330\006\022\220!\330\t\r\210S\220\001\330\006\022\220!\330\t\r\210S\220\001\330\006\022\220!\330\t\r\210S\220\001\330\010\016\210k\230\021\230$\230k\250\021\330\010\013\2104\210t\2201\330\n\026\220a\330\r\021\220\023\220A\330\n\026\220a\330\r\021\220\023\220A\330\n\026\220a\330\r\021\220\023\220A\330\n\026\220a\330\r\021\220\023\220A\330\n\026\220a\360\n\000\013\027\220a\330\004\013\2101\200!\360\030\000\005\032\230\032\2407\250!\2501\330\004\030\230\t\240\027\250\001\250\021\360\006\000\005\020\210q\330\004\020\220\007\220q\230\004\230K\320';\2701\330\004\007\200z\220\022\2201\330\006\014\210L\230\001\230\021\330\032&\240d\250!\330\004\017\320\017+\2501\250K\260q\340\004\014\210A\210Q\340\004\013\2101\200!\330\004\010\210\010\220\004\220A\200!\330\004\014\210A\210T\220\021\200!\340\004\014\210A\210T\220\032\2301\200!\340\004\014\210A\210T\220\033\230A\200!\360\026\000\005\r\320\014\034\230A\230W\240F\250!\330\004\007\200v\210R\210v\220R\220t\2301\330\006\014\210L\230\001\330\010\t\330\010\r\210Q\360\006\000\n\013\340\010\020\220\t\230\026\230q\240\010\250\005\250Q\250a\340\010\023\220<\230q\240\004\240A\340\010\027\320\027'\240q\250\003\2501\250G\2601\340\010\033\2301\230J\320&6\260a\260w\270a\270v\300Q\300a\330\034\035\340\010\016\210g\220Q\220d\230-\240t\250:\260^\3001\330\026#\2401\340\004\007\200t\2102\210Q\330\006\014\210L\230\001\330\010\t\340\004\017\210q\330\004\010\210\005\210U\220!\220;\230a\340\006\016\210e\2201\220B\220a\340\006\t\210\026\210r\220\021\360\010\000\t\017\320\016+\2501\250H\260E\270\021\270\"\270A\330,1\260\021\260$\260a\360\006\000\t\017\210a\340\006\016\210d\220!\2204\220q\330\006\013\2107\220!\2203\220a\330\006\016\210b\220\010\230\001\330\010\017\210u\220F\230$\230m\2507\260&\270\001\340\006\013\2106\220\035\230a\330\006\t\210\024\210U\220&\230\003\2301\340\010\013\210<\220q\230\004\230J\240c\250\021\330\n\017\210y\230\001""\230\021\340\006\t\210\024\210U\220&\230\003\2301\330\010\014\320\014\034\230A\230W\240A\340\006\016\210g\220Q\220a\360\010\000\005\013\210/\230\021\230$\230j\250\016\260m\3001\330\004\007\200t\2102\210Q\330\006\014\210L\230\001\230\021\340\004\014\210A\210Q\340\004\014\210A\210Q\340\004\010\210\001\210\021\340\004\013\2101\200!\360\030\000\005\021\220\004\220A\330\t\n\330\006\r\210\\\230\021\230!\330\006\017\210{\230,\240a\240q\330\006\014\210N\230!\2304\230}\250M\270\021\330\033#\240;\250a\330\004\007\200t\2102\210Q\330\006\014\210L\230\001\230\021\330\031\033\2304\230q\200!\360\032\000\007\035\230A\340\004\007\200t\210<\220r\230\021\330\006\014\210J\220a\320\027/\250r\260\024\260Q\340\004\017\320\017\"\240!\2404\240q\330\004\007\200y\220\002\220!\330\006\014\210L\230\001\230\021\330.0\260\004\260A\340\004\005\340\006\t\320\t\037\230q\240\n\250!\250=\270\002\270!\330\010\016\210l\230!\2301\33002\260$\260a\340\006\016\210a\210q\340\004\013\2104\210q\220\001\200!\360\024\000\005\035\230A\360\006\000\005\010\200v\210W\220D\230\016\240a\240q\330\004\007\200u\210C\210q\330\006\r\210Q\330\004\007\200t\2107\220%\220t\2303\230a\230u\240B\240a\330\006\014\210J\220a\320\0275\260Q\260c\270\021\270(\300!\3001\340\004\013\2102\210V\2202\220X\230V\2409\250D\260\003\260:\270Q\330\t\n\330\006\r\210\\\230\021\230!\330\006\017\210{\230,\240a\240q\330\006\014\210M\230\021\230$\230m\250=\270\001\330\032\033\2309\240A\330\004\007\200t\2102\210Q\330\006\014\210L\230\001\230\021\330\031\033\2304\230q\330\004\013\2101\200!\340\004\014\210B\210d\220!\330\004\010\210\013\2201\200!\360\026\000\005\025\220E\230\027\240\001\240\021\330\004\023\2204\220u\230G\2401\240A\340\004\013\2104\210q\330\004\010\320\010\030\320\030)\250\021\250&\260\004\260A\340\004\010\210\010\220\003\2201\220D\230\001\330\004\010\210\010\220\013\2301\230D\240\001\330\004\007\200t\2101\330\006\n\210.\230\013\2401\240D\250\001\360\006\000\006\n\320\t\037\230t\2401\330\010'\240q\250\004\250L\270\001\270\024\270X\300Q""\330(,\250H\260A\340\004\013\2101\340\004\017\210t\2208\2309\240C\240s\250'\260\021\260!\330\004\016\210d\220+\230W\240A\240Q\330\004\r\210T\220\033\230G\2401\240A\360\006\000\005\010\200z\220\021\220$\220g\230R\230z\250\023\250D\260\001\330\006\016\210b\220\006\220a\220t\2307\240&\250\004\250A\330\006\022\220,\230a\230q\340\006\016\210b\220\006\220a\220t\2306\240\024\240Q\330\006\022\220!\330\004\007\200t\2107\220#\220Q\340\006\n\210(\220%\220q\230\001\340\006\n\210(\220!\340\004\035\230S\240\001\240\022\2408\2504\250q\3200K\3101\340\004\010\210\016\220l\240!\2404\240|\260>\300\021\330\"&\240g\250T\260\027\270\004\270A\330\"&\240o\260T\270\021\330\"-\250Q\330\"&\240h\250l\270!\330\"&\240h\250a\330\"&\240h\250a\330\"&\240a\330\"#\330\004\007\200t\210<\220r\230\021\330\006\014\210L\230\001\320\0315\260R\260t\270:\300Q\340\004\007\200t\2108\2207\230!\2301\340\006 \240\001\240\024\240]\260)\2701\330!$\240A\240Y\250a\330\006 \240\001\240\024\240]\260+\270Q\330!$\240A\240Z\250q\330\006 \240\001\240\024\240]\260)\2701\330!$\240A\320%5\260Q\330\006\t\210\024\210X\220S\230\001\330\010\021\220\031\230\"\230F\240!\2401\240D\250\n\260&\270\001\340\010\033\2301\230D\240\r\250Z\260q\330\034\037\230v\240]\260!\2601\360\010\000\005\t\210\013\220?\240!\2404\240q\340\004\013\2104\210q\200!\360\"\000\005\r\210H\220H\230D\240\004\240A\330\004\010\210\003\2105\220\t\230\021\230$\230a\330\006\t\210\022\2103\210a\330\010\016\210g\220Q\220a\330\010\016\210g\220Q\220a\330\010\r\210W\220A\220Q\340\010\016\210g\220Q\220a\330\010\016\210g\220Q\220a\330\010\r\210W\220A\220Q\330\004\r\210R\210v\220Q\220h\230f\240A\330\004\r\210R\210v\220Q\220h\230f\240A\330\004\014\210B\210f\220A\220W\230F\240!\360\006\000\005\016\210[\230\014\240A\240Q\330\004\r\210[\230\014\240A\240Q\330\004\014\210K\220|\2401\240A\360\006\000\005\025\220G\320\033+\2508\2601\330\004\013\2105\220\003\2201\330\004\022\220,\230a\230q\330\004\027\220q\230\n\240/\260\021\330\030 \240\007\240x\250q\200!\360\022\000\005\035\230K""\240q\250\004\250A\330\004\031\230\021\360\006\000\005\020\210|\2301\230D\240\001\340\004\013\2105\220\001\330\004\023\320\023#\2401\240C\240q\250\006\250a\360\010\000\005\022\220\021\220!\340\004\010\210\010\220\001\330\006\n\320\n\034\230A\230[\250\001\360\006\000\005\010\200t\2105\220\006\220c\230\021\330\006\017\210v\220Q\220e\2306\240\022\2401\330\006\r\210Q\340\006\r\210\\\230\021\230!\360\006\000\n\013\330\010\016\210g\220Q\220d\230-\240t\250:\260^\3001\330\026#\2401\340\004\005\330\006\t\210\024\210R\210q\330\010\016\210l\230!\2301\360\006\000\007\n\210\024\210U\220&\230\003\2301\330\010\026\220a\220t\230=\250\016\260f\270K\300q\340\006\t\210\021\330\010\014\210A\210Q\330\010\021\220\021\360\006\000\005\r\210A\210Q\340\004\014\210A\210Q\340\004\007\200t\2105\220\006\220c\230\021\340\006\t\210\034\220Q\220d\230*\240C\240q\330\010\r\210Y\220a\220q\360\006\000\005\010\200t\2105\220\006\220c\230\021\330\006\n\320\n\032\230!\2307\240!\340\004\005\200!\360\022\000\005\035\230K\240q\250\004\250A\330\004\031\230\021\360\006\000\005\020\210|\2301\230D\240\001\340\004\013\2105\220\001\330\004\023\320\023#\2401\240C\240q\250\006\250a\360\006\000\005\027\220a\220z\240\021\330\027\037\230v\240[\260\014\270A\270Q\360\006\000\005\010\200t\2105\220\006\220c\230\021\330\006\017\210v\220Q\220e\2306\240\022\2401\330\006\r\210Q\340\006\r\210\\\230\021\230!\360\006\000\n\013\330\010\016\210g\220Q\220d\230-\240t\250:\260^\3001\330\026#\2401\340\004\005\330\006\t\210\024\210R\210q\330\010\016\210l\230!\2301\360\006\000\007\n\210\024\210U\220&\230\003\2301\330\010\026\220a\220t\230=\250\016\260f\270K\300q\340\006\t\210\021\330\010\014\210A\210Q\330\010\021\220\021\360\006\000\005\r\210A\210Q\340\004\014\210A\210Q\340\004\007\200t\2105\220\006\220c\230\021\340\006\t\210\034\220Q\220d\230*\240C\240q\330\010\r\210Y\220a\220q\360\006\000\005\010\200t\2105\220\006\220c\230\021\330\006\n\320\n\032\230!\2307\240!\340\004\005\200!\360\014\000\005\035\230K\240q\250\004\250A\330\004\031\230""\021\360\006\000\005\r\320\014\034\230A\230W\240F\250!\360\006\000\005\010\200t\2105\220\006\220c\230\021\330\006\017\210v\220Q\220f\230B\230a\330\006\r\210Q\340\006\r\210\\\230\021\230!\340\004\007\200w\210a\210v\220Q\330\006\017\210t\2201\340\006\020\220\001\360\006\000\n\013\330\010\016\210k\230\021\230$\230m\2504\250z\270\027\300\007\300q\330\032\"\240!\340\004\005\330\006\t\210\024\210R\210q\330\010\016\210l\230!\2301\360\006\000\007\n\210\024\210U\220&\230\003\2301\330\010\026\220a\220t\230=\250\016\260f\270K\300q\340\006\t\210\021\330\010\014\210A\210Q\330\010\021\220\021\340\004\007\200t\2105\220\006\220c\230\021\340\006\t\210\034\220Q\220d\230*\240C\240q\330\010\r\210Y\220a\220q\360\006\000\005\010\200t\2105\220\006\220c\230\021\330\006\n\320\n\032\230!\2307\240!\340\004\005\200!\360\016\000\005\035\230K\240q\250\004\250A\330\004\031\230\021\360\006\000\005\r\210K\220|\2401\240A\330\004\013\210;\220l\240!\2401\330\004\013\210;\220l\240!\2401\360\006\000\005\010\200t\2105\220\006\220c\230\021\330\006\017\210v\220Q\220e\2306\240\022\2401\330\006\r\210Q\340\006\r\210\\\230\021\230!\340\004\027\220t\2308\2409\250G\2601\260A\330\004\032\230!\340\t\n\330\010\016\320\016\037\230q\240\n\250$\320.D\300D\310\r\320UY\320YZ\330 '\240v\250V\2601\330\004\005\330\006\t\210\024\210R\210q\330\010\016\210l\230!\2301\330GI\310\021\360\006\000\007\n\210\024\210U\220&\230\003\2301\330\010\026\220a\220t\230=\250\016\260f\270K\300q\340\006\t\210\021\330\010\014\210A\210Q\330\010\021\220\021\340\004\007\200t\2105\220\006\220c\230\021\340\006\t\210\034\220Q\220d\230*\240C\240q\330\010\r\210Y\220a\220q\360\006\000\005\010\200t\2105\220\006\220c\230\021\330\006\n\320\n\032\230!\2307\240!\340\004\005\200!\330\006\r\320\r \240\001\240\024\240Q\200!\340\004\010\210\010\220\001\220\024\220Q\330\004\013\2101\200!\360\036\000\005\036\230Q\360\010\000\005\034\2301\340\004\024\220A\340\004\027\220x\230w\240a\240q\340\004\020\220\001\360\006\000\005\017\210d\220!\330\004\013\2101\340\004\n""\320\n\036\230a\230y\250\013\2601\260I\270Q\270a\330\037 \240\013\2501\250A\330\004\007\200t\2102\210Q\330\006\014\210L\230\001\230\021\330\032$\240D\250\001\360\006\000\005\t\210\005\210S\220\002\220$\220i\230s\240!\330\006\022\320\022,\250A\250Y\260k\300\021\300!\330-.\250a\330\006\t\210\032\2203\220a\330\010\013\2105\220\003\2201\330\n\025\220R\220u\230A\230Q\340\n\025\220R\220w\230a\230q\340\013\020\220\003\2201\330\010\023\320\023'\240q\250\013\260;\270a\330\010\023\2202\220U\230!\2301\340\010\023\320\023,\250A\250[\270\001\360\020\000\t\024\2208\2307\240!\2401\330\010\023\2202\220W\230A\230Q\340\006\t\210\021\330\010\014\210A\210Q\330\006\016\210a\210q\330\006\r\210Q\330\n\017\210s\220\"\220D\230\t\240\024\240^\260=\300\001\330\006\017\320\017\037\230q\240\t\250\032\2601\330\006\t\210\027\220\003\2201\330\010\020\220\005\220Q\220a\3304>\270d\300!\3304?\270q\330\010\014\320\014\035\230W\240A\240Q\330\010\017\210q\330\006\016\210a\360\n\000\007\016\210[\230\006\230a\230u\240B\240a\330\006\014\210N\230!\2309\240K\250q\330\006\t\210\024\210R\210q\330\010\016\210l\230!\2301\330\034&\240d\250!\330\006\016\210h\220a\220v\230Q\340\006\n\210!\2109\220A\360\006\000\007\010\330\010\020\220\t\320\031,\250A\250Y\3206G\300v\310Y\320VZ\320Zb\320bc\330\010\021\220\022\2206\230\022\2308\2401\330\r\016\330\010\013\2109\220C\220{\240$\320&8\270\001\270\021\330\n\026\320\026;\2701\270I\300Q\330<=\270\\\310\021\310!\330\n\r\210Z\220r\230\021\330\014\022\220,\230a\230q\330 *\250$\250a\360\006\000\013\016\210U\220#\220Q\360\014\000\r\030\220r\230\026\230q\240\001\330\016\"\240!\2407\250*\260A\260Q\330$*\250!\2507\260*\270A\270Q\330$%\330\020\024\220E\230\025\230a\230}\250A\360\n\000\r\030\220r\230\026\230q\330\016\017\210w\220j\240\001\240\023\240D\250\005\250U\260!\260=\300\001\330\n\022\220)\2301\360\006\000\013\017\210e\2205\230\001\230\021\330\014\020\220\001\220\032\2301\230A\330\n\016\210a\210q\340\n\021\220\021\360\n\000\t\021\220\005\220Q\220a\3304>\270d\300!\3304?\270q\330""\010\014\320\014\035\230W\240A\240Q\330\010\017\210q\360\006\000\005\017\210b\220\006\220a\220v\230X\240V\2501\340\004\013\210<\220q\230\001\340\004\n\320\n\035\230Q\230i\240{\260)\2701\330\004\007\200t\2102\210Q\330\006\014\210L\230\001\230\021\330\032$\240D\250\001\330\004\014\210A\210Q\340\004\007\200u\210B\210a\330\006\021\220\021\340\006\021\220\027\230\001\230\021\340\004\013\2101\200\001\330\004\n\210+\220Q\200\001\360\010\000\005\016\210T\220\027\230\004\230A\330\004\014\210G\2201\220F\230,\240a\330\004\007\200v\210W\220E\230\024\230Q\330\010\022\220!\330\010\027\220q\340\010\027\220t\2306\240\027\250\001\330\004\007\200q\330\010\017\320\017%\240T\250\021\250'\260\033\270G\3001\340\010\017\320\017%\240T\250\021\250'\260\033\270A\200\001\360\010\000\005\016\210T\220\033\230D\240\007\240t\2501\330\004\014\210G\2201\220F\230,\240a\330\004\007\200v\210W\220E\230\024\230Q\330\010\022\220!\330\010\027\220q\340\010\027\220t\2306\240\027\250\001\330\004\007\200q\330\010\017\320\017&\240d\250!\2507\260+\270W\300A\340\010\017\320\017&\240d\250!\2507\260+\270Q\200\001\360\010\000\005\016\210T\220\021\330\004\014\210G\2201\220F\230,\240a\330\004\007\200v\210W\220E\230\024\230Q\330\010\022\220!\330\010\027\220q\340\010\027\220t\2306\240\027\250\001\330\004\007\200q\330\010\017\320\017-\250T\260\021\260'\270\033\300G\3101\340\010\017\320\017-\250T\260\021\260'\270\033\300A\200\001\360\010\000\005\016\210T\220\037\240\004\240J\250d\260!\330\004\014\210G\2201\220F\230,\240a\330\004\007\200v\210W\220E\230\024\230Q\330\010\022\220!\330\010\027\220q\340\010\027\220t\2306\240\027\250\001\330\004\007\200q\330\010\017\320\017%\240T\250\021\250'\260\033\270G\3001\340\010\017\320\017%\240T\250\021\250'\260\033\270A\200!\330\004\010\210\010\220\001\340\004\010\210\r\220U\230!\200!\360\016\000\005\031\230\001\330\004\032\230!\360\010\000\005\t\210\006\210a\360\006\000\005\014\320\013\034\230A\230T\240\032\2506\260\021\330\004\007\200u\210B\210a\330\006\014\210L\230\001\230\021\360""\n\000\005\r\320\014%\240Q\240f\250A\330\004\007\200t\2101\330\006\014\210L\230\001\230\021\340\004\r\210Q\330\004\016\210a\330\004\013\320\013\034\230A\230T\240\032\2507\260(\270!\330\004\007\200u\210B\210a\330\006\014\210L\230\001\230\021\360\010\000\005\014\2101\200!\330\004\026\220a\330\004,\250A\330\004\037\320\037;\2701\330\004\036\320\0363\2601\330\004\033\2301\360\n\000\005\016\210V\2201\220A\330\004\007\200w\210g\220U\230$\230g\240W\250A\330\006\014\210J\220a\320\027?\270r\300\021\330\004\007\200w\210c\220\021\330\006\021\220\026\220t\2301\320\0345\260Q\330\006\020\220\006\220d\230!\320\0333\2601\330\006\022\220)\2302\230X\240U\250#\250^\2705\300\002\300!\330\006\021\220\030\230\022\2308\2405\250\003\250=\270\005\270R\270q\330\006\025\220_\240A\240Q\330\006\024\220O\2401\240A\360\006\000\005\t\210\010\220\001\360\006\000\005\017\210o\230Q\230a\360\006\000\005\t\210\n\220!\360\006\000\005\t\210\r\220Q\360\006\000\005\014\2107\220$\220e\2306\240\025\240g\250Q\340/1\260\021\340\004\014\210F\220$\220a\220q\330\004\007\200q\330\006\t\210\027\220\003\2201\330\010\020\220\005\220Q\220a\330(*\250!\330\013\017\210}\230A\230Q\330\010\016\210i\220q\230\001\360\010\000\005\010\200w\210c\220\021\330\006\n\210%\210{\230!\330\010\031\230\021\230#\230Q\340\006\026\220f\230D\240\001\320!>\270a\330\006\t\210\027\220\003\220<\230s\240!\330\010\031\230\021\230&\240\001\360\006\000\005\010\200q\330\006\017\210q\330\t\020\220\003\2201\330\006\017\210r\220\025\220g\230Q\230k\250\024\250R\250u\260G\2701\270A\340\006\017\210r\220\025\220g\230Q\230a\330\004\010\210\n\220%\220w\230d\240%\240v\250T\260\027\270\003\2704\270t\3001\340\004\026\220f\230D\240\001\320!4\260A\330\004\007\320\007\027\220t\2304\230t\2401\330\010\020\220\005\220Q\220a\340\t\n\330\006\030\230\003\2301\230A\330\006\026\320\026&\240c\320)9\270\022\2704\270s\300!\330\006\t\320\t\031\230\022\2304\230s\240$\240a\330\010\016\210j\230\001\230\021\360\010\000\007\026\220Y\230a\230q\330\006\014\320\014\034\230A\230^\2501""\330\006\t\210\024\210R\210q\330\010\020\220\001\220\021\330\010\016\210l\230!\2301\360\006\000\005\024\2209\230A\230Q\360\006\000\005\021\220\001\220\036\230q\330\021\027\220q\230\001\330\021\027\220q\230\001\330\021\027\220q\230\001\360\006\000\005\010\200w\210c\220\021\330\006\014\320\014\034\230A\230Q\330\t\020\220\003\2201\330\006\t\210\024\210Q\330\010\020\220\001\220\021\330\010\020\220\001\220\021\330\010\016\210l\230!\2301\330\006\014\320\014!\240\021\240!\330\"(\250\001\250\021\330\"(\250\001\250\021\330\"(\250\001\250\021\360\026\000\n\021\220\003\2201\330\006\t\210\024\210Q\330\010\020\220\001\220\021\330\010\020\220\001\220\021\330\010\016\210l\230!\2301\330\006\014\320\014\"\240!\2401\330\t\020\220\003\2201\330\006\014\320\014\035\230Q\230a\330\t\020\220\003\2201\330\006\014\320\014\034\230A\230Q\330\035#\2401\240A\330\035\036\330\006\t\210\021\330\010\026\220c\230\021\230!\330\010\024\220H\320\034,\250A\250Q\330\010\016\320\016\037\230q\240\016\250k\270\021\330\010\013\2104\210r\220\021\330\n\022\220!\2201\330\n\022\220!\2201\330\n\020\220\014\230A\230Q\360\022\000\n\021\220\003\2201\330\006\014\320\014\035\230Q\230n\250N\270!\330\036+\2501\330\004\007\200t\2102\210Q\330\006\n\210,\220a\220q\330\006\016\210a\210q\330\006\016\210a\210q\330\006\014\210A\340\004\007\200w\210c\220\021\330\006\n\210+\220W\230A\230Y\320&6\260a\330\t\020\220\003\2201\330\006\n\210+\220W\230A\230Y\240n\260A\330\t\020\220\003\2201\330\006\t\210\021\360\010\000\t\r\210K\220w\230a\230y\250\016\260a\340\010\014\210K\220y\240\001\240\031\250/\270\021\330!\"\330\t\020\220\003\2201\330\006\n\210+\220Y\230a\230y\250\017\260q\330\037 \340\004\007\200t\2109\220B\220a\330\010\014\210L\230\001\320\031;\2702\270Q\330\010\020\220\001\220\021\330\010\020\220\001\220\021\330\010\016\210a\340\004\014\210A\210Q\330\004\014\210A\210Q\360\006\000\005\023\220!\2204\220z\240\026\240q\250\001\360\006\000\005\032\230\021\230&\240\001\240\021\330\004\032\230!\2306\240\021\240!\200!\330\004\026\220a""\360\006\000\005\013\210+\220Q\220d\230.\250\001\250\021\330\004\007\200t\2102\210Q\330\006\014\210L\230\001\320\031>\270a\330\031\035\230Q\340\004\013\2107\220!\2205\230\007\230u\240A\200!\330\004\026\220a\360\006\000\005\013\210+\220Q\220d\230.\250\001\250\021\330\004\007\200t\2102\210Q\330\006\014\210L\230\001\320\031>\270a\330\031\035\230Q\340\004\013\210=\230\001\230\025\230h\240e\2508\2605\270\001\330\031\036\230a\200!\360\016\000\005\027\220a\330\004\030\230\001\360\006\000\005\024\320\023&\240a\240t\2501\330\004\007\200}\220B\220a\330\006\014\210L\230\001\230\021\340\004\n\320\n\032\230!\230>\250\021\250!\330\004\007\200t\2102\210Q\330\006\016\210a\210q\330\006\014\210L\230\001\230\021\340\004\014\210A\210Q\340\004\013\2101\200!\360\030\000\005\027\220a\330\004\030\230\001\340\004\n\210/\230\021\230$\230j\250\001\250\021\330\004\007\200t\2102\210Q\330\006\014\210L\230\001\230\021\340\004\013\2101\200!\360\034\000\n\013\330\006\017\210{\230,\240a\240q\330\006\014\320\014&\240a\240t\250=\270\001\330'(\250\r\260Q\260f\270A\270Q\330\004\007\200t\2102\210Q\330\006\014\210L\230\001\230\021\330\031\033\2304\230q\330\004\014\210A\210]\230&\240\t\250\025\250c\260\021\330\022\030\230\006\230a\200!\360\030\000\005\031\230\t\240\027\250\001\250\021\330\004\030\230\t\240\027\250\001\250\021\360\006\000\005\017\210g\220Q\220d\230+\320%8\270\001\330\004\007\200x\210r\220\021\330\006\014\210L\230\001\230\021\330\032%\240T\250\021\330\004\017\320\017+\2501\250I\260Q\340\004\014\210A\210Q\330\004\013\2101\200!\360\n\000\005\027\220g\230W\240A\240Q\330\004\026\220g\230W\240A\240Q\340\004\n\210'\220\021\220+\320\035.\250k\270\021\330\022\037\230q\330\004\007\200t\2102\210Q\330\006\014\210L\230\001\230\021\330\032'\240q\330\004\013\2101\200\001\340\004\037\230q\320 0\260\013\270;\300k\320QR\330\004\023\2204\220x\230q\240\001\330\004\007\200|\2207\230!\330\010&\240a\240w\250n\270A\330\004\013\2101\200\001\340\004\037\230q\320 0\260\013\270;\300k\320QR\330\004\023\2205\230\010""\240\001\240\021\330\004\007\200|\2207\230!\330\010'\240q\250\010\260\016\270a\330\004\013\2101\200\001\340\004\037\230q\320 0\260\013\270;\300k\320QR\330\004\023\220<\230x\240q\250\001\330\004\007\200|\2207\230!\330\010.\250a\250\177\270n\310A\330\004\013\2101\200!\360\034\000\005\030\220q\340\004\024\220A\340\004\023\2204\220w\230a\230q\340\004\014\210A\360\006\000\005\017\210d\220!\360\006\000\005\010\200z\220\021\220'\230\022\2301\330\006\016\210b\220\006\220a\220q\360\006\000\005\t\210\n\220!\2207\230\"\230J\240a\330\010\r\210V\2206\230\024\230U\240%\240u\250E\260\025\260e\2701\360\006\000\007\017\210e\2205\230\001\330\006\t\210\025\210f\220F\230#\230Q\330\010\025\220^\320#3\2601\260E\270\030\300\031\310$\310h\320VW\330\010\024\220J\230a\230q\330\010\022\320\022$\240A\240]\260!\340\010\020\220\005\220W\230A\230[\250\001\360\006\000\t\024\2204\220{\240!\2405\250\006\250a\330\010\024\220J\230a\230u\240F\250!\330\010\022\320\022#\2401\240J\250a\340\006\014\210I\220Q\330\006\r\210_\230A\230\\\250\021\250&\260\014\270A\270Q\340\006\014\320\014\037\230q\240\t\250\027\260\001\330 ,\250A\250V\2606\270\035\300a\300q\330\006\t\210\024\210R\210q\330\010\016\210l\230!\2301\330\034\"\240$\240a\340\006\n\210!\2108\2201\330\006\016\210a\210q\360\006\000\007\013\210*\220A\220W\230B\230j\250\001\330\n\017\210v\220V\2303\230d\240!\330\n\017\210w\220c\230\021\330\010\020\220\005\220Q\220c\230\027\240\001\240\021\330\010\017\210q\360\010\000\t\021\220\006\220f\230A\230W\240A\340\006\014\320\014&\240a\240y\260\007\260w\270c\300\021\300(\310!\330\006\t\210\024\210R\210q\330\010\016\210l\230!\2301\330\034\"\240$\240a\200!\360\n\000\005\013\210-\220q\230\004\230M\250\024\250Z\260q\330\004\007\200t\2102\210Q\330\006\014\210L\230\001\320\031<\270B\270a\340\004\020\220\004\220J\230a\330\004\007\200z\220\024\220Z\230q\340\006\n\210%\210q\220\004\220L\240\001\340\006\016\210d\220!\2204\220q\330\006\013\2101\210D\220\014\230H\240A\240Q\330\006\n\210)\2205\230\001\230\021\330\t\023\2204\220y""\240\001\330\006\n\210)\2201\340\006\014\210J\220a\320\0272\260\"\260A\200!\360\n\000\005\010\200q\330\006\r\210\\\230\021\230!\340\006\t\210\024\210U\220&\230\003\2301\330\010\014\320\014\034\230A\230W\240A\340\006\r\210Q\360\006\000\n\013\330\010\016\320\016%\240Q\240d\250-\260t\2701\330&0\260\004\260K\270q\340\004\007\200t\2102\210Q\330\006\014\210L\230\001\230\021\340\004\010\210\014\220D\230\n\240\"\240A\200!\330\004\007\200q\340\006\n\210/\230\021\330\006\n\210,\220a\330\006\n\320\n\033\2301\330\006\n\320\n\033\2301\330\004\t\210\022\2107\220!\2207\230&\240\001\200!\360\n\000\005\014\210<\220q\230\001\330\004\007\200q\340\006\t\210\024\210U\220&\230\003\2301\330\010\014\320\014\034\230A\230W\240A\360\006\000\n\013\330\010\016\320\016%\240Q\240d\250-\260t\2701\330&,\250J\260a\340\004\007\200t\2102\210Q\330\006\014\210L\230\001\230\021\340\004\013\2101\200\001\330\004\007\200s\210!\210=\230\003\2301\330\006\024\220J\230c\240\023\240A\330\023\031\230\033\240A\240S\250\003\2503\250k\270\021\270!\340\006\022\220!\360\010\000\005\021\220\001\330\020\026\220h\230g\240R\240v\250S\260\003\260;\270a\330\026 \240\003\2401\330\004\014\210J\220a\200!\360\024\000\005\010\200t\2103\210d\220!\330\006\014\210L\230\001\330\010\t\330\010\r\210Q\360\006\000\005\020\210|\2301\230D\240\001\340\004\n\210!\2105\220\001\330\004\t\210\021\210%\210q\340\004\n\320\n\035\230Q\230j\320(8\270\010\300\006\300g\310Q\330\004\007\200t\2102\210Q\330\006\016\210a\340\004\n\320\n\036\230a\230t\240=\260\004\260J\270j\310\001\310\021\330\004\007\200t\2102\210Q\330\006\016\210a\360\006\000\005\r\210A\210Q\340\004\013\2101\200!\360\024\000\005\010\200t\2105\220\006\220c\230\021\330\006\014\210J\220a\220q\340\004\017\210|\2301\230D\240\001\340\004\013\2105\220\001\330\004\023\320\023#\2401\240C\240q\250\006\250a\360\010\000\005\022\220\021\220!\340\004\010\210\010\220\001\330\006\n\320\n\034\230A\230[\250\001\360\006\000\005\014\210<\220q\230\001\360\006\000\005\010\200t\2105\220\006\220c\230\021\330""\006\n\320\n\032\230!\2307\240!\360\006\000\n\013\330\010\016\210h\220a\220t\230=\250\004\250J\260n\300A\330\027$\240A\340\004\007\200t\2102\210Q\330\006\014\210L\230\001\230\021\360\006\000\005\r\210A\210Q\340\004\014\210A\210Q\340\004\005\200!\360\024\000\005\010\200t\2105\220\006\220c\230\021\330\006\014\210J\220a\220q\340\004\017\210|\2301\230D\240\001\340\004\013\2105\220\001\330\004\023\320\023#\2401\240C\240q\250\006\250a\360\006\000\005\027\220a\220z\240\021\330\027\037\230v\240[\260\014\270A\270Q\360\006\000\005\014\210<\220q\230\001\360\006\000\005\010\200t\2105\220\006\220c\230\021\330\006\n\320\n\032\230!\2307\240!\360\006\000\n\013\330\010\016\210h\220a\220t\230=\250\004\250J\260n\300A\330\027$\240A\340\004\007\200t\2102\210Q\330\006\014\210L\230\001\230\021\360\006\000\005\r\210A\210Q\340\004\014\210A\210Q\340\004\005\200!\360\014\000\005\010\200t\2105\220\006\220c\230\021\330\006\014\210J\220a\220q\360\006\000\005\020\210\177\230a\230t\2407\250,\260a\260q\340\004\013\210<\220q\230\001\340\004\007\200t\2105\220\006\220c\230\021\330\006\n\320\n\032\230!\2307\240!\360\006\000\005\016\210T\220\021\330\t\n\330\010\016\320\016#\2401\240D\250\r\260T\270\032\3004\300q\330$(\250\007\250z\270\030\300\021\340\004\007\200t\2102\210Q\330\006\014\210L\230\001\230\021\340\004\010\210\001\210\021\340\004\014\210D\220\001\220\024\220Q\330\004\t\210\021\210$\210k\230\030\240\021\240$\240e\2501\250D\260\001\330\004\010\210\t\220\025\220a\220q\200!\360\026\000\005\010\200t\2105\220\006\220c\230\021\330\006\014\210J\220a\220q\340\004\013\210<\220q\230\001\340\004\014\210K\220|\2401\240A\330\004\013\210;\220l\240!\2401\330\004\014\210K\220|\2401\240A\360\006\000\005\010\200t\2105\220\006\220c\230\021\330\006\n\320\n\032\230!\2307\240!\360\006\000\n\013\330\010\016\320\016\"\240!\2404\240}\260D\270\n\300$\300a\330$+\2506\260\027\270\001\340\004\007\200t\2102\210Q\330\006\014\210L\230\001\230\021\330HJ\310!\340\004\005\200!\360\n\000\005\010\200t\2107\220#\220Q\330\006\r""\210Q\360\006\000\007\022\220\034\230Q\230d\240!\340\006\014\320\014 \240\001\240\024\240]\260$\260j\300\001\330!\"\240!\330\006\t\210\024\210R\210q\330\010\020\220\001\360\006\000\007\017\210a\210q\340\004\013\2101\200!\360\006\000\005\010\200t\2109\220C\220q\330\006\016\210a\210t\2201\330\004\007\200t\210>\230\023\230A\330\006\016\210a\210t\2201\330\004\007\200t\210>\230\023\230A\330\006\016\210a\210t\2201\330\004\007\200t\210<\220s\230!\330\006\016\210a\210t\2201\200!\340\004\007\200t\210<\220s\230!\330\010\020\220\001\220\024\220]\240!";
    PyObject *data = NULL;
    CYTHON_UNUSED_VAR(__Pyx_DecompressString);
    #endif
    PyObject **stringtab = __pyx_mstate->__pyx_string_tab;
    Py_ssize_t pos = 0;
    for (int i = 0; i < 608; i++) {
      Py_ssize_t bytes_length = index[i].length;
      PyObject *string = PyUnicode_DecodeUTF8(bytes + pos, bytes_length, NULL);
      if (likely(string) && i >= 112) PyUnicode_InternInPlace(&string);
      if (unlikely(!string)) {
        Py_XDECREF(data);
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
      stringtab[i] = string;
      pos += bytes_length;
    }
    for (int i = 608; i < 679; i++) {
      Py_ssize_t bytes_length = index[i].length;
      PyObject *string = PyBytes_FromStringAndSize(bytes + pos, bytes_length);
      stringtab[i] = string;
      pos += bytes_length;
      if (unlikely(!string)) {
        Py_XDECREF(data);
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
    }
    Py_XDECREF(data);
    for (Py_ssize_t i = 0; i < 679; i++) {
      if (unlikely(PyObject_Hash(stringtab[i]) == -1)) {
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
    }
    #if CYTHON_IMMORTAL_CONSTANTS
    {
      PyObject **table = stringtab + 608;
      for (Py_ssize_t i=0; i<71; ++i) {
        #if PY_VERSION_HEX >= 0x030F0000
        PyUnstable_SetImmortal(table[i]);
        #elif CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
        if ((PY_SSIZE_T_MAX <= _Py_IMMORTAL_REFCNT_LOCAL)) break;
        #if PY_VERSION_HEX < 0x030E0000
        if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
        #else
        if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
        #endif
        {
          Py_SET_REFCNT(table[i], ((Py_ssize_t)_Py_IMMORTAL_REFCNT_LOCAL + 1));
        }
        #else
        if ((PY_SSIZE_T_MAX < _Py_IMMORTAL_INITIAL_REFCNT)) break;
        Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
        #endif
      }
    }
    #endif
  }
  {
    PyObject **numbertab = __pyx_mstate->__pyx_number_tab + 0;
    int8_t const cint_constants_1[] = {0,-1,1};
    int32_t const cint_constants_4[] = {55151181L,123500641L,136983863L,223533300L};
    for (int i = 0; i < 7; i++) {
      numbertab[i] = PyLong_FromLong((i < 3 ? cint_constants_1[i - 0] : cint_constants_4[i - 3]));
      if (unlikely(!numbertab[i])) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_number_tab;
    for (Py_ssize_t i=0; i<7; ++i) {
      #if PY_VERSION_HEX >= 0x030F0000
      PyUnstable_SetImmortal(table[i]);
      #elif CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      if ((PY_SSIZE_T_MAX <= _Py_IMMORTAL_REFCNT_LOCAL)) break;
      #if PY_VERSION_HEX < 0x030E0000
      if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
      #else
      if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
      #endif
      {
        Py_SET_REFCNT(table[i], ((Py_ssize_t)_Py_IMMORTAL_REFCNT_LOCAL + 1));
      }
      #else
      if ((PY_SSIZE_T_MAX < _Py_IMMORTAL_INITIAL_REFCNT)) break;
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_codeobjects ### */
typedef struct {
    unsigned int argcount : 3;
    unsigned int num_posonly_args : 1;
    unsigned int num_kwonly_args : 1;
    unsigned int nlocals : 5;
    unsigned int flags : 10;
    unsigned int first_line : 12;
} __Pyx_PyCode_New_function_description;
/* NewCodeObj.proto */
static PyObject* __Pyx_PyCode_New(
        const __Pyx_PyCode_New_function_description descr,
        PyObject * const *varnames,
        PyObject *filename,
        PyObject *funcname,
        PyObject *line_table,
        PyObject *tuple_dedup_map
);


static int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate) {
  PyObject* tuple_dedup_map = PyDict_New();
  if (unlikely(!tuple_dedup_map)) return -1;
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 26, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_VARKEYWORDS), 445};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_pymode, __pyx_mstate->__pyx_n_u_params, __pyx_mstate->__pyx_n_u_err, __pyx_mstate->__pyx_n_u_access_plist, __pyx_mstate->__pyx_n_u_create_plist, __pyx_mstate->__pyx_n_u_meta_plist_id, __pyx_mstate->__pyx_n_u_raw_plist_id, __pyx_mstate->__pyx_n_u_img_buf_len, __pyx_mstate->__pyx_n_u_user_block_size, __pyx_mstate->__pyx_n_u_img_buf_p, __pyx_mstate->__pyx_n_u_encname, __pyx_mstate->__pyx_n_u_driver, __pyx_mstate->__pyx_n_u_meta_ext, __pyx_mstate->__pyx_n_u_raw_ext, __pyx_mstate->__pyx_n_u_meta_name, __pyx_mstate->__pyx_n_u_raw_name, __pyx_mstate->__pyx_n_u_enc_meta_ext, __pyx_mstate->__pyx_n_u_enc_raw_ext, __pyx_mstate->__pyx_n_u_image, __pyx_mstate->__pyx_n_u_n, __pyx_mstate->__pyx_n_u_backing_store, __pyx_mstate->__pyx_n_u_exists, __pyx_mstate->__pyx_n_u_is_pow_of_2, __pyx_mstate->__pyx_n_u_e};
    __pyx_mstate_global->__pyx_codeobj_tab[0] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_new, __pyx_mstate->__pyx_kp_b_iso88591_a_A_1_31_1_V1A_wgU_gWA_Ja_r_wc, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[0])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 630};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_size, __pyx_mstate->__pyx_n_u_buf_len, __pyx_mstate->__pyx_n_u_image, __pyx_mstate->__pyx_n_u_cimage};
    __pyx_mstate_global->__pyx_codeobj_tab[1] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_get_file_image, __pyx_mstate->__pyx_kp_b_iso88591_a_AT_6_uBa_L_QfA_t1_L_Q_a_AT_7, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[1])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 666};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_err, __pyx_mstate->__pyx_n_u_size};
    __pyx_mstate_global->__pyx_codeobj_tab[2] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_get_filesize, __pyx_mstate->__pyx_kp_b_iso88591_a_j_t2Q_L_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[2])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 687};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_err, __pyx_mstate->__pyx_n_u_size, __pyx_mstate->__pyx_n_u_create_plist};
    __pyx_mstate_global->__pyx_codeobj_tab[3] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_get_userblock_size, __pyx_mstate->__pyx_kp_b_iso88591_a_at1_Ba_L_t2Q_aq_L_AQ_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[3])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 712};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[4] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_get_file_id, __pyx_mstate->__pyx_kp_b_iso88591_4q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[4])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 715};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_file_handle, __pyx_mstate->__pyx_n_u_descriptor, __pyx_mstate->__pyx_n_u_err};
    __pyx_mstate_global->__pyx_codeobj_tab[5] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_fileno, __pyx_mstate->__pyx_kp_b_iso88591_1D_q_t2Q_L_2T_a_Qa, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[5])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 735};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_scope};
    __pyx_mstate_global->__pyx_codeobj_tab[6] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_flush_file, __pyx_mstate->__pyx_kp_b_iso88591_AT_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[6])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 740};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[7] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_close_file, __pyx_mstate->__pyx_kp_b_iso88591_Bd_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[7])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_state, __pyx_mstate->__pyx_n_u_dict_2, __pyx_mstate->__pyx_n_u_use_setstate};
    __pyx_mstate_global->__pyx_codeobj_tab[8] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_T_Jd_G1F_a_vWE_Q_q_t6_q_T_G1_T, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[8])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 16};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[9] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_6, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[9])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 760};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_node};
    __pyx_mstate_global->__pyx_codeobj_tab[10] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_new, __pyx_mstate->__pyx_kp_b_iso88591_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[10])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 763};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_node, __pyx_mstate->__pyx_n_u_a};
    __pyx_mstate_global->__pyx_codeobj_tab[11] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_list_attr, __pyx_mstate->__pyx_kp_b_iso88591_Q_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[11])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 16, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 769};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_node, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_value, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_dset_id, __pyx_mstate->__pyx_n_u_type_id, __pyx_mstate->__pyx_n_u_dims, __pyx_mstate->__pyx_n_u_ndv, __pyx_mstate->__pyx_n_u_byteorder, __pyx_mstate->__pyx_n_u_rabyteorder, __pyx_mstate->__pyx_n_u_baseatom, __pyx_mstate->__pyx_n_u_cname, __pyx_mstate->__pyx_n_u_encoded_name, __pyx_mstate->__pyx_n_u_cset, __pyx_mstate->__pyx_n_u_description};
    __pyx_mstate_global->__pyx_codeobj_tab[12] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_setattr, __pyx_mstate->__pyx_kp_b_iso88591_q_A_4waq_A_d_z_1_b_aq_7_Ja_V6_U, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[12])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 29, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 846};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_node, __pyx_mstate->__pyx_n_u_attrname, __pyx_mstate->__pyx_n_u_dims, __pyx_mstate->__pyx_n_u_class_id, __pyx_mstate->__pyx_n_u_type_size, __pyx_mstate->__pyx_n_u_mem_type, __pyx_mstate->__pyx_n_u_dset_id, __pyx_mstate->__pyx_n_u_type_id, __pyx_mstate->__pyx_n_u_native_type, __pyx_mstate->__pyx_n_u_rank, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_enumtype, __pyx_mstate->__pyx_n_u_rbuf, __pyx_mstate->__pyx_n_u_str_value, __pyx_mstate->__pyx_n_u_str_values, __pyx_mstate->__pyx_n_u_ndvalue, __pyx_mstate->__pyx_n_u_shape, __pyx_mstate->__pyx_n_u_stype_atom, __pyx_mstate->__pyx_n_u_shape_atom, __pyx_mstate->__pyx_n_u_retvalue, __pyx_mstate->__pyx_n_u_i, __pyx_mstate->__pyx_n_u_nelements, __pyx_mstate->__pyx_n_u_cattrname, __pyx_mstate->__pyx_n_u_encoded_attrname, __pyx_mstate->__pyx_n_u_cset, __pyx_mstate->__pyx_n_u_dtype_2, __pyx_mstate->__pyx_n_u_stype, __pyx_mstate->__pyx_n_u_shape_2};
    __pyx_mstate_global->__pyx_codeobj_tab[13] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_getattr, __pyx_mstate->__pyx_kp_b_iso88591_Q_1_A_xwaq_d_1_ay_1IQa_1A_t2Q_L, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[13])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 7, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1001};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_node, __pyx_mstate->__pyx_n_u_attrname, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_dset_id, __pyx_mstate->__pyx_n_u_cattrname, __pyx_mstate->__pyx_n_u_encoded_attrname};
    __pyx_mstate_global->__pyx_codeobj_tab[14] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_remove, __pyx_mstate->__pyx_kp_b_iso88591_1_xwaq_d_1IQ_t2Q_L_1_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[14])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_state, __pyx_mstate->__pyx_n_u_dict_2, __pyx_mstate->__pyx_n_u_use_setstate};
    __pyx_mstate_global->__pyx_codeobj_tab[15] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_T_G1F_a_vWE_Q_q_t6_q_T_G1_T_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[15])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 16};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[16] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_6_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[16])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1023};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_where, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_init};
    __pyx_mstate_global->__pyx_codeobj_tab[17] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_new, __pyx_mstate->__pyx_kp_b_iso88591_U_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[17])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1029};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_parent, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_encoded_name};
    __pyx_mstate_global->__pyx_codeobj_tab[18] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_delete, __pyx_mstate->__pyx_kp_b_iso88591_4uG1A_1F_a_t2Q_L_Rt1_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[18])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1044};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_oinfo};
    __pyx_mstate_global->__pyx_codeobj_tab[19] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_get_obj_info, __pyx_mstate->__pyx_kp_b_iso88591_a_Qd_t2Q_L_a_Q_7_5_uA, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[19])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1055};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_oinfo};
    __pyx_mstate_global->__pyx_codeobj_tab[20] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_get_obj_timestamps, __pyx_mstate->__pyx_kp_b_iso88591_a_Qd_t2Q_L_a_Q_he85_a, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[20])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_state, __pyx_mstate->__pyx_n_u_dict_2, __pyx_mstate->__pyx_n_u_use_setstate};
    __pyx_mstate_global->__pyx_codeobj_tab[21] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_T_A_G1F_a_vWE_Q_q_t6_q_T_G1_T_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[21])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 16};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[22] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_6, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[22])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1071};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_encoded_name};
    __pyx_mstate_global->__pyx_codeobj_tab[23] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_create, __pyx_mstate->__pyx_kp_b_iso88591_4uG1A_1D_N_q_t2Q_L_6b_A_A_4q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[23])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1087};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_encoded_name};
    __pyx_mstate_global->__pyx_codeobj_tab[24] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_open, __pyx_mstate->__pyx_kp_b_iso88591_4uG1A_l_t2Q_L_7r_Q_A_4q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[24])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 6, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1099};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_h5name, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_node_type, __pyx_mstate->__pyx_n_u_encoded_name, __pyx_mstate->__pyx_n_u_cname};
    __pyx_mstate_global->__pyx_codeobj_tab[25] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_get_objinfo, __pyx_mstate->__pyx_kp_b_iso88591_6_A_at_a_t4r_D_1_S_S_S_k_k_4t1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[25])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1137};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_parent, __pyx_mstate->__pyx_n_u_encoded_name};
    __pyx_mstate_global->__pyx_codeobj_tab[26] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_list_group, __pyx_mstate->__pyx_kp_b_iso88591_4uG1A_81F_N, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[26])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 7, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1147};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_group_name, __pyx_mstate->__pyx_n_u_attr_name, __pyx_mstate->__pyx_n_u_gchild_id, __pyx_mstate->__pyx_n_u_retvalue, __pyx_mstate->__pyx_n_u_encoded_group_name, __pyx_mstate->__pyx_n_u_encoded_attr_name};
    __pyx_mstate_global->__pyx_codeobj_tab[27] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_get_gchild_attr, __pyx_mstate->__pyx_kp_b_iso88591_7_1_q_q_K_1_z_1_L_d_1Kq_AQ_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[27])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 7, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1175};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_leaf_name, __pyx_mstate->__pyx_n_u_attr_name, __pyx_mstate->__pyx_n_u_leaf_id, __pyx_mstate->__pyx_n_u_retvalue, __pyx_mstate->__pyx_n_u_encoded_leaf_name, __pyx_mstate->__pyx_n_u_encoded_attr_name};
    __pyx_mstate_global->__pyx_codeobj_tab[28] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_get_lchild_attr, __pyx_mstate->__pyx_kp_b_iso88591_gQd_8_xr_L_T_1IQ_AQ_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[28])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1201};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[29] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_flush_group, __pyx_mstate->__pyx_kp_b_iso88591_AT_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[29])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1206};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_ret};
    __pyx_mstate_global->__pyx_codeobj_tab[30] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_close_group, __pyx_mstate->__pyx_kp_b_iso88591_4q_t2Q_L_9_4q_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[30])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {7, 0, 0, 10, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1215};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_oldparent, __pyx_mstate->__pyx_n_u_oldname, __pyx_mstate->__pyx_n_u_newparent, __pyx_mstate->__pyx_n_u_newname, __pyx_mstate->__pyx_n_u_oldpathname, __pyx_mstate->__pyx_n_u_newpathname, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_encoded_oldname, __pyx_mstate->__pyx_n_u_encoded_newname};
    __pyx_mstate_global->__pyx_codeobj_tab[31] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_move_node, __pyx_mstate->__pyx_kp_b_iso88591_gWAQ_gWAQ_k_q_t2Q_L_q_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[31])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_state, __pyx_mstate->__pyx_n_u_dict_2, __pyx_mstate->__pyx_n_u_use_setstate};
    __pyx_mstate_global->__pyx_codeobj_tab[32] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_T_D_t1_G1F_a_vWE_Q_q_t6_q_d_7_W, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[32])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 16};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[33] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_1F, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[33])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1235};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[34] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_get_storage_size, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[34])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1238};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_track_times, __pyx_mstate->__pyx_n_u_plist_id};
    __pyx_mstate_global->__pyx_codeobj_tab[35] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_get_obj_track_times, __pyx_mstate->__pyx_kp_b_iso88591_A_t_r_Ja_r_Q_4q_y_L_0_A_q_l_102, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[35])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1271};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_where, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_init};
    __pyx_mstate_global->__pyx_codeobj_tab[36] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_new, __pyx_mstate->__pyx_kp_b_iso88591_q_a_1_1_7_7, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[36])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 7, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1280};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_coords, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_offset, __pyx_mstate->__pyx_n_u_filter_mask, __pyx_mstate->__pyx_n_u_addr, __pyx_mstate->__pyx_n_u_size};
    __pyx_mstate_global->__pyx_codeobj_tab[37] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_chunk_info, __pyx_mstate->__pyx_kp_b_iso88591_aq_at_QfAQ_t2Q_L_4q_A_c_a, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[37])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 11, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1304};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_coords, __pyx_mstate->__pyx_n_u_out, __pyx_mstate->__pyx_n_u_rarr, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_offset, __pyx_mstate->__pyx_n_u_filters, __pyx_mstate->__pyx_n_u_rbuf, __pyx_mstate->__pyx_n_u__7, __pyx_mstate->__pyx_n_u_addr, __pyx_mstate->__pyx_n_u_size};
    __pyx_mstate_global->__pyx_codeobj_tab[38] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_read_chunk, __pyx_mstate->__pyx_kp_b_iso88591_A_vWD_aq_uCq_Q_t7_t3auBa_Ja_5Qc, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[38])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 8, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1334};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_coords, __pyx_mstate->__pyx_n_u_data, __pyx_mstate->__pyx_n_u_filters, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_offset, __pyx_mstate->__pyx_n_u_data_size, __pyx_mstate->__pyx_n_u_wbuf};
    __pyx_mstate_global->__pyx_codeobj_tab[39] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_write_chunk, __pyx_mstate->__pyx_kp_b_iso88591_A_aq_N_4_M_a_t2Q_L_4q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[39])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1399};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_size, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_classname, __pyx_mstate->__pyx_n_u_shape};
    __pyx_mstate_global->__pyx_codeobj_tab[40] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_truncate, __pyx_mstate->__pyx_kp_b_iso88591_q_M_Zq_t2Q_L_Ba_Ja_z_Zq_q_L_d_4, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[40])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1421};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[41] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_flush, __pyx_mstate->__pyx_kp_b_iso88591_t_s, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[41])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1426};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[42] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_close, __pyx_mstate->__pyx_kp_b_iso88591_t9Cq_at1_t_A_at1_t_A_at1_t_s_at, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[42])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[43] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[43])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[44] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[44])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1452};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_byteorder, __pyx_mstate->__pyx_n_u_filter_list, __pyx_mstate->__pyx_n_u_file_mode, __pyx_mstate->__pyx_n_u_opt_write, __pyx_mstate->__pyx_n_u_opt_read};
    __pyx_mstate_global->__pyx_codeobj_tab[45] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_supports_opt_blosc2_read_write, __pyx_mstate->__pyx_kp_b_iso88591_s_1_Jc_A_AS_3k_hgRvS_a_1_Ja, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[45])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 17, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1469};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_nparr, __pyx_mstate->__pyx_n_u_title, __pyx_mstate->__pyx_n_u_atom, __pyx_mstate->__pyx_n_u_i, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_rbuf, __pyx_mstate->__pyx_n_u_complib, __pyx_mstate->__pyx_n_u_version, __pyx_mstate->__pyx_n_u_class_2, __pyx_mstate->__pyx_n_u_dtype_2, __pyx_mstate->__pyx_n_u_atom_2, __pyx_mstate->__pyx_n_u_shape, __pyx_mstate->__pyx_n_u_dims, __pyx_mstate->__pyx_n_u_encoded_title, __pyx_mstate->__pyx_n_u_encoded_name, __pyx_mstate->__pyx_n_u_cset};
    __pyx_mstate_global->__pyx_codeobj_tab[46] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_create_array, __pyx_mstate->__pyx_kp_b_iso88591_1_E_4uG1A_XV1_uCuCt7_Q_uF_d_Qa, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[46])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 15, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1538};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_title, __pyx_mstate->__pyx_n_u_i, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_rbuf, __pyx_mstate->__pyx_n_u_complib, __pyx_mstate->__pyx_n_u_version, __pyx_mstate->__pyx_n_u_class_2, __pyx_mstate->__pyx_n_u_dflts, __pyx_mstate->__pyx_n_u_fill_data, __pyx_mstate->__pyx_n_u_extdim, __pyx_mstate->__pyx_n_u_atom, __pyx_mstate->__pyx_n_u_encoded_title, __pyx_mstate->__pyx_n_u_encoded_name, __pyx_mstate->__pyx_n_u_blocksize};
    __pyx_mstate_global->__pyx_codeobj_tab[47] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_create_carray, __pyx_mstate->__pyx_kp_b_iso88591_E_4uG1A_4q_A_1D_1D_t1_1D_t1_q_L, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[47])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 17, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1619};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_type_size, __pyx_mstate->__pyx_n_u_type_precision, __pyx_mstate->__pyx_n_u_class_id, __pyx_mstate->__pyx_n_u_cbyteorder, __pyx_mstate->__pyx_n_u_i, __pyx_mstate->__pyx_n_u_extdim, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_shape, __pyx_mstate->__pyx_n_u_chunkshapes, __pyx_mstate->__pyx_n_u_atom, __pyx_mstate->__pyx_n_u_fill_status, __pyx_mstate->__pyx_n_u_dflts, __pyx_mstate->__pyx_n_u_fill_data, __pyx_mstate->__pyx_n_u_encoded_name, __pyx_mstate->__pyx_n_u_byteorder, __pyx_mstate->__pyx_n_u_filters};
    __pyx_mstate_global->__pyx_codeobj_tab[48] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_open_array, __pyx_mstate->__pyx_kp_b_iso88591_4uG1A_gQd_nA_t_r_L_gT_1_t_d_at1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[48])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 7, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1710};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_nparr, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_extdim, __pyx_mstate->__pyx_n_u_dims_arr, __pyx_mstate->__pyx_n_u_rbuf, __pyx_mstate->__pyx_n_u_shape};
    __pyx_mstate_global->__pyx_codeobj_tab[49] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_append_2, __pyx_mstate->__pyx_kp_b_iso88591_t5_c_Jaq_at7_aq_q_t5_c_7_T_1D_T, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[49])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {5, 0, 0, 11, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1742};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_start, __pyx_mstate->__pyx_n_u_stop, __pyx_mstate->__pyx_n_u_step, __pyx_mstate->__pyx_n_u_nparr, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_rbuf, __pyx_mstate->__pyx_n_u_nrows, __pyx_mstate->__pyx_n_u_extdim, __pyx_mstate->__pyx_n_u_item_size, __pyx_mstate->__pyx_n_u_refbuf};
    __pyx_mstate_global->__pyx_codeobj_tab[50] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_read_array, __pyx_mstate->__pyx_kp_b_iso88591_Kq_A_AWF_t5_c_vQfBa_Q_wavQ_t1_k, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[50])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {5, 0, 0, 14, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1795};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_startl, __pyx_mstate->__pyx_n_u_stopl, __pyx_mstate->__pyx_n_u_stepl, __pyx_mstate->__pyx_n_u_nparr, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_start, __pyx_mstate->__pyx_n_u_stop, __pyx_mstate->__pyx_n_u_step, __pyx_mstate->__pyx_n_u_rbuf, __pyx_mstate->__pyx_n_u_item_size, __pyx_mstate->__pyx_n_u_refbuf, __pyx_mstate->__pyx_n_u_fname, __pyx_mstate->__pyx_n_u_filename};
    __pyx_mstate_global->__pyx_codeobj_tab[51] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_read_slice, __pyx_mstate->__pyx_kp_b_iso88591_Kq_A_K_1A_l_1_l_1_t5_c_vQe6_1_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[51])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 11, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1848};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_coords, __pyx_mstate->__pyx_n_u_nparr, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_space_id, __pyx_mstate->__pyx_n_u_mem_space_id, __pyx_mstate->__pyx_n_u_size, __pyx_mstate->__pyx_n_u_rbuf, __pyx_mstate->__pyx_n_u_mode, __pyx_mstate->__pyx_n_u_item_size, __pyx_mstate->__pyx_n_u_refbuf};
    __pyx_mstate_global->__pyx_codeobj_tab[52] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_read_coords, __pyx_mstate->__pyx_kp_b_iso88591_Kq_A_1D_5_1Cq_a_az_v_AQ_t5_c_vQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[52])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {7, 0, 0, 20, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1911};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_space_id, __pyx_mstate->__pyx_n_u_start, __pyx_mstate->__pyx_n_u_count, __pyx_mstate->__pyx_n_u_step, __pyx_mstate->__pyx_n_u_idx, __pyx_mstate->__pyx_n_u_mode, __pyx_mstate->__pyx_n_u_select_mode, __pyx_mstate->__pyx_n_u_start_2, __pyx_mstate->__pyx_n_u_count_2, __pyx_mstate->__pyx_n_u_step_2, __pyx_mstate->__pyx_n_u_startp, __pyx_mstate->__pyx_n_u_countp, __pyx_mstate->__pyx_n_u_stepp, __pyx_mstate->__pyx_n_u_startl, __pyx_mstate->__pyx_n_u_countl, __pyx_mstate->__pyx_n_u_stepl, __pyx_mstate->__pyx_n_u_i, __pyx_mstate->__pyx_n_u_x, __pyx_mstate->__pyx_n_u_select_modes};
    __pyx_mstate_global->__pyx_codeobj_tab[53] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_perform_selection, __pyx_mstate->__pyx_kp_b_iso88591_HHD_A_5_a_3a_gQa_gQa_WAQ_gQa_gQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[53])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 12, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1954};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_selection, __pyx_mstate->__pyx_n_u_nparr, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_space_id, __pyx_mstate->__pyx_n_u_mem_space_id, __pyx_mstate->__pyx_n_u_size, __pyx_mstate->__pyx_n_u_rbuf, __pyx_mstate->__pyx_n_u_mode, __pyx_mstate->__pyx_n_u_item_size, __pyx_mstate->__pyx_n_u_refbuf, __pyx_mstate->__pyx_n_u_args};
    __pyx_mstate_global->__pyx_codeobj_tab[54] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_read_selection, __pyx_mstate->__pyx_kp_b_iso88591_Kq_A_1D_5_1Cq_a_A_t5_c_vQe6_1_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[54])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {5, 0, 0, 11, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 2020};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_startl, __pyx_mstate->__pyx_n_u_stepl, __pyx_mstate->__pyx_n_u_countl, __pyx_mstate->__pyx_n_u_nparr, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_rbuf, __pyx_mstate->__pyx_n_u_temp, __pyx_mstate->__pyx_n_u_start, __pyx_mstate->__pyx_n_u_step, __pyx_mstate->__pyx_n_u_count};
    __pyx_mstate_global->__pyx_codeobj_tab[55] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_write_slice, __pyx_mstate->__pyx_kp_b_iso88591_t5_c_Jaq_q_K_1A_l_1_K_1A_t5_c_7, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[55])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 9, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 2056};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_coords, __pyx_mstate->__pyx_n_u_nparr, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_space_id, __pyx_mstate->__pyx_n_u_mem_space_id, __pyx_mstate->__pyx_n_u_size, __pyx_mstate->__pyx_n_u_rbuf, __pyx_mstate->__pyx_n_u_mode};
    __pyx_mstate_global->__pyx_codeobj_tab[56] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_write_coords, __pyx_mstate->__pyx_kp_b_iso88591_t5_c_Jaq_1D_5_1Cq_a_az_v_AQ_q_t, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[56])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 10, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 2101};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_selection, __pyx_mstate->__pyx_n_u_nparr, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_space_id, __pyx_mstate->__pyx_n_u_mem_space_id, __pyx_mstate->__pyx_n_u_size, __pyx_mstate->__pyx_n_u_rbuf, __pyx_mstate->__pyx_n_u_mode, __pyx_mstate->__pyx_n_u_args};
    __pyx_mstate_global->__pyx_codeobj_tab[57] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_write_selection, __pyx_mstate->__pyx_kp_b_iso88591_t5_c_Jaq_1D_5_1Cq_a_A_q_t5_c_7, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[57])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[58] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[58])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[59] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[59])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 16, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 2162};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_title, __pyx_mstate->__pyx_n_u_rank, __pyx_mstate->__pyx_n_u_dims, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_rbuf, __pyx_mstate->__pyx_n_u_complib, __pyx_mstate->__pyx_n_u_version, __pyx_mstate->__pyx_n_u_class_2, __pyx_mstate->__pyx_n_u_type_2, __pyx_mstate->__pyx_n_u_itemsize, __pyx_mstate->__pyx_n_u_atom, __pyx_mstate->__pyx_n_u_scatom, __pyx_mstate->__pyx_n_u_encoded_title, __pyx_mstate->__pyx_n_u_encoded_name, __pyx_mstate->__pyx_n_u_cset};
    __pyx_mstate_global->__pyx_codeobj_tab[60] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_create_array, __pyx_mstate->__pyx_kp_b_iso88591_1_E_4uG1A_4q_t7_6_T_T_avQ_a_t_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[60])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 13, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 2228};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_cbyteorder, __pyx_mstate->__pyx_n_u_i, __pyx_mstate->__pyx_n_u_enumtype, __pyx_mstate->__pyx_n_u_rank, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_nrecords, __pyx_mstate->__pyx_n_u_chunksize, __pyx_mstate->__pyx_n_u_shape, __pyx_mstate->__pyx_n_u_type_2, __pyx_mstate->__pyx_n_u_encoded_name, __pyx_mstate->__pyx_n_u_byteorder, __pyx_mstate->__pyx_n_u_atom};
    __pyx_mstate_global->__pyx_codeobj_tab[61] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_open_array, __pyx_mstate->__pyx_kp_b_iso88591_4uG1A_gQd_nA_t_r_L_gT_1_t_d_at1_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[61])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 2272};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_nparr, __pyx_mstate->__pyx_n_u_nobjects, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_rbuf};
    __pyx_mstate_global->__pyx_codeobj_tab[62] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_append_2, __pyx_mstate->__pyx_kp_b_iso88591_q_U_1_AWA_Q_Qd_t1_0_Kq_t2Q_L_D, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[62])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 6, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 2295};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_nrow, __pyx_mstate->__pyx_n_u_nparr, __pyx_mstate->__pyx_n_u_nobjects, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_rbuf};
    __pyx_mstate_global->__pyx_codeobj_tab[63] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_modify, __pyx_mstate->__pyx_kp_b_iso88591_q_q_U_1_AWA_Qd_t1_Ja_t2Q_L_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[63])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 2318};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_space_id, __pyx_mstate->__pyx_n_u_size, __pyx_mstate->__pyx_n_u_ret};
    __pyx_mstate_global->__pyx_codeobj_tab[64] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_get_memory_size, __pyx_mstate->__pyx_kp_b_iso88591_t7_Q_Q_Qd_j_Rq_aq_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[64])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 15, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 2339};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_start, __pyx_mstate->__pyx_n_u_stop, __pyx_mstate->__pyx_n_u_step, __pyx_mstate->__pyx_n_u_i, __pyx_mstate->__pyx_n_u_vllen, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_rdata, __pyx_mstate->__pyx_n_u_nrows, __pyx_mstate->__pyx_n_u_space_id, __pyx_mstate->__pyx_n_u_mem_space_id, __pyx_mstate->__pyx_n_u_buf, __pyx_mstate->__pyx_n_u_nparr, __pyx_mstate->__pyx_n_u_shape, __pyx_mstate->__pyx_n_u_datalist};
    __pyx_mstate_global->__pyx_codeobj_tab[65] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_read_array, __pyx_mstate->__pyx_kp_b_iso88591_AWF_vRvRt1_L_Q_q_Qa_q_A_q_1G1_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[65])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 7, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 2421};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_row, __pyx_mstate->__pyx_n_u_space_id, __pyx_mstate->__pyx_n_u_size, __pyx_mstate->__pyx_n_u_ret, __pyx_mstate->__pyx_n_u_offset, __pyx_mstate->__pyx_n_u_count};
    __pyx_mstate_global->__pyx_codeobj_tab[66] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_get_row_size, __pyx_mstate->__pyx_kp_b_iso88591_t3d_L_Q_1D_5_q_Qj_8_gQ_t2Q_a_at, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[66])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[67] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[67])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[68] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[68])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 2458};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_shape, __pyx_mstate->__pyx_n_u_cbyteorder, __pyx_mstate->__pyx_n_u_encoded_name, __pyx_mstate->__pyx_n_u_byteorder};
    __pyx_mstate_global->__pyx_codeobj_tab[69] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_open_unimplemented, __pyx_mstate->__pyx_kp_b_iso88591_4uG1A_L_q_E_Qj_gQd_nA_Qa_G_d, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[69])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 2474};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[70] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tables_hdf5extension_pyx, __pyx_mstate->__pyx_n_u_g_close, __pyx_mstate->__pyx_kp_b_iso88591_AT, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[70])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[71] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[71])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[72] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[72])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 4};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_pyx_type, __pyx_mstate->__pyx_n_u_pyx_checksum, __pyx_mstate->__pyx_n_u_pyx_state, __pyx_mstate->__pyx_n_u_pyx_result};
    __pyx_mstate_global->__pyx_codeobj_tab[73] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_pyx_unpickle_File, __pyx_mstate->__pyx_kp_b_iso88591_q_0_kQR_4xq_7_awnA_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[73])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 4};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_pyx_type, __pyx_mstate->__pyx_n_u_pyx_checksum, __pyx_mstate->__pyx_n_u_pyx_state, __pyx_mstate->__pyx_n_u_pyx_result};
    __pyx_mstate_global->__pyx_codeobj_tab[74] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_pyx_unpickle_AttributeSet, __pyx_mstate->__pyx_kp_b_iso88591_q_0_kQR_xq_7_a_nA_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[74])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 4};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_pyx_type, __pyx_mstate->__pyx_n_u_pyx_checksum, __pyx_mstate->__pyx_n_u_pyx_state, __pyx_mstate->__pyx_n_u_pyx_result};
    __pyx_mstate_global->__pyx_codeobj_tab[75] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_pyx_unpickle_Node, __pyx_mstate->__pyx_kp_b_iso88591_q_0_kQR_4xq_7_awnA_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[75])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 4};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_pyx_type, __pyx_mstate->__pyx_n_u_pyx_checksum, __pyx_mstate->__pyx_n_u_pyx_state, __pyx_mstate->__pyx_n_u_pyx_result};
    __pyx_mstate_global->__pyx_codeobj_tab[76] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_pyx_unpickle_Group, __pyx_mstate->__pyx_kp_b_iso88591_q_0_kQR_5_7_q_a_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[76])) goto bad;
  }
  Py_DECREF(tuple_dedup_map);
  return 0;
  bad:
  Py_DECREF(tuple_dedup_map);
  return -1;
}
/* #### Code section: init_globals ### */

static int __Pyx_InitGlobals(void) {
  /* PythonCompatibility.init */
  if (likely(__Pyx_init_co_variables() == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* AssertionsEnabled.init */
  if (likely(__Pyx_init_assertions_enabled() == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CommonTypesMetaclass.init */
  if (likely(__pyx_CommonTypesMetaclass_init(__pyx_m) == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CachedMethodType.init */
  #if CYTHON_COMPILING_IN_LIMITED_API
  {
      PyObject *typesModule=NULL;
      typesModule = PyImport_ImportModule("types");
      if (typesModule) {
          __pyx_mstate_global->__Pyx_CachedMethodType = PyObject_GetAttrString(typesModule, "MethodType");
          Py_DECREF(typesModule);
      }
  } // error handling follows
  #endif
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CythonFunctionShared.init */
  if (likely(__pyx_CyFunction_init(__pyx_m) == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cleanup_globals ### */
/* #### Code section: cleanup_module ### */
/* #### Code section: main_method ### */
/* #### Code section: utility_code_pragmas ### */
#ifdef _MSC_VER
#pragma warning( push )
/* Warning 4127: conditional expression is constant
 * Cython uses constant conditional expressions to allow in inline functions to be optimized at
 * compile-time, so this warning is not useful
 */
#pragma warning( disable : 4127 )
#endif



/* #### Code section: utility_code_def ### */

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyErrExceptionMatches (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    int result;
    PyObject *exc_type;
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *current_exception = tstate->current_exception;
    if (unlikely(!current_exception)) return 0;
    exc_type = (PyObject*) Py_TYPE(current_exception);
    if (exc_type == err) return 1;
#else
    exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
#endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(exc_type);
    #endif
    if (unlikely(PyTuple_Check(err))) {
        result = __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    } else {
        result = __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
    }
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(exc_type);
    #endif
    return result;
}
#endif

/* PyErrFetchRestore (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *tmp_value;
    assert(type == NULL || (value != NULL && type == (PyObject*) Py_TYPE(value)));
    if (value) {
        #if CYTHON_COMPILING_IN_CPYTHON
        if (unlikely(((PyBaseExceptionObject*) value)->traceback != tb))
        #endif
            PyException_SetTraceback(value, tb);
    }
    tmp_value = tstate->current_exception;
    tstate->current_exception = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
#else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject* exc_value;
    exc_value = tstate->current_exception;
    tstate->current_exception = 0;
    *value = exc_value;
    *type = NULL;
    *tb = NULL;
    if (exc_value) {
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        #if CYTHON_COMPILING_IN_CPYTHON
        *tb = ((PyBaseExceptionObject*) exc_value)->traceback;
        Py_XINCREF(*tb);
        #else
        *tb = PyException_GetTraceback(exc_value);
        #endif
    }
#else
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#endif
}
#endif

/* PyObjectGetAttrStr (used by PyObjectGetAttrStrNoError) */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError (used by GetBuiltinName) */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    (void) PyObject_GetOptionalAttr(obj, attr_name, &result);
    return result;
#else
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
#endif
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStrNoError(__pyx_mstate_global->__pyx_b, name);
    if (unlikely(!result) && !PyErr_Occurred()) {
        PyErr_Format(PyExc_NameError,
            "name '%U' is not defined", name);
    }
    return result;
}

/* GetTopmostException (used by SaveResetException) */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_value == NULL || exc_info->exc_value == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    PyObject *exc_value = exc_info->exc_value;
    if (exc_value == NULL || exc_value == Py_None) {
        *value = NULL;
        *type = NULL;
        *tb = NULL;
    } else {
        *value = exc_value;
        Py_INCREF(*value);
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        *tb = PyException_GetTraceback(exc_value);
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #endif
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    PyObject *tmp_value = exc_info->exc_value;
    exc_info->exc_value = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
  #else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
  #endif
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type = NULL, *local_value, *local_tb = NULL;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if PY_VERSION_HEX >= 0x030C0000
    local_value = tstate->current_exception;
    tstate->current_exception = 0;
  #else
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
  #endif
#elif __PYX_LIMITED_VERSION_HEX > 0x030C0000
    local_value = PyErr_GetRaisedException();
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
#if __PYX_LIMITED_VERSION_HEX > 0x030C0000
    if (likely(local_value)) {
        local_type = (PyObject*) Py_TYPE(local_value);
        Py_INCREF(local_type);
        local_tb = PyException_GetTraceback(local_value);
    }
#else
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
#endif // __PYX_LIMITED_VERSION_HEX > 0x030C0000
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
      #if PY_VERSION_HEX >= 0x030B00a4
        tmp_value = exc_info->exc_value;
        exc_info->exc_value = local_value;
        tmp_type = NULL;
        tmp_tb = NULL;
        Py_XDECREF(local_type);
        Py_XDECREF(local_tb);
      #else
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
      #endif
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#elif __PYX_LIMITED_VERSION_HEX >= 0x030b0000
    PyErr_SetHandledException(local_value);
    Py_XDECREF(local_value);
    Py_XDECREF(local_type);
    Py_XDECREF(local_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
#if __PYX_LIMITED_VERSION_HEX <= 0x030C0000
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
#endif
}

/* PyObjectCall (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = __Pyx_CyOrPyCFunction_GET_FUNCTION(func);
    self = __Pyx_CyOrPyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectFastCall */
#if PY_VERSION_HEX < 0x03090000 || CYTHON_COMPILING_IN_LIMITED_API
static PyObject* __Pyx_PyObject_FastCall_fallback(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs) {
    PyObject *argstuple;
    PyObject *result = 0;
    size_t i;
    argstuple = PyTuple_New((Py_ssize_t)nargs);
    if (unlikely(!argstuple)) return NULL;
    for (i = 0; i < nargs; i++) {
        Py_INCREF(args[i]);
        if (__Pyx_PyTuple_SET_ITEM(argstuple, (Py_ssize_t)i, args[i]) != (0)) goto bad;
    }
    result = __Pyx_PyObject_Call(func, argstuple, kwargs);
  bad:
    Py_DECREF(argstuple);
    return result;
}
#endif
#if CYTHON_VECTORCALL && !CYTHON_COMPILING_IN_LIMITED_API
  #if PY_VERSION_HEX < 0x03090000
    #define __Pyx_PyVectorcall_Function(callable) _PyVectorcall_Function(callable)
  #elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE vectorcallfunc __Pyx_PyVectorcall_Function(PyObject *callable) {
    PyTypeObject *tp = Py_TYPE(callable);
    #if defined(__Pyx_CyFunction_USED)
    if (__Pyx_CyFunction_CheckExact(callable)) {
        return __Pyx_CyFunction_func_vectorcall(callable);
    }
    #endif
    if (!PyType_HasFeature(tp, Py_TPFLAGS_HAVE_VECTORCALL)) {
        return NULL;
    }
    assert(PyCallable_Check(callable));
    Py_ssize_t offset = tp->tp_vectorcall_offset;
    assert(offset > 0);
    vectorcallfunc ptr;
    memcpy(&ptr, (char *) callable + offset, sizeof(ptr));
    return ptr;
}
  #else
    #define __Pyx_PyVectorcall_Function(callable) PyVectorcall_Function(callable)
  #endif
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject *const *args, size_t _nargs, PyObject *kwargs) {
    Py_ssize_t nargs = __Pyx_PyVectorcall_NARGS(_nargs);
#if CYTHON_COMPILING_IN_CPYTHON
    if (nargs == 0 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_NOARGS))
            return __Pyx_PyObject_CallMethO(func, NULL);
    }
    else if (nargs == 1 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_O))
            return __Pyx_PyObject_CallMethO(func, args[0]);
    }
#endif
    if (kwargs == NULL) {
        #if CYTHON_VECTORCALL
          #if CYTHON_COMPILING_IN_LIMITED_API
            return PyObject_Vectorcall(func, args, _nargs, NULL);
          #else
            vectorcallfunc f = __Pyx_PyVectorcall_Function(func);
            if (f) {
                return f(func, args, _nargs, NULL);
            }
          #endif
        #endif
    }
    if (nargs == 0) {
        return __Pyx_PyObject_Call(func, __pyx_mstate_global->__pyx_empty_tuple, kwargs);
    }
    #if PY_VERSION_HEX >= 0x03090000 && !CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_VectorcallDict(func, args, (size_t)nargs, kwargs);
    #else
    return __Pyx_PyObject_FastCall_fallback(func, args, (size_t)nargs, kwargs);
    #endif
}

/* RaiseException */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if PY_VERSION_HEX >= 0x030C00A6
        PyException_SetTraceback(value, tb);
#elif CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}

/* PyObjectCall2Args (used by PyObjectCallMethod1) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args[3] = {NULL, arg1, arg2};
    return __Pyx_PyObject_FastCall(function, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectCallOneArg (used by PyObjectCallMethod1) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *args[2] = {NULL, arg};
    return __Pyx_PyObject_FastCall(func, args+1, 1 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectGetMethod (used by PyObjectCallMethod1) */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    __Pyx_TypeName type_name;
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if defined(Py_TPFLAGS_METHOD_DESCRIPTOR) && Py_TPFLAGS_METHOD_DESCRIPTOR
        if (__Pyx_PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_METHOD_DESCRIPTOR))
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (likely(descr != NULL)) {
        *method = descr;
        return 0;
    }
    type_name = __Pyx_PyType_GetFullyQualifiedName(tp);
    PyErr_Format(PyExc_AttributeError,
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}
#endif

/* PyObjectCallMethod1 (used by append) */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
#endif
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
#if CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000))
    PyObject *args[2] = {obj, arg};
    (void) __Pyx_PyObject_CallOneArg;
    (void) __Pyx_PyObject_Call2Args;
    return PyObject_VectorcallMethod(method_name, args, 2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#else
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
#endif
}

/* append */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x) {
    if (likely(PyList_CheckExact(L))) {
        if (unlikely(__Pyx_PyList_Append(L, x) < 0)) return -1;
    } else {
        PyObject* retval = __Pyx_PyObject_CallMethod1(L, __pyx_mstate_global->__pyx_n_u_append, x);
        if (unlikely(!retval))
            return -1;
        Py_DECREF(retval);
    }
    return 0;
}

/* PyDictVersioning (used by GetModuleGlobalName) */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!__pyx_m)) {
        if (!PyErr_Occurred())
            PyErr_SetNone(PyExc_NameError);
        return NULL;
    }
    result = PyObject_GetAttr(__pyx_m, name);
    if (likely(result)) {
        return result;
    }
    PyErr_Clear();
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    if (unlikely(__Pyx_PyDict_GetItemRef(__pyx_mstate_global->__pyx_d, name, &result) == -1)) PyErr_Clear();
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return result;
    }
#else
    result = _PyDict_GetItem_KnownHash(__pyx_mstate_global->__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyObjectFastCallMethod */
#if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf) {
    PyObject *result;
    PyObject *attr = PyObject_GetAttr(args[0], name);
    if (unlikely(!attr))
        return NULL;
    result = __Pyx_PyObject_FastCall(attr, args+1, nargsf - 1);
    Py_DECREF(attr);
    return result;
}
#endif

/* RaiseUnboundLocalError */
static void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

/* TupleAndListFromArray (used by fastcall) */
#if !CYTHON_COMPILING_IN_CPYTHON && CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    Py_ssize_t i;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    for (i = 0; i < n; i++) {
        Py_INCREF(src[i]);
        if (unlikely(__Pyx_PyTuple_SET_ITEM(res, i, src[i]) < (0))) {
            Py_DECREF(res);
            return NULL;
        }
    }
    return res;
}
#elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject *const *CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyTupleObject*)res)->ob_item, n);
    return res;
}
static CYTHON_INLINE PyObject *
__Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return PyList_New(0);
    }
    res = PyList_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyListObject*)res)->ob_item, n);
    return res;
}
#endif

/* BytesEquals (used by UnicodeEquals) */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL ||\
        !(CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS)
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals (used by fastcall) */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length, length2;
        int kind;
        void *data1, *data2;
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        #endif
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length < 0)) return -1;
        #endif
        length2 = __Pyx_PyUnicode_GET_LENGTH(s2);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length2 < 0)) return -1;
        #endif
        if (length != length2) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    return (equals == Py_EQ);
return_ne:
    return (equals == Py_NE);
#endif
}

/* fastcall */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s)
{
    Py_ssize_t i, n = __Pyx_PyTuple_GET_SIZE(kwnames);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(n == -1)) return NULL;
    #endif
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        if (s == namei) return kwvalues[i];
    }
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        int eq = __Pyx_PyUnicode_Equals(s, namei, Py_EQ);
        if (unlikely(eq != 0)) {
            if (unlikely(eq < 0)) return NULL;
            return kwvalues[i];
        }
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues) {
    Py_ssize_t i, nkwargs;
    PyObject *dict;
#if !CYTHON_ASSUME_SAFE_SIZE
    nkwargs = PyTuple_Size(kwnames);
    if (unlikely(nkwargs < 0)) return NULL;
#else
    nkwargs = PyTuple_GET_SIZE(kwnames);
#endif
    dict = PyDict_New();
    if (unlikely(!dict))
        return NULL;
    for (i=0; i<nkwargs; i++) {
#if !CYTHON_ASSUME_SAFE_MACROS
        PyObject *key = PyTuple_GetItem(kwnames, i);
        if (!key) goto bad;
#else
        PyObject *key = PyTuple_GET_ITEM(kwnames, i);
#endif
        if (unlikely(PyDict_SetItem(dict, key, kwvalues[i]) < 0))
            goto bad;
    }
    return dict;
bad:
    Py_DECREF(dict);
    return NULL;
}
#endif
#endif

/* UnpackUnboundCMethod (used by CallUnboundCMethod0) */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *args, PyObject *kwargs) {
    PyObject *result;
    PyObject *selfless_args = PyTuple_GetSlice(args, 1, PyTuple_Size(args));
    if (unlikely(!selfless_args)) return NULL;
    result = PyObject_Call(method, selfless_args, kwargs);
    Py_DECREF(selfless_args);
    return result;
}
#elif CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject **args, Py_ssize_t nargs, PyObject *kwnames) {
        return _PyObject_Vectorcall
            (method, args ? args+1 : NULL, nargs ? nargs-1 : 0, kwnames);
}
#else
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames) {
    return
#if PY_VERSION_HEX < 0x03090000
    _PyObject_Vectorcall
#else
    PyObject_Vectorcall
#endif
        (method, args ? args+1 : NULL, nargs ? (size_t) nargs-1 : 0, kwnames);
}
#endif
static PyMethodDef __Pyx_UnboundCMethod_Def = {
     "CythonUnboundCMethod",
     __PYX_REINTERPRET_FUNCION(PyCFunction, __Pyx_SelflessCall),
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
     METH_VARARGS | METH_KEYWORDS,
#else
     METH_FASTCALL | METH_KEYWORDS,
#endif
     NULL
};
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method, *result=NULL;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    result = method;
#if CYTHON_COMPILING_IN_CPYTHON
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    } else
#endif
#if CYTHON_COMPILING_IN_PYPY
#else
    if (PyCFunction_Check(method))
#endif
    {
        PyObject *self;
        int self_found;
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        self = PyObject_GetAttrString(method, "__self__");
        if (!self) {
            PyErr_Clear();
        }
#else
        self = PyCFunction_GET_SELF(method);
#endif
        self_found = (self && self != Py_None);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        Py_XDECREF(self);
#endif
        if (self_found) {
            PyObject *unbound_method = PyCFunction_New(&__Pyx_UnboundCMethod_Def, method);
            if (unlikely(!unbound_method)) return -1;
            Py_DECREF(method);
            result = unbound_method;
        }
    }
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    if (unlikely(target->method)) {
        Py_DECREF(result);
    } else
#endif
    target->method = result;
    return 0;
}

/* CallUnboundCMethod0 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        if (likely(cfunc->flag == METH_NOARGS))
            return __Pyx_CallCFunction(cfunc, self, NULL);
        if (likely(cfunc->flag == METH_FASTCALL))
            return __Pyx_CallCFunctionFast(cfunc, self, NULL, 0);
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, NULL, 0, NULL);
        if (likely(cfunc->flag == (METH_VARARGS | METH_KEYWORDS)))
            return __Pyx_CallCFunctionWithKeywords(cfunc, self, __pyx_mstate_global->__pyx_empty_tuple, NULL);
        if (cfunc->flag == METH_VARARGS)
            return __Pyx_CallCFunction(cfunc, self, __pyx_mstate_global->__pyx_empty_tuple);
        return __Pyx__CallUnboundCMethod0(cfunc, self);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod0(&tmp_cfunc, self);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod0(cfunc, self);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *result;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
    result = __Pyx_PyObject_CallOneArg(cfunc->method, self);
    return result;
}

/* py_dict_items (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d) {
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_items, d);
}

/* py_dict_values (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d) {
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_values, d);
}

/* OwnedDictNext (used by ParseKeywordsImpl) */
#if CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, PyObject **ppos, PyObject **pkey, PyObject **pvalue) {
    PyObject *next = NULL;
    if (!*ppos) {
        if (pvalue) {
            PyObject *dictview = pkey ? __Pyx_PyDict_Items(p) : __Pyx_PyDict_Values(p);
            if (unlikely(!dictview)) goto bad;
            *ppos = PyObject_GetIter(dictview);
            Py_DECREF(dictview);
        } else {
            *ppos = PyObject_GetIter(p);
        }
        if (unlikely(!*ppos)) goto bad;
    }
    next = PyIter_Next(*ppos);
    if (!next) {
        if (PyErr_Occurred()) goto bad;
        return 0;
    }
    if (pkey && pvalue) {
        *pkey = __Pyx_PySequence_ITEM(next, 0);
        if (unlikely(*pkey)) goto bad;
        *pvalue = __Pyx_PySequence_ITEM(next, 1);
        if (unlikely(*pvalue)) goto bad;
        Py_DECREF(next);
    } else if (pkey) {
        *pkey = next;
    } else {
        assert(pvalue);
        *pvalue = next;
    }
    return 1;
  bad:
    Py_XDECREF(next);
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d0000
    PyErr_FormatUnraisable("Exception ignored in __Pyx_PyDict_NextRef");
#else
    PyErr_WriteUnraisable(__pyx_mstate_global->__pyx_n_u_Pyx_PyDict_NextRef);
#endif
    if (pkey) *pkey = NULL;
    if (pvalue) *pvalue = NULL;
    return 0;
}
#else // !CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, Py_ssize_t *ppos, PyObject **pkey, PyObject **pvalue) {
    int result = PyDict_Next(p, ppos, pkey, pvalue);
    if (likely(result == 1)) {
        if (pkey) Py_INCREF(*pkey);
        if (pvalue) Py_INCREF(*pvalue);
    }
    return result;
}
#endif

/* RaiseDoubleKeywords (used by ParseKeywordsImpl) */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
}

/* CallUnboundCMethod2 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            return __Pyx_CallCFunctionFast(cfunc, self, args, 2);
        }
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, 2, NULL);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod2(&tmp_cfunc, self, arg1, arg2);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        PyObject *result = NULL;
        PyObject *args = PyTuple_New(2);
        if (unlikely(!args)) return NULL;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = __Pyx_CallCFunctionWithKeywords(cfunc, self, args, NULL);
        else
            result = __Pyx_CallCFunction(cfunc, self, args);
        Py_DECREF(args);
        return result;
    }
#endif
    {
        PyObject *args[4] = {NULL, self, arg1, arg2};
        return __Pyx_PyObject_FastCall(cfunc->method, args+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
    }
}

/* ParseKeywordsImpl (used by ParseKeywords) */
static int __Pyx_ValidateDuplicatePosArgs(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char* function_name)
{
    PyObject ** const *name = argnames;
    while (name != first_kw_arg) {
        PyObject *key = **name;
        int found = PyDict_Contains(kwds, key);
        if (unlikely(found)) {
            if (found == 1) __Pyx_RaiseDoubleKeywordsError(function_name, key);
            goto bad;
        }
        name++;
    }
    return 0;
bad:
    return -1;
}
#if CYTHON_USE_UNICODE_INTERNALS
static CYTHON_INLINE int __Pyx_UnicodeKeywordsEqual(PyObject *s1, PyObject *s2) {
    int kind;
    Py_ssize_t len = PyUnicode_GET_LENGTH(s1);
    if (len != PyUnicode_GET_LENGTH(s2)) return 0;
    kind = PyUnicode_KIND(s1);
    if (kind != PyUnicode_KIND(s2)) return 0;
    const void *data1 = PyUnicode_DATA(s1);
    const void *data2 = PyUnicode_DATA(s2);
    return (memcmp(data1, data2, (size_t) len * (size_t) kind) == 0);
}
#endif
static int __Pyx_MatchKeywordArg_str(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    #if CYTHON_USE_UNICODE_INTERNALS
    Py_hash_t key_hash = ((PyASCIIObject*)key)->hash;
    if (unlikely(key_hash == -1)) {
        key_hash = PyObject_Hash(key);
        if (unlikely(key_hash == -1))
            goto bad;
    }
    #endif
    name = first_kw_arg;
    while (*name) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (key_hash == ((PyASCIIObject*)name_str)->hash && __Pyx_UnicodeKeywordsEqual(name_str, key)) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) {
                *index_found = (size_t) (name - argnames);
                return 1;
            }
        }
        #endif
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (unlikely(key_hash == ((PyASCIIObject*)name_str)->hash)) {
            if (__Pyx_UnicodeKeywordsEqual(name_str, key))
                goto arg_passed_twice;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            if (unlikely(name_str == key)) goto arg_passed_twice;
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) goto arg_passed_twice;
        }
        #endif
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
bad:
    return -1;
}
static int __Pyx_MatchKeywordArg_nostr(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    if (unlikely(!PyUnicode_Check(key))) goto invalid_keyword_type;
    name = first_kw_arg;
    while (*name) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (cmp == 1) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        if (unlikely(cmp == -1)) goto bad;
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (unlikely(cmp != 0)) {
            if (cmp == 1) goto arg_passed_twice;
            else goto bad;
        }
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
bad:
    return -1;
}
static CYTHON_INLINE int __Pyx_MatchKeywordArg(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    return likely(PyUnicode_CheckExact(key)) ?
        __Pyx_MatchKeywordArg_str(key, argnames, first_kw_arg, index_found, function_name) :
        __Pyx_MatchKeywordArg_nostr(key, argnames, first_kw_arg, index_found, function_name);
}
static void __Pyx_RejectUnknownKeyword(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char *function_name)
{
    #if CYTHON_AVOID_BORROWED_REFS
    PyObject *pos = NULL;
    #else
    Py_ssize_t pos = 0;
    #endif
    PyObject *key = NULL;
    __Pyx_BEGIN_CRITICAL_SECTION(kwds);
    while (
        #if CYTHON_AVOID_BORROWED_REFS
        __Pyx_PyDict_NextRef(kwds, &pos, &key, NULL)
        #else
        PyDict_Next(kwds, &pos, &key, NULL)
        #endif
    ) {
        PyObject** const *name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (!*name) {
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp != 1) {
                if (cmp == 0) {
                    PyErr_Format(PyExc_TypeError,
                        "%s() got an unexpected keyword argument '%U'",
                        function_name, key);
                }
                #if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(key);
                #endif
                break;
            }
        }
        #if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(key);
        #endif
    }
    __Pyx_END_CRITICAL_SECTION();
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(pos);
    #endif
    assert(PyErr_Occurred());
}
static int __Pyx_ParseKeywordDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t extracted = 0;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    name = first_kw_arg;
    while (*name && num_kwargs > extracted) {
        PyObject * key = **name;
        PyObject *value;
        int found = 0;
        #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        found = PyDict_GetItemRef(kwds, key, &value);
        #else
        value = PyDict_GetItemWithError(kwds, key);
        if (value) {
            Py_INCREF(value);
            found = 1;
        } else {
            if (unlikely(PyErr_Occurred())) goto bad;
        }
        #endif
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            extracted++;
        }
        name++;
    }
    if (num_kwargs > extracted) {
        if (ignore_unknown_kwargs) {
            if (unlikely(__Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name) == -1))
                goto bad;
        } else {
            __Pyx_RejectUnknownKeyword(kwds, argnames, first_kw_arg, function_name);
            goto bad;
        }
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordDictToDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t len;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    if (PyDict_Update(kwds2, kwds) < 0) goto bad;
    name = first_kw_arg;
    while (*name) {
        PyObject *key = **name;
        PyObject *value;
#if !CYTHON_COMPILING_IN_LIMITED_API && (PY_VERSION_HEX >= 0x030d00A2 || defined(PyDict_Pop))
        int found = PyDict_Pop(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
        }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        int found = PyDict_GetItemRef(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            if (unlikely(PyDict_DelItem(kwds2, key) < 0)) goto bad;
        }
#else
    #if CYTHON_COMPILING_IN_CPYTHON
        value = _PyDict_Pop(kwds2, key, kwds2);
    #else
        value = __Pyx_CallUnboundCMethod2(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_pop, kwds2, key, kwds2);
    #endif
        if (value == kwds2) {
            Py_DECREF(value);
        } else {
            if (unlikely(!value)) goto bad;
            values[name-argnames] = value;
        }
#endif
        name++;
    }
    len = PyDict_Size(kwds2);
    if (len > 0) {
        return __Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name);
    } else if (unlikely(len == -1)) {
        goto bad;
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordsTuple(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject *key = NULL;
    PyObject** const * name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    for (Py_ssize_t pos = 0; pos < num_kwargs; pos++) {
#if CYTHON_AVOID_BORROWED_REFS
        key = __Pyx_PySequence_ITEM(kwds, pos);
#else
        key = __Pyx_PyTuple_GET_ITEM(kwds, pos);
#endif
#if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!key)) goto bad;
#endif
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            PyObject *value = kwvalues[pos];
            values[name-argnames] = __Pyx_NewRef(value);
        } else {
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp == 1) {
                PyObject *value = kwvalues[pos];
                values[index_found] = __Pyx_NewRef(value);
            } else {
                if (unlikely(cmp == -1)) goto bad;
                if (kwds2) {
                    PyObject *value = kwvalues[pos];
                    if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
                } else if (!ignore_unknown_kwargs) {
                    goto invalid_keyword;
                }
            }
        }
        #if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(key);
        key = NULL;
        #endif
    }
    return 0;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    goto bad;
bad:
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(key);
    #endif
    return -1;
}

/* ParseKeywords */
static int __Pyx_ParseKeywords(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds)))
        return __Pyx_ParseKeywordsTuple(kwds, kwvalues, argnames, kwds2, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
    else if (kwds2)
        return __Pyx_ParseKeywordDictToDict(kwds, argnames, kwds2, values, num_pos_args, function_name);
    else
        return __Pyx_ParseKeywordDict(kwds, argnames, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* DictGetItem */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    if (unlikely(__Pyx_PyDict_GetItemRef(d, key, &value) == 0)) { // no value, no error
        if (unlikely(PyTuple_Check(key))) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args)) {
                PyErr_SetObject(PyExc_KeyError, args);
                Py_DECREF(args);
            }
        } else {
            PyErr_SetObject(PyExc_KeyError, key);
        }
    }
    return value;
}
#endif

/* CallUnboundCMethod1 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    int was_initialized =  __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return __Pyx_CallCFunction(cfunc, self, arg);
        } else if (flag == METH_FASTCALL) {
            return __Pyx_CallCFunctionFast(cfunc, self, &arg, 1);
        } else if (flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, &arg, 1, NULL);
        }
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod1(&tmp_cfunc, self, arg);
    }
#endif
    PyObject* result = __Pyx__CallUnboundCMethod1(cfunc, self, arg);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        PyObject *args = PyTuple_New(1);
        if (unlikely(!args)) return NULL;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = __Pyx_CallCFunctionWithKeywords(cfunc, self, args, NULL);
        else
            result = __Pyx_CallCFunction(cfunc, self, args);
        Py_DECREF(args);
    } else
#endif
    {
        result = __Pyx_PyObject_Call2Args(cfunc->method, self, arg);
    }
    return result;
}

/* dict_getitem_default */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value) {
    PyObject* value;
#if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (unlikely(PyErr_Occurred()))
            return NULL;
        value = default_value;
    }
    Py_INCREF(value);
    if ((1));
#else
    if (PyBytes_CheckExact(key) || PyUnicode_CheckExact(key) || PyLong_CheckExact(key)) {
        value = PyDict_GetItem(d, key);
        if (unlikely(!value)) {
            value = default_value;
        }
        Py_INCREF(value);
    }
#endif
    else {
        if (default_value == Py_None)
            value = __Pyx_CallUnboundCMethod1(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_get, d, key);
        else
            value = __Pyx_CallUnboundCMethod2(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_get, d, key, default_value);
    }
    return value;
}

/* RaiseUnexpectedTypeError */
static int
__Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj)
{
    __Pyx_TypeName obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError, "Expected %s, got " __Pyx_FMT_TYPENAME,
                 expected, obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* RejectKeywords */
static void __Pyx_RejectKeywords(const char* function_name, PyObject *kwds) {
    PyObject *key = NULL;
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds))) {
        key = __Pyx_PySequence_ITEM(kwds, 0);
    } else {
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *pos = NULL;
#else
        Py_ssize_t pos = 0;
#endif
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
        if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return;
#endif
        __Pyx_PyDict_NextRef(kwds, &pos, &key, NULL);
#if CYTHON_AVOID_BORROWED_REFS
        Py_XDECREF(pos);
#endif
    }
    if (likely(key)) {
        PyErr_Format(PyExc_TypeError,
            "%s() got an unexpected keyword argument '%U'",
            function_name, key);
        Py_DECREF(key);
    }
}

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
    else state = (PyGILState_STATE)0;
    CYTHON_UNUSED_VAR(clineno);
    CYTHON_UNUSED_VAR(lineno);
    CYTHON_UNUSED_VAR(filename);
    CYTHON_MAYBE_UNUSED_VAR(nogil);
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(0);
    }
    ctx = PyUnicode_FromString(name);
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
    if (nogil)
        PyGILState_Release(state);
}

/* GetAttr3 */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
#endif
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    int res = PyObject_GetOptionalAttr(o, n, &r);
    return (res != 0) ? r : __Pyx_NewRef(d);
#else
  #if CYTHON_USE_TYPE_SLOTS
    if (likely(PyUnicode_Check(n))) {
        r = __Pyx_PyObject_GetAttrStrNoError(o, n);
        if (unlikely(!r) && likely(!PyErr_Occurred())) {
            r = __Pyx_NewRef(d);
        }
        return r;
    }
  #endif
    r = PyObject_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
#endif
}

/* PyObjectVectorCallKwBuilder */
#if CYTHON_VECTORCALL
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_PyObject_FastCallDict;
    Py_INCREF(key);
    if (__Pyx_PyTuple_SET_ITEM(builder, n, key) != (0)) return -1;
    args[n] = value;
    return 0;
}
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_VectorcallBuilder_AddArgStr;
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n);
}
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    PyObject *pyKey = PyUnicode_FromString(key);
    if (!pyKey) return -1;
    return __Pyx_VectorcallBuilder_AddArg(pyKey, value, builder, args, n);
}
#else // CYTHON_VECTORCALL
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, CYTHON_UNUSED PyObject **args, CYTHON_UNUSED int n) {
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return PyDict_SetItem(builder, key, value);
}
#endif

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
    PyObject* exc_type;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    exc_type = __Pyx_PyErr_CurrentExceptionType();
    if (unlikely(exc_type)) {
        if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)))
            return -1;
        __Pyx_PyErr_Clear();
        return 0;
    }
    return 0;
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    }
    return __Pyx_IterFinish();
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (unlikely(!j)) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_SIZE
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS)) {
        return __Pyx_PyList_GetItemRefFast(o, wrapped_i, unsafe_shared);
    } else
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        return __Pyx_NewRef(PyList_GET_ITEM(o, wrapped_i));
    }
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
#else
    (void)wraparound;
    (void)boundscheck;
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        return __Pyx_NewRef(PyTuple_GET_ITEM(o, wrapped_i));
    }
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
#else
    (void)wraparound;
    (void)boundscheck;
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS)) {
            return __Pyx_PyList_GetItemRefFast(o, n, unsafe_shared);
        } else if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            return __Pyx_NewRef(PyList_GET_ITEM(o, n));
        }
    } else
    #if !CYTHON_AVOID_BORROWED_REFS
    if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            return __Pyx_NewRef(PyTuple_GET_ITEM(o, n));
        }
    } else
    #endif
#endif
#if CYTHON_USE_TYPE_SLOTS && !CYTHON_COMPILING_IN_PYPY
    {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (!is_list && mm && mm->mp_subscript) {
            PyObject *r, *key = PyLong_FromSsize_t(i);
            if (unlikely(!key)) return NULL;
            r = mm->mp_subscript(o, key);
            Py_DECREF(key);
            return r;
        }
        if (is_list || likely(sm && sm->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return sm->sq_item(o, i);
        }
    }
#else
    if (is_list || !PyMapping_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    (void)wraparound;
    (void)boundscheck;
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject *index) {
    PyObject *runerr = NULL;
    Py_ssize_t key_value;
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        __Pyx_TypeName index_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(index));
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError,
            "cannot fit '" __Pyx_FMT_TYPENAME "' into an index-sized integer", index_type_name);
        __Pyx_DECREF_TypeName(index_type_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem_Slow(PyObject *obj, PyObject *key) {
    __Pyx_TypeName obj_type_name;
    if (likely(PyType_Check(obj))) {
        PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(obj, __pyx_mstate_global->__pyx_n_u_class_getitem);
        if (!meth) {
            PyErr_Clear();
        } else {
            PyObject *result = __Pyx_PyObject_CallOneArg(meth, key);
            Py_DECREF(meth);
            return result;
        }
    }
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is not subscriptable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key) {
    PyTypeObject *tp = Py_TYPE(obj);
    PyMappingMethods *mm = tp->tp_as_mapping;
    PySequenceMethods *sm = tp->tp_as_sequence;
    if (likely(mm && mm->mp_subscript)) {
        return mm->mp_subscript(obj, key);
    }
    if (likely(sm && sm->sq_item)) {
        return __Pyx_PyObject_GetIndex(obj, key);
    }
    return __Pyx_PyObject_GetItem_Slow(obj, key);
}
#endif

/* PyObjectFormatAndDecref */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f) {
    if (unlikely(!s)) return NULL;
    if (likely(PyUnicode_CheckExact(s))) return s;
    return __Pyx_PyObject_FormatAndDecref(s, f);
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f) {
    PyObject *result;
    if (unlikely(!s)) return NULL;
    result = PyObject_Format(s, f);
    Py_DECREF(s);
    return result;
}

/* JoinPyUnicode */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind, kind_shift;
    Py_ssize_t i, char_pos;
    void *result_udata;
    if (max_char > 1114111) max_char = 1114111;
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    kind_shift = (result_ukind == PyUnicode_4BYTE_KIND) ? 2 : result_ukind - 1;
    result_udata = PyUnicode_DATA(result_uval);
    assert(kind_shift == 2 || kind_shift == 1 || kind_shift == 0);
    if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - result_ulength < 0))
        goto overflow;
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = values[i];
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (__Pyx_PyUnicode_READY(uval) == (-1))
            goto bad;
        #endif
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(ulength < 0)) goto bad;
        #endif
        if (unlikely(!ulength))
            continue;
        if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - ulength < char_pos))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (ukind == result_ukind) {
            memcpy((char *)result_udata + (char_pos << kind_shift), udata, (size_t) (ulength << kind_shift));
        } else {
            #if PY_VERSION_HEX >= 0x030d0000
            if (unlikely(PyUnicode_CopyCharacters(result_uval, char_pos, uval, 0, ulength) < 0)) goto bad;
            #elif CYTHON_COMPILING_IN_CPYTHON || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    Py_ssize_t i;
    PyObject *result = NULL;
    PyObject *value_tuple = PyTuple_New(value_count);
    if (unlikely(!value_tuple)) return NULL;
    CYTHON_UNUSED_VAR(max_char);
    CYTHON_UNUSED_VAR(result_ulength);
    for (i=0; i<value_count; i++) {
        Py_INCREF(values[i]);
        if (__Pyx_PyTuple_SET_ITEM(value_tuple, i, values[i]) != (0)) goto bad;
    }
    result = PyUnicode_Join(__pyx_mstate_global->__pyx_empty_unicode, value_tuple);
bad:
    Py_DECREF(value_tuple);
    return result;
#endif
}

/* CIntToDigits (used by CIntToPyUnicode) */
static const char DIGIT_PAIRS_10[2*10*10+1] = {
    "00010203040506070809"
    "10111213141516171819"
    "20212223242526272829"
    "30313233343536373839"
    "40414243444546474849"
    "50515253545556575859"
    "60616263646566676869"
    "70717273747576777879"
    "80818283848586878889"
    "90919293949596979899"
};
static const char DIGIT_PAIRS_8[2*8*8+1] = {
    "0001020304050607"
    "1011121314151617"
    "2021222324252627"
    "3031323334353637"
    "4041424344454647"
    "5051525354555657"
    "6061626364656667"
    "7071727374757677"
};
static const char DIGITS_HEX[2*16+1] = {
    "0123456789abcdef"
    "0123456789ABCDEF"
};

/* BuildPyUnicode (used by COrdinalToPyUnicode) */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, const char* chars, int clength,
                                                int prepend_sign, char padding_char) {
    PyObject *uval;
    Py_ssize_t uoffset = ulength - clength;
#if CYTHON_USE_UNICODE_INTERNALS
    Py_ssize_t i;
    void *udata;
    uval = PyUnicode_New(ulength, 127);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_DATA(uval);
    if (uoffset > 0) {
        i = 0;
        if (prepend_sign) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, 0, '-');
            i++;
        }
        for (; i < uoffset; i++) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, i, padding_char);
        }
    }
    for (i=0; i < clength; i++) {
        __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, uoffset+i, chars[i]);
    }
#else
    {
        PyObject *sign = NULL, *padding = NULL;
        uval = NULL;
        if (uoffset > 0) {
            prepend_sign = !!prepend_sign;
            if (uoffset > prepend_sign) {
                padding = PyUnicode_FromOrdinal(padding_char);
                if (likely(padding) && uoffset > prepend_sign + 1) {
                    PyObject *tmp = PySequence_Repeat(padding, uoffset - prepend_sign);
                    Py_DECREF(padding);
                    padding = tmp;
                }
                if (unlikely(!padding)) goto done_or_error;
            }
            if (prepend_sign) {
                sign = PyUnicode_FromOrdinal('-');
                if (unlikely(!sign)) goto done_or_error;
            }
        }
        uval = PyUnicode_DecodeASCII(chars, clength, NULL);
        if (likely(uval) && padding) {
            PyObject *tmp = PyUnicode_Concat(padding, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
        if (likely(uval) && sign) {
            PyObject *tmp = PyUnicode_Concat(sign, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
done_or_error:
        Py_XDECREF(padding);
        Py_XDECREF(sign);
    }
#endif
    return uval;
}

/* COrdinalToPyUnicode (used by CIntToPyUnicode) */
static CYTHON_INLINE int __Pyx_CheckUnicodeValue(int value) {
    return value <= 1114111;
}
static PyObject* __Pyx_PyUnicode_FromOrdinal_Padded(int value, Py_ssize_t ulength, char padding_char) {
    Py_ssize_t padding_length = ulength - 1;
    if (likely((padding_length <= 250) && (value < 0xD800 || value > 0xDFFF))) {
        char chars[256];
        if (value <= 255) {
            memset(chars, padding_char, (size_t) padding_length);
            chars[ulength-1] = (char) value;
            return PyUnicode_DecodeLatin1(chars, ulength, NULL);
        }
        char *cpos = chars + sizeof(chars);
        if (value < 0x800) {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xc0 | (value & 0x1f));
        } else if (value < 0x10000) {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xe0 | (value & 0x0f));
        } else {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xf0 | (value & 0x07));
        }
        cpos -= padding_length;
        memset(cpos, padding_char, (size_t) padding_length);
        return PyUnicode_DecodeUTF8(cpos, chars + sizeof(chars) - cpos, NULL);
    }
    if (value <= 127 && CYTHON_USE_UNICODE_INTERNALS) {
        const char chars[1] = {(char) value};
        return __Pyx_PyUnicode_BuildFromAscii(ulength, chars, 1, 0, padding_char);
    }
    {
        PyObject *uchar, *padding_uchar, *padding, *result;
        padding_uchar = PyUnicode_FromOrdinal(padding_char);
        if (unlikely(!padding_uchar)) return NULL;
        padding = PySequence_Repeat(padding_uchar, padding_length);
        Py_DECREF(padding_uchar);
        if (unlikely(!padding)) return NULL;
        uchar = PyUnicode_FromOrdinal(value);
        if (unlikely(!uchar)) {
            Py_DECREF(padding);
            return NULL;
        }
        result = PyUnicode_Concat(padding, uchar);
        Py_DECREF(padding);
        Py_DECREF(uchar);
        return result;
    }
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_enum__H5T_class_t(enum H5T_class_t value, Py_ssize_t width, char padding_char) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum H5T_class_t neg_one = (enum H5T_class_t) -1, const_zero = (enum H5T_class_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!(is_unsigned || value == 0 || value > 0) ||
                    !(sizeof(value) <= 2 || value & ~ (enum H5T_class_t) 0x01fffff || __Pyx_CheckUnicodeValue((int) value)))) {
        PyErr_SetString(PyExc_OverflowError, "%c arg not in range(0x110000)");
        return NULL;
    }
    if (width <= 1) {
        return PyUnicode_FromOrdinal((int) value);
    }
    return __Pyx_PyUnicode_FromOrdinal_Padded((int) value, width, padding_char);
}
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_enum__H5T_class_t(enum H5T_class_t value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(enum H5T_class_t)*3+2];
    char *dpos, *end = digits + sizeof(enum H5T_class_t)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    enum H5T_class_t remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum H5T_class_t neg_one = (enum H5T_class_t) -1, const_zero = (enum H5T_class_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (enum H5T_class_t) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (enum H5T_class_t) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (enum H5T_class_t) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    __Pyx_TypeName obj_type_name;
    __Pyx_TypeName type_name;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    type_name = __Pyx_PyType_GetFullyQualifiedName(type);
    PyErr_Format(PyExc_TypeError,
                 "Cannot convert " __Pyx_FMT_TYPENAME " to " __Pyx_FMT_TYPENAME,
                 obj_type_name, type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
}

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_value = exc_info->exc_value;
    exc_info->exc_value = *value;
    if (tmp_value == NULL || tmp_value == Py_None) {
        Py_XDECREF(tmp_value);
        tmp_value = NULL;
        tmp_type = NULL;
        tmp_tb = NULL;
    } else {
        tmp_type = (PyObject*) Py_TYPE(tmp_value);
        Py_INCREF(tmp_type);
        #if CYTHON_COMPILING_IN_CPYTHON
        tmp_tb = ((PyBaseExceptionObject*) tmp_value)->traceback;
        Py_XINCREF(tmp_tb);
        #else
        tmp_tb = PyException_GetTraceback(tmp_value);
        #endif
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
  #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
  #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* ArgTypeTestFunc (used by ArgTypeTest) */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    __Pyx_TypeName type_name;
    __Pyx_TypeName obj_type_name;
    PyObject *extra_info = __pyx_mstate_global->__pyx_empty_unicode;
    int from_annotation_subclass = 0;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (!exact) {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    } else if (exact == 2) {
        if (__Pyx_TypeCheck(obj, type)) {
            from_annotation_subclass = 1;
            extra_info = __pyx_mstate_global->__pyx_kp_u_Note_that_Cython_is_deliberately;
        }
    }
    type_name = __Pyx_PyType_GetFullyQualifiedName(type);
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected " __Pyx_FMT_TYPENAME
        ", got " __Pyx_FMT_TYPENAME ")"
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
        "%s%U"
#endif
        , name, type_name, obj_type_name
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
        , (from_annotation_subclass ? ". " : ""), extra_info
#endif
        );
#if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    if (exact == 2 && from_annotation_subclass) {
        PyObject *res;
        PyObject *vargs[2];
        vargs[0] = PyErr_GetRaisedException();
        vargs[1] = extra_info;
        res = PyObject_VectorcallMethod(__pyx_mstate_global->__pyx_kp_u_add_note, vargs, 2, NULL);
        Py_XDECREF(res);
        PyErr_SetRaisedException(vargs[0]);
    }
#endif
    __Pyx_DECREF_TypeName(type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_Py_ssize_t(Py_ssize_t value, Py_ssize_t width, char padding_char) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const Py_ssize_t neg_one = (Py_ssize_t) -1, const_zero = (Py_ssize_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!(is_unsigned || value == 0 || value > 0) ||
                    !(sizeof(value) <= 2 || value & ~ (Py_ssize_t) 0x01fffff || __Pyx_CheckUnicodeValue((int) value)))) {
        PyErr_SetString(PyExc_OverflowError, "%c arg not in range(0x110000)");
        return NULL;
    }
    if (width <= 1) {
        return PyUnicode_FromOrdinal((int) value);
    }
    return __Pyx_PyUnicode_FromOrdinal_Padded((int) value, width, padding_char);
}
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_Py_ssize_t(Py_ssize_t value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(Py_ssize_t)*3+2];
    char *dpos, *end = digits + sizeof(Py_ssize_t)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    Py_ssize_t remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const Py_ssize_t neg_one = (Py_ssize_t) -1, const_zero = (Py_ssize_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (Py_ssize_t) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (Py_ssize_t) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (Py_ssize_t) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
    __Pyx_TypeName obj_type_name;
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyLong_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyLong_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is unsliceable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
bad:
    return NULL;
}

/* HasAttr */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!PyUnicode_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_PyObject_GetAttrStrNoError(o, n);
    if (!r) {
        return (unlikely(PyErr_Occurred())) ? -1 : 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}
#endif

/* PyLongBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_Fallback___Pyx_PyLong_AddObjC(PyObject *op1, PyObject *op2, int inplace) {
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#if CYTHON_USE_PYLONG_INTERNALS
static PyObject* __Pyx_Unpacked___Pyx_PyLong_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(inplace);
    CYTHON_UNUSED_VAR(zerodivision_check);
    const long b = intval;
    long a;
    const PY_LONG_LONG llb = intval;
    PY_LONG_LONG lla;
    if (unlikely(__Pyx_PyLong_IsZero(op1))) {
        return __Pyx_NewRef(op2);
    }
    const int is_positive = __Pyx_PyLong_IsPos(op1);
    const digit* digits = __Pyx_PyLong_Digits(op1);
    const Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
    if (likely(size == 1)) {
        a = (long) digits[0];
        if (!is_positive) a *= -1;
    } else {
        switch (size) {
            case 2:
                if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                    a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                    lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    if (!is_positive) lla *= -1;
                    goto calculate_long_long;
                }
                break;
            case 3:
                if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                    a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                    lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    if (!is_positive) lla *= -1;
                    goto calculate_long_long;
                }
                break;
            case 4:
                if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                    a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                    lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    if (!is_positive) lla *= -1;
                    goto calculate_long_long;
                }
                break;
        }
        return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    calculate_long:
        {
            long x;
            x = a + b;
            return PyLong_FromLong(x);
        }
    calculate_long_long:
        {
            PY_LONG_LONG llx;
            llx = lla + llb;
            return PyLong_FromLongLong(llx);
        }
    
}
#endif
static PyObject* __Pyx_Float___Pyx_PyLong_AddObjC(PyObject *float_val, long intval, int zerodivision_check) {
    CYTHON_UNUSED_VAR(zerodivision_check);
    const long b = intval;
    double a = __Pyx_PyFloat_AS_DOUBLE(float_val);
        double result;
        
        result = ((double)a) + (double)b;
        return PyFloat_FromDouble(result);
}
static CYTHON_INLINE PyObject* __Pyx_PyLong_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(zerodivision_check);
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        return __Pyx_Unpacked___Pyx_PyLong_AddObjC(op1, op2, intval, inplace, zerodivision_check);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        return __Pyx_Float___Pyx_PyLong_AddObjC(op1, intval, zerodivision_check);
    }
    return __Pyx_Fallback___Pyx_PyLong_AddObjC(op1, op2, inplace);
}
#endif

/* PyObjectVectorCallMethodKwBuilder */
#if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_Object_VectorcallMethod_CallFromBuilder(PyObject *name, PyObject *const *args, size_t nargsf, PyObject *kwnames) {
    PyObject *result;
    PyObject *obj = PyObject_GetAttr(args[0], name);
    if (unlikely(!obj))
        return NULL;
    result = __Pyx_Object_Vectorcall_CallFromBuilder(obj, args+1, nargsf-1, kwnames);
    Py_DECREF(obj);
    return result;
}
#endif

/* PyLongCompare */
static CYTHON_INLINE int __Pyx_PyLong_BoolEqObjC(PyObject *op1, PyObject *op2, long intval, long inplace) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(inplace);
    if (op1 == op2) {
        return 1;
    }
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
        const digit* digits = __Pyx_PyLong_Digits(op1);
        if (intval == 0) {
            return (__Pyx_PyLong_IsZero(op1) == 1);
        } else if (intval < 0) {
            if (__Pyx_PyLong_IsNonNeg(op1))
                return 0;
            intval = -intval;
        } else {
            if (__Pyx_PyLong_IsNeg(op1))
                return 0;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        return (unequal == 0);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = __Pyx_PyFloat_AS_DOUBLE(op1);
        return ((double)a == (double)b);
    }
    return __Pyx_PyObject_IsTrueAndDecref(
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* AllocateExtensionType */
static PyObject *__Pyx_AllocateExtensionType(PyTypeObject *t, int is_final) {
    if (is_final || likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
        allocfunc alloc_func = __Pyx_PyType_GetSlot(t, tp_alloc, allocfunc);
        return alloc_func(t, 0);
    } else {
        newfunc tp_new = __Pyx_PyType_TryGetSlot(&PyBaseObject_Type, tp_new, newfunc);
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
        if (!tp_new) {
            PyObject *new_str = PyUnicode_FromString("__new__");
            if (likely(new_str)) {
                PyObject *o = PyObject_CallMethodObjArgs((PyObject *)&PyBaseObject_Type, new_str, t, NULL);
                Py_DECREF(new_str);
                return o;
            } else
                return NULL;
        } else
    #endif
        return tp_new(t, __pyx_mstate_global->__pyx_empty_tuple, 0);
    }
}

/* CallTypeTraverse */
#if !CYTHON_USE_TYPE_SPECS || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x03090000)
#else
static int __Pyx_call_type_traverse(PyObject *o, int always_call, visitproc visit, void *arg) {
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x03090000
    if (__Pyx_get_runtime_version() < 0x03090000) return 0;
    #endif
    if (!always_call) {
        PyTypeObject *base = __Pyx_PyObject_GetSlot(o, tp_base, PyTypeObject*);
        unsigned long flags = PyType_GetFlags(base);
        if (flags & Py_TPFLAGS_HEAPTYPE) {
            return 0;
        }
    }
    Py_VISIT((PyObject*)Py_TYPE(o));
    return 0;
}
#endif

/* LimitedApiGetTypeDict (used by SetItemOnTypeDict) */
#if CYTHON_COMPILING_IN_LIMITED_API
static Py_ssize_t __Pyx_GetTypeDictOffset(void) {
    PyObject *tp_dictoffset_o;
    Py_ssize_t tp_dictoffset;
    tp_dictoffset_o = PyObject_GetAttrString((PyObject*)(&PyType_Type), "__dictoffset__");
    if (unlikely(!tp_dictoffset_o)) return -1;
    tp_dictoffset = PyLong_AsSsize_t(tp_dictoffset_o);
    Py_DECREF(tp_dictoffset_o);
    if (unlikely(tp_dictoffset == 0)) {
        PyErr_SetString(
            PyExc_TypeError,
            "'type' doesn't have a dictoffset");
        return -1;
    } else if (unlikely(tp_dictoffset < 0)) {
        PyErr_SetString(
            PyExc_TypeError,
            "'type' has an unexpected negative dictoffset. "
            "Please report this as Cython bug");
        return -1;
    }
    return tp_dictoffset;
}
static PyObject *__Pyx_GetTypeDict(PyTypeObject *tp) {
    static Py_ssize_t tp_dictoffset = 0;
    if (unlikely(tp_dictoffset == 0)) {
        tp_dictoffset = __Pyx_GetTypeDictOffset();
        if (unlikely(tp_dictoffset == -1 && PyErr_Occurred())) {
            tp_dictoffset = 0; // try again next time?
            return NULL;
        }
    }
    return *(PyObject**)((char*)tp + tp_dictoffset);
}
#endif

/* SetItemOnTypeDict (used by FixUpExtensionType) */
static int __Pyx__SetItemOnTypeDict(PyTypeObject *tp, PyObject *k, PyObject *v) {
    int result;
    PyObject *tp_dict;
#if CYTHON_COMPILING_IN_LIMITED_API
    tp_dict = __Pyx_GetTypeDict(tp);
    if (unlikely(!tp_dict)) return -1;
#else
    tp_dict = tp->tp_dict;
#endif
    result = PyDict_SetItem(tp_dict, k, v);
    if (likely(!result)) {
        PyType_Modified(tp);
        if (unlikely(PyObject_HasAttr(v, __pyx_mstate_global->__pyx_n_u_set_name))) {
            PyObject *setNameResult = PyObject_CallMethodObjArgs(v, __pyx_mstate_global->__pyx_n_u_set_name,  (PyObject *) tp, k, NULL);
            if (!setNameResult) return -1;
            Py_DECREF(setNameResult);
        }
    }
    return result;
}

/* FixUpExtensionType */
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type) {
#if __PYX_LIMITED_VERSION_HEX > 0x030900B1
    CYTHON_UNUSED_VAR(spec);
    CYTHON_UNUSED_VAR(type);
    CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
#else
    const PyType_Slot *slot = spec->slots;
    int changed = 0;
#if !CYTHON_COMPILING_IN_LIMITED_API
    while (slot && slot->slot && slot->slot != Py_tp_members)
        slot++;
    if (slot && slot->slot == Py_tp_members) {
#if !CYTHON_COMPILING_IN_CPYTHON
        const
#endif  // !CYTHON_COMPILING_IN_CPYTHON)
            PyMemberDef *memb = (PyMemberDef*) slot->pfunc;
        while (memb && memb->name) {
            if (memb->name[0] == '_' && memb->name[1] == '_') {
                if (strcmp(memb->name, "__weaklistoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_weaklistoffset = memb->offset;
                    changed = 1;
                }
                else if (strcmp(memb->name, "__dictoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_dictoffset = memb->offset;
                    changed = 1;
                }
#if CYTHON_METH_FASTCALL
                else if (strcmp(memb->name, "__vectorcalloffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_vectorcall_offset = memb->offset;
                    changed = 1;
                }
#endif  // CYTHON_METH_FASTCALL
#if !CYTHON_COMPILING_IN_PYPY
                else if (strcmp(memb->name, "__module__") == 0) {
                    PyObject *descr;
                    assert(memb->type == T_OBJECT);
                    assert(memb->flags == 0 || memb->flags == READONLY);
                    descr = PyDescr_NewMember(type, memb);
                    if (unlikely(!descr))
                        return -1;
                    int set_item_result = PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr);
                    Py_DECREF(descr);
                    if (unlikely(set_item_result < 0)) {
                        return -1;
                    }
                    changed = 1;
                }
#endif  // !CYTHON_COMPILING_IN_PYPY
            }
            memb++;
        }
    }
#endif  // !CYTHON_COMPILING_IN_LIMITED_API
#if !CYTHON_COMPILING_IN_PYPY
    slot = spec->slots;
    while (slot && slot->slot && slot->slot != Py_tp_getset)
        slot++;
    if (slot && slot->slot == Py_tp_getset) {
        PyGetSetDef *getset = (PyGetSetDef*) slot->pfunc;
        while (getset && getset->name) {
            if (getset->name[0] == '_' && getset->name[1] == '_' && strcmp(getset->name, "__module__") == 0) {
                PyObject *descr = PyDescr_NewGetSet(type, getset);
                if (unlikely(!descr))
                    return -1;
                #if CYTHON_COMPILING_IN_LIMITED_API
                PyObject *pyname = PyUnicode_FromString(getset->name);
                if (unlikely(!pyname)) {
                    Py_DECREF(descr);
                    return -1;
                }
                int set_item_result = __Pyx_SetItemOnTypeDict(type, pyname, descr);
                Py_DECREF(pyname);
                #else
                CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
                int set_item_result = PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr);
                #endif
                Py_DECREF(descr);
                if (unlikely(set_item_result < 0)) {
                    return -1;
                }
                changed = 1;
            }
            ++getset;
        }
    }
#else
    CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
#endif  // !CYTHON_COMPILING_IN_PYPY
    if (changed)
        PyType_Modified(type);
#endif  // PY_VERSION_HEX > 0x030900B1
    return 0;
}

/* PyObjectCallNoArg (used by PyObjectCallMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
    PyObject *arg[2] = {NULL, NULL};
    return __Pyx_PyObject_FastCall(func, arg + 1, 0 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectCallMethod0 (used by PyType_Ready) */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
#if CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000))
    PyObject *args[1] = {obj};
    (void) __Pyx_PyObject_CallOneArg;
    (void) __Pyx_PyObject_CallNoArg;
    return PyObject_VectorcallMethod(method_name, args, 1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#else
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
#endif
}

/* ValidateBasesTuple (used by PyType_Ready) */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases) {
    Py_ssize_t i, n;
#if CYTHON_ASSUME_SAFE_SIZE
    n = PyTuple_GET_SIZE(bases);
#else
    n = PyTuple_Size(bases);
    if (unlikely(n < 0)) return -1;
#endif
    for (i = 1; i < n; i++)
    {
        PyTypeObject *b;
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *b0 = PySequence_GetItem(bases, i);
        if (!b0) return -1;
#elif CYTHON_ASSUME_SAFE_MACROS
        PyObject *b0 = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *b0 = PyTuple_GetItem(bases, i);
        if (!b0) return -1;
#endif
        b = (PyTypeObject*) b0;
        if (!__Pyx_PyType_HasFeature(b, Py_TPFLAGS_HEAPTYPE))
        {
            __Pyx_TypeName b_name = __Pyx_PyType_GetFullyQualifiedName(b);
            PyErr_Format(PyExc_TypeError,
                "base class '" __Pyx_FMT_TYPENAME "' is not a heap type", b_name);
            __Pyx_DECREF_TypeName(b_name);
#if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(b0);
#endif
            return -1;
        }
        if (dictoffset == 0)
        {
            Py_ssize_t b_dictoffset = 0;
#if CYTHON_USE_TYPE_SLOTS
            b_dictoffset = b->tp_dictoffset;
#else
            PyObject *py_b_dictoffset = PyObject_GetAttrString((PyObject*)b, "__dictoffset__");
            if (!py_b_dictoffset) goto dictoffset_return;
            b_dictoffset = PyLong_AsSsize_t(py_b_dictoffset);
            Py_DECREF(py_b_dictoffset);
            if (b_dictoffset == -1 && PyErr_Occurred()) goto dictoffset_return;
#endif
            if (b_dictoffset) {
                {
                    __Pyx_TypeName b_name = __Pyx_PyType_GetFullyQualifiedName(b);
                    PyErr_Format(PyExc_TypeError,
                        "extension type '%.200s' has no __dict__ slot, "
                        "but base type '" __Pyx_FMT_TYPENAME "' has: "
                        "either add 'cdef dict __dict__' to the extension type "
                        "or add '__slots__ = [...]' to the base type",
                        type_name, b_name);
                    __Pyx_DECREF_TypeName(b_name);
                }
#if !CYTHON_USE_TYPE_SLOTS
              dictoffset_return:
#endif
#if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(b0);
#endif
                return -1;
            }
        }
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(b0);
#endif
    }
    return 0;
}
#endif

/* PyType_Ready */
CYTHON_UNUSED static int __Pyx_PyType_HasMultipleInheritance(PyTypeObject *t) {
    while (t) {
        PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
        if (bases) {
            return 1;
        }
        t = __Pyx_PyType_GetSlot(t, tp_base, PyTypeObject*);
    }
    return 0;
}
static int __Pyx_PyType_Ready(PyTypeObject *t) {
#if CYTHON_USE_TYPE_SPECS || !CYTHON_COMPILING_IN_CPYTHON || defined(PYSTON_MAJOR_VERSION)
    (void)__Pyx_PyObject_CallMethod0;
#if CYTHON_USE_TYPE_SPECS
    (void)__Pyx_validate_bases_tuple;
#endif
    return PyType_Ready(t);
#else
    int r;
    if (!__Pyx_PyType_HasMultipleInheritance(t)) {
        return PyType_Ready(t);
    }
    PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
    if (bases && unlikely(__Pyx_validate_bases_tuple(t->tp_name, t->tp_dictoffset, bases) == -1))
        return -1;
#if !defined(PYSTON_MAJOR_VERSION)
    {
        int gc_was_enabled;
    #if PY_VERSION_HEX >= 0x030A00b1
        gc_was_enabled = PyGC_Disable();
        (void)__Pyx_PyObject_CallMethod0;
    #else
        PyObject *ret, *py_status;
        PyObject *gc = NULL;
        #if (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM+0 >= 0x07030400) &&\
                !CYTHON_COMPILING_IN_GRAAL
        gc = PyImport_GetModule(__pyx_mstate_global->__pyx_kp_u_gc);
        #endif
        if (unlikely(!gc)) gc = PyImport_Import(__pyx_mstate_global->__pyx_kp_u_gc);
        if (unlikely(!gc)) return -1;
        py_status = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_isenabled);
        if (unlikely(!py_status)) {
            Py_DECREF(gc);
            return -1;
        }
        gc_was_enabled = __Pyx_PyObject_IsTrue(py_status);
        Py_DECREF(py_status);
        if (gc_was_enabled > 0) {
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_disable);
            if (unlikely(!ret)) {
                Py_DECREF(gc);
                return -1;
            }
            Py_DECREF(ret);
        } else if (unlikely(gc_was_enabled == -1)) {
            Py_DECREF(gc);
            return -1;
        }
    #endif
        t->tp_flags |= Py_TPFLAGS_HEAPTYPE;
#if PY_VERSION_HEX >= 0x030A0000
        t->tp_flags |= Py_TPFLAGS_IMMUTABLETYPE;
#endif
#else
        (void)__Pyx_PyObject_CallMethod0;
#endif
    r = PyType_Ready(t);
#if !defined(PYSTON_MAJOR_VERSION)
        t->tp_flags &= ~Py_TPFLAGS_HEAPTYPE;
    #if PY_VERSION_HEX >= 0x030A00b1
        if (gc_was_enabled)
            PyGC_Enable();
    #else
        if (gc_was_enabled) {
            PyObject *tp, *v, *tb;
            PyErr_Fetch(&tp, &v, &tb);
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_enable);
            if (likely(ret || r == -1)) {
                Py_XDECREF(ret);
                PyErr_Restore(tp, v, tb);
            } else {
                Py_XDECREF(tp);
                Py_XDECREF(v);
                Py_XDECREF(tb);
                r = -1;
            }
        }
        Py_DECREF(gc);
    #endif
    }
#endif
    return r;
#endif
}

/* DelItemOnTypeDict (used by SetupReduce) */
static int __Pyx__DelItemOnTypeDict(PyTypeObject *tp, PyObject *k) {
    int result;
    PyObject *tp_dict;
#if CYTHON_COMPILING_IN_LIMITED_API
    tp_dict = __Pyx_GetTypeDict(tp);
    if (unlikely(!tp_dict)) return -1;
#else
    tp_dict = tp->tp_dict;
#endif
    result = PyDict_DelItem(tp_dict, k);
    if (likely(!result)) PyType_Modified(tp);
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStrNoError(meth, __pyx_mstate_global->__pyx_n_u_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_mstate_global->__pyx_n_u_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_mstate_global->__pyx_n_u_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_reduce_cython);
            if (likely(reduce_cython)) {
                ret = __Pyx_SetItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = __Pyx_DelItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_mstate_global->__pyx_n_u_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = __Pyx_SetItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = __Pyx_DelItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred()) {
        __Pyx_TypeName type_obj_name =
            __Pyx_PyType_GetFullyQualifiedName((PyTypeObject*)type_obj);
        PyErr_Format(PyExc_RuntimeError,
            "Unable to initialize pickling for " __Pyx_FMT_TYPENAME, type_obj_name);
        __Pyx_DECREF_TypeName(type_obj_name);
    }
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* SetVTable */
static int __Pyx_SetVtable(PyTypeObject *type, void *vtable) {
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
    if (unlikely(!ob))
        goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(PyObject_SetAttr((PyObject *) type, __pyx_mstate_global->__pyx_n_u_pyx_vtable, ob) < 0))
#else
    if (unlikely(PyDict_SetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable, ob) < 0))
#endif
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* GetVTable (used by MergeVTables) */
static void* __Pyx_GetVtable(PyTypeObject *type) {
    void* ptr;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *ob = PyObject_GetAttr((PyObject *)type, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
#else
    PyObject *ob = PyObject_GetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
#endif
    if (!ob)
        goto bad;
    ptr = PyCapsule_GetPointer(ob, 0);
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* MergeVTables */
static int __Pyx_MergeVtables(PyTypeObject *type) {
    int i=0;
    Py_ssize_t size;
    void** base_vtables;
    __Pyx_TypeName tp_base_name = NULL;
    __Pyx_TypeName base_name = NULL;
    void* unknown = (void*)-1;
    PyObject* bases = __Pyx_PyType_GetSlot(type, tp_bases, PyObject*);
    int base_depth = 0;
    {
        PyTypeObject* base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
        while (base) {
            base_depth += 1;
            base = __Pyx_PyType_GetSlot(base, tp_base, PyTypeObject*);
        }
    }
    base_vtables = (void**) PyMem_Malloc(sizeof(void*) * (size_t)(base_depth + 1));
    base_vtables[0] = unknown;
#if CYTHON_COMPILING_IN_LIMITED_API
    size = PyTuple_Size(bases);
    if (size < 0) goto other_failure;
#else
    size = PyTuple_GET_SIZE(bases);
#endif
    for (i = 1; i < size; i++) {
        PyObject *basei;
        void* base_vtable;
#if CYTHON_AVOID_BORROWED_REFS
        basei = PySequence_GetItem(bases, i);
        if (unlikely(!basei)) goto other_failure;
#elif !CYTHON_ASSUME_SAFE_MACROS
        basei = PyTuple_GetItem(bases, i);
        if (unlikely(!basei)) goto other_failure;
#else
        basei = PyTuple_GET_ITEM(bases, i);
#endif
        base_vtable = __Pyx_GetVtable((PyTypeObject*)basei);
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(basei);
#endif
        if (base_vtable != NULL) {
            int j;
            PyTypeObject* base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
            for (j = 0; j < base_depth; j++) {
                if (base_vtables[j] == unknown) {
                    base_vtables[j] = __Pyx_GetVtable(base);
                    base_vtables[j + 1] = unknown;
                }
                if (base_vtables[j] == base_vtable) {
                    break;
                } else if (base_vtables[j] == NULL) {
                    goto bad;
                }
                base = __Pyx_PyType_GetSlot(base, tp_base, PyTypeObject*);
            }
        }
    }
    PyErr_Clear();
    PyMem_Free(base_vtables);
    return 0;
bad:
    {
        PyTypeObject* basei = NULL;
        PyTypeObject* tp_base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
        tp_base_name = __Pyx_PyType_GetFullyQualifiedName(tp_base);
#if CYTHON_AVOID_BORROWED_REFS
        basei = (PyTypeObject*)PySequence_GetItem(bases, i);
        if (unlikely(!basei)) goto really_bad;
#elif !CYTHON_ASSUME_SAFE_MACROS
        basei = (PyTypeObject*)PyTuple_GetItem(bases, i);
        if (unlikely(!basei)) goto really_bad;
#else
        basei = (PyTypeObject*)PyTuple_GET_ITEM(bases, i);
#endif
        base_name = __Pyx_PyType_GetFullyQualifiedName(basei);
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(basei);
#endif
    }
    PyErr_Format(PyExc_TypeError,
        "multiple bases have vtable conflict: '" __Pyx_FMT_TYPENAME "' and '" __Pyx_FMT_TYPENAME "'", tp_base_name, base_name);
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
really_bad: // bad has failed!
#endif
    __Pyx_DECREF_TypeName(tp_base_name);
    __Pyx_DECREF_TypeName(base_name);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
other_failure:
#endif
    PyMem_Free(base_vtables);
    return -1;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType_3_2_8
#define __PYX_HAVE_RT_ImportType_3_2_8
static PyTypeObject *__Pyx_ImportType_3_2_8(PyObject *module, const char *module_name, const char *class_name,
    size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_2_8 check_size)
{
    PyObject *result = 0;
    Py_ssize_t basicsize;
    Py_ssize_t itemsize;
#if defined(Py_LIMITED_API) || (defined(CYTHON_COMPILING_IN_LIMITED_API) && CYTHON_COMPILING_IN_LIMITED_API)
    PyObject *py_basicsize;
    PyObject *py_itemsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#if !( defined(Py_LIMITED_API) || (defined(CYTHON_COMPILING_IN_LIMITED_API) && CYTHON_COMPILING_IN_LIMITED_API) )
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
    itemsize = ((PyTypeObject *)result)->tp_itemsize;
#else
    if (size == 0) {
        return (PyTypeObject *)result;
    }
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
    py_itemsize = PyObject_GetAttrString(result, "__itemsize__");
    if (!py_itemsize)
        goto bad;
    itemsize = PyLong_AsSsize_t(py_itemsize);
    Py_DECREF(py_itemsize);
    py_itemsize = 0;
    if (itemsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (itemsize) {
        if (size % alignment) {
            alignment = size % alignment;
        }
        if (itemsize < (Py_ssize_t)alignment)
            itemsize = (Py_ssize_t)alignment;
    }
    if ((size_t)(basicsize + itemsize) < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize+itemsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error_3_2_8 &&
            ((size_t)basicsize > size || (size_t)(basicsize + itemsize) < size)) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd-%zd from PyObject",
            module_name, class_name, size, basicsize, basicsize+itemsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn_3_2_8 && (size_t)basicsize > size) {
        if (PyErr_WarnFormat(NULL, 0,
                "%.200s.%.200s size changed, may indicate binary incompatibility. "
                "Expected %zd from C header, got %zd from PyObject",
                module_name, class_name, size, basicsize) < 0) {
            goto bad;
        }
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* PxdImportShared (used by FunctionImport) */
#ifndef __PYX_HAVE_RT_ImportFromPxd_3_2_8
#define __PYX_HAVE_RT_ImportFromPxd_3_2_8
static int __Pyx_ImportFromPxd_3_2_8(PyObject *module, const char *name, void **p, const char *sig, const char *what) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, "__pyx_capi__");
    if (!d)
        goto bad;
#if (defined(Py_LIMITED_API) && Py_LIMITED_API >= 0x030d0000) || (!defined(Py_LIMITED_API) && PY_VERSION_HEX >= 0x030d0000)
    PyDict_GetItemStringRef(d, name, &cobj);
#else
    cobj = PyDict_GetItemString(d, name);
    Py_XINCREF(cobj);
#endif
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C %.8s %.200s",
                PyModule_GetName(module), what, name);
        goto bad;
    }
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C %.8s %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             what, PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    Py_DECREF(cobj);
    return 0;
bad:
    Py_XDECREF(d);
    Py_XDECREF(cobj);
    return -1;
}
#endif

/* FunctionImport */
#ifndef __PYX_HAVE_RT_ImportFunction_3_2_8
#define __PYX_HAVE_RT_ImportFunction_3_2_8
static int __Pyx_ImportFunction_3_2_8(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    int result = __Pyx_ImportFromPxd_3_2_8(module, funcname, &tmp.p, sig, "function");
    if (result == 0) {
        *f = tmp.fp;
    }
    return result;
}
#endif

/* ImportImpl (used by Import) */
static int __Pyx__Import_GetModule(PyObject *qualname, PyObject **module) {
    PyObject *imported_module = PyImport_GetModule(qualname);
    if (unlikely(!imported_module)) {
        *module = NULL;
        if (PyErr_Occurred()) {
            return -1;
        }
        return 0;
    }
    *module = imported_module;
    return 1;
}
static int __Pyx__Import_Lookup(PyObject *qualname, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject **module) {
    PyObject *imported_module;
    PyObject *top_level_package_name;
    Py_ssize_t i;
    int status, module_found;
    Py_ssize_t dot_index;
    module_found = __Pyx__Import_GetModule(qualname, &imported_module);
    if (unlikely(!module_found || module_found == -1)) {
        *module = NULL;
        return module_found;
    }
    if (imported_names) {
        for (i = 0; i < len_imported_names; i++) {
            PyObject *imported_name = imported_names[i];
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
            int has_imported_attribute = PyObject_HasAttr(imported_module, imported_name);
#else
            int has_imported_attribute = PyObject_HasAttrWithError(imported_module, imported_name);
            if (unlikely(has_imported_attribute == -1)) goto error;
#endif
            if (!has_imported_attribute) {
                goto not_found;
            }
        }
        *module = imported_module;
        return 1;
    }
    dot_index = PyUnicode_FindChar(qualname, '.', 0, PY_SSIZE_T_MAX, 1);
    if (dot_index == -1) {
        *module = imported_module;
        return 1;
    }
    if (unlikely(dot_index == -2)) goto error;
    top_level_package_name = PyUnicode_Substring(qualname, 0, dot_index);
    if (unlikely(!top_level_package_name)) goto error;
    Py_DECREF(imported_module);
    status = __Pyx__Import_GetModule(top_level_package_name, module);
    Py_DECREF(top_level_package_name);
    return status;
error:
    Py_DECREF(imported_module);
    *module = NULL;
    return -1;
not_found:
    Py_DECREF(imported_module);
    *module = NULL;
    return 0;
}
static PyObject *__Pyx__Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, PyObject *moddict, int level) {
    PyObject *module = 0;
    PyObject *empty_dict = 0;
    PyObject *from_list = 0;
    int module_found;
    if (!qualname) {
        qualname = name;
    }
    module_found = __Pyx__Import_Lookup(qualname, imported_names, len_imported_names, &module);
    if (likely(module_found == 1)) {
        return module;
    } else if (unlikely(module_found == -1)) {
        return NULL;
    }
    empty_dict = PyDict_New();
    if (unlikely(!empty_dict))
        goto bad;
    if (imported_names) {
#if CYTHON_COMPILING_IN_CPYTHON
        from_list = __Pyx_PyList_FromArray(imported_names, len_imported_names);
        if (unlikely(!from_list))
            goto bad;
#else
        from_list = PyList_New(len_imported_names);
        if (unlikely(!from_list)) goto bad;
        for (Py_ssize_t i=0; i<len_imported_names; ++i) {
            if (PyList_SetItem(from_list, i, __Pyx_NewRef(imported_names[i])) < 0) goto bad;
        }
#endif
    }
    if (level == -1) {
        const char* package_sep = strchr(__Pyx_MODULE_NAME, '.');
        if (package_sep != (0)) {
            module = PyImport_ImportModuleLevelObject(
                name, moddict, empty_dict, from_list, 1);
            if (unlikely(!module)) {
                if (unlikely(!PyErr_ExceptionMatches(PyExc_ImportError)))
                    goto bad;
                PyErr_Clear();
            }
        }
        level = 0;
    }
    if (!module) {
        module = PyImport_ImportModuleLevelObject(
            name, moddict, empty_dict, from_list, level);
    }
bad:
    Py_XDECREF(from_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, int level) {
    return __Pyx__Import(name, imported_names, len_imported_names, qualname, __pyx_mstate_global->__pyx_d, level);
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        const char* module_name_str = 0;
        PyObject* module_name = 0;
        PyObject* module_dot = 0;
        PyObject* full_name = 0;
        PyErr_Clear();
        module_name_str = PyModule_GetName(module);
        if (unlikely(!module_name_str)) { goto modbad; }
        module_name = PyUnicode_FromString(module_name_str);
        if (unlikely(!module_name)) { goto modbad; }
        module_dot = PyUnicode_Concat(module_name, __pyx_mstate_global->__pyx_kp_u__3);
        if (unlikely(!module_dot)) { goto modbad; }
        full_name = PyUnicode_Concat(module_dot, name);
        if (unlikely(!full_name)) { goto modbad; }
        #if (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400) ||\
                CYTHON_COMPILING_IN_GRAAL
        {
            PyObject *modules = PyImport_GetModuleDict();
            if (unlikely(!modules))
                goto modbad;
            value = PyObject_GetItem(modules, full_name);
        }
        #else
        value = PyImport_GetModule(full_name);
        #endif
      modbad:
        Py_XDECREF(full_name);
        Py_XDECREF(module_dot);
        Py_XDECREF(module_name);
    }
    if (unlikely(!value)) {
        PyErr_Format(PyExc_ImportError, "cannot import name %S", name);
    }
    return value;
}

/* ListPack */
static PyObject *__Pyx_PyList_Pack(Py_ssize_t n, ...) {
    va_list va;
    PyObject *l = PyList_New(n);
    va_start(va, n);
    if (unlikely(!l)) goto end;
    for (Py_ssize_t i=0; i<n; ++i) {
        PyObject *arg = va_arg(va, PyObject*);
        Py_INCREF(arg);
        if (__Pyx_PyList_SET_ITEM(l, i, arg) != (0)) {
            Py_CLEAR(l);
            goto end;
        }
    }
    end:
    va_end(va);
    return l;
}

/* dict_setdefault (used by FetchCommonType) */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value) {
    PyObject* value;
#if __PYX_LIMITED_VERSION_HEX >= 0x030F0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4)
    PyDict_SetDefaultRef(d, key, default_value, &value);
#elif CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    PyObject *args[] = {d, key, default_value};
    value = PyObject_VectorcallMethod(__pyx_mstate_global->__pyx_n_u_setdefault, args, 3 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#elif CYTHON_COMPILING_IN_LIMITED_API
    value = PyObject_CallMethodObjArgs(d, __pyx_mstate_global->__pyx_n_u_setdefault, key, default_value, NULL);
#else
    value = PyDict_SetDefault(d, key, default_value);
    if (unlikely(!value)) return NULL;
    Py_INCREF(value);
#endif
    return value;
}

/* AddModuleRef (used by FetchSharedCythonModule) */
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  static PyObject *__Pyx_PyImport_AddModuleObjectRef(PyObject *name) {
      PyObject *module_dict = PyImport_GetModuleDict();
      PyObject *m;
      if (PyMapping_GetOptionalItem(module_dict, name, &m) < 0) {
          return NULL;
      }
      if (m != NULL && PyModule_Check(m)) {
          return m;
      }
      Py_XDECREF(m);
      m = PyModule_NewObject(name);
      if (m == NULL)
          return NULL;
      if (PyDict_CheckExact(module_dict)) {
          PyObject *new_m;
          (void)PyDict_SetDefaultRef(module_dict, name, m, &new_m);
          Py_DECREF(m);
          return new_m;
      } else {
           if (PyObject_SetItem(module_dict, name, m) != 0) {
                Py_DECREF(m);
                return NULL;
            }
            return m;
      }
  }
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *py_name = PyUnicode_FromString(name);
      if (!py_name) return NULL;
      PyObject *module = __Pyx_PyImport_AddModuleObjectRef(py_name);
      Py_DECREF(py_name);
      return module;
  }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#else
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *module = PyImport_AddModule(name);
      Py_XINCREF(module);
      return module;
  }
#endif

/* FetchSharedCythonModule (used by FetchCommonType) */
static PyObject *__Pyx_FetchSharedCythonABIModule(void) {
    return __Pyx_PyImport_AddModuleRef(__PYX_ABI_MODULE_NAME);
}

/* FetchCommonType (used by CommonTypesMetaclass) */
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
static PyObject* __Pyx_PyType_FromMetaclass(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *result = __Pyx_PyType_FromModuleAndSpec(module, spec, bases);
    if (result && metaclass) {
        PyObject *old_tp = (PyObject*)Py_TYPE(result);
    Py_INCREF((PyObject*)metaclass);
#if __PYX_LIMITED_VERSION_HEX >= 0x03090000
        Py_SET_TYPE(result, metaclass);
#else
        result->ob_type = metaclass;
#endif
        Py_DECREF(old_tp);
    }
    return result;
}
#else
#define __Pyx_PyType_FromMetaclass(me, mo, s, b) PyType_FromMetaclass(me, mo, s, b)
#endif
static int __Pyx_VerifyCachedType(PyObject *cached_type,
                               const char *name,
                               Py_ssize_t expected_basicsize) {
    Py_ssize_t basicsize;
    if (!PyType_Check(cached_type)) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s is not a type object", name);
        return -1;
    }
    if (expected_basicsize == 0) {
        return 0; // size is inherited, nothing useful to check
    }
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_basicsize;
    py_basicsize = PyObject_GetAttrString(cached_type, "__basicsize__");
    if (unlikely(!py_basicsize)) return -1;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = NULL;
    if (unlikely(basicsize == (Py_ssize_t)-1) && PyErr_Occurred()) return -1;
#else
    basicsize = ((PyTypeObject*) cached_type)->tp_basicsize;
#endif
    if (basicsize != expected_basicsize) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s has the wrong size, try recompiling",
            name);
        return -1;
    }
    return 0;
}
static PyTypeObject *__Pyx_FetchCommonTypeFromSpec(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *abi_module = NULL, *cached_type = NULL, *abi_module_dict, *new_cached_type, *py_object_name;
    int get_item_ref_result;
    const char* object_name = strrchr(spec->name, '.');
    object_name = object_name ? object_name+1 : spec->name;
    py_object_name = PyUnicode_FromString(object_name);
    if (!py_object_name) return NULL;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) goto done;
    abi_module_dict = PyModule_GetDict(abi_module);
    if (!abi_module_dict) goto done;
    get_item_ref_result = __Pyx_PyDict_GetItemRef(abi_module_dict, py_object_name, &cached_type);
    if (get_item_ref_result == 1) {
        if (__Pyx_VerifyCachedType(
              cached_type,
              object_name,
              spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    } else if (unlikely(get_item_ref_result == -1)) {
        goto bad;
    }
    cached_type = __Pyx_PyType_FromMetaclass(
        metaclass,
        CYTHON_USE_MODULE_STATE ? module : abi_module,
        spec, bases);
    if (unlikely(!cached_type)) goto bad;
    if (unlikely(__Pyx_fix_up_extension_type_from_spec(spec, (PyTypeObject *) cached_type) < 0)) goto bad;
    new_cached_type = __Pyx_PyDict_SetDefault(abi_module_dict, py_object_name, cached_type);
    if (unlikely(new_cached_type != cached_type)) {
        if (unlikely(!new_cached_type)) goto bad;
        Py_DECREF(cached_type);
        cached_type = new_cached_type;
        if (__Pyx_VerifyCachedType(
                cached_type,
                object_name,
                spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    } else {
        Py_DECREF(new_cached_type);
    }
done:
    Py_XDECREF(abi_module);
    Py_DECREF(py_object_name);
    assert(cached_type == NULL || PyType_Check(cached_type));
    return (PyTypeObject *) cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CommonTypesMetaclass (used by CythonFunctionShared) */
static PyObject* __pyx_CommonTypesMetaclass_get_module(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED void* context) {
    return PyUnicode_FromString(__PYX_ABI_MODULE_NAME);
}
#if __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject* __pyx_CommonTypesMetaclass_call(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED PyObject *args, CYTHON_UNUSED PyObject *kwds) {
    PyErr_SetString(PyExc_TypeError, "Cannot instantiate Cython internal types");
    return NULL;
}
static int __pyx_CommonTypesMetaclass_setattr(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED PyObject *attr, CYTHON_UNUSED PyObject *value) {
    PyErr_SetString(PyExc_TypeError, "Cython internal types are immutable");
    return -1;
}
#endif
static PyGetSetDef __pyx_CommonTypesMetaclass_getset[] = {
    {"__module__", __pyx_CommonTypesMetaclass_get_module, NULL, NULL, NULL},
    {0, 0, 0, 0, 0}
};
static PyType_Slot __pyx_CommonTypesMetaclass_slots[] = {
    {Py_tp_getset, (void *)__pyx_CommonTypesMetaclass_getset},
    #if __PYX_LIMITED_VERSION_HEX < 0x030A0000
    {Py_tp_call, (void*)__pyx_CommonTypesMetaclass_call},
    {Py_tp_new, (void*)__pyx_CommonTypesMetaclass_call},
    {Py_tp_setattro, (void*)__pyx_CommonTypesMetaclass_setattr},
    #endif
    {0, 0}
};
static PyType_Spec __pyx_CommonTypesMetaclass_spec = {
    __PYX_TYPE_MODULE_PREFIX "_common_types_metatype",
    0,
    0,
    Py_TPFLAGS_IMMUTABLETYPE |
    Py_TPFLAGS_DISALLOW_INSTANTIATION |
    Py_TPFLAGS_DEFAULT,
    __pyx_CommonTypesMetaclass_slots
};
static int __pyx_CommonTypesMetaclass_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    PyObject *bases = PyTuple_Pack(1, &PyType_Type);
    if (unlikely(!bases)) {
        return -1;
    }
    mstate->__pyx_CommonTypesMetaclassType = __Pyx_FetchCommonTypeFromSpec(NULL, module, &__pyx_CommonTypesMetaclass_spec, bases);
    Py_DECREF(bases);
    if (unlikely(mstate->__pyx_CommonTypesMetaclassType == NULL)) {
        return -1;
    }
    return 0;
}

/* PyMethodNew (used by CythonFunctionShared) */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    PyObject *result;
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    #if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    {
        PyObject *args[] = {func, self};
        result = PyObject_Vectorcall(__pyx_mstate_global->__Pyx_CachedMethodType, args, 2, NULL);
    }
    #else
    result = PyObject_CallFunctionObjArgs(__pyx_mstate_global->__Pyx_CachedMethodType, func, self, NULL);
    #endif
    return result;
}
#else
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    return PyMethod_New(func, self);
}
#endif

/* PyVectorcallFastCallDict (used by CythonFunctionShared) */
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static PyObject *__Pyx_PyVectorcall_FastCallDict_kw(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    PyObject *res = NULL;
    PyObject *kwnames;
    PyObject **newargs;
    PyObject **kwvalues;
    Py_ssize_t i;
    #if CYTHON_AVOID_BORROWED_REFS
    PyObject *pos;
    #else
    Py_ssize_t pos;
    #endif
    size_t j;
    PyObject *key, *value;
    unsigned long keys_are_strings;
    #if !CYTHON_ASSUME_SAFE_SIZE
    Py_ssize_t nkw = PyDict_Size(kw);
    if (unlikely(nkw == -1)) return NULL;
    #else
    Py_ssize_t nkw = PyDict_GET_SIZE(kw);
    #endif
    newargs = (PyObject **)PyMem_Malloc((nargs + (size_t)nkw) * sizeof(args[0]));
    if (unlikely(newargs == NULL)) {
        PyErr_NoMemory();
        return NULL;
    }
    for (j = 0; j < nargs; j++) newargs[j] = args[j];
    kwnames = PyTuple_New(nkw);
    if (unlikely(kwnames == NULL)) {
        PyMem_Free(newargs);
        return NULL;
    }
    kwvalues = newargs + nargs;
    pos = 0;
    i = 0;
    keys_are_strings = Py_TPFLAGS_UNICODE_SUBCLASS;
    while (__Pyx_PyDict_NextRef(kw, &pos, &key, &value)) {
        keys_are_strings &=
        #if CYTHON_COMPILING_IN_LIMITED_API
            PyType_GetFlags(Py_TYPE(key));
        #else
            Py_TYPE(key)->tp_flags;
        #endif
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(PyTuple_SetItem(kwnames, i, key) < 0)) goto cleanup;
        #else
        PyTuple_SET_ITEM(kwnames, i, key);
        #endif
        kwvalues[i] = value;
        i++;
    }
    if (unlikely(!keys_are_strings)) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        goto cleanup;
    }
    res = vc(func, newargs, nargs, kwnames);
cleanup:
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(pos);
    #endif
    Py_DECREF(kwnames);
    for (i = 0; i < nkw; i++)
        Py_DECREF(kwvalues[i]);
    PyMem_Free(newargs);
    return res;
}
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    Py_ssize_t kw_size =
        likely(kw == NULL) ?
        0 :
#if !CYTHON_ASSUME_SAFE_SIZE
        PyDict_Size(kw);
#else
        PyDict_GET_SIZE(kw);
#endif
    if (kw_size == 0) {
        return vc(func, args, nargs, NULL);
    }
#if !CYTHON_ASSUME_SAFE_SIZE
    else if (unlikely(kw_size == -1)) {
        return NULL;
    }
#endif
    return __Pyx_PyVectorcall_FastCallDict_kw(func, vc, args, nargs, kw);
}
#endif

/* CythonFunctionShared (used by CythonFunction) */
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunctionNoMethod(PyObject *func, void (*cfunc)(void)) {
    if (__Pyx_CyFunction_Check(func)) {
        return PyCFunction_GetFunction(((__pyx_CyFunctionObject*)func)->func) == (PyCFunction) cfunc;
    } else if (PyCFunction_Check(func)) {
        return PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void)) {
    if ((PyObject*)Py_TYPE(func) == __pyx_mstate_global->__Pyx_CachedMethodType) {
        int result;
        PyObject *newFunc = PyObject_GetAttr(func, __pyx_mstate_global->__pyx_n_u_func);
        if (unlikely(!newFunc)) {
            PyErr_Clear(); // It's only an optimization, so don't throw an error
            return 0;
        }
        result = __Pyx__IsSameCyOrCFunctionNoMethod(newFunc, cfunc);
        Py_DECREF(newFunc);
        return result;
    }
    return __Pyx__IsSameCyOrCFunctionNoMethod(func, cfunc);
}
#else
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void)) {
    if (PyMethod_Check(func)) {
        func = PyMethod_GET_FUNCTION(func);
    }
    return __Pyx_CyOrPyCFunction_Check(func) && __Pyx_CyOrPyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
}
#endif
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj) {
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    __Pyx_Py_XDECREF_SET(
        __Pyx_CyFunction_GetClassObj(f),
            ((classobj) ? __Pyx_NewRef(classobj) : NULL));
#else
    __Pyx_Py_XDECREF_SET(
        ((PyCMethodObject *) (f))->mm_class,
        (PyTypeObject*)((classobj) ? __Pyx_NewRef(classobj) : NULL));
#endif
}
static PyObject *
__Pyx_CyFunction_get_doc_locked(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_doc == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_doc = PyObject_GetAttrString(op->func, "__doc__");
        if (unlikely(!op->func_doc)) return NULL;
#else
        if (((PyCFunctionObject*)op)->m_ml->ml_doc) {
            op->func_doc = PyUnicode_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
#endif
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, void *closure) {
    PyObject *result;
    CYTHON_UNUSED_VAR(closure);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_doc_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_doc, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name_locked(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_name = PyObject_GetAttrString(op->func, "__name__");
#else
        op->func_name = PyUnicode_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, void *context)
{
    PyObject *result = NULL;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_name_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_name, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    PyObject *result;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    Py_INCREF(op->func_qualname);
    result = op->func_qualname;
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_qualname, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
#endif
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(op);
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = __Pyx_PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = __Pyx_PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyTuple_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__defaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->defaults_tuple, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = NULL;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_defaults_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__kwdefaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->defaults_kwdict, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_kwdefaults_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int __Pyx_CyFunction_set_annotate_in_dict_if_exists(PyObject *op_in, PyObject *value);
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value || value == Py_None) {
        value = NULL;
    } else if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_annotations, value);
    __Pyx_END_CRITICAL_SECTION();
    if (unlikely(__Pyx_CyFunction_set_annotate_in_dict_if_exists((PyObject*) op, Py_None) < 0)) return -1;
    return 0;
}
static int
__Pyx_CyFunction_get_dict_if_exists(PyObject *op_in, PyObject **dict) {
    /* Return 1 if the function dict exists, 0 otherwise.  This cannot fail:
     * _PyObject_GetDictPtr() may clear errors internally, but never reports them. */
#if CYTHON_COMPILING_IN_PYPY
    *dict = PyObject_GenericGetDict(op_in, NULL);
#elif CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX < 0x030C0000
    *dict = ((__pyx_CyFunctionObject*) op_in)->func_dict;
#else
    PyObject **dictptr = _PyObject_GetDictPtr(op_in);
    *dict = likely(dictptr) ? *dictptr : NULL;
#endif
    return *dict ? 1 : 0;
}
static int
__Pyx_CyFunction_get_annotate_from_dict_if_exists(PyObject *op_in, PyObject **annotate) {
    PyObject *dict;
    int dict_found;
    *annotate = NULL;
    dict_found = __Pyx_CyFunction_get_dict_if_exists(op_in, &dict);
    if (!dict_found) return 0;
    return __Pyx_PyDict_GetItemRef(dict, __pyx_mstate_global->__pyx_n_u_annotate, annotate);
}
static int
__Pyx_CyFunction_set_annotate_in_dict_if_exists(PyObject *op_in, PyObject *value) {
    PyObject *dict;
    int dict_found;
    dict_found = __Pyx_CyFunction_get_dict_if_exists(op_in, &dict);
    if (!dict_found) return 0;
    return PyDict_SetItem(dict, __pyx_mstate_global->__pyx_n_u_annotate, value);
}
static int
__Pyx_CyFunction_set_annotate_in_dict(PyObject *op_in, PyObject *value) {
    PyObject *dict;
    int result;
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    dict = __Pyx_CyFunction_get_dict((__pyx_CyFunctionObject*) op_in, NULL);
#else
    dict = PyObject_GenericGetDict(op_in, NULL);
#endif
    if (unlikely(!dict)) return -1;
    result = PyDict_SetItem(dict, __pyx_mstate_global->__pyx_n_u_annotate, value);
    Py_DECREF(dict);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_annotations_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_annotations(PyObject *op_in, void *context) {
    PyObject *annotate = NULL;
    PyObject *result = NULL;
    __pyx_CyFunctionObject *op = (__pyx_CyFunctionObject*) op_in;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op_in);
    result = __Pyx_XNewRef(op->func_annotations);
    __Pyx_END_CRITICAL_SECTION();
    if (result) return result;
    if (unlikely(__Pyx_CyFunction_get_annotate_from_dict_if_exists(op_in, &annotate) < 0)) return NULL;
    if (!annotate || annotate == Py_None) {
        Py_XDECREF(annotate);
        __Pyx_BEGIN_CRITICAL_SECTION(op_in);
        result = __Pyx_CyFunction_get_annotations_locked(op);
        __Pyx_END_CRITICAL_SECTION();
        return result;
    }
    PyObject *format = PyLong_FromLong(1L);  // annotationlib.Format.VALUE
    if (likely(format)) {
        result = __Pyx_PyObject_CallOneArg(annotate, format);
        Py_DECREF(format);
    }
    Py_DECREF(annotate);
    if (unlikely(!result)) return NULL;
    if (unlikely(!PyDict_Check(result))) {
        PyErr_SetString(PyExc_TypeError, "__annotate__ must return a dict");
        Py_DECREF(result);
        return NULL;
    }
    __Pyx_BEGIN_CRITICAL_SECTION(op_in);
    __Pyx_Py_XDECREF_SET(op->func_annotations, __Pyx_NewRef(result));
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static PyObject *__Pyx_CyFunction_annotate_impl(PyObject *self, PyObject *args) {
    CYTHON_UNUSED_VAR(args);
    if (unlikely(!self)) {
        PyErr_SetString(PyExc_SystemError, "cython __annotate__ called without 'self' argument");
    }
    Py_XINCREF(self);
    return self;
}
static PyMethodDef __Pyx_CyFunction_annotate_method = {
    "__annotate__",
    (PyCFunction)(void (*)(void))__Pyx_CyFunction_annotate_impl,
    METH_VARARGS,
    "Placeholder __annotate__ function to allow 'functools.wraps' to work "
    "on Cython functions."
};
static PyObject *
__Pyx_CyFunction_get_annotate(PyObject *op_in, void *context) {
    PyObject *annotate = NULL;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(__Pyx_CyFunction_get_annotate_from_dict_if_exists(op_in, &annotate) < 0)) return NULL;
    if (annotate) return annotate;
    PyObject *annotations = __Pyx_CyFunction_get_annotations(op_in, NULL);
    if (unlikely(!annotations)) return NULL;
    PyObject *method = PyCFunction_New(
        &__Pyx_CyFunction_annotate_method,
        annotations);
    Py_DECREF(annotations);
    return method;
}
static int
__Pyx_CyFunction_set_annotate(PyObject *op_in, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        PyErr_SetString(PyExc_TypeError, "__annotate__ cannot be deleted");
        return -1;
    }
    if (unlikely(value != Py_None && !PyCallable_Check(value))) {
        PyErr_SetString(PyExc_TypeError, "__annotate__ must be callable or None");
        return -1;
    }
    if (value != Py_None) {
        __pyx_CyFunctionObject *op = (__pyx_CyFunctionObject*) op_in;
        __Pyx_BEGIN_CRITICAL_SECTION(op_in);
        Py_CLEAR(op->func_annotations);
        __Pyx_END_CRITICAL_SECTION();
    }
    return __Pyx_CyFunction_set_annotate_in_dict(op_in, value);
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine_value(__pyx_CyFunctionObject *op) {
    int is_coroutine = op->flags & __Pyx_CYFUNCTION_COROUTINE;
    if (is_coroutine) {
        PyObject *is_coroutine_value, *module, *fromlist, *marker = __pyx_mstate_global->__pyx_n_u_is_coroutine;
        fromlist = PyList_New(1);
        if (unlikely(!fromlist)) return NULL;
        Py_INCREF(marker);
#if CYTHON_ASSUME_SAFE_MACROS
        PyList_SET_ITEM(fromlist, 0, marker);
#else
        if (unlikely(PyList_SetItem(fromlist, 0, marker) < 0)) {
            Py_DECREF(fromlist);
            return NULL;
        }
#endif
        module = PyImport_ImportModuleLevelObject(__pyx_mstate_global->__pyx_n_u_asyncio_coroutines, NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
        if (unlikely(!module)) goto ignore;
        is_coroutine_value = __Pyx_PyObject_GetAttrStr(module, marker);
        Py_DECREF(module);
        if (likely(is_coroutine_value)) {
            return is_coroutine_value;
        }
ignore:
        PyErr_Clear();
    }
    return __Pyx_PyBool_FromLong(is_coroutine);
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine(__pyx_CyFunctionObject *op, void *context) {
    PyObject *result;
    CYTHON_UNUSED_VAR(context);
    if (op->func_is_coroutine) {
        return __Pyx_NewRef(op->func_is_coroutine);
    }
    result = __Pyx_CyFunction_get_is_coroutine_value(op);
    if (unlikely(!result))
        return NULL;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    if (op->func_is_coroutine) {
        Py_DECREF(result);
        result = __Pyx_NewRef(op->func_is_coroutine);
    } else {
        op->func_is_coroutine = __Pyx_NewRef(result);
    }
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static void __Pyx_CyFunction_raise_argument_count_error(__pyx_CyFunctionObject *func, const char* message, Py_ssize_t size) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_name = __Pyx_CyFunction_get_name(func, NULL);
    if (!py_name) return;
    PyErr_Format(PyExc_TypeError,
        "%.200S() %s (%" CYTHON_FORMAT_SSIZE_T "d given)",
        py_name, message, size);
    Py_DECREF(py_name);
#else
    const char* name = ((PyCFunctionObject*)func)->m_ml->ml_name;
    PyErr_Format(PyExc_TypeError,
        "%.200s() %s (%" CYTHON_FORMAT_SSIZE_T "d given)",
        name, message, size);
#endif
}
static void __Pyx_CyFunction_raise_type_error(__pyx_CyFunctionObject *func, const char* message) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_name = __Pyx_CyFunction_get_name(func, NULL);
    if (!py_name) return;
    PyErr_Format(PyExc_TypeError,
        "%.200S() %s",
        py_name, message);
    Py_DECREF(py_name);
#else
    const char* name = ((PyCFunctionObject*)func)->m_ml->ml_name;
    PyErr_Format(PyExc_TypeError,
        "%.200s() %s",
        name, message);
#endif
}
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *
__Pyx_CyFunction_get_module(__pyx_CyFunctionObject *op, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_GetAttrString(op->func, "__module__");
}
static int
__Pyx_CyFunction_set_module(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_SetAttrString(op->func, "__module__", value);
}
#endif
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {"func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {"__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {"func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {"__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {"__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    {"func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)PyObject_GenericSetDict, 0, 0},
    {"__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)PyObject_GenericSetDict, 0, 0},
#else
    {"func_dict", (getter)PyObject_GenericGetDict, (setter)PyObject_GenericSetDict, 0, 0},
    {"__dict__", (getter)PyObject_GenericGetDict, (setter)PyObject_GenericSetDict, 0, 0},
#endif
    {"func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {"__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {"func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {"__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {"func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {"__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {"func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {"__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {"__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {"__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {"__annotate__", (getter)__Pyx_CyFunction_get_annotate, (setter)__Pyx_CyFunction_set_annotate, 0, 0},
    {"_is_coroutine", (getter)__Pyx_CyFunction_get_is_coroutine, 0, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", (getter)__Pyx_CyFunction_get_module, (setter)__Pyx_CyFunction_set_module, 0, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
#if !CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), 0, 0},
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    {"__dictoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_dict), READONLY, 0},
#endif
#if CYTHON_METH_FASTCALL
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__vectorcalloffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_vectorcall), READONLY, 0},
#else
    {"__vectorcalloffset__", T_PYSSIZET, offsetof(PyCFunctionObject, vectorcall), READONLY, 0},
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_weakreflist), READONLY, 0},
#else
    {"__weaklistoffset__", T_PYSSIZET, offsetof(PyCFunctionObject, m_weakreflist), READONLY, 0},
#endif
#endif
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, PyObject *args)
{
    PyObject *result = NULL;
    CYTHON_UNUSED_VAR(args);
    __Pyx_BEGIN_CRITICAL_SECTION(m);
    Py_INCREF(m->func_qualname);
    result = m->func_qualname;
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) (((PyCFunctionObject*)cyfunc)->m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
#if !CYTHON_COMPILING_IN_LIMITED_API
    PyCFunctionObject *cf = (PyCFunctionObject*) op;
#endif
    if (unlikely(op == NULL))
        return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    op->func = PyCFunction_NewEx(ml, (PyObject*)op, module);
    if (unlikely(!op->func)) return NULL;
#endif
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
#if !CYTHON_COMPILING_IN_LIMITED_API
    cf->m_ml = ml;
    cf->m_self = (PyObject *) op;
#endif
    Py_XINCREF(closure);
    op->func_closure = closure;
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_XINCREF(module);
    cf->m_module = module;
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_dict = NULL;
#endif
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_classobj = NULL;
#else
    ((PyCMethodObject*)op)->mm_class = NULL;
#endif
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    op->func_is_coroutine = NULL;
#if CYTHON_METH_FASTCALL
    switch (ml->ml_flags & (METH_VARARGS | METH_FASTCALL | METH_NOARGS | METH_O | METH_KEYWORDS | METH_METHOD)) {
    case METH_NOARGS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_NOARGS;
        break;
    case METH_O:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_O;
        break;
    case METH_METHOD | METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD;
        break;
    case METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS;
        break;
    case METH_VARARGS | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = NULL;
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        Py_DECREF(op);
        return NULL;
    }
#endif
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func);
#else
    Py_CLEAR(((PyCFunctionObject*)m)->m_module);
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func_dict);
#elif PY_VERSION_HEX < 0x030d0000
    _PyObject_ClearManagedDict((PyObject*)m);
#else
    PyObject_ClearManagedDict((PyObject*)m);
#endif
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(__Pyx_CyFunction_GetClassObj(m));
#else
    {
        PyObject *cls = (PyObject*) ((PyCMethodObject *) (m))->mm_class;
        ((PyCMethodObject *) (m))->mm_class = NULL;
        Py_XDECREF(cls);
    }
#endif
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    Py_CLEAR(m->func_is_coroutine);
    Py_CLEAR(m->defaults);
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    __Pyx_PyHeapTypeObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    {
        int e = __Pyx_call_type_traverse((PyObject*)m, 1, visit, arg);
        if (e) return e;
    }
    Py_VISIT(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func);
#else
    Py_VISIT(((PyCFunctionObject*)m)->m_module);
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func_dict);
#else
    {
        int e =
#if PY_VERSION_HEX < 0x030d0000
            _PyObject_VisitManagedDict
#else
            PyObject_VisitManagedDict
#endif
                ((PyObject*)m, visit, arg);
        if (e != 0) return e;
    }
#endif
    __Pyx_VISIT_CONST(m->func_name);
    __Pyx_VISIT_CONST(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    __Pyx_VISIT_CONST(m->func_code);
    Py_VISIT(__Pyx_CyFunction_GetClassObj(m));
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    Py_VISIT(m->func_annotations);
    Py_VISIT(m->func_is_coroutine);
    Py_VISIT(m->defaults);
    return 0;
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
    PyObject *repr;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    repr = PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
    __Pyx_END_CRITICAL_SECTION();
    return repr;
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *f = ((__pyx_CyFunctionObject*)func)->func;
    PyCFunction meth;
    int flags;
    meth = PyCFunction_GetFunction(f);
    if (unlikely(!meth)) return NULL;
    flags = PyCFunction_GetFlags(f);
    if (unlikely(flags < 0)) return NULL;
#else
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    int flags = f->m_ml->ml_flags;
#endif
    Py_ssize_t size;
    switch (flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void(*)(void))meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_SIZE
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 0))
                return (*meth)(self, NULL);
            __Pyx_CyFunction_raise_argument_count_error(
                (__pyx_CyFunctionObject*)func,
                "takes no arguments", size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_SIZE
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = __Pyx_PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            __Pyx_CyFunction_raise_argument_count_error(
                (__pyx_CyFunctionObject*)func,
                "takes exactly one argument", size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        return NULL;
    }
    __Pyx_CyFunction_raise_type_error(
        (__pyx_CyFunctionObject*)func, "takes no keyword arguments");
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *self, *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)func)->func);
    if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
    self = ((PyCFunctionObject*)func)->m_self;
#endif
    result = __Pyx_CyFunction_CallMethod(func, self, arg, kw);
    return result;
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
     __pyx_vectorcallfunc vc = __Pyx_CyFunction_func_vectorcall(cyfunc);
    if (vc) {
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
        return __Pyx_PyVectorcall_FastCallDict(func, vc, &PyTuple_GET_ITEM(args, 0), (size_t)PyTuple_GET_SIZE(args), kw);
#else
        (void) &__Pyx_PyVectorcall_FastCallDict;
        return PyVectorcall_Call(func, args, kw);
#endif
    }
#endif
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
#if CYTHON_ASSUME_SAFE_SIZE
        argc = PyTuple_GET_SIZE(args);
#else
        argc = PyTuple_Size(args);
        if (unlikely(argc < 0)) return NULL;
#endif
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static CYTHON_INLINE int __Pyx_CyFunction_Vectorcall_CheckArgs(__pyx_CyFunctionObject *cyfunc, Py_ssize_t nargs, PyObject *kwnames)
{
    int ret = 0;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        if (unlikely(nargs < 1)) {
            __Pyx_CyFunction_raise_type_error(
                cyfunc, "needs an argument");
            return -1;
        }
        ret = 1;
    }
    if (unlikely(kwnames) && unlikely(__Pyx_PyTuple_GET_SIZE(kwnames))) {
        __Pyx_CyFunction_raise_type_error(
            cyfunc, "takes no keyword arguments");
        return -1;
    }
    return ret;
}
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 0)) {
        __Pyx_CyFunction_raise_argument_count_error(
            cyfunc, "takes no arguments", nargs);
        return NULL;
    }
    return meth(self, NULL);
}
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 1)) {
        __Pyx_CyFunction_raise_argument_count_error(
            cyfunc, "takes exactly one argument", nargs);
        return NULL;
    }
    return meth(self, args[0]);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))meth)(self, args, nargs, kwnames);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyTypeObject *cls = (PyTypeObject *) __Pyx_CyFunction_GetClassObj(cyfunc);
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    #if PY_VERSION_HEX < 0x030e00A6
    size_t nargs_value = (size_t) nargs;
    #else
    Py_ssize_t nargs_value = nargs;
    #endif
    return ((__Pyx_PyCMethod)(void(*)(void))meth)(self, cls, args, nargs_value, kwnames);
}
#endif
static PyType_Slot __pyx_CyFunctionType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_CyFunction_dealloc},
    {Py_tp_repr, (void *)__Pyx_CyFunction_repr},
    {Py_tp_call, (void *)__Pyx_CyFunction_CallAsMethod},
    {Py_tp_traverse, (void *)__Pyx_CyFunction_traverse},
    {Py_tp_clear, (void *)__Pyx_CyFunction_clear},
    {Py_tp_methods, (void *)__pyx_CyFunction_methods},
    {Py_tp_members, (void *)__pyx_CyFunction_members},
    {Py_tp_getset, (void *)__pyx_CyFunction_getsets},
    {Py_tp_descr_get, (void *)__Pyx_PyMethod_New},
    {0, 0},
};
static PyType_Spec __pyx_CyFunctionType_spec = {
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if CYTHON_METH_FASTCALL
#if defined(Py_TPFLAGS_HAVE_VECTORCALL)
    Py_TPFLAGS_HAVE_VECTORCALL |
#elif defined(_Py_TPFLAGS_HAVE_VECTORCALL)
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
#endif // CYTHON_METH_FASTCALL
#if PY_VERSION_HEX >= 0x030C0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_TPFLAGS_MANAGED_DICT |
#endif
    Py_TPFLAGS_IMMUTABLETYPE | Py_TPFLAGS_DISALLOW_INSTANTIATION |
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    __pyx_CyFunctionType_slots
};
static int __pyx_CyFunction_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    mstate->__pyx_CyFunctionType = __Pyx_FetchCommonTypeFromSpec(
        mstate->__pyx_CommonTypesMetaclassType, module, &__pyx_CyFunctionType_spec, NULL);
    if (unlikely(mstate->__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_InitDefaults(PyObject *func, PyTypeObject *defaults_type) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_CallObject((PyObject*)defaults_type, NULL); // _PyObject_New(defaults_type);
    if (unlikely(!m->defaults))
        return NULL;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_mstate_global->__pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* CLineInTraceback (used by AddTraceback) */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
#define __Pyx_PyProbablyModule_GetDict(o) __Pyx_XNewRef(PyModule_GetDict(o))
#elif !CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyProbablyModule_GetDict(o) PyObject_GenericGetDict(o, NULL);
#else
PyObject* __Pyx_PyProbablyModule_GetDict(PyObject *o) {
    PyObject **dict_ptr = _PyObject_GetDictPtr(o);
    return dict_ptr ? __Pyx_XNewRef(*dict_ptr) : NULL;
}
#endif
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline = NULL;
    PyObject *ptype, *pvalue, *ptraceback;
    PyObject *cython_runtime_dict;
    CYTHON_MAYBE_UNUSED_VAR(tstate);
    if (unlikely(!__pyx_mstate_global->__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
    cython_runtime_dict = __Pyx_PyProbablyModule_GetDict(__pyx_mstate_global->__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, cython_runtime_dict,
            __Pyx_PyDict_SetDefault(cython_runtime_dict, __pyx_mstate_global->__pyx_n_u_cline_in_traceback, Py_False))
    }
    if (use_cline == NULL || use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    Py_XDECREF(use_cline);
    Py_XDECREF(cython_runtime_dict);
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache (used by AddTraceback) */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static __Pyx_CachedCodeObjectType *__pyx__find_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line) {
    __Pyx_CachedCodeObjectType* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!code_cache->entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
    if (unlikely(pos >= code_cache->count) || unlikely(code_cache->entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = code_cache->entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line) {
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
    (void)__pyx__find_code_object;
    return NULL; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just miss.
#else
    struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_nonatomic_int_type old_count = __pyx_atomic_incr_acq_rel(&code_cache->accessor_count);
    if (old_count < 0) {
        __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
        return NULL;
    }
#endif
    __Pyx_CachedCodeObjectType *result = __pyx__find_code_object(code_cache, code_line);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
#endif
    return result;
#endif
}
static void __pyx__insert_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line, __Pyx_CachedCodeObjectType* code_object)
{
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = code_cache->entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            code_cache->entries = entries;
            code_cache->max_count = 64;
            code_cache->count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
    if ((pos < code_cache->count) && unlikely(code_cache->entries[pos].code_line == code_line)) {
        __Pyx_CachedCodeObjectType* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_INCREF(code_object);
        Py_DECREF(tmp);
        return;
    }
    if (code_cache->count == code_cache->max_count) {
        int new_max = code_cache->max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            code_cache->entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        code_cache->entries = entries;
        code_cache->max_count = new_max;
    }
    for (i=code_cache->count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    code_cache->count++;
    Py_INCREF(code_object);
}
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object) {
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
    (void)__pyx__insert_code_object;
    return; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just fail.
#else
    struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_nonatomic_int_type expected = 0;
    if (!__pyx_atomic_int_cmp_exchange(&code_cache->accessor_count, &expected, INT_MIN)) {
        return;
    }
#endif
    __pyx__insert_code_object(code_cache, code_line, code_object);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_sub(&code_cache->accessor_count, INT_MIN);
#endif
#endif
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API && !defined(PYPY_VERSION)
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyCode_Replace_For_AddTraceback(PyObject *code, PyObject *scratch_dict,
                                                       PyObject *firstlineno, PyObject *name) {
    PyObject *replace = NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_firstlineno", firstlineno))) return NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_name", name))) return NULL;
    replace = PyObject_GetAttrString(code, "replace");
    if (likely(replace)) {
        PyObject *result = PyObject_Call(replace, __pyx_mstate_global->__pyx_empty_tuple, scratch_dict);
        Py_DECREF(replace);
        return result;
    }
    PyErr_Clear();
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyObject *code_object = NULL, *py_py_line = NULL, *py_funcname = NULL, *dict = NULL;
    PyObject *replace = NULL, *getframe = NULL, *frame = NULL;
    PyObject *exc_type, *exc_value, *exc_traceback;
    int success = 0;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(__Pyx_PyThreadState_Current, c_line);
    }
    PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
    code_object = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!code_object) {
        code_object = Py_CompileString("_getframe()", filename, Py_eval_input);
        if (unlikely(!code_object)) goto bad;
        py_py_line = PyLong_FromLong(py_line);
        if (unlikely(!py_py_line)) goto bad;
        if (c_line) {
            py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        } else {
            py_funcname = PyUnicode_FromString(funcname);
        }
        if (unlikely(!py_funcname)) goto bad;
        dict = PyDict_New();
        if (unlikely(!dict)) goto bad;
        {
            PyObject *old_code_object = code_object;
            code_object = __Pyx_PyCode_Replace_For_AddTraceback(code_object, dict, py_py_line, py_funcname);
            Py_DECREF(old_code_object);
        }
        if (unlikely(!code_object)) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, code_object);
    } else {
        dict = PyDict_New();
    }
    getframe = PySys_GetObject("_getframe");
    if (unlikely(!getframe)) goto bad;
    if (unlikely(PyDict_SetItemString(dict, "_getframe", getframe))) goto bad;
    frame = PyEval_EvalCode(code_object, dict, dict);
    if (unlikely(!frame) || frame == Py_None) goto bad;
    success = 1;
  bad:
    PyErr_Restore(exc_type, exc_value, exc_traceback);
    Py_XDECREF(code_object);
    Py_XDECREF(py_py_line);
    Py_XDECREF(py_funcname);
    Py_XDECREF(dict);
    Py_XDECREF(replace);
    if (success) {
        PyTraceBack_Here(
            (struct _frame*)frame);
    }
    Py_XDECREF(frame);
}
#else
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    if (c_line) {
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
    }
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    Py_XDECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_mstate_global->__pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}
#endif

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CheckUnpickleChecksum */
static void __Pyx_RaiseUnpickleChecksumError(long checksum, long checksum1, long checksum2, long checksum3, const char *members) {
    PyObject *pickle_module = PyImport_ImportModule("pickle");
    if (unlikely(!pickle_module)) return;
    PyObject *pickle_error = PyObject_GetAttrString(pickle_module, "PickleError");
    Py_DECREF(pickle_module);
    if (unlikely(!pickle_error)) return;
    if (checksum2 == checksum1) {
        PyErr_Format(pickle_error, "Incompatible checksums (0x%x vs (0x%x) = (%s))",
            checksum, checksum1, members);
    } else if (checksum3 == checksum2) {
        PyErr_Format(pickle_error, "Incompatible checksums (0x%x vs (0x%x, 0x%x) = (%s))",
            checksum, checksum1, checksum2, members);
    } else {
        PyErr_Format(pickle_error, "Incompatible checksums (0x%x vs (0x%x, 0x%x, 0x%x) = (%s))",
            checksum, checksum1, checksum2, checksum3, members);
    }
    Py_DECREF(pickle_error);
}
static int __Pyx_CheckUnpickleChecksum(long checksum, long checksum1, long checksum2, long checksum3, const char *members) {
    int found = 0;
    found |= checksum1 == checksum;
    found |= checksum2 == checksum;
    found |= checksum3 == checksum;
    if (likely(found))
        return 0;
    __Pyx_RaiseUnpickleChecksumError(checksum, checksum1, checksum2, checksum3, members);
    return -1;
}

/* Declarations */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = (float)(1.0) / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = (float)(1.0) / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if ((b.imag == 0) && (a.real >= 0)) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = (double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = (double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if ((b.imag == 0) && (a.real >= 0)) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_long_double_complex __pyx_t_long_double_complex_from_parts(long double x, long double y) {
      return ::std::complex< long double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_long_double_complex __pyx_t_long_double_complex_from_parts(long double x, long double y) {
      return x + y*(__pyx_t_long_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_long_double_complex __pyx_t_long_double_complex_from_parts(long double x, long double y) {
      __pyx_t_long_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
#else
    static CYTHON_INLINE int __Pyx_c_eq_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_sum_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
        __pyx_t_long_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_diff_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
        __pyx_t_long_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_prod_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
        __pyx_t_long_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_quot_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_long_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsl(b.real) >= fabsl(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_long_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                long double r = b.imag / b.real;
                long double s = (long double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_long_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            long double r = b.real / b.imag;
            long double s = (long double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_long_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_quot_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_long_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            long double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_long_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_neg_long__double(__pyx_t_long_double_complex a) {
        __pyx_t_long_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_long__double(__pyx_t_long_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_conj_long__double(__pyx_t_long_double_complex a) {
        __pyx_t_long_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE long double __Pyx_c_abs_long__double(__pyx_t_long_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtl(z.real*z.real + z.imag*z.imag);
          #else
            return hypotl(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_pow_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
            __pyx_t_long_double_complex z;
            long double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    long double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_long__double(a, a);
                    case 3:
                        z = __Pyx_c_prod_long__double(a, a);
                        return __Pyx_c_prod_long__double(z, a);
                    case 4:
                        z = __Pyx_c_prod_long__double(a, a);
                        return __Pyx_c_prod_long__double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if ((b.imag == 0) && (a.real >= 0)) {
                    z.real = powl(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2l(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_long__double(a);
                theta = atan2l(a.imag, a.real);
            }
            lnr = logl(r);
            z_r = expl(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosl(z_theta);
            z.imag = z_r * sinl(z_theta);
            return z;
        }
    #endif
#endif

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_hid_t(hid_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const hid_t neg_one = (hid_t) -1, const_zero = (hid_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(hid_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(hid_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(hid_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(hid_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(hid_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(hid_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(hid_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE hid_t __Pyx_PyLong_As_hid_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const hid_t neg_one = (hid_t) -1, const_zero = (hid_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        hid_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (hid_t) -1;
        val = __Pyx_PyLong_As_hid_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(hid_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(hid_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hid_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hid_t) >= 2 * PyLong_SHIFT)) {
                            return (hid_t) (((((hid_t)digits[1]) << PyLong_SHIFT) | (hid_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(hid_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hid_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hid_t) >= 3 * PyLong_SHIFT)) {
                            return (hid_t) (((((((hid_t)digits[2]) << PyLong_SHIFT) | (hid_t)digits[1]) << PyLong_SHIFT) | (hid_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(hid_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hid_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hid_t) >= 4 * PyLong_SHIFT)) {
                            return (hid_t) (((((((((hid_t)digits[3]) << PyLong_SHIFT) | (hid_t)digits[2]) << PyLong_SHIFT) | (hid_t)digits[1]) << PyLong_SHIFT) | (hid_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (hid_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(hid_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(hid_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(hid_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(hid_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(hid_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(hid_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hid_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hid_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (hid_t) (((hid_t)-1)*(((((hid_t)digits[1]) << PyLong_SHIFT) | (hid_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(hid_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hid_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hid_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (hid_t) ((((((hid_t)digits[1]) << PyLong_SHIFT) | (hid_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(hid_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hid_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hid_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (hid_t) (((hid_t)-1)*(((((((hid_t)digits[2]) << PyLong_SHIFT) | (hid_t)digits[1]) << PyLong_SHIFT) | (hid_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(hid_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hid_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hid_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (hid_t) ((((((((hid_t)digits[2]) << PyLong_SHIFT) | (hid_t)digits[1]) << PyLong_SHIFT) | (hid_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(hid_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hid_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hid_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (hid_t) (((hid_t)-1)*(((((((((hid_t)digits[3]) << PyLong_SHIFT) | (hid_t)digits[2]) << PyLong_SHIFT) | (hid_t)digits[1]) << PyLong_SHIFT) | (hid_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(hid_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hid_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hid_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (hid_t) ((((((((((hid_t)digits[3]) << PyLong_SHIFT) | (hid_t)digits[2]) << PyLong_SHIFT) | (hid_t)digits[1]) << PyLong_SHIFT) | (hid_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(hid_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(hid_t, long, PyLong_AsLong(x))
        } else if ((sizeof(hid_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(hid_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        hid_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (hid_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (hid_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (hid_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (hid_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(hid_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((hid_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(hid_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((hid_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((hid_t) 1) << (sizeof(hid_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (hid_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to hid_t");
    return (hid_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to hid_t");
    return (hid_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(int));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE uint32_t __Pyx_PyLong_As_uint32_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint32_t neg_one = (uint32_t) -1, const_zero = (uint32_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        uint32_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (uint32_t) -1;
        val = __Pyx_PyLong_As_uint32_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(uint32_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(uint32_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) >= 2 * PyLong_SHIFT)) {
                            return (uint32_t) (((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(uint32_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) >= 3 * PyLong_SHIFT)) {
                            return (uint32_t) (((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(uint32_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) >= 4 * PyLong_SHIFT)) {
                            return (uint32_t) (((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (uint32_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(uint32_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(uint32_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(uint32_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(uint32_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (uint32_t) (((uint32_t)-1)*(((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(uint32_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (uint32_t) ((((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (uint32_t) (((uint32_t)-1)*(((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(uint32_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (uint32_t) ((((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (uint32_t) (((uint32_t)-1)*(((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(uint32_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (uint32_t) ((((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(uint32_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint32_t, long, PyLong_AsLong(x))
        } else if ((sizeof(uint32_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint32_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        uint32_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (uint32_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (uint32_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (uint32_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (uint32_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(uint32_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((uint32_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(uint32_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((uint32_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((uint32_t) 1) << (sizeof(uint32_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (uint32_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint32_t");
    return (uint32_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint32_t");
    return (uint32_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE hsize_t __Pyx_PyLong_As_hsize_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const hsize_t neg_one = (hsize_t) -1, const_zero = (hsize_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        hsize_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (hsize_t) -1;
        val = __Pyx_PyLong_As_hsize_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(hsize_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(hsize_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hsize_t) >= 2 * PyLong_SHIFT)) {
                            return (hsize_t) (((((hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(hsize_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hsize_t) >= 3 * PyLong_SHIFT)) {
                            return (hsize_t) (((((((hsize_t)digits[2]) << PyLong_SHIFT) | (hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(hsize_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hsize_t) >= 4 * PyLong_SHIFT)) {
                            return (hsize_t) (((((((((hsize_t)digits[3]) << PyLong_SHIFT) | (hsize_t)digits[2]) << PyLong_SHIFT) | (hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (hsize_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(hsize_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(hsize_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(hsize_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(hsize_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(hsize_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(hsize_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hsize_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (hsize_t) (((hsize_t)-1)*(((((hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(hsize_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hsize_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (hsize_t) ((((((hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(hsize_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hsize_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (hsize_t) (((hsize_t)-1)*(((((((hsize_t)digits[2]) << PyLong_SHIFT) | (hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(hsize_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hsize_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (hsize_t) ((((((((hsize_t)digits[2]) << PyLong_SHIFT) | (hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(hsize_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hsize_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (hsize_t) (((hsize_t)-1)*(((((((((hsize_t)digits[3]) << PyLong_SHIFT) | (hsize_t)digits[2]) << PyLong_SHIFT) | (hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(hsize_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hsize_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hsize_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (hsize_t) ((((((((((hsize_t)digits[3]) << PyLong_SHIFT) | (hsize_t)digits[2]) << PyLong_SHIFT) | (hsize_t)digits[1]) << PyLong_SHIFT) | (hsize_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(hsize_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(hsize_t, long, PyLong_AsLong(x))
        } else if ((sizeof(hsize_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(hsize_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        hsize_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (hsize_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (hsize_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (hsize_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (hsize_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(hsize_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((hsize_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(hsize_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((hsize_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((hsize_t) 1) << (sizeof(hsize_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (hsize_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to hsize_t");
    return (hsize_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to hsize_t");
    return (hsize_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyLong_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 2 * PyLong_SHIFT)) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 3 * PyLong_SHIFT)) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 4 * PyLong_SHIFT)) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (int) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(int) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(int) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(int) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(int) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
        } else if ((sizeof(int) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        int val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (int) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (int) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (int) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (int) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(int) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((int) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(int) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((int) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((int) 1) << (sizeof(int) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (int) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyLong_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 2 * PyLong_SHIFT)) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 3 * PyLong_SHIFT)) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 4 * PyLong_SHIFT)) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (long) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(long) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(long) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(long) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(long) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
        } else if ((sizeof(long) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        long val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (long) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (long) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (long) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (long) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(long) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((long) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(long) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((long) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((long) 1) << (sizeof(long) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (long) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_hsize_t(hsize_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const hsize_t neg_one = (hsize_t) -1, const_zero = (hsize_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(hsize_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(hsize_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(hsize_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(hsize_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(hsize_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(hsize_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(hsize_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyLong_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyLong_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(size_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(size_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 2 * PyLong_SHIFT)) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(size_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 3 * PyLong_SHIFT)) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(size_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 4 * PyLong_SHIFT)) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (size_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(size_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(size_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(size_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(size_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(size_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(size_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
        } else if ((sizeof(size_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        size_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (size_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (size_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (size_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (size_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(size_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((size_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(size_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((size_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((size_t) 1) << (sizeof(size_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (size_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE hbool_t __Pyx_PyLong_As_hbool_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const hbool_t neg_one = (hbool_t) -1, const_zero = (hbool_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        hbool_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (hbool_t) -1;
        val = __Pyx_PyLong_As_hbool_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(hbool_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(hbool_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hbool_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hbool_t) >= 2 * PyLong_SHIFT)) {
                            return (hbool_t) (((((hbool_t)digits[1]) << PyLong_SHIFT) | (hbool_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(hbool_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hbool_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hbool_t) >= 3 * PyLong_SHIFT)) {
                            return (hbool_t) (((((((hbool_t)digits[2]) << PyLong_SHIFT) | (hbool_t)digits[1]) << PyLong_SHIFT) | (hbool_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(hbool_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hbool_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hbool_t) >= 4 * PyLong_SHIFT)) {
                            return (hbool_t) (((((((((hbool_t)digits[3]) << PyLong_SHIFT) | (hbool_t)digits[2]) << PyLong_SHIFT) | (hbool_t)digits[1]) << PyLong_SHIFT) | (hbool_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (hbool_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(hbool_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(hbool_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(hbool_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(hbool_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(hbool_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(hbool_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hbool_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hbool_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (hbool_t) (((hbool_t)-1)*(((((hbool_t)digits[1]) << PyLong_SHIFT) | (hbool_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(hbool_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hbool_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hbool_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (hbool_t) ((((((hbool_t)digits[1]) << PyLong_SHIFT) | (hbool_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(hbool_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hbool_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hbool_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (hbool_t) (((hbool_t)-1)*(((((((hbool_t)digits[2]) << PyLong_SHIFT) | (hbool_t)digits[1]) << PyLong_SHIFT) | (hbool_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(hbool_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hbool_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hbool_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (hbool_t) ((((((((hbool_t)digits[2]) << PyLong_SHIFT) | (hbool_t)digits[1]) << PyLong_SHIFT) | (hbool_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(hbool_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hbool_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hbool_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (hbool_t) (((hbool_t)-1)*(((((((((hbool_t)digits[3]) << PyLong_SHIFT) | (hbool_t)digits[2]) << PyLong_SHIFT) | (hbool_t)digits[1]) << PyLong_SHIFT) | (hbool_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(hbool_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(hbool_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(hbool_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (hbool_t) ((((((((((hbool_t)digits[3]) << PyLong_SHIFT) | (hbool_t)digits[2]) << PyLong_SHIFT) | (hbool_t)digits[1]) << PyLong_SHIFT) | (hbool_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(hbool_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(hbool_t, long, PyLong_AsLong(x))
        } else if ((sizeof(hbool_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(hbool_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        hbool_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (hbool_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (hbool_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (hbool_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (hbool_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(hbool_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((hbool_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(hbool_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((hbool_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((hbool_t) 1) << (sizeof(hbool_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (hbool_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to hbool_t");
    return (hbool_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to hbool_t");
    return (hbool_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_uintptr_t(uintptr_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uintptr_t neg_one = (uintptr_t) -1, const_zero = (uintptr_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uintptr_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(uintptr_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(uintptr_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uintptr_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(uintptr_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(uintptr_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(uintptr_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE enum H5F_scope_t __Pyx_PyLong_As_enum__H5F_scope_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum H5F_scope_t neg_one = (enum H5F_scope_t) -1, const_zero = (enum H5F_scope_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        enum H5F_scope_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (enum H5F_scope_t) -1;
        val = __Pyx_PyLong_As_enum__H5F_scope_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(enum H5F_scope_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(enum H5F_scope_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(enum H5F_scope_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(enum H5F_scope_t) >= 2 * PyLong_SHIFT)) {
                            return (enum H5F_scope_t) (((((enum H5F_scope_t)digits[1]) << PyLong_SHIFT) | (enum H5F_scope_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(enum H5F_scope_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(enum H5F_scope_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(enum H5F_scope_t) >= 3 * PyLong_SHIFT)) {
                            return (enum H5F_scope_t) (((((((enum H5F_scope_t)digits[2]) << PyLong_SHIFT) | (enum H5F_scope_t)digits[1]) << PyLong_SHIFT) | (enum H5F_scope_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(enum H5F_scope_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(enum H5F_scope_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(enum H5F_scope_t) >= 4 * PyLong_SHIFT)) {
                            return (enum H5F_scope_t) (((((((((enum H5F_scope_t)digits[3]) << PyLong_SHIFT) | (enum H5F_scope_t)digits[2]) << PyLong_SHIFT) | (enum H5F_scope_t)digits[1]) << PyLong_SHIFT) | (enum H5F_scope_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (enum H5F_scope_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(enum H5F_scope_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(enum H5F_scope_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(enum H5F_scope_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(enum H5F_scope_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(enum H5F_scope_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(enum H5F_scope_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(enum H5F_scope_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(enum H5F_scope_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (enum H5F_scope_t) (((enum H5F_scope_t)-1)*(((((enum H5F_scope_t)digits[1]) << PyLong_SHIFT) | (enum H5F_scope_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(enum H5F_scope_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(enum H5F_scope_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(enum H5F_scope_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (enum H5F_scope_t) ((((((enum H5F_scope_t)digits[1]) << PyLong_SHIFT) | (enum H5F_scope_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(enum H5F_scope_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(enum H5F_scope_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(enum H5F_scope_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (enum H5F_scope_t) (((enum H5F_scope_t)-1)*(((((((enum H5F_scope_t)digits[2]) << PyLong_SHIFT) | (enum H5F_scope_t)digits[1]) << PyLong_SHIFT) | (enum H5F_scope_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(enum H5F_scope_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(enum H5F_scope_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(enum H5F_scope_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (enum H5F_scope_t) ((((((((enum H5F_scope_t)digits[2]) << PyLong_SHIFT) | (enum H5F_scope_t)digits[1]) << PyLong_SHIFT) | (enum H5F_scope_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(enum H5F_scope_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(enum H5F_scope_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(enum H5F_scope_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (enum H5F_scope_t) (((enum H5F_scope_t)-1)*(((((((((enum H5F_scope_t)digits[3]) << PyLong_SHIFT) | (enum H5F_scope_t)digits[2]) << PyLong_SHIFT) | (enum H5F_scope_t)digits[1]) << PyLong_SHIFT) | (enum H5F_scope_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(enum H5F_scope_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(enum H5F_scope_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(enum H5F_scope_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (enum H5F_scope_t) ((((((((((enum H5F_scope_t)digits[3]) << PyLong_SHIFT) | (enum H5F_scope_t)digits[2]) << PyLong_SHIFT) | (enum H5F_scope_t)digits[1]) << PyLong_SHIFT) | (enum H5F_scope_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(enum H5F_scope_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(enum H5F_scope_t, long, PyLong_AsLong(x))
        } else if ((sizeof(enum H5F_scope_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(enum H5F_scope_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        enum H5F_scope_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyErr_SetString(PyExc_RuntimeError,
                        "_PyLong_AsByteArray() or PyLong_AsNativeBytes() not available, cannot convert large enums");
        val = (enum H5F_scope_t) -1;
#endif
        if (unlikely(ret))
            return (enum H5F_scope_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to enum H5F_scope_t");
    return (enum H5F_scope_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to enum H5F_scope_t");
    return (enum H5F_scope_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_htri_t(htri_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const htri_t neg_one = (htri_t) -1, const_zero = (htri_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(htri_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(htri_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(htri_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(htri_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(htri_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(htri_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(htri_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_haddr_t(haddr_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const haddr_t neg_one = (haddr_t) -1, const_zero = (haddr_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(haddr_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(haddr_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(haddr_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(haddr_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(haddr_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(haddr_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(haddr_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_unsigned_int(unsigned int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned int) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(unsigned int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(unsigned int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned int) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(unsigned int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned int),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(unsigned int));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_time_t(time_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const time_t neg_one = (time_t) -1, const_zero = (time_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(time_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(time_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(time_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(time_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(time_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(time_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(time_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_hbool_t(hbool_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const hbool_t neg_one = (hbool_t) -1, const_zero = (hbool_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(hbool_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(hbool_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(hbool_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(hbool_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(hbool_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(hbool_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(hbool_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_npy_intp(npy_intp value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const npy_intp neg_one = (npy_intp) -1, const_zero = (npy_intp) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(npy_intp) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(npy_intp) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(npy_intp) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(npy_intp) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(npy_intp) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(npy_intp),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(npy_intp));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE npy_intp __Pyx_PyLong_As_npy_intp(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const npy_intp neg_one = (npy_intp) -1, const_zero = (npy_intp) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        npy_intp val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (npy_intp) -1;
        val = __Pyx_PyLong_As_npy_intp(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(npy_intp, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(npy_intp) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(npy_intp, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(npy_intp) >= 2 * PyLong_SHIFT)) {
                            return (npy_intp) (((((npy_intp)digits[1]) << PyLong_SHIFT) | (npy_intp)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(npy_intp) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(npy_intp, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(npy_intp) >= 3 * PyLong_SHIFT)) {
                            return (npy_intp) (((((((npy_intp)digits[2]) << PyLong_SHIFT) | (npy_intp)digits[1]) << PyLong_SHIFT) | (npy_intp)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(npy_intp) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(npy_intp, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(npy_intp) >= 4 * PyLong_SHIFT)) {
                            return (npy_intp) (((((((((npy_intp)digits[3]) << PyLong_SHIFT) | (npy_intp)digits[2]) << PyLong_SHIFT) | (npy_intp)digits[1]) << PyLong_SHIFT) | (npy_intp)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (npy_intp) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(npy_intp) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(npy_intp, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(npy_intp) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(npy_intp, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(npy_intp, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(npy_intp) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(npy_intp, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(npy_intp) - 1 > 2 * PyLong_SHIFT)) {
                            return (npy_intp) (((npy_intp)-1)*(((((npy_intp)digits[1]) << PyLong_SHIFT) | (npy_intp)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(npy_intp) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(npy_intp, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(npy_intp) - 1 > 2 * PyLong_SHIFT)) {
                            return (npy_intp) ((((((npy_intp)digits[1]) << PyLong_SHIFT) | (npy_intp)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(npy_intp) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(npy_intp, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(npy_intp) - 1 > 3 * PyLong_SHIFT)) {
                            return (npy_intp) (((npy_intp)-1)*(((((((npy_intp)digits[2]) << PyLong_SHIFT) | (npy_intp)digits[1]) << PyLong_SHIFT) | (npy_intp)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(npy_intp) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(npy_intp, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(npy_intp) - 1 > 3 * PyLong_SHIFT)) {
                            return (npy_intp) ((((((((npy_intp)digits[2]) << PyLong_SHIFT) | (npy_intp)digits[1]) << PyLong_SHIFT) | (npy_intp)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(npy_intp) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(npy_intp, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(npy_intp) - 1 > 4 * PyLong_SHIFT)) {
                            return (npy_intp) (((npy_intp)-1)*(((((((((npy_intp)digits[3]) << PyLong_SHIFT) | (npy_intp)digits[2]) << PyLong_SHIFT) | (npy_intp)digits[1]) << PyLong_SHIFT) | (npy_intp)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(npy_intp) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(npy_intp, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(npy_intp) - 1 > 4 * PyLong_SHIFT)) {
                            return (npy_intp) ((((((((((npy_intp)digits[3]) << PyLong_SHIFT) | (npy_intp)digits[2]) << PyLong_SHIFT) | (npy_intp)digits[1]) << PyLong_SHIFT) | (npy_intp)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(npy_intp) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(npy_intp, long, PyLong_AsLong(x))
        } else if ((sizeof(npy_intp) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(npy_intp, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        npy_intp val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (npy_intp) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (npy_intp) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (npy_intp) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (npy_intp) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(npy_intp) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((npy_intp) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(npy_intp) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((npy_intp) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((npy_intp) 1) << (sizeof(npy_intp) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (npy_intp) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to npy_intp");
    return (npy_intp) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to npy_intp");
    return (npy_intp) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(long));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_enum__H5T_order_t(enum H5T_order_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum H5T_order_t neg_one = (enum H5T_order_t) -1, const_zero = (enum H5T_order_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum H5T_order_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(enum H5T_order_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(enum H5T_order_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum H5T_order_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(enum H5T_order_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(enum H5T_order_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(enum H5T_order_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_herr_t(herr_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const herr_t neg_one = (herr_t) -1, const_zero = (herr_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(herr_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(herr_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(herr_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(herr_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(herr_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(herr_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(herr_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_enum__H5S_seloper_t(enum H5S_seloper_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum H5S_seloper_t neg_one = (enum H5S_seloper_t) -1, const_zero = (enum H5S_seloper_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum H5S_seloper_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(enum H5S_seloper_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(enum H5S_seloper_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum H5S_seloper_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(enum H5S_seloper_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(enum H5S_seloper_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(enum H5S_seloper_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_PY_LONG_LONG(PY_LONG_LONG value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const PY_LONG_LONG neg_one = (PY_LONG_LONG) -1, const_zero = (PY_LONG_LONG) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(PY_LONG_LONG) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(PY_LONG_LONG) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(PY_LONG_LONG) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(PY_LONG_LONG) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(PY_LONG_LONG) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(PY_LONG_LONG),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(PY_LONG_LONG));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* UpdateUnpickledDict */
static int __Pyx__UpdateUnpickledDict(PyObject *obj, PyObject *state, Py_ssize_t index) {
    PyObject *state_dict = __Pyx_PySequence_ITEM(state, index);
    if (unlikely(!state_dict)) {
        return -1;
    }
    int non_empty = PyObject_IsTrue(state_dict);
    if (non_empty == 0) {
        Py_DECREF(state_dict);
        return 0;
    } else if (unlikely(non_empty == -1)) {
        return -1;
    }
    PyObject *dict;
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    dict = PyObject_GetAttrString(obj, "__dict__");
    #else
    dict = PyObject_GenericGetDict(obj, NULL);
    #endif
    if (unlikely(!dict)) {
        Py_DECREF(state_dict);
        return -1;
    }
    int result;
    if (likely(PyDict_CheckExact(dict))) {
        result = PyDict_Update(dict, state_dict);
    } else {
        PyObject *obj_result = __Pyx_PyObject_CallMethod1(dict, __pyx_mstate_global->__pyx_n_u_update, state_dict);
        if (likely(obj_result)) {
            Py_DECREF(obj_result);
            result = 0;
        } else {
            result = -1;
        }
    }
    Py_DECREF(state_dict);
    Py_DECREF(dict);
    return result;
}
static int __Pyx_UpdateUnpickledDict(PyObject *obj, PyObject *state, Py_ssize_t index) {
    Py_ssize_t state_size = __Pyx_PyTuple_GET_SIZE(state);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(state_size == -1)) return -1;
    #endif
    if (state_size <= index) {
        return 0;
    }
    return __Pyx__UpdateUnpickledDict(obj, state, index);
}

/* FormatTypeName */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
static __Pyx_TypeName
__Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp)
{
    PyObject *module = NULL, *name = NULL, *result = NULL;
    #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
    name = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_mstate_global->__pyx_n_u_qualname);
    #else
    name = PyType_GetQualName(tp);
    #endif
    if (unlikely(name == NULL) || unlikely(!PyUnicode_Check(name))) goto bad;
    module = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_mstate_global->__pyx_n_u_module);
    if (unlikely(module == NULL) || unlikely(!PyUnicode_Check(module))) goto bad;
    if (PyUnicode_CompareWithASCIIString(module, "builtins") == 0) {
        result = name;
        name = NULL;
        goto done;
    }
    result = PyUnicode_FromFormat("%U.%U", module, name);
    if (unlikely(result == NULL)) goto bad;
  done:
    Py_XDECREF(name);
    Py_XDECREF(module);
    return result;
  bad:
    PyErr_Clear();
    if (name) {
        result = name;
        name = NULL;
    } else {
        result = __Pyx_NewRef(__pyx_mstate_global->__pyx_kp_u__6);
    }
    goto done;
}
#endif

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = __Pyx_PyType_GetSlot(a, tp_base, PyTypeObject*);
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (cls == a || cls == b) return 1;
    mro = cls->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            PyObject *base = PyTuple_GET_ITEM(mro, i);
            if (base == (PyObject *)a || base == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(cls, a) || __Pyx_InBases(cls, b);
}
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    if (exc_type1) {
        return __Pyx_IsAnySubtype2((PyTypeObject*)err, (PyTypeObject*)exc_type1, (PyTypeObject*)exc_type2);
    } else {
        return __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
}
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* GetRuntimeVersion */
#if __PYX_LIMITED_VERSION_HEX < 0x030b0000
void __Pyx_init_runtime_version(void) {
    if (__Pyx_cached_runtime_version == 0) {
        const char* rt_version = Py_GetVersion();
        unsigned long version = 0;
        unsigned long factor = 0x01000000UL;
        unsigned int digit = 0;
        int i = 0;
        while (factor) {
            while ('0' <= rt_version[i] && rt_version[i] <= '9') {
                digit = digit * 10 + (unsigned int) (rt_version[i] - '0');
                ++i;
            }
            version += factor * digit;
            if (rt_version[i] != '.')
                break;
            digit = 0;
            factor >>= 8;
            ++i;
        }
        __Pyx_cached_runtime_version = version;
    }
}
#endif
static unsigned long __Pyx_get_runtime_version(void) {
#if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
    return Py_Version & ~0xFFUL;
#else
    return __Pyx_cached_runtime_version;
#endif
}

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer) {
    const unsigned long MAJOR_MINOR = 0xFFFF0000UL;
    if ((rt_version & MAJOR_MINOR) == (ct_version & MAJOR_MINOR))
        return 0;
    if (likely(allow_newer && (rt_version & MAJOR_MINOR) > (ct_version & MAJOR_MINOR)))
        return 1;
    {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compile time Python version %d.%d "
                      "of module '%.100s' "
                      "%s "
                      "runtime version %d.%d",
                       (int) (ct_version >> 24), (int) ((ct_version >> 16) & 0xFF),
                       __Pyx_MODULE_NAME,
                       (allow_newer) ? "was newer than" : "does not match",
                       (int) (rt_version >> 24), (int) ((rt_version >> 16) & 0xFF)
       );
        return PyErr_WarnEx(NULL, message, 1);
    }
}

/* NewCodeObj */
#if CYTHON_COMPILING_IN_LIMITED_API
    static PyObject* __Pyx__PyCode_New(int a, int p, int k, int l, int s, int f,
                                       PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                       PyObject *fv, PyObject *cell, PyObject* fn,
                                       PyObject *name, int fline, PyObject *lnos) {
        PyObject *exception_table = NULL;
        PyObject *types_module=NULL, *code_type=NULL, *result=NULL;
        #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
        PyObject *version_info;
        PyObject *py_minor_version = NULL;
        #endif
        long minor_version = 0;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        #if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
        minor_version = 11;
        #else
        if (!(version_info = PySys_GetObject("version_info"))) goto end;
        if (!(py_minor_version = PySequence_GetItem(version_info, 1))) goto end;
        minor_version = PyLong_AsLong(py_minor_version);
        Py_DECREF(py_minor_version);
        if (minor_version == -1 && PyErr_Occurred()) goto end;
        #endif
        if (!(types_module = PyImport_ImportModule("types"))) goto end;
        if (!(code_type = PyObject_GetAttrString(types_module, "CodeType"))) goto end;
        if (minor_version <= 7) {
            (void)p;
            result = PyObject_CallFunction(code_type, "iiiiiOOOOOOiOOO", a, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else if (minor_version <= 10) {
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOiOOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else {
            if (!(exception_table = PyBytes_FromStringAndSize(NULL, 0))) goto end;
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOOiOOOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, name, fline, lnos, exception_table, fv, cell);
        }
    end:
        Py_XDECREF(code_type);
        Py_XDECREF(exception_table);
        Py_XDECREF(types_module);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return result;
    }
#elif PY_VERSION_HEX >= 0x030B0000
  static PyCodeObject* __Pyx__PyCode_New(int a, int p, int k, int l, int s, int f,
                                         PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                         PyObject *fv, PyObject *cell, PyObject* fn,
                                         PyObject *name, int fline, PyObject *lnos) {
    PyCodeObject *result;
    result =
      #if PY_VERSION_HEX >= 0x030C0000
        PyUnstable_Code_NewWithPosOnlyArgs
      #else
        PyCode_NewWithPosOnlyArgs
      #endif
        (a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, name, fline, lnos, __pyx_mstate_global->__pyx_empty_bytes);
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030c00A1
    if (likely(result))
        result->_co_firsttraceable = 0;
    #endif
    return result;
  }
#elif !CYTHON_COMPILING_IN_PYPY
  #define __Pyx__PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_NewWithPosOnlyArgs(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx__PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
static PyObject* __Pyx_PyCode_New(
        const __Pyx_PyCode_New_function_description descr,
        PyObject * const *varnames,
        PyObject *filename,
        PyObject *funcname,
        PyObject *line_table,
        PyObject *tuple_dedup_map
) {
    PyObject *code_obj = NULL, *varnames_tuple_dedup = NULL, *code_bytes = NULL;
    Py_ssize_t var_count = (Py_ssize_t) descr.nlocals;
    PyObject *varnames_tuple = PyTuple_New(var_count);
    if (unlikely(!varnames_tuple)) return NULL;
    for (Py_ssize_t i=0; i < var_count; i++) {
        Py_INCREF(varnames[i]);
        if (__Pyx_PyTuple_SET_ITEM(varnames_tuple, i, varnames[i]) != (0)) goto done;
    }
    #if CYTHON_COMPILING_IN_LIMITED_API
    varnames_tuple_dedup = PyDict_GetItem(tuple_dedup_map, varnames_tuple);
    if (!varnames_tuple_dedup) {
        if (unlikely(PyDict_SetItem(tuple_dedup_map, varnames_tuple, varnames_tuple) < 0)) goto done;
        varnames_tuple_dedup = varnames_tuple;
    }
    #else
    varnames_tuple_dedup = PyDict_SetDefault(tuple_dedup_map, varnames_tuple, varnames_tuple);
    if (unlikely(!varnames_tuple_dedup)) goto done;
    #endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(varnames_tuple_dedup);
    #endif
    if (__PYX_LIMITED_VERSION_HEX >= (0x030b0000) && line_table != NULL && !CYTHON_COMPILING_IN_GRAAL) {
        Py_ssize_t line_table_length = __Pyx_PyBytes_GET_SIZE(line_table);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(line_table_length == -1)) goto done;
        #endif
        Py_ssize_t code_len = (line_table_length * 2 + 4) & ~3LL;
        code_bytes = PyBytes_FromStringAndSize(NULL, code_len);
        if (unlikely(!code_bytes)) goto done;
        char* c_code_bytes = PyBytes_AsString(code_bytes);
        if (unlikely(!c_code_bytes)) goto done;
        memset(c_code_bytes, 0, (size_t) code_len);
    }
    code_obj = (PyObject*) __Pyx__PyCode_New(
        (int) descr.argcount,
        (int) descr.num_posonly_args,
        (int) descr.num_kwonly_args,
        (int) descr.nlocals,
        0,
        (int) descr.flags,
        code_bytes ? code_bytes : __pyx_mstate_global->__pyx_empty_bytes,
        __pyx_mstate_global->__pyx_empty_tuple,
        __pyx_mstate_global->__pyx_empty_tuple,
        varnames_tuple_dedup,
        __pyx_mstate_global->__pyx_empty_tuple,
        __pyx_mstate_global->__pyx_empty_tuple,
        filename,
        funcname,
        (int) descr.first_line,
        (__PYX_LIMITED_VERSION_HEX >= (0x030b0000) && line_table) ? line_table : __pyx_mstate_global->__pyx_empty_bytes
    );
done:
    Py_XDECREF(code_bytes);
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(varnames_tuple_dedup);
    #endif
    Py_DECREF(varnames_tuple);
    return code_obj;
}

/* DecompressString */
static PyObject *__Pyx_DecompressString(const char *s, Py_ssize_t length, int algo) {
    PyObject *module = NULL, *decompress, *compressed_bytes, *decompressed;
    const char* module_name = algo == 3 ? "compression.zstd" : algo == 2 ? "bz2" : "zlib";
    PyObject *methodname = PyUnicode_FromString("decompress");
    if (unlikely(!methodname)) return NULL;
    #if __PYX_LIMITED_VERSION_HEX >= 0x030e0000
    if (algo == 3) {
        PyObject *fromlist = Py_BuildValue("[O]", methodname);
        if (unlikely(!fromlist)) goto bad;
        module = PyImport_ImportModuleLevel("compression.zstd", NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
    } else
    #endif
        module = PyImport_ImportModule(module_name);
    if (unlikely(!module)) goto import_failed;
    decompress = PyObject_GetAttr(module, methodname);
    if (unlikely(!decompress)) goto import_failed;
    {
        #ifdef __cplusplus
            char *memview_bytes = const_cast<char*>(s);
        #else
            #if defined(__clang__)
              #pragma clang diagnostic push
              #pragma clang diagnostic ignored "-Wcast-qual"
            #elif !defined(__INTEL_COMPILER) && defined(__GNUC__)
              #pragma GCC diagnostic push
              #pragma GCC diagnostic ignored "-Wcast-qual"
            #endif
            char *memview_bytes = (char*) s;
            #if defined(__clang__)
              #pragma clang diagnostic pop
            #elif !defined(__INTEL_COMPILER) && defined(__GNUC__)
              #pragma GCC diagnostic pop
            #endif
        #endif
        #if CYTHON_COMPILING_IN_LIMITED_API && !defined(PyBUF_READ)
        int memview_flags = 0x100;
        #else
        int memview_flags = PyBUF_READ;
        #endif
        compressed_bytes = PyMemoryView_FromMemory(memview_bytes, length, memview_flags);
    }
    if (unlikely(!compressed_bytes)) {
        Py_DECREF(decompress);
        goto bad;
    }
    decompressed = PyObject_CallFunctionObjArgs(decompress, compressed_bytes, NULL);
    Py_DECREF(compressed_bytes);
    Py_DECREF(decompress);
    Py_DECREF(module);
    Py_DECREF(methodname);
    return decompressed;
import_failed:
    PyErr_Format(PyExc_ImportError,
        "Failed to import '%.20s.decompress' - cannot initialise module strings. "
        "String compression was configured with the C macro 'CYTHON_COMPRESS_STRINGS=%d'.",
        module_name, algo);
bad:
    Py_XDECREF(module);
    Py_DECREF(methodname);
    return NULL;
}

#include <string.h>
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s) {
    size_t len = strlen(s);
    if (unlikely(len > (size_t) PY_SSIZE_T_MAX)) {
        PyErr_SetString(PyExc_OverflowError, "byte string is too long");
        return -1;
    }
    return (Py_ssize_t) len;
}
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return __Pyx_PyUnicode_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return PyByteArray_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    {
        const char* result;
        Py_ssize_t unicode_length;
        CYTHON_MAYBE_UNUSED_VAR(unicode_length); // only for __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        #if __PYX_LIMITED_VERSION_HEX < 0x030A0000
        if (unlikely(PyArg_Parse(o, "s#", &result, length) < 0)) return NULL;
        #else
        result = PyUnicode_AsUTF8AndSize(o, length);
        #endif
        #if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        unicode_length = PyUnicode_GetLength(o);
        if (unlikely(unicode_length < 0)) return NULL;
        if (unlikely(unicode_length != *length)) {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
        #endif
        return result;
    }
#else
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
}
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
    if (PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
    if (PyByteArray_Check(o)) {
#if (CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS) || (CYTHON_COMPILING_IN_PYPY && (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE)))
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
#else
        *length = PyByteArray_Size(o);
        if (*length == -1) return NULL;
        return PyByteArray_AsString(o);
#endif
    } else
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_LongWrongResultType(PyObject* result) {
    __Pyx_TypeName result_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(result));
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ").  "
                "The ability to return an instance of a strict subclass of int is deprecated, "
                "and may be removed in a future version of Python.",
                result_type_name)) {
            __Pyx_DECREF_TypeName(result_type_name);
            Py_DECREF(result);
            return NULL;
        }
        __Pyx_DECREF_TypeName(result_type_name);
        return result;
    }
    PyErr_Format(PyExc_TypeError,
                 "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ")",
                 result_type_name);
    __Pyx_DECREF_TypeName(result_type_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  PyObject *res = NULL;
  if (likely(PyLong_Check(x)))
      return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  if (likely(m && m->nb_int)) {
      res = m->nb_int(x);
  }
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
      res = PyNumber_Long(x);
  }
#endif
  if (likely(res)) {
      if (unlikely(!PyLong_CheckExact(res))) {
          return __Pyx_PyNumber_LongWrongResultType(res);
      }
  }
  else if (!PyErr_Occurred()) {
      PyErr_SetString(PyExc_TypeError,
                      "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(__Pyx_PyLong_IsCompact(b))) {
        return __Pyx_PyLong_CompactValue(b);
    } else {
      const digit* digits = __Pyx_PyLong_Digits(b);
      const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(b);
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyLong_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyLong_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b) {
    CYTHON_UNUSED_VAR(b);
    return __Pyx_NewRef(Py_None);
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return __Pyx_NewRef(b ? Py_True: Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t ival) {
    return PyLong_FromSize_t(ival);
}


/* MultiPhaseInitModuleState */
#if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
#ifndef CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
#if (CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX >= 0x030C0000)
  #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 1
#else
  #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 0
#endif
#endif
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE && !CYTHON_ATOMICS
#error "Module state with PEP489 requires atomics. Currently that's one of\
 C11, C++11, gcc atomic intrinsics or MSVC atomic intrinsics"
#endif
#if !CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
#define __Pyx_ModuleStateLookup_Lock()
#define __Pyx_ModuleStateLookup_Unlock()
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d0000
static PyMutex __Pyx_ModuleStateLookup_mutex = {0};
#define __Pyx_ModuleStateLookup_Lock() PyMutex_Lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() PyMutex_Unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(__cplusplus) && __cplusplus >= 201103L
#include <mutex>
static std::mutex __Pyx_ModuleStateLookup_mutex;
#define __Pyx_ModuleStateLookup_Lock() __Pyx_ModuleStateLookup_mutex.lock()
#define __Pyx_ModuleStateLookup_Unlock() __Pyx_ModuleStateLookup_mutex.unlock()
#elif defined(__STDC_VERSION__) && (__STDC_VERSION__ > 201112L) && !defined(__STDC_NO_THREADS__)
#include <threads.h>
static mtx_t __Pyx_ModuleStateLookup_mutex;
static once_flag __Pyx_ModuleStateLookup_mutex_once_flag = ONCE_FLAG_INIT;
static void __Pyx_ModuleStateLookup_initialize_mutex(void) {
    mtx_init(&__Pyx_ModuleStateLookup_mutex, mtx_plain);
}
#define __Pyx_ModuleStateLookup_Lock()\
  call_once(&__Pyx_ModuleStateLookup_mutex_once_flag, __Pyx_ModuleStateLookup_initialize_mutex);\
  mtx_lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() mtx_unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(HAVE_PTHREAD_H)
#include <pthread.h>
static pthread_mutex_t __Pyx_ModuleStateLookup_mutex = PTHREAD_MUTEX_INITIALIZER;
#define __Pyx_ModuleStateLookup_Lock() pthread_mutex_lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() pthread_mutex_unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(_WIN32)
#include <Windows.h>  // synchapi.h on its own doesn't work
static SRWLOCK __Pyx_ModuleStateLookup_mutex = SRWLOCK_INIT;
#define __Pyx_ModuleStateLookup_Lock() AcquireSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() ReleaseSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
#else
#error "No suitable lock available for CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE.\
 Requires C standard >= C11, or C++ standard >= C++11,\
 or pthreads, or the Windows 32 API, or Python >= 3.13."
#endif
typedef struct {
    int64_t id;
    PyObject *module;
} __Pyx_InterpreterIdAndModule;
typedef struct {
    char interpreter_id_as_index;
    Py_ssize_t count;
    Py_ssize_t allocated;
    __Pyx_InterpreterIdAndModule table[1];
} __Pyx_ModuleStateLookupData;
#define __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE 32
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static __pyx_atomic_int_type __Pyx_ModuleStateLookup_read_counter = 0;
#endif
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static __pyx_atomic_ptr_type __Pyx_ModuleStateLookup_data = 0;
#else
static __Pyx_ModuleStateLookupData* __Pyx_ModuleStateLookup_data = NULL;
#endif
static __Pyx_InterpreterIdAndModule* __Pyx_State_FindModuleStateLookupTableLowerBound(
        __Pyx_InterpreterIdAndModule* table,
        Py_ssize_t count,
        int64_t interpreterId) {
    __Pyx_InterpreterIdAndModule* begin = table;
    __Pyx_InterpreterIdAndModule* end = begin + count;
    if (begin->id == interpreterId) {
        return begin;
    }
    while ((end - begin) > __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
        __Pyx_InterpreterIdAndModule* halfway = begin + (end - begin)/2;
        if (halfway->id == interpreterId) {
            return halfway;
        }
        if (halfway->id < interpreterId) {
            begin = halfway;
        } else {
            end = halfway;
        }
    }
    for (; begin < end; ++begin) {
        if (begin->id >= interpreterId) return begin;
    }
    return begin;
}
static PyObject *__Pyx_State_FindModule(CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return NULL;
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData* data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
    {
        __pyx_atomic_incr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
        if (likely(data)) {
            __Pyx_ModuleStateLookupData* new_data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_acquire(&__Pyx_ModuleStateLookup_data);
            if (likely(data == new_data)) {
                goto read_finished;
            }
        }
        __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
        __Pyx_ModuleStateLookup_Lock();
        __pyx_atomic_incr_relaxed(&__Pyx_ModuleStateLookup_read_counter);
        data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
        __Pyx_ModuleStateLookup_Unlock();
    }
  read_finished:;
#else
    __Pyx_ModuleStateLookupData* data = __Pyx_ModuleStateLookup_data;
#endif
    __Pyx_InterpreterIdAndModule* found = NULL;
    if (unlikely(!data)) goto end;
    if (data->interpreter_id_as_index) {
        if (interpreter_id < data->count) {
            found = data->table+interpreter_id;
        }
    } else {
        found = __Pyx_State_FindModuleStateLookupTableLowerBound(
            data->table, data->count, interpreter_id);
    }
  end:
    {
        PyObject *result=NULL;
        if (found && found->id == interpreter_id) {
            result = found->module;
        }
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
        __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
#endif
        return result;
    }
}
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static void __Pyx_ModuleStateLookup_wait_until_no_readers(void) {
    while (__pyx_atomic_load(&__Pyx_ModuleStateLookup_read_counter) != 0);
}
#else
#define __Pyx_ModuleStateLookup_wait_until_no_readers()
#endif
static int __Pyx_State_AddModuleInterpIdAsIndex(__Pyx_ModuleStateLookupData **old_data, PyObject* module, int64_t interpreter_id) {
    Py_ssize_t to_allocate = (*old_data)->allocated;
    while (to_allocate <= interpreter_id) {
        if (to_allocate == 0) to_allocate = 1;
        else to_allocate *= 2;
    }
    __Pyx_ModuleStateLookupData *new_data = *old_data;
    if (to_allocate != (*old_data)->allocated) {
         new_data = (__Pyx_ModuleStateLookupData *)realloc(
            *old_data,
            sizeof(__Pyx_ModuleStateLookupData)+(to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
        if (!new_data) {
            PyErr_NoMemory();
            return -1;
        }
        for (Py_ssize_t i = new_data->allocated; i < to_allocate; ++i) {
            new_data->table[i].id = i;
            new_data->table[i].module = NULL;
        }
        new_data->allocated = to_allocate;
    }
    new_data->table[interpreter_id].module = module;
    if (new_data->count < interpreter_id+1) {
        new_data->count = interpreter_id+1;
    }
    *old_data = new_data;
    return 0;
}
static void __Pyx_State_ConvertFromInterpIdAsIndex(__Pyx_ModuleStateLookupData *data) {
    __Pyx_InterpreterIdAndModule *read = data->table;
    __Pyx_InterpreterIdAndModule *write = data->table;
    __Pyx_InterpreterIdAndModule *end = read + data->count;
    for (; read<end; ++read) {
        if (read->module) {
            write->id = read->id;
            write->module = read->module;
            ++write;
        }
    }
    data->count = write - data->table;
    for (; write<end; ++write) {
        write->id = 0;
        write->module = NULL;
    }
    data->interpreter_id_as_index = 0;
}
static int __Pyx_State_AddModule(PyObject* module, CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return -1;
    int result = 0;
    __Pyx_ModuleStateLookup_Lock();
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData *old_data = (__Pyx_ModuleStateLookupData *)
            __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
#else
    __Pyx_ModuleStateLookupData *old_data = __Pyx_ModuleStateLookup_data;
#endif
    __Pyx_ModuleStateLookupData *new_data = old_data;
    if (!new_data) {
        new_data = (__Pyx_ModuleStateLookupData *)calloc(1, sizeof(__Pyx_ModuleStateLookupData));
        if (!new_data) {
            result = -1;
            PyErr_NoMemory();
            goto end;
        }
        new_data->allocated = 1;
        new_data->interpreter_id_as_index = 1;
    }
    __Pyx_ModuleStateLookup_wait_until_no_readers();
    if (new_data->interpreter_id_as_index) {
        if (interpreter_id < __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
            result = __Pyx_State_AddModuleInterpIdAsIndex(&new_data, module, interpreter_id);
            goto end;
        }
        __Pyx_State_ConvertFromInterpIdAsIndex(new_data);
    }
    {
        Py_ssize_t insert_at = 0;
        {
            __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
                new_data->table, new_data->count, interpreter_id);
            assert(lower_bound);
            insert_at = lower_bound - new_data->table;
            if (unlikely(insert_at < new_data->count && lower_bound->id == interpreter_id)) {
                lower_bound->module = module;
                goto end;  // already in table, nothing more to do
            }
        }
        if (new_data->count+1 >= new_data->allocated) {
            Py_ssize_t to_allocate = (new_data->count+1)*2;
            new_data =
                (__Pyx_ModuleStateLookupData*)realloc(
                    new_data,
                    sizeof(__Pyx_ModuleStateLookupData) +
                    (to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
            if (!new_data) {
                result = -1;
                new_data = old_data;
                PyErr_NoMemory();
                goto end;
            }
            new_data->allocated = to_allocate;
        }
        ++new_data->count;
        int64_t last_id = interpreter_id;
        PyObject *last_module = module;
        for (Py_ssize_t i=insert_at; i<new_data->count; ++i) {
            int64_t current_id = new_data->table[i].id;
            new_data->table[i].id = last_id;
            last_id = current_id;
            PyObject *current_module = new_data->table[i].module;
            new_data->table[i].module = last_module;
            last_module = current_module;
        }
    }
  end:
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, new_data);
#else
    __Pyx_ModuleStateLookup_data = new_data;
#endif
    __Pyx_ModuleStateLookup_Unlock();
    return result;
}
static int __Pyx_State_RemoveModule(CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return -1;
    __Pyx_ModuleStateLookup_Lock();
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData *data = (__Pyx_ModuleStateLookupData *)
            __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
#else
    __Pyx_ModuleStateLookupData *data = __Pyx_ModuleStateLookup_data;
#endif
    if (data->interpreter_id_as_index) {
        if (interpreter_id < data->count) {
            data->table[interpreter_id].module = NULL;
        }
        goto done;
    }
    {
        __Pyx_ModuleStateLookup_wait_until_no_readers();
        __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
            data->table, data->count, interpreter_id);
        if (!lower_bound) goto done;
        if (lower_bound->id != interpreter_id) goto done;
        __Pyx_InterpreterIdAndModule *end = data->table+data->count;
        for (;lower_bound<end-1; ++lower_bound) {
            lower_bound->id = (lower_bound+1)->id;
            lower_bound->module = (lower_bound+1)->module;
        }
    }
    --data->count;
    if (data->count == 0) {
        free(data);
        data = NULL;
    }
  done:
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, data);
#else
    __Pyx_ModuleStateLookup_data = data;
#endif
    __Pyx_ModuleStateLookup_Unlock();
    return 0;
}
#endif

/* #### Code section: utility_code_pragmas_end ### */
#ifdef _MSC_VER
#pragma warning( pop )
#endif



/* #### Code section: end ### */
#endif /* Py_PYTHON_H */
